(function () {
  const dependencies = ['ionic', 'ngMaterial', 'ngMessages', 'ngCordova', 'ngAnimate', 'ngLodash', 'monospaced.elastic', 'yaru22.angular-timeago', 'ui.calendar', 'flexcalendar', 'flexcalendar.defaultTranslation', 'chart.js', 'prettyBytes', 'angular-svg-round-progressbar', 'ngCountTo', 'mfw-ionic.deep-links', 'ionic-color-picker'];

  angular.module('MyPTHub', dependencies).config(config).run(run);

  config.$inject = ['$ionicConfigProvider', '$mdThemingProvider', '$provide', '$compileProvider', '$mfwiLinksProvider'];

  function config($ionicConfigProvider, $mdThemingProvider, $provide, $compileProvider, $mfwiLinksProvider) {
    $ionicConfigProvider.spinner.icon('android');
    $ionicConfigProvider.views.swipeBackEnabled(false);

    $compileProvider.imgSrcSanitizationWhitelist(/^\s*(https?|file|blob|cdvfile|content):|data:image\//);
    $compileProvider.aHrefSanitizationWhitelist(/^\s*(https?|ftp|mailto|webcal|tel):/);

    /**
     * Angular material theme config
     */

    $mdThemingProvider.theme('default').primaryPalette('blue').accentPalette('light-blue');
    $mdThemingProvider.alwaysWatchTheme(true);

    /**
     * Disable Ink Ripple animations for Android, as typically Android
     * devices are lower power than iOS and the animations cause lag in
     * the UI.
     *
     * @TODO: Enable this when upgraded to Angular Material 1.1.0
     */
    // $mdInkRippleProvider.disableInkRipple();


    /**
     * Customise the $q primise service to provide success() and error()
     * callbacks, to match the $http promises.
     */

    $provide.decorator('$q', $delegate => {
      function httpResponseWrapper(fn) {
        return function (res) {
          if (res && res.hasOwnProperty('data') && res.hasOwnProperty('status') && res.hasOwnProperty('headers') && res.hasOwnProperty('config') && res.hasOwnProperty('statusText')) {
            return fn(res.data, res.status, res.headers, res.config, res.statusText);
          }

          return fn(res);
        };
      }

      function decorator(promise) {
        promise.success = function (fn) {
          return decorator(promise.then(httpResponseWrapper(fn)));
        };

        promise.error = function (fn) {
          return decorator(promise.then(null, httpResponseWrapper(fn)));
        };

        return promise;
      }

      const defer = $delegate.defer;

      $delegate.defer = function () {
        const deferred = defer();

        decorator(deferred.promise);

        return deferred;
      };

      return $delegate;
    });
    $mfwiLinksProvider.config({
      // All registered routes (imperative and declarative) will be prepended with this prefix
      routesPrefix: '',
      // Milliseconds of delay between loading parent state and the matched one
      nestedStatesDelay: 700,
      // Callback executed for each match
      matchCallback: ['$log', '$match', function ($log, $match) {
        $log.log('Match found', $match);
      }],
      // Callback executed for each non match
      nomatchCallback: ['$log', '$match', function ($log, $match) {
        // $state.href($state.current.name, $state.params, {absolute: true})
        $log.log('No Match found', $match);
      }]
    }).addRoute('/signup', ['$timeout', '$log', '$state', '$match', function ($timeout, $log, $state, $match) {
      $log.log('>>>Received match', $match);

      const code = $match.$args.code;
      /**
       * This is redirecting 2 times
       */
      if (!$state.current) {
        $state.go('app.client_settings', { code });
      } else {
        $timeout(() => {
          $state.go('app.client_settings', { code });
        }, 1000);
      }

      SafariViewController.hide();
    }]);
  }

  run.$inject = ['$ionicPlatform', 'SqlStorageService', 'mpthconf', 'AuthenticationStateService', 'PermissionsService', 'AuthenticationService', 'NetworkStateService', '$state', '$timeout', '$rootScope', '$ionicHistory', 'SessionStoreService', 'WorkoutStoreService', 'CustomerStoreService', 'DeviceInfoService', 'PushNotificationService', '$cordovaStatusbar', 'IntroService', 'BrandingService', 'MessagingService', 'AnalyticsService', 'DeeplinkService', 'SideMenuService', 'ScheduleEventStoreService', 'CredStoreService', 'ThreeDeeTouchQuickActionsService', 'InAppPurchaseService', 'IosIconService'];

  function run($ionicPlatform, SqlStorageService, mpthconf, AuthenticationStateService, PermissionsService, AuthenticationService, NetworkStateService, $state, $timeout, $rootScope, $ionicHistory, SessionStoreService, WorkoutStoreService, CustomerStoreService, DeviceInfoService, PushNotificationService, $cordovaStatusbar, IntroService, BrandingService, MessagingService, AnalyticsService, DeeplinkService, SideMenuService, ScheduleEventStoreService, CredStoreService, ThreeDeeTouchQuickActionsService, InAppPurchaseService, IosIconService) {
    /**
     * Ionic configuration.
     */

    $ionicPlatform.ready(() => {
      if (typeof Stripe !== 'undefined') {
        Stripe(mpthconf[mpthconf.env].stripe_key);
      }

      ionic.Platform.isFullScreen = false;

      if (window.cordova && window.cordova.plugins.Keyboard) {
        cordova.plugins.Keyboard.hideKeyboardAccessoryBar(false);
        cordova.plugins.Keyboard.disableScroll(true);
      }

      if (window.StatusBar && !DeviceInfoService.isAndroid()) {
        $cordovaStatusbar.overlaysWebView(true);
        // $cordovaStatusbar.style(1);
      }

      if (window.screen && typeof screen.lockOrientation !== 'undefined' && typeof screen.unlockOrientation !== 'undefined') {
        if (window.isTablet) {
          screen.unlockOrientation();
          console.log('Device orientation unlocked');
        } else {
          console.log('Device orientation fixed to portrait.');
          screen.lockOrientation('portrait');
        }
      }
    });

    /**
     * Ensure local SQL database is setup with the correct tables for
     * caching and task queues for offline actions.
     */

    SqlStorageService.initDB(mpthconf.sql.db_name);

    /**
     * Bind the events that keep track of the state of internet
     * connectivity.
     */

    NetworkStateService.setup();

    /**
     * Setup Session Store service, registers scheduled task to sync
     * offline sessions once a working internet connection is active.
     */

    SessionStoreService.setup();

    /**
     * Setup Schedule Event Store service
     */

    ScheduleEventStoreService.setup();

    /**
     * Setup Workout Store, keeps a local database of the current user's
     * workouts so they can be used in offline ad-hoc (not pre-booked)
     * sessions.
     */

    WorkoutStoreService.setup();

    /**
     * Setup customer store, keeps a local database of the trainer's
     * clients so they can be used for offline ad-hoc session starting and
     * other stuff.
     */

    CustomerStoreService.setup();

    /**
     * Setup push notifications after login
     */

    AuthenticationService.registerLoginSuccessEventHandler(resolve => {
      PushNotificationService.setup();
      resolve(true);
    }, true);

    /**
     * Setup Intro state service
     */

    IntroService.setup();

    /**
     * Side Menu setup
     */

    SideMenuService.setup();

    /**
     * Branding parse scss files ready for use
     */

    BrandingService.applyStyles(true);

    if (!BrandingService.isCustomApp()) {
      AuthenticationService.registerLoginSuccessEventHandler(resolve => {
        BrandingService.applyStyles();
        resolve(true);
      }, true);
    }

    IosIconService.setup();

    /**
     * Seting messaging service that handles push notification related
     * stuff.
     */

    MessagingService.setup();

    /**
     * Fire up the Analytics
     */

    AnalyticsService.setup();

    /**
     * Seting deeplink service.
     */

    DeeplinkService.setup();

    /**
     * 3d Touch Service Setup
     */
    ThreeDeeTouchQuickActionsService.setup().then(() => {
      AuthenticationService.registerLoginSuccessEventHandler(resolve => {
        ThreeDeeTouchQuickActionsService.setActions();
      });
    });

    /**
     * In App Purchase setup
     */
    InAppPurchaseService.setup();

    /**
     * Airbrake identification setup
     */

    /**
     * Intercept route navigation for first load and attempts to navigate
     * to login pages when logged in already.
     */
    $rootScope.$on('$stateChangeStart', (event, next) => {
      const nextViewOptions = {};
      let hasOpts = false;

      if (DeviceInfoService.isAndroid()) {
        nextViewOptions.disableAnimate = true;
        hasOpts = true;
      }

      if (next.params && next.params.historyRoot) {
        nextViewOptions.disableBack = true;
        nextViewOptions.historyRoot = true;
        hasOpts = true;
      }

      if (hasOpts) {
        $ionicHistory.nextViewOptions(nextViewOptions);
      }

      // Prevent access to the login pages when already logged in
      if (next.params && next.params.isLogin && AuthenticationStateService.isLoggedIn()) {
        event.preventDefault();
        if (PermissionsService.userIsTrainer()) {
          $timeout($state.go('app.trainer_home'));
        } else {
          $timeout($state.go('app.client_home'));
        }
      }
    });

    /**
     * Go
     */

    CredStoreService.setup().then(() => {
      AuthenticationService.setup().then(() => {
        if (AuthenticationStateService.isLoggedIn()) {
          if (PermissionsService.userIsTrainer()) {
            $timeout($state.go('app.trainer_home'));
          } else {
            $timeout($state.go('app.client_home'));
          }
        } else {
          $timeout($state.go('app.login'));
        }
      }, () => {
        $timeout($state.go('app.login'));
      });
    });
  }
})();
(function () {
  'use strict';

  /**
   * For Android builds only.
   *
   * This constant is populated with an object that contains branding params
   * for custom apps. This happens during the automated build process.
   */

  angular.module('MyPTHub').constant('baseBrand', { "trainer_id": 17643, "business_name": "LJ Health Fitness Nutrition", "branding_enabled": 1, "branding_primary": "#0ded3e", "branding_navbar": "#333333", "logo_url": "https:\/\/d3qcjjq30235ix.cloudfront.net\/logos\/e40d852e1bf0afbff127df0ae806ed88.png", "is_custom_logo": true, "currency": "AUD", "currency_symbol": "$", "currency_is_dollar": true, "ios_app_url": "https:\/\/apps.apple.com\/app\/my-pt-hub\/id1473947709?ls=1", "android_app_url": "https:\/\/play.google.com\/store\/apps\/details?id=com.mypthub.lghealthfitnessnutrition", "advance_booking_value": "weeks", "advance_booking_amount": 2, "awin_id": null, "facebook_id": null, "signed_up_from": "2017-06-12 02:50:28" });
})();
(() => {
  angular.module('MyPTHub').constant('mpthconf', {

    // env: 'local',
    // env: 'staging',
    env: 'production',

    local: {
      api: {
        current: 'https://api.mypthub.test/'
        // current: 'http://3d957ca3.ngrok.io/v4/'
      },
      webcal: {
        current: 'webcal://api.mypthub.test/v4/users/calendar/'
      },
      webapp: {
        current: 'https://{subdomain}.mypthub.test/'
      },
      stripe_key: 'pk_test_6uD9mCUaW115FqGiSoes84Nu'
    },
    staging: {
      api: {
        current: 'https://api.mypthub.xyz/'
      },
      webcal: {
        current: 'webcal://api.mypthub.xyz/v4/users/calendar/'
      },
      webapp: {
        current: 'https://{subdomain}.mypthub.xyz/'
      },
      stripe_key: 'pk_test_Xev3EFQEx5RyUxO9IuLACZFo'
    },
    production: {
      api: {
        current: 'https://api.mypthub.net/'
      },
      webcal: {
        current: 'webcal://api.mypthub.net/v4/users/calendar/'
      },
      webapp: {
        current: 'https://{subdomain}.mypthub.net/'
      },
      stripe_key: 'pk_live_SA7eTQilrB2jpZcMAbpy165q'
    },

    gcm: {
      sender_id: '512045806277'
    },

    sql: {
      db_name: 'mypthub_db.db'
    },

    analytics: {
      id: 'UA-58860005-2'
    },

    fileCache: {
      maxAge: {
        img: 1000 * 60 * 60 * 8,
        vid: 0
      }
    },

    cdn: {
      base: 'https://d3qcjjq30235ix.cloudfront.net/'
    },

    airbrake: {
      project_id: 150258,
      project_key: '46730e3def529564fd0363981eeb153b'
    },

    brand: null

  });
})();
(function () {
  'use strict';

  /**
   * For iOS builds only.
   *
   * This constant is populated with an object that contains info about
   * which trainers have custom app icons. This happens during the automated
   * build process.
   */

  angular.module('MyPTHub').constant('AvailableiOSIcons', { "0": true, "7": true, "10580": true, "1046": true, "9343": true, "654": true, "10189": true, "85": true, "9471": true, "8386": true, "7217": true, "9905": true, "4112": true, "7145": true, "3658": true, "5132": true, "10049": true, "7932": true, "4367": true, "8792": true, "2355": true, "6020": true, "8264": true, "2850": true, "10318": true, "10259": true, "7733": true, "10326": true, "3216": true, "6099": true, "1049": true, "10444": true, "10520": true, "10365": true, "9612": true, "6717": true, "695": true, "4265": true, "10629": true, "10574": true, "8529": true, "5187": true, "10565": true, "10408": true, "10387": true, "10608": true, "10441": true, "8427": true, "10483": true, "10523": true, "8365": true, "10735": true, "10711": true, "6693": true, "8242": true, "6956": true, "10671": true, "10454": true, "1088": true, "6798": true, "7246": true, "9086": true, "5436": true, "10397": true, "10919": true, "4264": true, "10912": true, "7690": true, "10494": true, "10989": true, "10325": true, "6268": true, "3065": true, "9696": true, "3082": true, "10895": true, "10714": true, "11084": true, "6343": true, "11098": true, "8821": true, "9113": true, "8314": true, "2263": true, "3734": true, "11262": true, "10276": true, "10888": true, "2570": true, "10337": true, "6443": true, "10764": true, "5804": true, "11386": true, "10154": true, "11162": true, "9879": true, "10849": true, "10838": true, "11021": true, "1907": true, "10708": true, "11428": true, "2906": true, "11181": true, "1446": true, "11571": true, "12": true, "9993": true, "9146": true, "4768": true, "11614": true, "11611": true, "4561": true, "7198": true, "11598": true, "1340": true, "3943": true, "11417": true, "11714": true, "10816": true, "11683": true, "4976": true, "11664": true, "6298": true, "9921": true, "11863": true, "11434": true, "10891": true, "11313": true, "11852": true, "11957": true, "3661": true, "10591": true, "12017": true, "12056": true, "4289": true, "12121": true, "6610": true, "5222": true, "11920": true, "2654": true, "3321": true, "8005": true, "11716": true, "11653": true, "6648": true, "11751": true, "12187": true, "11744": true, "11103": true, "11565": true, "11785": true, "11380": true, "12006": true, "11335": true, "9147": true, "1973": true, "11701": true, "7873": true, "1375": true, "12431": true, "11332": true, "12474": true, "10138": true, "11941": true, "12331": true, "11902": true, "11645": true, "11898": true, "11737": true, "12479": true, "12394": true, "9281": true, "1068": true, "11430": true, "3625": true, "12597": true, "12628": true, "9887": true, "1783": true, "10890": true, "12693": true, "4994": true, "12553": true, "12608": true, "12746": true, "12076": true, "12558": true, "9922": true, "12661": true, "9490": true, "9969": true, "10641": true, "12767": true, "10844": true, "9485": true, "12855": true, "12780": true, "10923": true, "9427": true, "12980": true, "12964": true, "13014": true, "10827": true, "12250": true, "12951": true, "13046": true, "13036": true, "12846": true, "4286": true, "10396": true, "4023": true, "13106": true, "12438": true, "12224": true, "12204": true, "3646": true, "12834": true, "11860": true, "12715": true, "13055": true, "9764": true, "13035": true, "12517": true, "6611": true, "11202": true, "10720": true, "879": true, "11290": true, "4578": true, "13290": true, "9673": true, "13080": true, "12265": true, "13386": true, "13484": true, "12992": true, "13479": true, "1382": true, "12273": true, "13142": true, "13537": true, "12181": true, "13502": true, "13336": true, "7841": true, "12728": true, "13578": true, "13455": true, "13581": true, "12785": true, "13255": true, "12596": true, "10552": true, "12489": true, "13685": true, "6351": true, "11323": true, "10939": true, "8791": true, "13608": true, "13709": true, "12108": true, "11873": true, "13213": true, "13572": true, "7267": true, "12253": true, "12449": true, "3346": true, "13831": true, "10446": true, "13560": true, "13849": true, "10250": true, "13916": true, "13889": true, "12944": true, "12196": true, "13772": true, "13860": true, "12912": true, "13143": true, "3132": true, "12572": true, "14053": true, "14056": true, "13933": true, "14122": true, "13892": true, "11562": true, "13669": true, "11764": true, "14061": true, "9277": true, "6382": true, "14211": true, "14212": true, "14299": true, "13998": true, "12435": true, "13941": true, "11539": true, "14307": true, "10727": true, "1975": true, "13809": true, "14197": true, "5016": true, "9510": true, "2566": true, "13940": true, "14417": true, "14335": true, "10088": true, "14291": true, "14095": true, "2236": true, "8830": true, "11719": true, "5318": true, "14052": true, "14104": true, "14626": true, "13946": true, "12211": true, "14663": true, "10261": true, "10053": true, "13620": true, "13069": true, "9682": true, "7603": true, "13746": true, "13147": true, "14716": true, "4412": true, "13990": true, "8280": true, "9795": true, "8101": true, "14285": true, "14803": true, "14799": true, "14654": true, "14851": true, "13981": true, "14900": true, "14177": true, "13434": true, "14953": true, "14288": true, "11782": true, "14352": true, "14804": true, "7054": true, "15042": true, "7859": true, "5924": true, "15085": true, "6612": true, "14583": true, "15007": true, "14891": true, "13291": true, "12177": true, "14833": true, "6919": true, "14608": true, "15020": true, "15081": true, "1195": true, "15048": true, "14927": true, "1691": true, "12968": true, "14582": true, "14609": true, "15345": true, "14117": true, "13177": true, "15333": true, "4392": true, "7897": true, "7624": true, "12389": true, "14701": true, "15329": true, "14555": true, "14141": true, "8971": true, "13218": true, "10726": true, "15544": true, "14484": true, "3452": true, "14954": true, "14973": true, "1439": true, "15618": true, "15589": true, "15361": true, "12798": true, "15654": true, "14416": true, "10425": true, "8726": true, "15699": true, "14393": true, "9975": true, "12199": true, "15732": true, "14786": true, "15648": true, "11590": true, "11158": true, "15624": true, "15245": true, "15805": true, "1196": true, "15563": true, "14876": true, "15827": true, "13322": true, "15225": true, "8862": true, "15786": true, "5744": true, "15512": true, "14089": true, "15609": true, "15725": true, "15673": true, "15997": true, "4390": true, "15855": true, "15176": true, "16065": true, "7259": true, "15746": true, "12232": true, "9190": true, "16110": true, "10218": true, "15761": true, "16128": true, "16145": true, "16151": true, "10178": true, "6448": true, "15886": true, "15469": true, "16169": true, "13047": true, "5036": true, "15766": true, "16308": true, "14386": true, "18742": true, "16070": true, "11016": true, "16505": true, "15251": true, "16520": true, "16509": true, "15291": true, "11690": true, "15362": true, "15703": true, "14252": true, "16562": true, "16052": true, "14272": true, "16468": true, "16581": true, "16519": true, "16744": true, "16647": true, "15962": true, "4529": true, "15116": true, "16553": true, "2800": true, "16728": true, "13170": true, "7622": true, "2538": true, "14100": true, "16913": true, "16923": true, "8459": true, "14279": true, "16852": true, "14068": true, "16905": true, "4797": true, "10743": true, "17008": true, "16934": true, "14872": true, "16912": true, "9077": true, "15832": true, "16891": true, "17085": true, "13176": true, "10434": true, "17017": true, "15202": true, "17211": true, "15951": true, "15978": true, "11944": true, "17048": true, "17138": true, "17244": true, "16666": true, "16987": true, "16484": true, "16673": true, "16816": true, "3735": true, "13815": true, "17067": true, "16160": true, "16095": true, "14846": true, "17375": true, "16717": true, "17438": true, "16201": true, "5418": true, "16397": true, "17267": true, "17466": true, "15207": true, "10636": true, "14193": true, "17283": true, "15271": true, "1201": true, "5350": true, "17507": true, "566": true, "8463": true, "14490": true, "6867": true, "14328": true, "17718": true, "15924": true, "6701": true, "10310": true, "17396": true, "8254": true, "17180": true, "16476": true, "17129": true, "17842": true, "18226": true, "15050": true, "16855": true, "12909": true, "7803": true, "18044": true, "18088": true, "16163": true, "18000": true, "17265": true, "14171": true, "16974": true, "17731": true, "17785": true, "17146": true, "17609": true, "16466": true, "18229": true, "17755": true, "18335": true, "17907": true, "16776": true, "17190": true, "12921": true, "17408": true, "17855": true, "18275": true, "17531": true, "12335": true, "17944": true, "17813": true, "4942": true, "15889": true, "10620": true, "17353": true, "17676": true, "7062": true, "18309": true, "18565": true, "7544": true, "18202": true, "18292": true, "8146": true, "18593": true, "18606": true, "18626": true, "12008": true, "18588": true, "18619": true, "17949": true, "14737": true, "13964": true, "17372": true, "18804": true, "17680": true, "13813": true, "17179": true, "13263": true, "17895": true, "17727": true, "3001": true, "18793": true, "16786": true, "18194": true, "18882": true, "9582": true, "18323": true, "18054": true, "14220": true, "17770": true, "7452": true, "18751": true, "8804": true, "11560": true, "18599": true, "18610": true, "19090": true, "1467": true, "15900": true, "15454": true, "18005": true, "17804": true, "18816": true, "19168": true, "11775": true, "1627": true, "15529": true, "12656": true, "19426": true, "18883": true, "2112": true, "14082": true, "5195": true, "16677": true, "15599": true, "19044": true, "14035": true, "19653": true, "15272": true, "19701": true, "19593": true, "18403": true, "19765": true, "17485": true, "16849": true, "19721": true, "2384": true, "5366": true, "19899": true, "9742": true, "19397": true, "19105": true, "19087": true, "19522": true, "19599": true, "18330": true, "18111": true, "20006": true, "20072": true, "9091": true, "17941": true, "13564": true, "19968": true, "1339": true, "19466": true, "20271": true, "19921": true, "20243": true, "13567": true, "20151": true, "20164": true, "13967": true, "20311": true, "20322": true, "16328": true, "20121": true, "20228": true, "16930": true, "20425": true, "20168": true, "20562": true, "20213": true, "15481": true, "20328": true, "20209": true, "9792": true, "17521": true, "10094": true, "18859": true, "9823": true, "19521": true, "20629": true, "18894": true, "20820": true, "17643": true, "19879": true, "20809": true, "20927": true, "20791": true, "20547": true, "20963": true, "18493": true, "21009": true, "15919": true, "19684": true, "20579": true, "20973": true, "13556": true, "19430": true, "18061": true, "21188": true, "18223": true, "21323": true, "20315": true, "18301": true, "20997": true, "16208": true, "18788": true, "21396": true, "20263": true, "21532": true, "20881": true, "21599": true, "21235": true, "21539": true, "21570": true, "15957": true, "21572": true, "16033": true, "1261": true, "20321": true, "20753": true, "12025": true, "18996": true, "21751": true, "19895": true, "19412": true, "19345": true, "21767": true, "4556": true, "20283": true, "15455": true, "21685": true, "21843": true, "20613": true, "20871": true, "21904": true, "21655": true, "7639": true, "2853": true, "20435": true, "21848": true, "12506": true, "19408": true, "11875": true, "15436": true, "21688": true, "643": true, "10255": true, "20807": true, "22129": true, "18641": true, "19828": true, "19912": true, "20928": true, "22049": true, "8872": true, "20387": true, "22074": true, "7185": true, "22216": true, "21674": true, "22209": true, "16689": true, "14074": true, "16723": true, "20859": true, "20230": true, "22029": true, "22137": true, "22042": true, "22455": true, "22068": true, "22491": true, "21789": true, "21870": true, "21922": true, "16291": true, "21953": true, "22308": true, "22307": true, "18424": true, "17751": true, "13284": true, "21648": true, "22598": true, "22671": true, "5503": true, "10488": true, "12756": true, "22738": true, "18966": true, "22618": true, "22766": true, "21746": true, "22306": true, "22867": true, "22872": true, "22277": true, "19637": true, "22087": true, "22452": true, "21422": true, "22695": true, "23011": true, "22763": true, "22796": true, "22169": true, "18507": true, "23083": true, "15726": true, "21929": true, "23007": true, "23125": true, "22579": true, "22676": true, "22850": true, "20120": true, "4108": true, "23248": true, "11546": true, "11122": true, "3987": true, "241": true, "19716": true, "20528": true, "19447": true, "22939": true, "3146": true, "11576": true, "22663": true, "17430": true, "22635": true, "9367": true, "22119": true, "21371": true, "22393": true, "3583": true, "23456": true, "23385": true, "23426": true, "23480": true, "14347": true, "5813": true, "19221": true, "22789": true, "21565": true, "23349": true, "21978": true, "23345": true, "23566": true, "22498": true, "23575": true, "22311": true, "21257": true, "23465": true, "23442": true, "22539": true, "12819": true, "22811": true, "23092": true, "23304": true, "23719": true, "23091": true, "22979": true, "21472": true, "21734": true, "9252": true, "23793": true, "23546": true, "23160": true, "22159": true, "22093": true, "23401": true, "23127": true, "23889": true, "23895": true, "23356": true, "23773": true, "23306": true, "23467": true, "19738": true, "13965": true, "22301": true, "21820": true, "23452": true, "315": true, "23998": true, "469": true, "22517": true, "23545": true, "8799": true, "11805": true, "23212": true, "23486": true, "19862": true, "13364": true, "6242": true, "24011": true, "10780": true, "20475": true, "19840": true, "23732": true, "23966": true, "13997": true, "18935": true, "24020": true, "23709": true, "9698": true, "24024": true, "12885": true, "23593": true, "24038": true, "23852": true, "23983": true, "19242": true, "15273": true, "15211": true, "7576": true, "24083": true, "12957": true, "17863": true, "3234": true, "22343": true, "24017": true, "2640": true, "15206": true, "23130": true, "9093": true, "19346": true, "21567": true, "22269": true, "24033": true, "9695": true, "5426": true, "24086": true, "11226": true, "23963": true, "24117": true, "18774": true, "1720": true, "12644": true, "4076": true, "23410": true, "24088": true, "19474": true, "21197": true, "17216": true, "7810": true, "22443": true, "22726": true, "22563": true, "22606": true, "6425": true, "24044": true, "11026": true, "24052": true, "12625": true, "22489": true, "22356": true, "21359": true, "24120": true, "22361": true, "19974": true, "12770": true, "23024": true, "24131": true, "21943": true, "19245": true, "22580": true, "18906": true, "24106": true, "19410": true, "23314": true, "17096": true, "23642": true, "19218": true, "24336": true, "21721": true, "24204": true, "24398": true, "24394": true, "24343": true, "24489": true, "23893": true, "6512": true, "16495": true, "22025": true, "24353": true, "20989": true, "24301": true, "23189": true, "23431": true, "24099": true, "24286": true, "15652": true, "24591": true, "24518": true, "22264": true, "24601": true, "13602": true, "24603": true, "23887": true, "24442": true, "21361": true, "7899": true, "24102": true, "24141": true, "24645": true, "17401": true, "24384": true, "24543": true, "17957": true, "5073": true, "24716": true, "24245": true, "23591": true, "24791": true, "22773": true, "35106": true, "17051": true, "24504": true, "4327": true, "23075": true, "9926": true, "15015": true, "24922": true, "24277": true, "24012": true, "23618": true, "11858": true, "23577": true, "24764": true, "24644": true, "24799": true, "12631": true, "15": true, "21330": true, "13072": true, "24984": true, "23904": true, "24945": true, "22707": true, "21915": true, "9910": true, "24615": true, "24452": true, "24760": true, "21910": true, "24376": true, "10899": true, "25049": true, "3500": true, "17966": true, "24971": true, "18986": true, "24405": true, "24877": true, "25007": true, "23910": true, "25124": true, "25104": true, "24724": true, "25149": true, "24953": true, "23779": true, "10627": true, "22921": true, "25247": true, "24950": true, "23492": true, "24584": true, "2300": true, "20618": true, "25179": true, "25122": true, "25181": true, "19545": true, "25449": true, "25145": true, "19905": true, "12324": true, "22207": true, "24876": true, "14235": true, "17845": true, "24933": true, "24589": true, "25440": true, "23221": true, "1424": true, "25267": true, "25148": true, "25042": true, "25647": true, "25119": true, "25765": true, "25345": true, "25237": true, "14847": true, "25841": true, "23018": true, "23165": true, "23641": true, "23802": true, "25503": true, "25904": true, "17101": true, "25831": true, "25884": true, "24146": true, "26043": true, "26004": true, "23787": true, "26093": true, "26091": true, "25636": true, "25958": true, "20846": true, "25164": true, "25726": true, "26216": true, "25376": true, "20524": true, "25610": true, "26162": true, "15617": true, "25834": true, "21731": true, "25982": true, "26365": true, "24629": true, "25885": true, "24618": true, "17121": true, "26297": true, "25305": true, "22875": true, "26370": true, "26402": true, "24055": true, "26441": true, "25620": true, "19594": true, "2027": true, "23040": true, "8322": true, "24305": true, "25648": true, "25044": true, "26250": true, "25609": true, "26546": true, "25954": true, "25415": true, "24635": true, "26619": true, "21580": true, "25172": true, "19382": true, "26351": true, "22923": true, "24766": true, "26577": true, "26363": true, "23200": true, "15456": true, "26859": true, "17407": true, "24993": true, "26689": true, "26453": true, "19611": true, "1097": true, "23976": true, "26706": true, "23263": true, "26474": true, "26382": true, "24786": true, "26919": true, "10004": true, "26892": true, "25812": true, "27089": true, "26665": true, "25512": true, "25208": true, "13476": true, "24321": true, "25899": true, "19786": true, "21898": true, "21923": true, "13749": true, "663": true, "23875": true, "27066": true, "4673": true, "26300": true, "23032": true, "27173": true, "27243": true, "26770": true, "27342": true, "11855": true, "26203": true, "21660": true, "26711": true, "27437": true, "25322": true, "25616": true, "22781": true, "24964": true, "6561": true, "27589": true, "19163": true, "27597": true, "27431": true, "26679": true, "27115": true, "3178": true, "26450": true, "27124": true, "15966": true, "27458": true, "26125": true, "22903": true, "27472": true, "26138": true, "17092": true, "20325": true, "27152": true, "27883": true, "26482": true, "22692": true, "27226": true, "17542": true, "27950": true, "27799": true, "23517": true, "27825": true, "27210": true, "23710": true, "23741": true, "27954": true, "27927": true, "27726": true, "9454": true, "12237": true, "27891": true, "27777": true, "25784": true, "21306": true, "28095": true, "9116": true, "28142": true, "27332": true, "22274": true, "28147": true, "27154": true, "23580": true, "26682": true, "26972": true, "27733": true, "27686": true, "27630": true, "27090": true, "25803": true, "26095": true, "27543": true, "27579": true, "25231": true, "23242": true, "7844": true, "26547": true, "27091": true, "26193": true, "17347": true, "18041": true, "27644": true, "26965": true, "27640": true, "28453": true, "28110": true, "17644": true, "25126": true, "27720": true, "22928": true, "28357": true, "26863": true, "27165": true, "28643": true, "12563": true, "28036": true, "11917": true, "1874": true, "18973": true, "28165": true, "27007": true, "19101": true, "28403": true, "28537": true, "27691": true, "22392": true, "27939": true, "11401": true, "28692": true, "25601": true, "28693": true, "23476": true, "6456": true, "28891": true, "22007": true, "28570": true, "28519": true, "6918": true, "28189": true, "28089": true, "17520": true, "28448": true, "28385": true, "28944": true, "28871": true, "4826": true, "27655": true, "27550": true, "15520": true, "28609": true, "28710": true, "21844": true, "27857": true, "24228": true, "25157": true, "29070": true, "29278": true, "28217": true, "28684": true, "28339": true, "28939": true, "28498": true, "28584": true, "28571": true, "28275": true, "21950": true, "29237": true, "7795": true, "28893": true, "29044": true, "29270": true, "26108": true, "25645": true, "29387": true, "29412": true, "29178": true, "29064": true, "29225": true, "29102": true, "28736": true, "27674": true, "28847": true, "28595": true, "15593": true, "24109": true, "28387": true, "28877": true, "23520": true, "26123": true, "28341": true, "27779": true, "29120": true, "28651": true, "27285": true, "27183": true, "29087": true, "29649": true, "27182": true, "25700": true, "29851": true, "29850": true, "29677": true, "558": true, "29655": true, "29243": true, "25382": true, "28691": true, "29881": true, "28135": true, "26876": true, "29962": true, "29839": true, "17414": true, "28572": true, "28922": true, "29946": true, "30023": true, "30062": true, "21896": true, "29954": true, "27250": true, "29357": true, "29781": true, "29050": true, "13616": true, "27901": true, "29348": true, "25482": true, "29354": true, "30247": true, "26353": true, "29941": true, "29861": true, "30308": true, "29216": true, "29823": true, "29911": true, "30359": true, "21984": true, "11624": true, "30329": true, "20647": true, "26129": true, "28908": true, "29314": true, "30345": true, "8300": true, "30496": true, "29390": true, "30495": true, "30472": true, "29210": true, "28789": true, "26517": true, "29220": true, "5831": true, "30026": true, "24609": true, "27896": true, "29719": true, "5381": true, "13696": true, "30332": true, "14907": true, "30337": true, "29736": true, "26486": true, "30670": true, "30300": true, "30299": true, "6329": true, "29748": true, "26991": true, "29590": true, "29956": true, "28085": true, "27497": true, "30587": true, "30837": true, "27775": true, "30104": true, "29224": true, "31130": true, "23157": true, "23012": true, "26418": true, "30167": true, "30476": true, "30610": true, "30927": true, "27628": true, "20519": true, "31262": true, "28290": true, "29928": true, "31373": true, "31244": true, "31394": true, "30926": true, "31412": true, "30766": true, "28677": true, "11281": true, "29656": true, "31253": true, "30814": true, "30298": true, "30847": true, "19953": true, "30568": true, "30276": true, "30366": true, "10768": true, "28219": true, "157": true, "18208": true, "30182": true, "31551": true, "25538": true, "31634": true, "31662": true, "31644": true, "30940": true, "23727": true, "30606": true, "31468": true, "24103": true, "31049": true, "24100": true, "31779": true, "24449": true, "30049": true, "30315": true, "7836": true, "29507": true, "771": true, "31538": true, "31709": true, "31886": true, "31536": true, "31454": true, "28943": true, "30962": true, "31381": true, "31543": true, "25372": true, "29447": true, "31773": true, "31939": true, "30528": true, "30628": true, "31835": true, "26644": true, "31865": true, "31859": true, "31771": true, "32030": true, "31999": true, "32012": true, "29905": true, "30898": true, "31958": true, "20573": true, "31037": true, "31014": true, "31984": true, "32290": true, "31055": true, "32264": true, "31836": true, "26703": true, "16821": true, "31985": true, "32138": true, "32393": true, "32284": true, "2672": true, "28255": true, "13514": true, "23420": true, "32447": true, "31754": true, "24359": true, "31058": true, "29616": true, "25123": true, "30759": true, "27132": true, "8511": true, "32591": true, "17374": true, "32061": true, "32391": true, "32442": true, "10016": true, "6209": true, "32586": true, "32557": true, "31495": true, "31650": true, "32412": true, "26154": true, "32409": true, "32080": true, "30892": true, "32320": true, "15999": true, "32344": true, "30768": true, "15160": true, "30735": true, "32885": true, "23763": true, "16471": true, "31972": true, "32706": true, "30805": true, "32605": true, "33006": true, "33027": true, "33007": true, "31424": true, "33059": true, "32713": true, "32555": true, "22114": true, "33088": true, "25640": true, "20102": true, "32787": true, "27697": true, "32473": true, "24108": true, "33118": true, "15874": true, "33332": true, "32332": true, "31619": true, "33402": true, "10007": true, "32871": true, "1144": true, "29255": true, "33301": true, "17549": true, "7300": true, "32950": true, "33109": true, "30462": true, "30671": true, "30439": true, "25547": true, "28019": true, "31981": true, "33465": true, "31045": true, "33601": true, "30780": true, "15550": true, "30119": true, "26376": true, "33024": true, "32592": true, "33726": true, "15736": true, "33376": true, "32681": true, "28798": true, "33457": true, "26128": true, "28617": true, "27238": true, "33776": true, "33882": true, "33893": true, "33678": true, "31687": true, "14727": true, "34027": true, "8090": true, "14458": true, "31432": true, "33836": true, "26714": true, "5349": true, "33078": true, "29187": true, "33797": true, "32947": true, "1093": true, "28919": true, "32659": true, "33927": true, "25949": true, "7480": true, "31658": true, "33624": true, "32452": true, "34238": true, "27631": true, "33588": true, "19070": true, "29308": true, "34407": true, "31581": true, "31642": true, "34093": true, "32853": true, "33865": true, "33806": true, "29029": true, "34454": true, "34537": true, "34208": true, "7108": true, "33145": true, "34554": true, "34152": true, "34372": true, "31680": true, "31277": true, "18763": true, "31142": true, "30394": true, "34403": true, "34658": true, "34448": true, "33948": true, "28128": true, "3592": true, "33658": true, "31614": true, "34606": true, "27827": true, "34458": true, "17164": true, "22201": true, "4937": true, "33269": true, "29867": true, "34786": true, "34699": true, "34400": true, "33290": true, "25624": true, "34952": true, "9965": true, "33978": true, "15086": true, "5089": true, "35025": true, "34742": true, "35085": true, "34624": true, "35161": true, "35183": true, "35200": true, "33607": true, "3063": true, "21421": true, "34174": true, "34653": true, "33319": true, "17528": true, "34794": true, "34062": true, "8770": true, "34775": true, "1184": true, "33312": true, "3946": true, "14274": true, "33909": true, "35080": true, "9128": true, "27460": true, "35362": true, "33994": true, "20975": true, "35398": true, "32600": true, "34419": true, "18931": true, "35385": true, "28185": true, "33217": true, "35249": true, "35428": true, "2950": true, "18672": true, "14510": true, "27318": true, "34006": true, "33474": true, "31838": true, "34749": true, "35529": true, "23137": true, "34202": true, "30446": true, "34986": true, "35459": true, "35734": true, "34157": true, "35580": true, "26887": true, "35767": true, "16422": true, "35810": true, "35851": true, "33690": true, "35374": true, "33840": true, "28348": true, "34689": true, "35170": true, "35150": true, "33929": true, "603": true, "31224": true, "18657": true, "29460": true, "24684": true, "30781": true, "26929": true, "30515": true, "35497": true, "35938": true, "36377": true, "35668": true, "34601": true, "36106": true, "35761": true, "2280": true, "33687": true, "26574": true, "22063": true, "13160": true, "35318": true, "13303": true, "36249": true, "36079": true, "30054": true, "8972": true, "35689": true, "36293": true, "36033": true, "36336": true, "33849": true, "34615": true, "34125": true, "9291": true, "32200": true, "36274": true, "24549": true, "36404": true, "7818": true, "36399": true, "36458": true, "36162": true, "31358": true, "35048": true, "36493": true, "33987": true, "25691": true, "21960": true, "9880": true, "32723": true, "8526": true, "34449": true, "36206": true, "34861": true, "35291": true, "33645": true, "35615": true, "36747": true, "4936": true, "34224": true, "36280": true, "35855": true, "35286": true, "29148": true, "36817": true, "35067": true, "7320": true, "35330": true, "35178": true, "15680": true, "36337": true, "35118": true, "36947": true, "32474": true, "16164": true, "30637": true, "36848": true, "36621": true, "28149": true, "33438": true, "37116": true, "19794": true, "37005": true, "36866": true, "26678": true, "36099": true, "36571": true, "36691": true, "36429": true, "21972": true, "35343": true, "37185": true, "35389": true, "37135": true, "30642": true, "35425": true, "28375": true, "32009": true, "36615": true, "37246": true, "37276": true, "27035": true, "35711": true, "36373": true, "34394": true, "35087": true, "36661": true, "36793": true, "36824": true, "37149": true, "30214": true, "17139": true, "37506": true, "37554": true, "28682": true, "37478": true, "23845": true, "36968": true, "36595": true, "36450": true, "32844": true, "36719": true, "21957": true, "37155": true, "37732": true, "36346": true, "35962": true, "37550": true, "35236": true, "10034": true, "34762": true, "36594": true, "9830": true, "36867": true, "37849": true, "35802": true, "27384": true, "37188": true, "36962": true, "37988": true, "37288": true, "26894": true, "37838": true, "30963": true, "38084": true, "38096": true, "37876": true, "38125": true, "2740": true, "18962": true, "38060": true, "36757": true, "10908": true, "37996": true, "9523": true, "5670": true, "21464": true, "32495": true, "365": true, "35014": true, "30101": true, "38256": true, "37137": true, "37915": true, "36477": true, "32928": true, "36870": true, "38390": true, "36664": true, "37754": true, "5526": true, "38274": true, "37426": true, "37321": true, "37964": true, "38340": true, "38263": true, "37190": true, "38184": true, "38317": true, "35784": true, "38385": true, "12442": true, "33745": true, "33628": true, "24421": true, "37712": true, "8248": true, "36611": true, "37409": true, "38475": true, "23064": true, "35413": true, "33880": true, "38513": true, "38505": true, "38267": true, "38586": true, "38483": true, "38598": true, "38604": true, "36805": true, "23387": true, "38382": true, "38030": true, "31837": true, "34499": true, "37226": true, "29251": true, "36857": true, "36195": true, "37901": true, "22747": true, "25630": true, "34096": true, "31021": true, "38012": true, "38197": true, "37682": true, "38691": true, "4275": true, "36577": true, "36202": true, "37251": true, "27070": true, "16512": true, "38806": true, "38811": true, "37319": true, "9669": true, "38436": true, "38140": true, "33826": true, "38849": true, "21283": true, "38887": true, "36725": true, "38817": true, "37256": true, "38196": true, "29480": true, "38974": true, "36431": true, "38704": true, "39003": true, "36319": true, "18427": true, "37834": true, "8883": true, "38373": true, "39010": true, "34679": true, "39068": true, "35967": true, "37785": true, "25378": true, "39060": true, "37060": true, "28781": true, "38815": true, "39123": true, "39056": true, "13206": true, "30468": true, "18502": true, "37394": true, "37196": true, "22375": true, "39209": true, "36852": true, "37951": true, "34582": true, "38526": true, "34602": true, "36755": true, "38180": true, "13222": true, "39283": true, "38493": true, "39314": true, "27481": true, "1811": true, "29467": true, "22849": true, "38951": true, "2434": true, "39185": true, "39267": true, "37282": true, "38739": true, "38078": true, "38524": true, "24710": true, "39450": true, "39137": true, "35405": true, "39227": true, "21426": true, "22464": true, "34050": true, "18402": true, "39449": true, "33104": true, "39560": true, "28497": true, "39379": true, "38246": true, "38997": true, "38432": true, "39484": true, "39583": true, "38152": true, "39611": true, "12432": true, "39624": true, "14536": true, "38343": true, "39680": true, "38428": true, "38920": true, "39363": true, "39691": true, "39663": true, "37313": true, "39685": true, "27594": true, "27713": true, "36844": true, "24213": true, "39724": true, "39730": true, "370": true, "18186": true, "34066": true, "29173": true, "39739": true, "30730": true, "39140": true, "37723": true, "37200": true, "12761": true, "25581": true, "18338": true, "4102": true, "25205": true, "18445": true, "30530": true, "38570": true, "25046": true, "34097": true, "24804": true, "36764": true, "34965": true, "38295": true, "25842": true, "39171": true, "30808": true, "37306": true, "32779": true, "38669": true, "13992": true, "31893": true, "38881": true, "21492": true, "27037": true, "27617": true, "17292": true, "38201": true, "1170": true, "31105": true, "37019": true, "39716": true, "37593": true, "39735": true, "34255": true, "36821": true, "15453": true, "31186": true, "20335": true, "10680": true, "30443": true, "3555": true, "8356": true, "19822": true, "30269": true, "18755": true, "39428": true, "6304": true, "39755": true, "5385": true, "34821": true, "34994": true, "15293": true, "15214": true, "37375": true, "39694": true, "38315": true, "38820": true, "976": true, "34528": true, "36366": true, "37698": true, "7007": true, "32479": true, "39756": true, "38018": true, "35164": true, "15845": true, "32588": true, "20092": true, "32090": true, "34636": true, "30843": true, "38217": true, "38352": true, "38224": true, "27390": true, "25373": true, "38656": true, "38825": true, "38293": true, "38726": true, "19343": true, "29819": true, "14538": true, "36834": true, "39763": true, "39797": true, "27052": true, "9956": true, "36015": true, "16263": true, "7655": true, "39422": true, "36204": true, "5968": true, "29077": true, "39643": true, "39820": true, "1118": true, "29803": true, "39108": true, "31633": true, "32220": true, "39806": true, "37074": true, "18877": true, "38055": true, "10578": true, "28757": true, "39846": true, "15445": true, "28815": true, "25103": true, "3258": true, "38644": true, "36727": true, "39859": true, "39798": true, "39511": true, "38793": true, "39677": true, "32653": true, "38312": true, "10917": true, "38843": true, "39540": true, "12701": true, "36031": true, "37181": true, "36616": true, "39868": true, "23022": true, "15075": true, "19189": true, "18190": true, "23054": true, "27560": true, "30680": true, "33430": true, "37244": true, "39223": true, "35261": true, "19179": true, "35007": true, "29765": true, "38848": true, "36784": true, "7450": true, "30176": true, "20983": true, "38899": true, "39778": true, "24649": true, "39653": true, "36888": true, "19946": true, "39153": true, "22251": true, "8468": true, "21302": true, "39072": true, "39590": true, "31068": true, "38660": true, "34743": true, "26454": true, "31524": true, "20159": true, "39870": true, "35115": true, "7315": true, "24156": true, "24454": true, "14495": true, "39909": true, "39181": true, "34278": true, "32939": true, "15524": true, "37362": true, "24749": true, "38942": true, "34826": true, "17801": true, "23700": true, "32063": true, "37330": true, "28718": true, "39902": true, "27706": true, "28161": true, "28181": true, "39485": true, "2842": true, "37802": true, "7983": true, "27235": true, "39967": true, "31693": true, "27296": true, "39971": true, "38164": true, "5116": true, "14762": true, "15851": true, "39089": true, "14292": true, "39889": true, "19498": true, "23466": true, "39013": true, "39546": true, "38090": true, "38043": true, "10051": true, "39903": true, "31104": true, "18629": true, "40008": true, "38894": true, "13879": true, "27714": true, "39955": true, "39993": true, "39432": true, "22032": true, "32065": true, "39849": true, "36537": true, "5257": true, "27785": true, "40048": true, "38363": true, "35542": true, "20229": true, "27907": true, "36891": true, "17298": true, "32997": true, "37789": true, "11296": true, "24216": true, "37686": true, "37665": true, "38973": true, "39777": true, "40216": true, "39917": true, "36560": true, "1586": true, "35718": true, "39610": true, "39729": true, "39836": true, "24057": true, "39627": true, "40171": true, "40098": true, "36261": true, "40038": true, "16445": true, "40110": true, "39988": true, "40046": true, "40409": true, "38595": true, "40247": true, "33148": true, "29234": true, "38392": true, "17691": true, "39736": true, "40303": true, "39090": true, "20869": true, "30035": true, "40248": true, "648": true, "38809": true, "4135": true, "35341": true, "36144": true, "40454": true, "15702": true, "39109": true, "40019": true, "40294": true, "40607": true, "37764": true, "40655": true, "40656": true, "40522": true, "40496": true, "39541": true, "40155": true, "38245": true, "39765": true, "38355": true, "38361": true, "40013": true, "40550": true, "19705": true, "40753": true, "39266": true, "40765": true, "40717": true, "40080": true, "39676": true, "28551": true, "37486": true, "11183": true, "40821": true, "18219": true, "40529": true, "10077": true, "31179": true, "40756": true, "40849": true, "3242": true, "40874": true, "28478": true, "40725": true, "40885": true, "39929": true, "40236": true, "40095": true, "36534": true, "40836": true, "40927": true, "16369": true, "40062": true, "40965": true, "30464": true, "39830": true, "34881": true, "39905": true, "36731": true, "40856": true, "40999": true, "40220": true, "34045": true, "2864": true, "24126": true, "27601": true, "16249": true, "39662": true, "41098": true, "41048": true, "40848": true, "29541": true, "40026": true, "40944": true, "41088": true, "8493": true, "31627": true, "11101": true, "39681": true, "41125": true, "41182": true, "41186": true, "40973": true, "26610": true, "8871": true, "41164": true, "36407": true, "32532": true, "40881": true, "41051": true, "20097": true, "19735": true, "41141": true, "40906": true, "34992": true, "40375": true, "28433": true, "14359": true, "40227": true, "9244": true, "40408": true, "40614": true, "40794": true, "8937": true, "40666": true, "34017": true, "41332": true, "40745": true, "23436": true, "40964": true, "40138": true, "41354": true, "40140": true, "36647": true, "37891": true, "40937": true, "41435": true, "33045": true, "40420": true, "21027": true, "23126": true, "41232": true, "27919": true, "39383": true, "41026": true, "39812": true, "39953": true, "37312": true, "41316": true, "28472": true, "33676": true, "41306": true, "33819": true, "41591": true, "41567": true, "38958": true, "41651": true, "10751": true, "41720": true, "33719": true, "41162": true, "38447": true, "36889": true, "41737": true, "41780": true, "35445": true, "32497": true, "41250": true, "41730": true, "41634": true, "13914": true, "19114": true, "11142": true, "41155": true, "3721": true, "40679": true, "41747": true, "38212": true, "41796": true, "41881": true, "27807": true, "37168": true, "41341": true, "30928": true, "41457": true, "41705": true, "41873": true, "41905": true, "40654": true, "41911": true, "41950": true, "41791": true, "41753": true, "40483": true, "41335": true, "9982": true, "40843": true, "38944": true, "32005": true, "26161": true, "37544": true, "42059": true, "42026": true, "42040": true, "35237": true, "42064": true, "39981": true, "3635": true, "39649": true, "42111": true, "1264": true, "13156": true, "40747": true, "41394": true, "39019": true, "42152": true, "42202": true, "19325": true, "40872": true, "38707": true, "6211": true, "33960": true, "40733": true, "25615": true, "40895": true, "40834": true, "28775": true, "42295": true, "42222": true, "26061": true, "42314": true, "42349": true, "38333": true, "21035": true, "28641": true, "42336": true, "42356": true, "42127": true, "28215": true, "41963": true, "866": true, "42493": true, "40146": true, "42496": true, "42357": true, "26606": true, "42082": true, "41863": true, "42563": true, "41677": true, "41855": true, "21269": true, "42584": true, "40059": true, "31028": true, "42636": true, "4071": true, "42400": true, "41401": true, "42570": true, "41386": true, "42662": true, "27569": true, "10368": true, "42590": true, "41226": true, "37920": true, "40788": true, "42694": true, "42764": true, "31088": true, "42607": true, "10449": true, "41887": true, "10363": true, "32878": true, "42711": true, "42879": true, "34585": true, "42902": true, "30853": true, "41817": true, "29557": true, "42175": true, "42758": true, "40441": true, "40035": true, "42930": true, "42008": true, "35192": true, "42028": true, "41790": true, "43012": true, "42334": true, "42337": true, "17803": true, "39919": true, "42689": true, "41592": true, "41940": true, "3859": true, "42197": true, "42716": true, "43147": true, "43092": true, "13710": true, "41543": true, "40531": true, "41832": true, "41924": true, "43110": true, "41395": true, "39648": true, "38700": true, "42783": true, "37822": true, "18813": true, "36770": true, "25897": true, "43246": true, "41167": true, "42113": true, "43355": true, "41845": true, "43265": true, "27328": true, "42929": true, "41783": true, "36416": true, "38536": true, "42795": true, "43378": true, "40509": true, "38525": true, "43303": true, "33626": true, "43055": true, "43486": true, "39545": true, "42638": true, "43393": true, "41879": true, "18537": true, "42404": true, "39040": true, "42530": true, "43254": true, "16649": true, "43511": true, "41227": true, "42161": true, "35504": true, "37406": true, "41871": true, "42718": true, "27038": true, "42058": true, "42811": true, "42299": true, "43685": true, "43701": true, "43282": true, "4473": true, "35506": true, "35574": true, "43178": true, "41946": true, "43326": true, "42440": true, "40769": true, "30690": true, "19541": true, "42143": true, "43756": true, "40219": true, "34211": true, "43171": true, "38290": true, "43749": true, "42540": true, "25895": true, "43523": true, "38554": true, "33203": true, "42434": true, "33058": true, "36247": true, "3965": true, "43721": true, "39614": true, "42574": true, "23267": true, "29499": true, "43959": true, "41775": true, "41575": true, "1158": true, "40389": true, "9616": true, "44052": true, "38241": true, "10117": true, "43025": true, "44104": true, "39807": true, "44118": true, "42458": true, "43446": true, "40121": true, "37939": true, "43808": true, "43208": true, "44090": true, "21697": true, "19446": true, "30351": true, "40525": true, "43748": true, "44284": true, "26986": true, "44009": true, "44143": true, "34075": true, "44074": true, "44106": true, "43852": true, "169": true, "44304": true, "41190": true, "43779": true, "44417": true, "43579": true, "43824": true, "43449": true, "43938": true, "43376": true, "41711": true, "43469": true, "37024": true, "44014": true, "44511": true, "44498": true, "41680": true, "43432": true, "42921": true, "40773": true, "30873": true, "39004": true, "44592": true, "42502": true, "43809": true, "44529": true, "44024": true, "42861": true, "15980": true, "44168": true, "43401": true, "42641": true, "44665": true, "44648": true, "44519": true, "44393": true, "44056": true, "44517": true, "44621": true, "44546": true, "42873": true, "44501": true, "27834": true, "338": true, "43712": true, "44909": true, "43397": true, "44825": true, "44895": true, "27050": true, "44488": true, "32926": true, "44934": true, "44360": true, "44713": true, "43864": true, "43834": true, "33720": true, "43302": true, "43687": true, "44896": true, "38953": true, "43843": true, "39938": true, "45089": true, "44701": true, "41626": true, "43063": true, "44856": true, "20127": true, "8187": true, "43876": true, "44986": true, "45138": true, "25263": true, "45188": true, "43924": true, "45156": true, "45126": true, "1481": true, "45220": true, "45223": true, "45256": true, "45200": true, "44327": true, "45269": true, "26356": true, "42746": true, "45255": true, "23888": true, "43175": true, "40069": true, "44107": true, "16844": true, "44275": true, "16873": true, "31635": true, "31199": true, "45203": true, "44340": true, "40923": true, "3840": true, "10949": true, "44765": true, "45280": true, "41325": true, "44770": true, "29569": true, "43345": true, "44602": true, "23519": true, "43603": true, "45174": true, "45476": true, "10467": true, "44688": true, "44156": true, "45450": true, "13509": true, "45298": true, "45505": true, "41579": true, "44276": true, "11635": true, "41429": true, "13351": true, "45222": true, "35363": true, "44756": true, "44582": true, "45150": true, "43943": true, "32899": true, "45446": true, "31647": true, "18366": true, "32895": true, "45522": true, "45032": true, "44334": true, "31413": true, "45512": true, "45207": true, "36797": true, "11596": true, "40817": true, "45792": true, "45628": true, "45806": true, "37413": true, "45873": true, "45882": true, "45903": true, "45798": true, "41091": true, "44523": true, "45856": true, "45422": true, "22558": true, "31251": true, "40903": true, "45454": true, "44878": true, "45971": true, "44824": true, "29287": true, "44889": true, "41941": true, "31154": true, "46007": true, "38281": true, "40428": true, "45492": true, "36772": true, "40404": true, "44414": true, "29581": true, "45520": true, "8738": true, "45823": true, "43707": true, "36379": true, "41571": true, "44119": true, "46140": true, "36226": true, "20687": true, "20709": true, "22706": true, "46221": true, "44709": true, "13873": true, "16214": true, "21739": true, "45518": true, "40407": true, "45101": true, "46159": true, "46283": true, "46314": true, "45901": true, "41006": true, "46205": true, "45917": true, "46064": true, "46293": true, "45236": true, "45818": true, "33859": true, "42842": true, "44758": true, "45752": true, "44324": true, "27984": true, "46334": true, "46065": true, "46039": true, "41906": true, "42210": true, "41674": true, "46530": true, "43528": true, "18819": true, "13197": true, "46557": true, "45145": true, "46006": true, "45822": true, "40741": true, "46610": true, "46609": true, "45642": true, "14143": true, "46638": true, "45661": true, "46654": true, "46619": true, "44465": true, "43753": true, "45825": true, "12924": true, "40746": true, "46249": true, "46193": true, "46748": true, "37386": true, "46196": true, "39464": true, "46176": true, "43492": true, "42835": true, "45618": true, "46514": true, "45648": true, "45567": true, "45448": true, "46754": true, "44230": true, "45154": true, "46872": true, "46840": true, "46843": true, "46669": true, "29374": true, "46887": true, "32669": true, "31613": true, "46913": true, "42942": true, "45617": true, "46460": true, "46631": true, "46784": true, "41757": true, "31093": true, "45627": true, "46687": true, "46428": true, "40649": true, "27219": true, "46547": true, "26111": true, "46989": true, "27433": true, "46599": true, "15497": true, "34737": true, "40077": true, "18608": true, "46294": true, "46979": true, "47061": true, "44102": true, "46675": true, "27758": true, "46342": true, "45848": true, "47073": true, "14589": true, "47028": true, "47052": true, "10624": true, "45047": true, "47124": true, "46997": true, "47183": true, "43684": true, "46013": true, "47235": true, "45373": true, "13279": true, "46802": true, "22916": true, "36533": true, "20472": true, "41883": true, "47074": true, "33651": true, "14651": true, "46192": true, "42432": true, "47314": true, "44245": true, "29389": true, "33380": true, "30306": true, "47157": true, "46565": true, "41805": true, "47199": true, "44268": true, "45283": true, "45141": true, "16304": true, "45847": true, "47447": true, "46757": true, "47422": true, "46092": true, "47366": true, "47495": true, "18413": true, "47505": true, "45674": true, "44472": true, "36877": true, "45664": true, "46646": true, "11580": true, "46045": true, "47584": true, "47552": true, "46386": true, "47268": true, "47094": true, "47385": true, "23610": true, "47583": true, "47642": true, "45638": true, "26449": true, "46479": true, "44958": true, "47578": true, "18807": true, "34639": true, "47738": true, "47507": true, "35006": true, "47754": true, "46714": true, "47817": true, "46126": true, "47778": true, "47895": true, "46602": true, "47341": true, "47907": true, "47223": true, "46817": true, "45914": true, "48012": true, "47381": true, "47906": true, "47855": true, "45828": true, "39872": true, "38770": true, "11557": true, "47719": true, "23008": true, "9962": true, "47539": true, "47530": true, "48220": true, "47813": true, "47998": true, "48224": true, "47072": true, "46537": true, "47909": true, "45675": true, "48277": true, "38213": true, "47369": true, "7871": true, "27802": true, "47435": true, "1473": true, "44242": true, "47746": true, "44190": true, "48284": true, "48034": true, "31587": true, "47023": true, "46543": true, "48153": true, "15119": true, "21682": true, "8590": true, "45938": true, "38437": true, "48382": true, "46813": true, "48027": true, "48432": true, "48008": true, "46482": true, "48478": true, "47064": true, "34099": true, "48496": true, "28752": true, "48457": true, "46484": true, "47441": true, "48161": true, "48527": true, "36194": true, "47802": true, "47892": true, "48395": true, "19450": true, "27247": true, "46060": true, "48469": true, "38786": true, "48444": true, "48439": true, "48320": true, "47884": true, "48652": true, "48258": true, "47875": true, "47248": true, "48690": true, "45720": true, "48597": true, "48447": true, "48430": true, "23969": true, "31676": true, "47090": true, "44159": true, "48152": true, "47897": true, "47707": true, "40199": true, "27076": true, "48723": true, "48767": true, "25740": true, "48517": true, "48777": true, "48551": true, "47824": true, "16154": true, "24918": true, "48511": true, "20285": true, "48678": true, "48822": true, "46160": true, "23782": true, "46768": true, "48504": true, "48096": true, "48747": true, "45461": true, "10785": true, "29494": true, "14812": true, "7441": true, "48871": true, "47212": true, "48932": true, "48938": true, "35818": true, "46769": true, "18844": true, "47905": true, "19031": true, "48960": true, "38576": true, "47928": true, "10836": true, "37029": true, "49014": true, "47606": true, "48672": true, "48557": true, "47260": true, "18911": true, "46663": true, "47171": true, "46928": true, "34665": true, "43206": true, "49045": true, "48147": true, "44975": true, "43899": true, "47632": true, "48336": true, "49054": true, "48223": true, "34087": true, "47645": true, "47667": true, "34541": true, "40158": true, "43644": true, "48811": true, "16896": true, "49142": true, "40787": true, "48716": true, "49169": true, "49174": true, "48229": true, "48000": true, "18080": true, "48741": true, "48459": true, "41069": true, "49218": true, "48169": true, "43785": true, "49238": true, "47288": true, "48294": true, "47977": true, "47962": true, "8521": true, "40274": true, "47560": true, "26749": true, "36221": true, "49051": true, "47620": true, "49235": true, "48713": true, "48816": true, "38306": true, "15096": true, "35474": true, "49278": true, "49381": true, "7968": true, "48337": true, "36971": true, "47444": true, "48506": true, "49037": true, "47866": true, "38566": true, "49285": true, "45319": true, "48936": true, "32536": true, "47614": true, "49249": true, "49412": true, "44939": true, "38207": true, "49209": true, "5505": true, "8953": true, "49550": true, "49518": true, "49572": true, "47133": true, "28305": true, "49292": true, "49471": true, "49066": true, "48899": true, "49615": true, "48663": true, "49033": true, "49444": true, "47077": true, "42734": true, "14023": true, "49164": true, "49653": true, "48774": true, "49291": true, "45707": true, "37955": true, "47749": true, "38928": true, "49401": true, "49978": true, "50154": true, "52391": true, "53084": true, "2147483647": true, "50908": true, "25416": true });
})();
(() => {
  angular.module('MyPTHub').config(config).controller('AppController', controller);

  function config($stateProvider) {
    $stateProvider.state('app', {
      abstract: true,
      templateUrl: 'core/app.html',
      controller: 'AppController as vm'
    });
  }

  function controller($mdUtil, $mdSidenav, $ionicHistory, $ionicPlatform, $mdDialog, $mdBottomSheet, $mdMenu, $mdSelect, NavigateService, PermissionsService, AuthenticationService, AuthenticationStateService, ConfirmDialogService, AvatarUploadService, $scope, $rootScope, ScheduleDayPickerService, DayPickerService, LogoService, SessionLoggerService, MessagesService, SubscriptionStateService) {
    const vm = this;

    vm.toggleLeft = $mdUtil.debounce(() => {
      $mdSidenav('left').toggle();
    }, 0);

    $scope.$on('user:updated', () => {
      setup();
    });

    const setup = resolve => {
      const currentUser = AuthenticationStateService.getCurrentUser();
      vm.isTrainer = PermissionsService.userIsTrainer();
      vm.currentUser = currentUser;
      vm.logoUrl = LogoService.getLogo();
      vm.enablePackages = PermissionsService.merchantProvisioned();
      vm.schedulerEnabled = PermissionsService.schedulerIsEnabled();
      vm.ultimatePlusEnabled = SubscriptionStateService.hasUnlimitedPlus();
      if (vm.isTrainer) {
        if (currentUser.hasOwnProperty('permissions_active')) {
          vm.packagesActive = currentUser.permissions_active.can_view_packages && currentUser.merchant_is_provisioned;
        } else {
          vm.packagesActive = true;
        }
      } else {
        vm.packagesActive = true;
      }
      if (currentUser.hasOwnProperty('features')) {
        vm.canMessage = AuthenticationStateService.getCurrentUser().features.messages;
      } else {
        vm.canMessage = true;
      }

      if (typeof resolve !== 'undefined') {
        resolve(true);
      }
    };

    AuthenticationService.registerLoginSuccessEventHandler(setup, true);
    if (AuthenticationStateService.isLoggedIn()) {
      setup();
    }

    SubscriptionStateService.registerChangeEventHandler($scope, () => {
      vm.schedulerEnabled = PermissionsService.schedulerIsEnabled();
    });

    AvatarUploadService.registerUploadSuccessEventHandler($scope, () => {
      vm.currentUser = AuthenticationStateService.getCurrentUser();
    });

    vm.unseenMessages = !MessagesService.isSeen();
    MessagesService.onSeenStateChange($scope, (ev, isSeen) => {
      vm.unseenMessages = !isSeen;
    });

    vm.navigateTo = (stateName, isHome = false, data = false) => {
      $mdSidenav('left').close().then(() => {
        const nav = NavigateService.disableAnimation();

        if (isHome) {
          nav.disableBack().isRoot();
        }

        nav.go(stateName, data);
      });
    };

    vm.goToCalendar = () => {
      let stateName = 'app.client_calendar';
      if (vm.isTrainer) {
        stateName = 'app.trainer_calendar';
      }

      ScheduleDayPickerService.reset();

      vm.navigateTo(stateName);
    };

    // closeSideNav is for close side navigation
    // It will use with event on-swipe-left="closeSideNav()" on-drag-left="closeSideNav()"
    // When user swipe or drag md-sidenav to left side
    vm.closeSideNav = () => {
      $mdSidenav('left').close();
    };
    // End closeSideNav

    //  $ionicPlatform.registerBackButtonAction(callback, priority, [actionId])
    //
    //     Register a hardware back button action. Only one action will execute
    //  when the back button is clicked, so this method decides which of
    //  the registered back button actions has the highest priority.
    //
    //     For example, if an actionsheet is showing, the back button should
    //  close the actionsheet, but it should not also go back a page view
    //  or close a modal which may be open.
    //
    //  The priorities for the existing back button hooks are as follows:
    //  Return to previous view = 100
    //  Close side menu         = 150
    //  Dismiss modal           = 200
    //  Close action sheet      = 300
    //  Dismiss popup           = 400
    //  Dismiss loading overlay = 500
    //
    //  Your back button action will override each of the above actions
    //  whose priority is less than the priority you provide. For example,
    //  an action assigned a priority of 101 will override the ‘return to
    //  previous view’ action, but not any of the other actions.
    //
    //  Learn more at : http://ionicframework.com/docs/api/service/$ionicPlatform/#registerBackButtonAction

    $ionicPlatform.registerBackButtonAction(() => {
      if (DayPickerService.isOpen()) {
        DayPickerService.close(true);
      } else if (SessionLoggerService.isActive()) {
        SessionLoggerService.quit();
      } else if (ScheduleDayPickerService.isOpen()) {
        ScheduleDayPickerService.close(true);
      } else if ($mdSidenav('left').isOpen()) {
        // If side navigation is open it will close and then return
        $mdSidenav('left').close();
      } else if (jQuery('md-bottom-sheet').length > 0) {
        // If bottom sheet is open it will close and then return
        $mdBottomSheet.cancel();
      } else if (jQuery('[id^=dialog]').length > 0) {
        // If popup dialog is open it will close and then return
        $mdDialog.cancel();
      } else if (jQuery('md-menu-content').length > 0) {
        // If md-menu is open it will close and then return
        $mdMenu.hide();
      } else if (jQuery('md-select-menu').length > 0) {
        // If md-select is open it will close and then return
        $mdSelect.hide();
      } else if (jQuery('.fabBackdrop').length > 0) {
        // If fab is open it will close and then return
        $rootScope.$emit('closeFAB', {});
      } else {
        // If control :
        // side navigation,
        // bottom sheet,
        // popup dialog,
        // md-menu,
        // md-select
        // is not opening, It will show $mdDialog to ask for
        // Confirmation to close the application or go to the view of lasted state.

        // Check for the current state that not have previous state.
        // It will show $mdDialog to ask for Confirmation to close the application.

        if ($ionicHistory.backView() == null) {
          // Check is popup dialog is not open.
          if (jQuery('[id^=dialog]').length == 0) {
            // mdDialog for show $mdDialog to ask for
            // Confirmation to close the application.

            ConfirmDialogService.open({
              title: 'Confirm exit',
              message: 'Do you want to close the app?',
              buttonConfirm: 'Confirm',
              buttonCancel: 'Cancel'
            }).then(() => {
              // If user tap Confirm at the popup dialog.
              // Application will close.
              ionic.Platform.exitApp();
            }, () => {
              // For cancel button actions.
            }); // End mdDialog
          }
        } else {
          // Go to the view of lasted state.
          $ionicHistory.goBack();
        }
      }
    }, 100);
    // End of $ionicPlatform.registerBackButtonAction
  }
})();
(() => {
  class ClientEventsController {
    /**
     * @param ApiClientService
     * @param bookings
     * @param $ionicActionSheet
     * @param NavigateService
     */
    constructor(ApiClientService, bookings, $ionicActionSheet, NavigateService) {
      this.ApiClientService = ApiClientService;
      this.bookings = bookings;
      this.$ionicActionSheet = $ionicActionSheet;
      this.NavigateService = NavigateService;

      this.futureBookings = this.bookings.filter(booking => moment(booking.event_date).isAfter(moment()));
      this.pastBookings = this.bookings.filter(booking => moment(booking.event_date).isBefore(moment()));
    }

    openBookingMenu(booking) {
      this.$ionicActionSheet.show({
        buttons: [{
          text: 'View Booking',
          action: () => {
            this.NavigateService.go('app.trainer_calendar_event_booking', {
              eventId: booking.event_id,
              eventTimeId: booking.event_time_id,
              date_time: booking.event_date
            });
          }
        }],
        cancelText: 'Cancel',
        buttonClicked(index) {
          if (typeof this.buttons[index] !== 'undefined' && typeof this.buttons[index].action === 'function') {
            this.buttons[index].action();
          }
          return true;
        }
      });
    }
  }

  const config = $stateProvider => {
    $stateProvider.state('app.client_events', {
      url: '/client/events',
      views: {
        menuContent: {
          templateUrl: 'client/events/views/events.html',
          controller: 'ClientEventsController',
          controllerAs: 'vm'
        }
      },
      resolve: {
        bookings(ApiClientService, AuthenticationStateService) {
          const cId = AuthenticationStateService.getCurrentUser().customer_id;
          return ApiClientService.calendarEventsBookings.getCustomerBookings(cId).then(response => response.bookings);
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('ClientEventsController', ClientEventsController);
})();
(function () {
  'use strict';

  angular.module('MyPTHub').config(config).controller('ClientFormsController', controller);

  config.$inject = ['$stateProvider'];

  function config($stateProvider) {

    $stateProvider.state('app.client_forms', {
      url: "/client_forms",
      views: {
        'menuContent': {
          templateUrl: "client/forms/forms.html",
          controller: 'ClientFormsController as vm'
        }
      },
      resolve: {
        forms(PermissionsService, ApiClientService) {
          const customerId = PermissionsService.getCurrentUserEntityId();

          return ApiClientService.forms.getAllForCustomer(customerId).then(response => response.forms);
        }
      }
    });
  }

  controller.$inject = ['$scope', 'PermissionsService', 'ApiClientService', '$filter', 'NavigateService', 'AnalyticsService', 'forms'];

  function controller($scope, PermissionsService, ApiClientService, $filter, NavigateService, AnalyticsService, forms) {
    var vm = this;
    var customerId = PermissionsService.getCurrentUserEntityId();

    var loadAll = function (refreshing) {
      refreshing = refreshing || false;

      if (refreshing) {
        return ApiClientService.forms.getAllForCustomer(customerId).then(response => {
          vm.forms = response.forms;
          vm.filteredForms = angular.copy(response.forms);
          $scope.$broadcast('scroll.refreshComplete');
        });
      }

      vm.noForms = false;

      vm.formsSearchFilter = '';

      vm.forms = forms;
      vm.filteredForms = angular.copy(forms);

      if (!forms.length) {
        vm.noForms = true;
      }
    };

    vm.refresh = function () {
      loadAll(true);
    };

    vm.viewForm = function (form) {
      NavigateService.go('app.view_customer_form', {
        customerId: customerId,
        formId: form.id,
        readonly: form.completed
      });
    };

    // All Forms Search filter
    $scope.$watch(function () {
      return vm.formsSearchFilter;
    }, function (newVal, oldVal) {

      if (typeof newVal === 'undefined') {
        return false;
      }

      if (newVal.length > 0) {

        vm.filteredForms = $filter('filter')(vm.forms, {
          title: newVal
        });
      } else {
        vm.filteredForms = vm.forms;
      }
    });

    $scope.$on('$ionicView.beforeEnter', function () {
      vm.forms = forms;

      loadAll();
    });

    $scope.$on('$ionicView.enter', function () {
      AnalyticsService.trackView('Client - Forms');
    });
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('ClientHomeController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.client_home', {
            url: "/client_home",
            params: {
                isAnimated: false,
                historyRoot: true
            },
            views: {
                'menuContent': {
                    templateUrl: "client/home/home.html",
                    controller: 'ClientHomeController as vm'
                }
            }
        });
    }

    var vm;

    controller.$inject = ['$scope', 'SessionStoreService', 'SyncManagerService', 'NutritionLogService', 'AnalyticsService', 'SideMenuService', '$mdSidenav', 'AuthenticationStateService', 'NavigateService', '$ionicScrollDelegate', 'NotificationsService', 'SubscriptionStateService', 'ApiClientService'];

    function controller($scope, SessionStoreService, SyncManagerService, NutritionLogService, AnalyticsService, SideMenuService, $mdSidenav, AuthenticationStateService, NavigateService, $ionicScrollDelegate, NotificationsService, SubscriptionStateService, ApiClientService) {
        vm = this;
        vm.apiConnected = null;
        vm.trainerSubscribed = SubscriptionStateService.trainerIsSubscribed();

        vm.refresh = function () {
            if (!vm.canRefresh) {
                $scope.$broadcast('scroll.refreshComplete');
                $ionicScrollDelegate.resize();
                return false;
            }
            vm.loadActivityLog().finally(function () {
                $scope.$broadcast('scroll.refreshComplete');
            });
        };

        vm.startShow = function () {
            vm.canRefresh = false;
        };

        vm.activityLogShown = false;
        vm.activityLogShow = function () {
            if (vm.activityLogShown) {
                vm.canRefresh = true;
                return false;
            }

            vm.loadActivityLog().finally(function () {
                vm.canRefresh = true;
                vm.activityLogShown = true;
            });
        };

        var updateSyncingStatus = function () {

            vm.unsyncedSessions = 0;

            SessionStoreService.countUnSyncedSessions().then(function (count) {
                vm.unsyncedSessions = count;
            });
        };

        SessionStoreService.registerEventHandler($scope, 'offline_sessions_syncing', updateSyncingStatus);

        SessionStoreService.registerEventHandler($scope, 'local_session_deleted', updateSyncingStatus);

        SessionStoreService.registerEventHandler($scope, 'offline_sessions_synced', function () {
            updateSyncingStatus();
            vm.refresh();
        });

        SessionStoreService.registerEventHandler($scope, 'online_session_synced', vm.refresh);

        vm.openSyncManager = function () {
            SyncManagerService.open();
        };

        // Do on every load despite cache
        $scope.$on("$ionicView.beforeEnter", function () {

            var today = moment();

            vm.todayDate = {
                day: today.format('D'),
                month: today.format('MMMM')
            };

            updateSyncingStatus();
        });

        // Do on every load despite cache
        $scope.$on("$ionicView.enter", function (scopes, states) {

            vm.currentUser = AuthenticationStateService.getCurrentUser();

            AnalyticsService.trackView('Client - Home');

            if (!SideMenuService.hasSeenSideMenu()) {
                $mdSidenav('left').open();
                SideMenuService.sideMenuSeen();
            }

            NotificationsService.load();

            if (vm.trainerSubscribed) {
                checkFitbitAvailability();
            };
        });

        NotificationsService.onDirty($scope, function () {
            NotificationsService.load();
        });

        vm.logNutrition = function () {
            vm.startingAction = true;
            NutritionLogService.startLogger().finally(function () {
                vm.startingAction = false;
            });
        };

        vm.goToSettings = function () {
            NavigateService.go('app.client_settings');
        };

        vm.startSession = function () {
            NavigateService.go('app.start_adhoc_session');
        };

        vm.addPhoto = function () {
            NavigateService.go('app.client_progress_photos');
        };

        vm.updateMetrics = function () {
            NavigateService.go('app.client_update_metrics');
        };

        const checkFitbitAvailability = () => {
            ApiClientService.hardware.checkAuthentication(vm.currentUser.id).success(function (response) {
                if (response == 'true') {
                    vm.apiConnected = true;
                } else {
                    vm.apiConnected = false;
                }
            });
        };
    }
})();
// Intro Controller
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('ClientIntroController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.client_intro', {
            url: "/client_intro",
            cache: false,
            params: {
                isAnimated: false
            },
            views: {
                'menuContent': {
                    templateUrl: "client/intro/intro.html",
                    controller: 'ClientIntroController as vm'
                }
            }
        });
    }

    controller.$inject = ['$ionicSlideBoxDelegate', 'IntroService', 'NavigateService', 'PermissionsService'];

    function controller($ionicSlideBoxDelegate, IntroService, NavigateService, PermissionsService) {

        var vm = this;

        vm.businessName = PermissionsService.getCurrentUser().branding.business_name;

        vm.slideIndex = 0;

        vm.start = function () {
            NavigateService.disableBack().go('app.client_home');
        };

        vm.next = function () {
            if (vm.slideIndex != 6) {
                $ionicSlideBoxDelegate.next();
            } else if (vm.slideIndex === 6) {
                vm.start();
            }
        };

        vm.previous = function () {

            if (vm.slideIndex === 0) {
                vm.start();
            } else {
                $ionicSlideBoxDelegate.previous();
            }
        };

        // Called each time the slide changes
        vm.slideChanged = function (index) {
            vm.slideIndex = index;
        };

        // Ensure we set the flag in local storage that the intro has been seen
        // so it doesn't run again unless local storage is killed for any reason
        IntroService.introSeen();
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('ClientIntroService', service);

    service.$inject = ['LocalStorageService', 'BrandingService'];

    function service(LocalStorageService, BrandingService) {

        var service = {};

        var state = {
            seen: false
        };

        var storeState = function () {

            LocalStorageService.setKey('introState', JSON.stringify(state));
        };

        var getStoredState = function () {

            var getData = LocalStorageService.getKey('introState');

            if (!getData) {
                return false;
            }

            return JSON.parse(getData);
        };

        service.setup = function () {

            var storedState = getStoredState();

            if (storedState) {

                state.seen = storedState.seen;
            }
        };

        service.hasSeenIntro = function () {

            if (BrandingService.isCustomApp()) {
                return true;
            }

            return state.seen;
        };

        service.introSeen = function () {

            state.seen = true;

            storeState();
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('NotesController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.client_notes', {
            url: "/notes",
            views: {
                'menuContent': {
                    templateUrl: "client/notes/notes.html",
                    controller: 'NotesController as vm'
                }
            }
        });
    }

    controller.$inject = ['ApiClientService', '$scope', '$ionicActionSheet', 'ModalService', 'PermissionsService', 'ToastService', 'ConfirmDialogService'];

    function controller(ApiClientService, $scope, $ionicActionSheet, ModalService, PermissionsService, ToastService, ConfirmDialogService) {

        var vm = this;
        vm.customer = PermissionsService.getCurrentUser();

        var loadNotes = function (refresh) {

            refresh = refresh || false;

            if (!refresh && vm.notes) {
                return false;
            }

            return ApiClientService.customers.getNotes(PermissionsService.getCurrentUserEntityId()).success(function (response) {
                vm.notes = response.notes;
                vm.noNotes = response.notes.length === 0;
            });
        };

        vm.refresh = function () {
            var refresh = loadNotes(true);

            if (refresh) {
                refresh.finally(refreshComplete);
                return true;
            }

            refreshComplete();
        };

        vm.notesShow = function () {
            resetShown();
            loadNotes();
        };

        vm.showSheet = function (note) {
            // Show the action sheet
            var hideSheet = $ionicActionSheet.show({
                buttons: [{ text: 'Edit' }],
                destructiveText: 'Delete',
                titleText: 'Manage Note',
                cancelText: 'Cancel',
                buttonClicked: function (index) {
                    // Edit button
                    ModalService.show({
                        templateUrl: "common/customerNoteForm/customerNoteFormModal.html",
                        controller: 'CustomerNoteFormModalController as vm',
                        params: {
                            customer_id: vm.customer.customer_id,
                            note: note
                        }
                    }).then(function (note) {

                        loadNotes(true);
                    });

                    return true;
                },
                destructiveButtonClicked: function () {
                    // Delete button
                    ConfirmDialogService.open({
                        title: "Confirm Delete",
                        message: "Are you sure you want to delete this note?",
                        buttonConfirm: "Delete",
                        buttonCancel: "Cancel"
                    }).then(function () {
                        ApiClientService.customers.deleteNote(vm.customer.customer_id, note.id).success(function () {
                            ToastService.show('Note successfully deleted');
                        });

                        vm.notes.splice(vm.notes.indexOf(note), 1);
                    });

                    return true;
                }
            });
        };

        vm.showCreateNoteForm = function () {
            ModalService.show({
                templateUrl: "common/customerNoteForm/customerNoteFormModal.html",
                controller: 'CustomerNoteFormModalController as vm',
                params: {
                    customer_id: vm.customer.customer_id
                }
            }).then(function (note) {

                loadNotes(true);
            });
        };

        $scope.$on('$ionicView.beforeEnter', function () {

            loadNotes();
        });

        var refreshComplete = function () {
            $scope.$broadcast('scroll.refreshComplete');
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('ClientNutritionController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.client_nutrition', {
            url: "/client_nutrition",
            views: {
                'menuContent': {
                    templateUrl: "client/nutrition/nutrition.html",
                    controller: 'ClientNutritionController as vm'
                }
            }
        });
    }

    controller.$inject = ['$scope', 'ApiClientService', '$filter', 'PermissionsService', 'NavigateService', 'AnalyticsService', 'NutritionLogService'];

    function controller($scope, ApiClientService, $filter, PermissionsService, NavigateService, AnalyticsService, NutritionLogService) {

        var vm = this;

        var customerId = PermissionsService.getCurrentUserEntityId();

        var load = function (refreshing) {

            refreshing = refreshing || false;

            if (!vm.allNutritionPlans || refreshing) {

                vm.allNutritionPlansSearchFilter = '';

                ApiClientService.customers.listNutritionPlans(customerId, false).success(function (response) {

                    vm.allNutritionPlans = response.nutritionPlans;
                    vm.filteredAllNutritionPlans = angular.copy(response.nutritionPlans);

                    vm.noAllNutritionPlans = vm.allNutritionPlans.length === 0;
                }).finally(function () {
                    if (refreshing) {
                        $scope.$broadcast('scroll.refreshComplete');
                    }
                });
            }
        };

        // All Nutrition Plans Search filter
        $scope.$watch(function () {
            return vm.allNutritionPlansSearchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.filteredAllNutritionPlans = $filter('filter')(vm.allNutritionPlans, {
                    title: newVal
                });
            } else {
                vm.filteredAllNutritionPlans = vm.allNutritionPlans;
            }
        });

        vm.refresh = function () {
            load(true);
        };

        vm.viewNutritionPlan = function (nutritionPlan) {
            NavigateService.go('app.view_nutrition_plan', {
                nutritionPlanId: nutritionPlan.id
            });
        };

        vm.logNutrition = function () {
            NutritionLogService.startLogger();
        };

        $scope.$on('$ionicView.beforeEnter', function () {
            load();
        });

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Client - Nutrition');
        });
    }
})();
(() => {
  'use strict';

  angular.module('MyPTHub').config(config).controller('ClientNutritionLogController', controller);

  config.$inject = ['$stateProvider'];

  function config($stateProvider) {
    $stateProvider.state('app.client_log_nutrition', {
      url: '/nutritionlog/log',
      views: {
        'menuContent': {
          templateUrl: 'client/nutritionLog/nutritionLog.html',
          controller: 'ClientNutritionLogController as vm'
        }
      }
    });
  }

  controller.$inject = ['$scope', 'NutritionEditorService', 'NutritionLogEditService', 'NavigateService', 'PromptDialogService', 'DayPickerService', '$q', 'ApiClientService', 'PermissionsService', 'NoteViewerModalService', 'AnalyticsService', 'ToastService', 'ScheduleEventStoreService', 'ConfirmDialogService', 'NutritionLogService', 'NotificationsService', 'ModalService'];

  function controller($scope, NutritionEditorService, NutritionLogEditService, NavigateService, PromptDialogService, DayPickerService, $q, ApiClientService, PermissionsService, NoteViewerModalService, AnalyticsService, ToastService, ScheduleEventStoreService, ConfirmDialogService, NutritionLogService, NotificationsService, ModalService) {

    var vm = this;
    vm.datePickerToggle = DayPickerService.toggleOpen;

    vm.doUpdate = function () {
      vm.saving = true;
      NutritionLogEditService.saveChanges().then(function () {
        NutritionEditorService.reset();
        ScheduleEventStoreService.setDirty();
        NotificationsService.dirty();
        NavigateService.goBack();
        ToastService.show('Saved Nutrition Log');
        vm.saving = false;
      }, function () {
        vm.saving = false;
      });
    };

    vm.addMeal = function () {
      PromptDialogService.open({
        title: 'Add Meal',
        message: 'Please enter a name for the meal',
        buttonConfirm: 'Create',
        buttonCancel: 'Cancel',
        required: true
      }).then(function (name) {
        vm.nutritionLogDay.addMeal(name);
        vm.nutritionLogDay.updateTotals();
      });
    };

    vm.addMealFromTemplate = () => {
      ModalService.show({
        templateUrl: 'client/nutritionLog/selectMealTemplateModal.html',
        controller: 'SelectMealTemplateModalController as vm'
      }).then(function (assigned) {
        angular.forEach(assigned, template => {
          vm.nutritionLogDay.addMealFromTemplate(template);
        });
      });
    };

    vm.viewNote = function () {
      NoteViewerModalService.show(vm.nutritionLogDay.note);
    };

    vm.templateChosen = function () {
      ConfirmDialogService.open({
        title: 'Change Nutrition Plan',
        message: 'If you have already started logging nutrition for today, these will be overwritten. Are you sure you want to load a different nutrition plan?',
        buttonConfirm: 'Yes',
        buttonCancel: 'No'
      }).then(function () {
        vm.loadingTemplate = true;

        vm.nutritionLogDay.resetMeals();

        ApiClientService.nutrition.getPlan(vm.usePlan).success(function (response) {
          NutritionEditorService.setTemplate(response.nutritionPlan, 'log');
          vm.loadingTemplate = false;
        });
      });
    };

    var loadTemplates = function () {
      if (vm.nutritionLogDay.created) {
        return $q.resolve(false);
      }

      return ApiClientService.customers.listNutritionPlans(vm.customerId).success(function (response) {
        vm.nutritionPlans = response.nutritionPlans;
      });
    };

    var loadLog = function () {

      var deferred = $q.defer();

      var currentItem = NutritionEditorService.getItem();

      if (!currentItem || currentItem.mode !== 'log' || currentItem.day !== vm.date.format('YYYY-MM-DD')) {

        NutritionLogEditService.getDay(vm.date.format('YYYY-MM-DD')).then(function (logDay) {

          vm.nutritionLogDay = NutritionEditorService.setItem(logDay, 'log');

          deferred.resolve();
        }, function () {
          ToastService.show('Could not load - please try again');
        });
      } else {

        vm.nutritionLogDay = NutritionEditorService.getItem();

        deferred.resolve();
      }

      return deferred.promise;
    };

    var setDate = function () {
      if (!vm.entered) {
        return false;
      }

      if (!vm.date.isSame(DayPickerService.getDate(), 'day')) {
        vm.ready = false;
        NutritionLogService.startLogger(DayPickerService.getDate(), true);
      }
    };

    DayPickerService.registerDayPickedEventHandler($scope, setDate);

    $scope.$on('$ionicView.beforeEnter', function (event, data) {
      vm.date = moment(DayPickerService.getDate());

      vm.ready = false;
      vm.entered = false;

      vm.currentDate = DayPickerService.getDate();
      vm.currentDateFormatted = vm.currentDate.format('Do MMMM YYYY');

      vm.nutritionPlans = [];

      loadLog().then(loadTemplates).then(function () {
        vm.ready = true;
      });

      vm.customerId = PermissionsService.getCurrentUserEntityId();
    });

    $scope.$on('$ionicView.enter', function () {
      vm.entered = true;
      AnalyticsService.trackView('Client - Log Nutrition');
    });

    $scope.$on('$ionicView.beforeLeave', function () {
      vm.active = false;
    });
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('NutritionLogEditService', service);

    service.$inject = ['ApiClientService', 'NutritionEditorService', 'ToastService', 'PermissionsService', '$q'];

    function service(ApiClientService, NutritionEditorService, ToastService, PermissionsService, $q) {

        var service = {};

        var update = function (item) {
            return ApiClientService.nutritionlog.updateDay(item.id, NutritionEditorService.getPayload());
        };
        var create = function (item) {
            return ApiClientService.nutritionlog.createDay(NutritionEditorService.getPayload());
        };

        service.saveChanges = function () {

            var item = NutritionEditorService.getItem();

            if (item.created) {

                return update(item).error(function (response) {
                    ToastService.show('Could not save - please try again');
                    $q.reject();
                });
            } else {

                var deferred = $q.defer();

                create(item).then(function (response) {
                    deferred.resolve(response.data);
                }, function (response) {
                    if (response.data.error && response.data.error.message === 'A day log has already been created for this day.') {
                        // Try update instead
                        update(item).then(function (response) {
                            deferred.resolve(response.data);
                        }, function () {
                            deferred.reject();
                        });
                    } else {
                        ToastService.show('Could not save - please try again');
                        deferred.reject();
                    }
                });

                return deferred.promise;
            }
        };

        service.getDay = function (date) {

            return $q(function (resolve, reject) {

                ApiClientService.nutritionlog.dayLog(PermissionsService.getCurrentUserEntityId(), date).then(function (response) {

                    var dayLog = response.nutritionLogDay;

                    dayLog.created = true;

                    resolve(dayLog);
                }, function (response) {

                    if (response && response.error && response.error.error_name && response.error.error_name === 'CUSTOMER_NUTRITION_DAY_LOG_NOT_CREATED_YET') {

                        resolve({
                            day: date,
                            meals: [],
                            use_macro: 0,
                            macro_type: 'grams',
                            created: false
                        });
                    } else {
                        reject();
                    }

                    return false;
                });
            });
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('NutritionLogService', service);

    service.$inject = ['NavigateService', 'NutritionLogEditService', 'ToastService', '$q', 'NutritionEditorService', 'ApiClientService', 'PermissionsService', 'ScheduleEventStoreService', 'ScheduleEventService', 'DayPickerService'];

    function service(NavigateService, NutritionLogEditService, ToastService, $q, NutritionEditorService, ApiClientService, PermissionsService, ScheduleEventStoreService, ScheduleEventService, DayPickerService) {

        var service = {};

        var scheduledPlans = [];
        var assignedPlans = [];

        var getScheduled = function (date, customerId) {
            var getCached = function () {
                return ScheduleEventService.getCustomerEventsOnDate(date, customerId, ['nutrition']).then(function (events) {
                    scheduledPlans = events;
                    return events;
                });
            };

            if (ScheduleEventStoreService.isDirty(customerId)) {
                return ScheduleEventStoreService.downloadCustomerEvents(customerId).then(getCached);
            } else {
                return getCached();
            }
        };

        var getAssigned = function (customerId) {
            return ApiClientService.customers.listNutritionPlans(customerId, false).then(function (response) {
                assignedPlans = response.nutritionPlans;
                return assignedPlans;
            });
        };

        var goToStartPage = function (dayObject, replaceState) {
            if (replaceState) {
                NavigateService.replaceCurrentState().transitionTo('app.client_log_nutrition_start', {
                    date: dayObject.day
                }, {
                    reload: true
                });
            } else {
                NavigateService.transitionTo('app.client_log_nutrition_start', {
                    date: dayObject.day
                }, {
                    reload: true
                });
            }
        };

        var goToLog = function (replaceState) {
            if (replaceState) {
                NavigateService.replaceCurrentState().go('app.client_log_nutrition', {}, {
                    reload: true
                });
            } else {
                NavigateService.go('app.client_log_nutrition', {}, {
                    reload: true
                });
            }
        };

        var getLogSuccess = function (logDay, replaceState) {
            var customerId = PermissionsService.getCurrentUserEntityId();

            scheduledPlans = [];
            assignedPlans = [];

            var dayObject = NutritionEditorService.setItem(logDay, 'log');

            // This day has already had data saved for it, go to it
            if (dayObject.created) {
                goToLog(replaceState);
                return $q.reject();
            }

            // If we got this far, we'll check if there are scheduled items
            return getScheduled(dayObject.day, customerId)

            // If There are sheduled plans, so go to the start page
            .then(function (response) {
                if (response.length > 0) {
                    goToStartPage(dayObject, replaceState);
                    return $q.reject();
                } else {
                    return $q.when();
                }
            })

            // No Scheduled plans, check for assigned ones
            .then(function () {
                return getAssigned(customerId);
            })

            // If there are assigned plans, so go to the start page
            .then(function (response) {
                if (response.length > 0) {
                    goToStartPage(dayObject, replaceState);
                    return $q.reject();
                } else {
                    return $q.when();
                }
            })
            // Nothing available to use as a template for today, go to logger.
            .then(function () {
                goToLog(replaceState);
                return $q.reject();
            });
        };

        service.startLogger = function (date, replaceState) {

            date = date || false;
            replaceState = replaceState || false;

            if (!date) {
                date = moment();
            }

            if (typeof date === 'string') {
                date = moment(date);
            }

            DayPickerService.setDate(date, true);

            return NutritionLogEditService.getDay(date.format('YYYY-MM-DD')).then(function (logDay) {
                return getLogSuccess(logDay, replaceState);
            }, function () {
                ToastService.show('Could not load - please try again');
            });
        };

        service.getScheduledPlans = function () {
            return scheduledPlans;
        };

        service.getAssignedPlans = function () {
            return assignedPlans;
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('ClientNutritionLogStartController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.client_log_nutrition_start', {
            url: "/nutritionlog/log/start",
            views: {
                'menuContent': {
                    templateUrl: "client/nutritionLog/nutritionLogStart.html",
                    controller: 'ClientNutritionLogStartController as vm'
                }
            }
        });
    }

    controller.$inject = ['$scope', 'NutritionEditorService', 'NutritionLogEditService', 'NavigateService', 'PromptDialogService', 'DayPickerService', '$q', 'ApiClientService', 'PermissionsService', 'NoteViewerModalService', 'AnalyticsService', 'ToastService', 'ScheduleEventStoreService', 'NetworkStateService', 'ScheduleEventService', '$ionicActionSheet', 'ConfirmDialogService', 'NutritionLogService'];

    function controller($scope, NutritionEditorService, NutritionLogEditService, NavigateService, PromptDialogService, DayPickerService, $q, ApiClientService, PermissionsService, NoteViewerModalService, AnalyticsService, ToastService, ScheduleEventStoreService, NetworkStateService, ScheduleEventService, $ionicActionSheet, ConfirmDialogService, NutritionLogService) {

        var vm = this;

        vm.datePickerToggle = DayPickerService.toggleOpen;

        var resetChecked = function () {
            // Reset existing checked
            angular.forEach(vm.plans, function (planRow) {
                planRow.use = false;
            });
        };

        var findChecked = function () {
            return $q(function (resolve) {
                angular.forEach(vm.plans, function (planRow) {
                    if (planRow.use) {
                        resolve(planRow);
                    }
                });
            });
        };

        vm.checkNutritionPlan = function (plan) {
            resetChecked();
            plan.use = true;
            vm.skipPlan = false;
            vm.canGo = true;
        };

        vm.noPlan = function () {
            resetChecked();
            vm.skipPlan = true;
            vm.canGo = true;
        };

        var goToLog = function () {
            NavigateService.replaceCurrentState().go('app.client_log_nutrition');
        };

        vm.go = function () {
            if (!vm.canGo) {
                return false;
            }

            // Chose to not follow plan, go.
            if (vm.skipPlan) {
                goToLog();
                return false;
            }

            // Chose to follow plan, load it in and go.
            findChecked().then(function (usePlan) {
                var planId;
                if (usePlan.hasOwnProperty('data')) {
                    planId = usePlan.data.nutrition_plan_id;
                } else {
                    planId = usePlan.id;
                }

                vm.loadingTemplate = true;

                vm.nutritionLogDay.resetMeals();

                ApiClientService.nutrition.getPlan(planId).success(function (response) {
                    NutritionEditorService.setTemplate(response.nutritionPlan, 'log');
                    goToLog();
                }).finally(function () {
                    vm.loadingTemplate = false;
                });
            });
        };

        var setup = function () {
            vm.ready = true;
            vm.canGo = false;
            vm.skipPlan = false;
            vm.date = DayPickerService.getDate();

            var scheduledPlans = NutritionLogService.getScheduledPlans();
            vm.hasScheduledPlans = scheduledPlans.length > 0;

            if (vm.hasScheduledPlans) {
                vm.plans = scheduledPlans;
            }

            var assignedPlans = NutritionLogService.getAssignedPlans();
            vm.hasAssignedPlans = !vm.hasScheduledPlans && assignedPlans.length > 0;

            if (!vm.hasScheduledPlans) {
                vm.plans = assignedPlans;
            }

            // If there's only 1 plan, check it automatically.
            if (vm.plans.length === 1) {
                vm.checkNutritionPlan(vm.plans[0]);
            }

            vm.nutritionLogDay = NutritionEditorService.getItem();

            vm.customerId = PermissionsService.getCurrentUserEntityId();
        };

        DayPickerService.registerDayPickedEventHandler($scope, function () {
            if (!vm.entered) {
                return false;
            }

            var newDate = DayPickerService.getDate();
            var dateChanged = vm.date && !newDate.isSame(vm.date, 'day');
            vm.currentDateFormatted = newDate.format('Do MMMM YYYY');
            if (dateChanged) {
                vm.ready = false;
                NutritionLogService.startLogger(newDate, true).finally(setup);
            }
        });

        $scope.$on('$ionicView.beforeEnter', function () {
            vm.entered = false;
            setup();
        });

        $scope.$on('$ionicView.enter', function () {
            vm.entered = true;
            var date = moment(vm.date, 'YYYY-MM-DD').hour(12);
            vm.currentDateFormatted = date.format('Do MMMM YYYY');
            AnalyticsService.trackView('Client - Start Log Nutrition');
        });

        $scope.$on('$ionicView.beforeLeave', function () {
            vm.active = false;
        });
    }
})();
(() => {
  angular.module('MyPTHub').controller('SelectMealTemplateModalController', controller);

  controller.$inject = ['parameters', 'ApiClientService', '$scope'];

  function controller(parameters, ApiClientService, $scope) {
    const vm = this;
    vm.selectedTemplate = 0;

    const loadTemplates = () => {
      const load = ApiClientService.mealTemplates.listAll();

      load.success(resp => {
        vm.templates = resp.meal_templates.map(template => {
          template.checked = false;
          return template;
        });
        console.log(vm.templates);
      });
    };

    vm.save = () => {
      const tpls = vm.templates.filter(temp => temp.checked);
      console.log(tpls);
      $scope.closeModal(tpls);
    };

    loadTemplates();
  }
})();
(() => {
  class ClientPackagesController {
    /**
     * @param ApiClientService
     * @param packages
     * @param PackagesService
     */
    constructor(ApiClientService, packages, PackagesService, $sce, mpthconf, MarkdownService, NavigateService) {
      this.ApiClientService = ApiClientService;
      this.packages = packages.all;
      this.PackagesService = PackagesService;
      this.$sce = $sce;
      this.mpthconf = mpthconf;
      this.MarkdownService = MarkdownService;
      this.NavigateService = NavigateService;

      this.markdownConfig = {
        listitem(text) {
          return `<li><i class="material-icons">check</i><span>${text}</span></li>`;
        }
      };

      this.fixPackages();
    }

    fixPackages() {
      this.packages.map(pkg => {
        pkg.display_price = this.PackagesService.renderPrice(pkg);
        pkg.short_description = this.$sce.trustAsHtml(this.MarkdownService.markdownToHtml(pkg.short_description, this.markdownConfig) || '');
        if (pkg.video_type === 'youtube' && pkg.video_id) {
          pkg.videoUrl = this.$sce.trustAsResourceUrl(`https://www.youtube-nocookie.com/embed/${pkg.video_id}?rel=0&controls=0&showinfo=0`);
        }
        if (pkg.video_type === 'vimeo' && pkg.video_id) {
          pkg.videoUrl = this.$sce.trustAsResourceUrl(`https://player.vimeo.com/video/${pkg.video_id}?title=0&byline=0&portrait=0&badge=0`);
        }
        if (pkg.photo) {
          pkg.photoUrl = this.$sce.trustAsResourceUrl(`${this.mpthconf.cdn.base}package/m/${pkg.photo}`);
        }
        return pkg;
      });
    }

    purchase(packageId) {
      console.log('click', packageId);
      this.NavigateService.go('app.client_package_purchase', {
        packageId
      });
    }
  }

  const config = $stateProvider => {
    $stateProvider.state('app.client_packages', {
      url: '/client_packages',
      views: {
        menuContent: {
          templateUrl: 'client/packages/views/packages.html',
          controller: 'ClientPackagesController',
          controllerAs: 'vm'
        }
      },
      resolve: {
        packages(ApiClientService, AuthenticationStateService) {
          const user = AuthenticationStateService.getCurrentUser();
          return ApiClientService.packages.listCustomer(user.customer_id).then(resp => resp.packages);
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('ClientPackagesController', ClientPackagesController);
})();
(() => {
  class ClientPackagePurchaseController {
    /**
     * @param ApiClientService
     * @param getPackage
     * @param AuthenticationStateService
     * @param PaymentCardService
     * @param PackagesService
     * @param $q
     * @param ToastService
     * @param $state
     * @param $sce
     * @param AuthenticationService
     * @param MarkdownService
     * @param DialogService
     * @param NavigateService
     * @returns {string}
     */
    constructor(ApiClientService, getPackage, AuthenticationStateService, PaymentCardService, PackagesService, $q, ToastService, $state, $sce, AuthenticationService, MarkdownService, NavigateService, mpthconf) {
      this.ApiClientService = ApiClientService;
      this.getPackage = getPackage;
      this.PaymentCardService = PaymentCardService;
      this.PackagesService = PackagesService;
      this.$q = $q;
      this.ToastService = ToastService;
      this.$state = $state;
      this.$sce = $sce;
      this.AuthenticationService = AuthenticationService;
      this.MarkdownService = MarkdownService;
      this.NavigateService = NavigateService;
      this.AuthenticationStateService = AuthenticationStateService;
      this.mpthconf = mpthconf;

      this.Stripe = Stripe(mpthconf[mpthconf.env].stripe_key);

      this.currentUser = AuthenticationStateService.getCurrentUser();

      this.coupon = false;
      this.card = {
        billing_cardholder: `${this.currentUser.first_name} ${this.currentUser.last_name}`,
        billing_address1: this.currentUser.address1,
        billing_town_city: this.currentUser.town_city,
        billing_county: this.currentUser.county,
        billing_postcode: this.currentUser.postcode,
        billing_country: this.currentUser.country
      };

      this.markdownConfig = {
        listitem(text) {
          return `<li><i class="material-icons">check</i><span>${text}</span></li>`;
        }
      };
      this.getPackage.description = $sce.trustAsHtml(MarkdownService.markdownToHtml(this.getPackage.description, this.markdownConfig) || '');
      this.cardYears = [];
      const startYear = parseInt(new Date().getFullYear().toString().substr(2, 2), 10);
      const endYear = startYear + 10;
      for (let y = startYear; y <= endYear; y++) {
        this.cardYears.push(y);
      }

      this.defineMedia();
      this.setupPrice();

      this.initElements();
    }

    defineMedia() {
      if (this.getPackage.video_type === 'youtube' && this.getPackage.video_id) {
        this.getPackage.videoUrl = this.$sce.trustAsResourceUrl(`https://www.youtube-nocookie.com/embed/${this.getPackage.video_id}?rel=0&controls=0&showinfo=0`);
      }
      if (this.getPackage.video_type === 'vimeo' && this.getPackage.video_id) {
        this.getPackage.videoUrl = this.$sce.trustAsResourceUrl(`https://player.vimeo.com/video/${this.getPackage.video_id}?title=0&byline=0&portrait=0&badge=0`);
      }
      if (this.getPackage.images) {
        this.getPackage.photo = this.$sce.trustAsResourceUrl(`${this.mpthconf.cdn.base}package/banner/${this.getPackage.images.filename}`);
      }
    }

    setupPrice() {
      this.getPackage.display_price = this.PackagesService.renderPrice(this.getPackage, this.coupon);
      this.getPackage.price_description = this.getPackage.type === 'recurring' ? this.PackagesService.buildPriceDescription(this.getPackage) : '';
    }

    packageIsFree() {
      return this.getPackage.free;
    }

    applyCoupon() {
      this.checkingCoupon = true;

      const getCoupon = this.ApiClientService.packages.getCouponByCode(this.getPackage.trainer_user_id, this.couponCode);

      getCoupon.success(response => {
        this.coupon = response.coupon;
        this.checkingCoupon = false;
        this.setupPrice();

        this.ToastService.show('Discount successfully applied');
      });

      getCoupon.error(() => {
        this.coupon = false;
        this.checkingCoupon = false;
        this.setupPrice();
      });
    }

    purchase(token = false) {
      const data = {};

      if (token) {
        data.card_token = token.id;
      }

      if (this.coupon) {
        data.coupon_code = this.coupon.code;
      }

      const purchase = this.ApiClientService.packages.purchase(this.getPackage.id, data);

      purchase.success(() => {
        this.ToastService.show('Package successfully activated');

        const changePage = () => Promise.resolve().then(() => this.AuthenticationService.refreshUser().then(user => {
          this.AuthenticationStateService.setCurrentUser(user);
        })).then(() => {
          this.NavigateService.clearCache();
          this.NavigateService.clearHistory();
          this.NavigateService.goHome();
        });

        changePage();
      });

      purchase.error(purchaseResponse => {
        let message = 'An error occurred, please try again';

        if (typeof purchaseResponse.error.message === 'string') {
          message = purchaseResponse.error.message;
        } else if (purchaseResponse.error.message.hasOwnProperty('stripe_error_type') && purchaseResponse.error.message.stripe_error_type === 'Stripe_CardError') {
          message = purchaseResponse.error.message.stripe_error_message;
        }

        this.ToastService(message);

        this.processing = false;
      });

      purchase.finally(() => {
        this.processing = false;
      });
    }

    processResult(result) {
      const errorElement = document.getElementById('card-errors');
      if (result.error) {
        errorElement.textContent = result.error.message;
      } else {
        errorElement.textContent = '';
        this.purchase(result.token);
        // token is in result.token
      }
    }

    async initElements() {
      await setTimeout(() => {
        const elements = this.Stripe.elements();
        this.card = elements.create('card');

        this.card.mount('#card-element');

        const form = document.getElementById('payment-form');

        form.addEventListener('submit', event => {
          this.processing = true;
          event.preventDefault();

          this.Stripe.createToken(this.card).then(result => this.processResult(result));
        });
      }, 100);
    }
  }

  const config = $stateProvider => {
    $stateProvider.state('app.client_package_purchase', {
      url: '/purchase/:packageId',
      views: {
        menuContent: {
          templateUrl: 'client/packages/views/purchase.html',
          controller: 'ClientPackagePurchaseController',
          controllerAs: 'vm'
        }
      },
      resolve: {
        getPackage(ApiClientService, $stateParams) {
          return ApiClientService.packages.get($stateParams.packageId).then(resp => resp.package);
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('ClientPackagePurchaseController', ClientPackagePurchaseController);
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('ClientProgressPhotosController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.client_progress_photos', {
            url: "/client_progress_photos",
            views: {
                'menuContent': {
                    templateUrl: "client/progressphotos/progressphotos.html",
                    controller: 'ClientProgressPhotosController as vm'
                }
            }
        });
    }

    controller.$inject = ['$scope', 'ApiClientService', 'ImageCacheService', '$q', 'PermissionsService', 'ProgressPhotoUploadService', 'AnalyticsService', '$filter', 'ConfirmDialogService', 'ScheduleEventStoreService', 'NotificationsService', 'ModalService'];

    function controller($scope, ApiClientService, ImageCacheService, $q, PermissionsService, ProgressPhotoUploadService, AnalyticsService, $filter, ConfirmDialogService, ScheduleEventStoreService, NotificationsService, ModalService) {

        var vm = this;

        var loadPhotos = function (refresh) {

            refresh = refresh || false;

            if (!refresh && vm.photos) {
                vm.ready = true;
                return false;
            }

            return ApiClientService.progressPhotos.list(vm.customerId).success(function (response) {

                var cacheResolvers = [];

                var brokenImageSmall = 'img/broken-image.png';

                response.photos.map(function (photo) {

                    cacheResolvers.push(ImageCacheService.get(photo.small_url).then(function (newUrl) {
                        photo.small_url = newUrl;
                    }, function () {
                        photo.small_url = brokenImageSmall;
                    }));
                });

                $q.all(cacheResolvers).finally(function () {

                    vm.photos = response.photos;

                    vm.photosRange = function (cols) {

                        var range = [];

                        for (var i = 0; i < vm.photos.length; i = i + cols) {
                            range.push(i);
                        }

                        return range;
                    };

                    vm.noPhotos = vm.photos.length === 0;
                });
            }).finally(function () {

                if (refresh) {
                    $scope.$broadcast('scroll.refreshComplete');
                }

                vm.ready = true;
            });
        };

        vm.refresh = function () {
            loadPhotos(true);
        };

        vm.takePhoto = function () {

            vm.uploading = true;

            ProgressPhotoUploadService.takePhoto(vm.customerId).finally(function () {
                ScheduleEventStoreService.setDirty();
                NotificationsService.dirty();
                vm.uploading = false;
            });
        };

        vm.browsePhoto = function () {

            vm.uploading = true;

            ProgressPhotoUploadService.browsePhoto(vm.customerId).finally(function () {
                ScheduleEventStoreService.setDirty();
                NotificationsService.dirty();
                vm.uploading = false;
            });
        };

        vm.showLargePhoto = function (photo) {
            if (vm.photoDeleteMode) {
                vm.toggleDeleteFlag(photo);
                vm.toggledDelete();
                return true;
            }
            if (!window.PhotoViewer) {
                console.log('Device does not support native photo viewer.');
            }

            var date = moment.utc(photo.created_at, 'YYYY-MM-DD HH:mm:ss').local().format('Do MMMM YYYY');

            ModalService.show({
                templateUrl: "common/photoViewer/photoViewerModal.html",
                controller: 'PhotoViewerModalController as vm',
                params: {
                    url: photo.large_url,
                    date: date
                }
            });
        };

        vm.toggleDeleteFlag = function (photo) {
            photo.toDelete = !photo.toDelete;
        };

        var getPhotosWithDeleteChecked = function () {

            if (!vm.photos) {
                return [];
            }

            return $filter('filter')(vm.photos, {
                toDelete: true
            });
        };

        var clearCheckedPhotosForDelete = function () {
            getPhotosWithDeleteChecked().map(function (item) {
                item.toDelete = false;
            });
        };

        vm.exitPhotosDeleteMode = function () {
            vm.photoDeleteMode = false;
            vm.hasCheckedPhotosForDelete = false;
            clearCheckedPhotosForDelete();
        };

        vm.toggledDelete = function () {
            vm.hasCheckedPhotosForDelete = getPhotosWithDeleteChecked().length > 0;
        };

        vm.deleteCheckedPhotos = function () {

            ConfirmDialogService.open({
                title: "Confirm Delete",
                message: "Are you sure you want to delete these photos?",
                buttonConfirm: "Delete Photos",
                buttonCancel: "Cancel"
            }).then(function () {

                getPhotosWithDeleteChecked().map(function (item) {

                    vm.photos.splice(vm.photos.indexOf(item), 1);

                    ApiClientService.progressPhotos.delete(vm.customerId, item.id);
                });

                vm.exitPhotosDeleteMode();
            });
        };

        vm.photoDeleteMode = false;
        vm.showPhotoOptions = function () {

            vm.photoDeleteMode = true;
        };

        $scope.$on('$ionicView.beforeEnter', function () {

            vm.noPhotos = false;

            vm.customerId = PermissionsService.getCurrentUserEntityId();

            vm.ready = false;

            loadPhotos();

            ProgressPhotoUploadService.registerUploadSuccessEventHandler($scope, function () {
                loadPhotos(true);
            });
        });

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Client - Progress Photos');
        });
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('ClientSchedulesController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.client_schedules', {
            url: "/client_schedules",
            views: {
                'menuContent': {
                    templateUrl: "client/schedules/schedules.html",
                    controller: 'ClientSchedulesController as vm'
                }
            }
        });
    }

    controller.$inject = ['ApiClientService', 'NavigateService', 'PermissionsService', '$filter', '$scope'];

    function controller(ApiClientService, NavigateService, PermissionsService, $filter, $scope) {
        var vm = this;
        vm.schedules = {};
        vm.noSchedules = true;
        vm.processing = true;
        vm.scheduleStarted = false;
        vm.selectedTabIndex = 0;

        var customerid = PermissionsService.getCurrentUserEntityId();

        var loadCurrent = function (refreshing) {
            refreshing = refreshing || false;

            if (vm.currentSchedules && !refreshing) {
                return false;
            }

            ApiClientService.customers.getCurrentSchedules(customerid).then(function (response) {
                if (response.schedules.length === 1 && !refreshing) {
                    // go to schedule
                    NavigateService.replaceCurrentState().go('app.view_customer_schedule', {
                        customerId: customerid,
                        scheduleId: response.schedules[0].id
                    });
                } else {
                    angular.forEach(response.schedules, function (schedule) {

                        if (schedule.start_date) {
                            var startDate = moment.tz(schedule.start_date, schedule.customerTimezone).format('Do MMM YYYY');

                            schedule.description = 'Starts on ' + startDate;

                            var endDate = false;
                            if (schedule.started) {
                                endDate = moment.tz(schedule.end_date, schedule.customerTimezone).format('Do MMM YYYY');

                                schedule.description = startDate + ' - ' + endDate;
                            }
                        } else {
                            schedule.description = 'No start date specified';
                        }

                        schedule.active = schedule.started && !schedule.ended;
                    });

                    vm.currentSchedules = response.schedules;
                    vm.filteredCurrentSchedules = angular.copy(response.schedules);
                    vm.noCurrentSchedules = vm.currentSchedules.length === 0;
                    vm.processing = false;
                    vm.ready = true;
                }
            }).finally(function () {
                if (refreshing) {
                    $scope.$broadcast('scroll.refreshComplete');
                }
            });
        };

        var loadPrevious = function (refreshing) {
            refreshing = refreshing || false;

            if (vm.previousSchedules && !refreshing) {
                return false;
            }

            ApiClientService.customers.getPreviousSchedules(customerid).then(function (response) {

                angular.forEach(response.schedules, function (schedule) {

                    var startDate = moment.tz(schedule.start_date, schedule.customerTimezone).format('Do MMM YYYY');

                    var endDate = false;
                    if (schedule.end_date) {
                        endDate = moment.tz(schedule.end_date, schedule.customerTimezone).format('Do MMM YYYY');
                    }

                    if (endDate) {
                        schedule.description = 'Finished (' + startDate + ' - ' + endDate + ')';
                    } else {
                        schedule.description = startDate;
                    }
                });

                vm.previousSchedules = response.schedules;
                vm.filteredPreviousSchedules = angular.copy(response.schedules);
                vm.noPreviousSchedules = vm.previousSchedules.length === 0;
                vm.processing = false;
            }).finally(function () {
                if (refreshing) {
                    $scope.$broadcast('scroll.refreshComplete');
                }
            });
        };

        vm.currentShow = function () {
            loadCurrent();
        };

        vm.previousShow = function () {
            loadPrevious();
        };

        vm.refresh = function () {
            switch (vm.selectedTabIndex) {
                case 0:
                    loadCurrent(true);
                    break;
                case 1:
                    loadPrevious(true);
                    break;
            }
        };

        // Current Search filter
        $scope.$watch(function () {
            return vm.currentSearchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {

                vm.filteredCurrentSchedules = $filter('filter')(vm.currentSchedules, {
                    name: newVal
                });
            } else {
                vm.filteredCurrentSchedules = vm.currentSchedules;
            }
        });

        // Previous Search filter
        $scope.$watch(function () {
            return vm.previousSearchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {

                vm.filteredPreviousSchedules = $filter('filter')(vm.previousSchedules, {
                    name: newVal
                });
            } else {
                vm.filteredPreviousSchedules = vm.previousSchedules;
            }
        });

        $scope.$on('$ionicView.beforeEnter', function (scopes, states) {
            if (typeof vm.selectedTabIndex !== 'undefined') {
                switch (vm.selectedTabIndex) {
                    case 0:
                        loadCurrent();
                        break;
                    case 1:
                        loadPrevious();
                        break;
                }
            }
        });

        vm.goToSchedule = function (schedule) {
            NavigateService.go('app.view_customer_schedule', {
                customerId: customerid,
                scheduleId: schedule.id
            });
        };
    }
})();
(() => {
  class ClientSessionCreditQuotaController {
    /**
     * @param {object} $scope service
     * @param {object} PermissionsService service
     * @param {object} AnalyticsService service
     */
    constructor($scope, PermissionsService, AnalyticsService) {
      this.addedHistory = [];
      this.redeemedHistory = [];
      $scope.$on('$ionicView.beforeEnter', () => {
        this.customerId = PermissionsService.getCurrentUserEntityId();
      });

      $scope.$on('$ionicView.enter', () => {
        AnalyticsService.trackView('Client - Sessions and Credits Quota');
      });

      $scope.$on('updateCreditHistory', (event, data) => {
        this.addedHistory = data.addedHistory;
        this.redeemedHistory = data.redeemedHistory;
      });
    }
  }

  const config = $stateProvider => {
    $stateProvider.state('app.client_session_credit_quota', {
      url: '/client-sessions-and-credits',
      views: {
        menuContent: {
          templateUrl: 'client/sessions-credits/views/session-credit-quota.html',
          controller: 'ClientSessionCreditQuotaController as vm'
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('ClientSessionCreditQuotaController', ClientSessionCreditQuotaController);
})();
(function () {
  'use strict';

  angular.module('MyPTHub').config(config).controller('ClientSettingsController', controller);

  config.$inject = ['$stateProvider'];

  function config($stateProvider) {

    $stateProvider.state('app.client_settings', {
      url: "/client_settings",
      views: {
        'menuContent': {
          templateUrl: "client/settings/settings.html",
          controller: 'ClientSettingsController as vm'
        }
      },
      params: {
        code: null
      },
      data: {
        deepLink: ''
      }
    });
  }

  controller.$inject = ['AuthenticationHelperService', 'NavigateService', '$scope', 'AuthenticationStateService', 'ApiClientService', 'AvatarUploadService', 'AnalyticsService', 'mpthconf', 'ModalService', 'BrandingService', 'DeviceInfoService', 'AuthenticationService', 'BrandingMenuService', '$stateParams'];

  function controller(AuthenticationHelperService, NavigateService, $scope, AuthenticationStateService, ApiClientService, AvatarUploadService, AnalyticsService, mpthconf, ModalService, BrandingService, DeviceInfoService, AuthenticationService, BrandingMenuService, $stateParams) {

    // temp horrible hack to work around issue of disappearing tabs
    // in some weird circumstances.
    $scope.$on('$ionicView.afterEnter', function () {
      $(window).trigger('resize');
    });

    var vm = this;

    vm.setMode = function (mode) {
      vm.mode = mode;
    };

    vm.code = $stateParams.code;

    vm.webcalUrl = mpthconf[mpthconf.env].webcal.current;

    vm.customer = {};

    vm.fitbitLoading = true;

    vm.fitbitConnected = false;

    vm.available = 'nope';

    vm.urlLog = [];

    vm.accountSwapper = () => {
      NavigateService.go('app.accounts');
    };

    vm.showAccountSwapper = () => {
      return AuthenticationStateService.getCurrentUser().accounts > 1;
    };

    var setCustomer = function (data) {
      vm.customer.first_name = data.first_name;
      vm.customer.last_name = data.last_name;
      vm.customer.description = data.description;
      vm.customer.address1 = data.address1;
      vm.customer.address2 = data.address2;
      vm.customer.town_city = data.town_city;
      vm.customer.postcode = data.postcode;
      vm.customer.country = data.country;
      vm.customer.timezone = data.timezone;
      vm.customer.dob = data.dob;

      vm.webcalUrl = vm.webcalUrl + data.id + '/ical?token=' + data.cal_token;
    };

    vm.openReportIssueModal = function () {

      ModalService.show({
        templateUrl: "common/bugReport/bugReportModal.html",
        controller: 'BugReportModalController as vm'
      });
    };

    var setDob = function (source) {
      vm.dob = moment(source, 'YYYY-MM-DD').toDate();
    };

    var parseDob = function (source) {
      return moment(source).format('YYYY-MM-DD');
    };

    vm.logout = AuthenticationHelperService.logout;

    vm.saveSettings = function () {

      vm.saving = true;

      if (vm.dob) {
        vm.customer.dob = parseDob(vm.dob);
      }

      ApiClientService.customers.update(vm.user.customer_id, vm.customer).success(function () {
        NavigateService.goBack();
      }).finally(function () {
        vm.saving = false;
      });
    };

    vm.takePhoto = function () {

      vm.uploading = true;

      AvatarUploadService.takePhoto(vm.user.id).then(function (response) {
        vm.user = AuthenticationStateService.getCurrentUser();
        setCustomer(vm.user);
      }).finally(function () {
        vm.uploading = false;
      });
    };

    vm.browsePhoto = function () {

      vm.uploading = true;

      AvatarUploadService.browsePhoto(vm.user.id).then(function (response) {
        vm.user = AuthenticationStateService.getCurrentUser();
        setCustomer(vm.user);
      }).finally(function () {
        vm.uploading = false;
      });
    };

    vm.openBrandingMenu = function () {
      BrandingMenuService.start();
    };

    DeviceInfoService.appVersion().then(function (appVer) {
      vm.appVer = appVer;
    });

    $scope.$on("$ionicView.beforeEnter", function () {

      vm.loaded = false;

      vm.user = AuthenticationStateService.getCurrentUser();

      setDob(vm.user.dob);

      setCustomer(vm.user);

      ApiClientService.customers.get(vm.user.customer_id).success(function (response) {

        vm.user = response.currentUser;

        setDob(vm.user.dob);

        setCustomer(vm.user);

        vm.loaded = true;
      });

      if (!vm.code) {
        vm.checkConnection(vm.user.id);
      } else {
        vm.fitbitAuthenticate(vm.code);
      }
    });

    $scope.$on('$ionicView.enter', function () {
      AnalyticsService.trackView('Client - Settings');
    });

    vm.checkConnection = function (user_id) {
      var clientApiKey = ApiClientService.hardware.getAuthKey(user_id);
      clientApiKey.success(function (response) {
        vm.clientApiKey = response;
      });

      var checkConnection = ApiClientService.hardware.checkAuthentication(user_id);

      checkConnection.success(function (response) {

        vm.fitbitLoading = false;
        vm.fitbitConnected = JSON.parse(response);
      });
    };

    vm.changeFitbitStatus = function (newStatus) {
      if (!vm.fitbitConnected) {
        vm.fitbitDisconnect();
      } else {
        vm.fitbitConnect();
      }
    };

    vm.fitbitConnect = function () {
      /**
       * Add a guard
       **/
      if (!vm.clientApiKey) {
        return;
      }
      const website = `https://www.fitbit.com/oauth2/authorize?response_type=code&client_id=${vm.clientApiKey}&scope=activity%20profile%20heartrate%20sleep%20weight%20nutrition&expires_in=31536000&redirect_uri=mypthub%3A%2F%2Ffitbit/signup`;
      vm.openUrl(website, false);
    };

    vm.fitbitDisconnect = function () {

      var clearConnection = ApiClientService.hardware.clearAuthentication(vm.user.id);

      clearConnection.success(function (response) {
        if (response == 'true') {
          vm.fitbitConnected = false;
        }
      });
    };

    vm.fitbitExtraOptions = function () {
      if (!vm.fitbitLoading) {
        if (vm.fitbitConnected) {} else {
          vm.fitbitConnect();
        }
      }
    };

    vm.fitbitAuthenticate = function (code) {
      var connection = ApiClientService.hardware.setup(code);

      connection.success(function () {
        vm.checkConnection(vm.user.id);
      });
    };

    vm.openUrl = function (url, readerMode) {
      vm.available = 'testing';

      SafariViewController.isAvailable(function (available) {
        if (available) {
          SafariViewController.show({
            url: url,
            hidden: false, // default false. You can use this to load cookies etc in the background (see issue #1 for details).
            animated: true, // default true, note that 'hide' will reuse this preference (the 'Done' button will always animate though)
            transition: 'slide', // (this only works in iOS 9.1/9.2 and lower) unless animated is false you can choose from: curl, flip, fade, slide (default)
            enterReaderModeIfAvailable: readerMode, // default false
            tintColor: '#00afb8', // default is ios blue
            barColor: '#359dd0', // on iOS 10+ you can change the background color as well
            controlTintColor: '#ffffff', // on iOS 10+ you can override the default tintColor
            toolbarColor: '#359dd0',
            showDefaultShareMenuItem: false
          },
          // this success handler will be invoked for the lifecycle events 'opened', 'loaded' and 'closed'
          function (result) {
            if (result.event === 'opened') {
              console.log('opened');
            } else if (result.event === 'loaded') {
              console.log('loaded');
            } else if (result.event === 'closed') {
              console.log('closed');
              vm.checkConnection(vm.user.id);
            }
          }, function (msg) {
            console.log('KO: ' + msg);
          });
        } else {
          // potentially powered by InAppBrowser because that (currently) clobbers window.open
          window.open(url, '_blank', 'location=yes');
        }
      });
    };

    function dismissSafari() {
      SafariViewController.hide();
    }

    function handleOpenURL(url) {
      setTimeout(function () {
        // SafariViewController.hide();
        var data = decodeURIComponent(url.substr(url.indexOf('=') + 1));
        console.log('Browser data received: ' + data);
      }, 0);
    }

    function dismissSafari() {
      SafariViewController.hide();
    }
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('ClientShoppingListController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.client_shopping_list', {
            url: "/client_shopping_list",
            views: {
                'menuContent': {
                    templateUrl: "client/shoppinglist/shoppinglist.html",
                    controller: 'ClientShoppingListController as vm'
                }
            }
        });
    }

    controller.$inject = ['$scope', 'ApiClientService', 'PermissionsService', 'AnalyticsService'];

    function controller($scope, ApiClientService, PermissionsService, AnalyticsService) {

        var vm = this;

        $scope.$on('$ionicView.beforeEnter', function () {

            ApiClientService.nutrition.getShoppingList(PermissionsService.getCurrentUserEntityId()).success(function (response) {
                vm.shoppingList = response.shoppingList;
            });
        });

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Client - Shopping List');
        });

        vm.changePurchase = function (foodId, purchased) {

            if (purchased) {

                ApiClientService.shoppingList.addPurchase(foodId);
            } else {

                ApiClientService.shoppingList.removePurchase(foodId);
            }
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('ClientWorkoutsController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.client_workouts', {
            url: "/client_workouts",
            views: {
                'menuContent': {
                    templateUrl: "client/workouts/workouts.html",
                    controller: 'ClientWorkoutsController as vm'
                }
            }
        });
    }

    controller.$inject = ['$scope', 'PermissionsService', 'ApiClientService', '$filter', 'NavigateService', 'AnalyticsService'];

    function controller($scope, PermissionsService, ApiClientService, $filter, NavigateService, AnalyticsService) {

        var vm = this;

        vm.selectedTabIndex = 0;

        var customerId = PermissionsService.getCurrentUserEntityId();

        var loadAll = function (refreshing) {

            refreshing = refreshing || false;

            vm.workoutsSearchFilter = '';

            if (!vm.workouts || refreshing) {

                ApiClientService.customers.listWorkouts(customerId, false, false).success(function (response) {

                    vm.workouts = response.workouts;
                    vm.filteredWorkouts = angular.copy(response.workouts);
                    vm.noWorkouts = response.workouts.length === 0;
                }).finally(function () {
                    if (refreshing) {
                        $scope.$broadcast('scroll.refreshComplete');
                    }
                });
            }
        };

        var loadCreated = function (refreshing) {

            refreshing = refreshing || false;

            vm.createdWorkoutsSearchFilter = '';

            if (!vm.createdWorkouts || refreshing) {

                ApiClientService.customers.listCreatedWorkouts(customerId).success(function (response) {

                    vm.createdWorkouts = response.workouts;
                    vm.filteredCreatedWorkouts = angular.copy(response.workouts);

                    vm.noCreatedWorkouts = response.workouts.length === 0;
                }).finally(function () {
                    if (refreshing) {
                        $scope.$broadcast('scroll.refreshComplete');
                    }
                });
            }
        };

        vm.refresh = function () {
            if (vm.selectedTabIndex === 0) {
                loadAll(true);
            }

            if (vm.selectedTabIndex === 1) {
                loadCreated(true);
            }
        };

        vm.viewWorkout = function (workout) {
            NavigateService.go('app.view_workout', {
                workoutId: workout.id
            });
        };

        vm.editWorkout = function (workout) {
            if (workout.enabled) {
                NavigateService.go('app.edit_workout', {
                    workoutId: workout.id
                });
            }
        };

        // All Workouts Search filter
        $scope.$watch(function () {
            return vm.workoutsSearchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {

                vm.filteredWorkouts = $filter('filter')(vm.workouts, {
                    title: newVal
                });
            } else {
                vm.filteredWorkouts = vm.workouts;
            }
        });

        // All Workouts Search filter
        $scope.$watch(function () {
            return vm.createdWorkoutsSearchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {

                vm.filteredCreatedWorkouts = $filter('filter')(vm.createdWorkouts, {
                    title: newVal
                });
            } else {
                vm.filteredCreatedWorkouts = vm.createdWorkouts;
            }
        });

        vm.createdWorkoutsShow = function () {
            loadCreated();
        };

        $scope.$on('$ionicView.beforeEnter', function () {

            loadAll();
        });

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Client - Workouts');
        });
    }
})();
(() => {
  angular.module('MyPTHub').factory('AccountSwapperService', service);

  function service(ApiClientService, AuthenticationService, AuthenticationStateService, BrandingService, SessionHistoryService, $rootScope, NavigateService, ModalService, BrandingMenuService) {
    const redirect = account => {

      const PleaseWait = ModalService.show({
        templateUrl: "common/accountSwapper/accountSwapperModal.html",
        controller: 'accountSwapperModalController as vm'
      });

      function change() {
        return Promise.resolve().then(() => {
          return AuthenticationService.refreshUser();
        }).then(() => {
          return AuthenticationStateService.getCurrentUser();
        }).then(user => {
          return AuthenticationService.updateCurrentUser(user);
        }).then(() => {
          return NavigateService.clearCache();
        }).then(() => {
          return NavigateService.clearHistory();
        }).then(() => {
          return SessionHistoryService.clearCache();
        }).then(() => {
          return BrandingService.clearCache();
        }).then(() => {
          return BrandingService.applyStyles();
        }).then(() => {
          return setTimeout(() => {
            $rootScope.$broadcast('activeUsed:updated');
            $rootScope.$broadcast('user:updated');
            NavigateService.goHome();
            $rootScope.$broadcast('accountSwapperCloseModal:close');
          }, 1000);
        });
      }

      change();
    };

    return {
      changeAccount: (account, isTrainer) => {
        ApiClientService.customerTrainers.updateCurrentCustomer({ trainer: account, trainerAccount: isTrainer }).success(response => {
          redirect(account);
        }).error(error => {});
      },
      UpdateUser: () => {
        redirect();
        console.log('activated');
      }
    };
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('AlertDialogService', service);

    service.$inject = ['$q'];

    function service($q) {

        var service = {};

        service.show = function (config) {

            if (!config.hasOwnProperty('buttonText') || !config.buttonText) {
                config.buttonText = 'OK';
            }

            return $q(function (resolve, reject) {

                navigator.notification.alert(config.message, resolve, config.title, config.buttonText);
            });
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('PromptDialogService', service);

    service.$inject = ['$q'];

    function service($q) {

        var service = {};

        service.open = function (config) {

            return $q(function (resolve, reject) {

                var onComplete = function (button) {

                    if (button.buttonIndex === 1) {

                        if (config.required && button.input1.length === 0) {

                            navigator.notification.prompt(config.message, onComplete, config.title, [config.buttonConfirm, config.buttonCancel], config.defaultText);
                        } else {
                            resolve(button.input1);
                        }
                    } else {
                        reject();
                    }
                };

                navigator.notification.prompt(config.message, onComplete, config.title, [config.buttonConfirm, config.buttonCancel], config.defaultText);
            });
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('AnalyticsService', service);

    service.$inject = ['mpthconf'];

    function service(mpthconf) {

        var service = {};

        var enabled = false;

        service.setup = function () {

            if (typeof window.analytics !== 'undefined') {

                console.log('Analytics property set:', mpthconf.analytics.id);

                window.analytics.startTrackerWithId(mpthconf.analytics.id);

                enabled = true;
            } else {

                console.log("Google Analytics Unavailable");
            }
        };

        service.trackView = function (viewName) {

            if (!enabled) {
                return false;
            }

            console.log('Tracked view:', viewName);
            window.analytics.trackView(viewName);
        };

        service.trackEvent = function (category, action) {

            if (!enabled) {
                return false;
            }

            console.log('Tracked event:', category, action);
            window.analytics.trackEvent(category, action);
        };

        service.setUserId = function (userId) {

            if (!enabled) {
                return false;
            }

            console.log('User ID set for analytics:', userId);
            window.analytics.setUserId(userId);
        };

        service.setUserType = function (setType) {

            if (!enabled) {
                return false;
            }

            var type = 'Client';

            if (setType === 'trainer') {
                type = 'Trainer';
            }

            window.analytics.addCustomDimension('User Type', type);
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('ApiCacheService', service);

    service.$inject = ['HttpRequestService', 'NetworkStateService', 'SqlStorageService', 'PermissionsService', 'CryptographyService', '$q'];

    function service(HttpRequestService, NetworkStateService, SqlStorageService, PermissionsService, CryptographyService, $q) {

        var generateKey = function (endpoint, params, type) {

            var userId = PermissionsService.getLoggedInUserId();

            return CryptographyService.md5([userId, endpoint, JSON.stringify(params), type].join('|'));
        };

        var cacheResponse = function (endpoint, params, response) {

            var key = generateKey(endpoint, params, 'get');

            SqlStorageService.query('DELETE FROM api_cache WHERE key = ?', [key]).then(function () {
                SqlStorageService.query('INSERT INTO api_cache (key, value) VALUES (?,?)', [key, JSON.stringify(response)]);
            });
        };

        var getFromCache = function (endpoint, params) {

            var key = generateKey(endpoint, params, 'get');

            return $q(function (resolve, reject) {

                SqlStorageService.select('SELECT value FROM api_cache WHERE key = ?', [key]).then(function (result) {

                    resolve(JSON.parse(result.value));
                }, function () {

                    reject(false);
                });
            });
        };

        return {

            get: function (endpoint, params) {

                var deferred = $q.defer();

                var online = NetworkStateService.isOnline();

                // Online - get from live API and cache it.
                if (online) {

                    var onlineRequest = HttpRequestService.get(endpoint, params);

                    onlineRequest.success(function (response) {

                        cacheResponse(endpoint, params, response);

                        deferred.resolve(response, false);
                    }).error(function (response) {

                        deferred.reject(response);
                    });

                    return deferred.promise;
                }

                // Offline - get from cache if it exists in cache.
                getFromCache(endpoint, params).then(function (response) {
                    deferred.resolve(response, true);
                }, function (response) {
                    deferred.reject(response);
                });

                return deferred.promise;
            }

        };
    }
})();
(() => {
  angular.module('MyPTHub').factory('ApiClientService', service);

  service.$inject = ['HttpRequestService', 'ApiCacheService', 'MessagesService'];

  function service(HttpRequestService, ApiCacheService, MessagesService) {
    const methods = {};
    let cacheGetRequests = true;

    const getRequest = (endpoint, params = null, cancelPromise = false) => {
      if (!cacheGetRequests) {
        cacheGetRequests = true;

        const doGet = HttpRequestService.get(endpoint, params, cancelPromise);

        doGet.then(response => {
          if (response.data && response.data.currentUser && response.data.currentUser.unread_messages > 0) {
            MessagesService.markUnseen();
          } else {
            MessagesService.markSeen();
          }
        });

        return doGet;
      }

      return ApiCacheService.get(endpoint, params);
    };

    methods.noCache = () => {
      cacheGetRequests = false;

      return methods;
    };

    /**
     * Bug Report
     */
    methods.bugReport = {};

    // Activate User
    methods.bugReport.send = data => HttpRequestService.post('report', data);

    /**
     * Users
     */
    methods.users = {};

    // Update Email
    methods.users.update = user => HttpRequestService.put('users', user);

    // Activate User
    methods.users.activate = (userId, code) => HttpRequestService.post(`users/${userId}/activate`, {
      activation_code: code
    });

    // Activate User Set Password
    methods.users.activateSetPassword = (userId, code, password) => HttpRequestService.post(`users/${userId}/activate_set_password`, {
      activation_code: code,
      password
    });

    // Set Photo
    methods.users.setPhoto = (userId, photoData) => HttpRequestService.put(`users/${userId}/photo`, {
      photo: photoData
    });

    // Set Password
    methods.users.setPassword = password => HttpRequestService.put('users/password', {
      password
    });

    // Set Email
    methods.users.setEmail = email => HttpRequestService.put('users/email', {
      email
    });

    // Set Email
    methods.users.setDisplayUnits = (displayUnits, type) => {
      const data = {};
      data[type] = displayUnits;
      return HttpRequestService.put('users/display_units', data);
    };

    // Request Password Reset
    methods.users.requestPasswordReset = email => HttpRequestService.post('users/requestpasswordreset', {
      email
    });

    // Do Password Reset
    methods.users.resetPassword = (userId, token, password) => HttpRequestService.post('users/dopasswordreset', {
      user_id: userId,
      token,
      password
    });

    // Get basic user info from activation code
    methods.users.getInfoByActivationCode = (userId, code) => getRequest(`users/${userId}/info/${code}`, {});

    // Resend Activation Email
    methods.users.resendActivationEmail = (emailAddress = false, userId = false) => {
      const data = {};

      if (emailAddress) {
        data.email = emailAddress;
      }

      if (userId) {
        data.user_id = userId;
      }

      return HttpRequestService.post('users/activate/resend', data);
    };

    // Unsubscribe
    methods.users.unsubscribe = (email, hash) => HttpRequestService.post('users/unsubscribe', {
      email,
      hash
    });

    /**
     * Trainers
     */
    methods.trainers = {};

    // Create Organisation
    methods.trainers.create = data => HttpRequestService.post('trainers', data);

    // Update
    methods.trainers.update = (trainerId, data) => HttpRequestService.put(`trainers/${trainerId}`, data);

    // Get one
    methods.trainers.get = trainerId => getRequest(`trainers/${trainerId}`, {});

    // Get Schedule
    methods.trainers.getSchedule = (dateStart, dateEnd, trainerUserId = false, fullWorkoutData = false) => {
      const params = {
        start: dateStart,
        end: dateEnd
      };

      if (trainerUserId) {
        params.trainer_user_id = trainerUserId;
      }

      if (fullWorkoutData) {
        params.full_workout_data = 1;
      }

      return getRequest('trainers/schedule', params);
    };

    // Set Logo
    methods.trainers.setLogo = (trainerId, photoData) => HttpRequestService.put(`trainers/${trainerId}/logo`, {
      logo: photoData
    });

    // Get Subdomain Branding
    methods.trainers.getSubdomainBranding = subdomain => getRequest(`domainbranding/${subdomain}`, {});

    // Get Branding By Trainer ID
    methods.trainers.getTrainerBranding = trainerId => getRequest(`domainbranding/id/${trainerId}`, {});

    // Get All Trainer's Workouts
    methods.trainers.getAllWorkouts = (forUser = false, fullWorkoutData = false) => {
      const data = {};

      if (forUser) {
        data.trainer_user_id = forUser;
      }

      if (fullWorkoutData) {
        data.full_workout_data = 1;
      }

      return getRequest('workouts', data);
    };

    // Get All Trainer's Workouts (Grouped by Workout Group)
    methods.trainers.getAllWorkoutsByGroup = forUser => {
      const data = {};

      if (forUser) {
        data.trainer_user_id = forUser;
      }

      return getRequest('workouts/by_group', data);
    };

    // Get All Trainer's Workouts Grouped By Customer
    methods.trainers.getAllWorkoutsGrouped = () => getRequest('workouts/grouped', {});

    // Get All Trainer's Nutrition Plans
    methods.trainers.getAllNutritionPlans = forUser => {
      const data = {};

      if (forUser) {
        data.trainer_user_id = forUser;
      }

      return getRequest('nutritionplans', data);
    };

    // Get All Trainer's Nutrition Plans (Grouped by Nutrition Plan Group)
    methods.trainers.getAllNutritionPlansByGroup = forUser => {
      const data = {};

      if (forUser) {
        data.trainer_user_id = forUser;
      }

      return getRequest('nutritionplans/by_group', data);
    };

    // List All
    methods.trainers.getAllNutritionPlansGrouped = forUser => {
      const data = {};

      if (forUser) {
        data.trainer_user_id = forUser;
      }

      return getRequest('nutritionplans/grouped', data);
    };

    // Check Subdomain is available
    methods.trainers.checkSubdomainAvailability = subdomain => getRequest(`trainers/check_subdomain_availability/${subdomain}`, {}, true);

    /**
     * Payment Settings
     */
    methods.paymentSettings = {};

    // Get Current Card
    methods.paymentSettings.get_card = () => getRequest('trainers/billing/card', {});

    // Update Card
    methods.paymentSettings.update_card = cardToken => HttpRequestService.put('trainers/billing/card', {
      stripe_token: cardToken
    });

    /**
     * Trainer Users
     */
    methods.trainerUsers = {};

    // Buy Trainer User Credits
    methods.trainerUsers.buy = data => HttpRequestService.post('trainers/users/buy', data);

    // Get Trainer User Buy Info
    methods.trainerUsers.buy_info = () => getRequest('trainers/users/buy_info', {});

    // Buy Trainer User Credits
    methods.trainerUsers.reactivate = data => HttpRequestService.post('trainers/users/reactivate', data);

    // Create Trainer User
    methods.trainerUsers.create = data => HttpRequestService.post('trainers/users/create', data);

    // Update Trainer User
    methods.trainerUsers.update = (userId, data) => HttpRequestService.put(`trainers/users/${userId}`, data);

    // Get Trainer User
    methods.trainerUsers.get = userId => getRequest(`trainers/users/${userId}`, {});

    // Get Trainer User
    methods.trainerUsers.delete = userId => HttpRequestService.delete(`trainers/users/${userId}`, {});

    // List All Trainer's Users
    methods.trainerUsers.list = excludeTrainerUserId => {
      const data = {};

      if (excludeTrainerUserId) {
        data.exclude_user_id = excludeTrainerUserId;
      }

      return getRequest('trainers/users', data);
    };

    /**
     * Scheduler
     */

    methods.scheduler = {};

    // Schedular Template Index (Trainer User)
    methods.scheduler.scheduleTemplateIndexTrainerUser = trainerUserId => getRequest('scheduler/templates', {
      trainer_user_id: trainerUserId
    });

    // Schedular Template Get
    methods.scheduler.scheduleTemplateGet = id => getRequest(`scheduler/templates/${id}`, {});

    // Schedular Template Create
    methods.scheduler.scheduleTemplateCreate = data => HttpRequestService.post('scheduler/templates', data);

    // Schedular Template Update
    methods.scheduler.scheduleTemplateUpdate = (id, data) => HttpRequestService.put(`scheduler/templates/${id}`, data);

    // Schedular Template Delete
    methods.scheduler.scheduleTemplateDelete = id => HttpRequestService.delete(`scheduler/templates/${id}`, {});
    // Assign Customer Group To Schedule
    methods.scheduler.assignCustomerGroup = (scheduleId, data) => HttpRequestService.post(`scheduler/templates/${scheduleId}/assign_customer_group`, data);

    // Unassign Customer Group From Schedule
    methods.scheduler.unassignCustomerGroup = (scheduleId, customerGroupId) => HttpRequestService.post(`scheduler/templates/${scheduleId}/unassign_customer_group`, {
      customer_group_id: customerGroupId
    });

    /**
     * Schedule Library
     */

    methods.scheduleLibrary = {};

    // List All Library Schedules
    methods.scheduleLibrary.listAll = () => getRequest('scheduler/library/all', {});

    // Schedular Template Get
    methods.scheduleLibrary.get = id => getRequest(`scheduler/templates/${id}`, {});

    // Schedular Template Duplicate
    methods.scheduleLibrary.duplicate = id => HttpRequestService.post(`scheduler/templates/${id}/duplicate`);

    /**
     * Customers
     */
    methods.customers = {};

    // List
    methods.customers.list = (trainerUserId = false, includeCounts = false) => {
      let qStr = '';

      if (trainerUserId) {
        qStr = `?trainer_user_id=${trainerUserId}`;
      }

      if (includeCounts) {
        if (qStr.length > 0) {
          qStr += '&';
        } else {
          qStr += '?';
        }

        qStr += `include_counts=${includeCounts.join(',')}`;
      }

      return getRequest(`customers${qStr}`, {});
    };

    // List All For Organisation
    methods.customers.listAllForOrg = includeCounts => {
      let qStr = '';
      if (includeCounts) {
        qStr += `?include_counts=${includeCounts.join(',')}`;
      }

      return getRequest(`customers/all${qStr}`, {});
    };

    // Create
    methods.customers.create = data => HttpRequestService.post('customers', data);

    // Update
    methods.customers.update = (customerId, data) => HttpRequestService.put(`customers/${customerId}`, data);

    // Delete
    methods.customers.delete = customerId => HttpRequestService.delete(`customers/${customerId}`, {});

    // Get one
    methods.customers.get = customerId => getRequest(`customers/${customerId}`, {});

    // Get calendar events for schedule
    methods.customers.getSchedulerEvents = customerId => getRequest(`customers/${customerId}/scheduler/events`, {});

    // Get Calendar Events (sessions and scheduler)
    methods.customers.getCalendarEvents = (customerId, dateStart, dateEnd) => getRequest(`customers/${customerId}/calendar_events`, {
      start: dateStart,
      end: dateEnd
    });

    // Get active days of all active schedules for a customer
    methods.customers.getSchedulerActiveDays = customerId => getRequest(`customers/${customerId}/scheduler/activeDays`, {});

    // Start Schedule
    methods.customers.startSchedule = scheduleId => HttpRequestService.post(`schedule/${scheduleId}/start`, {});

    // Update customers schedule
    methods.customers.scheduleUpdate = (scheduleId, data) => HttpRequestService.put(`customers/schedule/${scheduleId}`, data);

    // Remove customers schedule
    methods.customers.scheduleDelete = scheduleId => HttpRequestService.delete(`customers/schedule/${scheduleId}`);

    // Get a customers current schedules
    methods.customers.getCurrentSchedules = customerId => getRequest(`customers/${customerId}/scheduler/current`, {});

    // Get a customers completed schedules
    methods.customers.getPreviousSchedules = customerId => getRequest(`customers/${customerId}/scheduler/previous`, {});

    // Get a specific schedule
    methods.customers.getSpecificSchedule = scheduleId => getRequest(`scheduler/schedule/${scheduleId}`, {});

    // Get one note
    methods.customers.getNote = (customerId, noteId) => getRequest(`customers/${customerId}/notes/${noteId}`, {});

    // Get all notes
    methods.customers.getNotes = customerId => getRequest(`customers/${customerId}/notes`, {});

    // Create Note
    methods.customers.createNote = (customerId, data) => HttpRequestService.post(`customers/${customerId}/notes`, data);

    // Update Note
    methods.customers.updateNote = (customerId, noteId, data) => HttpRequestService.put(`customers/${customerId}/notes/${noteId}`, data);

    // Delete Note
    methods.customers.deleteNote = (customerId, noteId) => HttpRequestService.delete(`customers/${customerId}/notes/${noteId}`, {});

    // List Workouts
    methods.customers.listWorkouts = (customerId, directOnly = false, fullWorkoutData = false, cancelPromise = false) => {
      const params = {};

      if (directOnly) {
        params.direct_only = 1;
      }

      if (fullWorkoutData) {
        params.full_workout_data = 1;
      }

      return getRequest(`customers/${customerId}/workouts`, params, cancelPromise);
    };

    // List Customer Created Workouts
    methods.customers.listCreatedWorkouts = (customerId, fullWorkoutData = false) => {
      const params = {};
      if (fullWorkoutData) {
        params.full_workout_data = 1;
      }

      return getRequest(`customers/${customerId}/created_workouts`, params);
    };

    // List Nutrition Plans
    methods.customers.listNutritionPlans = (customerId, directOnly = false) => {
      const params = {};

      if (directOnly) {
        params.direct_only = 1;
      }

      return getRequest(`customers/${customerId}/nutritionplans`, params);
    };

    // Get Progress
    methods.customers.getProgress = (customerId, dateStart, dateEnd) => {
      const params = {
        start: dateStart,
        end: dateEnd
      };

      return getRequest(`customers/${customerId}/progress`, params);
    };

    // Get Schedule
    methods.customers.getSchedule = (customerId, dateStart, dateEnd, fullWorkoutData = false) => {
      const params = {
        start: dateStart,
        end: dateEnd
      };

      if (fullWorkoutData) {
        params.full_workout_data = 1;
      }

      return getRequest(`customers/${customerId}/schedule`, params);
    };

    methods.customers.getFiles = customerId => getRequest(`customers/${customerId}/files`);

    /**
     * Nutrition
     */
    methods.nutrition = {};

    // Save Nutrition Settings
    methods.nutrition.saveSettings = (customerId, data) => HttpRequestService.put(`customers/${customerId}/settings/nutrition`, data);

    // Get Nutrition Settings
    methods.nutrition.getSettings = customerId => getRequest(`customers/${customerId}/settings/nutrition`, {});

    // Get Nutrition Plan
    methods.nutrition.getPlan = planId => getRequest(`nutrition/plans/${planId}`, {});

    // Add Nutrition Plan
    methods.nutrition.addPlan = data => HttpRequestService.post('nutrition/plans', data);

    // Update Nutrition Plan
    methods.nutrition.updatePlan = (planId, data) => HttpRequestService.put(`nutrition/plans/${planId}`, data);

    // Duplicate Nutrition Plan
    methods.nutrition.duplicatePlan = (planId, forUser = false) => {
      const data = {};
      if (forUser) {
        data.trainer_user_id = forUser;
      }

      return HttpRequestService.post(`nutrition/plans/${planId}/duplicate`, data);
    };

    // Delete Nutrition Plan
    methods.nutrition.deletePlan = planId => HttpRequestService.delete(`nutrition/plans/${planId}`, {});

    // Get the Recent and Frequent foods
    methods.nutrition.getRecentFrequentPersonal = data => getRequest('nutrition/recentfrequentpersonal', data);

    // Search Food
    methods.nutrition.searchFood = (query, forUser = false) => {
      let forUserStr = '';
      if (forUser) {
        forUserStr = `&forUser=${forUser}`;
      }

      return getRequest(`nutrition/food/search?query=${encodeURIComponent(query)}${forUserStr}`, {});
    };

    // Assign Customer To Nutrition Plan
    methods.nutrition.assignCustomer = (planId, customerId) => HttpRequestService.post(`nutrition/plans/${planId}/assign_customer`, {
      customer_id: customerId
    });

    // Unassign Customer From Nutrition Plan
    methods.nutrition.unassignCustomer = (planId, customerId) => HttpRequestService.post(`nutrition/plans/${planId}/unassign_customer`, {
      customer_id: customerId
    });

    // Assign Customer Group To Nutrition Plan
    methods.nutrition.assignCustomerGroup = (planId, customerGroupId) => HttpRequestService.post(`nutrition/plans/${planId}/assign_customer_group`, {
      customer_group_id: customerGroupId
    });

    // Unassign Customer Group From Nutrition Plan
    methods.nutrition.unassignCustomerGroup = (planId, customerGroupId) => HttpRequestService.post(`nutrition/plans/${planId}/unassign_customer_group`, {
      customer_group_id: customerGroupId
    });

    // Get Food
    methods.nutrition.getFood = foodId => getRequest(`nutrition/food/${foodId}`, {});

    // Get custom nutrition item
    methods.nutrition.getCustomNutritionItem = foodId => getRequest(`nutrition/custom/${foodId}`, {});

    // Get Food By External ID
    methods.nutrition.getFoodByExternalId = externalId => getRequest(`nutrition/food/external/${externalId}`, {});

    methods.nutrition.findNutritionByBarcode = barcode => getRequest(`nutrition/food/barcode/${barcode}`, {});

    // Add a Custom Nutrition Item
    methods.nutrition.addNutritionItem = data => HttpRequestService.post('nutrition/custom', data);

    // Update a Custom Nutrition Item
    methods.nutrition.updateNutritionItem = (itemId, data) => HttpRequestService.put(`nutrition/custom/${itemId}`, data);

    // Delete a Custom Nutrition Item
    methods.nutrition.deleteCustomNutritionItem = deleteId => HttpRequestService.delete(`nutrition/custom/${deleteId}`, {});

    // Get Shopping List
    methods.nutrition.getShoppingList = customerId => getRequest(`customers/${customerId}/shoppinglist`, {});

    /**
     * Nutrition Plan Groups
     */
    methods.nutritionGroups = {};

    // Create Group
    methods.nutritionGroups.create = (data, forUser = false) => {
      if (forUser) {
        data.trainer_user_id = forUser;
      }

      return HttpRequestService.post('nutritionplans/groups', data);
    };

    // Update Group
    methods.nutritionGroups.update = (groupId, data) => HttpRequestService.put(`nutritionplans/groups/${groupId}`, data);

    // Get Group
    methods.nutritionGroups.get = groupId => getRequest(`nutritionplans/groups/${groupId}`, {});

    // Duplicate Group
    methods.nutritionGroups.duplicate = (groupId, forUser = false) => {
      const data = {};
      if (forUser) {
        data.trainer_user_id = forUser;
      }

      return HttpRequestService.post(`nutritionplans/groups/${groupId}/duplicate`, data);
    };

    // Delete Group
    methods.nutritionGroups.delete = groupId => HttpRequestService.delete(`nutritionplans/groups/${groupId}`, {});

    // List All
    methods.nutritionGroups.list = forUser => {
      const data = {};

      if (forUser) {
        data.trainer_user_id = forUser;
      }

      return getRequest('nutritionplans/groups', data);
    };

    // Get Group
    methods.nutritionGroups.listForCustomer = (customerId, includeAssignedViaCustomerGroups = false) => {
      const data = {};
      if (includeAssignedViaCustomerGroups) {
        data.include_assigned_via_customer_groups = true;
      }

      return getRequest(`customers/${customerId}/nutritionplans/groups`, data);
    };

    // Assign Customer To Nutrition Group
    methods.nutritionGroups.assignCustomer = (groupId, customerId) => HttpRequestService.post(`nutritionplans/groups/${groupId}/assign_customer`, {
      customer_id: customerId
    });

    // Unassign Customer From Nutrition Group
    methods.nutritionGroups.unassignCustomer = (groupId, customerId) => HttpRequestService.post(`nutritionplans/groups/${groupId}/unassign_customer`, {
      customer_id: customerId
    });

    // Assign Customer Group To Nutrition Group
    methods.nutritionGroups.assignCustomerGroup = (groupId, customerGroupId) => HttpRequestService.post(`nutritionplans/groups/${groupId}/assign_customer_group`, {
      customer_group_id: customerGroupId
    });

    // Unassign Customer Group From Nutrition Group
    methods.nutritionGroups.unassignCustomerGroup = (groupId, customerGroupId) => HttpRequestService.post(`nutritionplans/groups/${groupId}/unassign_customer_group`, {
      customer_group_id: customerGroupId
    });

    /**
     * Shopping Lists
     */
    methods.shoppingList = {};

    // Add Purchase
    methods.shoppingList.addPurchase = foodId => HttpRequestService.post('shoppinglist/purchases', {
      food_id: foodId
    });

    // Remove Purchase
    methods.shoppingList.removePurchase = foodId => HttpRequestService.delete(`shoppinglist/purchases/${foodId}`, {});

    /**
     * Nutrition Logs
     */
    methods.nutritionlog = {};

    // Get Nutrition Log By Date
    methods.nutritionlog.dayLog = (customerId, date) => getRequest(`customers/${customerId}/nutrition/log/day/on/${date}`, {});

    // Create Log Day
    methods.nutritionlog.createDay = data => HttpRequestService.post('nutrition/log/day', data);

    // Update Log Day
    methods.nutritionlog.updateDay = (dayId, data) => HttpRequestService.put(`nutrition/log/day/${dayId}`, data);

    /**
     * Messages
     */
    methods.messages = {};

    // Send
    methods.messages.send = (toUserId, message) => HttpRequestService.post('messages', {
      to_user_id: toUserId,
      message
    });

    // Inbox
    methods.messages.getInbox = data => HttpRequestService.requestv5('messages', data);

    methods.messages.getLatest = (conversationId, id) => HttpRequestService.requestv5(`messages/conversation/${conversationId}/latest/${id}`);

    // View Thread
    methods.messages.getThread = (conversationId, data) => HttpRequestService.requestv5(`messages/conversation/${conversationId}`, data);

    // View Message

    // Get Contacts
    methods.messages.getContacts = () => getRequest('messages/contacts', {});

    /**
     * Sessions
     */
    methods.sessions = {};

    // Book Session
    methods.sessions.book = (workoutId, customerId, date, instant, trainerUserId = false, rrule, endDate = false, scheduleId = false, scheduleType = false, location) => {
      const postData = {
        workout_id: workoutId,
        customer_id: customerId,
        date,
        instant: instant ? 1 : 0,
        rrule,
        location
      };

      if (trainerUserId) {
        postData.trainer_user_id = trainerUserId;
      }

      if (endDate) {
        postData.end_date = endDate;
      }

      if (scheduleId) {
        postData.schedule_id = scheduleId;
      }

      if (scheduleType) {
        postData.schedule_type = scheduleType;
      }

      return HttpRequestService.post('sessions', postData);
    };

    // Book Session from schedule
    methods.sessions.bookFromSchedule = (workoutId, customerId, date, scheduleId, scheduleType) => {
      const postData = {
        workout_id: workoutId,
        customer_id: customerId,
        date,
        schedule_id: scheduleId,
        schedule_type: scheduleType
      };

      return HttpRequestService.post('sessions', postData);
    };

    // Update Session
    methods.sessions.update = (sessionId, workoutId, date, occurrenceDate, updateOccurrences, location) => {
      let reqUri = `sessions/${sessionId}`;

      if (occurrenceDate) {
        const date = occurrenceDate.split('+').join(' ');
        reqUri = `${reqUri}/occurrence/${date}`;
      }

      return HttpRequestService.put(reqUri, {
        workout_id: workoutId, // Ignored for an occurrence
        date,
        update_recurring: updateOccurrences,
        location
      });
    };

    // Get Session
    methods.sessions.get = (sessionId, occurrenceDate = false) => {
      let uri = `sessions/${sessionId}`;
      if (occurrenceDate) {
        uri = `${uri}/occurrence/${occurrenceDate}`;
      }
      return getRequest(uri, {});
    };

    // Complete Session
    methods.sessions.complete = (sessionId, data) => HttpRequestService.post(`sessions/${sessionId}/complete`, data);

    // Complete Session with AdHoc Workout
    methods.sessions.completeWithAdhocWorkout = (sessionId, data) => HttpRequestService.post(`sessions/${sessionId}/complete_with_adhoc_workout`, data);

    // Cancel Session
    methods.sessions.cancel = (sessionId, reason, dateTime = false, cancelAllOccurrences = false, billable = false, billableToQuota = false) => {
      let date;
      if (dateTime) {
        date = dateTime.split('+').join(' ');
      } else {
        date = dateTime;
      }

      const data = {
        reason,
        datetime: date,
        cancelAllOccurrences
      };

      if (billable) {
        data.billable = billable;
      }

      if (billableToQuota) {
        data.billable_to_quota = billableToQuota;
      }

      return HttpRequestService.post(`sessions/${sessionId}/cancel`, data);
    };

    // Request Session Cancellation
    methods.sessions.requestCancellation = (sessionId, reason, dateTime) => HttpRequestService.post(`sessions/${sessionId}/cancellation_request`, {
      reason,
      datetime: dateTime
    });

    // Request Session Cancellation
    methods.sessions.rejectCancellationRequest = (sessionId, reason, dateTime) => HttpRequestService.post(`sessions/${sessionId}/reject_cancellation_request`, {
      reason,
      datetime: dateTime
    });

    /**
     * Subscription Permissions Checkers
     */
    methods.subscription_checker = {};

    // Can Create Customer
    methods.subscription_checker.canCreateCustomer = () => getRequest('subscription_checker/can_create_customer', {});

    // Can Additional Trainer
    methods.subscription_checker.canCreateAdditionalTrainer = () => getRequest('subscription_checker/can_create_additional_trainer', {});

    // Can Additional Trainer
    methods.subscription_checker.canCreateWorkout = () => getRequest('subscription_checker/can_create_workout', {});

    // Can Additional Trainer
    methods.subscription_checker.canCreateNutritionPlan = () => getRequest('subscription_checker/can_create_nutrition_plan', {});

    /**
     * Payment
     */
    methods.payment = {};

    // Pay for a subscription
    methods.payment.subscribe = data => HttpRequestService.post('payments/subscribe', data);

    // Get available subscription plans
    methods.payment.plans = coupon => getRequest('payments/plans', {
      coupon
    });

    /**
     * Payment
     */
    methods.coupons = {};

    methods.coupons.getByCode = code => getRequest(`coupons/code/${code}`, {});

    /**
     * Workouts
     */
    methods.workouts = {};

    // Add Workout
    methods.workouts.create = workout => HttpRequestService.post('workouts', workout);

    // Update Workout
    methods.workouts.update = (workoutId, workout) => HttpRequestService.put(`workouts/${workoutId}`, workout);

    // Assign Customer To Workout
    methods.workouts.assignCustomer = (workoutId, customerId) => HttpRequestService.post(`workouts/${workoutId}/assign_customer`, {
      customer_id: customerId
    });

    // Unassign Customer From Workout
    methods.workouts.unassignCustomer = (workoutId, customerId) => HttpRequestService.post(`workouts/${workoutId}/unassign_customer`, {
      customer_id: customerId
    });

    // Assign Customer Group To Workout
    methods.workouts.assignCustomerGroup = (workoutId, customerGroupId) => HttpRequestService.post(`workouts/${workoutId}/assign_customer_group`, {
      customer_group_id: customerGroupId
    });

    // Unassign Customer Group From Workout
    methods.workouts.unassignCustomerGroup = (workoutId, customerGroupId) => HttpRequestService.post(`workouts/${workoutId}/unassign_customer_group`, {
      customer_group_id: customerGroupId
    });

    // Duplicate Workout
    methods.workouts.duplicate = (workoutId, forUser = false) => {
      const data = {};
      if (forUser) {
        data.trainer_user_id = forUser;
      }

      return HttpRequestService.post(`workouts/${workoutId}/duplicate`, data);
    };

    // Delete Workout
    methods.workouts.delete = workoutId => HttpRequestService.delete(`workouts/${workoutId}`, {});

    // Get Workout
    methods.workouts.get = (workoutId, standardSort = false) => {
      const data = {};

      if (standardSort) {
        data.standardSort = 1;
      }

      return getRequest(`workouts/${workoutId}`, data);
    };

    // List Categories
    methods.workouts.getCategories = () => getRequest('workouts/categories', {});

    // Exercise History
    methods.workouts.getExerciseHistory = (id, customerId, allWorkouts = false, idType, fields = {}, workoutId = false) => getRequest(`workouts/history/exercise/${id}/${customerId}/${allWorkouts ? 'all' : 'this'}/${idType}${workoutId ? `/${workoutId}` : ''}`, fields);

    /**
     * Workout Groups
     */
    methods.workoutGroups = {};

    // Create Group
    methods.workoutGroups.create = (data, forUser = false) => {
      if (forUser) {
        data.trainer_user_id = forUser;
      }

      return HttpRequestService.post('workouts/groups', data);
    };

    // Delete Group
    methods.workoutGroups.update = (groupId, data) => HttpRequestService.put(`workouts/groups/${groupId}`, data);

    // Get Group
    methods.workoutGroups.get = (groupId, tagFilter = null) => {
      const data = {};
      if (tagFilter) {
        data.tag = tagFilter;
      }

      return getRequest(`workouts/groups/${groupId}`, data);
    };

    // Duplicate Group
    methods.workoutGroups.duplicate = (groupId, forUser = false) => {
      const data = {};
      if (forUser) {
        data.trainer_user_id = forUser;
      }

      return HttpRequestService.post(`workouts/groups/${groupId}/duplicate`, data);
    };

    // Delete Group
    methods.workoutGroups.delete = groupId => HttpRequestService.delete(`workouts/groups/${groupId}`, {});

    // List All
    methods.workoutGroups.list = forUser => {
      const data = {};
      if (forUser) {
        data.trainer_user_id = forUser;
      }

      return getRequest('workouts/groups', data);
    };

    // Get Group
    methods.workoutGroups.listForCustomer = (customerId, includeAssignedViaCustomerGroups = false) => {
      const data = {};
      if (includeAssignedViaCustomerGroups) {
        data.include_assigned_via_customer_groups = true;
      }

      return getRequest(`customers/${customerId}/workouts/groups`, data);
    };

    // Assign Customer To Workout
    methods.workoutGroups.assignCustomer = (groupId, customerId) => HttpRequestService.post(`workouts/groups/${groupId}/assign_customer`, {
      customer_id: customerId
    });

    // Unassign Customer From Workout
    methods.workoutGroups.unassignCustomer = (groupId, customerId) => HttpRequestService.post(`workouts/groups/${groupId}/unassign_customer`, {
      customer_id: customerId
    });

    // Assign Customer Group To Workout Group
    methods.workoutGroups.assignCustomerGroup = (groupId, customerGroupId) => HttpRequestService.post(`workouts/groups/${groupId}/assign_customer_group`, {
      customer_group_id: customerGroupId
    });

    // Unassign Customer Group From Workout Group
    methods.workoutGroups.unassignCustomerGroup = (groupId, customerGroupId) => HttpRequestService.post(`workouts/groups/${groupId}/unassign_customer_group`, {
      customer_group_id: customerGroupId
    });

    /**
     * Exercises
     */
    methods.exercises = {};

    // Search Exercises
    methods.exercises.search = (query, perPage, currentPage = '1') => getRequest(`exercises/suggest?query=${encodeURIComponent(query)}&per_page=${encodeURIComponent(perPage)}&current_page=${encodeURIComponent(currentPage)}`, {});

    // Get suggested alternatives
    methods.exercises.getAlternatives = exerciseId => getRequest(`exercises/${exerciseId}/alternatives`, {});

    // Create Draft Exercise
    methods.exercises.createDraft = () => HttpRequestService.post('exercises/draft/create', {});

    // Get Exercise
    methods.exercises.get = exerciseId => getRequest(`exercises/${exerciseId}`, {});

    // Get Exercise
    methods.exercises.getAllForTrainer = (perPage, currentPage) => {
      const data = {};

      if (perPage) {
        data.per_page = perPage;
      }

      if (currentPage) {
        data.current_page = currentPage;
      }

      return getRequest('exercises/trainer', data);
    };

    // Get exercises frequently used
    methods.exercises.getFrequentlyUsed = (perPage = false, currentPage = false) => {
      const data = {};
      if (perPage) {
        data.per_page = perPage;
      }

      if (currentPage) {
        data.current_page = currentPage;
      }

      return getRequest('exercises/frequently_used', data);
    };

    // Get exercises recently used
    methods.exercises.getRecentlyUsed = (perPage = false, currentPage = false) => {
      const data = {};
      if (perPage) {
        data.per_page = perPage;
      }

      if (currentPage) {
        data.current_page = currentPage;
      }

      return getRequest('exercises/recently_used', data);
    };

    // Update
    methods.exercises.update = (exerciseId, exercise, keepStatus = false) => {
      exercise.keep_status = keepStatus;
      return HttpRequestService.put(`exercises/${exerciseId}`, exercise);
    };

    // Set Status
    methods.exercises.setStatus = (exerciseId, status) => HttpRequestService.put(`exercises/${exerciseId}/status`, {
      status
    });

    // List Categories
    methods.exercises.categories = () => getRequest('exercises/categories/list', {});

    // Get Exercise Attributes
    methods.exercises.getAttributes = exerciseId => getRequest(`exercises/${exerciseId}/attributes`, {});

    // Get Available Attributes
    methods.exercises.getAvailableAttributes = () => getRequest('exercises/available_attributes', {});

    // Get attribute values for name ID
    methods.exercises.getAttributeValuesByNameId = nameId => getRequest(`exercises/attributes_by_name_id/${nameId}`, {});

    // Get exercises values for attribute value ID
    methods.exercises.getExercisesByValueId = (valueId, perPage, currentPage = '1') => getRequest(`exercises/attribute_value_id/${valueId}?per_page=${encodeURIComponent(perPage)}&current_page=${encodeURIComponent(currentPage)}`, {});

    // Get exercises values for attribute value ID
    methods.exercises.getCustomerUsedExercises = (customerId, perPage, currentPage = '1', attributeValueId = false, excludeIds = []) => getRequest(`exercises/customer/${customerId}?per_page=${encodeURIComponent(perPage)}&current_page=${encodeURIComponent(currentPage)}${attributeValueId ? `&attribute_value_id=${attributeValueId}` : ''}${excludeIds.length > 0 ? `&exclude_ids=${excludeIds.join(',')}` : ''}`, {});

    // Get Photos
    methods.exercises.getPhotos = exerciseId => getRequest(`exercises/${exerciseId}/photos`, {});

    // Add Photo
    methods.exercises.addPhoto = (exerciseId, photoData, sort) => HttpRequestService.post(`exercises/${exerciseId}/photos`, {
      image: photoData,
      sort
    });

    // Update Photo Sort Orders
    methods.exercises.updatePhotoSort = data => HttpRequestService.post('exercises/photos/sorts', data);

    // Delete Photo
    methods.exercises.deletePhoto = photoId => HttpRequestService.delete(`exercises/photos/${photoId}`, {});

    /**
     * Customer Metrics
     */
    methods.metrics = {};

    // Update Metrics
    methods.metrics.update = (customerId, data) => HttpRequestService.post(`customers/${customerId}/metrics`, data);

    // Latest Metrics
    methods.metrics.latest = customerId => getRequest(`customers/${customerId}/metrics/latest`, {});

    // Get Metrics For Date
    methods.metrics.getForDate = (customerId, date) => getRequest(`customers/${customerId}/metrics/${date}`, {});

    // All Previous Metrics For Customer
    methods.metrics.all = customerId => getRequest(`customers/${customerId}/metrics`, {});

    // Set Legacy Data Side
    methods.metrics.swapLegacyDataSide = (customerId, side, type) => HttpRequestService.post(`customers/${customerId}/metrics/swap_legacy_data_side`, {
      side,
      type
    });

    /**
     * Notification Settings
     */
    methods.notificationSettings = {};

    // Update Metrics
    methods.notificationSettings.get = () => getRequest('settings/notifications', {});

    // Latest Metrics
    methods.notificationSettings.set = data => HttpRequestService.put('settings/notifications', {
      notifications: data
    });

    /**
     * Notifications
     */
    methods.notifications = {};

    // Get index of notifications for user
    methods.notifications.index = (userId, page = 1, perPage = 25) => getRequest(`users/${userId}/notifications?page=${page}&perPage=${perPage}`, {});

    // Get unread notifications for user
    methods.notifications.unreadIndex = userId => getRequest(`users/${userId}/notifications/unread`, {});

    // Count unread notifications for user
    methods.notifications.countUnread = userId => getRequest(`users/${userId}/notifications/unread/count`, {});

    // Mark a notification as read
    methods.notifications.markAsRead = notificationId => HttpRequestService.put(`users/notifications/${notificationId}/read`, {});

    // Mark a notification as read
    methods.notifications.markAllAsRead = userId => HttpRequestService.put(`users/${userId}/notifications/readall`, {}, 'PUT', true);

    // Delete a notification
    methods.notifications.delete = notificationId => HttpRequestService.delete(`users/notifications/${notificationId}`, {});

    // Delete all notifications
    methods.notifications.deleteAll = userId => HttpRequestService.delete(`users/${userId}/notifications/deleteall`, {});

    /**
     * Customer Progress Photos
     */
    methods.progressPhotos = {};

    // Add Photo
    methods.progressPhotos.add = (customerId, data) => HttpRequestService.post(`customers/${customerId}/progressphotos`, data);

    // Delete Photo
    methods.progressPhotos.delete = (customerId, photoId) => HttpRequestService.delete(`customers/${customerId}/progressphotos/${photoId}`, {});

    // List Customer Photos
    methods.progressPhotos.list = customerId => getRequest(`customers/${customerId}/progressphotos`, {});

    // List Customer Photos
    methods.progressPhotos.updateSorts = data => HttpRequestService.put('customers/progressphotos/sorts', data);

    /**
     * PAR-Q
     */
    methods.parq = {};

    // Set Answers
    methods.parq.setAnswers = (customerId, answers) => HttpRequestService.put(`customers/${customerId}/parq/answers`, {
      answers
    });

    // Get Questions (and answers)
    methods.parq.getQuestions = customerId => getRequest(`customers/${customerId}/parq/questions`, {});

    /**
     * Tour Log
     */
    methods.tourLog = {};

    // Log Tour
    methods.tourLog.log = tourName => HttpRequestService.post('tourlog/log', {
      tour_name: tourName
    });

    // Get Log
    methods.tourLog.getLog = () => getRequest('tourlog/get');

    /**
     * Analytics
     */
    methods.analytics = {};

    // Bodyweight
    methods.analytics.bodyweight = (customerId, fromDate, toDate) => getRequest(`customers/${customerId}/analytics/bodyweight`, {
      dateFrom: fromDate,
      dateTo: toDate
    });

    // Measurements
    methods.analytics.measurements = (customerId, fromDate, toDate) => getRequest(`customers/${customerId}/analytics/measurements`, {
      dateFrom: fromDate,
      dateTo: toDate
    });

    // Bodyfat
    methods.analytics.bodyfat = (customerId, fromDate, toDate) => getRequest(`customers/${customerId}/analytics/bodyfat`, {
      dateFrom: fromDate,
      dateTo: toDate
    });

    // Completed Workouts
    methods.analytics.completedWorkouts = (customerId, fromDate, toDate) => getRequest(`customers/${customerId}/analytics/completed_workouts`, {
      dateFrom: fromDate,
      dateTo: toDate
    });

    // Distance Covered
    methods.analytics.distanceCovered = (customerId, fromDate, toDate) => getRequest(`customers/${customerId}/analytics/sum/distance_covered`, {
      dateFrom: fromDate,
      dateTo: toDate
    });

    // Weight Lifted
    methods.analytics.weightLifted = (customerId, fromDate, toDate) => getRequest(`customers/${customerId}/analytics/sum/weight_lifted`, {
      dateFrom: fromDate,
      dateTo: toDate
    });

    // Time Spent
    methods.analytics.timeSpent = (customerId, fromDate, toDate) => getRequest(`customers/${customerId}/analytics/sum/time_spent`, {
      dateFrom: fromDate,
      dateTo: toDate
    });

    // Session Time Elapsed
    methods.analytics.sessionTime = (customerId, fromDate, toDate) => getRequest(`customers/${customerId}/analytics/sum/session_time`, {
      dateFrom: fromDate,
      dateTo: toDate
    });

    // Specific Exercises
    methods.analytics.specificExercises = (customerId, exerciseId, fromDate, toDate) => getRequest(`customers/${customerId}/analytics/exercise/${exerciseId}`, {
      dateFrom: fromDate,
      dateTo: toDate
    });

    // Activities
    methods.analytics.activities = (customerId, toDate) => getRequest(`hardware/fitbit/getdayactivity/date/${customerId}/${toDate}`);

    // Steps Made
    methods.analytics.steps = (customerId, fromDate, toDate) => getRequest(`hardware/fitbit/getsteps/date/${customerId}/${fromDate}/${toDate}`);

    // Heart Rate
    methods.analytics.heartRate = (customerId, fromDate, toDate) => getRequest(`hardware/fitbit/getheartrate/date/${customerId}/${fromDate}/${toDate}`);

    /**
     * Activity Log
     */
    methods.activityLog = {};

    // Current User
    methods.activityLog.currentUser = (perPage, currentPage, type = false, seen = null, dateFrom = false, dateTo = false) => {
      const data = {
        per_page: perPage,
        current_page: currentPage,
        enable_packages: 1,
        enable_forms: 1,
        enable_scheduler: 1
      };

      if (type) {
        data.type = type;
      }

      if (seen !== null) {
        data.seen = seen ? 1 : 0;
      }

      if (dateFrom) {
        data.date_from = dateFrom;
      }

      if (dateTo) {
        data.date_to = dateTo;
      }

      return getRequest('activity_log', data);
    };

    // Customer
    methods.activityLog.customer = (customerId, perPage, currentPage, type = false, seen = null, dateFrom = false, dateTo = false) => {
      const data = {
        per_page: perPage,
        current_page: currentPage,
        enable_packages: 1,
        enable_forms: 1
      };

      if (type) {
        data.type = type;
      }

      if (seen !== null) {
        data.seen = seen ? 1 : 0;
      }

      if (dateFrom) {
        data.date_from = dateFrom;
      }

      if (dateTo) {
        data.date_to = dateTo;
      }

      return getRequest(`customers/${customerId}/activity_log`, data);
    };

    // Trainer User
    methods.activityLog.trainerUser = (trainerUserId, perPage, currentPage, type = false, seen = null, dateFrom = false, dateTo = false) => {
      const data = {
        per_page: perPage,
        current_page: currentPage,
        enable_packages: 1,
        enable_forms: 1
      };

      if (type) {
        data.type = type;
      }

      if (seen !== null) {
        data.seen = seen ? 1 : 0;
      }

      if (dateFrom) {
        data.date_from = dateFrom;
      }

      if (dateTo) {
        data.date_to = dateTo;
      }

      return getRequest(`trainers/users/${trainerUserId}/activity_log`, data);
    };

    // Mark Customer Activity As Seen
    methods.activityLog.markCustomerSeen = (customerId, type = false) => HttpRequestService.post(`customers/${customerId}/activity_log/mark_all_as_seen`, {
      type
    });

    /**
     * Device
     */
    methods.device = {};

    // Register Device Token
    methods.device.register = (deviceId, platform, bundleId) => HttpRequestService.post('devices/register', {
      device_id: deviceId,
      platform,
      bundle_id: bundleId
    });

    // Unregister Device Token
    methods.device.unregister = (deviceId, platform) => HttpRequestService.post('devices/unregister', {
      device_id: deviceId,
      platform
    });

    /**
     * Packages
     */
    methods.packages = {};

    // Get package (authed)
    methods.packages.get = packageId => getRequest(`packages/${packageId}`, {});

    // Get public package
    methods.packages.getPublic = packageId => getRequest(`packages/public/${packageId}`, {});

    // List public packages for all trainers in organisation
    methods.packages.listPublic = trainerId => getRequest(`trainers/${trainerId}/packages`, {});

    // List public packages for trainer
    methods.packages.listPublicForTrainer = (trainerId, trainerUserId) => getRequest(`trainers/${trainerId}/users/${trainerUserId}/packages`, {});

    // List all packages
    methods.packages.list = trainerUserId => {
      const data = {};

      if (trainerUserId) {
        data.trainer_user_id = trainerUserId;
      }

      return getRequest('packages', data);
    };

    // List all available customer packages
    methods.packages.listCustomer = customerId => getRequest(`customers/${customerId}/packages`, {});

    // List all deleted packages
    methods.packages.listDeleted = trainerUserId => {
      const data = {};

      if (trainerUserId) {
        data.trainer_user_id = trainerUserId;
      }

      return getRequest('packages/deleted/all', data);
    };

    // Create Package
    methods.packages.create = data => HttpRequestService.post('packages', data, 'POST');

    // Update Package
    methods.packages.update = (packageId, data) => HttpRequestService.put(`packages/${packageId}`, data);

    // Duplicate Package
    methods.packages.duplicate = packageId => HttpRequestService.post(`packages/${packageId}/duplicate`, {});

    // Invite Customers To Package
    methods.packages.invite = (packageId, data) => HttpRequestService.post(`packages/${packageId}/invite`, data);

    // Soft Delete Package
    methods.packages.softDelete = packageId => HttpRequestService.delete(`packages/${packageId}`, {});

    // Soft Delete Package
    methods.packages.restoreSoftDeleted = packageId => HttpRequestService.post(`packages/${packageId}/restore_deleted`, {});

    // Hard Delete Package
    methods.packages.hardDelete = packageId => HttpRequestService.delete(`packages/${packageId}/hard_delete`, {});

    // Purchase Package
    methods.packages.purchase = (packageId, data) => HttpRequestService.post(`packages/${packageId}/purchase`, data);

    // RePurchase Package
    methods.packages.repurchase = purchaseId => HttpRequestService.post(`packages/purchases/${purchaseId}/repurchase`, {});

    // Purchase Public Package
    methods.packages.purchasePublic = (packageId, data) => HttpRequestService.post(`packages/${packageId}/purchase_public`, data);

    // Get customer sessions quota
    methods.packages.getSessionQuota = customerId => getRequest(`customers/${customerId}/quota`, {});

    // Get customer active session quotas
    methods.packages.getActiveSessionQuotas = customerId => getRequest(`customers/${customerId}/quotas/active`, {});

    // Get customer sessions quota history
    methods.packages.getSessionQuotaHistory = customerId => getRequest(`customers/${customerId}/quota/history`, {});

    // List all customer purchases
    methods.packages.listCustomerPurchases = customerId => getRequest(`customers/${customerId}/packages/purchases`, {});

    // List all active customer purchases
    methods.packages.listActiveCustomerPurchases = customerId => getRequest(`customers/${customerId}/packages/purchases/active`, {});

    // List all active customer purchases with sessions remaining
    methods.packages.listActiveCustomerPurchasesWithRemainingSessions = customerId => getRequest(`customers/${customerId}/packages/purchases/active/with_remaining_sessions`);

    // List all active customer purchases with workouts
    methods.packages.listActiveCustomerPurchasesWithWorkouts = customerId => getRequest(`customers/${customerId}/packages/purchases/active/with_workouts`);

    // List all active customer purchases with nutrition plans
    methods.packages.listActiveCustomerPurchasesWithNutritionPlans = customerId => getRequest(`customers/${customerId}/packages/purchases/active/with_nutrition_plans`);

    // List all package tags
    methods.packages.listAllTags = () => getRequest('packages/tags/all', {});

    // List all coupons
    methods.packages.listAllCoupons = trainerUserId => {
      const data = {};

      if (trainerUserId) {
        data.trainer_user_id = trainerUserId;
      }

      return getRequest('packages/coupons/all', data);
    };

    // Get coupon by ID
    methods.packages.getCoupon = couponId => getRequest(`packages/coupons/${couponId}`, {});

    // Get coupon by Code
    methods.packages.getCouponByCode = (trainerUserId, code) => getRequest(`packages/coupons/${trainerUserId}/${code}`, {});

    // Create coupon
    methods.packages.createCoupon = data => HttpRequestService.post('packages/coupons', data);

    // Update coupon
    methods.packages.updateCoupon = (couponId, data) => HttpRequestService.put(`packages/coupons/${couponId}`, data);

    // Delete coupon
    methods.packages.deleteCoupon = couponId => HttpRequestService.delete(`packages/coupons/${couponId}`, {});

    methods.packages.createTopUpPackage = data => HttpRequestService.post('package-credits/craete-topup-package', data);

    /**
     * Merchant
     */
    methods.merchant = {};

    // Provision merchant
    methods.merchant.provision = data => HttpRequestService.post('merchant/provision', data);

    // Upload identity document
    methods.merchant.uploadIdDocument = data => HttpRequestService.post('merchant/provision/upload_id_photo', data);

    // Get required verification fields
    methods.merchant.getRequiredVerificationFields = () => getRequest('merchant/provision/verification_fields', {});

    // Provide required verification fields
    methods.merchant.provideRequiredVerificationFields = data => HttpRequestService.put('merchant/provision/verification_fields', data);

    // Update bank account
    methods.merchant.updateBankAccount = data => HttpRequestService.put('merchant/bank_account/update', data);

    // Get bank account
    methods.merchant.getBankAccount = () => getRequest('merchant/bank_account', {});

    // Get balance
    methods.merchant.getBalance = () => getRequest('merchant/balance', {});

    // Get transactions
    methods.merchant.getBalanceTransactions = data => getRequest('merchant/balance/transactions', data);

    // Get transfer
    methods.merchant.getTransfer = transferId => getRequest(`merchant/transfers/${transferId}`, {});

    // Get transfers
    methods.merchant.getTransfers = data => getRequest('merchant/transfers', data);

    // Get transfer settings
    methods.merchant.getTransferSettings = () => getRequest('merchant/transfers/settings', {});

    // Update transfer settings
    methods.merchant.updateTransferSettings = data => HttpRequestService.put('merchant/transfers/settings', data);

    // Update Customer Payment Card
    methods.merchant.updateCustomerPaymentCard = (customerId, data) => HttpRequestService.put(`customers/${customerId}/card/update`, data);

    // Get Charge
    methods.merchant.getCharge = chargeId => getRequest(`merchant/charges/${chargeId}`, {});

    // Retry Charge
    methods.merchant.retryCharge = chargeId => HttpRequestService.post(`merchant/charges/${chargeId}/retry`, {});

    // Resend Client Charge Receipt
    methods.merchant.resendClientChargeReceipt = chargeId => HttpRequestService.post(`merchant/charges/${chargeId}/resend_receipt`, {});

    // Refund Charge
    methods.merchant.refundCharge = (chargeId, data = {}) => HttpRequestService.post(`merchant/charges/${chargeId}/refund`, data);

    // Get Refund
    methods.merchant.getRefund = refundId => getRequest(`merchant/refunds/${refundId}`, {});

    // List Charges
    methods.merchant.getCharges = data => getRequest('merchant/charges', data);

    // List Customer Charges
    methods.merchant.getCustomerCharges = (customerId, data) => getRequest(`customers/${customerId}/charges`, data);

    // List Purchase Invoices/Charges
    methods.merchant.getPurchaseInvoices = (purchaseId, data) => getRequest(`merchant/packages/purchases/${purchaseId}/invoices`, data);

    // List Provisioned Customers
    methods.merchant.getCustomers = data => getRequest('merchant/customers', data);

    // Get Purchase
    methods.merchant.getPurchase = purchaseId => getRequest(`merchant/packages/purchases/${purchaseId}`, {});

    // Cancel Purchase
    methods.merchant.cancelPurchase = (purchaseId, data) => HttpRequestService.post(`merchant/packages/purchases/${purchaseId}/cancel`, data);

    // Gift Sessions on Purchase
    methods.merchant.giftPackageSessions = (purchaseId, data) => HttpRequestService.post(`merchant/packages/purchases/${purchaseId}/gift_sessions`, data);

    // Gift Sessions (not on Purchase)
    methods.merchant.giftSessions = (customerId, data) => HttpRequestService.post(`customers/${customerId}/gift_sessions`, data);

    // Deduct Sessions on Quota
    methods.merchant.deductQuotaSessions = (quotaId, data) => HttpRequestService.post(`merchant/packages/purchases/${quotaId}/deduct_sessions`, data);

    // Get Analytics Overview
    methods.merchant.getAnalyticsOverview = data => getRequest('merchant/analytics/overview', data);

    // Get Analytics Charge Chart Data
    methods.merchant.getAnalyticsChargeChartData = data => getRequest('merchant/analytics/charges/chart', data);

    // Get Analytics Customer Chart Data
    methods.merchant.getAnalyticsCustomerChartData = data => getRequest('merchant/analytics/customers/chart', data);

    /**
     * Forms
     */
    methods.forms = {};

    // Get Single Form
    methods.forms.get = formId => getRequest(`forms/${formId}`, {});

    // Get All Trainer Forms For Given Customer
    methods.forms.getAllForCustomer = customerId => getRequest(`clients/${customerId}/forms`);

    // Notify User Of Custom Form
    methods.forms.notify = (customerId, formId) => getRequest(`forms/${formId}/notify/${customerId}`);

    /**
     * Form Questions
     */
    methods.form_questions = {};

    // Get Form Questions
    methods.form_questions.getAll = (formId, customerId = false) => {
      const data = {};

      if (customerId) {
        data.customerId = customerId;
      }

      return getRequest(`forms/${formId}/questions`, data);
    };

    /**
     * Form Question Answers
     */

    methods.form_question_answers = {};

    // Create A Batch Of Form Question Answers
    methods.form_question_answers.createBatch = data => HttpRequestService.post('forms/questions/answers/create/batch', {
      answers: data
    });

    // Update Form Question Answer
    methods.form_question_answers.updateBatch = data => HttpRequestService.put('forms/questions/answers/update/batch', {
      answers: data
    });

    /**
     * Workout Library
     */
    methods.workoutLibrary = {};

    // Get library groups
    methods.workoutLibrary.getLibraryGroups = () => getRequest('workouts/library/groups', {});

    /**
     * Nutrition Library
     */
    methods.nutritionLibrary = {};

    // Get library groups
    methods.nutritionLibrary.getLibraryGroups = () => getRequest('nutrition/plans/library/groups', {});

    /**
     * In App Purchase
     */
    methods.inAppPurchase = {};

    // Get library groups
    methods.inAppPurchase.verifyReceipt = receipt => HttpRequestService.post('iap/apple/validate_receipt', {
      receipt
    });

    /**
     * Customer Groups
     */
    methods.customerGroups = {};

    // Create Group
    methods.customerGroups.create = (data, forUser = null) => {
      if (forUser) {
        data.trainer_user_id = forUser;
      }

      return HttpRequestService.post('customer_groups', data);
    };

    // Delete Group
    methods.customerGroups.update = (groupId, data) => HttpRequestService.put(`customer_groups/${groupId}`, data);

    // Get Group
    methods.customerGroups.get = (groupId, includeCounts = null) => {
      const data = {};

      if (includeCounts) {
        data.include_counts = includeCounts.join(',');
      }

      return getRequest(`customer_groups/${groupId}`, data);
    };

    // Duplicate Group
    methods.customerGroups.duplicate = (groupId, forUser = false) => {
      const data = {};
      if (forUser) {
        data.trainer_user_id = forUser;
      }

      return HttpRequestService.post(`customer_groups/${groupId}/duplicate`, data);
    };

    // Delete Group
    methods.customerGroups.delete = groupId => HttpRequestService.delete(`customer_groups/${groupId}`, {});

    // List All
    methods.customerGroups.list = (forUser = false, includeCounts = false) => {
      const data = {};
      if (forUser) {
        data.trainer_user_id = forUser;
      }

      if (includeCounts) {
        data.include_counts = includeCounts.join(',');
      }

      return getRequest('customer_groups', data);
    };

    // Get Groups for Customer
    methods.customerGroups.listForCustomer = customerId => getRequest(`customers/${customerId}/customer_groups`, {});

    // Get Groups Containing Workouts for Customer
    methods.customerGroups.listWithWorkoutsForCustomer = customerId => getRequest(`customers/${customerId}/customer_groups/with_workouts`, {});

    // Get Groups Containing Nutrition Plans for Customer
    methods.customerGroups.listWithNutritionPlansForCustomer = customerId => getRequest(`customers/${customerId}/customer_groups/with_nutrition_plans`, {});

    // Assign Customer To Customer Group
    methods.customerGroups.assignCustomer = (groupId, customerId) => HttpRequestService.post(`customer_groups/${groupId}/assign_customer`, {
      customer_id: customerId
    });

    // Unassign Customer From Customer Group
    methods.customerGroups.unassignCustomer = (groupId, customerId) => HttpRequestService.post(`customer_groups/${groupId}/unassign_customer`, {
      customer_id: customerId
    });

    methods.customerGroups.message = (groupId, data) => HttpRequestService.post(`customer_groups/${groupId}/message`, data);

    methods.files = {};

    methods.files.listAll = () => getRequest('files');
    methods.files.update = (fileId, data) => HttpRequestService.put(`files/${fileId}`, data);
    methods.files.delete = fileId => HttpRequestService.delete(`files/${fileId}`);

    /**
     * Hardware
     */
    methods.hardware = {};
    methods.hardware.setup = code => getRequest(`hardware/fitbit/getauthmobile?code=${code}`, {});
    methods.hardware.getAuthKey = () => getRequest('hardware/fitbit/getauthkey');
    methods.hardware.checkAuthentication = customerId => getRequest(`hardware/fitbit/checkauth/${customerId}`, {});
    methods.hardware.clearAuthentication = customerId => getRequest(`hardware/fitbit/clearauth/${customerId}`, {});
    methods.hardware.refreshAuthentication = customerId => getRequest(`hardware/fitbit/refreshauth/${customerId}`, {});
    methods.hardware.checkScopes = customerId => getRequest(`hardware/fitbit/checkscope/${customerId}`, {});

    /**
     * Like
     */
    methods.like = {};

    methods.like.toggle = data => HttpRequestService.post('like', data);

    /**
     * Meal Templates
     */

    methods.mealTemplates = {};
    methods.mealTemplates.listAll = () => getRequest('meal-templates');
    methods.mealTemplates.get = templateId => getRequest(`meal-templates/${templateId}`);
    methods.mealTemplates.create = data => HttpRequestService.post('meal-templates', data);
    methods.mealTemplates.update = (templateId, data) => HttpRequestService.put(`meal-templates/${templateId}`, data);
    methods.mealTemplates.delete = templateId => HttpRequestService.delete(`meal-templates/${templateId}`);

    /**
     * Customer Trainers
     */
    methods.customerTrainers = {};
    methods.customerTrainers.get = () => getRequest('customers-trainers');
    methods.customerTrainers.getInvitations = () => getRequest('customers-invitations');
    methods.customerTrainers.getUnassignedClients = () => getRequest('unassigned-customers');
    methods.customerTrainers.assignTrainerToCustomer = data => HttpRequestService.post('unassigned-customers', data);
    methods.customerTrainers.updateCurrentCustomer = data => HttpRequestService.post('activate-customer', data);
    methods.customerTrainers.updateUserTrainers = data => HttpRequestService.put('customers-trainers', data);
    methods.customerTrainers.accountSwapperChecker = data => HttpRequestService.post('account-swapper-checker', data);

    methods.events = {};

    methods.events.getAll = () => getRequest('calendar-events');
    methods.events.create = event => HttpRequestService.post('calendar-events', event);
    methods.events.get = id => getRequest(`calendar-events/${id}`);
    methods.events.update = (id, data) => HttpRequestService.put(`calendar-events/${id}`, data);
    methods.events.delete = id => HttpRequestService.delete(`calendar-events/${id}`);

    methods.calendarEvents = {};
    methods.calendarEvents.getAllFrom = data => getRequest('calendar-events', data);
    methods.calendarEvents.getGroups = () => getRequest('calendar-events/groups');
    methods.calendarEvents.getGroup = id => getRequest(`calendar-events/groups/${id}`);
    methods.calendarEvents.attachGroup = (id, data) => HttpRequestService.post(`calendar-events/groups/${id}/add`, data);
    methods.calendarEvents.detachGroup = (id, data) => HttpRequestService.post(`calendar-events/groups/${id}/remove`, data);
    methods.calendarEvents.get = (eventID, data) => getRequest(`calendar-events/${eventID}`, data);
    methods.calendarEvents.create = data => HttpRequestService.post('calendar-events', data);
    methods.calendarEvents.duplicate = (eventId, data) => HttpRequestService.post(`calendar-events/${eventId}/duplicate`, data);
    methods.calendarEvents.update = (eventID, data) => HttpRequestService.put(`calendar-events/${eventID}`, data);
    methods.calendarEvents.delete = eventID => HttpRequestService.delete(`calendar-events/${eventID}`, {});

    methods.calendarEventTimes = {};
    methods.calendarEventTimes.getAllFrom = data => getRequest('calendar-event-times', data);
    methods.calendarEventTimes.get = eventTimeID => getRequest(`calendar-event-times/${eventTimeID}`);
    methods.calendarEventTimes.complete = (eventTimeID, data) => HttpRequestService.post(`calendar-event-times/${eventTimeID}/complete`, data);
    methods.calendarEventTimes.addException = (eventTimeID, data) => HttpRequestService.put(`calendar-event-times/${eventTimeID}/update`, data);
    methods.calendarEventTimes.cancel = (eventTimeID, data) => HttpRequestService.put(`calendar-event-times/${eventTimeID}/cancel`, data);
    methods.calendarEventTimes.updateEndDate = (eventTimeID, data) => HttpRequestService.put(`calendar-event-times/${eventTimeID}/cancel-all`, data);
    methods.calendarEventTimes.create = data => HttpRequestService.post('calendar-event-times', data);
    methods.calendarEventTimes.update = (eventTimeID, data) => HttpRequestService.put(`calendar-event-times/${eventTimeID}`, data);
    methods.calendarEventTimes.delete = eventTimeID => HttpRequestService.delete(`calendar-event-times/${eventTimeID}`);

    methods.calendarEventsBookings = {};
    methods.calendarEventsBookings.getAllFrom = data => getRequest('calendar-events', data, 'GET');
    methods.calendarEventsBookings.get = eventID => getRequest(`calendar-events/${eventID}`);
    methods.calendarEventsBookings.getCustomerBookings = (customerId, data) => getRequest(`customers/${customerId}/bookings`, data);
    methods.calendarEventsBookings.create = (eventID, data) => HttpRequestService.post(`calendar-events/${eventID}/book`, data);
    methods.calendarEventsBookings.bookTrainers = (eventID, data) => HttpRequestService.post(`calendar-events/${eventID}/book/trainers`, data);
    methods.calendarEventsBookings.bookWorkouts = (eventID, data) => HttpRequestService.post(`calendar-events/${eventID}/book/workouts`, data);
    methods.calendarEventsBookings.update = (eventID, data) => HttpRequestService.put(`calendar-events/${eventID}`, data);
    methods.calendarEventsBookings.cancel = (eventID, data) => HttpRequestService.put(`calendar-events-bookings/${eventID}/cancel`, data);
    methods.calendarEventsBookings.restore = (eventID, data) => HttpRequestService.put(`calendar-events-bookings/${eventID}/restore`, data);
    methods.calendarEventsBookings.checkIn = (eventID, data) => HttpRequestService.put(`calendar-events-bookings/${eventID}/checkIn`, data);
    methods.calendarEventsBookings.forceBooking = (eventID, data) => HttpRequestService.put(`calendar-events-bookings/${eventID}/force-booking`, data);
    methods.calendarEventsBookings.moveToBookList = (eventID, data) => HttpRequestService.put(`calendar-events-bookings/${eventID}/set-active`, data);
    methods.calendarEventsBookings.delete = eventID => HttpRequestService.delete(`calendar-events/${eventID}`, {});

    methods.packagesCredits = {};
    methods.packagesCredits.getAll = data => getRequest('package-credits', data, 'GET');
    methods.packagesCredits.getHistory = (id, data) => getRequest(`package-credits/${id}/history`, data, 'GET');
    methods.packagesCredits.getTrainers = data => getRequest('package-credits/trainers', data, 'GET');
    methods.packagesCredits.create = data => HttpRequestService.post('package-credits', data, 'POST');
    methods.packagesCredits.addAllowance = (id, data) => HttpRequestService.put(`package-credits/${id}/add-allowance`, data);
    methods.packagesCredits.deductAllowance = (id, data) => HttpRequestService.put(`package-credits/${id}/deduct-allowance`, data);
    methods.packagesCredits.getByCustomersWithCredits = data => getRequest('package-credits/get-customers-with-credits', data);

    methods.permissions = {};
    methods.permissions.getAll = data => getRequest('trainer-permissions', data);
    methods.permissions.updateGlobal = data => HttpRequestService.put('trainer-permissions', data);
    methods.permissions.create = data => HttpRequestService.post('trainer-permissions', data);

    /**
     * Swapper
     */
    methods.swapper = {};
    methods.swapper.changeTrainerAccount = data => HttpRequestService.post('change-trainer-account', data);
    methods.swapper.createSecondaryAccount = data => HttpRequestService.post('create-secondary-trainer-account', data);

    return methods;
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('AuthenticationHelperService', service);

    service.$inject = ['PushNotificationService', 'AuthenticationService', 'NavigateService', 'SqlStorageService', 'ThreeDeeTouchQuickActionsService', '$mdSidenav', 'DayPickerService', 'ScheduleDayPickerService', '$ionicHistory'];

    function service(PushNotificationService, AuthenticationService, NavigateService, SqlStorageService, ThreeDeeTouchQuickActionsService, $mdSidenav, DayPickerService, ScheduleDayPickerService, $ionicHistory) {

        var service = {};

        service.logout = function () {

            PushNotificationService.stop();

            AuthenticationService.reset();

            NavigateService.clearCache();

            NavigateService.clearHistory();

            SqlStorageService.rebuildDb();

            ThreeDeeTouchQuickActionsService.resetActions();

            if ($mdSidenav("left").isOpen()) {
                $mdSidenav('left').close();
            }

            if (DayPickerService.isOpen()) {
                DayPickerService.close(true);
            }

            if (ScheduleDayPickerService.isOpen()) {
                ScheduleDayPickerService.close(true);
            }

            NavigateService.disableBack().disableAnimation().replaceCurrentState().go('app.login').then(() => $ionicHistory.clearCache());
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('AuthenticationService', service);

    service.$inject = ['AuthenticationStateService', '$q', 'HttpRequestService', 'CredStoreService', '$rootScope'];

    function service(AuthenticationStateService, $q, HttpRequestService, CredStoreService, $rootScope) {

        var service = {};

        var fireEventHandlers = function () {

            var queue = [];

            for (var i = 0; i < loginSuccessEventHandlers.length; i++) {
                queue.push(loginSuccessEventHandlers[i]);
            }

            if (queue.length > 0) {

                async.each(queue, function (handler, callback) {
                    $q(handler).finally(callback);
                });
            }
        };

        service.setup = function () {
            var deferredToken = $q.defer();
            var deferredUser = $q.defer();

            CredStoreService.getToken().then(function (storedToken) {
                AuthenticationStateService.setToken(storedToken);

                deferredToken.resolve(storedToken);
            }, deferredToken.reject);

            CredStoreService.getUser().then(function (data) {
                try {
                    AuthenticationStateService.setCurrentUser(JSON.parse(data));

                    deferredUser.resolve(data);
                } catch (e) {
                    AuthenticationStateService.reset();
                    deferredUser.reject();
                }
            }, deferredUser.reject);

            var all = $q.all([deferredToken.promise, deferredUser.promise]);

            all.then(function (responses) {
                if (responses[0] && responses[1]) {
                    fireEventHandlers();
                }
            });

            return all;
        };

        var loginSuccessEventHandlers = [];

        service.registerLoginSuccessEventHandler = function (handler, highPriority) {

            highPriority = highPriority || false;

            if (highPriority) {

                loginSuccessEventHandlers.unshift(handler);
            } else {

                loginSuccessEventHandlers.push(handler);
            }
        };

        service.updateCurrentUser = function (currentUser) {
            AuthenticationStateService.setCurrentUser(currentUser);

            CredStoreService.saveUser(JSON.stringify(currentUser));
        };

        service.passwordLogin = function (username, password, resolver) {

            resolver = resolver || false;

            var deferred = $q.defer();

            HttpRequestService.post('auth/token', {
                username: username,
                password: password
            }).then(function (response) {

                CredStoreService.saveToken(response.data.access_token).then(function () {

                    AuthenticationStateService.setToken(response.data.access_token);

                    HttpRequestService.post('users/check_credentials').then(function (response) {

                        service.updateCurrentUser(response.data.currentUser);
                        if (resolver && typeof resolver === 'function') {

                            var resolve = resolver();
                            var afterResolve = function () {
                                deferred.resolve(response.data.currentUser);
                                fireEventHandlers();
                            };

                            if (resolve) {
                                resolve.then(afterResolve);
                            } else {
                                afterResolve();
                            }
                        } else {

                            deferred.resolve(response.data.currentUser);
                            fireEventHandlers();
                        }
                    }, function (response) {
                        AuthenticationStateService.reset();
                        CredStoreService.reset();
                        deferred.reject(response);
                    });
                }, function (response) {
                    AuthenticationStateService.reset();
                    CredStoreService.reset();
                    deferred.reject(response);
                });
            }, function (response) {
                AuthenticationStateService.reset();
                CredStoreService.reset();
                deferred.reject(response);
            });

            return deferred.promise;
        };

        service.refreshUser = () => {
            const deferred = $q.defer();

            HttpRequestService.post('users/check_credentials').then(response => {
                service.updateCurrentUser(response.data.currentUser);
                deferred.resolve(response.data.currentUser);
            }, response => {
                deferred.reject(response.data);
            });

            return deferred.promise;
        };

        service.reset = function () {
            AuthenticationStateService.reset();
            CredStoreService.reset();
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('AuthenticationStateService', service);

    service.$inject = [];

    function service() {

        var user = null;
        var token = null;

        return {

            getCurrentUser: function () {
                if (user) {
                    return user;
                }

                return false;
            },

            getToken: function () {
                if (token) {
                    return token;
                }

                return false;
            },

            setCurrentUser: function (currentUser) {
                user = currentUser;
            },

            setToken: function (newToken) {
                token = newToken;
            },

            isLoggedIn: function () {
                return user !== null && token !== null;
            },

            reset: function () {
                user = null;
                token = null;
            }

        };
    }
})();
(() => {
  angular.module('MyPTHub').factory('PermissionsService', service);

  service.$inject = ['AuthenticationStateService'];

  function service(AuthenticationStateService) {
    const currentUser = () => AuthenticationStateService.getCurrentUser();

    const service = {};

    const getSubscription = () => {
      const user = currentUser();

      if (!user || !user.subscription) {
        return false;
      }

      return user.subscription;
    };

    service.getCurrentUser = currentUser;

    service.getLoggedInUserId = () => currentUser().id;

    service.getLoggedInUserType = () => {
      if (!currentUser()) {
        return false;
      }

      return currentUser().type;
    };

    service.getCurrentUserEntityId = () => {
      if (service.userIsTrainer()) {
        return currentUser().trainer_id;
      }

      return currentUser().customer_id;
    };

    service.userIsTrainer = () => service.getLoggedInUserType() === 'trainer';

    service.userIsCustomer = () => service.getLoggedInUserType() === 'customer';

    service.userIsTrainerAdmin = () => service.userIsTrainer() && currentUser().is_admin;

    service.userIsTrainerOwner = () => service.userIsTrainer() && currentUser().is_owner;

    service.userHasUnlimitedPlus = () => service.userIsTrainer() && currentUser().subscription.permissions.can_use_unlimited_plus;

    service.userIsTrainerAdminOrOwner = objectTrainerUserId => {
      if (service.userIsTrainerAdmin()) {
        return true;
      }

      if (currentUser().id === objectTrainerUserId) {
        return true;
      }

      return false;
    };

    service.merchantProvisioned = () => {
      const currentUserDetails = currentUser();

      if (!currentUserDetails) {
        return false;
      }

      if (service.userIsTrainer()) {
        return currentUserDetails.merchant_is_provisioned;
      }

      return currentUserDetails.trainer_details.merchant_is_provisioned && currentUserDetails.is_provisioned_as_merchant_customer;
    };

    service.schedulerIsEnabled = () => {
      const subscription = getSubscription();

      if (service.userIsCustomer()) {
        return currentUser().trainer_details.scheduler_enabled;
      }

      return subscription.scheduler_enabled;
    };

    return service;
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('avatar', directive);

    function directive() {
        return {
            restrict: 'A',
            replace: true,
            scope: {
                filename: "=filename",
                size: "@size"
            },
            template: '<img ng-src="{{ vm.url }}" />',
            controller: controller,
            controllerAs: 'vm',
            link: link
        };
    }

    var defaultImage = 'img/empty-avatar.png';

    function link(scope, element, attrs) {

        element.addClass('notLoaded');
        element.addClass('hideUntilLoaded');

        element.bind("load", function (e) {
            element.removeClass('notLoaded');
        });

        element.bind('error', function () {
            attrs.$set('src', defaultImage);
        });
    }

    controller.$inject = ['$scope', 'ImageCacheService', 'mpthconf'];

    function controller($scope, ImageCacheService, mpthconf) {

        var vm = this;

        var fileStoreBase = mpthconf.cdn.base;

        var setUri = function () {
            var avatarUrl;

            if ($scope.filename && $scope.filename.length > 0) {

                avatarUrl = fileStoreBase + 'users/' + ($scope.size === 'large' ? 'l' : 's') + '/' + $scope.filename;

                ImageCacheService.get(avatarUrl).then(function (newUrl) {
                    vm.url = newUrl;
                }, function () {
                    vm.url = defaultImage;
                });
            } else {

                vm.url = defaultImage;
            }
        };

        $scope.$watch('filename', setUri);
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('AvatarUploadService', service);

    service.$inject = ['CameraService', 'ApiClientService', 'AuthenticationService', '$rootScope'];

    function service(CameraService, ApiClientService, AuthenticationService, $rootScope) {

        var service = {};

        var fireEventHandlers = function () {
            $rootScope.$emit('avatarUploadSuccess');
        };

        service.takePhoto = function (userId) {

            return CameraService.takePhoto().then(function (photoData) {
                return ApiClientService.users.setPhoto(userId, photoData).success(function (response) {
                    AuthenticationService.updateCurrentUser(response.currentUser);
                    fireEventHandlers();
                });
            });
        };

        service.browsePhoto = function (userId) {

            return CameraService.browsePhoto().then(function (photoData) {
                return ApiClientService.users.setPhoto(userId, photoData).success(function (response) {
                    AuthenticationService.updateCurrentUser(response.currentUser);
                    fireEventHandlers();
                });
            });
        };

        service.registerUploadSuccessEventHandler = function (scope, callback) {

            var unsub = $rootScope.$on('avatarUploadSuccess', callback);

            scope.$on('$destroy', unsub);

            return unsub;
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('Base64Service', service);

    function service() {

        var keyStr = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';

        return {
            encode: function (input) {
                var output = "";
                var chr1,
                    chr2,
                    chr3 = "";
                var enc1,
                    enc2,
                    enc3,
                    enc4 = "";
                var i = 0;

                do {
                    chr1 = input.charCodeAt(i++);
                    chr2 = input.charCodeAt(i++);
                    chr3 = input.charCodeAt(i++);

                    enc1 = chr1 >> 2;
                    enc2 = (chr1 & 3) << 4 | chr2 >> 4;
                    enc3 = (chr2 & 15) << 2 | chr3 >> 6;
                    enc4 = chr3 & 63;

                    if (isNaN(chr2)) {
                        enc3 = enc4 = 64;
                    } else if (isNaN(chr3)) {
                        enc4 = 64;
                    }

                    output = output + keyStr.charAt(enc1) + keyStr.charAt(enc2) + keyStr.charAt(enc3) + keyStr.charAt(enc4);
                    chr1 = chr2 = chr3 = "";
                    enc1 = enc2 = enc3 = enc4 = "";
                } while (i < input.length);

                return output;
            },

            decode: function (input) {
                var output = "";
                var chr1,
                    chr2,
                    chr3 = "";
                var enc1,
                    enc2,
                    enc3,
                    enc4 = "";
                var i = 0;

                // remove all characters that are not A-Z, a-z, 0-9, +, /, or =
                var base64test = /[^A-Za-z0-9\+\/\=]/g;
                if (base64test.exec(input)) {
                    /*window.alert("There were invalid base64 characters in the input text.\n" +
                        "Valid base64 characters are A-Z, a-z, 0-9, '+', '/',and '='\n" +
                        "Expect errors in decoding.");*/
                }
                input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");

                do {
                    enc1 = keyStr.indexOf(input.charAt(i++));
                    enc2 = keyStr.indexOf(input.charAt(i++));
                    enc3 = keyStr.indexOf(input.charAt(i++));
                    enc4 = keyStr.indexOf(input.charAt(i++));

                    chr1 = enc1 << 2 | enc2 >> 4;
                    chr2 = (enc2 & 15) << 4 | enc3 >> 2;
                    chr3 = (enc3 & 3) << 6 | enc4;

                    output = output + String.fromCharCode(chr1);

                    if (enc3 != 64) {
                        output = output + String.fromCharCode(chr2);
                    }
                    if (enc4 != 64) {
                        output = output + String.fromCharCode(chr3);
                    }

                    chr1 = chr2 = chr3 = "";
                    enc1 = enc2 = enc3 = enc4 = "";
                } while (i < input.length);

                return output;
            }
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('BeepService', service);

    service.$inject = ['DeviceInfoService'];

    function service(DeviceInfoService) {

        return {
            beep: function (times) {

                if (DeviceInfoService.isBrowser()) {
                    return false;
                }

                navigator.notification.beep(times);
            }
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('BrandingMenuService', service);

    service.$inject = ['$ionicActionSheet', 'BrandingService', 'ApiClientService', 'AuthenticationService', 'IosIconService', 'PermissionsService'];

    function service($ionicActionSheet, BrandingService, ApiClientService, AuthenticationService, IosIconService, PermissionsService) {

        var service = {};

        var refreshingBranding = false;
        function refreshBranding() {
            if (refreshingBranding) {
                return true;
            }

            refreshingBranding = true;

            var currentUser = PermissionsService.getCurrentUser();

            var getEntity;
            if (PermissionsService.userIsTrainer()) {
                getEntity = ApiClientService.trainerUsers.get(currentUser.id);
            } else {
                getEntity = ApiClientService.customers.get(currentUser.customer_id);
            }

            getEntity.then(function (response) {
                AuthenticationService.updateCurrentUser(response.currentUser);
                BrandingService.clearCache();
                BrandingService.applyStyles();
            }).finally(function () {
                refreshingBranding = false;
            });

            return true;
        }

        function useCustomAppIcon() {
            IosIconService.useAvailableBrandedIcon();
            return true;
        }

        function resetAppIcon() {
            IosIconService.resetIcon(true);
            return true;
        }

        service.start = function () {

            var buttons = [];

            buttons.push({
                text: 'Sync. Branding',
                action: refreshBranding
            });

            if (IosIconService.canChangeIcons()) {
                if (IosIconService.usingCustomIcon()) {
                    buttons.push({
                        text: 'Reset App Icon',
                        action: resetAppIcon
                    });
                } else {
                    buttons.push({
                        text: 'Use Custom App Icon',
                        action: useCustomAppIcon
                    });
                }
            }

            $ionicActionSheet.show({
                buttons: buttons,
                titleText: 'Branding',
                cancelText: 'Cancel',
                cancel: function () {
                    // cancel
                },
                buttonClicked: function (index) {
                    if (typeof buttons[index] !== 'undefined' && typeof buttons[index].action === 'function') {
                        buttons[index].action(index);
                    }
                    return true;
                }
            });
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('BrandingService', service);

    service.$inject = ['AuthenticationStateService', '$q', '$cordovaStatusbar', 'baseBrand', 'ApiClientService', 'LogoService', 'LocalStorageService', 'IosIconService'];

    function service(AuthenticationStateService, $q, $cordovaStatusbar, baseBrand, ApiClientService, LogoService, LocalStorageService, IosIconService) {

        var sass;

        var service = {};

        var baseBrandCached = baseBrand;
        var branding = baseBrand ? baseBrand : null;

        var updateBrandingData = function () {

            var currentUser = AuthenticationStateService.getCurrentUser();

            if (currentUser && currentUser.branding) {
                branding = currentUser.branding;
            }
        };

        var availableColors = ['#ffffff', '#2b3e51', '#000000'];

        function mostReadable(color) {

            var bestColor = null;
            var bestScore = 0;
            var readability;

            for (var i = 0; i < availableColors.length; i++) {
                readability = Math.ceil(tinycolor.readability(color, availableColors[i]));

                if (readability > bestScore) {
                    bestScore = readability;
                    bestColor = availableColors[i];
                }
            }

            return bestColor;
        }

        function brandReadableWhiteBg(color) {
            var brandReadability = tinycolor.readability('#FFFFFF', color);

            if (brandReadability > 1.5) {
                return color;
            }

            return '#969696';
        }

        service.forceBranding = function (newBranding) {
            branding = newBranding;
        };

        var getWorker = function () {
            if (sass) {
                return sass;
            }

            sass = new Sass('lib/sass.js/dist/sass.worker.js');

            return sass;
        };

        var loadSass = function () {

            return $q(function (resolve) {
                getWorker().preloadFiles('../../../sass/', '', ['branding.scss', 'buttons.scss', 'empty-area.scss', 'forms.scss', 'general-styles.scss', 'hub.scss', 'lists.scss', 'loader.scss', 'main-menu.scss', 'material-ui-base.scss', 'messages.scss', 'modals.scss', 'profile.scss', 'schedule.scss', 'splash.scss', 'stopwatch.scss', 'sync-manager.scss', 'tabs.scss', 'workouts.scss', 'style.scss', 'login.scss', 'packages.scss'], resolve);
            });
        };

        var compileSass = function () {
            var deferred = $q.defer();

            var branding_primary_readable = mostReadable(branding.branding_primary);
            var branding_primary_readable_white_bg = brandReadableWhiteBg(branding.branding_primary);
            var branding_navbar_readable = mostReadable(branding.branding_navbar);

            loadSass().then(function () {

                var worker = getWorker();

                worker.writeFile('branding.scss', "$brand: " + branding.branding_primary + "; " + "$brandContrast: " + branding_primary_readable + "; " + "$brandWhiteBgReadable: " + branding_primary_readable_white_bg + "; " + "$navBar: " + branding.branding_navbar + "; " + "$navBarContrast: " + branding_navbar_readable + ";", function () {

                    var imports = '@import "buttons.scss";' + '@import "empty-area.scss";' + '@import "forms.scss";' + '@import "general-styles.scss";' + '@import "hub.scss";' + '@import "lists.scss";' + '@import "loader.scss";' + '@import "main-menu.scss";' + '@import "material-ui-base.scss";' + '@import "messages.scss";' + '@import "modals.scss";' + '@import "profile.scss";' + '@import "schedule.scss";' + '@import "splash.scss";' + '@import "stopwatch.scss";' + '@import "sync-manager.scss";' + '@import "tabs.scss";' + '@import "login.scss";' + '@import "packages.scss";' + '@import "workouts.scss";';

                    worker.compile(imports, function (result) {

                        jQuery('#customBrandStyles').replaceWith(jQuery('<style id="customBrandStyles">' + result.text + '</style>'));

                        if (window.StatusBar) {
                            var statusBarStyle = tinycolor(branding.branding_navbar).isLight() ? 0 : 1;
                            $cordovaStatusbar.style(statusBarStyle);
                            console.log('status bar style', statusBarStyle);
                            LocalStorageService.setKey('renderedBrandingSylesStatusBarStyle', statusBarStyle);
                        }

                        jQuery('html').addClass('app-init-style-ready');

                        LocalStorageService.setKey('renderedBrandingSyles', result.text);
                        LocalStorageService.setKey('renderedBrandingNavColour', branding.branding_navbar);
                        LocalStorageService.setKey('renderedBrandingBrandColour', branding.branding_primary);

                        deferred.resolve();
                    });
                });
            });

            return deferred.promise;
        };

        var getCachedStyles = function () {
            var cached = LocalStorageService.getKey('renderedBrandingSyles');
            var cachedPrimary = LocalStorageService.getKey('renderedBrandingBrandColour');
            var cachedNavbar = LocalStorageService.getKey('renderedBrandingNavColour');

            if (cached && branding.branding_primary === cachedPrimary && branding.branding_navbar === cachedNavbar) {
                return cached;
            }

            return false;
        };

        var getCachedStatusBarStyle = function () {
            return parseInt(LocalStorageService.getKey('renderedBrandingSylesStatusBarStyle'));
        };

        var renderSass = function () {

            var cachedStyles = getCachedStyles();

            if (cachedStyles) {

                console.log('detected cached style - using');

                jQuery('#customBrandStyles').replaceWith(jQuery('<style id="customBrandStyles">' + cachedStyles + '</style>'));

                if (window.StatusBar) {
                    console.log('status bar style', getCachedStatusBarStyle());
                    $cordovaStatusbar.style(getCachedStatusBarStyle());
                }

                jQuery('html').addClass('app-init-style-ready');

                return $q.when();
            }

            return compileSass();
        };

        service.applyStyles = function (init) {

            init = init || false;

            updateBrandingData();

            if (!branding) {
                jQuery('html').addClass('app-init-style-ready');
                return $q.reject();
            }

            if (service.isCustomApp()) {

                var deferred = $q.defer();

                ApiClientService.trainers.getTrainerBranding(baseBrand.trainer_id).then(function (response) {

                    baseBrandCached = response.branding;
                    branding = response.branding;

                    if (init) {
                        LogoService.setLogo(branding.logo_url);
                    }

                    renderSass().then(function () {
                        deferred.resolve();
                    });
                }, function () {

                    renderSass().then(function () {
                        deferred.resolve();
                    });
                });

                return deferred.promise;
            } else {
                LogoService.setLogo(branding.logo_url);
                IosIconService.updateBrandedIcon(branding.custom_app_live);
                return renderSass();
            }
        };

        service.brandingEnabled = function () {
            updateBrandingData();
            return branding && branding.branding_enabled;
        };

        service.clearCache = function () {
            LocalStorageService.removeKey('renderedBrandingSyles');
            LocalStorageService.removeKey('renderedBrandingSylesStatusBarStyle');
            LocalStorageService.removeKey('renderedBrandingNavColour');
            LocalStorageService.removeKey('renderedBrandingBrandColour');
        };

        service.clearBranding = function () {

            branding = baseBrandCached;

            jQuery('#customBrandStyles').replaceWith(jQuery('<style id="customBrandStyles"></style>'));

            jQuery('body').removeClass('app-init-style-ready');

            service.clearCache();
        };

        service.isCustomApp = function () {
            return baseBrand ? true : false;
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('IosIconService', service);

    service.$inject = ['LocalStorageService', 'AuthenticationStateService', 'DeviceInfoService', 'AvailableiOSIcons'];

    function service(LocalStorageService, AuthenticationStateService, DeviceInfoService, AvailableiOSIcons) {

        var enabled = false;

        var service = {};

        function changeIcon(iconNumber) {
            if (!enabled) {
                return false;
            }

            var latestIconNumber = currentIcon();
            var current = parseInt(latestIconNumber || 0);
            console.log('current', current);
            console.log('iconNumber', iconNumber);
            if (current === iconNumber) {
                console.log("Icon not changed");
                return false;
            }

            storeCurrentIcon(iconNumber);

            AppIconChanger.changeIcon({
                iconName: "trainer-" + iconNumber,
                suppressUserNotification: false
            }, function () {
                console.log("Icon set OK: ", iconNumber);
            }, function (err) {
                console.log('could not change icon:', err);
            });
        }

        function resetIcon() {
            changeIcon(0);
        }

        function currentIcon() {
            return LocalStorageService.getKey('currentIcon');
        }

        function storeCurrentIcon(iconNumber) {
            LocalStorageService.setKey('currentIcon', iconNumber);
        }

        function manualIconOverrideActive() {
            return LocalStorageService.getKey('manualIconOverrideActive');
        }

        function setManualIconOverrideActive(iconNumber) {
            return LocalStorageService.setKey('manualIconOverrideActive', iconNumber);
        }

        function unsetManualIconOverrideActive() {
            return LocalStorageService.removeKey('manualIconOverrideActive');
        }

        function customIconIsAvailable(trainerId) {
            return AvailableiOSIcons !== null && AvailableiOSIcons.hasOwnProperty(trainerId);
        }

        service.setup = function () {
            if (typeof window.AppIconChanger === 'undefined') {
                return false;
            }

            if (!DeviceInfoService.isIos()) {
                return false;
            }

            if (typeof AvailableiOSIcons !== 'object') {
                return false;
            }

            AppIconChanger.isSupported(function (supported) {
                enabled = supported;
            });
        };

        service.updateBrandedIcon = function (brandedIconEnabled) {
            if (!enabled) {
                return false;
            }

            if (!brandedIconEnabled) {
                resetIcon();
                return false;
            }

            var currentUser = AuthenticationStateService.getCurrentUser();

            var trainerId = currentUser.trainer_id;

            if (!customIconIsAvailable(trainerId)) {
                console.log('Did not attempt to set app icon as it is not compiled in this bundle.');
                resetIcon();
                return false;
            }

            var override = parseInt(manualIconOverrideActive());

            if (override === 0 || override === trainerId) {
                console.log('Did not change icon due to manual override.');
                return false;
            }

            changeIcon(trainerId);
        };

        service.resetIcon = function (manual) {
            manual = manual || false;
            if (manual) {
                setManualIconOverrideActive(0);
            } else {
                unsetManualIconOverrideActive();
            }
            resetIcon();
        };

        service.useAvailableBrandedIcon = function () {
            var currentUser = AuthenticationStateService.getCurrentUser();
            var trainerId = currentUser.trainer_id;

            setManualIconOverrideActive(trainerId);

            changeIcon(trainerId);
        };

        service.canChangeIcons = function () {
            if (!enabled) {
                return false;
            }

            var currentUser = AuthenticationStateService.getCurrentUser();
            var trainerId = currentUser.trainer_id;

            return customIconIsAvailable(trainerId);
        };

        service.usingCustomIcon = function () {
            var latestIconNumber = currentIcon();
            var current = parseInt(latestIconNumber || 0);
            return current !== 0;
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('LogoService', service);

    service.$inject = ['$rootScope'];

    function service($rootScope) {

        var logo = 'img/logo-white.png';
        var customLogoIsSet = false;

        var service = {};

        var fireEventHandlers = function (newUrl) {
            $rootScope.$emit('brandingLogoUpdated', newUrl);
        };

        service.registerLogoChangedHandler = function (scope, handler) {

            var unsub = $rootScope.$on('brandingLogoUpdated', handler);

            scope.$on('$destroy', unsub);

            return unsub;
        };

        service.customLogoIsSet = function () {
            return customLogoIsSet;
        };

        service.setLogo = function (newLogo) {
            logo = newLogo;
            customLogoIsSet = true;
            fireEventHandlers(logo);
            return logo;
        };

        service.getLogo = function () {
            return logo;
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('CameraService', service);

    service.$inject = ['$cordovaCamera'];

    function service($cordovaCamera) {

        var service = {};

        service.takePhoto = function () {

            var options = {
                quality: 100,
                destinationType: Camera.DestinationType.DATA_URL,
                sourceType: Camera.PictureSourceType.CAMERA,
                allowEdit: true,
                encodingType: Camera.EncodingType.JPEG,
                saveToPhotoAlbum: false,
                correctOrientation: true
            };

            return $cordovaCamera.getPicture(options);
        };

        service.browsePhoto = function () {

            var options = {
                quality: 100,
                destinationType: Camera.DestinationType.DATA_URL,
                sourceType: Camera.PictureSourceType.PHOTOLIBRARY,
                allowEdit: true,
                encodingType: Camera.EncodingType.JPEG,
                saveToPhotoAlbum: false,
                correctOrientation: true
            };

            return $cordovaCamera.getPicture(options);
        };

        return service;
    }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').directive('mpthCountryPicker', directive).factory('mpthCountryPickerService', service);

  directive.$inject = ['mpthCountryPickerService'];

  function directive(mpthCountryPickerService) {
    const newTemplate = '<md-select ng-model="countryValue" ng-required="required">' + '<md-option ng-repeat="country in countries" value="{{country.code}}">' + '{{ country.name }}' + '</md-option>' + '</md-select>';
    const oldTemplate = '<select ng-model="countryValue" ng-required="required" name="country" ng-options="country.code as country.name for country in countries"></select>';

    return {
      template: function (elem, attrs) {
        return attrs.newStyle ? newTemplate : oldTemplate;
      },
      restrict: 'E',
      replace: true,
      scope: {
        countryValue: '=',
        required: '=?',
        newStyle: '@'
      },
      link: function (scope, elem, attrs) {
        scope.countries = mpthCountryPickerService.countries();
        scope.newStyle = attrs.newStyle;
      }
    };
  }

  service.$inject = [];

  function service() {

    var countries = [{
      'name': 'Afghanistan',
      'code': 'AF',
      'timezones': [{
        'name': 'Asia\/Kabul',
        'displayName': 'Asia\/Kabul'
      }]
    }, {
      'name': 'Aland Islands',
      'code': 'AX',
      'timezones': [{
        'name': 'Europe\/Mariehamn',
        'displayName': 'Europe\/Mariehamn'
      }]
    }, {
      'name': 'Albania',
      'code': 'AL',
      'timezones': [{
        'name': 'Europe\/Tirane',
        'displayName': 'Europe\/Tirane'
      }]
    }, {
      'name': 'Algeria',
      'code': 'DZ',
      'timezones': [{
        'name': 'Africa\/Algiers',
        'displayName': 'Africa\/Algiers'
      }]
    }, {
      'name': 'American Samoa',
      'code': 'AS',
      'timezones': [{
        'name': 'Pacific\/Pago_Pago',
        'displayName': 'Pacific\/Pago Pago'
      }]
    }, {
      'name': 'Andorra',
      'code': 'AD',
      'timezones': [{
        'name': 'Europe\/Andorra',
        'displayName': 'Europe\/Andorra'
      }]
    }, {
      'name': 'Angola',
      'code': 'AO',
      'timezones': [{
        'name': 'Africa\/Luanda',
        'displayName': 'Africa\/Luanda'
      }]
    }, {
      'name': 'Anguilla',
      'code': 'AI',
      'timezones': [{
        'name': 'America\/Anguilla',
        'displayName': 'America\/Anguilla'
      }]
    }, {
      'name': 'Antarctica',
      'code': 'AQ',
      'timezones': [{
        'name': 'Antarctica\/Casey',
        'displayName': 'Antarctica\/Casey'
      }, {
        'name': 'Antarctica\/Davis',
        'displayName': 'Antarctica\/Davis'
      }, {
        'name': 'Antarctica\/DumontDUrville',
        'displayName': 'Antarctica\/DumontDUrville'
      }, {
        'name': 'Antarctica\/Mawson',
        'displayName': 'Antarctica\/Mawson'
      }, {
        'name': 'Antarctica\/McMurdo',
        'displayName': 'Antarctica\/McMurdo'
      }, {
        'name': 'Antarctica\/Palmer',
        'displayName': 'Antarctica\/Palmer'
      }, {
        'name': 'Antarctica\/Rothera',
        'displayName': 'Antarctica\/Rothera'
      }, {
        'name': 'Antarctica\/Syowa',
        'displayName': 'Antarctica\/Syowa'
      }, {
        'name': 'Antarctica\/Troll',
        'displayName': 'Antarctica\/Troll'
      }, {
        'name': 'Antarctica\/Vostok',
        'displayName': 'Antarctica\/Vostok'
      }]
    }, {
      'name': 'Antigua and Barbuda',
      'code': 'AG',
      'timezones': [{
        'name': 'America\/Antigua',
        'displayName': 'America\/Antigua'
      }]
    }, {
      'name': 'Argentina',
      'code': 'AR',
      'timezones': [{
        'name': 'America\/Argentina\/Buenos_Aires',
        'displayName': 'America\/Argentina\/Buenos Aires'
      }, {
        'name': 'America\/Argentina\/Catamarca',
        'displayName': 'America\/Argentina\/Catamarca'
      }, {
        'name': 'America\/Argentina\/Cordoba',
        'displayName': 'America\/Argentina\/Cordoba'
      }, {
        'name': 'America\/Argentina\/Jujuy',
        'displayName': 'America\/Argentina\/Jujuy'
      }, {
        'name': 'America\/Argentina\/La_Rioja',
        'displayName': 'America\/Argentina\/La Rioja'
      }, {
        'name': 'America\/Argentina\/Mendoza',
        'displayName': 'America\/Argentina\/Mendoza'
      }, {
        'name': 'America\/Argentina\/Rio_Gallegos',
        'displayName': 'America\/Argentina\/Rio Gallegos'
      }, {
        'name': 'America\/Argentina\/Salta',
        'displayName': 'America\/Argentina\/Salta'
      }, {
        'name': 'America\/Argentina\/San_Juan',
        'displayName': 'America\/Argentina\/San Juan'
      }, {
        'name': 'America\/Argentina\/San_Luis',
        'displayName': 'America\/Argentina\/San Luis'
      }, {
        'name': 'America\/Argentina\/Tucuman',
        'displayName': 'America\/Argentina\/Tucuman'
      }, {
        'name': 'America\/Argentina\/Ushuaia',
        'displayName': 'America\/Argentina\/Ushuaia'
      }]
    }, {
      'name': 'Armenia',
      'code': 'AM',
      'timezones': [{
        'name': 'Asia\/Yerevan',
        'displayName': 'Asia\/Yerevan'
      }]
    }, {
      'name': 'Aruba',
      'code': 'AW',
      'timezones': [{
        'name': 'America\/Aruba',
        'displayName': 'America\/Aruba'
      }]
    }, {
      'name': 'Australia',
      'code': 'AU',
      'timezones': [{
        'name': 'Antarctica\/Macquarie',
        'displayName': 'Antarctica\/Macquarie'
      }, {
        'name': 'Australia\/Adelaide',
        'displayName': 'Australia\/Adelaide'
      }, {
        'name': 'Australia\/Brisbane',
        'displayName': 'Australia\/Brisbane'
      }, {
        'name': 'Australia\/Broken_Hill',
        'displayName': 'Australia\/Broken Hill'
      }, {
        'name': 'Australia\/Currie',
        'displayName': 'Australia\/Currie'
      }, {
        'name': 'Australia\/Darwin',
        'displayName': 'Australia\/Darwin'
      }, {
        'name': 'Australia\/Eucla',
        'displayName': 'Australia\/Eucla'
      }, {
        'name': 'Australia\/Hobart',
        'displayName': 'Australia\/Hobart'
      }, {
        'name': 'Australia\/Lindeman',
        'displayName': 'Australia\/Lindeman'
      }, {
        'name': 'Australia\/Lord_Howe',
        'displayName': 'Australia\/Lord Howe'
      }, {
        'name': 'Australia\/Melbourne',
        'displayName': 'Australia\/Melbourne'
      }, {
        'name': 'Australia\/Perth',
        'displayName': 'Australia\/Perth'
      }, {
        'name': 'Australia\/Sydney',
        'displayName': 'Australia\/Sydney'
      }]
    }, {
      'name': 'Austria',
      'code': 'AT',
      'timezones': [{
        'name': 'Europe\/Vienna',
        'displayName': 'Europe\/Vienna'
      }]
    }, {
      'name': 'Azerbaijan',
      'code': 'AZ',
      'timezones': [{
        'name': 'Asia\/Baku',
        'displayName': 'Asia\/Baku'
      }]
    }, {
      'name': 'Bahamas',
      'code': 'BS',
      'timezones': [{
        'name': 'America\/Nassau',
        'displayName': 'America\/Nassau'
      }]
    }, {
      'name': 'Bahrain',
      'code': 'BH',
      'timezones': [{
        'name': 'Asia\/Bahrain',
        'displayName': 'Asia\/Bahrain'
      }]
    }, {
      'name': 'Bangladesh',
      'code': 'BD',
      'timezones': [{
        'name': 'Asia\/Dhaka',
        'displayName': 'Asia\/Dhaka'
      }]
    }, {
      'name': 'Barbados',
      'code': 'BB',
      'timezones': [{
        'name': 'America\/Barbados',
        'displayName': 'America\/Barbados'
      }]
    }, {
      'name': 'Belarus',
      'code': 'BY',
      'timezones': [{
        'name': 'Europe\/Minsk',
        'displayName': 'Europe\/Minsk'
      }]
    }, {
      'name': 'Belgium',
      'code': 'BE',
      'timezones': [{
        'name': 'Europe\/Brussels',
        'displayName': 'Europe\/Brussels'
      }]
    }, {
      'name': 'Belize',
      'code': 'BZ',
      'timezones': [{
        'name': 'America\/Belize',
        'displayName': 'America\/Belize'
      }]
    }, {
      'name': 'Benin',
      'code': 'BJ',
      'timezones': [{
        'name': 'Africa\/Porto-Novo',
        'displayName': 'Africa\/Porto-Novo'
      }]
    }, {
      'name': 'Bermuda',
      'code': 'BM',
      'timezones': [{
        'name': 'Atlantic\/Bermuda',
        'displayName': 'Atlantic\/Bermuda'
      }]
    }, {
      'name': 'Bhutan',
      'code': 'BT',
      'timezones': [{
        'name': 'Asia\/Thimphu',
        'displayName': 'Asia\/Thimphu'
      }]
    }, {
      'name': 'Bolivia',
      'code': 'BO',
      'timezones': [{
        'name': 'America\/La_Paz',
        'displayName': 'America\/La Paz'
      }]
    }, {
      'name': 'Bonaire, Saint Eustatius and Saba ',
      'code': 'BQ',
      'timezones': [{
        'name': 'America\/Kralendijk',
        'displayName': 'America\/Kralendijk'
      }]
    }, {
      'name': 'Bosnia and Herzegovina',
      'code': 'BA',
      'timezones': [{
        'name': 'Europe\/Sarajevo',
        'displayName': 'Europe\/Sarajevo'
      }]
    }, {
      'name': 'Botswana',
      'code': 'BW',
      'timezones': [{
        'name': 'Africa\/Gaborone',
        'displayName': 'Africa\/Gaborone'
      }]
    }, {
      'name': 'Bouvet Island',
      'code': 'BV',
      'timezones': []
    }, {
      'name': 'Brazil',
      'code': 'BR',
      'timezones': [{
        'name': 'America\/Araguaina',
        'displayName': 'America\/Araguaina'
      }, {
        'name': 'America\/Bahia',
        'displayName': 'America\/Bahia'
      }, {
        'name': 'America\/Belem',
        'displayName': 'America\/Belem'
      }, {
        'name': 'America\/Boa_Vista',
        'displayName': 'America\/Boa Vista'
      }, {
        'name': 'America\/Campo_Grande',
        'displayName': 'America\/Campo Grande'
      }, {
        'name': 'America\/Cuiaba',
        'displayName': 'America\/Cuiaba'
      }, {
        'name': 'America\/Eirunepe',
        'displayName': 'America\/Eirunepe'
      }, {
        'name': 'America\/Fortaleza',
        'displayName': 'America\/Fortaleza'
      }, {
        'name': 'America\/Maceio',
        'displayName': 'America\/Maceio'
      }, {
        'name': 'America\/Manaus',
        'displayName': 'America\/Manaus'
      }, {
        'name': 'America\/Noronha',
        'displayName': 'America\/Noronha'
      }, {
        'name': 'America\/Porto_Velho',
        'displayName': 'America\/Porto Velho'
      }, {
        'name': 'America\/Recife',
        'displayName': 'America\/Recife'
      }, {
        'name': 'America\/Rio_Branco',
        'displayName': 'America\/Rio Branco'
      }, {
        'name': 'America\/Santarem',
        'displayName': 'America\/Santarem'
      }, {
        'name': 'America\/Sao_Paulo',
        'displayName': 'America\/Sao Paulo'
      }]
    }, {
      'name': 'British Indian Ocean Territory',
      'code': 'IO',
      'timezones': [{
        'name': 'Indian\/Chagos',
        'displayName': 'Indian\/Chagos'
      }]
    }, {
      'name': 'British Virgin Islands',
      'code': 'VG',
      'timezones': [{
        'name': 'America\/Tortola',
        'displayName': 'America\/Tortola'
      }]
    }, {
      'name': 'Brunei',
      'code': 'BN',
      'timezones': [{
        'name': 'Asia\/Brunei',
        'displayName': 'Asia\/Brunei'
      }]
    }, {
      'name': 'Bulgaria',
      'code': 'BG',
      'timezones': [{
        'name': 'Europe\/Sofia',
        'displayName': 'Europe\/Sofia'
      }]
    }, {
      'name': 'Burkina Faso',
      'code': 'BF',
      'timezones': [{
        'name': 'Africa\/Ouagadougou',
        'displayName': 'Africa\/Ouagadougou'
      }]
    }, {
      'name': 'Burundi',
      'code': 'BI',
      'timezones': [{
        'name': 'Africa\/Bujumbura',
        'displayName': 'Africa\/Bujumbura'
      }]
    }, {
      'name': 'Cambodia',
      'code': 'KH',
      'timezones': [{
        'name': 'Asia\/Phnom_Penh',
        'displayName': 'Asia\/Phnom Penh'
      }]
    }, {
      'name': 'Cameroon',
      'code': 'CM',
      'timezones': [{
        'name': 'Africa\/Douala',
        'displayName': 'Africa\/Douala'
      }]
    }, {
      'name': 'Canada',
      'code': 'CA',
      'timezones': [{
        'name': 'America\/Atikokan',
        'displayName': 'America\/Atikokan'
      }, {
        'name': 'America\/Blanc-Sablon',
        'displayName': 'America\/Blanc-Sablon'
      }, {
        'name': 'America\/Cambridge_Bay',
        'displayName': 'America\/Cambridge Bay'
      }, {
        'name': 'America\/Creston',
        'displayName': 'America\/Creston'
      }, {
        'name': 'America\/Dawson',
        'displayName': 'America\/Dawson'
      }, {
        'name': 'America\/Dawson_Creek',
        'displayName': 'America\/Dawson Creek'
      }, {
        'name': 'America\/Edmonton',
        'displayName': 'America\/Edmonton'
      }, {
        'name': 'America\/Glace_Bay',
        'displayName': 'America\/Glace Bay'
      }, {
        'name': 'America\/Goose_Bay',
        'displayName': 'America\/Goose Bay'
      }, {
        'name': 'America\/Halifax',
        'displayName': 'America\/Halifax'
      }, {
        'name': 'America\/Inuvik',
        'displayName': 'America\/Inuvik'
      }, {
        'name': 'America\/Iqaluit',
        'displayName': 'America\/Iqaluit'
      }, {
        'name': 'America\/Moncton',
        'displayName': 'America\/Moncton'
      }, {
        'name': 'America\/Nipigon',
        'displayName': 'America\/Nipigon'
      }, {
        'name': 'America\/Pangnirtung',
        'displayName': 'America\/Pangnirtung'
      }, {
        'name': 'America\/Rainy_River',
        'displayName': 'America\/Rainy River'
      }, {
        'name': 'America\/Rankin_Inlet',
        'displayName': 'America\/Rankin Inlet'
      }, {
        'name': 'America\/Regina',
        'displayName': 'America\/Regina'
      }, {
        'name': 'America\/Resolute',
        'displayName': 'America\/Resolute'
      }, {
        'name': 'America\/St_Johns',
        'displayName': 'America\/St Johns'
      }, {
        'name': 'America\/Swift_Current',
        'displayName': 'America\/Swift Current'
      }, {
        'name': 'America\/Thunder_Bay',
        'displayName': 'America\/Thunder Bay'
      }, {
        'name': 'America\/Toronto',
        'displayName': 'America\/Toronto'
      }, {
        'name': 'America\/Vancouver',
        'displayName': 'America\/Vancouver'
      }, {
        'name': 'America\/Whitehorse',
        'displayName': 'America\/Whitehorse'
      }, {
        'name': 'America\/Winnipeg',
        'displayName': 'America\/Winnipeg'
      }, {
        'name': 'America\/Yellowknife',
        'displayName': 'America\/Yellowknife'
      }]
    }, {
      'name': 'Cape Verde',
      'code': 'CV',
      'timezones': [{
        'name': 'Atlantic\/Cape_Verde',
        'displayName': 'Atlantic\/Cape Verde'
      }]
    }, {
      'name': 'Cayman Islands',
      'code': 'KY',
      'timezones': [{
        'name': 'America\/Cayman',
        'displayName': 'America\/Cayman'
      }]
    }, {
      'name': 'Central African Republic',
      'code': 'CF',
      'timezones': [{
        'name': 'Africa\/Bangui',
        'displayName': 'Africa\/Bangui'
      }]
    }, {
      'name': 'Chad',
      'code': 'TD',
      'timezones': [{
        'name': 'Africa\/Ndjamena',
        'displayName': 'Africa\/Ndjamena'
      }]
    }, {
      'name': 'Chile',
      'code': 'CL',
      'timezones': [{
        'name': 'America\/Santiago',
        'displayName': 'America\/Santiago'
      }, {
        'name': 'Pacific\/Easter',
        'displayName': 'Pacific\/Easter'
      }]
    }, {
      'name': 'China',
      'code': 'CN',
      'timezones': [{
        'name': 'Asia\/Shanghai',
        'displayName': 'Asia\/Shanghai'
      }, {
        'name': 'Asia\/Urumqi',
        'displayName': 'Asia\/Urumqi'
      }]
    }, {
      'name': 'Christmas Island',
      'code': 'CX',
      'timezones': [{
        'name': 'Indian\/Christmas',
        'displayName': 'Indian\/Christmas'
      }]
    }, {
      'name': 'Cocos Islands',
      'code': 'CC',
      'timezones': [{
        'name': 'Indian\/Cocos',
        'displayName': 'Indian\/Cocos'
      }]
    }, {
      'name': 'Colombia',
      'code': 'CO',
      'timezones': [{
        'name': 'America\/Bogota',
        'displayName': 'America\/Bogota'
      }]
    }, {
      'name': 'Comoros',
      'code': 'KM',
      'timezones': [{
        'name': 'Indian\/Comoro',
        'displayName': 'Indian\/Comoro'
      }]
    }, {
      'name': 'Cook Islands',
      'code': 'CK',
      'timezones': [{
        'name': 'Pacific\/Rarotonga',
        'displayName': 'Pacific\/Rarotonga'
      }]
    }, {
      'name': 'Costa Rica',
      'code': 'CR',
      'timezones': [{
        'name': 'America\/Costa_Rica',
        'displayName': 'America\/Costa Rica'
      }]
    }, {
      'name': 'Croatia',
      'code': 'HR',
      'timezones': [{
        'name': 'Europe\/Zagreb',
        'displayName': 'Europe\/Zagreb'
      }]
    }, {
      'name': 'Cuba',
      'code': 'CU',
      'timezones': [{
        'name': 'America\/Havana',
        'displayName': 'America\/Havana'
      }]
    }, {
      'name': 'Cura\u00e7ao',
      'code': 'CW',
      'timezones': [{
        'name': 'America\/Curacao',
        'displayName': 'America\/Curacao'
      }]
    }, {
      'name': 'Cyprus',
      'code': 'CY',
      'timezones': [{
        'name': 'Asia\/Nicosia',
        'displayName': 'Asia\/Nicosia'
      }]
    }, {
      'name': 'Czech Republic',
      'code': 'CZ',
      'timezones': [{
        'name': 'Europe\/Prague',
        'displayName': 'Europe\/Prague'
      }]
    }, {
      'name': 'Democratic Republic of the Congo',
      'code': 'CD',
      'timezones': [{
        'name': 'Africa\/Kinshasa',
        'displayName': 'Africa\/Kinshasa'
      }, {
        'name': 'Africa\/Lubumbashi',
        'displayName': 'Africa\/Lubumbashi'
      }]
    }, {
      'name': 'Denmark',
      'code': 'DK',
      'timezones': [{
        'name': 'Europe\/Copenhagen',
        'displayName': 'Europe\/Copenhagen'
      }]
    }, {
      'name': 'Djibouti',
      'code': 'DJ',
      'timezones': [{
        'name': 'Africa\/Djibouti',
        'displayName': 'Africa\/Djibouti'
      }]
    }, {
      'name': 'Dominica',
      'code': 'DM',
      'timezones': [{
        'name': 'America\/Dominica',
        'displayName': 'America\/Dominica'
      }]
    }, {
      'name': 'Dominican Republic',
      'code': 'DO',
      'timezones': [{
        'name': 'America\/Santo_Domingo',
        'displayName': 'America\/Santo Domingo'
      }]
    }, {
      'name': 'East Timor',
      'code': 'TL',
      'timezones': [{
        'name': 'Asia\/Dili',
        'displayName': 'Asia\/Dili'
      }]
    }, {
      'name': 'Ecuador',
      'code': 'EC',
      'timezones': [{
        'name': 'America\/Guayaquil',
        'displayName': 'America\/Guayaquil'
      }, {
        'name': 'Pacific\/Galapagos',
        'displayName': 'Pacific\/Galapagos'
      }]
    }, {
      'name': 'Egypt',
      'code': 'EG',
      'timezones': [{
        'name': 'Africa\/Cairo',
        'displayName': 'Africa\/Cairo'
      }]
    }, {
      'name': 'El Salvador',
      'code': 'SV',
      'timezones': [{
        'name': 'America\/El_Salvador',
        'displayName': 'America\/El Salvador'
      }]
    }, {
      'name': 'Equatorial Guinea',
      'code': 'GQ',
      'timezones': [{
        'name': 'Africa\/Malabo',
        'displayName': 'Africa\/Malabo'
      }]
    }, {
      'name': 'Eritrea',
      'code': 'ER',
      'timezones': [{
        'name': 'Africa\/Asmara',
        'displayName': 'Africa\/Asmara'
      }]
    }, {
      'name': 'Estonia',
      'code': 'EE',
      'timezones': [{
        'name': 'Europe\/Tallinn',
        'displayName': 'Europe\/Tallinn'
      }]
    }, {
      'name': 'Ethiopia',
      'code': 'ET',
      'timezones': [{
        'name': 'Africa\/Addis_Ababa',
        'displayName': 'Africa\/Addis Ababa'
      }]
    }, {
      'name': 'Falkland Islands',
      'code': 'FK',
      'timezones': [{
        'name': 'Atlantic\/Stanley',
        'displayName': 'Atlantic\/Stanley'
      }]
    }, {
      'name': 'Faroe Islands',
      'code': 'FO',
      'timezones': [{
        'name': 'Atlantic\/Faroe',
        'displayName': 'Atlantic\/Faroe'
      }]
    }, {
      'name': 'Fiji',
      'code': 'FJ',
      'timezones': [{
        'name': 'Pacific\/Fiji',
        'displayName': 'Pacific\/Fiji'
      }]
    }, {
      'name': 'Finland',
      'code': 'FI',
      'timezones': [{
        'name': 'Europe\/Helsinki',
        'displayName': 'Europe\/Helsinki'
      }]
    }, {
      'name': 'France',
      'code': 'FR',
      'timezones': [{
        'name': 'Europe\/Paris',
        'displayName': 'Europe\/Paris'
      }]
    }, {
      'name': 'French Guiana',
      'code': 'GF',
      'timezones': [{
        'name': 'America\/Cayenne',
        'displayName': 'America\/Cayenne'
      }]
    }, {
      'name': 'French Polynesia',
      'code': 'PF',
      'timezones': [{
        'name': 'Pacific\/Gambier',
        'displayName': 'Pacific\/Gambier'
      }, {
        'name': 'Pacific\/Marquesas',
        'displayName': 'Pacific\/Marquesas'
      }, {
        'name': 'Pacific\/Tahiti',
        'displayName': 'Pacific\/Tahiti'
      }]
    }, {
      'name': 'French Southern Territories',
      'code': 'TF',
      'timezones': [{
        'name': 'Indian\/Kerguelen',
        'displayName': 'Indian\/Kerguelen'
      }]
    }, {
      'name': 'Gabon',
      'code': 'GA',
      'timezones': [{
        'name': 'Africa\/Libreville',
        'displayName': 'Africa\/Libreville'
      }]
    }, {
      'name': 'Gambia',
      'code': 'GM',
      'timezones': [{
        'name': 'Africa\/Banjul',
        'displayName': 'Africa\/Banjul'
      }]
    }, {
      'name': 'Georgia',
      'code': 'GE',
      'timezones': [{
        'name': 'Asia\/Tbilisi',
        'displayName': 'Asia\/Tbilisi'
      }]
    }, {
      'name': 'Germany',
      'code': 'DE',
      'timezones': [{
        'name': 'Europe\/Berlin',
        'displayName': 'Europe\/Berlin'
      }, {
        'name': 'Europe\/Busingen',
        'displayName': 'Europe\/Busingen'
      }]
    }, {
      'name': 'Ghana',
      'code': 'GH',
      'timezones': [{
        'name': 'Africa\/Accra',
        'displayName': 'Africa\/Accra'
      }]
    }, {
      'name': 'Gibraltar',
      'code': 'GI',
      'timezones': [{
        'name': 'Europe\/Gibraltar',
        'displayName': 'Europe\/Gibraltar'
      }]
    }, {
      'name': 'Greece',
      'code': 'GR',
      'timezones': [{
        'name': 'Europe\/Athens',
        'displayName': 'Europe\/Athens'
      }]
    }, {
      'name': 'Greenland',
      'code': 'GL',
      'timezones': [{
        'name': 'America\/Danmarkshavn',
        'displayName': 'America\/Danmarkshavn'
      }, {
        'name': 'America\/Godthab',
        'displayName': 'America\/Godthab'
      }, {
        'name': 'America\/Scoresbysund',
        'displayName': 'America\/Scoresbysund'
      }, {
        'name': 'America\/Thule',
        'displayName': 'America\/Thule'
      }]
    }, {
      'name': 'Grenada',
      'code': 'GD',
      'timezones': [{
        'name': 'America\/Grenada',
        'displayName': 'America\/Grenada'
      }]
    }, {
      'name': 'Guadeloupe',
      'code': 'GP',
      'timezones': [{
        'name': 'America\/Guadeloupe',
        'displayName': 'America\/Guadeloupe'
      }]
    }, {
      'name': 'Guam',
      'code': 'GU',
      'timezones': [{
        'name': 'Pacific\/Guam',
        'displayName': 'Pacific\/Guam'
      }]
    }, {
      'name': 'Guatemala',
      'code': 'GT',
      'timezones': [{
        'name': 'America\/Guatemala',
        'displayName': 'America\/Guatemala'
      }]
    }, {
      'name': 'Guernsey',
      'code': 'GG',
      'timezones': [{
        'name': 'Europe\/Guernsey',
        'displayName': 'Europe\/Guernsey'
      }]
    }, {
      'name': 'Guinea',
      'code': 'GN',
      'timezones': [{
        'name': 'Africa\/Conakry',
        'displayName': 'Africa\/Conakry'
      }]
    }, {
      'name': 'Guinea-Bissau',
      'code': 'GW',
      'timezones': [{
        'name': 'Africa\/Bissau',
        'displayName': 'Africa\/Bissau'
      }]
    }, {
      'name': 'Guyana',
      'code': 'GY',
      'timezones': [{
        'name': 'America\/Guyana',
        'displayName': 'America\/Guyana'
      }]
    }, {
      'name': 'Haiti',
      'code': 'HT',
      'timezones': [{
        'name': 'America\/Port-au-Prince',
        'displayName': 'America\/Port-au-Prince'
      }]
    }, {
      'name': 'Heard Island and McDonald Islands',
      'code': 'HM',
      'timezones': []
    }, {
      'name': 'Honduras',
      'code': 'HN',
      'timezones': [{
        'name': 'America\/Tegucigalpa',
        'displayName': 'America\/Tegucigalpa'
      }]
    }, {
      'name': 'Hong Kong',
      'code': 'HK',
      'timezones': [{
        'name': 'Asia\/Hong_Kong',
        'displayName': 'Asia\/Hong Kong'
      }]
    }, {
      'name': 'Hungary',
      'code': 'HU',
      'timezones': [{
        'name': 'Europe\/Budapest',
        'displayName': 'Europe\/Budapest'
      }]
    }, {
      'name': 'Iceland',
      'code': 'IS',
      'timezones': [{
        'name': 'Atlantic\/Reykjavik',
        'displayName': 'Atlantic\/Reykjavik'
      }]
    }, {
      'name': 'India',
      'code': 'IN',
      'timezones': [{
        'name': 'Asia\/Kolkata',
        'displayName': 'Asia\/Kolkata'
      }]
    }, {
      'name': 'Indonesia',
      'code': 'ID',
      'timezones': [{
        'name': 'Asia\/Jakarta',
        'displayName': 'Asia\/Jakarta'
      }, {
        'name': 'Asia\/Jayapura',
        'displayName': 'Asia\/Jayapura'
      }, {
        'name': 'Asia\/Makassar',
        'displayName': 'Asia\/Makassar'
      }, {
        'name': 'Asia\/Pontianak',
        'displayName': 'Asia\/Pontianak'
      }]
    }, {
      'name': 'Iran',
      'code': 'IR',
      'timezones': [{
        'name': 'Asia\/Tehran',
        'displayName': 'Asia\/Tehran'
      }]
    }, {
      'name': 'Iraq',
      'code': 'IQ',
      'timezones': [{
        'name': 'Asia\/Baghdad',
        'displayName': 'Asia\/Baghdad'
      }]
    }, {
      'name': 'Republic of Ireland',
      'code': 'IE',
      'timezones': [{
        'name': 'Europe\/Dublin',
        'displayName': 'Europe\/Dublin'
      }]
    }, {
      'name': 'Isle of Man',
      'code': 'IM',
      'timezones': [{
        'name': 'Europe\/Isle_of_Man',
        'displayName': 'Europe\/Isle of Man'
      }]
    }, {
      'name': 'Israel',
      'code': 'IL',
      'timezones': [{
        'name': 'Asia\/Jerusalem',
        'displayName': 'Asia\/Jerusalem'
      }]
    }, {
      'name': 'Italy',
      'code': 'IT',
      'timezones': [{
        'name': 'Europe\/Rome',
        'displayName': 'Europe\/Rome'
      }]
    }, {
      'name': 'Ivory Coast',
      'code': 'CI',
      'timezones': [{
        'name': 'Africa\/Abidjan',
        'displayName': 'Africa\/Abidjan'
      }]
    }, {
      'name': 'Jamaica',
      'code': 'JM',
      'timezones': [{
        'name': 'America\/Jamaica',
        'displayName': 'America\/Jamaica'
      }]
    }, {
      'name': 'Japan',
      'code': 'JP',
      'timezones': [{
        'name': 'Asia\/Tokyo',
        'displayName': 'Asia\/Tokyo'
      }]
    }, {
      'name': 'Jersey',
      'code': 'JE',
      'timezones': [{
        'name': 'Europe\/Jersey',
        'displayName': 'Europe\/Jersey'
      }]
    }, {
      'name': 'Jordan',
      'code': 'JO',
      'timezones': [{
        'name': 'Asia\/Amman',
        'displayName': 'Asia\/Amman'
      }]
    }, {
      'name': 'Kazakhstan',
      'code': 'KZ',
      'timezones': [{
        'name': 'Asia\/Almaty',
        'displayName': 'Asia\/Almaty'
      }, {
        'name': 'Asia\/Aqtau',
        'displayName': 'Asia\/Aqtau'
      }, {
        'name': 'Asia\/Aqtobe',
        'displayName': 'Asia\/Aqtobe'
      }, {
        'name': 'Asia\/Oral',
        'displayName': 'Asia\/Oral'
      }, {
        'name': 'Asia\/Qyzylorda',
        'displayName': 'Asia\/Qyzylorda'
      }]
    }, {
      'name': 'Kenya',
      'code': 'KE',
      'timezones': [{
        'name': 'Africa\/Nairobi',
        'displayName': 'Africa\/Nairobi'
      }]
    }, {
      'name': 'Kiribati',
      'code': 'KI',
      'timezones': [{
        'name': 'Pacific\/Enderbury',
        'displayName': 'Pacific\/Enderbury'
      }, {
        'name': 'Pacific\/Kiritimati',
        'displayName': 'Pacific\/Kiritimati'
      }, {
        'name': 'Pacific\/Tarawa',
        'displayName': 'Pacific\/Tarawa'
      }]
    }, {
      'name': 'Kosovo',
      'code': 'XK',
      'timezones': []
    }, {
      'name': 'Kuwait',
      'code': 'KW',
      'timezones': [{
        'name': 'Asia\/Kuwait',
        'displayName': 'Asia\/Kuwait'
      }]
    }, {
      'name': 'Kyrgyzstan',
      'code': 'KG',
      'timezones': [{
        'name': 'Asia\/Bishkek',
        'displayName': 'Asia\/Bishkek'
      }]
    }, {
      'name': 'Laos',
      'code': 'LA',
      'timezones': [{
        'name': 'Asia\/Vientiane',
        'displayName': 'Asia\/Vientiane'
      }]
    }, {
      'name': 'Latvia',
      'code': 'LV',
      'timezones': [{
        'name': 'Europe\/Riga',
        'displayName': 'Europe\/Riga'
      }]
    }, {
      'name': 'Lebanon',
      'code': 'LB',
      'timezones': [{
        'name': 'Asia\/Beirut',
        'displayName': 'Asia\/Beirut'
      }]
    }, {
      'name': 'Lesotho',
      'code': 'LS',
      'timezones': [{
        'name': 'Africa\/Maseru',
        'displayName': 'Africa\/Maseru'
      }]
    }, {
      'name': 'Liberia',
      'code': 'LR',
      'timezones': [{
        'name': 'Africa\/Monrovia',
        'displayName': 'Africa\/Monrovia'
      }]
    }, {
      'name': 'Libya',
      'code': 'LY',
      'timezones': [{
        'name': 'Africa\/Tripoli',
        'displayName': 'Africa\/Tripoli'
      }]
    }, {
      'name': 'Liechtenstein',
      'code': 'LI',
      'timezones': [{
        'name': 'Europe\/Vaduz',
        'displayName': 'Europe\/Vaduz'
      }]
    }, {
      'name': 'Lithuania',
      'code': 'LT',
      'timezones': [{
        'name': 'Europe\/Vilnius',
        'displayName': 'Europe\/Vilnius'
      }]
    }, {
      'name': 'Luxembourg',
      'code': 'LU',
      'timezones': [{
        'name': 'Europe\/Luxembourg',
        'displayName': 'Europe\/Luxembourg'
      }]
    }, {
      'name': 'Macao',
      'code': 'MO',
      'timezones': [{
        'name': 'Asia\/Macau',
        'displayName': 'Asia\/Macau'
      }]
    }, {
      'name': 'Macedonia',
      'code': 'MK',
      'timezones': [{
        'name': 'Europe\/Skopje',
        'displayName': 'Europe\/Skopje'
      }]
    }, {
      'name': 'Madagascar',
      'code': 'MG',
      'timezones': [{
        'name': 'Indian\/Antananarivo',
        'displayName': 'Indian\/Antananarivo'
      }]
    }, {
      'name': 'Malawi',
      'code': 'MW',
      'timezones': [{
        'name': 'Africa\/Blantyre',
        'displayName': 'Africa\/Blantyre'
      }]
    }, {
      'name': 'Malaysia',
      'code': 'MY',
      'timezones': [{
        'name': 'Asia\/Kuala_Lumpur',
        'displayName': 'Asia\/Kuala Lumpur'
      }, {
        'name': 'Asia\/Kuching',
        'displayName': 'Asia\/Kuching'
      }]
    }, {
      'name': 'Maldives',
      'code': 'MV',
      'timezones': [{
        'name': 'Indian\/Maldives',
        'displayName': 'Indian\/Maldives'
      }]
    }, {
      'name': 'Mali',
      'code': 'ML',
      'timezones': [{
        'name': 'Africa\/Bamako',
        'displayName': 'Africa\/Bamako'
      }]
    }, {
      'name': 'Malta',
      'code': 'MT',
      'timezones': [{
        'name': 'Europe\/Malta',
        'displayName': 'Europe\/Malta'
      }]
    }, {
      'name': 'Marshall Islands',
      'code': 'MH',
      'timezones': [{
        'name': 'Pacific\/Kwajalein',
        'displayName': 'Pacific\/Kwajalein'
      }, {
        'name': 'Pacific\/Majuro',
        'displayName': 'Pacific\/Majuro'
      }]
    }, {
      'name': 'Martinique',
      'code': 'MQ',
      'timezones': [{
        'name': 'America\/Martinique',
        'displayName': 'America\/Martinique'
      }]
    }, {
      'name': 'Mauritania',
      'code': 'MR',
      'timezones': [{
        'name': 'Africa\/Nouakchott',
        'displayName': 'Africa\/Nouakchott'
      }]
    }, {
      'name': 'Mauritius',
      'code': 'MU',
      'timezones': [{
        'name': 'Indian\/Mauritius',
        'displayName': 'Indian\/Mauritius'
      }]
    }, {
      'name': 'Mayotte',
      'code': 'YT',
      'timezones': [{
        'name': 'Indian\/Mayotte',
        'displayName': 'Indian\/Mayotte'
      }]
    }, {
      'name': 'Mexico',
      'code': 'MX',
      'timezones': [{
        'name': 'America\/Bahia_Banderas',
        'displayName': 'America\/Bahia Banderas'
      }, {
        'name': 'America\/Cancun',
        'displayName': 'America\/Cancun'
      }, {
        'name': 'America\/Chihuahua',
        'displayName': 'America\/Chihuahua'
      }, {
        'name': 'America\/Hermosillo',
        'displayName': 'America\/Hermosillo'
      }, {
        'name': 'America\/Matamoros',
        'displayName': 'America\/Matamoros'
      }, {
        'name': 'America\/Mazatlan',
        'displayName': 'America\/Mazatlan'
      }, {
        'name': 'America\/Merida',
        'displayName': 'America\/Merida'
      }, {
        'name': 'America\/Mexico_City',
        'displayName': 'America\/Mexico City'
      }, {
        'name': 'America\/Monterrey',
        'displayName': 'America\/Monterrey'
      }, {
        'name': 'America\/Ojinaga',
        'displayName': 'America\/Ojinaga'
      }, {
        'name': 'America\/Santa_Isabel',
        'displayName': 'America\/Santa Isabel'
      }, {
        'name': 'America\/Tijuana',
        'displayName': 'America\/Tijuana'
      }]
    }, {
      'name': 'Micronesia',
      'code': 'FM',
      'timezones': [{
        'name': 'Pacific\/Chuuk',
        'displayName': 'Pacific\/Chuuk'
      }, {
        'name': 'Pacific\/Kosrae',
        'displayName': 'Pacific\/Kosrae'
      }, {
        'name': 'Pacific\/Pohnpei',
        'displayName': 'Pacific\/Pohnpei'
      }]
    }, {
      'name': 'Moldova',
      'code': 'MD',
      'timezones': [{
        'name': 'Europe\/Chisinau',
        'displayName': 'Europe\/Chisinau'
      }]
    }, {
      'name': 'Monaco',
      'code': 'MC',
      'timezones': [{
        'name': 'Europe\/Monaco',
        'displayName': 'Europe\/Monaco'
      }]
    }, {
      'name': 'Mongolia',
      'code': 'MN',
      'timezones': [{
        'name': 'Asia\/Choibalsan',
        'displayName': 'Asia\/Choibalsan'
      }, {
        'name': 'Asia\/Hovd',
        'displayName': 'Asia\/Hovd'
      }, {
        'name': 'Asia\/Ulaanbaatar',
        'displayName': 'Asia\/Ulaanbaatar'
      }]
    }, {
      'name': 'Montenegro',
      'code': 'ME',
      'timezones': [{
        'name': 'Europe\/Podgorica',
        'displayName': 'Europe\/Podgorica'
      }]
    }, {
      'name': 'Montserrat',
      'code': 'MS',
      'timezones': [{
        'name': 'America\/Montserrat',
        'displayName': 'America\/Montserrat'
      }]
    }, {
      'name': 'Morocco',
      'code': 'MA',
      'timezones': [{
        'name': 'Africa\/Casablanca',
        'displayName': 'Africa\/Casablanca'
      }]
    }, {
      'name': 'Mozambique',
      'code': 'MZ',
      'timezones': [{
        'name': 'Africa\/Maputo',
        'displayName': 'Africa\/Maputo'
      }]
    }, {
      'name': 'Myanmar',
      'code': 'MM',
      'timezones': [{
        'name': 'Asia\/Rangoon',
        'displayName': 'Asia\/Rangoon'
      }]
    }, {
      'name': 'Namibia',
      'code': 'NA',
      'timezones': [{
        'name': 'Africa\/Windhoek',
        'displayName': 'Africa\/Windhoek'
      }]
    }, {
      'name': 'Nauru',
      'code': 'NR',
      'timezones': [{
        'name': 'Pacific\/Nauru',
        'displayName': 'Pacific\/Nauru'
      }]
    }, {
      'name': 'Nepal',
      'code': 'NP',
      'timezones': [{
        'name': 'Asia\/Kathmandu',
        'displayName': 'Asia\/Kathmandu'
      }]
    }, {
      'name': 'Netherlands',
      'code': 'NL',
      'timezones': [{
        'name': 'Europe\/Amsterdam',
        'displayName': 'Europe\/Amsterdam'
      }]
    }, {
      'name': 'Netherlands Antilles',
      'code': 'AN',
      'timezones': []
    }, {
      'name': 'New Caledonia',
      'code': 'NC',
      'timezones': [{
        'name': 'Pacific\/Noumea',
        'displayName': 'Pacific\/Noumea'
      }]
    }, {
      'name': 'New Zealand',
      'code': 'NZ',
      'timezones': [{
        'name': 'Pacific\/Auckland',
        'displayName': 'Pacific\/Auckland'
      }, {
        'name': 'Pacific\/Chatham',
        'displayName': 'Pacific\/Chatham'
      }]
    }, {
      'name': 'Nicaragua',
      'code': 'NI',
      'timezones': [{
        'name': 'America\/Managua',
        'displayName': 'America\/Managua'
      }]
    }, {
      'name': 'Niger',
      'code': 'NE',
      'timezones': [{
        'name': 'Africa\/Niamey',
        'displayName': 'Africa\/Niamey'
      }]
    }, {
      'name': 'Nigeria',
      'code': 'NG',
      'timezones': [{
        'name': 'Africa\/Lagos',
        'displayName': 'Africa\/Lagos'
      }]
    }, {
      'name': 'Niue',
      'code': 'NU',
      'timezones': [{
        'name': 'Pacific\/Niue',
        'displayName': 'Pacific\/Niue'
      }]
    }, {
      'name': 'Norfolk Island',
      'code': 'NF',
      'timezones': [{
        'name': 'Pacific\/Norfolk',
        'displayName': 'Pacific\/Norfolk'
      }]
    }, {
      'name': 'North Korea',
      'code': 'KP',
      'timezones': [{
        'name': 'Asia\/Pyongyang',
        'displayName': 'Asia\/Pyongyang'
      }]
    }, {
      'name': 'Northern Mariana Islands',
      'code': 'MP',
      'timezones': [{
        'name': 'Pacific\/Saipan',
        'displayName': 'Pacific\/Saipan'
      }]
    }, {
      'name': 'Norway',
      'code': 'NO',
      'timezones': [{
        'name': 'Europe\/Oslo',
        'displayName': 'Europe\/Oslo'
      }]
    }, {
      'name': 'Oman',
      'code': 'OM',
      'timezones': [{
        'name': 'Asia\/Muscat',
        'displayName': 'Asia\/Muscat'
      }]
    }, {
      'name': 'Pakistan',
      'code': 'PK',
      'timezones': [{
        'name': 'Asia\/Karachi',
        'displayName': 'Asia\/Karachi'
      }]
    }, {
      'name': 'Palau',
      'code': 'PW',
      'timezones': [{
        'name': 'Pacific\/Palau',
        'displayName': 'Pacific\/Palau'
      }]
    }, {
      'name': 'Palestinian Territory',
      'code': 'PS',
      'timezones': [{
        'name': 'Asia\/Gaza',
        'displayName': 'Asia\/Gaza'
      }, {
        'name': 'Asia\/Hebron',
        'displayName': 'Asia\/Hebron'
      }]
    }, {
      'name': 'Panama',
      'code': 'PA',
      'timezones': [{
        'name': 'America\/Panama',
        'displayName': 'America\/Panama'
      }]
    }, {
      'name': 'Papua New Guinea',
      'code': 'PG',
      'timezones': [{
        'name': 'Pacific\/Bougainville',
        'displayName': 'Pacific\/Bougainville'
      }, {
        'name': 'Pacific\/Port_Moresby',
        'displayName': 'Pacific\/Port Moresby'
      }]
    }, {
      'name': 'Paraguay',
      'code': 'PY',
      'timezones': [{
        'name': 'America\/Asuncion',
        'displayName': 'America\/Asuncion'
      }]
    }, {
      'name': 'Peru',
      'code': 'PE',
      'timezones': [{
        'name': 'America\/Lima',
        'displayName': 'America\/Lima'
      }]
    }, {
      'name': 'Philippines',
      'code': 'PH',
      'timezones': [{
        'name': 'Asia\/Manila',
        'displayName': 'Asia\/Manila'
      }]
    }, {
      'name': 'Pitcairn',
      'code': 'PN',
      'timezones': [{
        'name': 'Pacific\/Pitcairn',
        'displayName': 'Pacific\/Pitcairn'
      }]
    }, {
      'name': 'Poland',
      'code': 'PL',
      'timezones': [{
        'name': 'Europe\/Warsaw',
        'displayName': 'Europe\/Warsaw'
      }]
    }, {
      'name': 'Portugal',
      'code': 'PT',
      'timezones': [{
        'name': 'Atlantic\/Azores',
        'displayName': 'Atlantic\/Azores'
      }, {
        'name': 'Atlantic\/Madeira',
        'displayName': 'Atlantic\/Madeira'
      }, {
        'name': 'Europe\/Lisbon',
        'displayName': 'Europe\/Lisbon'
      }]
    }, {
      'name': 'Puerto Rico',
      'code': 'PR',
      'timezones': [{
        'name': 'America\/Puerto_Rico',
        'displayName': 'America\/Puerto Rico'
      }]
    }, {
      'name': 'Qatar',
      'code': 'QA',
      'timezones': [{
        'name': 'Asia\/Qatar',
        'displayName': 'Asia\/Qatar'
      }]
    }, {
      'name': 'Republic of the Congo',
      'code': 'CG',
      'timezones': [{
        'name': 'Africa\/Brazzaville',
        'displayName': 'Africa\/Brazzaville'
      }]
    }, {
      'name': 'Reunion',
      'code': 'RE',
      'timezones': [{
        'name': 'Indian\/Reunion',
        'displayName': 'Indian\/Reunion'
      }]
    }, {
      'name': 'Romania',
      'code': 'RO',
      'timezones': [{
        'name': 'Europe\/Bucharest',
        'displayName': 'Europe\/Bucharest'
      }]
    }, {
      'name': 'Russia',
      'code': 'RU',
      'timezones': [{
        'name': 'Asia\/Anadyr',
        'displayName': 'Asia\/Anadyr'
      }, {
        'name': 'Asia\/Chita',
        'displayName': 'Asia\/Chita'
      }, {
        'name': 'Asia\/Irkutsk',
        'displayName': 'Asia\/Irkutsk'
      }, {
        'name': 'Asia\/Kamchatka',
        'displayName': 'Asia\/Kamchatka'
      }, {
        'name': 'Asia\/Khandyga',
        'displayName': 'Asia\/Khandyga'
      }, {
        'name': 'Asia\/Krasnoyarsk',
        'displayName': 'Asia\/Krasnoyarsk'
      }, {
        'name': 'Asia\/Magadan',
        'displayName': 'Asia\/Magadan'
      }, {
        'name': 'Asia\/Novokuznetsk',
        'displayName': 'Asia\/Novokuznetsk'
      }, {
        'name': 'Asia\/Novosibirsk',
        'displayName': 'Asia\/Novosibirsk'
      }, {
        'name': 'Asia\/Omsk',
        'displayName': 'Asia\/Omsk'
      }, {
        'name': 'Asia\/Sakhalin',
        'displayName': 'Asia\/Sakhalin'
      }, {
        'name': 'Asia\/Srednekolymsk',
        'displayName': 'Asia\/Srednekolymsk'
      }, {
        'name': 'Asia\/Ust-Nera',
        'displayName': 'Asia\/Ust-Nera'
      }, {
        'name': 'Asia\/Vladivostok',
        'displayName': 'Asia\/Vladivostok'
      }, {
        'name': 'Asia\/Yakutsk',
        'displayName': 'Asia\/Yakutsk'
      }, {
        'name': 'Asia\/Yekaterinburg',
        'displayName': 'Asia\/Yekaterinburg'
      }, {
        'name': 'Europe\/Kaliningrad',
        'displayName': 'Europe\/Kaliningrad'
      }, {
        'name': 'Europe\/Moscow',
        'displayName': 'Europe\/Moscow'
      }, {
        'name': 'Europe\/Samara',
        'displayName': 'Europe\/Samara'
      }, {
        'name': 'Europe\/Simferopol',
        'displayName': 'Europe\/Simferopol'
      }, {
        'name': 'Europe\/Volgograd',
        'displayName': 'Europe\/Volgograd'
      }]
    }, {
      'name': 'Rwanda',
      'code': 'RW',
      'timezones': [{
        'name': 'Africa\/Kigali',
        'displayName': 'Africa\/Kigali'
      }]
    }, {
      'name': 'Saint Barth\u00e9lemy',
      'code': 'BL',
      'timezones': [{
        'name': 'America\/St_Barthelemy',
        'displayName': 'America\/St Barthelemy'
      }]
    }, {
      'name': 'Saint Helena',
      'code': 'SH',
      'timezones': [{
        'name': 'Atlantic\/St_Helena',
        'displayName': 'Atlantic\/St Helena'
      }]
    }, {
      'name': 'Saint Kitts and Nevis',
      'code': 'KN',
      'timezones': [{
        'name': 'America\/St_Kitts',
        'displayName': 'America\/St Kitts'
      }]
    }, {
      'name': 'Saint Lucia',
      'code': 'LC',
      'timezones': [{
        'name': 'America\/St_Lucia',
        'displayName': 'America\/St Lucia'
      }]
    }, {
      'name': 'Saint Martin',
      'code': 'MF',
      'timezones': [{
        'name': 'America\/Marigot',
        'displayName': 'America\/Marigot'
      }]
    }, {
      'name': 'Saint Pierre and Miquelon',
      'code': 'PM',
      'timezones': [{
        'name': 'America\/Miquelon',
        'displayName': 'America\/Miquelon'
      }]
    }, {
      'name': 'Saint Vincent and the Grenadines',
      'code': 'VC',
      'timezones': [{
        'name': 'America\/St_Vincent',
        'displayName': 'America\/St Vincent'
      }]
    }, {
      'name': 'Samoa',
      'code': 'WS',
      'timezones': [{
        'name': 'Pacific\/Apia',
        'displayName': 'Pacific\/Apia'
      }]
    }, {
      'name': 'San Marino',
      'code': 'SM',
      'timezones': [{
        'name': 'Europe\/San_Marino',
        'displayName': 'Europe\/San Marino'
      }]
    }, {
      'name': 'Sao Tome and Principe',
      'code': 'ST',
      'timezones': [{
        'name': 'Africa\/Sao_Tome',
        'displayName': 'Africa\/Sao Tome'
      }]
    }, {
      'name': 'Saudi Arabia',
      'code': 'SA',
      'timezones': [{
        'name': 'Asia\/Riyadh',
        'displayName': 'Asia\/Riyadh'
      }]
    }, {
      'name': 'Senegal',
      'code': 'SN',
      'timezones': [{
        'name': 'Africa\/Dakar',
        'displayName': 'Africa\/Dakar'
      }]
    }, {
      'name': 'Serbia',
      'code': 'RS',
      'timezones': [{
        'name': 'Europe\/Belgrade',
        'displayName': 'Europe\/Belgrade'
      }]
    }, {
      'name': 'Serbia and Montenegro',
      'code': 'CS',
      'timezones': []
    }, {
      'name': 'Seychelles',
      'code': 'SC',
      'timezones': [{
        'name': 'Indian\/Mahe',
        'displayName': 'Indian\/Mahe'
      }]
    }, {
      'name': 'Sierra Leone',
      'code': 'SL',
      'timezones': [{
        'name': 'Africa\/Freetown',
        'displayName': 'Africa\/Freetown'
      }]
    }, {
      'name': 'Singapore',
      'code': 'SG',
      'timezones': [{
        'name': 'Asia\/Singapore',
        'displayName': 'Asia\/Singapore'
      }]
    }, {
      'name': 'Sint Maarten',
      'code': 'SX',
      'timezones': [{
        'name': 'America\/Lower_Princes',
        'displayName': 'America\/Lower Princes'
      }]
    }, {
      'name': 'Slovakia',
      'code': 'SK',
      'timezones': [{
        'name': 'Europe\/Bratislava',
        'displayName': 'Europe\/Bratislava'
      }]
    }, {
      'name': 'Slovenia',
      'code': 'SI',
      'timezones': [{
        'name': 'Europe\/Ljubljana',
        'displayName': 'Europe\/Ljubljana'
      }]
    }, {
      'name': 'Solomon Islands',
      'code': 'SB',
      'timezones': [{
        'name': 'Pacific\/Guadalcanal',
        'displayName': 'Pacific\/Guadalcanal'
      }]
    }, {
      'name': 'Somalia',
      'code': 'SO',
      'timezones': [{
        'name': 'Africa\/Mogadishu',
        'displayName': 'Africa\/Mogadishu'
      }]
    }, {
      'name': 'South Africa',
      'code': 'ZA',
      'timezones': [{
        'name': 'Africa\/Johannesburg',
        'displayName': 'Africa\/Johannesburg'
      }]
    }, {
      'name': 'South Georgia and the South Sandwich Islands',
      'code': 'GS',
      'timezones': [{
        'name': 'Atlantic\/South_Georgia',
        'displayName': 'Atlantic\/South Georgia'
      }]
    }, {
      'name': 'South Korea',
      'code': 'KR',
      'timezones': [{
        'name': 'Asia\/Seoul',
        'displayName': 'Asia\/Seoul'
      }]
    }, {
      'name': 'South Sudan',
      'code': 'SS',
      'timezones': [{
        'name': 'Africa\/Juba',
        'displayName': 'Africa\/Juba'
      }]
    }, {
      'name': 'Spain',
      'code': 'ES',
      'timezones': [{
        'name': 'Africa\/Ceuta',
        'displayName': 'Africa\/Ceuta'
      }, {
        'name': 'Atlantic\/Canary',
        'displayName': 'Atlantic\/Canary'
      }, {
        'name': 'Europe\/Madrid',
        'displayName': 'Europe\/Madrid'
      }]
    }, {
      'name': 'Sri Lanka',
      'code': 'LK',
      'timezones': [{
        'name': 'Asia\/Colombo',
        'displayName': 'Asia\/Colombo'
      }]
    }, {
      'name': 'Sudan',
      'code': 'SD',
      'timezones': [{
        'name': 'Africa\/Khartoum',
        'displayName': 'Africa\/Khartoum'
      }]
    }, {
      'name': 'Suriname',
      'code': 'SR',
      'timezones': [{
        'name': 'America\/Paramaribo',
        'displayName': 'America\/Paramaribo'
      }]
    }, {
      'name': 'Svalbard and Jan Mayen',
      'code': 'SJ',
      'timezones': [{
        'name': 'Arctic\/Longyearbyen',
        'displayName': 'Arctic\/Longyearbyen'
      }]
    }, {
      'name': 'Swaziland',
      'code': 'SZ',
      'timezones': [{
        'name': 'Africa\/Mbabane',
        'displayName': 'Africa\/Mbabane'
      }]
    }, {
      'name': 'Sweden',
      'code': 'SE',
      'timezones': [{
        'name': 'Europe\/Stockholm',
        'displayName': 'Europe\/Stockholm'
      }]
    }, {
      'name': 'Switzerland',
      'code': 'CH',
      'timezones': [{
        'name': 'Europe\/Zurich',
        'displayName': 'Europe\/Zurich'
      }]
    }, {
      'name': 'Syria',
      'code': 'SY',
      'timezones': [{
        'name': 'Asia\/Damascus',
        'displayName': 'Asia\/Damascus'
      }]
    }, {
      'name': 'Taiwan',
      'code': 'TW',
      'timezones': [{
        'name': 'Asia\/Taipei',
        'displayName': 'Asia\/Taipei'
      }]
    }, {
      'name': 'Tajikistan',
      'code': 'TJ',
      'timezones': [{
        'name': 'Asia\/Dushanbe',
        'displayName': 'Asia\/Dushanbe'
      }]
    }, {
      'name': 'Tanzania',
      'code': 'TZ',
      'timezones': [{
        'name': 'Africa\/Dar_es_Salaam',
        'displayName': 'Africa\/Dar es Salaam'
      }]
    }, {
      'name': 'Thailand',
      'code': 'TH',
      'timezones': [{
        'name': 'Asia\/Bangkok',
        'displayName': 'Asia\/Bangkok'
      }]
    }, {
      'name': 'Togo',
      'code': 'TG',
      'timezones': [{
        'name': 'Africa\/Lome',
        'displayName': 'Africa\/Lome'
      }]
    }, {
      'name': 'Tokelau',
      'code': 'TK',
      'timezones': [{
        'name': 'Pacific\/Fakaofo',
        'displayName': 'Pacific\/Fakaofo'
      }]
    }, {
      'name': 'Tonga',
      'code': 'TO',
      'timezones': [{
        'name': 'Pacific\/Tongatapu',
        'displayName': 'Pacific\/Tongatapu'
      }]
    }, {
      'name': 'Trinidad and Tobago',
      'code': 'TT',
      'timezones': [{
        'name': 'America\/Port_of_Spain',
        'displayName': 'America\/Port of Spain'
      }]
    }, {
      'name': 'Tunisia',
      'code': 'TN',
      'timezones': [{
        'name': 'Africa\/Tunis',
        'displayName': 'Africa\/Tunis'
      }]
    }, {
      'name': 'Turkey',
      'code': 'TR',
      'timezones': [{
        'name': 'Europe\/Istanbul',
        'displayName': 'Europe\/Istanbul'
      }]
    }, {
      'name': 'Turkmenistan',
      'code': 'TM',
      'timezones': [{
        'name': 'Asia\/Ashgabat',
        'displayName': 'Asia\/Ashgabat'
      }]
    }, {
      'name': 'Turks and Caicos Islands',
      'code': 'TC',
      'timezones': [{
        'name': 'America\/Grand_Turk',
        'displayName': 'America\/Grand Turk'
      }]
    }, {
      'name': 'Tuvalu',
      'code': 'TV',
      'timezones': [{
        'name': 'Pacific\/Funafuti',
        'displayName': 'Pacific\/Funafuti'
      }]
    }, {
      'name': 'U.S. Virgin Islands',
      'code': 'VI',
      'timezones': [{
        'name': 'America\/St_Thomas',
        'displayName': 'America\/St Thomas'
      }]
    }, {
      'name': 'Uganda',
      'code': 'UG',
      'timezones': [{
        'name': 'Africa\/Kampala',
        'displayName': 'Africa\/Kampala'
      }]
    }, {
      'name': 'Ukraine',
      'code': 'UA',
      'timezones': [{
        'name': 'Europe\/Kiev',
        'displayName': 'Europe\/Kiev'
      }, {
        'name': 'Europe\/Uzhgorod',
        'displayName': 'Europe\/Uzhgorod'
      }, {
        'name': 'Europe\/Zaporozhye',
        'displayName': 'Europe\/Zaporozhye'
      }]
    }, {
      'name': 'United Arab Emirates',
      'code': 'AE',
      'timezones': [{
        'name': 'Asia\/Dubai',
        'displayName': 'Asia\/Dubai'
      }]
    }, {
      'name': 'United Kingdom',
      'code': 'GB',
      'timezones': [{
        'name': 'Europe\/London',
        'displayName': 'Europe\/London'
      }]
    }, {
      'name': 'United States',
      'code': 'US',
      'timezones': [{
        'name': 'America\/Adak',
        'displayName': 'America\/Adak'
      }, {
        'name': 'America\/Anchorage',
        'displayName': 'America\/Anchorage'
      }, {
        'name': 'America\/Boise',
        'displayName': 'America\/Boise'
      }, {
        'name': 'America\/Chicago',
        'displayName': 'America\/Chicago'
      }, {
        'name': 'America\/Denver',
        'displayName': 'America\/Denver'
      }, {
        'name': 'America\/Detroit',
        'displayName': 'America\/Detroit'
      }, {
        'name': 'America\/Indiana\/Indianapolis',
        'displayName': 'America\/Indiana\/Indianapolis'
      }, {
        'name': 'America\/Indiana\/Knox',
        'displayName': 'America\/Indiana\/Knox'
      }, {
        'name': 'America\/Indiana\/Marengo',
        'displayName': 'America\/Indiana\/Marengo'
      }, {
        'name': 'America\/Indiana\/Petersburg',
        'displayName': 'America\/Indiana\/Petersburg'
      }, {
        'name': 'America\/Indiana\/Tell_City',
        'displayName': 'America\/Indiana\/Tell City'
      }, {
        'name': 'America\/Indiana\/Vevay',
        'displayName': 'America\/Indiana\/Vevay'
      }, {
        'name': 'America\/Indiana\/Vincennes',
        'displayName': 'America\/Indiana\/Vincennes'
      }, {
        'name': 'America\/Indiana\/Winamac',
        'displayName': 'America\/Indiana\/Winamac'
      }, {
        'name': 'America\/Juneau',
        'displayName': 'America\/Juneau'
      }, {
        'name': 'America\/Kentucky\/Louisville',
        'displayName': 'America\/Kentucky\/Louisville'
      }, {
        'name': 'America\/Kentucky\/Monticello',
        'displayName': 'America\/Kentucky\/Monticello'
      }, {
        'name': 'America\/Los_Angeles',
        'displayName': 'America\/Los Angeles'
      }, {
        'name': 'America\/Menominee',
        'displayName': 'America\/Menominee'
      }, {
        'name': 'America\/Metlakatla',
        'displayName': 'America\/Metlakatla'
      }, {
        'name': 'America\/New_York',
        'displayName': 'America\/New York'
      }, {
        'name': 'America\/Nome',
        'displayName': 'America\/Nome'
      }, {
        'name': 'America\/North_Dakota\/Beulah',
        'displayName': 'America\/North Dakota\/Beulah'
      }, {
        'name': 'America\/North_Dakota\/Center',
        'displayName': 'America\/North Dakota\/Center'
      }, {
        'name': 'America\/North_Dakota\/New_Salem',
        'displayName': 'America\/North Dakota\/New Salem'
      }, {
        'name': 'America\/Phoenix',
        'displayName': 'America\/Phoenix'
      }, {
        'name': 'America\/Sitka',
        'displayName': 'America\/Sitka'
      }, {
        'name': 'America\/Yakutat',
        'displayName': 'America\/Yakutat'
      }, {
        'name': 'Pacific\/Honolulu',
        'displayName': 'Pacific\/Honolulu'
      }]
    }, {
      'name': 'United States Minor Outlying Islands',
      'code': 'UM',
      'timezones': [{
        'name': 'Pacific\/Johnston',
        'displayName': 'Pacific\/Johnston'
      }, {
        'name': 'Pacific\/Midway',
        'displayName': 'Pacific\/Midway'
      }, {
        'name': 'Pacific\/Wake',
        'displayName': 'Pacific\/Wake'
      }]
    }, {
      'name': 'Uruguay',
      'code': 'UY',
      'timezones': [{
        'name': 'America\/Montevideo',
        'displayName': 'America\/Montevideo'
      }]
    }, {
      'name': 'Uzbekistan',
      'code': 'UZ',
      'timezones': [{
        'name': 'Asia\/Samarkand',
        'displayName': 'Asia\/Samarkand'
      }, {
        'name': 'Asia\/Tashkent',
        'displayName': 'Asia\/Tashkent'
      }]
    }, {
      'name': 'Vanuatu',
      'code': 'VU',
      'timezones': [{
        'name': 'Pacific\/Efate',
        'displayName': 'Pacific\/Efate'
      }]
    }, {
      'name': 'Vatican',
      'code': 'VA',
      'timezones': [{
        'name': 'Europe\/Vatican',
        'displayName': 'Europe\/Vatican'
      }]
    }, {
      'name': 'Venezuela',
      'code': 'VE',
      'timezones': [{
        'name': 'America\/Caracas',
        'displayName': 'America\/Caracas'
      }]
    }, {
      'name': 'Vietnam',
      'code': 'VN',
      'timezones': [{
        'name': 'Asia\/Ho_Chi_Minh',
        'displayName': 'Asia\/Ho Chi Minh'
      }]
    }, {
      'name': 'Wallis and Futuna',
      'code': 'WF',
      'timezones': [{
        'name': 'Pacific\/Wallis',
        'displayName': 'Pacific\/Wallis'
      }]
    }, {
      'name': 'Western Sahara',
      'code': 'EH',
      'timezones': [{
        'name': 'Africa\/El_Aaiun',
        'displayName': 'Africa\/El Aaiun'
      }]
    }, {
      'name': 'Yemen',
      'code': 'YE',
      'timezones': [{
        'name': 'Asia\/Aden',
        'displayName': 'Asia\/Aden'
      }]
    }, {
      'name': 'Zambia',
      'code': 'ZM',
      'timezones': [{
        'name': 'Africa\/Lusaka',
        'displayName': 'Africa\/Lusaka'
      }]
    }, {
      'name': 'Zimbabwe',
      'code': 'ZW',
      'timezones': [{
        'name': 'Africa\/Harare',
        'displayName': 'Africa\/Harare'
      }]
    }];

    return {
      countries: function () {
        return countries;
      }
    };
  }
})();
/**
 * Stores auth tokens in the KeyChain (iOS) or or SecureStorage (android).
 * If an android device has no secure lock screen set, then there is no
 * SecureStorage available, so we have to fall back to using local storage on
 * the unsecure device.
 */
(function () {
    'use strict';

    angular.module('MyPTHub').factory('CredStoreService', service);

    service.$inject = ['$q', 'LocalStorageService', 'DeviceInfoService'];

    function service($q, LocalStorageService, DeviceInfoService) {

        var mode = 'ss';

        var ss;

        var service = {};

        function save(key, value) {
            return $q(function (resolve, reject) {
                if (mode === 'ss') {
                    // Secure Storage mode
                    ss.set(resolve, reject, key, value);
                } else {
                    // Local Storage mode
                    LocalStorageService.setKey(key, value);
                    resolve(value);
                }
            });
        }

        function get(key) {
            return $q(function (resolve, reject) {
                if (mode === 'ss') {
                    // Secure Storage mode
                    ss.get(resolve, reject, key);
                } else {
                    // Local Storage mode
                    var token = LocalStorageService.getKey(key);
                    if (token) {
                        resolve(token);
                    } else {
                        reject();
                    }
                }
            });
        }

        function remove(key) {
            return $q(function (resolve, reject) {
                if (mode === 'ss') {
                    // Secure Storage mode
                    ss.remove(resolve, reject, key);
                } else {
                    // Local Storage mode
                    LocalStorageService.removeKey(key);
                }
                resolve();
            });
        }

        service.setup = function () {
            return $q(function (resolve, reject) {
                if (!window.cordova || !window.cordova.plugins.SecureStorage || DeviceInfoService.isVirtual()) {
                    mode = 'ls';
                    resolve();
                } else {
                    ss = new window.cordova.plugins.SecureStorage(function () {
                        resolve();
                    }, function (error) {
                        mode = 'ls';
                        resolve();
                    }, 'MyPTHub');
                }
            });
        };

        service.saveToken = function (token) {
            return save('token', token);
        };

        service.getToken = function () {
            return get('token');
        };

        service.saveUser = function (user) {
            return save('user', user);
        };

        service.getUser = function () {
            return get('user');
        };

        service.reset = function () {
            remove('token');
            remove('user');
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('CryptographyService', service);

    /*
        This is an injectable wrapper for CryptoJS.
     */

    function service() {

        return {

            md5: function (value) {
                return CryptoJS.MD5(value).toString();
            },

            AESEncrypt: function (payload, key) {

                var encrypted = CryptoJS.AES.encrypt(payload, key);

                return encrypted.toString();
            },

            AESDecrypt: function (payload, key) {

                var decrypted = CryptoJS.AES.decrypt(payload, key);

                return decrypted.toString(CryptoJS.enc.Utf8);
            }

        };
    }
})();
(() => {
  angular.module('MyPTHub').factory('CurrencyService', service);

  function service() {
    const currencies = {
      USD: {
        currency: 'USD',
        symbol: '$',
        dollar: true
      },
      CAD: {
        currency: 'CAD',
        symbol: '$',
        dollar: true
      },
      AUD: {
        currency: 'AUD',
        symbol: '$',
        dollar: true
      },
      EUR: {
        currency: 'EUR',
        symbol: '€',
        dollar: false
      },
      GBP: {
        currency: 'GBP',
        symbol: '£',
        dollar: false
      }
    };

    const service = {};

    service.getCurrencies = () => currencies;

    service.getCurrency = currencyName => {
      const name = currencyName.toUpperCase();

      if (!currencies.hasOwnProperty(name)) {
        return false;
      }

      return currencies[name];
    };

    return service;
  }
})();
(function () {
  angular.module('MyPTHub').factory('CustomerSwitcherService', service).component('customerSwitcher', {
    template: '' + '<a class="item item-avatar item-icon-right" ' + 'ng-click="vm.openChooserModal()" ' + 'ng-if="vm.currentlySelected">' + '<img avatar filename="vm.currentlySelected.photo" size="small" /> ' + '<h2>{{ vm.currentlySelected.isCurrentUser ? \'Your Calendar\' : vm.currentlySelected.name }}</h2> ' + '<p>Viewing ' + '{{ vm.currentlySelected.isCurrentUser ? \'your\' : vm.currentlySelected.first_name + \'\\\'s\' }} ' + 'calendar</p>' + '<i class="icon material-icons">arrow_drop_down</i> ' + '</a>',
    controller,
    controllerAs: 'vm',
    bindings: {
      currentCustomerId: '=?',
      company: '=?',
      trainer: '=?',
      currentTrainerId: '=?'
    }
  });

  service.$inject = ['$rootScope'];

  function service($rootScope) {
    const service = {};

    const fireEventHandlers = function (type, data) {
      data = data || false;
      $rootScope.$emit(`CustomerSwitcherService_${type}`, data);
    };

    const registerEventHandler = function (scope, type, handler) {
      const unsub = $rootScope.$on(`CustomerSwitcherService_${type}`, handler);
      scope.$on('$destroy', unsub);
      return unsub;
    };

    service.switched = function (customer) {
      fireEventHandlers('switched', customer);
    };

    service.onSwitched = function (scope, handler) {
      return registerEventHandler(scope, 'switched', handler);
    };

    return service;
  }

  controller.$inject = ['$scope', 'CustomerStoreService', 'PermissionsService', 'lodash', 'CustomerSwitcherChooserModalService', 'CustomerSwitcherService', 'ApiClientService'];

  function controller($scope, CustomerStoreService, PermissionsService, lodash, CustomerSwitcherChooserModalService, CustomerSwitcherService, ApiClientService) {
    const vm = this;
    const currentUser = PermissionsService.getCurrentUser();

    vm.openChooserModal = function () {
      CustomerSwitcherChooserModalService.open().then(CustomerSwitcherService.switched);
    };

    if (vm.currentCustomerId) {
      const loadCustomers = function () {
        CustomerStoreService.listAllTrainerCustomers().then(customers => {
          const selectedCustomer = lodash.find(customers, customer => customer.id === vm.currentCustomerId);

          if (selectedCustomer) {
            vm.currentlySelected = {
              photo: selectedCustomer.data.user.photo,
              name: `${selectedCustomer.data.user.first_name} ${selectedCustomer.data.user.last_name}`,
              first_name: selectedCustomer.data.user.first_name,
              isCurrentUser: false
            };
          }
        });
      };
      loadCustomers();

      CustomerStoreService.onDownloadComplete($scope, loadCustomers);

      if (CustomerStoreService.isDirty()) {
        CustomerStoreService.downloadTrainerCustomers();
      }
    } else if (vm.currentTrainerId) {
      if (PermissionsService.userIsTrainer()) {
        ApiClientService.trainerUsers.get(vm.currentTrainerId).then(response => {
          vm.currentlySelected = {
            photo: null,
            name: `${response.trainerUser.first_name} ${response.trainerUser.last_name}`,
            first_name: response.trainerUser.first_name,
            isCurrentUser: false,
            type: 'company'
          };
        });
      } else {
        const currentUser = PermissionsService.getCurrentUser();

        vm.currentlySelected = {
          photo: currentUser.trainer_details.photo,
          name: `${currentUser.trainer_details.first_name} ${currentUser.trainer_details.last_name}`,
          first_name: currentUser.trainer_details.first_name,
          isCurrentUser: false
        };
      }
    } else if (vm.company) {
      vm.currentlySelected = {
        photo: null,
        name: 'Company Calendar',
        first_name: currentUser.branding.business_name,
        isCurrentUser: false,
        type: 'company'
      };
    } else if (vm.trainer && !PermissionsService.userIsTrainer()) {
      vm.currentlySelected = {
        photo: currentUser.trainer_details.photo,
        name: 'Trainer Calendar',
        first_name: currentUser.trainer_details.first_name,
        isCurrentUser: false
      };
    } else {
      vm.currentlySelected = {
        photo: currentUser.photo,
        name: `${currentUser.first_name} ${currentUser.last_name}`,
        first_name: currentUser.first_name,
        isCurrentUser: true
      };
    }

    if (!vm.currentlySelected) {
      const currentUser = PermissionsService.getCurrentUser();

      vm.currentlySelected = {
        photo: currentUser.photo,
        name: `${currentUser.first_name} ${currentUser.last_name}`,
        first_name: currentUser.first_name,
        isCurrentUser: true
      };
    }
  }
})();
(() => {
  class CustomerSwitcherChooserModalController {
    /**
     * @param parameters
     * @param AnalyticsService
     * @param CustomerStoreService
     * @param PermissionsService
     * @param $scope
     * @param $filter
     * @param ApiClientService
     * @param SubscriptionStateService
     */
    constructor(parameters, AnalyticsService, CustomerStoreService, PermissionsService, $scope, $filter, ApiClientService, SubscriptionStateService) {
      this.parameters = parameters;
      this.AnalyticsService = AnalyticsService;
      this.CustomerStoreService = CustomerStoreService;
      this.PermissionsService = PermissionsService;
      this.$scope = $scope;
      this.$filter = $filter;
      this.ApiClientService = ApiClientService;
      this.SubscriptionStateService = SubscriptionStateService;

      this.ready = false;
      this.users = [];
      this.currentUser = PermissionsService.getCurrentUser();
      this.isTrainer = PermissionsService.userIsTrainer();

      CustomerStoreService.onDownloadComplete($scope, this.loadCustomers);

      if (CustomerStoreService.isDirty()) {
        CustomerStoreService.downloadTrainerCustomers();
      }

      // eslint-disable-next-line consistent-return
      this.$scope.$watch(() => this.searchFilter, newVal => {
        if (typeof newVal === 'undefined') {
          return false;
        }

        if (newVal.length > 0) {
          this.searchFilterActive = true;
          this.filteredUsers = $filter('filter')(this.users, {
            name: newVal
          });
        } else {
          this.searchFilterActive = false;
          this.filteredUsers = this.users;
        }
      });

      if (SubscriptionStateService.hasCompanyStripe()) {
        this.users.push({
          photo: null, // this.currentUser.branding.logo_url,
          name: 'Company Calendar',
          first_name: this.currentUser.branding.business_name,
          isCurrentUser: false,
          type: 'company'
        });
      }

      if (this.isTrainer) {
        this.loadCustomers();
      } else {
        this.loadCustomerDropdown();
      }
      this.AnalyticsService.trackView('Trainer - Customer Calendar Switcher Choose Customer Modal');
    }

    /**
     * @param customer
     */
    choose(customer) {
      this.$scope.closeModal(customer);
    }

    loadCustomerDropdown() {
      this.users.push({
        customer_id: this.currentUser.customer_id,
        photo: this.currentUser.photo,
        name: 'Your Calendar',
        first_name: 'your',
        isCurrentUser: true
      });

      this.users.push({
        photo: this.currentUser.trainer_details.photo,
        name: 'Trainer Calendar',
        first_name: this.currentUser.trainer_details.first_name,
        isCurrentUser: false
      });

      this.filteredUsers = this.users;
    }

    /**
     *
     */
    loadCustomers() {
      this.CustomerStoreService.listAllTrainerCustomers().then(customers => {
        this.users.push({
          photo: this.currentUser.photo,
          name: `Client: ${this.currentUser.first_name} ${this.currentUser.last_name}`,
          first_name: this.currentUser.first_name,
          isCurrentUser: true
        });
        if (this.PermissionsService.userIsTrainerAdmin()) {
          this.ApiClientService.trainerUsers.list(this.currentUser.id).then(resp => {
            const trainers = resp.trainerUsers;
            angular.forEach(trainers, trainer => {
              this.users.push({
                trainer_user_id: trainer.id,
                photo: trainer.photo,
                name: `Trainer: ${trainer.first_name} ${trainer.last_name}`,
                first_name: trainer.first_name,
                isCurrentUser: false
              });
            });

            angular.forEach(customers, customer => {
              this.users.push({
                customer_id: customer.id,
                photo: customer.data.user.photo,
                name: `Client: ${customer.data.user.first_name} ${customer.data.user.last_name}`,
                first_name: customer.data.user.first_name,
                isCurrentUser: false
              });
            });

            if (!this.searchFilterActive) {
              this.filteredUsers = angular.copy(this.users);
            }

            this.noClients = customers.length === 0;
            this.ready = true;
          });
        } else {
          angular.forEach(customers, customer => {
            this.users.push({
              customer_id: customer.id,
              photo: customer.data.user.photo,
              name: `Client: ${customer.data.user.first_name} ${customer.data.user.last_name}`,
              first_name: customer.data.user.first_name,
              isCurrentUser: false
            });
          });

          if (!this.searchFilterActive) {
            this.filteredUsers = angular.copy(this.users);
          }

          this.noClients = customers.length === 0;
          this.ready = true;
        }
      });
    }
  }

  const service = ModalService => ({
    open() {
      return ModalService.show({
        templateUrl: 'core/CustomerSwitcher/CustomerSwitcherChooserModal.html',
        controller: 'CustomerSwitcherChooserModalController as vm',
        params: {}
      });
    }
  });

  angular.module('MyPTHub').factory('CustomerSwitcherChooserModalService', service).controller('CustomerSwitcherChooserModalController', CustomerSwitcherChooserModalController);
})();
(() => {
  // const tpl = '<md-datepicker ng-model="picker.startDate" md-placeholder="From date" md-max-date="todayDate" ng-change="picker.rangeChanged()" md-open-on-focus></md-datepicker> ' +
  //             '<md-datepicker ng-model="picker.endDate" md-placeholder="To date" md-max-date="todayDate" ng-change="picker.rangeChanged()" md-open-on-focus></md-datepicker>';

  const tpl = '  <md-input-container style="margin-left:1.4em;">\n' + '    <label>Start</label>\n' + '    <md-datepicker ng-model="picker.startDate" md-placeholder="From date" md-max-date="picker.today" ng-change="picker.rangeChanged()" md-open-on-focus md-hide-icons="calendar"></md-datepicker>\n' + '  </md-input-container>\n' + '  <md-input-container style="margin-left:2.2em;">\n' + '    <label>End</label>\n' + '    <md-datepicker ng-model="picker.endDate" md-placeholder="To date" md-max-date="picker.today" ng-change="picker.rangeChanged()" md-open-on-focus md-hide-icons="calendar">\n' + '    </md-datepicker>\n' + '  </md-input-container>';

  angular.module('MyPTHub').factory('DateRangeService', service).component('dateRange', {
    template: tpl,
    controller,
    controllerAs: 'picker'
  });

  function controller(DateRangeService, $rootScope) {
    const picker = this;

    picker.startDate = DateRangeService.getStartDate().toDate();
    picker.endDate = DateRangeService.getEndDate().toDate();
    picker.today = new Date();

    $rootScope.$on('addReceipt', (event, value) => {
      picker.startDate = DateRangeService.getStartDate().toDate();
      picker.endDate = DateRangeService.getEndDate().toDate();
    });

    picker.rangeChanged = function () {
      DateRangeService.setRange(moment(picker.startDate).startOf('day').utc(), moment(picker.endDate).endOf('day').utc());
    };
  }

  function service($rootScope) {
    const service = {};

    let startDate, endDate;

    service.reset = function () {
      startDate = moment().subtract(6, 'days').startOf('day').utc();
      endDate = moment().endOf('day').utc();
    };

    service.reset();

    service.setRange = function (start, end) {
      startDate = start;
      endDate = end;

      $rootScope.$emit('dateRangeChanged', startDate, endDate);
    };

    service.setStartDate = function (date) {
      startDate = date;

      $rootScope.$emit('dateRangeChanged', startDate, endDate);
    };

    service.getStartDate = function () {
      return startDate;
    };

    service.getStartDateApiFormatted = function () {
      return startDate.format('YYYY-MM-DD HH:mm:ss');
    };

    service.setEndDate = function (date) {
      endDate = date;

      $rootScope.$emit('dateRangeChanged', startDate, endDate);
    };

    service.getEndDate = function () {
      return endDate;
    };

    service.getEndDateApiFormatted = function () {
      return endDate.format('YYYY-MM-DD HH:mm:ss');
    };

    service.registerRangeChangedHandler = function (scope, callback) {
      const unsub = $rootScope.$on('dateRangeChanged', callback);

      scope.$on('$destroy', unsub);

      return unsub;
    };

    return service;
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).directive('dayPicker', directive);

    config.$inject = ['$translateProvider'];

    function config($translateProvider) {

        $translateProvider.translations('en', {
            JANUARY: 'January',
            FEBRUARY: 'February',
            MARCH: 'March',
            APRIL: 'April',
            MAI: 'May',
            JUNE: 'June',
            JULY: 'July',
            AUGUST: 'August',
            SEPTEMBER: 'September',
            OCTOBER: 'October',
            NOVEMBER: 'November',
            DECEMBER: 'December',

            SUNDAY: 'Sunday',
            MONDAY: 'Monday',
            TUESDAY: 'Tuesday',
            WEDNESDAY: 'Wednesday',
            THURSDAY: 'Thurday',
            FRIDAY: 'Friday',
            SATURDAY: 'Saturday'
        });
    }

    function directive() {
        return {
            restrict: 'AE',
            template: '<div class="scheduleDayPicker"> ' + '<div class="scheduleDayPickerBackdrop" ng-if="dayPickerCtrl.open" ng-click="dayPickerCtrl.close()"></div> ' + '<div class="pickerContainer" ng-if="dayPickerCtrl.open"> ' + '<flex-calendar options="dayPickerCtrl.options" events="dayPickerCtrl.events"></flex-calendar> ' + '</div> ' + '</div>',
            replace: true,
            controller: controller,
            controllerAs: 'dayPickerCtrl'
        };
    }

    controller.$inject = ['$scope', 'DayPickerService'];

    function controller($scope, DayPickerService) {

        var dateStart, dateEnd;

        var setDateRange = function (month, year) {

            month = month || false;
            year = year || false;

            if (month && year) {

                dateStart = moment().month(month).year(year).startOf('month');
                dateEnd = moment().month(month).year(year).endOf('month');
            } else {

                var currentSelection = DayPickerService.getDate();

                dateStart = moment(currentSelection).startOf('month');
                dateEnd = moment(currentSelection).endOf('month');
            }
        };

        var dayPickerCtrl = this;

        dayPickerCtrl.events = [];

        dayPickerCtrl.open = false;

        DayPickerService.setup($scope);

        DayPickerService.registerOpen(function () {

            setDateRange();

            dayPickerCtrl.open = true;
        });

        DayPickerService.registerClose(function (apply) {

            apply = apply || false;

            dayPickerCtrl.open = false;

            // Hardware back button fix
            if (apply) {
                $scope.$digest();
            }
        });

        dayPickerCtrl.close = DayPickerService.close;

        dayPickerCtrl.options = {
            dayNamesLength: 1, // 1 for "M", 2 for "Mo", 3 for "Mon"; 9 will show full day names. Default is 1.
            defaultDate: DayPickerService.getDate().format('YYYY-MM-DD'),
            mondayIsFirstDay: true, //set monday as first day of week. Default is false
            eventClick: function (date) {// called before dateClick and only if clicked day has events

            },
            dateClick: function (day) {

                DayPickerService.setDate(moment(day.date).hour(12));

                DayPickerService.close();
            },
            changeMonth: function (month, year) {

                var monthNo = month.index;

                setDateRange(monthNo, year);
            },
            filteredEventsChange: function (filteredEvents) {}
        };

        /*[
         {foo: 'bar', eventClass: 'expired', date: "2015-08-18"}, //value of eventClass will be added to CSS class of the day element
         {foo: 'bar', date: "2015-08-20"}
         ];*/
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('DayPickerService', service);

    service.$inject = ['$rootScope'];

    function service($rootScope) {

        var service = {};

        var currentScope;

        var closeEvent, openEvent;

        var isOpen = false;

        var chosenDate = moment();

        var fireDateChangeEventHandlers = function () {

            $rootScope.$emit('dayPickerDatePicked');
        };

        service.setup = function (scope) {

            currentScope = scope;

            currentScope.$on('$ionicView.beforeEnter', function () {
                service.close();
            });

            currentScope.$on('$ionicView.leave', function () {
                service.close();
            });
        };

        service.isOpen = function () {
            return isOpen;
        };

        service.open = function () {

            isOpen = true;
            openEvent();
        };

        service.close = function (apply) {

            apply = apply || false;

            isOpen = false;
            closeEvent(apply);
        };

        service.toggleOpen = function () {

            if (isOpen) {
                service.close();
            } else {
                service.open();
            }
        };

        service.registerOpen = function (func) {
            openEvent = func;
        };

        service.registerClose = function (func) {
            closeEvent = func;
        };

        service.setDate = function (dateObj, noEvent) {
            noEvent = noEvent || false;
            chosenDate = dateObj;
            if (noEvent === false) {
                fireDateChangeEventHandlers();
            }
        };

        service.reset = function () {
            chosenDate = moment();
            fireDateChangeEventHandlers();
        };

        service.getDate = function () {
            return chosenDate;
        };

        service.registerDayPickedEventHandler = function (scope, callback) {

            var unsub = $rootScope.$on('dayPickerDatePicked', callback);

            scope.$on('$destroy', unsub);

            return unsub;
        };

        return service;
    }
})();
(() => {
  angular.module('MyPTHub').factory('DayScheduleService', service).component('daySchedule', {
    template: '<div ui-calendar="vm.calendarConfig" ' + 'ng-model="vm.eventSources" ' + 'id="dayScheduleCalendar" ' + 'calendar="dayScheduleCalendar"' + 'class="daySchedule"></div>',
    controller,
    controllerAs: 'vm'
  });

  service.$inject = ['$rootScope'];

  function service($rootScope) {
    let sessions = [];
    let schedulerEvents = [];
    let events = [];
    let date = moment();

    const fireEventHandlers = function (type, data) {
      data = data || false;
      $rootScope.$emit(`DayScheduleService_${type}`, data);
    };

    const registerEventHandler = function (scope, type, handler) {
      const unsub = $rootScope.$on(`DayScheduleService_${type}`, handler);
      scope.$on('$destroy', unsub);
      return unsub;
    };

    const service = {};

    service.events = [];
    service.sessions = [];
    service.schedulerEvents = [];

    service.reset = function () {
      if (sessions) {
        sessions.length = 0;
      }
      if (schedulerEvents) {
        schedulerEvents.length = 0;
      }
      if (events) {
        events.length = 0;
      }
      fireEventHandlers('reset');
    };

    service.onReset = function (scope, handler) {
      registerEventHandler(scope, 'reset', handler);
    };

    service.setSessions = function (sessionsData) {
      sessions = sessionsData;
      fireEventHandlers('set_sessions', sessionsData);
    };

    service.setEvents = eventsData => {
      events = eventsData;
      fireEventHandlers('set_events', eventsData);
    };

    service.getSessions = function () {
      return sessions;
    };

    service.onSetSessions = function (scope, handler) {
      registerEventHandler(scope, 'set_sessions', handler);
    };

    service.onSetEvents = function (scope, handler) {
      registerEventHandler(scope, 'set_events', handler);
    };

    service.setSchedulerEvents = function (events) {
      schedulerEvents = events;
      fireEventHandlers('scheduler_events', events);
    };

    service.getSchedulerEvents = function () {
      return schedulerEvents;
    };

    service.onSetSchedulerEvents = function (scope, handler) {
      registerEventHandler(scope, 'scheduler_events', handler);
    };

    service.setCalendarEvents = function (data) {
      sessions = data[0];
      schedulerEvents = data[1];
      console.log('setCalendarEvents', data);
      fireEventHandlers('calendar_events', data);
    };

    service.getCalendarEvents = function () {
      return [schedulerEvents, sessions, events];
    };

    service.onSetCalendarEvents = function (scope, handler) {
      registerEventHandler(scope, 'calendar_events', handler);
    };

    service.setDate = function (newDate) {
      date = newDate;
      fireEventHandlers('set_date', newDate);
    };

    service.getDate = function () {
      return date;
    };

    service.onSetDate = function (scope, handler) {
      registerEventHandler(scope, 'set_date', handler);
    };

    service.loaded = function () {
      fireEventHandlers('loaded');
    };

    service.onLoaded = function (scope, handler) {
      registerEventHandler(scope, 'loaded', handler);
    };

    return service;
  }

  controller.$inject = ['$scope', 'NavigateService', 'uiCalendarConfig', 'PermissionsService', 'DayScheduleService', 'SchedulerService', '$timeout', 'NutritionLogService'];

  function controller($scope, NavigateService, uiCalendarConfig, PermissionsService, DayScheduleService, SchedulerService, $timeout, NutritionLogService) {
    const vm = this;

    const sessions = [];
    const schedulerEvents = [];
    const events = [];

    vm.eventSources = [{
      events: sessions,
      className: 'sessions'
    }, {
      events: schedulerEvents,
      className: 'schedulerEvents'
    }, {
      events,
      className: 'bookedEvents'
    }];

    const mergeEvents = function () {
      /*
       If the scheduled workout is done, the session should be removed
       and the scheduled workout item updated to match the session.
       */
      angular.forEach(schedulerEvents, scheduledEvent => {
        angular.forEach(sessions, session => {
          if ((scheduledEvent.type === 'workout_am' || scheduledEvent.type === 'workout_pm') && session.data.schedule_type && session.data.schedule_id && scheduledEvent.completed && scheduledEvent.data.session_id === session.data.id) {
            scheduledEvent.start = moment.utc(session.data.date_completed, 'YYYY-MM-DD HH:mm:ss').local();
            sessions.splice(sessions.indexOf(session), 1);
          }
        });
      });
    };

    const processEvents = function (eventsResponse) {
      events.length = 0;
      eventsResponse.forEach(event => events.push(event));
    };

    const processSessions = function (sessionsResponse) {
      sessions.length = 0;
      console.log('Processing Sessions', sessionsResponse);
      const currentUser = PermissionsService.getCurrentUser();

      const addLocation = location => location ? ` (${location})` : '';
      if (PermissionsService.userIsTrainer()) {
        angular.forEach(sessionsResponse, session => {
          let sessionTitle = 'Session';
          if (session.data.workout) {
            sessionTitle = session.data.workout.title + addLocation(session.data.location);
          }

          if (session.data.cancellation_request) {
            sessionTitle += ' (Cancellation Requested)';
          }

          session.start = moment.utc(session.data.created_at).isAfter('2018-07-31') ? moment.tz(moment.utc(session.data.date), currentUser.timezone) : session.data.date;

          if (session.data.customer) {
            session.title = `${session.data.customer.user.first_name} ${session.data.customer.user.last_name} - ${sessionTitle}`;
          } else {
            session.title = sessionTitle;
          }
          session.allDay = false;
          session.instanceType = 'session';
          session.viewType = session.data.viewType;
          session.end = session.data.end;
          session.date = session.data.start;

          // Detect if this is a phantom session
          if (session.data.hasOwnProperty('phantom') && session.data.hasOwnProperty('parent_session_id')) {
            session.url = `/sessions/${session.data.parent_session_id}/occurrence/${session.date}`;
          } else if (session.data.status === 'live') {
            session.url = `/sessions/${session.data.id}`;
          } else {
            session.url = `/sessions/${session.data.id}/logged`;
          }

          sessions.push(angular.copy(session));
        });
      } else {
        angular.forEach(sessionsResponse, session => {
          let sessionTitle = 'Session';
          if (session.data.workout) {
            sessionTitle = session.data.workout.title + addLocation(session.data.location);
          }

          if (session.data.cancellation_request) {
            sessionTitle += ' (Cancellation Requested)';
          }

          session.start = moment.utc(session.data.created_at).isAfter('2018-07-31') ? moment.tz(moment.utc(session.data.date), currentUser.timezone) : session.data.date;
          session.title = sessionTitle;
          session.allDay = false;
          session.instanceType = 'session';

          sessions.push(angular.copy(session));
        });
      }
    };

    const processSchedulerEvents = function (events) {
      schedulerEvents.length = 0;

      angular.forEach(events, event => {
        let validEvent = false;

        switch (event.type) {
          case 'nutrition':
            event.title = event.data.title ? event.data.title : 'Log Nutrition';
            event.allDay = true;
            event.date = moment(event.date).toDate();
            validEvent = true;
            break;
          case 'update_metrics':
            event.title = 'Update Measurements';
            event.allDay = true;
            event.date = moment(event.date).toDate();
            event.completed = false;
            validEvent = true;
            break;
          case 'update_photo':
            event.title = 'Update Progress Photo';
            event.allDay = true;
            event.date = moment(event.date).toDate();
            event.completed = false;
            validEvent = true;
            break;
          case 'workout_am':
            event.title = event.data.title;
            event.date = moment(event.date).hour(event.data.preferred_hour).minute(event.data.preferred_minute).toDate();
            event.completed = false;
            validEvent = true;
            break;
          case 'workout_pm':
            event.title = event.data.title;
            event.date = moment(event.date).hour(event.data.preferred_hour).minute(event.data.preferred_minute).toDate();
            event.completed = false;
            validEvent = true;
            break;
        }

        if (event.hasOwnProperty('data')) {
          event.completed = event.data.completed;
        }

        event.instanceType = 'scheduled_item';

        schedulerEvents.push(angular.copy(event));
      });
    };

    const setFCCurrentDate = function (currentDate) {
      if (!uiCalendarConfig.calendars.dayScheduleCalendar || typeof currentDate === 'undefined') {
        return false;
      }
      $timeout(() => {
        uiCalendarConfig.calendars.dayScheduleCalendar.fullCalendar('gotoDate', currentDate);
      }, 100);
      loadedSchedule = false;
    };

    var unwatchCalendarObj = $scope.$watch(() => uiCalendarConfig.calendars.dayScheduleCalendar, response => {
      if (typeof response !== 'undefined') {
        setFCCurrentDate(DayScheduleService.getDate());
        unwatchCalendarObj();
      }
    });

    DayScheduleService.onReset($scope, () => {
      sessions.length = 0;
      schedulerEvents.length = 0;
      events.length = 0;
    });

    DayScheduleService.onSetDate($scope, (event, date) => {
      setFCCurrentDate(date);
    });

    DayScheduleService.onSetSessions($scope, (event, sessionsData) => {
      processSessions(sessionsData);
    });

    DayScheduleService.onSetEvents($scope, (event, eventData) => {
      processEvents(eventData);
    });

    DayScheduleService.onSetSchedulerEvents($scope, (event, schedulerEvents) => {
      processSchedulerEvents(schedulerEvents);
    });

    DayScheduleService.onSetCalendarEvents($scope, (event, data) => {
      if (data.hasOwnProperty('events')) {
        if (typeof data.events !== 'undefined' && data.events.hasOwnProperty('events')) {
          processEvents(data.events.events);
        }
      }
      processSchedulerEvents(data.schedulerEvents);
      processSessions(data.sessions);
      mergeEvents();
    });

    const viewRender = function (view, element) {};

    const eventRender = function (event, element) {
      const eventStart = () => {
        if (event.start) {
          return event.start.format('h:mm a');
        }
      };

      const eventEnd = () => {
        return event.end !== null ? `- ${event.end.format('h:mm a')}` : '';
      };

      const allowed = () => event.viewType === 'session' || event.instanceType === 'scheduled_item' || event.instanceType === 'session';

      const spacesAvailable = () => {
        return allowed() ? '' : `(${event.bookings_count}/${event.spaces_available})`;
      };

      const template = `
        <div class="time">
          <span>${eventStart()} ${eventEnd()}</span>
        </div>
        <div class="fc-title">${event.title} ${spacesAvailable()}</div>
        </div>
        <div class="fc-bg">`;

      element.context.innerHTML = template;

      if (event.data && event.data.cancellation_request) {
        element.addClass('cancellationRequestPending');
      }

      if (event.instanceType === 'scheduled_item') {
        element.addClass(event.type);
      } else if (event.instanceType === 'session') {
        element.addClass('session');
        if (event.data.completed_by === 'customer' && event.data.type === 'instant') {
          element.addClass('client_session');
        } else {
          element.addClass('pt_session');
        }
      }

      if (event.completed) {
        element.append($('<i class="material-icons completed-item">check_circle</i>'));
      }
    };

    const eventClick = function (sessionRow, jsEvent) {
      jsEvent.preventDefault();
      const date = moment.utc(sessionRow.date).local();

      switch (sessionRow.type) {
        case 'nutrition':
          // Date is today or earlier, go to log mode
          if (SchedulerService.dayIsNotInFuture(date)) {
            if (PermissionsService.userIsTrainer()) {
              NavigateService.go('app.trainer_view_nutrition_log_day', {
                customerId: sessionRow.customer_id,
                date: date.format('YYYY-MM-DD')
              });
              return true;
            }

            NutritionLogService.startLogger(date);
            return true;
          }

          // If we got here, the day is in the future, so look at the
          // assigned nutrition plan for the day.
          NavigateService.go('app.view_nutrition_plan', {
            date: date.format('YYYY-MM-DD'),
            nutritionPlanId: sessionRow.data.nutrition_plan_id
          });
          return true;
          break;
        case 'update_metrics':
          if (!SchedulerService.canLogDate(date)) {
            return false;
          }

          if (PermissionsService.userIsTrainer()) {
            NavigateService.go('app.trainer_update_customer_metrics', {
              customerId: sessionRow.customer_id,
              date: date.format('YYYY-MM-DD')
            });
            return true;
          }

          NavigateService.go('app.client_update_metrics');
          return true;
          break;
        case 'update_photo':
          if (!SchedulerService.canLogDate(date)) {
            return false;
          }

          NavigateService.go('app.view_customer_schedule', {
            customerId: sessionRow.customer_id,
            scheduleId: sessionRow.schedule_id,
            trigger: 'update_photo',
            triggerDate: date.format('YYYY-MM-DD')
          });
          return true;
          break;
        case 'workout_am':
        case 'workout_pm':
          if (sessionRow.completed) {
            NavigateService.go('app.session_preview', {
              sessionId: sessionRow.data.session_id
            });
          } else {
            NavigateService.go('app.view_workout_schedule', {
              workoutId: sessionRow.data.workout_id,
              date: date.format('YYYY-MM-DD'),
              scheduleId: sessionRow.schedule_id,
              ampm: sessionRow.type === 'workout_am' ? 'am' : 'pm',
              customerId: sessionRow.customer_id
            });
          }
          return true;
          break;
        default:
          if (sessionRow.instanceType === 'session') {
            if (PermissionsService.userIsTrainer() || sessionRow.data && sessionRow.data.workout) {
              NavigateService.go('app.session_preview', {
                sessionId: sessionRow.id
              });
              return true;
            }
          } else {
            NavigateService.go('app.trainer_calendar_event_booking', {
              eventId: sessionRow.event_id,
              eventTimeId: sessionRow.event_time_id,
              date_time: moment.utc(sessionRow.original_date).format('YYYY-MM-DD HH:mm:ss')
            });
          }
      }

      return false;
    };

    vm.calendarConfig = {
      editable: false,
      header: false,
      defaultView: 'agendaDay',
      height: 'auto',
      viewRender,
      eventRender,
      timeFormat: 'HH:mm',
      ignoreTimezone: false,
      // allDaySlot: false,
      nowIndicator: true,
      eventClick,
      defaultTimedEventDuration: '01:00:00'
    };
  }
})();
/*
    Allows push notifications etc to deeplink to states behind the login, by
    adding a post-login hook.
 */
(function () {
    'use strict';

    angular.module('MyPTHub').factory('DeeplinkService', service);

    service.$inject = ['NavigateService', 'AuthenticationStateService', 'AuthenticationService'];

    function service(NavigateService, AuthenticationStateService, AuthenticationService) {

        var pendingState = false;

        var service = {};

        service.setup = function () {

            AuthenticationService.registerLoginSuccessEventHandler(function (resolve) {

                if (service.hasPendingState()) {
                    service.goToPendingState();
                }

                resolve(true);
            });

            if (AuthenticationStateService.isLoggedIn()) {
                if (service.hasPendingState()) {
                    service.goToPendingState();
                }
            }
        };

        service.go = function (state, params) {

            pendingState = false;

            console.log('go to state', state);

            if (AuthenticationStateService.isLoggedIn()) {

                NavigateService.go(state, params);

                service.resetState();
            } else {

                pendingState = state;
            }
        };

        service.hasPendingState = function () {
            return pendingState !== false;
        };

        service.goToPendingState = function () {

            if (!pendingState) {
                return false;
            }

            service.resetState();

            NavigateService.go(pendingState.name, pendingState.params);
        };

        service.resetState = function () {
            pendingState = false;
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('DeviceInfoService', service);

    service.$inject = ['$q'];

    function service($q) {

        return {

            platform: function () {

                var platform = ionic.Platform.platform();

                if (!platform) {
                    return 'browser';
                }

                if (!ionic.Platform.isWebView()) {
                    return 'browser';
                }

                return platform.toLowerCase();
            },

            version: function () {
                return ionic.Platform.version();
            },

            isBrowser: function () {
                return this.platform() === 'browser';
            },

            isIos: function () {
                return this.platform() === 'ios';
            },

            isAndroid: function () {
                return this.platform() === 'android';
            },

            isVirtual: function () {
                return typeof device !== 'undefined' && device.isVirtual;
            },

            appVersion: function () {

                if (typeof cordova === 'undefined' || !cordova) {
                    return $q.resolve('xxx');
                }

                return cordova.getAppVersion.getVersionNumber();
            },

            bundleId: function () {

                if (typeof cordova === 'undefined' || !cordova) {
                    return $q.resolve(null);
                }

                return cordova.getAppVersion.getPackageName();
            }

        };
    }
})();
// (function () {
//   'use strict';
//
//   angular
//     .module('MyPTHub')
//     .factory('AirbrakeService', service)
//     .factory('$exceptionHandler', handler);
//
//   service.$inject = ['mpthconf'];
//
//   function service(mpthconf) {
//
//     var airbrake = new airbrakeJs.Client({
//       projectId: mpthconf.airbrake.project_id,       // Airbrake project id
//       projectKey: mpthconf.airbrake.project_key, // Airbrake project API key
//     });
//
//     airbrake.addFilter(function (notice) {
//       notice.context.environment = mpthconf.env;
//       return notice;
//     });
//
//     var service = {};
//
//     service.get = function () {
//       return airbrake;
//     };
//
//     return service;
//   }
//
//   handler.$inject = ['$log', 'AirbrakeService', 'mpthconf'];
//
//   function handler($log, AirbrakeService, mpthconf) {
//
//     return function (exception, cause) {
//       $log.error(exception);
//       $log.error(cause);
//       //
//       // if (exception.message.startsWith('[$rootScope:inprog]')) {
//       //   return;
//       // }
//       //
//       // if (exception.message.startsWith('$rootScope:inprog')) {
//       //   return;
//       // }
//       //
//       // if (exception.message.startsWith('TypeError: Cannot read property \'classList\' of null')) {
//       //   return;
//       // }
//       //
//       // return;
//
//       // if (mpthconf.env === 'production') {
//       //   var airbrake = AirbrakeService.get();
//       //
//       //   airbrake.notify({
//       //     error: exception,
//       //     params: {
//       //       angular_cause: cause,
//       //     },
//       //   });
//       // }
//     };
//   }
//
// })();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('extSubmit', directive);

    directive.$inject = ['$timeout'];

    function directive($timeout) {
        return {
            link: function ($scope, $el, $attr) {
                $scope.$on('makeSubmit', function (event, data) {
                    console.log('makeSubmit rec', data);
                    console.log(data.formName, $attr.name);
                    if (data.formName === $attr.name) {
                        $timeout(function () {
                            $el.triggerHandler('submit'); //<<< This is Important
                            //$el[0].dispatchEvent(new Event('submit')) //equivalent with native event
                        }, 0, false);
                    }
                });
            }
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('fabSpeedDialController', controller);

    controller.$inject = ['$timeout', '$scope', 'NavigateService', '$q', '$rootScope'];

    function controller($timeout, $scope, NavigateService, $q, $rootScope) {

        var fsd = this;

        fsd.isOpen = false;
        fsd.showLabels = false;
        fsd.enabled = false;

        $scope.$watch(function () {
            return fsd.isOpen;
        }, function (isOpen) {

            if (cancel && !fsd.isOpen) {
                cancel();
            }

            var cancel = $timeout(function () {
                fsd.showLabels = fsd.isOpen;
            }, isOpen ? 500 : 0);
        });

        fsd.go = function (state, params) {

            params = params || {};
            state = state || false;

            fsd.showLabels = false;

            if (state) {
                NavigateService.go(state, params);
            }

            return $q.when();
        };

        $scope.$parent.$on('$ionicView.enter', function () {
            fsd.enabled = true;
        });

        $scope.$parent.$on('$ionicView.leave', function () {
            fsd.showLabels = false;
            fsd.isOpen = false;
            fsd.enabled = false;
        });

        // for use in a modal
        $scope.$parent.$on('modal.shown', function () {
            fsd.enabled = true;
        });

        $scope.$parent.$on('modal.hidden', function () {
            fsd.enabled = false;
            fsd.isOpen = false;
            fsd.showLabels = false;
        });

        $rootScope.$on('closeFAB', function (event, payload) {
            $scope.$apply(function () {
                fsd.isOpen = false;
            });
        });
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').run(init).factory('ImageCacheService', service);

    init.$inject = ['$ionicPlatform'];

    function init($ionicPlatform) {

        ImgCache.options.debug = false;
        ImgCache.options.skipURIencoding = true;
        ImgCache.options.chromeQuota = 50 * 1024 * 1024;

        $ionicPlatform.ready(function () {
            ImgCache.init();
        });
    }

    service.$inject = ['$q', 'DeviceInfoService'];

    function service($q, DeviceInfoService) {

        var cacheEnabled = true;

        if (DeviceInfoService.isBrowser()) {
            cacheEnabled = false;
        }

        function cacheFile(src) {

            return $q(function (resolve, reject) {
                ImgCache.cacheFile(src, resolve, reject);
            });
        }

        function isCached(src) {

            var defer = $q.defer();

            ImgCache.isCached(src, function (path, success) {

                if (success) {

                    defer.resolve(path);
                } else {

                    defer.reject();
                }
            }, defer.reject);

            return defer.promise;
        }

        function getFromCache(src) {

            return $q(function (resolve, reject) {
                ImgCache.getCachedFileURL(src, function (originalUrl, cachedUrl) {
                    resolve(cachedUrl);
                }, reject);
            });
        }

        return {

            get: function (uri) {

                if (!cacheEnabled) {
                    return $q.resolve(uri);
                }

                var defer = $q.defer();

                isCached(uri).then(function () {
                    getFromCache(uri).then(defer.resolve, defer.reject);
                }, function () {

                    cacheFile(uri).then(function () {
                        getFromCache(uri).then(defer.resolve, defer.reject);
                    }, defer.reject);
                });

                return defer.promise;
            }

        };
    }
})();
(() => {
  angular.module('MyPTHub').filter('bytes', filter);

  function filter() {
    const units = ['bytes', 'KB', 'MB', 'GB', 'TB', 'PB'];

    return (bytesParam, precision) => {
      let bytes = bytesParam;
      if (isNaN(parseFloat(bytes)) || !isFinite(bytes)) {
        return '?';
      }

      let unit = 0;

      while (bytes >= 1024) {
        bytes /= 1024;
        unit += 1;
      }

      return `${bytes.toFixed(+precision)} ${units[unit]}`;
    };
  }
})();

(function () {
    'use strict';

    angular.module('MyPTHub').filter('declbs', filter);

    filter.$inject = [];

    function filter() {

        return function (number, active) {

            /*
             * Converts a decimal representation of lbs/oz into one that makes sense
             * for output.
             *
             * e.g. 1.81 becomes 1lb 13oz
             *
             * Split the number at the decimal point, [0] is the lbs, [1]*16 (rounded)
             * is the oz.
             */

            if (!active) {
                return number;
            }

            var no = (number + "").split(".");

            var lbs = parseInt(no[0]);
            var oz = 0;

            if (no[1]) {
                oz = Math.round(parseFloat('0.' + no[1]) * 16);
            }

            var out = lbs;
            if (oz) {
                out += '.' + oz;
            }

            return out;
        };
    }
})();

(function () {
    'use strict';

    angular.module('MyPTHub').filter('exerciseState', filter);

    filter.$inject = [];

    function filter() {

        return function (input) {
            input = parseInt(input) || 1;
            var output = input;

            switch (input) {
                case 1:
                    output = 'Easy';
                    break;
                case 2:
                    output = 'Medium';
                    break;
                case 3:
                    output = 'Hard';
                    break;
            }

            return output;
        };
    }
})();

(function () {
    'use strict';

    angular.module('MyPTHub').filter('momentdate', filter);

    filter.$inject = [];

    function filter() {

        return function (strdate, format, strdateIsUTC) {

            strdateIsUTC = typeof strdateIsUTC !== 'undefined' ? strdateIsUTC : true;

            var parseFormat = 'YYYY-MM-DD HH:mm:ss';

            if (strdateIsUTC) {
                var instance = moment.utc(strdate, parseFormat).local();
            } else {
                var instance = moment(strdate, parseFormat);
            }

            return instance.format(format);
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').filter('nl2br', filter);

    filter.$inject = ['$sce'];

    function filter($sce) {

        return function (text) {
            return text ? $sce.trustAsHtml(text.replace(/\n/g, '<br/>')) : '';
        };
    }
})();

(function () {
    'use strict';

    angular.module('MyPTHub').filter('secondsdisplay', filter);

    filter.$inject = [];

    function filter() {

        return function (seconds) {

            if (seconds < 60) {
                seconds += 's';
            } else {
                var minutes = Math.floor(seconds / 60);
                var seconds = seconds - minutes * 60;
                seconds = minutes + 'm' + (seconds > 0 ? ' ' + seconds + 's' : '');
            }

            return seconds;
        };
    }
})();

(function () {
    'use strict';

    angular.module('MyPTHub').filter('secs2minsecs', filter);

    filter.$inject = [];

    function filter() {

        return function (input, type) {
            switch (type.toUpperCase()) {
                case 'M':
                    return Math.floor(input / 60);
                default:
                    var secs = (input % 60).toString();
                    if (secs.length == 1) secs = '0' + secs;
                    return secs;
            }
        };
    }
})();
(() => {
  'use strict';

  angular.module('MyPTHub').filter('setcoldisplayname', filter);

  function filter() {

    return function (colname, type = 'long', override = false) {
      const names = {
        'short': {
          'weight': 'Weight',
          'reps': 'Reps',
          'rep_range': override ? 'Reps' : 'Rep Range',
          'rest_period': 'Rest',
          'time': 'Time',
          'heart_rate': 'HR',
          'distance': 'Distance',
          'resistance': 'Resistance',
          'speed': 'Speed',
          'incline': 'Incline',
          'kcals': 'kcals',
          'rpe': 'RPE',
          'intensity': 'Intensity',
          'sets': 'Set',
          'bodyweight': 'Bodyweight',
          'plate_level': 'Plate / Level',
          'state': 'State',
          steps: 'Steps',
          tempo: 'Tempo',
          amrap: override ? 'Reps' : 'AMRAP',
          'n/a': 'n/a'
        },
        'long': {
          'weight': 'Weight',
          'reps': 'Reps',
          'rep_range': override ? 'Reps' : 'Rep Range',
          'rest_period': 'Rest Time',
          'time': 'Time',
          'heart_rate': 'Heart Rate',
          'distance': 'Distance',
          'resistance': 'Resistance',
          'speed': 'Speed',
          'incline': 'Incline',
          'kcals': 'kcals',
          'rpe': 'RPE',
          'intensity': 'Percentage of Maximum Intensity',
          'sets': 'Sets',
          'bodyweight': 'Bodyweight',
          'plate_level': 'Plate / Level',
          'state': 'State',
          steps: 'Steps',
          tempo: 'Tempo',
          amrap: override ? 'Reps' : 'AMRAP',
          'n/a': 'n/a'
        },
        'set_editor': {
          'weight': 'Weight',
          'reps': 'Reps',
          'rep_range': override ? 'Reps' : 'Rep Range',
          'rest_period': 'Rest Period',
          'time': 'Time',
          'heart_rate': 'Heart Rate',
          'distance': 'Distance (km)',
          'resistance': 'Resistance',
          'speed': 'Speed',
          'incline': 'Incline',
          'kcals': 'kcals',
          'rpe': 'RPE',
          'intensity': 'Percentage of Max Intensity',
          'sets': 'Sets',
          'bodyweight': 'Bodyweight',
          'plate_level': 'Plate / Level',
          'state': 'State',
          steps: 'Steps',
          tempo: 'Tempo',
          amrap: override ? 'Reps' : 'AMRAP',
          'n/a': 'n/a'
        },
        'logger': {
          'weight': 'Weight',
          'reps': 'Reps',
          'rep_range': override ? 'Reps' : 'Rep Range',
          'rest_period': 'Rest',
          'time': 'Time',
          'heart_rate': 'HR',
          'distance': 'Distance',
          'resistance': 'Res.',
          'speed': 'Speed',
          'incline': 'Incline',
          'kcals': 'kcals',
          'rpe': 'RPE',
          'intensity': '%',
          'sets': '#',
          'bodyweight': 'Bodyweight',
          'plate_level': 'Level',
          'state': 'State',
          steps: 'Steps',
          tempo: 'Tempo',
          amrap: override ? 'Reps' : 'AMRAP',
          'n/a': 'n/a'
        }
      };

      return names[type][colname];
    };
  }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').filter('setValueDisplay', filter);

  filter.$inject = ['$filter'];

  function filter($filter) {

    var handlers = {

      state: function (value, label) {

        var states = {
          1: 'Easy', 2: 'Medium', 3: 'Hard'
        };

        return states[value];
      },

      speed: function (value, label) {

        return value + (label ? ' ' + label : '');
      },

      heart_rate: function (value, label) {

        return value + ' BPM';
      },

      kcals: function (value, label) {

        return value + ' Kcal';
      },

      steps: function (value, label) {
        return value;
      },

      rpe: function (value, label) {

        return value; // + ' RPE';
      },

      intensity: function (value, label) {

        return value + '%';
      },

      weight: function (value, label) {

        if (value === null) {
          value = 0;
        }

        if (label === 'lbs') {

          // Decimal lbs to lbs
          var converted = $filter('declbs')(value, true);

          // Break apart at the decimal point
          var convertedSplit = (converted + '').split('.');

          var output = convertedSplit[0] + ' lbs';

          if (convertedSplit[1]) {
            output += ' ' + convertedSplit[1] + ' oz';
          }

          return output;
        }

        return value + ' kg';
      },

      time: function (value, label) {

        return $filter('secondsdisplay')(value);
      },

      rest_period: function (value, label) {

        return $filter('secondsdisplay')(value);
      },

      distance: function (value, label) {

        return value + (label ? ' ' + label : '');
      },

      bodyweight: function (value, label) {

        return 'Bodyweight';
      },

      reps: function (value, label, view) {
        if (label && view) {
          return `${value} - ${label}`;
        }

        return value;
      },

      rep_range: function (value, label, view) {
        if (!view) {
          return value;
        }
        const values = value.toString().split('.');
        const min = parseInt(values[0], 10) || null;
        const max = parseInt(values[1], 10) || null;
        if (!min || !max) {
          return '';
        }
        return min + ' - ' + max;
      },

      default: function (value, label) {

        return value;
      }

    };

    return function (value, label, type, view = false) {
      if (value === null) {
        value = 0;
      }

      var handler;

      if (handlers.hasOwnProperty(type)) {
        handler = handlers[type];
      } else {
        handler = handlers['default'];
      }

      return handler(value, label, view);
    };
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('ngFormCommit', directive);

    directive.$inject = ['$timeout'];

    function directive($timeout) {
        return {
            require: "form",
            link: function ($scope, $el, $attr, $form) {
                $form.commit = function () {
                    $timeout(function () {
                        $el[0].dispatchEvent(new Event('submit'));
                    }, 0, false);
                };
            }
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('HttpRequestService', service);

    service.$inject = ['$http', 'mpthconf', 'AuthenticationStateService', 'NavigateService'];

    function service($http, mpthconf, AuthenticationStateService, NavigateService) {

        let authEnabled = true;

        const apiBase = mpthconf[mpthconf.env].api.current;

        const incorrectCredentialsCallbackDebounced = _.debounce(function (e) {
            NavigateService.go('app.logout');
        }, 2000);

        const request = function (methodUrl, data, requestType, cancelPromise) {

            data = data || {};
            requestType = requestType || 'GET';
            cancelPromise = cancelPromise || false;

            const req = {
                method: requestType,
                url: `${apiBase}v4/${methodUrl}`,
                headers: setHeaders()
            };

            if (requestType === 'GET') {
                req.params = data;
            } else {
                req.data = data;
            }

            if (cancelPromise) {
                req.timeout = cancelPromise;
            }

            const request = $http(req);

            request.then(function (response) {}, function (response) {
                if (response.data && response.data.error && response.data.error.error_name === "INCORRECT_CREDENTIALS") {
                    incorrectCredentialsCallbackDebounced();
                }
            });

            return request;

            /*
            request.error(function(response, statusCode) {
                 if (response && response.error && response.error.error_name === "INCORRECT_CREDENTIALS") {
                    //Auth.ClearCredentials();
                    //$location.path('/login');
                }
                 if (statusCode === 503) {
                    if (response && response.error && response.error.message) {
                        //SystemNoticeService.setNotice(response.error.message);
                    } else {
                        //SystemNoticeService.setNotice("We are currently offline for system upgrades. Please try again later.");
                    }
                }
             });
             request.success(function(response) {
                if (response.hasOwnProperty('currentUser')) {
                    //Auth.SetCurrentUser(response.currentUser);
                }
                 //SystemNoticeService.clearNotice();
            });
            */
        };

        const requestv5 = function (methodUrl, data, requestType, cancelPromise) {

            data = data || {};
            requestType = requestType || 'GET';
            cancelPromise = cancelPromise || false;

            const req = {
                method: requestType,
                url: `${apiBase}v5/${methodUrl}`,
                headers: setHeaders()
            };

            if (requestType === 'GET') {
                req.params = data;
            } else {
                req.data = data;
            }

            if (cancelPromise) {
                req.timeout = cancelPromise;
            }

            const request = $http(req);

            request.then(function (response) {}, function (response) {
                if (response.data && response.data.error && response.data.error.error_name === "INCORRECT_CREDENTIALS") {
                    incorrectCredentialsCallbackDebounced();
                }
            });

            return request;
        };

        const setHeaders = function () {

            const headers = {};

            if (!authEnabled) {

                // re-enable for next req
                authEnabled = true;

                return headers;
            }

            const token = AuthenticationStateService.getToken();
            if (token) {
                headers['Authorization'] = 'Bearer ' + token;
            }

            return headers;
        };

        return {

            request,
            requestv5,

            noAuth: function () {

                authEnabled = false;

                return this;
            },

            get: function (methodUrl, data, cancelPromise) {
                cancelPromise = cancelPromise || false;
                return request(methodUrl, data, 'GET', cancelPromise);
            },

            post: function (methodUrl, data, cancelPromise) {
                cancelPromise = cancelPromise || false;
                return request(methodUrl, data, 'POST', cancelPromise);
            },

            put: function (methodUrl, data, cancelPromise) {
                cancelPromise = cancelPromise || false;
                return request(methodUrl, data, 'PUT', cancelPromise);
            },

            delete: function (methodUrl, data, cancelPromise) {
                cancelPromise = cancelPromise || false;
                return request(methodUrl, data, 'DELETE', cancelPromise);
            }

        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('imageLoadFade', directive);

    function directive() {
        return {
            restrict: 'A',
            link: link
        };
    }

    var defaultImage = 'img/broken-image.png';

    function link(scope, element, attrs) {

        element.addClass('notLoaded');
        element.addClass('hideUntilLoaded');

        element.bind("load", function (e) {
            element.removeClass('notLoaded');
        });

        element.bind('error', function () {
            attrs.$set('src', defaultImage);
        });
    }
})();
(function () {
    'use strict';

    /**
     * Wrapper for https://github.com/AlexDisler/cordova-plugin-inapppurchase
     */

    angular.module('MyPTHub').factory('InAppPurchaseService', service);

    service.$inject = ['DeviceInfoService', '$q'];

    function service(DeviceInfoService, $q) {

        var enabled = false;

        var service = {};

        service.setup = function () {
            if (DeviceInfoService.isIos() && window.inAppPurchase) {
                enabled = true;
            } else {
                console.log('InAppPurchaseService not enabled - not available');
            }
        };

        service.getProducts = function (productIds) {
            if (!enabled) {
                return $q.reject('IAP not available.');
            }

            return inAppPurchase.getProducts(productIds);
        };

        service.subscribe = function (productId) {
            if (!enabled) {
                return $q.reject('IAP not available.');
            }

            console.log('Subscribing to: ', productId);

            return inAppPurchase.subscribe(productId);
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').component('minsecField', {
        template: "<span class=\"minSecField\">" + "<input class=\"single mins\" " + "select-on-focus " + "whole-number " + "type=\"number\" " + "placeholder=\"{{ vm.timeFieldsPlaceholders.minutes }}\" " + "ng-model=\"vm.timeFields.minutes\" " + "ng-change=\"vm.timeChanged()\" " + "step=\"1\" " + "min=\"0\" " + "required /><span " + "class=\"decimal\">:</span><input " + "class=\"single secs\" " + "select-on-focus " + "whole-number " + "type=\"number\" " + "placeholder=\"{{ vm.timeFieldsPlaceholders.seconds }}\" " + "ng-model=\"vm.timeFields.seconds\" " + "ng-change=\"vm.timeChanged()\" " + "step=\"1\" " + "min=\"0\" " + "required /></span>",
        controller: controller,
        controllerAs: 'vm',
        bindings: {
            'time': '=',
            'placeholder': '=',
            'set': '=',
            'field': '=',
            'change': '&?'
        }
    });

    controller.$inject = ['$scope', '$timeout'];

    function controller($scope, $timeout) {

        var vm = this;

        var updating = false;

        vm.timeFields = {};

        vm.timeFieldsPlaceholders = {};

        vm.setFields = function () {

            if (updating) {
                updating = false;
                return false;
            }

            var time = parseInt(vm.time);
            var timePlaceholder = parseInt(vm.placeholder);

            if (time > 0) {

                var minutes = Math.floor(time / 60);
                var secs = time - minutes * 60;

                vm.timeFields.minutes = minutes > 0 ? minutes : 0;
                vm.timeFields.seconds = secs > 0 ? secs : 0;
            } else {

                if (typeof vm.set !== 'undefined' && vm.set.isDone) {
                    vm.timeFields.minutes = 0;
                    vm.timeFields.seconds = 0;
                } else {
                    delete vm.timeFields.minutes;
                    delete vm.timeFields.seconds;
                }
            }

            if (timePlaceholder > 0) {

                var minutes = Math.floor(timePlaceholder / 60);
                var secs = timePlaceholder - minutes * 60;

                vm.timeFieldsPlaceholders.minutes = minutes > 0 ? minutes : 0;
                vm.timeFieldsPlaceholders.seconds = secs > 0 ? secs : 0;
            } else {

                vm.timeFieldsPlaceholders.minutes = 0;
                vm.timeFieldsPlaceholders.seconds = 0;
            }
        };
        vm.setFields();

        $scope.$watch(function () {
            return vm.time;
        }, function () {
            vm.setFields();
        });

        var recalcTimer;
        vm.timeChanged = function () {
            updating = true;
            vm.time = 0;

            $timeout.cancel(recalcTimer);

            var numbers;
            // Just commenting this out so that people can log more than 99 minutes of exercise...

            // if (vm.timeFields.minutes > 99) {
            //     numbers = vm.timeFields.minutes.toString().split('');
            //     numbers = numbers.slice(Math.max(numbers.length - 2, 1));
            //     vm.timeFields.minutes = parseInt(numbers.join(''));
            // }

            if (vm.timeFields.seconds > 99) {
                numbers = vm.timeFields.seconds.toString().split('');
                numbers = numbers.slice(Math.max(numbers.length - 2, 1));
                vm.timeFields.seconds = parseInt(numbers.join(''));
            }

            if (vm.timeFields.minutes) {
                vm.time += parseInt(vm.timeFields.minutes) * 60;
            }

            if (vm.timeFields.seconds) {
                vm.time += parseInt(vm.timeFields.seconds);
            }

            if (!vm.timeFields.minutes) {
                vm.timeFields.minutes = 0;
            }

            if (!vm.timeFields.seconds) {
                vm.timeFields.seconds = 0;
            }

            if (typeof vm.set !== 'undefined' && vm.set.isDone && vm.set.done[vm.field.name]) {
                vm.set.current_target[vm.field.name] = vm.set.done[vm.field.name];
            }

            if (typeof vm.change === 'function') {
                $timeout(function () {
                    vm.change();
                });
            }

            recalcTimer = $timeout(vm.setFields, 500);
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('repRangeField', directive);

    directive.$inject = [];

    function directive() {
        return {
            restrict: 'E',
            replace: true,
            templateUrl: "core/Inputs/RepRangeField.html",
            scope: {
                range: '=range'
            },
            controller: ['$scope', $scope => {
                $scope.rangeFields = {
                    min: 1,
                    max: 2
                };

                $scope.setFields = () => {
                    if ($scope.range) {
                        const values = $scope.range.toString().split('.');
                        const min = parseInt(values[0], 10) || 1;
                        let max = parseInt(values[1], 10) || 2;
                        if (min >= max) {
                            max = min + 1;
                        }
                        $scope.rangeFields = { min, max };
                        const maxField = $scope.rangeFields.max <= 9 ? `0${$scope.rangeFields.max}` : $scope.rangeFields.max;
                        $scope.range = $scope.rangeFields.min + '.' + maxField;;
                    }
                };
                $scope.setFields();

                $scope.$watch('range', () => {
                    $scope.setFields();
                });
                $scope.rangeChanged = () => {
                    const maxField = $scope.rangeFields.max <= 9 ? `0${$scope.rangeFields.max}` : $scope.rangeFields.max;
                    $scope.range = $scope.rangeFields.min + '.' + maxField;
                };
            }]
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('repRangeLogField', directive);

    directive.$inject = [];

    function directive() {
        return {
            restrict: 'E',
            replace: true,
            templateUrl: "core/Inputs/RepRangeLogField.html",
            scope: {
                display: '=display',
                value: '=value'
            },
            controller: ['$scope', $scope => {
                if ($scope.display) {
                    /**
                     * Not sure why we are modifying the values but this would do the trick
                     */
                    const notModify = $scope.display.toString().split('.');
                    const values = parseFloat($scope.display).toFixed(2).toString().split('.');
                    const min = values[0] || 1;
                    const max = notModify[1] >= 10 ? values[1] : notModify[1] || 2;

                    $scope.range = min + '-' + max;
                } else {
                    $scope.range = '';
                }
            }]
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('weightField', directive);

    directive.$inject = [];

    function directive() {

        return {
            restrict: 'E',
            replace: true,
            templateUrl: "core/Inputs/WeightField.html",
            scope: {
                'weight': '=weight',
                'weightType': '=weightType',
                'unitEditable': '=unitEditable'
            },
            controller: ['$scope', function ($scope) {

                $scope.enableWeightType = $scope.unitEditable !== false;

                $scope.$watch('weight', function (weight) {

                    if ($scope.weightType === 'lbs') {

                        var no = (weight + "").split(".");

                        var lbs = parseInt(no[0]);
                        var oz = 0;

                        if (no[1]) {
                            oz = Math.round(parseFloat('0.' + no[1]) * 16);
                        }

                        var out = lbs;
                        if (oz) {
                            out = parseFloat(out + '.' + oz);
                        }

                        $scope.weightVal = out;
                    } else {
                        $scope.weightVal = weight;
                    }
                });

                $scope.$watch('weightVal', function (val) {

                    if ($scope.weightType === 'lbs') {

                        var no = (val + "").split(".");

                        var lbs = parseInt(no[0]);
                        var oz = 0;

                        if (no[1]) {
                            var ozDec = parseInt(no[1]) * 0.0625;
                            ozDec = ('' + ozDec).split(".");
                            if (ozDec[1]) {
                                oz = ozDec[1];
                            }
                        }

                        var out = lbs;
                        if (oz) {
                            out = parseFloat(out + '.' + oz).toFixed(2);
                        }

                        $scope.weight = out;
                    } else {
                        $scope.weight = val;
                    }
                });
            }]
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('wholeNumber', directive).directive('stringToNumber', function () {
        return {
            require: 'ngModel',
            link: function (scope, element, attrs, ngModel) {
                ngModel.$formatters.push(function (value) {
                    return parseFloat(value);
                });
            }
        };
    });

    directive.$inject = [];

    function directive() {

        return {
            restrict: 'A',
            link: function (scope, element, attrs) {

                scope.$watch(attrs.ngModel, function (value, oldValue) {

                    // if decimal detected, round it down
                    if (value !== oldValue && value % 1 !== 0) {

                        if (attrs.allowEmpty && !value) {
                            return false;
                        }

                        var val = Math.floor(value);
                        attrs.ngModel = val;
                        element.val(val);
                    }
                });
            }
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('selectOnFocus', directive);

    directive.$inject = ['$timeout', '$window', 'DeviceInfoService'];

    function directive($timeout, $window, DeviceInfoService) {

        return {
            restrict: 'A',
            link: function (scope, element, attrs) {

                var active = false;

                var ios = DeviceInfoService.isIos();

                element.on('focus', function () {

                    var self = this;

                    if (active || self.value.length === 0) {
                        return false;
                    }

                    active = true;

                    $timeout(function () {

                        self.select();

                        if (ios && !$window.getSelection().toString()) {
                            // Required for mobile Safari
                            self.setSelectionRange(0, self.value.length);
                        }

                        active = false;
                    }, 10);
                });
            }
        };
    }
})();
(() => {
  function ActivityLikeController() {}

  ActivityLikeController.prototype = {
    getObjectId() {
      return this.activity.id;
    },

    isLiked() {
      return this.activity.is_liked > 0;
    },

    getLikes() {
      return this.activity.likes_count;
    },

    getType() {
      return 'activity_log_item';
    }
  };

  angular.module('MyPTHub').component('activityLike', {
    templateUrl: 'core/Like/activity-like.html',
    controller: ActivityLikeController,
    controllerAs: 'vm',
    bindings: {
      activity: '='
    }
  });
})();
(() => {
  angular.module('MyPTHub').component('like', {
    templateUrl: 'core/Like/primary-like.html',
    controller: LikeController,
    controllerAs: 'vm',
    bindings: {
      type: '@',
      id: '<',
      likes: '<',
      liked: '<'
    }
  });

  LikeController.$inject = ['$mdToast', 'likeService'];

  function LikeController($mdToast, likeService) {
    this.processing = false;
    const handleSuccess = () => {
      this.likes = this.liked ? this.likes - 1 : this.likes + 1;
      this.liked = !this.liked;
    };

    const handleError = () => {
      const message = $mdToast.simple().textContent('Your like action failed. Please try again.').position('top right');

      $mdToast.show(message);
    };

    const toggleProcessing = () => this.processing = !this.processing;

    this.handleLike = () => {
      if (!this.processing) {
        toggleProcessing();
        likeService.toggle(this.id, this.type).then(handleSuccess, handleError).finally(toggleProcessing);
      }
    };

    this.getLikes = () => {
      if (this.liked && this.likes > 1) {
        const formattedNumberOfLikes = abbrNum(this.likes - 1, 1);
        const otherLable = this.likes > 2 ? 'others' : 'other';
        return `You and ${formattedNumberOfLikes} ${otherLable}`;
      }
      return abbrNum(this.likes, 1);
    };
  }

  /* eslint-disable no-restricted-properties */
  const abbrNum = (_number, _decPlaces) => {
    let number = _number;
    // 2 decimal places => 100, 3 => 1000, etc
    const decPlaces = Math.pow(10, _decPlaces);

    // Enumerate number abbreviations
    const abbrev = ['k', 'm', 'b', 't'];

    // Go through the array backwards, so we do the largest first
    for (let i = abbrev.length - 1; i >= 0; i -= 1) {
      // Convert array index to "1000", "1000000", etc
      const size = Math.pow(10, (i + 1) * 3);

      // If the number is bigger or equal do the abbreviation
      if (size <= number) {
        // Here, we multiply by decPlaces, round, and then divide by decPlaces.
        // This gives us nice rounding to a particular decimal place.
        number = Math.round(number * (decPlaces / size)) / decPlaces;

        // Handle special case where we round up to the next abbreviation
        if (number === 1000 && i < abbrev.length - 1) {
          number = 1;
          i += 1;
        }
        // Add the letter for the abbreviation
        number += abbrev[i];
        break;
      }
    }
    return number;
  };
})();
const likeService = ApiClientService => ({
  toggle(id, type) {
    return ApiClientService.like.toggle({
      likeable_id: id,
      likeable_type: type
    });
  }
});

likeService.$inject = ['ApiClientService'];

angular.module('MyPTHub').factory('likeService', likeService);
(function () {
    'use strict';

    angular.module('MyPTHub').factory('LocalStorageService', service);

    service.$inject = [];

    function service() {

        return {

            setKey: function (key, val) {
                window.localStorage.setItem(key, val);
            },

            getKey: function (key) {
                return window.localStorage.getItem(key);
            },

            removeKey: function (key) {
                window.localStorage.removeItem(key);
            },

            clear: function () {
                window.localStorage.clear();
            }

        };
    }
})();
// Workouts Controller
(function () {
    'use strict';

    angular.module('MyPTHub').factory('MessagingService', service);

    service.$inject = ['$rootScope', 'NavigateService', 'AuthenticationStateService', 'AuthenticationService', 'MessagesService'];

    function service($rootScope, NavigateService, AuthenticationStateService, AuthenticationService, MessagesService) {

        var newMessage = false;
        var prev_newMessage = false;

        var pendingViewConvoId = false;
        var latestConversationId = false;

        var service = {};

        var fireEventHandlers = function () {

            if (prev_newMessage === newMessage) {
                return false;
            }

            $rootScope.$emit('messagingStateChange');
        };

        service.setup = function () {

            AuthenticationService.registerLoginSuccessEventHandler(function (resolve) {

                if (service.hasPendingConversationView()) {
                    service.goToPendingConversation();
                }

                resolve(true);
            });

            if (AuthenticationStateService.isLoggedIn()) {
                if (service.hasPendingConversationView()) {
                    service.goToPendingConversation();
                }
            }
        };

        service.pushHandler = function (payload) {

            MessagesService.markUnseen();

            pendingViewConvoId = false;

            if (!payload.additionalData.foreground && payload.additionalData.conversationId) {

                console.log('go to conversation', payload.additionalData.conversationId);

                if (AuthenticationStateService.isLoggedIn()) {

                    NavigateService.go('app.message_conversation', {
                        conversationId: payload.additionalData.conversationId
                    });

                    service.resetState();
                } else {

                    pendingViewConvoId = payload.additionalData.conversationId;
                }
            }

            latestConversationId = payload.additionalData.conversationId;

            prev_newMessage = newMessage;
            newMessage = true;
            fireEventHandlers();
        };

        service.hasPendingConversationView = function () {
            return pendingViewConvoId !== false;
        };

        service.goToPendingConversation = function () {

            if (!pendingViewConvoId) {
                return false;
            }

            service.resetState();

            NavigateService.go('app.message_conversation', {
                conversationId: pendingViewConvoId
            });
        };

        service.goToLatestConversation = function () {

            if (!latestConversationId) {
                return false;
            }

            service.resetState();

            NavigateService.go('app.message_conversation', {
                conversationId: latestConversationId
            });
        };

        service.hasNewMessage = function () {
            return newMessage;
        };

        service.resetState = function () {
            prev_newMessage = newMessage;
            newMessage = false;
            pendingViewConvoId = false;
            fireEventHandlers();
        };

        service.registerEventHandler = function (scope, callback) {

            var unsub = $rootScope.$on('messagingStateChange', callback);

            scope.$on('$destroy', unsub);

            return unsub;
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('ModalService', service);

    service.$inject = ['$ionicModal', '$rootScope', '$q', '$controller', '$timeout'];

    function service($ionicModal, $rootScope, $q, $controller, $timeout) {

        var _cleanup = function (scope, delay) {
            if (scope.modal) {
                $timeout(function () {
                    scope.modal.remove();
                    scope.$destroy();
                }, delay);
            }
        };

        var _evalController = function (ctrlName) {

            var result = {
                isControllerAs: false,
                controllerName: '',
                propName: ''
            };

            var fragments = (ctrlName || '').trim().split(/\s+/);

            result.isControllerAs = fragments.length === 3 && (fragments[1] || '').toLowerCase() === 'as';

            if (result.isControllerAs) {

                result.controllerName = fragments[0];
                result.propName = fragments[2];
            } else {

                result.controllerName = ctrlName;
            }

            return result;
        };

        var modalOpen = function (conf, deferred) {

            var parameters = {};

            if (conf.hasOwnProperty('params')) {
                parameters = conf.params;
            }

            // Grab the injector and create a new scope
            var ctrlInstance,
                modalScope = $rootScope.$new(),
                thisScopeId = modalScope.$id;

            $ionicModal.fromTemplateUrl(conf.templateUrl, {
                scope: modalScope,
                animation: 'slide-in-up'
            }).then(function (modal) {

                modalScope.modal = modal;

                var resultValue = {};

                modalScope.openModal = function () {
                    modalScope.modal.show();
                };

                modalScope.closeModal = function (result) {
                    resultValue.resolve = result;
                    modalScope.modal.hide();
                };

                modalScope.cancelModal = function (result) {
                    resultValue.reject = result;
                    modalScope.modal.hide();
                };

                modalScope.$on('modal.hidden', function (thisModal) {

                    var delayCleanup = 250;

                    if (conf.hasOwnProperty('closeAnimation') && !conf.closeAnimation) {
                        delayCleanup = 0;
                    }

                    // delay to allow close animation to complete
                    if (thisModal.currentScope) {

                        var modalScopeId = thisModal.currentScope.$id;

                        if (thisScopeId === modalScopeId) {
                            //deferred.resolve(null);
                            _cleanup(thisModal.currentScope, delayCleanup);
                        }
                    }
                });

                /**
                 * We resolve the promise only after the modal is fully
                 * removed, to allow immediate opening of another modal. This
                 * allows modals to be used as part of a multi-step process.
                 * The promise will be resolved after any closing animation
                 * has completed (if applicable).
                 */
                modalScope.$on('modal.removed', function () {

                    if (resultValue.hasOwnProperty('resolve')) {
                        deferred.resolve(resultValue.resolve);
                    }

                    if (resultValue.hasOwnProperty('reject')) {
                        deferred.reject(resultValue.reject);
                    }
                });

                // Invoke the controller
                var locals = {
                    '$scope': modalScope,
                    'parameters': parameters
                };
                var ctrlEval = _evalController(conf.controller);
                ctrlInstance = $controller(conf.controller, locals);

                if (ctrlEval.isControllerAs) {
                    ctrlInstance.openModal = modalScope.openModal;
                    ctrlInstance.closeModal = modalScope.closeModal;
                }

                modalScope.modal.show();
            }, function (err) {
                deferred.reject(err);
            });
        };

        var show = function (conf) {

            if (!conf.hasOwnProperty('params')) {
                conf.params = {};
            }

            var deferred = $q.defer();

            if (conf.hasOwnProperty('resolve')) {

                var resolvers = [];
                var resolverNames = [];

                for (var resolverName in conf.resolve) {
                    if (conf.resolve.hasOwnProperty(resolverName)) {
                        var resolver = conf.resolve[resolverName];
                        resolvers.push(resolver());
                        resolverNames.push(resolverName);
                    }
                }

                $q.all(resolvers).then(function (results) {

                    for (var i = 0; i < results.length; i++) {

                        var resolverName = resolverNames[i];
                        conf.params[resolverName] = results[i];
                    }

                    modalOpen(conf, deferred);
                });
            } else {

                modalOpen(conf, deferred);
            }

            return deferred.promise;
        };

        return {
            show: show
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('NetworkStateService', service);

    service.$inject = ['$rootScope', '$cordovaNetwork'];

    function service($rootScope, $cordovaNetwork) {

        /**
         * Override online/offline detection in Cordova Network, as network
         * type UNKNOWN is not Offline, and some old Android devices report
         * network as UNKOWN when connected to mobile data networks.
         */

        $cordovaNetwork.isOnline = function () {
            return navigator.connection.type !== Connection.NONE;
        };

        $cordovaNetwork.isOffline = function () {
            return navigator.connection.type === Connection.NONE;
        };

        /**
         * End override
         */

        var state = {
            online: false,
            class: 0,
            type: 'UNKNOWN'
        };
        var prevState = {
            online: false,
            class: 0,
            type: 'UNKNOWN'
        };

        var service = {

            isOnline: function () {
                return state.online;
            },

            isLAN: function () {
                return state.class >= 2;
            }

        };

        var fireEventHandlers = function () {

            if (state.online === prevState.online) {
                return false;
            }

            $rootScope.$emit('networkStateChange');
        };

        var updateNetworkState = function (isSetup) {

            isSetup = isSetup || false;

            prevState = angular.copy(state);

            if (window.Connection) {

                state.type = $cordovaNetwork.getNetwork();
                state.online = $cordovaNetwork.isOnline();

                switch (state.net_type) {
                    case Connection.ETHERNET:
                        state.net_class = 2;
                        break;
                    case Connection.WIFI:
                        state.net_class = 2;
                        break;
                    case Connection.CELL_2G:
                        state.net_class = 1;
                        break;
                    case Connection.CELL_3G:
                        state.net_class = 1;
                        break;
                    case Connection.CELL_4G:
                        state.net_class = 1;
                        break;
                    case Connection.CELL:
                        state.net_class = 1;
                        break;
                    case Connection.NONE:
                        state.net_class = 0;
                        break;
                    case Connection.UNKNOWN:
                        state.net_class = 0;
                        break;
                }

                if (!isSetup) {
                    fireEventHandlers();
                }

                return true;
            }

            // Not running on mobile device.
            state.online = true;

            if (!isSetup) {
                fireEventHandlers();
            }

            return true;
        };

        var debounce = function (func, wait, immediate) {
            var timeout;
            return function () {
                var context = this,
                    args = arguments;
                var later = function () {
                    timeout = null;
                    if (!immediate) func.apply(context, args);
                };
                var callNow = immediate && !timeout;
                clearTimeout(timeout);
                timeout = setTimeout(later, wait);
                if (callNow) func.apply(context, args);
            };
        };

        service.setup = function () {

            var handle = debounce(function () {
                updateNetworkState(false);
            }, 10000);

            $rootScope.$on('$cordovaNetwork:online', handle);
            $rootScope.$on('$cordovaNetwork:offline', handle);

            updateNetworkState(true);
        };

        service.registerEventHandler = function (scope, callback) {

            var unsub = $rootScope.$on('networkStateChange', callback);

            scope.$on('$destroy', unsub);

            return unsub;
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').component('notificationIndicator', {
        template: "<md-button md-no-ink class=\"md-icon-button ion-nav-button-right\" ng-click=\"vm.go()\" ng-if=\"vm.enabled\"> " + "<i class=\"material-icons\">{{ vm.seen ? 'notifications_none' : 'notifications' }}</i>" + "<span class=\"noteAdded\" ng-if=\"!vm.seen\"></span>" + "</md-button>",
        controller: controller,
        controllerAs: 'vm',
        bindings: {}
    });

    controller.$inject = ['$scope', 'PermissionsService', 'NotificationsService', '$state'];

    function controller($scope, PermissionsService, NotificationsService, $state) {

        var vm = this;

        vm.enabled = PermissionsService.schedulerIsEnabled();

        vm.go = function () {
            $state.go('app.notifications');
        };

        vm.seen = NotificationsService.isSeen();

        NotificationsService.onSeenStateChange($scope, function (ev, isSeen) {
            vm.seen = isSeen;
        });
    }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').factory('NutritionEditorService', service);

  service.$inject = ['$filter'];

  function service($filter) {
    let nutritionItem;

    // Public
    const service = {};

    service.reset = function () {
      nutritionItem = null;
    };

    service.getItem = function () {
      return nutritionItem;
    };

    service.setItem = function (item, mode) {
      // Creates new instance and hydrates it
      nutritionItem = new Item(mode);

      if (item.id) {
        nutritionItem.id = item.id;
      }

      if (mode === 'log') {
        nutritionItem.day = item.day;
        nutritionItem.created = item.created;
      }
      nutritionItem.macro_type = item.macro_type;
      nutritionItem.note = item.note;
      nutritionItem.target_calories = item.target_calories;
      nutritionItem.target_carbs = item.target_carbs;
      nutritionItem.target_fat = item.target_fat;
      nutritionItem.target_fibre = item.target_fibre;
      nutritionItem.target_protein = item.target_protein;
      nutritionItem.target_sodium = item.target_sodium;
      nutritionItem.target_sugar = item.target_sugar;
      nutritionItem.use_macro = !!item.use_macro;
      nutritionItem.canEdit = item.hasOwnProperty('canEdit') ? item.canEdit : true;

      if (mode === 'plan') {
        nutritionItem.trainer_user_id = item.trainer_user_id;
        nutritionItem.title = item.title;
      }

      angular.forEach(item.meals, function (meal) {
        var newMeal = nutritionItem.addMeal(meal.title, meal.notes);
        angular.forEach(meal.foods, function (food) {
          var source, enable;

          if (mode === 'log') {
            source = food.source;
            enable = food.enable;
            // Log mode + not enabled + not editable = food in the bin
            if (!enable && !nutritionItem.canEdit) {
              return false;
            }
          }

          var newFood = newMeal.addFood(food, food.food_serving_id, food.qty, source, enable);
        });
      });

      nutritionItem.updateTotals();

      return this.getItem();
    };

    service.setTemplate = function (item, mode) {

      nutritionItem.macro_type = item.macro_type;
      nutritionItem.note = item.note;
      nutritionItem.target_calories = item.target_calories;
      nutritionItem.target_carbs = item.target_carbs;
      nutritionItem.target_fat = item.target_fat;
      nutritionItem.target_fibre = item.target_fibre;
      nutritionItem.target_protein = item.target_protein;
      nutritionItem.target_sodium = item.target_sodium;
      nutritionItem.target_sugar = item.target_sugar;
      nutritionItem.use_macro = item.use_macro ? true : false;

      angular.forEach(item.meals, function (meal) {

        var newMeal = nutritionItem.addMeal(meal.title, meal.notes);

        angular.forEach(meal.foods, function (food) {

          var source, enable;

          if (mode === 'log') {
            source = 'plan';
            enable = false;
          }

          var newFood = newMeal.addFood(food, food.food_serving_id, food.qty, source, enable);
        });
      });

      nutritionItem.updateTotals();

      return this.getItem();
    };

    service.newInstance = function (mode) {

      nutritionItem = new Item(mode);

      nutritionItem.updateTotals();

      return this.getItem();
    };

    service.getPayload = function () {
      return JSON.stringify(nutritionItem);
    };

    // Sets the current meal, to allow adding food across state changes
    var currentMeal;
    service.setCurrentMeal = function (meal) {
      currentMeal = meal;
    };

    service.getCurrentMeal = function () {
      return currentMeal;
    };

    function Item(mode) {

      var item = this;

      if (mode === 'plan') {
        this.trainer_user_id = null;
      }

      if (mode === 'log') {
        this.day = moment().format('YYYY-MM-DD');
      }

      this.mode = mode;
      this.canEdit = true;
      this.target_calories = 0;
      this.use_macro = true;
      this.macro_type = 'grams';
      this.target_protein = 0;
      this.target_carbs = 0;
      this.target_fat = 0;
      this.target_fibre = 0;
      this.target_sugar = 0;
      this.target_sodium = 0;
      this.notes = null;
      this.meals = [];

      this.addMeal = function (title, notes) {
        item.notes = notes || null;

        var meal = new Meal(title, item);

        item.meals.push(meal);

        return meal;
      };

      this.addMealFromTemplate = meal => {
        const newMeal = this.addMeal(meal.title, null);

        angular.forEach(meal.foods, food => {
          newMeal.addFood(food, food.food_serving_id, food.qty, '', true);
        });

        this.updateTotals();

        return meal;
      };

      this.deleteMeal = function (meal) {
        var mealIndex = this.meals.indexOf(meal);
        this.meals.splice(mealIndex, 1);
      };

      this.resetMeals = function () {
        item.meals.length = 0;
      };

      this.updateTotals = function () {

        item._totals = {
          calories: 0,
          protein: 0,
          carbohydrate: 0,
          fat: 0,
          fibre: 0,
          sodium: 0,
          sugar: 0
        };

        item.meals.map(function (meal) {

          meal.updateTotals();

          var foodTotals = meal.getTotals();

          item._totals.calories += foodTotals.calories;
          item._totals.protein += foodTotals.protein;
          item._totals.carbohydrate += foodTotals.carbohydrate;
          item._totals.fat += foodTotals.fat;
          item._totals.fibre += foodTotals.fibre;
          item._totals.sodium += foodTotals.sodium;
          item._totals.sugar += foodTotals.sugar;
        });
      };

      this.getTotals = function () {
        return item._totals;
      };

      this.getTargets = function () {
        var self = this;

        return {
          calories: self.target_calories,
          protein: self.target_protein,
          carbohydrate: self.target_carbs,
          fat: self.target_fat,
          fibre: self.target_fibre,
          sugar: self.target_sugar,
          sodium: self.target_sodium
        };
      };

      this.getRemainingTotals = function () {

        var self = this;

        var calculate = function (target, current) {

          var remaining = target - current;

          if (remaining > 0) {
            return remaining;
          }

          return 0;
        };

        return {
          calories: calculate(self.target_calories, item._totals.calories),
          protein: calculate(self.target_protein, item._totals.protein),
          carbohydrate: calculate(self.target_carbs, item._totals.carbohydrate),
          fat: calculate(self.target_fat, item._totals.fat),
          fibre: calculate(self.target_fibre, item._totals.fibre),
          sugar: calculate(self.target_sugar, item._totals.sugar),
          sodium: calculate(self.target_sodium, item._totals.sodium)
        };
      };

      return this;
    }

    function Meal(title, item) {

      var meal = this;

      title = title || null;

      this.title = title;
      this.notes = item.notes;
      this.foods = [];

      if (item.hasOwnProperty('foods')) {
        if (item.foods.length) {

          item.foods.forEach(foodItem => {
            let meal = this;
            let food = new Food(foodItem, foodItem.food_serving_id, foodItem.qty, '', true, item, meal);

            this.foods.push(food);
          }, meal);
        }
      }

      this.updateTotals = function () {
        meal._totals = {
          calories: 0,
          protein: 0,
          carbohydrate: 0,
          fat: 0,
          fibre: 0,
          sodium: 0,
          sugar: 0
        };

        meal.foods.map(function (food) {
          food.updateTotals();
          if (!food.hasOwnProperty('enable') || food.enable) {
            var foodTotals = food.getTotals();
            meal._totals.calories += foodTotals.calories;
            meal._totals.protein += foodTotals.protein;
            meal._totals.carbohydrate += foodTotals.carbohydrate;
            meal._totals.fat += foodTotals.fat;
            meal._totals.fibre += foodTotals.fibre;
            meal._totals.sodium += foodTotals.sodium;
            meal._totals.sugar += foodTotals.sugar;
          }
        });
      };

      this.getTotals = function () {
        return meal._totals;
      };

      this.addFood = function (data, servingId, qty, source, enable) {
        var food = new Food(data, servingId, qty, source, enable, item, meal);

        meal.foods.push(food);

        return food;
      };

      this.deleteFood = function (food) {

        var foodIndex = this.foods.indexOf(food);

        this.foods.splice(foodIndex, 1);
      };

      this.delete = function () {
        item.deleteMeal(this);
      };

      return this;
    }

    function Food(data, servingId, qty, source, enable, item, meal) {
      var food = this;

      source = source || null;
      enable = typeof enable !== 'undefined' ? enable : true;

      this.food_id = data.food_id;
      this.source = source;
      this.enable = enable;
      this._data = data;

      this.toggleEnabled = function () {
        food.enable = !food.enable;
      };

      this.setName = function () {
        if (food._serving) {
          var unitQty = Math.floor(food._serving.number_of_units * food.qty * 100) / 100;
          var unitDesc;
          if (food._serving.measurement_description === 'serving') {
            unitDesc = ' - ' + unitQty + ' serving' + (unitQty !== 1 ? 's' : '') + ' of ' + food._serving.serving_description;
          } else {
            unitDesc = ' - ' + unitQty + ' ' + food._serving.measurement_description;
          }
          food.name = data.name + unitDesc;
        } else {
          food.name = data.name;
        }
      };

      this.updateTotals = function () {
        var calculate = function (macro) {
          if (food._serving) {
            return Math.round(food._serving[macro] * food.qty);
          } else {
            return Math.round(food._data[macro]);
          }
        };

        food._totals = {
          calories: calculate('calories'),
          protein: calculate('protein'),
          carbohydrate: calculate('carbohydrate'),
          fat: calculate('fat'),
          fibre: calculate('fibre'),
          sodium: calculate('sodium'),
          sugar: calculate('sugar')
        };
      };

      this.getTotals = function () {
        return food._totals;
      };

      this.setQty = function (newQty) {
        newQty = parseFloat(newQty);
        if (newQty <= 0) {
          newQty = 1;
        }
        food.qty = newQty;
        food.setName();
      };

      this.setServing = function (servingId) {
        if (food._servings) {
          food._serving = $filter('filter')(food._servings, {
            id: servingId
          }, true)[0];
        }
        food.food_serving_id = servingId;
        food.setName();
      };

      this.setServings = function (servings) {
        food._servings = servings;
      };

      this.setData = function (data) {
        this._data = data;
      };

      this.delete = function () {
        meal.deleteFood(this);
      };

      if (data.servings) {
        this.setServings(data.servings);
      }

      this.setServing(servingId);
      this.setQty(qty);

      return this;
    }

    return service;
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('NutritionItemFoodEditModalController', controller).factory('NutritionItemFoodEditModalService', service);

    controller.$inject = ['$scope', 'NutritionEditorService', 'parameters', 'ApiClientService', 'ConfirmDialogService', 'NutritionPlanEditService', 'NutritionLogEditService'];

    function controller($scope, NutritionEditorService, parameters, ApiClientService, ConfirmDialogService, NutritionPlanEditService, NutritionLogEditService) {

        var vm = this;

        vm.food = parameters.food;

        vm.nutritionItem = NutritionEditorService.getItem();

        var editService = NutritionPlanEditService;

        if (vm.nutritionItem.mode === 'log') {
            editService = NutritionLogEditService;
        }

        vm.ready = false;

        vm.servingQty = vm.food.qty;

        ApiClientService.nutrition.getFood(vm.food.food_id).success(function (response) {

            vm.food._data.name = response.food.name;

            vm.food.setServings(response.food.servings);
            vm.food.setServing(vm.food.food_serving_id);

            vm.serving = vm.food._serving;

            vm.updateServing();

            vm.ready = true;
        });

        var calculateNutrient = function (value, qty) {

            if (value === null) {
                return null;
            }

            return Math.round(value * qty);
        };

        vm.updateServing = function () {

            var qty = parseFloat(vm.servingQty);

            vm.totals = {
                calories: calculateNutrient(vm.serving.calories, qty),
                carbohydrate: calculateNutrient(vm.serving.carbohydrate, qty),
                cholesterol: calculateNutrient(vm.serving.cholesterol, qty),
                fat: calculateNutrient(vm.serving.fat, qty),
                fibre: calculateNutrient(vm.serving.fibre, qty),
                monounsaturated_fat: calculateNutrient(vm.serving.monounsaturated_fat, qty),
                polyunsaturated_fat: calculateNutrient(vm.serving.polyunsaturated_fat, qty),
                potassium: calculateNutrient(vm.serving.potassium, qty),
                protein: calculateNutrient(vm.serving.protein, qty),
                saturated_fat: calculateNutrient(vm.serving.saturated_fat, qty),
                sodium: calculateNutrient(vm.serving.sodium, qty),
                sugar: calculateNutrient(vm.serving.sugar, qty),

                calcium: calculateNutrient(vm.serving.calcium, qty),
                iron: calculateNutrient(vm.serving.iron, qty),
                vitamin_a: calculateNutrient(vm.serving.vitamin_a, qty),
                vitamin_c: calculateNutrient(vm.serving.vitamin_c, qty)
            };
        };

        vm.saveFood = function () {

            vm.saving = true;

            vm.food.setServing(vm.serving.id);
            vm.food.setQty(vm.servingQty);

            vm.nutritionItem.updateTotals();

            editService.saveChanges().then(function () {
                $scope.closeModal(true);
            }, function () {
                vm.saving = false;
            });
        };

        vm.deleteFood = function () {

            ConfirmDialogService.open({
                title: "Confirm delete",
                message: "Do you want to remove this?",
                buttonConfirm: "Confirm",
                buttonCancel: "Cancel"
            }).then(function () {

                vm.saving = true;

                vm.food.delete();

                vm.nutritionItem.updateTotals();

                $scope.closeModal(true);
            });
        };
    }

    service.$inject = ['ModalService'];

    function service(ModalService) {

        return {

            show: function (food) {

                return ModalService.show({
                    templateUrl: "core/NutritionEditor/foodEditModal.html",
                    controller: 'NutritionItemFoodEditModalController as vm',
                    params: {
                        food: food
                    }
                });
            }

        };
    }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').controller('NutritionItemMacroViewerModalController', controller).factory('NutritionItemMacroViewerModalService', service);

  controller.$inject = ['$scope', 'NutritionEditorService', 'parameters'];

  function controller($scope, NutritionEditorService, meal) {

    var vm = this;

    vm.nutritionItem = NutritionEditorService.getItem();
    vm.meal = meal.meal;
    if (meal.meal) {
      vm.totals = meal.meal._totals;
    } else {
      vm.totals = vm.nutritionItem.getTotals();
    }
    vm.targets = vm.nutritionItem.getTargets();
    vm.remaining = vm.nutritionItem.getRemainingTotals();
  }

  service.$inject = ['ModalService'];

  function service(ModalService) {

    return {

      show: function (meal = false) {

        return ModalService.show({
          templateUrl: 'core/NutritionEditor/macroViewerModal.html',
          controller: 'NutritionItemMacroViewerModalController as vm',
          params: {
            meal
          }
        });
      }

    };
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('NutritionNotesEditModalController', controller);

    controller.$inject = ['$scope', 'parameters'];

    function controller($scope, parameters) {

        var vm = this;
        vm.meal = parameters.meal;

        vm.saveNotes = function () {
            $scope.closeModal(vm.meal.notes);
        };
    }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').directive('nutritionEditor', directive);

  function directive() {

    return {
      restrict: 'E',
      replace: false,
      templateUrl: 'core/NutritionEditor/nutritionEditorDirective.html',
      controller: controller,
      controllerAs: 'nutritionEditorVm' // needs to be unique, as used inside other scopes
    };
  }

  controller.$inject = ['NutritionEditorService', 'NoteViewerModalService', '$ionicActionSheet', 'NutritionItemMacroViewerModalService', 'NutritionItemFoodEditModalService', 'ConfirmDialogService', 'PromptDialogService', 'NavigateService', 'ModalService', 'ApiClientService', 'ToastService'];

  function controller(NutritionEditorService, NoteViewerModalService, $ionicActionSheet, NutritionItemMacroViewerModalService, NutritionItemFoodEditModalService, ConfirmDialogService, PromptDialogService, NavigateService, ModalService, ApiClientService, ToastService) {

    var nutritionEditorVm = this;
    nutritionEditorVm.nutritionItem = NutritionEditorService.getItem();
    nutritionEditorVm.isLog = false;

    if (nutritionEditorVm.nutritionItem.mode === 'log') {
      nutritionEditorVm.isLog = true;
    }

    nutritionEditorVm.showMacros = function () {
      NutritionItemMacroViewerModalService.show();
    };

    var toggleFood = function (food) {
      food.toggleEnabled();
      nutritionEditorVm.nutritionItem.updateTotals();
    };

    nutritionEditorVm.foodAction = function (food) {
      if (nutritionEditorVm.isLog && !food.enable) {
        toggleFood(food);
      } else {
        NutritionItemFoodEditModalService.show(food);
      }
    };

    var addNotes = function (meal) {
      ModalService.show({
        templateUrl: 'core/NutritionEditor/notesModal.html',
        controller: 'NutritionNotesEditModalController as vm',
        params: {
          meal: meal
        }
      });
    };

    var viewNotes = function (notes) {
      NoteViewerModalService.show(notes);
    };

    var renameMeal = function (meal) {

      PromptDialogService.open({
        title: 'Rename Meal',
        message: 'Please enter a new name for the meal',
        buttonConfirm: 'Save',
        buttonCancel: 'Cancel',
        required: true,
        defaultText: meal.title
      }).then(function (name) {
        meal.title = name;
      });
    };

    const saveAsTemplate = meal => {
      PromptDialogService.open({
        title: 'Save meal as template',
        message: 'Please enter a name for the template',
        buttonConfirm: 'Save',
        buttonCancel: 'Cancel',
        required: true,
        defaultText: meal.title
      }).then(name => {
        meal.title = name;
        const save = ApiClientService.mealTemplates.create(meal);
        save.success(resp => {
          ToastService.show(`${resp.meal_template.title} created`);
        }).error(resp => {
          ToastService.show(`Error saving ${meal.title}, ${resp.message}`);
        });
      });
    };

    var deleteMeal = function (meal) {

      ConfirmDialogService.open({
        title: 'Confirm delete',
        message: 'Do you want to remove this?',
        buttonConfirm: 'Confirm',
        buttonCancel: 'Cancel'
      }).then(function () {
        meal.delete();
        nutritionEditorVm.nutritionItem.updateTotals();
      });
    };

    nutritionEditorVm.mealMenu = function (meal) {

      var buttons = [];

      var hasNotes = typeof meal.notes !== 'undefined' && meal.notes !== null && meal.notes.length > 0;

      if (!nutritionEditorVm.isLog) {
        if (nutritionEditorVm.nutritionItem.canEdit) {
          buttons.push({
            text: hasNotes ? 'Edit Notes' : 'Add Notes',
            click: function () {
              addNotes(meal);
            }
          });
        } else {
          buttons.push({
            text: 'View Notes',
            click: function () {
              viewNotes(meal.notes);
            }
          });
        }
      } else if (hasNotes) {
        buttons.push({
          text: 'View Notes',
          click: function () {
            viewNotes(meal.notes);
          }
        });
      }

      if (nutritionEditorVm.nutritionItem.canEdit) {
        buttons.push({
          text: 'Rename Meal',
          click: function () {
            renameMeal(meal);
          }
        });
      }

      buttons.push({
        text: 'View Macros',
        click: () => {
          console.log('clicked macaronis');
          NutritionItemMacroViewerModalService.show(meal);
        }
      });

      buttons.push({
        text: 'Save meal as template',
        click: () => {
          saveAsTemplate(meal);
        }
      });

      var config = {
        titleText: meal.title,
        buttons: buttons,
        cancelText: 'Cancel',
        buttonClicked: function (index) {
          buttons[index].click();
          return true;
        }
      };

      if (nutritionEditorVm.nutritionItem.canEdit) {
        config.destructiveText = 'Delete Meal';
        config.destructiveButtonClicked = function () {
          deleteMeal(meal);
          return true;
        };
      }

      $ionicActionSheet.show(config);
    };

    nutritionEditorVm.addFood = function (meal) {

      NutritionEditorService.setCurrentMeal(meal);

      NavigateService.go('app.food_search');
    };
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('nutritionItemConfig', directive);

    directive.$inject = [];

    function directive() {

        return {
            restrict: 'E',
            replace: false,
            templateUrl: "core/NutritionEditor/nutritionItemConfigDirective.html",
            controller: controller,
            controllerAs: 'vm',
            scope: {
                nutritionPlans: '=?'
            }
        };
    }

    controller.$inject = ['NutritionEditorService', '$scope'];

    function controller(NutritionEditorService, $scope) {

        var vm = this;

        vm.nutritionItem = NutritionEditorService.getItem();
        vm.nutritionPlans = $scope.nutritionPlans;
        vm.nutritionItem.useTemplate = 'new';

        vm.isCreate = !vm.nutritionItem.hasOwnProperty('id');

        // Name / Use Macro
        vm.macroChoice = function (choice) {
            vm.nutritionItem.macro_type = 'grams';
            vm.nutritionItem.use_macro = choice ? 1 : 0;
        };

        // Keeps the percent fields correctly set when in grams mode, in case you want to switch between
        vm.gramsToPercent = function () {

            var cals = vm.nutritionItem.target_calories;

            if (cals > 0) {

                // Protein
                if (vm.nutritionItem.target_protein > 0) {
                    vm.percent_protein = Math.round(vm.nutritionItem.target_protein * 4 / cals * 100);
                } else {
                    vm.percent_protein = 0;
                }

                // Carbs
                if (vm.nutritionItem.target_carbs > 0) {
                    vm.percent_carbs = Math.round(vm.nutritionItem.target_carbs * 4 / cals * 100);
                } else {
                    vm.percent_carbs = 0;
                }

                // Fat
                if (vm.nutritionItem.target_fat > 0) {
                    vm.percent_fat = Math.round(vm.nutritionItem.target_fat * 9 / cals * 100);
                } else {
                    vm.percent_fat = 0;
                }
            }
        };

        vm.gramsToCalories = function () {

            var cals = 0;

            // Protein
            if (vm.nutritionItem.target_protein > 0) {
                cals += vm.nutritionItem.target_protein * 4;
            }

            // Carbs
            if (vm.nutritionItem.target_carbs > 0) {
                cals += vm.nutritionItem.target_carbs * 4;
            }

            // Fat
            if (vm.nutritionItem.target_fat > 0) {
                cals += vm.nutritionItem.target_fat * 9;
            }

            vm.nutritionItem.target_calories = cals;

            vm.gramsToPercent();
        };

        // Percent of macro calories to grams
        vm.percent_protein = 0;
        vm.percent_carbs = 0;
        vm.percent_fat = 0;
        vm.percentToGrams = function () {
            var cals = parseInt(vm.nutritionItem.target_calories);

            if (cals > 0) {

                var calsPc = cals / 100;

                // Protein
                if (vm.percent_protein > 0) {
                    vm.nutritionItem.target_protein = Math.round(vm.percent_protein * calsPc / 4);
                } else {
                    vm.nutritionItem.target_protein = 0;
                }

                // Carbs
                if (vm.percent_carbs > 0) {
                    vm.nutritionItem.target_carbs = Math.round(vm.percent_carbs * calsPc / 4);
                } else {
                    vm.nutritionItem.target_carbs = 0;
                }

                // Fat
                if (vm.percent_fat > 0) {
                    vm.nutritionItem.target_fat = Math.round(vm.percent_fat * calsPc / 9);
                } else {
                    vm.nutritionItem.target_fat = 0;
                }
            }
        };

        vm.clearTargets = function () {
            vm.percent_protein = 0;
            vm.percent_carbs = 0;
            vm.percent_fat = 0;
            vm.nutritionItem.target_protein = 0;
            vm.nutritionItem.target_carbs = 0;
            vm.nutritionItem.target_fat = 0;
        };

        vm.gramsToPercent();
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('ProgressPhotoUploadService', service);

    service.$inject = ['CameraService', 'ApiClientService', '$rootScope'];

    function service(CameraService, ApiClientService, $rootScope) {

        var service = {};

        var fireEventHandlers = function (response) {
            $rootScope.$emit('progressPhotoUploadSuccess', response);
        };

        var saveData = function (customerId, photoData, date) {
            date = date || false;

            var data = {
                photo: photoData
            };

            if (date) {
                data.date = date;
            }

            return ApiClientService.progressPhotos.add(customerId, data).success(function (response) {
                fireEventHandlers(response);
            });
        };

        service.takePhoto = function (customerId, date) {
            date = date || false;

            return CameraService.takePhoto().then(function (photoData) {
                return saveData(customerId, photoData, date);
            });
        };

        service.browsePhoto = function (customerId, date) {
            date = date || false;

            return CameraService.browsePhoto().then(function (photoData) {
                return saveData(customerId, photoData, date);
            });
        };

        service.registerUploadSuccessEventHandler = function (scope, callback) {

            var unsub = $rootScope.$on('progressPhotoUploadSuccess', callback);

            scope.$on('$destroy', unsub);

            return unsub;
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('PromptDialogService', service);

    service.$inject = ['$q'];

    function service($q) {

        var service = {};

        service.open = function (config) {

            return $q(function (resolve, reject) {

                if (navigator.notification) {

                    var onComplete = function (button) {

                        if (button.buttonIndex === 1) {

                            if (config.required && button.input1.length === 0) {

                                navigator.notification.prompt(config.message, onComplete, config.title, [config.buttonConfirm, config.buttonCancel], config.defaultText);
                            } else {
                                resolve(button.input1);
                            }
                        } else {
                            reject();
                        }
                    };

                    navigator.notification.prompt(config.message, onComplete, config.title, [config.buttonConfirm, config.buttonCancel], config.defaultText);
                } else {

                    var browserPrompt = function () {

                        var result = prompt(config.title + ' - ' + config.message, config.defaultText);

                        if (config.required && result !== null && result.length === 0) {
                            browserPrompt();
                        } else if (result === null) {
                            reject();
                        } else {
                            resolve(result);
                        }
                    };

                    browserPrompt();
                }
            });
        };

        return service;
    }
})();
(() => {
  angular.module('MyPTHub').factory('PushNotificationService', service);

  service.$inject = ['mpthconf', 'ApiClientService', 'DeviceInfoService', 'MessagingService', 'LocalStorageService', 'DeeplinkService', 'NotificationsService', 'NotificationsService', 'NutritionLogService', 'MessagesService'];

  function service(mpthconf, ApiClientService, DeviceInfoService, MessagingService, LocalStorageService, DeeplinkService, NotificationsService, NutritionLogService, MessagesService) {
    const service = {};

    let push;

    service.setup = function () {
      DeviceInfoService.bundleId().then(response => {
        console.log('bundle:', response);
      });

      if (!window.PushNotification) {
        console.log('No PushNotification service available');
        return false;
      }

      push = PushNotification.init({
        android: {
          senderID: mpthconf.gcm.sender_id,
          icon: 'icon',
          alert: true,
          sound: true
        },
        ios: {
          alert: 'true',
          badge: 'true',
          sound: 'true',
          clearBadge: 'true'
        },
        windows: {}
      });

      push.on('registration', data => {
        DeviceInfoService.bundleId().then(bundleId => {
          const deviceId = data.registrationId;
          const platform = DeviceInfoService.platform();

          const register = ApiClientService.device.register(deviceId, platform, bundleId);

          register.success(() => {
            console.log('Device registered successfully');
            LocalStorageService.setKey('pushNotificationDeviceId', deviceId);
          });

          register.error(() => {
            console.log('Device could not be registered');
          });
        });
      });

      push.on('notification', data => {
        console.log('push: notification', data);

        NotificationsService.dirty();

        const additionalData = data.additionalData;

        /**
         * If the notification arrived while app is in the foreground,
         * we don't want to navigate to it, as it would interrupt what
         * the user is doing.
         */
        if (additionalData.foreground) {
          return false;
        }

        switch (data.additionalData.event_type) {
          case 'new_message':
            MessagingService.pushHandler(data);
            break;

          case 'customer_metrics_updated':
            DeeplinkService.go('app.trainer_update_customer_metrics', {
              customerId: additionalData.customerId
            });
            break;

          case 'nutrition_logged':
            DeeplinkService.go('app.trainer_view_nutrition_log_day', {
              customerId: additionalData.customerId,
              date: additionalData.date
            });
            break;

          case 'progress_photo_uploaded':
            DeeplinkService.go('app.trainer_client_profile', {
              customerId: additionalData.customerId,
              tab: 'photos'
            });
            break;

          case 'session_completed':
            DeeplinkService.go('app.session_preview', {
              sessionId: additionalData.sessionId
            });
            break;

          case 'session_booked':
            DeeplinkService.go('app.session_preview', {
              sessionId: additionalData.sessionId
            });
            break;

          case 'session_edited':
            DeeplinkService.go('app.session_preview', {
              sessionId: additionalData.sessionId
            });
            break;

          case 'session_cancelled':
            // Nothing
            break;

          case 'session_cancel_request':

            var id = additionalData.sessionId;
            if (additionalData.occurrence) {
              id += `_${additionalData.occurrence}`;
            }

            DeeplinkService.go('app.session_preview', {
              sessionId: id
            });

            break;

          case 'session_cancel_request_rejected':

            var id = additionalData.sessionId;
            if (additionalData.occurrence) {
              id += `_${additionalData.occurrence}`;
            }

            DeeplinkService.go('app.session_preview', {
              sessionId: id
            });

            break;

          case 'schedule_assigned_schedule':
            DeeplinkService.go('app.view_customer_schedule', {
              customerId: additionalData.customer_id,
              scheduleId: additionalData.schedule_id
            });
            break;

          case 'schedule_workout_am':
          case 'schedule_workout_pm':
            DeeplinkService.go('app.view_workout_schedule', {
              workoutId: additionalData.workout_id,
              date: additionalData.date,
              scheduleId: additionalData.schedule_id,
              ampm: additionalData.ampm,
              customerId: additionalData.customer_id
            });
            break;

          case 'schedule_nutrition':
            NutritionLogService.startLogger(additionalData.date);
            break;

          case 'schedule_file':
            DeeplinkService.go('app.view_customer_schedule', {
              customerId: additionalData.customer_id,
              scheduleId: additionalData.schedule_id
            });
            break;

          case 'schedule_update_photo':
            DeeplinkService.go('app.view_customer_schedule', {
              customerId: additionalData.customer_id,
              scheduleId: additionalData.schedule_id,
              trigger: 'update_photo',
              triggerDate: additionalData.date
            });
            break;

          case 'schedule_update_metrics':
            DeeplinkService.go('app.client_update_metrics');
            break;

          case 'schedule_started_schedule':
            DeeplinkService.go('app.view_customer_schedule', {
              customerId: additionalData.customer_id,
              scheduleId: additionalData.schedule_id
            });
            break;

          case 'schedule_trainer_week_before':
            DeeplinkService.go('app.view_customer_schedule', {
              customerId: additionalData.customer_id,
              scheduleId: additionalData.schedule_id
            });
            break;

          case 'schedule_trainer_day_before':
            DeeplinkService.go('app.view_customer_schedule', {
              customerId: additionalData.customer_id,
              scheduleId: additionalData.schedule_id
            });
            break;

          case 'schedule_customer_updated_metrics_from_schedule':
            DeeplinkService.go('app.trainer_update_customer_metrics', {
              customerId: additionalData.customer_id
            });
            break;

          case 'schedule_customer_updated_metrics':
            DeeplinkService.go('app.trainer_update_customer_metrics', {
              customerId: additionalData.customer_id
            });
            break;

          case 'customer_uploaded_files':
            DeeplinkService.go('app.files');
            break;

          case 'file_assigned':
            DeeplinkService.go('app.client_files');
            break;

          case 'booking_created':
            DeeplinkService.go('app.trainer_calendar_event_booking', {
              eventId: additionalData.event_id,
              eventTimeId: additionalData.event_time_id,
              date_time: moment.utc(additionalData.date_time).format('YYYY-MM-DD HH:mm:ss')
            });
            break;

          case 'booking_slot_available':
            DeeplinkService.go('app.trainer_calendar_event_booking', {
              eventId: additionalData.event_id,
              eventTimeId: additionalData.event_time_id,
              date_time: moment.utc(additionalData.date_time).format('YYYY-MM-DD HH:mm:ss')
            });
            break;

          case 'booking_updated':
            DeeplinkService.go('app.trainer_calendar_event_booking', {
              eventId: additionalData.event_id,
              eventTimeId: additionalData.event_time_id,
              date_time: moment.utc(additionalData.date_time).format('YYYY-MM-DD HH:mm:ss')
            });
            break;

          case 'booking_cancelled':
            DeeplinkService.go('app.trainer_calendar_event_booking', {
              eventId: additionalData.event_id,
              eventTimeId: additionalData.event_time_id,
              date_time: moment.utc(additionalData.date_time).format('YYYY-MM-DD HH:mm:ss')
            });
            break;
        }
      });

      push.on('error', data => {
        console.log('push: error', data);
      });
    };

    service.stop = function () {
      const deviceId = LocalStorageService.getKey('pushNotificationDeviceId');

      if (!deviceId) {
        return false;
      }

      const platform = DeviceInfoService.platform();

      const unregister = ApiClientService.device.unregister(deviceId, platform);

      LocalStorageService.removeKey('pushNotificationDeviceId');

      unregister.success(() => {
        console.log('Device unregistered successfully');
      });
    };

    return service;
  }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').config(config).component('scheduleDayPicker', {
    template: '' + '<div class="scheduleDayPicker"> ' + '<div class="scheduleDayPickerBackdrop" ng-if="vm.open" ng-click="vm.close()"></div> ' + '<div class="pickerContainer" ng-if="vm.open"> ' + '<flex-calendar options="vm.options" events="vm.events"></flex-calendar> ' + '</div> ' + '</div>',
    controller: controller,
    controllerAs: 'vm',
    bindings: {
      trainerUserId: '=?',
      customerId: '=?'
    }
  });

  config.$inject = ['$translateProvider'];

  function config($translateProvider) {

    $translateProvider.translations('en', {
      JANUARY: 'January',
      FEBRUARY: 'February',
      MARCH: 'March',
      APRIL: 'April',
      MAI: 'May',
      JUNE: 'June',
      JULY: 'July',
      AUGUST: 'August',
      SEPTEMBER: 'September',
      OCTOBER: 'October',
      NOVEMBER: 'November',
      DECEMBER: 'December',

      SUNDAY: 'Sunday',
      MONDAY: 'Monday',
      TUESDAY: 'Tuesday',
      WEDNESDAY: 'Wednesday',
      THURSDAY: 'Thurday',
      FRIDAY: 'Friday',
      SATURDAY: 'Saturday'
    });
  }

  controller.$inject = ['$scope', 'ScheduleDayPickerService', 'PermissionsService', 'SessionScheduleService', 'SessionStoreService', 'NetworkStateService', 'ScheduleEventStoreService', 'ScheduleEventService', '$q'];

  function controller($scope, ScheduleDayPickerService, PermissionsService, SessionScheduleService, SessionStoreService, NetworkStateService, ScheduleEventStoreService, ScheduleEventService, $q) {
    var vm = this;

    var dateStart, dateEnd;

    var loaded = function (response) {

      vm.events = [];

      angular.forEach(response, function (session) {

        var localDate = moment.utc(session.date, 'YYYY-MM-DD HH:mm:ss');

        vm.events.push({
          date: localDate
        });
      });
    };

    var loadSchedulerEvents = function (customerId) {
      if (ScheduleEventStoreService.isDirty(customerId)) {
        return ScheduleEventStoreService.downloadCustomerEvents(customerId);
      }

      return ScheduleEventService.getCustomerEvents(dateStart, dateEnd, customerId);
    };

    var loadMonthsEvents = function (reload) {
      reload = reload || false;
      if (!vm.customerId && PermissionsService.userIsTrainer()) {

        SessionScheduleService.getTrainerSchedule(dateStart, dateEnd, vm.trainerUserId).then(loaded);

        if (!reload && NetworkStateService.isOnline()) {

          SessionStoreService.downloadTrainerSessions(dateStart, dateEnd, vm.trainerUserId, false).then(function () {
            loadMonthsEvents(true);
          });
        }
      } else {

        $q.all([SessionScheduleService.getCustomerSchedule(dateStart, dateEnd, vm.customerId), loadSchedulerEvents(vm.customerId)]).then(function (responses) {

          var sessions = responses[0];
          var schedulerEvents = responses[1];

          schedulerEvents.map(function (event) {
            event.date = moment(event.date).utc();
          });

          loaded(sessions.concat(schedulerEvents));
        });

        if (!reload && NetworkStateService.isOnline()) {

          SessionStoreService.downloadCustomerSessions(vm.customerId, dateStart, dateEnd, false).then(function () {
            loadMonthsEvents(true);
          });
        }
      }
    };

    var setDateRange = function (month, year) {

      month = month || false;
      year = year || false;

      if (month && year) {

        dateStart = moment().month(month).year(year).startOf('month');
        dateEnd = moment().month(month).year(year).endOf('month');
      } else {

        var currentSelection = ScheduleDayPickerService.getDate();

        dateStart = moment(currentSelection).startOf('month');
        dateEnd = moment(currentSelection).endOf('month');
      }
    };

    vm.events = [];

    vm.open = false;

    ScheduleDayPickerService.setup($scope.$parent);

    ScheduleDayPickerService.registerOpen(function () {

      setDateRange();

      vm.open = true;

      loadMonthsEvents();
    });

    ScheduleDayPickerService.registerClose(function (apply) {

      apply = apply || false;

      vm.open = false;

      // Hardware back button fix
      if (apply) {
        $scope.$digest();
      }
    });

    vm.close = ScheduleDayPickerService.close;

    vm.options = {
      defaultDate: ScheduleDayPickerService.getDate(),
      dayNamesLength: 1, // 1 for "M", 2 for "Mo", 3 for "Mon"; 9 will show full day names. Default is 1.
      mondayIsFirstDay: true, //set monday as first day of week. Default is false
      eventClick: function (date) {// called before dateClick and only if clicked day has events

      },
      dateClick: function (day) {

        ScheduleDayPickerService.setDate(moment(day.date));

        ScheduleDayPickerService.close();
      },
      changeMonth: function (month, year) {

        var monthNo = month.index;

        setDateRange(monthNo, year);

        loadMonthsEvents();
      },
      filteredEventsChange: function (filteredEvents) {}
    };

    /*[
        {foo: 'bar', eventClass: 'expired', date: "2015-08-18"}, //value of eventClass will be added to CSS class of the day element
        {foo: 'bar', date: "2015-08-20"}
    ];*/
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('ScheduleDayPickerService', service);

    service.$inject = ['$rootScope'];

    function service($rootScope) {

        var service = {};

        var currentScope;

        var closeEvent, openEvent;

        var isOpen = false;

        var chosenDate = moment();

        var fireDateChangeEventHandlers = function () {

            $rootScope.$emit('scheduleDayPickerDatePicked');
        };

        service.setup = function (scope) {

            currentScope = scope;

            currentScope.$on('$ionicView.beforeEnter', function () {
                service.close();
            });

            currentScope.$on('$ionicView.leave', function () {
                service.close();
            });
        };

        service.isOpen = function () {
            return isOpen;
        };

        service.open = function () {

            isOpen = true;
            openEvent();
        };

        service.close = function (apply) {

            apply = apply || false;

            isOpen = false;
            closeEvent(apply);
        };

        service.toggleOpen = function () {

            if (isOpen) {
                service.close();
            } else {
                service.open();
            }
        };

        service.registerOpen = function (func) {
            openEvent = func;
        };

        service.registerClose = function (func) {
            closeEvent = func;
        };

        service.setDate = function (dateObj) {

            chosenDate = dateObj;

            fireDateChangeEventHandlers();
        };

        service.reset = function () {
            chosenDate = moment();
            fireDateChangeEventHandlers();
        };

        service.getDate = function () {
            return chosenDate;
        };

        service.registerDayPickedEventHandler = function (scope, callback) {

            var unsub = $rootScope.$on('scheduleDayPickerDatePicked', callback);

            scope.$on('$destroy', unsub);

            return unsub;
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('ScreenStateService', service);

    function service() {

        var enabled = typeof window.plugins !== 'undefined' && typeof window.plugins.insomnia !== 'undefined';

        var service = {};

        service.stayOn = function () {

            if (!enabled) {
                return false;
            }

            return window.plugins.insomnia.keepAwake(function () {
                console.log('insomnia.keepAwake success');
            }, function (error) {
                console.log('insomnia.keepAwake error: ', error);
            });
        };

        service.dontStayOn = function () {

            if (!enabled) {
                return false;
            }

            return window.plugins.insomnia.allowSleepAgain(function () {
                console.log('insomnia.allowSleepAgain success');
            }, function (error) {
                console.log('insomnia.allowSleepAgain error: ', error);
            });
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('ScrollService', service);

    service.$inject = ['$ionicScrollDelegate', '$timeout', '$location'];

    function service($ionicScrollDelegate, $timeout, $location) {

        return {

            top: function () {

                $timeout(function () {

                    $ionicScrollDelegate.scrollTop(true);
                }, 100);
            },

            bottom: function () {

                $timeout(function () {

                    $ionicScrollDelegate.scrollBottom(true);
                }, 100);
            },

            anchor: function (id, handle) {

                $timeout(function () {

                    $location.hash(id);

                    if (handle) {

                        $ionicScrollDelegate.$getByHandle(handle).anchorScroll(true);
                    } else {

                        $ionicScrollDelegate.anchorScroll(true);
                    }
                }, 100);
            },

            position: function (xPos, yPos, handle) {

                $timeout(function () {

                    if (handle) {

                        $ionicScrollDelegate.$getByHandle(handle).scrollTo(xPos, yPos, true);
                    } else {

                        $ionicScrollDelegate.scrollTo(xPos, yPos, true);
                    }
                }, 100);
            },

            element: function (elementId, handle) {

                var el = document.getElementById(elementId);

                var top = 0;
                var left = 0;

                do {

                    top += el.offsetTop || 0;
                    left += el.offsetLeft || 0;

                    el = el.offsetParent;

                    if (el && el.tagName === 'ION-CONTENT') {
                        el = false;
                    }
                } while (el);

                if (el) {
                    this.position(0, top, handle);
                }
            }

        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('SideMenuService', service);

    service.$inject = ['LocalStorageService'];

    function service(LocalStorageService) {

        var service = {};

        var state = {
            seen: false
        };

        var storeState = function () {

            LocalStorageService.setKey('sideMenuState', JSON.stringify(state));
        };

        var getStoredState = function () {

            var getData = LocalStorageService.getKey('sideMenuState');

            if (!getData) {
                return false;
            }

            return JSON.parse(getData);
        };

        service.setup = function () {

            var storedState = getStoredState();

            if (storedState) {

                state.seen = storedState.seen;
            }
        };

        service.hasSeenSideMenu = function () {
            return state.seen;
        };

        service.sideMenuSeen = function () {

            state.seen = true;

            storeState();
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('SqlStorageService', service);

    service.$inject = ['$q'];

    function service($q) {

        var db;

        var commands = {

            table: function (name, fields) {

                return $q(function (resolve, reject) {

                    var sql = 'CREATE TABLE IF NOT EXISTS ' + name + ' (' + fields.join(', ') + ')';

                    commands.query(sql).then(function () {

                        console.log('SqlStorageService: ' + name + ' table is present.');
                        resolve(true);
                    }, function () {

                        console.error('SqlStorageService: unable to create ' + name + ' table in DB');
                        reject(true);
                    });
                });
            },

            dropTable: function (name) {

                return $q(function (resolve, reject) {

                    var sql = 'DROP TABLE IF EXISTS ' + name;

                    commands.query(sql).then(function () {

                        console.log('SqlStorageService: ' + name + ' table was dropped.');
                        resolve(true);
                    }, function () {

                        console.error('SqlStorageService: unable to drop ' + name + ' from DB');
                        reject(true);
                    });
                });
            },

            query: function (query, values) {

                values = values || [];

                var deferred = $q.defer();

                db.transaction(function (tx) {
                    tx.executeSql(query, values, function (tx, res) {

                        if (res) {

                            deferred.resolve(res, tx);
                        } else {

                            deferred.reject(false);
                        }
                    }, function (tx, err) {

                        deferred.reject(err);
                    });
                });

                return deferred.promise;
            },

            select: function (query, values) {

                values = values || [];

                var deferred = $q.defer();

                commands.query(query, values).then(function (res) {

                    if (res.rows.length > 0) {

                        deferred.resolve(res.rows.item(0));
                    } else {

                        deferred.reject({
                            message: '0 rows returned.'
                        });
                    }
                }, function (error) {

                    deferred.reject(error);
                });

                return deferred.promise;
            },

            selectAll: function (query, values) {

                values = values || [];

                var deferred = $q.defer();

                commands.query(query, values).then(function (results, tx) {

                    var resultArray = [];

                    for (var i = 0; i < results.rows.length; ++i) {
                        resultArray.push(results.rows.item(i));
                    }

                    deferred.resolve(resultArray, tx);
                }, function (error) {

                    deferred.reject(error);
                });

                return deferred.promise;
            }
        };

        var service = {

            openDB: function (db_name) {

                if (db) {
                    return db;
                }

                if (window.sqlitePlugin && device.platform.toUpperCase() === 'IOS') {

                    db = window.sqlitePlugin.openDatabase({
                        'name': db_name,
                        'location': 2,
                        'createFromLocation': 1
                    });
                } else if (window.openDatabase) {

                    db = window.openDatabase(db_name, '0.1', 'MyPTHub database', 1024 * 1024 * 5);
                } else {

                    db = null;
                }

                return db;
            },

            migrate: function () {

                var migrations = [{
                    sequence: 1,
                    run: function () {
                        return commands.query('ALTER TABLE sessions ADD COLUMN sync_fail_count INTEGER(11) NOT NULL DEFAULT 0;');
                    }
                }, {
                    sequence: 2,
                    run: function () {
                        return commands.query('ALTER TABLE workouts ADD COLUMN created_by VARCHAR(32) NOT NULL DEFAULT \'trainer\';');
                    }
                }, {
                    sequence: 3,
                    run: function () {
                        return commands.query('ALTER TABLE workouts ADD COLUMN customer_id INTEGER(11) DEFAULT NULL;');
                    }
                }, {
                    sequence: 4,
                    run: function () {
                        return commands.table('schedule_events', ['customer_id INTEGER(11) NOT NULL', 'trainer_id INTEGER(11) NOT NULL', 'trainer_user_id INTEGER(11) NOT NULL', 'schedule_id INTEGER(11) NOT NULL', 'data TEXT', 'date DATE NOT NULL', 'type TEXT NOT NULL']).then(function () {
                            commands.query('CREATE INDEX date_type_idx ON schedule_events (customer_id, schedule_id, date, type)');
                        });
                    }
                }];

                migrations.reduce(function (currentMigration, nextMigration) {

                    return currentMigration.then(function () {

                        var deferred = $q.defer();

                        commands.select('SELECT sequence FROM migrations WHERE sequence = ?', [nextMigration.sequence]).then(function (result) {

                            console.log('Migration already applied: ', nextMigration.sequence);
                            deferred.resolve();
                        }, function () {

                            nextMigration.run().then(commands.query("INSERT INTO migrations (sequence) VALUES(?)", [nextMigration.sequence])).then(function () {
                                console.log('Migration applied: ', nextMigration.sequence);
                                deferred.resolve();
                            }, function (response) {
                                console.log('Migration failed:', nextMigration.sequence);
                                console.log(response);
                            });
                        });

                        return deferred.promise;
                    });
                }, $q.when());
            },

            initDB: function (db_name) {

                if (!db && db_name) {
                    this.openDB(db_name);
                }

                // Create the DB tables
                commands.table('api_cache', ['key CHAR(64) NOT NULL PRIMARY KEY', 'value TEXT']).then(function () {
                    return commands.table('sessions', ['id TEXT NOT NULL', 'customer_id INTEGER(11) NOT NULL', 'trainer_id INTEGER(11) NOT NULL', 'trainer_user_id INTEGER(11) NOT NULL', 'completed INTEGER(1) NOT NULL DEFAULT 0', 'synced INTEGER(1) NOT NULL DEFAULT 0', 'adhoc INTEGER(1) NOT NULL DEFAULT 0', 'last_sync_attempt DATETIME', 'payload TEXT', 'data TEXT', 'date DATETIME', 'last_data_update DATETIME']).then(function () {
                        commands.query('CREATE INDEX id_idx ON sessions (id)');
                    });
                }).then(function () {
                    commands.table('workouts', ['id INTEGER(11) NOT NULL', 'title TEXT', 'user_id INTEGER(11) NOT NULL', 'trainer_id INTEGER(11) NOT NULL', 'trainer_user_id INTEGER(11) NOT NULL', 'enabled INTEGER(1) NOT NULL DEFAULT 0', 'data TEXT NOT NULL', 'last_data_update DATETIME NOT NULL']).then(function () {
                        commands.query('CREATE INDEX id_idx ON workouts (id)');
                    });
                }).then(function () {
                    return commands.table('customers', ['id INTEGER(11) NOT NULL', 'first_name TEXT NOT NULL', 'last_name TEXT NOT NULL', 'user_id INTEGER(11) NOT NULL', 'trainer_id INTEGER(11) NOT NULL', 'trainer_user_id INTEGER(11) NOT NULL', 'enabled INTEGER(1) NOT NULL DEFAULT 0', 'data TEXT NOT NULL', 'last_data_update DATETIME NOT NULL']).then(function () {
                        commands.query('CREATE INDEX id_idx ON customers (id)');
                    });
                }).then(commands.table('migrations', ['sequence INTEGER(11) NOT NULL PRIMARY KEY'])).then(service.migrate);
            },

            rebuildDb: function () {

                console.log('rebuilding DB');

                return commands.dropTable('api_cache').then(commands.dropTable('sessions')).then(commands.dropTable('workouts')).then(commands.dropTable('customers')).then(commands.dropTable('migrations')).then(commands.dropTable('schedule_events')).then(service.initDB());
            },

            getDB: function () {
                return db;
            },

            query: commands.query,

            select: commands.select,

            selectAll: commands.selectAll

        };

        return service;
    }
})();
(() => {
  angular.module('MyPTHub').factory('SubscriptionStateService', service);

  service.$inject = ['PermissionsService', '$rootScope', 'AuthenticationStateService'];

  function service(PermissionsService, $rootScope, AuthenticationStateService) {
    const service = {};

    function fireEventHandler() {
      $rootScope.$emit('SubscriptionStateChange');
    }

    const getSubscription = () => {
      const currentUser = PermissionsService.getCurrentUser();

      if (!currentUser || !currentUser.subscription) {
        return false;
      }

      return currentUser.subscription;
    };

    service.hasUnlimitedPlus = () => {
      if (PermissionsService.userIsTrainer()) {
        const subscription = getSubscription();
        if (subscription.permissions) {
          if (subscription.permissions.hasOwnProperty('can_use_unlimited_plus')) {
            return subscription.permissions.can_use_unlimited_plus;
          }
        }
      } else {
        const currentUser = PermissionsService.getCurrentUser();
        if (currentUser.permissions) {
          if (currentUser.permissions.hasOwnProperty('can_use_unlimited_plus')) {
            return currentUser.permissions.can_use_unlimited_plus;
          }
        }
      }
      return false;
    };

    service.hasCompanyStripe = () => PermissionsService.getCurrentUser().account_has_company_stripe;

    service.userCanUpgrade = () => {
      const subscription = getSubscription();

      if (!subscription) {
        return false;
      }

      if (!PermissionsService.userIsTrainer() || !PermissionsService.userIsTrainerOwner()) {
        return false;
      }

      return !subscription.is_subscribed || subscription.on_free_trial && !subscription.is_paid_subscription;
    };

    service.onFreeTrial = () => {
      const subscription = getSubscription();

      if (!subscription) {
        return false;
      }

      return subscription.on_free_trial && !subscription.is_paid_subscription;
    };

    service.getTrialEnd = () => {
      const subscription = getSubscription();

      if (!subscription) {
        return false;
      }

      return subscription.trial_ends_at;
    };

    service.canAccessPremiumFeatures = () => {
      const subscription = getSubscription();

      if (!subscription) {
        return false;
      }

      return subscription.is_subscribed;
    };

    service.hasUpgraded = () => {
      const subscription = getSubscription();

      return subscription.is_subscribed && subscription.is_paid_subscription;
    };

    service.everUpgraded = () => {
      const subscription = getSubscription();

      return subscription.ever_subscribed;
    };

    service.getPlan = () => {
      const subscription = getSubscription();

      return subscription.plan;
    };

    service.getPlatform = () => {
      const subscription = getSubscription();

      return subscription.subscription_platform;
    };

    service.schedulerIsEnabled = () => {
      const subscription = getSubscription();
      const currentUser = PermissionsService.getCurrentUser();

      if (PermissionsService.userIsCustomer()) {
        return currentUser.trainer_details.scheduler_enabled;
      }
      return subscription.scheduler_enabled;
    };

    service.stateChanged = () => {
      fireEventHandler();
    };

    service.registerChangeEventHandler = (scope, callback) => {
      const unsub = $rootScope.$on('SubscriptionStateChange', callback);

      scope.$on('$destroy', unsub);

      return unsub;
    };

    service.trainerIsSubscribed = () => {
      const subscription = getSubscription();
      const currentUser = AuthenticationStateService.getCurrentUser();

      if (PermissionsService.userIsCustomer()) {
        return currentUser.trainer_details.is_subscribed;
      }
      return subscription.is_subscribed;
    };

    return service;
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('ThreeDeeTouchQuickActionsService', service);

    service.$inject = ['PermissionsService', '$q', 'NavigateService'];

    function service(PermissionsService, $q, NavigateService) {
        var enabled = false;

        var service = {};

        var registerEventHandlers = function () {
            ThreeDeeTouch.onHomeIconPressed = function (payload) {
                switch (payload.type) {
                    case 'start_workout':
                        NavigateService.go('app.start_adhoc_session');
                        break;
                    case 'log_nutrition':
                        NavigateService.go('app.client_log_nutrition');
                        break;
                    case 'progress_photo':
                        NavigateService.go('app.client_progress_photos');
                        break;
                    case 'update_progress_photo':
                        NavigateService.go('app.client_progress_photos');
                        break;
                    case 'add_client':
                        NavigateService.go('app.trainer_create_client');
                        break;
                    case 'book_session':
                        NavigateService.go('app.trainer_book_session');
                        break;
                    case 'messages':
                        NavigateService.go('app.messages');
                        break;
                    case 'new_message':
                        NavigateService.go('app.new_message');
                        break;
                }
            };
        };

        service.setup = function () {
            if (typeof ThreeDeeTouch === 'undefined') {
                return $q.reject();
            }

            return $q(function (resolve, reject) {
                ThreeDeeTouch.isAvailable(function (available) {
                    enabled = available;

                    registerEventHandlers();

                    if (available) {
                        resolve();
                    } else {
                        reject();
                    }
                });
            });
        };

        service.setActions = function () {
            if (!enabled) {
                return false;
            }

            var actions = [];

            if (PermissionsService.userIsCustomer()) {
                actions.push({
                    type: 'start_workout',
                    title: 'Start Workout',
                    iconType: 'Time'
                });
                actions.push({
                    type: 'log_nutrition',
                    title: 'Log Nutrition',
                    iconType: 'Add'
                });
                actions.push({
                    type: 'messages',
                    title: 'Messages',
                    iconType: 'Message'
                });
                actions.push({
                    type: 'progress_photo',
                    title: 'Take Progress Photo',
                    iconType: 'CapturePhoto'
                });
            } else {
                actions.push({
                    type: 'add_client',
                    title: 'Add Client',
                    iconType: 'Contact'
                });
                actions.push({
                    type: 'book_session',
                    title: 'Book Session',
                    iconType: 'Date'
                });
                actions.push({
                    type: 'new_message',
                    title: 'New Message',
                    iconType: 'Compose'
                });
            }

            ThreeDeeTouch.configureQuickActions(actions);
        };

        service.resetActions = function () {
            if (!enabled) {
                return false;
            }

            ThreeDeeTouch.configureQuickActions([]);
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('timezonePicker', directive);

    function directive() {
        return {
            restrict: 'E',
            replace: true,
            scope: {
                tz: "=tz"
            },
            templateUrl: "core/TimezonePicker/TimezonePicker.html"
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('ToastService', service);

    service.$inject = ['$mdToast'];

    function service($mdToast) {

        return {

            show: function (message) {
                $mdToast.show($mdToast.simple().textContent(message));
            }

        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('UnitService', service);

    service.$inject = ['$rootScope', 'ApiClientService', 'PermissionsService', '$filter'];

    function service($rootScope, ApiClientService, PermissionsService, $filter) {

        var unitTypeFields = {
            bodyweight: 'bodyweight_display_units',
            weight: 'weight_display_units',
            height: 'height_display_units',
            length: 'length_display_units',
            distance: 'distance_display_units'
        };

        var currentUser = PermissionsService.getCurrentUser();

        if (currentUser) {

            var unitType = {
                bodyweight: currentUser[unitTypeFields.bodyweight],
                weight: currentUser[unitTypeFields.weight],
                height: currentUser[unitTypeFields.height],
                length: currentUser[unitTypeFields.length],
                distance: currentUser[unitTypeFields.distance]
            };
        } else {

            var unitType = {
                bodyweight: 'metric',
                weight: 'metric',
                height: 'metric',
                length: 'metric',
                distance: 'metric'
            };
        }

        var inchFractions = [{
            label: '',
            decimal: 0
        }, {
            label: '1/16',
            decimal: 0.0625
        }, {
            label: '1/8',
            decimal: 0.125
        }, {
            label: '3/16',
            decimal: 0.1875
        }, {
            label: '1/4',
            decimal: 0.25
        }, {
            label: '5/16',
            decimal: 0.3125
        }, {
            label: '3/8',
            decimal: 0.375
        }, {
            label: '7/16',
            decimal: 0.4375
        }, {
            label: '1/2',
            decimal: 0.5
        }, {
            label: '9/16',
            decimal: 0.5625
        }, {
            label: '5/8',
            decimal: 0.625
        }, {
            label: '11/16',
            decimal: 0.6875
        }, {
            label: '3/4',
            decimal: 0.75
        }, {
            label: '13/16',
            decimal: 0.8125
        }, {
            label: '7/8',
            decimal: 0.875
        }, {
            label: '15/16',
            decimal: 0.9375
        }];
        var fractionDecimals = [];
        inchFractions.map(function (fraction) {
            fractionDecimals.push(fraction.decimal);
        });

        var nearestInchFraction = function (val) {

            for (var i = 0, len = fractionDecimals.length; i < len; i++) {

                if (fractionDecimals[i] >= val) {

                    var d = fractionDecimals[i] - val;

                    var fractionDecimal = fractionDecimals[i - 1] + d >= val ? fractionDecimals[i - 1] : fractionDecimals[i];

                    var fractionObject = $filter('filter')(inchFractions, {
                        decimal: fractionDecimal
                    }, true);

                    if (fractionObject.length > 0) {
                        return fractionObject[0].label;
                    }
                }
            }

            return 0;
        };

        var metricLengthsToInches = function (type, value, decimalOutput) {

            decimalOutput = decimalOutput || false;

            if (value <= 0) {
                return 0;
            }

            var inchesDecimal = 0;

            // CM
            if (type === 'cm') {

                inchesDecimal = value * 0.3937008;

                // MM
            } else {

                inchesDecimal = value * 0.03937008;
            }

            if (decimalOutput) {
                return Math.round(inchesDecimal * 10) / 10;
            }

            var full = 0;
            var fraction = 0;

            var sourceValueParts = inchesDecimal.toString().split('.');

            if (sourceValueParts.length < 2) {

                if (sourceValueParts.length > 0) {

                    full = parseInt(sourceValueParts[0]);
                } else {

                    full = 0;
                }

                fraction = 0;

                return false;
            }

            var dec = parseFloat('0.' + sourceValueParts[1]).toFixed(4);

            full = parseInt(sourceValueParts[0]);

            fraction = nearestInchFraction(dec);

            if (fraction === 0 && dec > inchFractions[inchFractions.length - 1].decimal) {
                full += 1;
            }

            return (!fraction || fraction && full ? full : '') + (fraction ? ' ' + fraction : '');
        };

        var matericHeightToFeet = function (value) {

            var inches = value * 0.3937008;
            var remainder = inches / 12 * 12;

            return inches + '"' + (remainder > 0 ? ' ' + remainder + '\'' : '');
        };

        var metricWeightToStones = function (value) {

            var result = {
                stone: 0,
                lbs: 0
            };

            var splitWeight = (Math.round(value * 0.157473 * 100) / 100).toString().split('.');

            if (splitWeight[0]) {
                result.stone = parseInt(splitWeight[0]);
            }

            if (splitWeight[1]) {
                result.lbs = Math.round(parseFloat('0.' + splitWeight[1]) * 14);
            } else {
                result.lbs = 0;
            }

            return result;
        };

        var service = {

            changeUnitType: function (type, system, broadcast) {

                broadcast = broadcast || false;

                unitType[type] = system;

                if (broadcast) {
                    $rootScope.$broadcast(broadcast, true);
                    $rootScope.$broadcast('displayUnitChanged', true);
                }

                ApiClientService.users.setDisplayUnits(system, unitTypeFields[type]);
            },

            unitLabel: function (type, subType) {

                subType = subType || false;

                switch (type) {

                    case 'weight':
                        if (unitType.weight === 'imperial') {
                            return 'lbs';
                        }
                        return 'kg';
                        break;

                    case 'height':
                        if (unitType.height === 'imperial') {
                            return 'feet';
                        }
                        return 'm';
                        break;

                    case 'bodyweight':
                        if (unitType.bodyweight === 'imperial_us') {
                            return 'lbs';
                        }
                        if (unitType.bodyweight === 'imperial_gb') {
                            return 'st';
                        }
                        if (unitType.bodyweight === 'metric') {
                            return 'kg';
                        }
                        break;

                    case 'distance':
                        if (unitType.distance === 'imperial') {
                            return 'miles';
                        }
                        return 'km';
                        break;

                    case 'length':
                        if (unitType.length === 'imperial') {
                            return '"';
                        }
                        return subType || 'cm';
                        break;

                }
            },

            unitDisplay: function (type, metricValue, subType, addLabel, decimalOutput) {

                subType = subType || false;
                addLabel = addLabel || false;
                decimalOutput = decimalOutput || false;

                switch (type) {

                    case 'weight':

                        if (unitType['weight'] === 'imperial') {
                            // kg to lbs
                            var result = Math.round(metricValue * 2.20462 * 10) / 10;

                            if (addLabel) {
                                result += service.unitLabel(type, subType);
                            }

                            return result;
                        }

                        break;

                    case 'bodyweight':

                        if (unitType['bodyweight'] === 'imperial_gb') {

                            var imperialValues = metricWeightToStones(metricValue);

                            if (addLabel) {
                                result = (imperialValues.stone == -0 ? '- ' : '') + (imperialValues.stone !== 0 ? imperialValues.stone + 'st. ' : '') + (imperialValues.lbs !== 0 ? imperialValues.lbs + 'lbs ' : '');
                            } else {
                                result = parseFloat(imperialValues.stone + Math.round(imperialValues.lbs / 14 * 100) / 100);
                            }

                            return result;
                        } else if (unitType['bodyweight'] === 'imperial_us') {

                            // kg to lbs
                            var result = Math.round(metricValue * 2.20462 * 10) / 10;

                            if (addLabel) {
                                result += service.unitLabel(type, subType);
                            }

                            return result;
                        }

                        break;

                    case 'height':

                        var result;

                        if (unitType['height'] === 'imperial') {

                            var splitHeight = (Math.round(metricValue * 100) / 100 * 0.032808399).toFixed(2).toString().split('.');

                            if (splitHeight[0]) {
                                result = splitHeight[0] + '"';
                            }

                            if (splitHeight[1]) {
                                result += ' ' + Math.round(parseFloat('0.' + splitHeight[1]) * 12) + '\'';
                            }
                        } else {

                            result = (Math.round(metricValue) / 100).toFixed(2) + 'm';
                        }

                        return result;

                        break;

                    case 'distance':

                        if (unitType['distance'] === 'imperial') {

                            // km to miles
                            var result = Math.round(metricValue * 0.621371 * 10) / 10;

                            if (addLabel) {
                                result += service.unitLabel(type, subType);
                            }

                            return result;
                        }

                        break;

                    case 'length':

                        if (unitType['length'] === 'imperial') {

                            // cm/mm to inches
                            var result = metricLengthsToInches(subType, metricValue, decimalOutput);

                            if (addLabel) {
                                result += service.unitLabel(type, subType);
                            }

                            return result;
                        }

                        break;
                }

                var result = Math.round(metricValue * 10) / 10;

                if (addLabel) {
                    result += service.unitLabel(type, subType);
                }

                return result;
            },

            getCurrentUnitType: function (unit) {
                return unitType[unit];
            }

        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('unitServiceSwitcher', directive);

    function directive() {
        return {
            restrict: 'AE',
            scope: {
                unit: '@unit',
                broadcastOnChange: '@?',
                metricUnitLabel: '@?',
                imperialUnitLabel: '@?',
                imperialGbUnitLabel: '@?'
            },
            templateUrl: "core/UnitSwitcher/UnitSwitcherDirective.html",
            replace: true,
            controller: controller
        };
    }

    controller.$inject = ['$scope', 'UnitService'];

    function controller($scope, UnitService) {

        $scope.unitType = UnitService.getCurrentUnitType($scope.unit);

        if ($scope.unit === 'bodyweight' && $scope.unitType === 'imperial_us') {
            $scope.unitType = 'imperial';
        }

        if ($scope.metricUnitLabel) {
            $scope.metricLabel = $scope.metricUnitLabel;
        }

        if ($scope.imperialUnitLabel) {
            $scope.imperialLabel = $scope.imperialUnitLabel;
        }

        if (!$scope.metricLabel || !$scope.imperialLabel) {

            switch ($scope.unit) {

                case 'weight':
                    if (!$scope.metricLabel) {
                        $scope.metricLabel = 'kg';
                    }
                    if (!$scope.imperialLabel) {
                        $scope.imperialLabel = 'lbs';
                    }
                    break;

                case 'height':
                    if (!$scope.metricLabel) {
                        $scope.metricLabel = 'metres';
                    }
                    if (!$scope.imperialLabel) {
                        $scope.imperialLabel = 'feet';
                    }
                    break;

                case 'bodyweight':
                    if (!$scope.metricLabel) {
                        $scope.metricLabel = 'kg';
                    }
                    if (!$scope.imperialGbLabel) {
                        $scope.imperialGbLabel = 'st';
                    }
                    if (!$scope.imperialLabel) {
                        $scope.imperialLabel = 'lbs';
                    }
                    break;

                case 'distance':
                    if (!$scope.metricLabel) {
                        $scope.metricLabel = 'km';
                    }
                    if (!$scope.imperialLabel) {
                        $scope.imperialLabel = 'miles';
                    }
                    break;

                case 'length':
                    if (!$scope.metricLabel) {
                        $scope.metricLabel = 'cm';
                    }
                    if (!$scope.imperialLabel) {
                        $scope.imperialLabel = 'inches';
                    }
                    break;

            }
        }

        $scope.changeUnit = function (type) {

            $scope.unitType = type;

            if ($scope.unit === 'bodyweight' && $scope.unitType === 'imperial') {
                type = 'imperial_us';
            }

            UnitService.changeUnitType($scope.unit, type, $scope.broadcastOnChange);
        };

        $scope.clicked = function (event) {
            event.stopPropagation();
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('VibrateService', service);

    service.$inject = ['$cordovaVibration', 'DeviceInfoService'];

    function service($cordovaVibration, DeviceInfoService) {

        return {
            vibrate: function (times) {

                if (DeviceInfoService.isBrowser()) {
                    return false;
                }

                $cordovaVibration.vibrate(times);
            }
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('workoutViewer', directive);

    function directive() {

        return {
            restrict: 'E',
            replace: false,
            templateUrl: "core/WorkoutViewer/workoutViewerDirective.html",
            controller: controller,
            scope: {
                workout: '=',
                status: '=?',
                gender: '=?'
            },
            controllerAs: 'workoutViewerVm' // needs to be unique, as used inside other scopes
        };
    }

    controller.$inject = ['$scope', 'ExerciseViewerService'];

    function controller($scope, ExerciseViewerService) {

        var workoutViewerVm = this;

        workoutViewerVm.workout = $scope.workout;

        var gender = $scope.gender;

        workoutViewerVm.status = 'live';
        if ($scope.status) {
            workoutViewerVm.status = $scope.status;
        }

        workoutViewerVm.viewExercise = function (exercise) {

            var nameId = false;

            if (exercise.name_id) {
                nameId = exercise.name_id;
            }

            ExerciseViewerService.open(exercise, nameId, gender);
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('ConfirmDialogService', service);

    service.$inject = ['$q'];

    function service($q) {

        var service = {};

        service.open = function (config) {

            return $q(function (resolve, reject) {

                var onConfirm = function (buttonIndex) {

                    if (buttonIndex === 1) {
                        resolve();
                    } else {
                        reject();
                    }
                };

                // device
                if (navigator.notification) {

                    navigator.notification.confirm(config.message, onConfirm, config.title, [config.buttonConfirm, config.buttonCancel]);

                    // browser (for dev purposes)
                } else {

                    if (confirm(config.message)) {
                        resolve();
                    } else {
                        reject();
                    }
                }
            });
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('exerciseImage', directive);

    directive.$inject = ['$sce', 'ImageCacheService', 'mpthconf', '$http'];

    function directive($sce, ImageCacheService, mpthconf, $http) {
        return {
            restrict: 'A',
            replace: false,
            scope: {
                exercise: '=exerciseImage'
            },
            link: function (scope, element, attrs) {

                element.addClass('notLoaded');
                element.addClass('hideUntilLoaded');

                var exercise = scope.exercise;

                var defaultImage = 'img/novid.jpg';

                scope.$watch('exercise', function (exercise) {

                    if (typeof exercise === 'undefined' || !exercise) {
                        return false;
                    }

                    // Default image when no video
                    var imageUrl = defaultImage;

                    if (exercise.video_id) {
                        if (exercise.video_type == 'vimeo') {
                            $http.jsonp('https://vimeo.com/api/v2/video/' + exercise.video_id + '.json?callback=JSON_CALLBACK').success(function (data) {
                                imageUrl = data[0]['thumbnail_medium'];

                                attrs.$set('src', $sce.trustAsResourceUrl(imageUrl));
                            });
                        } else {
                            imageUrl = 'https://img.youtube.com/vi/' + exercise.video_id + '/mqdefault.jpg';
                        }
                    } else if (exercise.images && exercise.images.length > 0) {
                        imageUrl = mpthconf.cdn.base + 'exercise/s/' + exercise.images[0].filename;
                    }

                    if (imageUrl === defaultImage) {
                        attrs.$set('src', $sce.trustAsResourceUrl(imageUrl));
                    } else {
                        ImageCacheService.get(imageUrl).then(function (result) {
                            attrs.$set('src', $sce.trustAsResourceUrl(result));
                        });
                    }
                });

                element.bind("load", function (e) {
                    element.removeClass('notLoaded');
                });

                element.bind('error', function () {
                    attrs.$set('src', defaultImage);
                });
            }
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('MarkdownService', service);

    service.$inject = ['$q'];

    function service($q) {

        var service = {};

        var und = new upndown();

        service.markdownToHtml = function (sourceString, renderers) {

            renderers = renderers || {};

            if (!sourceString) {
                return false;
            }

            var renderer = new marked.Renderer();

            for (var key in renderers) {
                console.log('render set:', key);
                if (renderers.hasOwnProperty(key)) {
                    renderer[key] = renderers[key];
                }
            }

            return marked(sourceString, {
                renderer: renderer
            });
        };

        service.htmlToMarkdown = function (sourceString) {

            var deferred = $q.defer();

            und.convert(sourceString, function (err, markdown) {

                if (err) {
                    deferred.reject(err);
                    return false;
                }

                deferred.resolve(markdown || '');
            }, { keepHtml: false });

            return deferred.promise;
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('multiBodyWeight', directive);

    function directive() {
        return {
            restrict: 'A',
            replace: true,
            scope: {
                weight: '=value',
                system: '=system',
                label: '@?'
            },
            templateUrl: "core/metricfields/bodyWeight.html",
            controller: controller
        };
    }

    controller.$inject = ['$scope'];

    function controller($scope) {

        $scope.weight_dec = 0;
        $scope.weight_main = 0;

        $scope.applyWeight = function () {

            if (!$scope.weight_main) {
                $scope.weight_main = 0;
            }

            if (!$scope.weight_dec) {
                $scope.weight_dec = 0;
            }

            if ($scope.system === 'metric') {

                $scope.weight = parseFloat($scope.weight_main + '.' + $scope.weight_dec);
            } else if ($scope.system === 'imperial_gb') {

                var stoneToKg = parseInt($scope.weight_main) * 6.35029;
                var lbsToKg = parseInt($scope.weight_dec) * 0.453592;

                $scope.weight = stoneToKg + lbsToKg;
            } else {

                $scope.weight = (parseFloat($scope.weight_main) + parseFloat('0.' + $scope.weight_dec)) * 0.453592;
            }
        };

        $scope.setWeightSelects = function () {

            if (!$scope.weight) {
                $scope.weight = 0;
            }

            switch ($scope.system) {

                case 'metric':

                    // Kg

                    var splitWeight = (Math.round($scope.weight * 10) / 10).toString().split('.');

                    if (splitWeight[0]) {
                        $scope.weight_main = parseInt(splitWeight[0]);

                        if ($scope.weight_main > 500) {
                            $scope.weightMainNumbers.push($scope.weight_main);
                        }
                    }

                    if (splitWeight[1]) {
                        $scope.weight_dec = Math.round(parseFloat('0.' + splitWeight[1]) * 10);
                    } else {
                        $scope.weight_dec = 0;
                    }

                    break;

                case 'imperial_gb':

                    // Stone / Lbs

                    var splitWeight = (Math.round($scope.weight * 0.157473 * 100) / 100).toString().split('.');

                    if (splitWeight[0]) {
                        $scope.weight_main = parseInt(splitWeight[0]);

                        if ($scope.weight_main > 78) {
                            $scope.weightMainNumbers.push($scope.weight_main);
                        }
                    }

                    if (splitWeight[1]) {
                        $scope.weight_dec = Math.round(parseFloat('0.' + splitWeight[1]) * 14);
                    } else {
                        $scope.weight_dec = 0;
                    }

                    break;

                case 'imperial_us':

                    // Lbs

                    var splitWeight = (Math.round($scope.weight * 2.20462 * 100) / 100).toString().split('.');

                    if (splitWeight[0]) {
                        $scope.weight_main = parseInt(splitWeight[0]);

                        if ($scope.weight_main > 450) {
                            $scope.weightMainNumbers.push($scope.weight_main);
                        }
                    }

                    if (splitWeight[1]) {
                        $scope.weight_dec = Math.round(parseFloat('0.' + splitWeight[1]) * 10);
                    } else {
                        $scope.weight_dec = 0;
                    }

                    break;

            }
        };

        $scope.changeWeightType = function () {

            $scope.weightMainNumbers = [0];
            $scope.weightDecNumbers = [0];

            var maxMain = 0;
            var maxDec = 0;

            switch ($scope.system) {

                case 'metric':

                    // Kg

                    maxMain = 500;
                    maxDec = 9;

                    break;

                case 'imperial_gb':

                    // Stone / Lbs
                    maxMain = 78;
                    maxDec = 13;

                    break;

                case 'imperial_us':

                    // Lbs
                    maxMain = 650;
                    maxDec = 9;

                    break;

            }

            var i = 0;

            while (i < maxMain) {

                i++;

                $scope.weightMainNumbers.push(i);
            }

            i = 0;

            while (i < maxDec) {

                i++;

                $scope.weightDecNumbers.push(i);
            }

            $scope.setWeightSelects();
        };

        var watchSystem = function () {
            $scope.$watch('system', function (system) {
                $scope.changeWeightType();
            });
        };

        var unregister = $scope.$watch('weight', function (val) {

            if (typeof val !== "undefined") {

                watchSystem();

                if (val) {
                    unregister();
                }
            }
        });
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('multiHeight', directive);

    function directive() {
        return {
            restrict: 'A',
            replace: true,
            scope: {
                height: '=value',
                system: '=system',
                label: '@?'
            },
            templateUrl: "core/metricfields/height.html",
            controller: controller
        };
    }

    controller.$inject = ['$scope'];

    function controller($scope) {

        $scope.height_dec = 0;
        $scope.height_main = 0;

        $scope.applyHeight = function () {
            if ($scope.system === 'metric') {
                var m_to_cm = parseInt($scope.height_main) * 100;
                $scope.height = m_to_cm + parseInt($scope.height_dec);
            } else {
                var height_feet_to_cm = parseInt($scope.height_main) * 30.48;
                var height_inches_to_cm = parseInt($scope.height_dec) * 2.54;
                $scope.height = height_feet_to_cm + height_inches_to_cm;
            }
        };

        $scope.setHeightSelects = function () {

            if (!$scope.height) {
                return false;
            }

            if ($scope.system === 'imperial') {

                var splitHeight = (Math.round($scope.height * 100) / 100 * 0.032808399).toFixed(2).toString().split('.');

                if (splitHeight[0]) {
                    $scope.height_main = parseInt(splitHeight[0]);
                }

                if (splitHeight[1]) {
                    $scope.height_dec = Math.round(parseFloat('0.' + splitHeight[1]) * 12);
                } else {
                    $scope.height_dec = 0;
                }
            } else {

                var splitHeight = (Math.round($scope.height) / 100).toFixed(2).toString().split('.');

                if (splitHeight[0]) {
                    $scope.height_main = parseInt(splitHeight[0]);
                }

                if (splitHeight[1]) {
                    $scope.height_dec = parseInt(splitHeight[1]);
                } else {
                    $scope.height_dec = 0;
                }
            }
        };

        $scope.changeHeightType = function () {

            $scope.heightMainNumbers = [0];
            var b = 0;

            if ($scope.system === "metric") {
                var max = 2;
            } else {
                var max = 9;
            }

            while (b < max) {
                b += 1;
                $scope.heightMainNumbers.push(b);
            }

            $scope.heightDecNumbers = [0];
            var b = 0;

            if ($scope.system === "metric") {
                var max = 99;
            } else {
                var max = 11;
            }

            while (b < max) {
                b += 1;
                $scope.heightDecNumbers.push(b);
            }

            $scope.setHeightSelects();
        };

        var watchSystem = function () {
            $scope.$watch('system', function (system) {
                $scope.changeHeightType();
            });
        };

        var unregister = $scope.$watch('height', function (val) {

            if (typeof val !== "undefined") {
                watchSystem();

                if (val) {
                    unregister();
                }
            }
        });
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('multiLength', directive);

    function directive() {
        return {
            restrict: 'A',
            replace: true,
            scope: {
                sourceValue: '=value',
                valueType: '@valueType',
                system: '=system',
                label: '@?'
            },
            templateUrl: "core/metricfields/length.html",
            controller: controller
        };
    }

    controller.$inject = ['$scope'];

    function controller($scope) {

        $scope.fractions = [{
            label: '0',
            decimal: 0
        }, {
            label: '1/16',
            decimal: 0.0625
        }, {
            label: '1/8',
            decimal: 0.125
        }, {
            label: '3/16',
            decimal: 0.1875
        }, {
            label: '1/4',
            decimal: 0.25
        }, {
            label: '5/16',
            decimal: 0.3125
        }, {
            label: '3/8',
            decimal: 0.375
        }, {
            label: '7/16',
            decimal: 0.4375
        }, {
            label: '1/2',
            decimal: 0.5
        }, {
            label: '9/16',
            decimal: 0.5625
        }, {
            label: '5/8',
            decimal: 0.625
        }, {
            label: '11/16',
            decimal: 0.6875
        }, {
            label: '3/4',
            decimal: 0.75
        }, {
            label: '13/16',
            decimal: 0.8125
        }, {
            label: '7/8',
            decimal: 0.875
        }, {
            label: '15/16',
            decimal: 0.9375
        }];

        var fractionDecimals = [];
        $scope.fractions.map(function (fraction) {
            fractionDecimals.push(fraction.decimal);
        });

        $scope.length_main = 0;
        $scope.length_fraction = 0;

        $scope.applyValueToModel = function () {

            if (!$scope.length_main) {
                $scope.length_main = 0;
            }

            if (!$scope.length_fraction) {
                $scope.length_fraction = 0;
            }

            if ($scope.system === 'metric') {

                $scope.sourceValue = parseFloat($scope.length_main);
            } else {

                // CM
                if ($scope.valueType === 'cm') {

                    $scope.sourceValue = (parseFloat($scope.length_main) + parseFloat($scope.length_fraction)) * 2.54;

                    //MM
                } else {

                    $scope.sourceValue = (parseFloat($scope.length_main) + parseFloat($scope.length_fraction)) * 25.4;
                }
            }
        };

        var metricToImperial = function () {

            if ($scope.valueType === 0) {
                return 0;
            }

            // CM
            if ($scope.valueType === 'cm') {

                return $scope.sourceValue * 0.3937008;

                // MM
            } else {

                return $scope.sourceValue * 0.03937008;
            }
        };

        var nearestFraction = function (val) {

            for (var i = 0, len = fractionDecimals.length; i < len; i++) {

                if (fractionDecimals[i] >= val) {

                    var d = fractionDecimals[i] - val;

                    return fractionDecimals[i - 1] + d >= val ? fractionDecimals[i - 1] : fractionDecimals[i];
                }
            }

            return 0;
        };

        $scope.setupFields = function () {

            if (!$scope.sourceValue) {
                $scope.sourceValue = 0;
            }

            if ($scope.system === 'metric') {

                var metricMainVal = Math.round($scope.sourceValue * 10) / 10;

                $scope.length_main = metricMainVal > 0 ? metricMainVal : null;
                $scope.length_fraction = 0;

                $scope.showFraction = false;

                return false;
            }

            var sourceValueParts = metricToImperial().toString().split('.');

            $scope.showFraction = true;

            if (sourceValueParts.length < 2) {

                if (sourceValueParts.length > 0) {

                    var impMainvaL = parseInt(sourceValueParts[0]);

                    $scope.length_main = impMainvaL > 0 ? impMainvaL : null;
                } else {

                    $scope.length_main = null;
                }

                $scope.length_fraction = 0;

                return false;
            }

            var dec = parseFloat('0.' + sourceValueParts[1]).toFixed(4);

            $scope.length_main = parseInt(sourceValueParts[0]);
            $scope.length_fraction = nearestFraction(dec);

            if ($scope.length_fraction === 0 && dec > $scope.fractions[$scope.fractions.length - 1].decimal) {
                $scope.length_main += 1;
            }
        };

        var watchSystem = function () {
            $scope.$watch('system', function (system) {
                $scope.setupFields();
            });
        };

        var unregister = $scope.$watch('sourceValue', function (val) {

            if (typeof val !== "undefined") {

                watchSystem();

                if (val) {
                    unregister();
                }
            }
        });
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('multiWeight', directive);

    function directive() {
        return {
            restrict: 'A',
            replace: true,
            scope: {
                weight: '=value',
                system: '=system',
                label: '@?'
            },
            templateUrl: "core/metricfields/weight.html",
            controller: controller
        };
    }

    controller.$inject = ['$scope'];

    function controller($scope) {

        $scope.weight_dec = 0;
        $scope.weight_main = 0;

        $scope.applyWeight = function () {

            if (!$scope.weight_main) {
                $scope.weight_main = 0;
            }

            if (!$scope.weight_dec) {
                $scope.weight_dec = 0;
            }

            if ($scope.system === 'metric') {

                $scope.weight = parseFloat($scope.weight_main);
            } else {

                $scope.weight = (parseFloat($scope.weight_main) + parseFloat('0.' + $scope.weight_dec)) * 0.453592;
            }
        };

        $scope.setWeightSelects = function () {

            if (!$scope.weight) {
                $scope.weight = 0;
            }

            switch ($scope.system) {

                case 'metric':

                    // Kg

                    var metricMainVal = Math.round($scope.weight * 10) / 10;

                    $scope.weight_main = metricMainVal > 0 ? metricMainVal : null;
                    $scope.weight_dec = 0;

                    break;

                case 'imperial':

                    // Lbs

                    var splitWeight = (Math.round($scope.weight * 2.20462 * 100) / 100).toString().split('.');

                    if (splitWeight[0]) {
                        $scope.weight_main = parseInt(splitWeight[0]);

                        if ($scope.weight_main > 450) {
                            $scope.weightMainNumbers.push($scope.weight_main);
                        }
                    }

                    if (splitWeight[1]) {
                        $scope.weight_dec = Math.round(parseFloat('0.' + splitWeight[1]) * 10);
                    } else {
                        $scope.weight_dec = 0;
                    }

                    break;

            }
        };

        $scope.changeWeightType = function () {

            $scope.weightMainNumbers = [0];
            $scope.weightDecNumbers = [0];

            var maxMain = 0;
            var maxDec = 0;

            switch ($scope.system) {

                case 'metric':

                    // Kg

                    maxMain = 500;
                    maxDec = 9;

                    break;

                case 'imperial':

                    // Lbs
                    maxMain = 450;
                    maxDec = 9;

                    break;

            }

            var i = 0;

            while (i < maxMain) {

                i++;

                $scope.weightMainNumbers.push(i);
            }

            i = 0;

            while (i < maxDec) {

                i++;

                $scope.weightDecNumbers.push(i);
            }

            $scope.setWeightSelects();
        };

        var watchSystem = function () {
            $scope.$watch('system', function (system) {
                $scope.changeWeightType();
            });
        };

        var unregister = $scope.$watch('weight', function (val) {

            if (typeof val !== "undefined") {

                watchSystem();

                if (val) {
                    unregister();
                }
            }
        });
    }
})();
(() => {
  angular.module('MyPTHub').factory('NavigateService', service);

  service.$inject = ['$ionicHistory', '$state', 'PermissionsService'];

  function service($ionicHistory, $state, PermissionsService) {
    const animationDefault = ionic.Platform.isAndroid();
    let disableBack = false;
    let disableAnimate = false;
    let root = false;
    let locationReplace = false;

    const methods = {
      disableBack: () => {
        disableBack = true;
        return methods;
      },

      disableAnimation: () => {
        disableAnimate = true;
        return methods;
      },

      isRoot: () => {
        root = true;
        return methods;
      },

      replaceCurrentState: () => {
        locationReplace = true;
        return methods;
      },

      go: (stateName, params = {}, options = {}, transitionTo = false) => {
        if (locationReplace) {
          options.location = 'replace';
        }

        $ionicHistory.nextViewOptions({
          disableAnimate,
          disableBack,
          historyRoot: root
        });

        let doNav;
        if (transitionTo) {
          doNav = $state.transitionTo(stateName, params, options);
        } else {
          doNav = $state.go(stateName, params, options);
        }

        if (disableBack) {
          disableBack = false;
        }

        if (disableAnimate && !animationDefault) {
          disableAnimate = animationDefault;
        }

        if (root) {
          root = false;
        }

        return methods.clearCache().then(() => doNav.then(() => {
          if (locationReplace) {
            $ionicHistory.removeBackView();
            locationReplace = false;
          }
        }));
      },

      transitionTo: (stateName, params, options) => methods.go(stateName, params, options, true),

      goHome: () => {
        let stateName;

        if (PermissionsService.userIsTrainer()) {
          stateName = 'app.trainer_home';
        } else {
          stateName = 'app.client_home';
        }

        methods.clearCache().then(() => {
          methods.isRoot().replaceCurrentState().disableBack().go(stateName);
        });
      },

      goBack: amount => {
        let howManySteps = amount;

        if (typeof howManySteps === 'undefined') {
          howManySteps = -1;
        }

        $ionicHistory.goBack(howManySteps);
      },

      clearCache: () => $ionicHistory.clearCache(),

      removeCache: states => $ionicHistory.clearCache(states),

      clearHistory: () => {
        $ionicHistory.clearHistory();
        return methods;
      }
    };

    return methods;
  }
})();
(() => {
  class TrainerCalendarController {
    /**
     * @param $scope
     * @param AnalyticsService
     * @param CustomerSwitcherService
     * @param NavigateService
     * @param ScheduleDayPickerService
     * @param DayScheduleService
     * @param SessionScheduleService
     * @param NetworkStateService
     * @param SessionStoreService
     * @param $stateParams
     * @param PermissionsService
     * @param ApiClientService
     * @param SubscriptionStateService
     */
    constructor($scope, AnalyticsService, CustomerSwitcherService, NavigateService, ScheduleDayPickerService, DayScheduleService, SessionScheduleService, NetworkStateService, SessionStoreService, $stateParams, PermissionsService, ApiClientService, SubscriptionStateService) {
      this.$scope = $scope;
      this.AnalyticsService = AnalyticsService;
      this.CustomerSwitcherService = CustomerSwitcherService;
      this.NavigateService = NavigateService;
      this.ScheduleDayPickerService = ScheduleDayPickerService;
      this.DayScheduleService = DayScheduleService;
      this.SessionScheduleService = SessionScheduleService;
      this.NetworkStateService = NetworkStateService;
      this.SessionStoreService = SessionStoreService;
      this.$stateParams = $stateParams;
      this.PermissionsService = PermissionsService;
      this.ApiClientService = ApiClientService;
      this.datePickerToggle = ScheduleDayPickerService.toggleOpen;
      this.SubscriptionStateService = SubscriptionStateService;

      this.active = false;
      this.isMine = false;
      this.hasUnlimitedPlus = SubscriptionStateService.hasUnlimitedPlus();
      this.userIsTrainer = PermissionsService.userIsTrainer();
      this.currentUser = PermissionsService.getCurrentUser();
      this.selectedTrainerId = $stateParams.trainerUserId;

      if (this.userIsTrainer) {
        if (this.currentUser.hasOwnProperty('permissions_active')) {
          this.canBookEvents = this.currentUser.permissions_active.can_book_events;
        } else {
          this.canBookEvents = true;
        }
      }

      // eslint-disable-next-line consistent-return
      CustomerSwitcherService.onSwitched($scope, (ev, customer) => {
        if (!this.active) {
          return false;
        }

        if (customer.customer_id) {
          console.log('clicked customer');
          this.NavigateService.replaceCurrentState().go('app.client_calendar', {
            customerId: customer.customer_id
          });
        } else if (customer.hasOwnProperty('type')) {
          console.log('clicked company');
          this.NavigateService.replaceCurrentState().go('app.company_calendar');
        } else {
          console.log('clicked trainer', customer);
          NavigateService.replaceCurrentState().go('app.trainer_calendar', {
            trainerUserId: customer.trainer_user_id ? customer.trainer_user_id : this.currentUser.trainer_user_id
          });
        }
      });

      // eslint-disable-next-line consistent-return
      ScheduleDayPickerService.registerDayPickedEventHandler($scope, () => {
        if (!this.active) {
          return false;
        }

        this.setDate();
      });

      $scope.$on('$ionicView.afterEnter', () => {
        this.AnalyticsService.trackView('Trainer - Schedule');
        this.active = true;
      });

      $scope.$on('$ionicView.beforeEnter', () => {
        this.trainerUserId = $stateParams.trainerUserId || null;
        this.isMine = !this.trainerUserId || this.trainerUserId === PermissionsService.getLoggedInUserId();
        this.setDate();
      });

      $scope.$on('$ionicView.beforeLeave', () => {
        this.active = false;
      });
    }

    setDate(reset = false) {
      if (reset) {
        this.DayScheduleService.reset();
      }

      this.currentDate = this.ScheduleDayPickerService.getDate();
      this.currentDateFormatted = this.currentDate.format('Do MMMM YYYY');
      this.load(true);
      this.DayScheduleService.setDate(this.currentDate);
    }

    load(init = false) {
      const chosenDate = this.ScheduleDayPickerService.getDate();
      const dateStart = moment(chosenDate).startOf('day'); //.subtract(6, 'week');
      const dateEnd = moment(chosenDate).endOf('day'); //.add(6, 'week');

      if (!this.userIsTrainer) {
        const data = {
          startDate: dateStart.format('YYYY-MM-DD'),
          endDate: dateEnd.format('YYYY-MM-DD'),
          selectedUser: this.trainerUserId,
          type: 'trainer'
        };

        this.ApiClientService.calendarEventTimes.getAllFrom(data).then(response => {
          this.DayScheduleService.setEvents(response.events);
        });
      } else {
        // eslint-disable-next-line consistent-return
        this.SessionScheduleService.getTrainerSchedule(dateStart, dateEnd, this.trainerUserId).then(response => {
          if (!this.active) {
            return false;
          }

          const data = {
            startDate: dateStart.format('YYYY-MM-DD'),
            endDate: dateEnd.format('YYYY-MM-DD'),
            selectedUser: this.trainerUserId ? this.trainerUserId : this.currentUser.id,
            type: 'trainer'
          };

          this.ApiClientService.calendarEventTimes.getAllFrom(data).then(response => {
            this.DayScheduleService.setEvents(response.events);
          });

          this.DayScheduleService.setSessions(response);
        });

        // eslint-disable-next-line consistent-return
        if (init && this.NetworkStateService.isOnline()) {
          this.SessionStoreService.downloadTrainerSessions(dateStart, dateEnd, this.trainerUserId).then(() => {
            this.load();
          });
        }
      }
    }
  }

  const config = $stateProvider => {
    $stateProvider.state('app.trainer_calendar', {
      url: '/trainer_calendar/:trainerUserId?',
      views: {
        menuContent: {
          templateUrl: 'trainer/calendar/calendar.html',
          controller: 'TrainerCalendarController',
          controllerAs: 'vm'
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('TrainerCalendarController', TrainerCalendarController);
})();
(() => {
  class TrainerCalendarEventBookingController {
    /**
     * @param event
     * @param $stateParams
     * @param $timeout
     * @param AuthenticationStateService
     * @param $ionicActionSheet
     * @param NavigateService
     * @param PermissionsService
     * @param ApiClientService
     */
    constructor(event, $stateParams, $timeout, AuthenticationStateService, $ionicActionSheet, NavigateService, PermissionsService, ApiClientService, ToastService, credits, $state) {
      this.event = event;
      this.$stateParams = $stateParams;
      this.$timeout = $timeout;
      this.AuthenticationStateService = AuthenticationStateService;
      this.$ionicActionSheet = $ionicActionSheet;
      this.NavigateService = NavigateService;
      this.PermissionsService = PermissionsService;
      this.ApiClientService = ApiClientService;
      this.ToastService = ToastService;
      this.credits = credits;
      this.$state = $state;

      this.showImage = true;
      this.status = null;
      this.currentUser = AuthenticationStateService.getCurrentUser();
      this.userIsTrainer = PermissionsService.userIsTrainer();
      this.credit_id = false;
      this.processing = false;
      this.image = `https://d3qcjjq30235ix.cloudfront.net/calendarEvent/banner/${this.event.image}`;
      if (!this.event.image) {
        this.showImage = false;
      }

      this.activeBookings = () => this.event.bookings.filter(booking => booking.status === 'active' || booking.status === 'attended');
      this.waitingBookings = () => this.event.bookings.filter(booking => booking.status === 'waiting');
      this.cancelledBookings = () => this.event.bookings.filter(booking => booking.status === 'cancelled');
      this.checkInCustomerCount = () => this.event.bookings.filter(booking => booking.status === 'attended');

      this.initMap();
      this.checkStatus();
      $timeout(() => {
        const updateTimes = () => {
          this.startTime = moment.utc($stateParams.date_time).tz(this.currentUser.timezone).format('h:mm A');
          this.endTime = moment.utc($stateParams.date_time).add('minutes', this.event.duration).tz(this.currentUser.timezone).format('h:mm A');
          if (this.event.global_exceptions && this.event.global_exceptions.new_date_time) {
            this.startTime = moment.utc(this.event.global_exceptions.new_date_time).tz(this.currentUser.timezone).format('h:mm A');
            this.endTime = moment.utc(this.event.global_exceptions.new_date_time).add('minutes', this.event.duration).tz(this.currentUser.timezone).format('h:mm A');
          }
          if (this.event.exceptions) {
            this.startTime = moment.utc(this.event.exceptions.new_date_time).tz(this.currentUser.timezone).format('h:mm A');
            this.endTime = moment.utc(this.event.exceptions.new_date_time).add('minutes', this.event.duration).tz(this.currentUser.timezone).format('h:mm A');
          }
        };
        updateTimes();
      });
    }

    checkStatus() {
      if (this.event.completed) {
        this.status = 'completed';
      } else {
        this.status = null;
      }
    }

    /**
     * @returns {boolean}
     */
    isCancelled() {
      if (!this.event.exceptions) {
        return false;
      }

      return this.event.exceptions.cancelled;
    }

    /**
     * @returns {boolean}
     */
    canStart() {
      return !this.isCancelled() && this.status !== 'completed';
    }

    /**
     * @description Either display trainer or trainers depending on the
     * on number of trainers on the current event.
     * @returns {string}
     */
    displayTrainers() {
      return this.event.trainers < 1 ? 'Trainers' : 'Trainer';
    }

    showAnyButton() {
      const statuses = ['active', 'waiting'];
      return this.event.waiting_list || this.event.customers_attending < this.event.spaces_available || this.event.booking && statuses.includes(this.event.booking.status);
    }

    /**
     * @returns {boolean}
     */
    canBookClients() {
      const currentTime = moment();
      const eventDate = moment(this.$stateParams.date_time).add(6, 'hours');

      return currentTime >= eventDate;
    }

    /**
     * @param status
     */
    changeEventStatus(status = null) {
      this.status = status;
    }

    /**
     * Probably unused!
     * @returns {*}
     */
    location() {
      const address = this.currentUser.address1;
      const city = this.currentUser.town_city;
      const postcode = this.currentUser.postcode;
      this.fullAddress = `${address} ${city} ${postcode}`;
      let latitude = null;
      let longitude = null;
      let cords = {};

      const currentLocation = this.event.dates.find(time => time.id === parseInt(this.$stateParams.eventTimeId, 10));

      if (this.event.global_exceptions && this.event.global_exceptions.location) {
        return this.event.global_exceptions.location.location;
      }
      if (this.event.exceptions && this.event.exceptions.location) {
        return this.event.exceptions.location.location;
      }
      if (currentLocation.location) {
        return currentLocation.location.location;
      }

      console.log('full address', this.fullAddress);

      const geocoder = new google.maps.Geocoder();
      return geocoder.geocode({
        address: this.fullAddress
      }, (results, status) => {
        console.log(results, status);
        if (status === google.maps.GeocoderStatus.OK) {
          latitude = results[0].geometry.location.lat();
          longitude = results[0].geometry.location.lng();
        }

        const map = new google.maps.Map(document.getElementById('GoogleMapView'), {
          center: {
            lat: latitude,
            lng: longitude
          },
          zoom: 16
        });

        return new google.maps.Marker({
          position: {
            lat: latitude,
            lng: longitude
          },
          map
        });
      });
    }

    initMap() {
      const map = new google.maps.Map(document.getElementById('GoogleMapView'), {
        center: this.location(),
        zoom: 16
      });

      new google.maps.Marker({
        position: this.location(),
        map
      });
    }

    /**
     * @returns {*}
     */
    getNewDate() {
      if (!this.event.exceptions) {
        return this.date;
      }
      if (this.event.exceptions.hasOwnProperty('new_date_time')) {
        return this.event.exceptions.new_date_time;
      }

      return this.date;
    }

    /**
     * @returns {number}
     */
    slotsAvailable() {
      let slots;
      if (this.PermissionsService.userIsCustomer()) {
        slots = this.event.spaces_available - this.event.customers_attending;
      } else {
        slots = this.event.spaces_available - this.event.customers_attending_ids.length;
      }
      return slots <= 0 ? 0 : slots;
    }

    /**
     * @returns {*}
     */
    waitingListDisplay() {
      if (!this.event.waiting_list) {
        return 'Disabled';
      }
      return this.waitingBookings.length;
    }

    /**
     * @returns {boolean}
     */
    canStartEvent() {
      if (this.PermissionsService.userIsTrainer()) {
        if (this.PermissionsService.userIsTrainerAdminOrOwner(this.event.event_owner.id)) {
          return true;
        }

        return this.event.trainers_ids.includes(this.currentUser.id);
      }
    }

    /**
     * @returns {boolean}
     */
    canEditEvent() {
      return this.PermissionsService.userIsTrainerAdminOrOwner(this.event.event_owner.id) && !this.event.completed;
    }

    /**
     * @param event
     * @returns {boolean}
     */
    showEventMenu(event) {
      this.$ionicActionSheet.show({
        buttons: [{
          text: 'Edit Event',
          action: () => {
            this.editEvent(event);
          }
        }],
        cancelText: 'Cancel',
        destructiveText: 'Cancel Event',
        destructiveButtonClicked: () => {
          this.NavigateService.go('app.trainer_calendar_event_cancel', {
            eventId: this.$stateParams.eventId,
            eventTimeId: this.$stateParams.eventTimeId,
            date_time: this.$stateParams.date_time
          });

          return true;
        },
        buttonClicked(index) {
          if (typeof this.buttons[index] !== 'undefined' && typeof this.buttons[index].action === 'function') {
            this.buttons[index].action();
          }
          return true;
        }
      });
    }

    /**
     * @returns {boolean}
     */
    showTrainerMenu() {
      this.$ionicActionSheet.show({
        buttons: [{
          text: 'Edit Trainers',
          action: () => {
            this.editTrainers();
          }
        }],
        cancelText: 'Cancel',
        buttonClicked(index) {
          if (typeof this.buttons[index] !== 'undefined' && typeof this.buttons[index].action === 'function') {
            this.buttons[index].action();
          }
          return true;
        }
      });
    }

    /**
     * @returns {boolean}
     */
    showClientMenu(currentEvent) {
      this.$ionicActionSheet.show({
        buttons: [{
          text: 'Book Clients',
          action: () => {
            this.bookClients();
          },
          show: !currentEvent.completed
        }, {
          text: 'View Clients',
          action: () => {
            this.viewClients();
          },
          show: true
        }].filter(btn => btn.show),
        cancelText: 'Cancel',
        buttonClicked(index) {
          if (typeof this.buttons[index] !== 'undefined' && typeof this.buttons[index].action === 'function') {
            this.buttons[index].action();
          }
          return true;
        }
      });
    }

    /**
     * @returns {*}
     */
    editEvent() {
      return this.NavigateService.go('app.trainer_calendar_event_edit', {
        eventId: this.event.id,
        eventTimeId: this.$stateParams.eventTimeId,
        date_time: this.$stateParams.date_time
      });
    }

    /**
     * @returns {*}
     */
    bookClients() {
      return this.NavigateService.go('app.trainer_calendar_event_client_bookings', {
        eventId: this.event.id,
        eventTimeId: this.$stateParams.eventTimeId,
        date_time: this.$stateParams.date_time
      });
    }

    /**
     * @returns {*}
     */
    viewClients() {
      return this.NavigateService.go('app.trainer_calendar_event_client_view', {
        eventId: this.event.id,
        eventTimeId: this.$stateParams.eventTimeId,
        date_time: this.$stateParams.date_time
      });
    }

    /**
     * @returns {*}
     */
    startEvent() {
      return this.NavigateService.go('app.trainer_calendar_event_register', {
        eventId: this.event.id,
        eventTimeId: this.$stateParams.eventTimeId,
        date_time: this.$stateParams.date_time
      });
    }

    /**
     * @returns {*}
     */
    editTrainers() {
      return this.NavigateService.go('app.trainer_calendar_event_trainer_assignment', {
        eventId: this.event.id,
        eventTimeId: this.$stateParams.eventTimeId,
        date_time: this.$stateParams.date_time
      });
    }

    moveIntoEvent() {
      this.ApiClientService.calendarEventsBookings.restore(this.event.booking.id).success(response => {
        this.event.booking = response.booking;
        const message = !this.slotsAvailable() ? 'You were added to the waiting list' : 'Booked in!';
        this.event.isCurrentUserBookedIn = true;
        this.event.customers_attending += 1;
        this.ToastService.show(message);
        this.processing = false;
        this.$state.reload();
      }).error(err => {
        this.ToastService.show('Unable to restore booking');
        this.processing = false;
      });
    }

    bookEvent() {
      this.processing = true;
      if (this.hasCancelledBooking()) {
        this.ApiClientService.calendarEventsBookings.restore(this.event.booking.id).success(response => {
          this.event.booking = response.booking;
          const message = !this.slotsAvailable() ? 'You were added to the waiting list' : 'Booked in!';
          this.event.isCurrentUserBookedIn = true;
          this.event.customers_attending += 1;
          this.ToastService.show(message);
          this.processing = false;
          this.$state.reload();
        }).error(err => {
          this.ToastService.show('Unable to restore booking');
          this.processing = false;
        });
        // restore
      } else {
        // create!
        const data = {
          customer_id: this.currentUser.customer_id,
          calendar_event_id: this.event.id,
          calendar_event_time_id: this.$stateParams.eventTimeId,
          date: this.$stateParams.date_time
        };

        if (this.credit_id) {
          data.credit_id = this.credit_id;
        }

        this.ApiClientService.calendarEventsBookings.create(this.event.id, data).success(response => {
          this.event.booking = response.booking;
          const message = !this.slotsAvailable() ? 'You were added to the waiting list' : 'Booked in!';
          this.event.customers_attending += 1;
          this.event.isCurrentUserBookedIn = true;
          this.ToastService.show(message);
          this.processing = false;
          this.$state.reload();
        }).error(err => {
          this.processing = false;
          this.ToastService.show(err.error.message);
          this.notEnoughCredits = true;
        });
      }
    }

    cancelBooking() {
      this.ApiClientService.calendarEventsBookings.cancel(this.event.booking.id).success(response => {
        this.event.isCurrentUserBookedIn = false;
        this.event.customers_attending -= 1;
        this.event.booking = response.booking;
        this.ToastService.show('Cancelled Booking');
        this.$state.reload();
      }).catch(() => this.ToastService.show('Unable to cancel booking'));
    }

    hasCancelledBooking() {
      return this.event.booking !== null && this.event.booking.cancelled;
    }
  }

  const config = $stateProvider => {
    $stateProvider.state('app.trainer_calendar_event_booking', {
      url: '/calendar/event/{eventId:[0-9]+}/time/{eventTimeId:[0-9]+}/book?{date_time}',
      views: {
        menuContent: {
          templateUrl: 'trainer/calendar/views/calendarEvent.html',
          controller: 'TrainerCalendarEventBookingController',
          controllerAs: 'vm'
        }
      },
      resolve: {
        event($stateParams, ApiClientService) {
          return ApiClientService.calendarEvents.get($stateParams.eventId, {
            date_time: $stateParams.date_time,
            event_time: $stateParams.eventTimeId
          }).success(response => response.event);
        },
        credits(ApiClientService, PermissionsService) {
          if (PermissionsService.userIsTrainer()) {
            return false;
          }

          return ApiClientService.packagesCredits.getAll().then(response => response.credits);
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('TrainerCalendarEventBookingController', TrainerCalendarEventBookingController);
})();
(() => {
  class TrainerCalendarEventClientBookings {
    /**
     * @param {Object} ApiClientService Service
     * @param {Object} event event response from API
     * @param {Object} $stateParams Service
     * @param {Object} AuthenticationStateService Service
     * @param {Object} NavigateService Service
     * @param {Object} ToastService Service
     */
    constructor(ApiClientService, event, $stateParams, AuthenticationStateService, NavigateService, ToastService) {
      this.ApiClientService = ApiClientService;
      this.event = event;
      this.customers = [];
      this.filteredClients = [];
      this.fetched = false;
      this.getCustomers();
      this.$stateParams = $stateParams;
      this.AuthenticationStateService = AuthenticationStateService;
      this.NavigateService = NavigateService;
      this.ToastService = ToastService;

      this.currentUser = AuthenticationStateService.getCurrentUser();

      this.date = this.$stateParams.date_time;
      this.selectedClients = [];
    }

    /**
     * Gets the customers from the API and sets them on
     * this.customers and this.filteredClients
     */
    getCustomers() {
      const data = {
        attending_trainers: this.event.trainers_ids.join(','),
        credit_cost: this.event.credit_cost,
        event_id: this.event.id
      };

      if (!this.event.credit_cost) {
        return this.freeCustomers();
      }

      this.ApiClientService.packagesCredits.getByCustomersWithCredits(data).success(response => {
        this.customers = response.credits;
        this.filteredClients = this.customers.filter(credit => !this.event.bookings_customers_ids.includes(credit.customer.id));
      });
      this.fetched = true;
    }

    /**
     * Allow all customers
     */
    freeCustomers() {
      this.ApiClientService.customers.listAllForOrg().success(response => {
        this.filteredClients = response.customers.map(customer => ({
          owner: `${customer.user.first_name} ${customer.user.last_name}`,
          trainer_fullname: `${customer.trainer_user.first_name} ${customer.trainer_user.last_name}`,
          customer
        })).filter(credit => !this.event.bookings_customers_ids.includes(credit.customer.id));
        this.fetched = true;
      });
    }

    /**
     * Stores the selected user?
     *
     * @param {Number} value Selected client
     * @param {Boolean} skip bool
     * @returns {*} returns the selectedClients
     */
    selectStored(value, skip = false) {
      const index = this.selectedClients.indexOf(value);
      if (index > -1) {
        return skip ? null : this.selectedClients.splice(index, 1);
      }
      return this.selectedClients.push(value);
    }

    /**
     * shows warning
     */
    showWarning() {
      const total = this.selectedClients.length + this.event.bookings_customers_ids.length;
      return total > this.event.spaces_available;
    }

    /**
     * Updates the clients attending the booking.
     */
    update() {
      this.processing = true;
      const eventDate = this.date;
      const data = {
        selectedCustomers: this.currentUser.is_owner ? this.selectedClients : this.currentUser.id,
        calendar_event_id: this.event.id,
        calendar_event_time_id: this.$stateParams.eventTimeId,
        date: eventDate
      };
      this.ApiClientService.calendarEventsBookings.create(this.event.id, data).then(() => {
        this.processing = false;
        const message = this.selectedClients.length === 1 ? 'Client' : 'Clients';
        this.ToastService.show(`${message} booked in`);
        this.NavigateService.go('app.trainer_calendar_event_booking', {
          eventId: this.$stateParams.eventId,
          eventTimeId: this.$stateParams.eventTimeId,
          date_time: this.$stateParams.date_time
        });
        this.NavigateService.clearCache();
      }).catch(() => {
        this.processing = false;
        this.ToastService.show('Unable to book clients in');
      });
    }
  }

  /**
   * Sets up the state.
   *
   * @param {Object} $stateProvider Service
   */
  const config = $stateProvider => {
    $stateProvider.state('app.trainer_calendar_event_client_bookings', {
      url: '/calendar/event/clients/{eventId:[0-9]+}/time/{eventTimeId:[0-9]+}/clients?{date_time}',
      views: {
        menuContent: {
          templateUrl: 'trainer/calendar/views/calendarEventBookClients.html',
          controller: 'TrainerCalendarEventClientBookings',
          controllerAs: 'vm'
        }
      },
      resolve: {
        /**
         * @param {Object} $stateParams Service
         * @param {Object} ApiClientService Service
         * @returns {Object} The event
         */
        event($stateParams, ApiClientService) {
          return ApiClientService.calendarEvents.get($stateParams.eventId, {
            date_time: $stateParams.date_time,
            event_time: $stateParams.eventTimeId
          }).success(response => response.event);
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('TrainerCalendarEventClientBookings', TrainerCalendarEventClientBookings);
})();
(() => {
  class CalendarEventCancelController {
    /**
     * @param {Object} ApiClientService Service
     * @param {Object} event The event
     * @param {Object} $stateParams Service
     * @param {Object} ToastService Service
     * @param {Object} NavigateService Service
     */
    constructor(ApiClientService, event, $stateParams, ToastService, NavigateService) {
      this.ApiClientService = ApiClientService;
      this.event = event;
      this.$stateParams = $stateParams;
      this.ToastService = ToastService;
      this.NavigateService = NavigateService;
      this.cancellationReason = null;
      this.isRecurring = this.event.dates.find(time => time.id === parseInt($stateParams.eventTimeId, 10)).recurring;
    }

    cancelAll() {
      this.ApiClientService.calendarEventTimes.updateEndDate(this.$stateParams.eventTimeId, {
        calendar_event_id: this.event.id,
        calendar_event_time_id: this.$stateParams.eventTimeId,
        date_time: this.$stateParams.date_time,
        end_date: this.getNewDate(),
        cancelled: 1,
        reason: this.cancellationReason
      }).then(() => {
        this.finishCancel('Events have been cancelled');
      });
    }

    cancelSingle() {
      this.ApiClientService.calendarEventTimes.cancel(this.$stateParams.eventTimeId, {
        calendar_event_id: this.event.id,
        calendar_event_time_id: this.$stateParams.eventTimeId,
        date_time: this.$stateParams.date_time,
        cancelled: 1,
        reason: this.cancellationReason
      }).then(() => {
        this.finishCancel('Event Cancelled');
      });
    }

    cancel(applyToAll = false) {
      if (this.isRecurring && applyToAll) {
        this.cancelAll();
      } else {
        this.cancelSingle();
      }
    }

    finishCancel(message) {
      this.ToastService.show(message);
      this.NavigateService.clearCache().then(() => {
        this.NavigateService.isRoot().replaceCurrentState().go('app.trainer_calendar');
      });
    }

    getNewDate() {
      if (!this.event.exceptions) {
        return this.$stateParams.date_time;
      }
      if (this.event.exceptions.hasOwnProperty('new_date_time')) {
        return this.event.exceptions.new_date_time;
      }

      return this.$stateParams.date_time;
    }
  }

  /**
   * @param {Object} $stateProvider Service
   */
  const config = $stateProvider => {
    $stateProvider.state('app.trainer_calendar_event_cancel', {
      url: '/calendar/event/cancel/{eventId:[0-9]+}/time/{eventTimeId:[0-9]+}/{date_time}',
      views: {
        menuContent: {
          templateUrl: 'trainer/calendar/views/calendarEventCancel.html',
          controller: 'CalendarEventCancelController',
          controllerAs: 'vm'
        }
      },
      resolve: {
        event($stateParams, ApiClientService) {
          return ApiClientService.calendarEvents.get($stateParams.eventId, {
            date_time: $stateParams.date_time,
            event_time: $stateParams.eventTimeId
          }).success(response => response.event);
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('CalendarEventCancelController', CalendarEventCancelController);
})();
(() => {
  class CalendarEventEditController {
    /**
     * @param ApiClientService
     * @param event
     * @param $stateParams
     * @param AuthenticationStateService
     * @param $timeout
     * @param NavigateService
     * @param ToastService
     */
    constructor(ApiClientService, event, $stateParams, AuthenticationStateService, $timeout, NavigateService, ToastService) {
      this.ApiClientService = ApiClientService;
      this.event = event;
      this.$stateParams = $stateParams;
      this.AuthenticationStateService = AuthenticationStateService;
      this.$timeout = $timeout;
      this.NavigateService = NavigateService;
      this.ToastService = ToastService;

      this.currentUser = AuthenticationStateService.getCurrentUser();
      this.todayDate = new Date();
      this.date = this.$stateParams.date_time;
      this.originalDate = moment(this.getNewDate()).format('YYYY-MM-DD');
      this.isRecurring = this.event.dates.find(time => time.id === parseInt(this.$stateParams.eventTimeId, 10));
      this.booking = {
        date_time: moment(this.getNewDate()).toDate(),
        recurring: false,
        time: {}
      };
      this.updateTimes();
      this.showUpdateAll = moment(this.originalDate).isSame(moment(this.booking.date_time), 'day') || !this.isRecurring;
      this.hours = this.constructor.createHours();
      this.minutes = this.constructor.createMinutes();

      console.log(this.event);

      this.fullAddress = null;
      this.initMap();
    }

    $onInit() {
      this.NavigateService.removeCache(['app.trainer_calendar', 'app.client_calendar', 'app.company_calendar']);
    }

    getNewDate() {
      if (!this.event.exceptions) {
        return this.$stateParams.date_time;
      }
      if (this.event.exceptions.hasOwnProperty('new_date_time')) {
        return this.event.exceptions.new_date_time;
      }

      return this.$stateParams.date_time;
    }

    updateTimes() {
      this.booking.time.hour = moment.utc(this.date).tz(this.currentUser.timezone).format('hh');
      this.booking.time.minute = moment.utc(this.date).tz(this.currentUser.timezone).format('mm');
      if (this.event.global_exceptions && this.event.global_exceptions.new_date_time) {
        this.booking.time.hour = moment.utc(this.event.global_exceptions.new_date_time).tz(this.currentUser.timezone).format('hh');
        this.booking.time.minute = moment.utc(this.event.global_exceptions.new_date_time).tz(this.currentUser.timezone).format('mm');
      }
      if (this.event.exceptions) {
        this.booking.time.hour = moment.utc(this.event.exceptions.new_date_time).tz(this.currentUser.timezone).format('hh');
        this.booking.time.minute = moment.utc(this.event.exceptions.new_date_time).tz(this.currentUser.timezone).format('mm');
      }

      console.log(this.booking);
    }

    /**
     * Creates an array of the hours of the day
     *
     * @returns {string[]}
     */
    static createHours() {
      return [...Array(24).keys()].map(i => i < 10 ? `0${i}` : `${i}`);
    }

    /**
     * @returns {string[]}
     */
    static createMinutes() {
      return [...Array(60).keys()].map(i => i < 10 ? `0${i}` : `${i}`);
    }

    convertDate(date, time) {
      date = moment(date).format('YYYY-MM-DD');
      const hours = parseInt(time.hour, 10);
      const minutes = parseInt(time.minute, 10);
      return moment.tz(date, this.currentUser.timezone).hours(hours).minutes(minutes).format('YYYY-MM-DD HH:mm');
    }

    update(applyToAll = false) {
      this.ApiClientService.calendarEventTimes.addException(this.$stateParams.eventTimeId, {
        calendar_event_id: this.event.id,
        calendar_event_time_id: this.$stateParams.eventTimeId,
        date_time: this.date,
        location: this.event.location,
        new_date_time: this.convertDate(this.booking.date_time, this.booking.time),
        spaces_available: this.event.spaces_available,
        cancelled: 0,
        applyToAll
      }).then(() => {
        this.ToastService.show('Booking updated');
        this.NavigateService.disableBack().go('app.trainer_calendar');
      });
    }

    getLocation() {
      const currentLocation = this.event.dates.find(time => time.id === parseInt(this.$stateParams.eventTimeId, 10));

      if (this.event.global_exceptions && this.event.global_exceptions.location) {
        return this.event.global_exceptions.location.location;
      }
      if (this.event.exceptions && this.event.exceptions.location) {
        return this.event.exceptions.location.location;
      }
      if (currentLocation.location) {
        return currentLocation.location.location;
      }
      return {
        lat: 50.999041,
        lng: -0.106333
      };
    }

    initMap() {
      const map = new google.maps.Map(document.getElementById('GoogleMap'), {
        center: this.getLocation(),
        zoom: 15,
        disableDefaultUI: true
      });

      google.maps.event.addListenerOnce(map, 'idle', () => {
        $('.pac-container').attr('data-tap-disabled', 'true');
      });

      const input = document.getElementById('pac-input');

      const searchBox = new google.maps.places.SearchBox(input);
      map.controls[google.maps.ControlPosition.TOP_LEFT].push(input);

      // Bias the SearchBox results towards current map's viewport.
      map.addListener('bounds_changed', () => {
        searchBox.setBounds(map.getBounds());
      });

      let markers = [];
      // Listen for the event fired when the user selects a prediction and retrieve
      // more details for that place.
      searchBox.addListener('places_changed', () => {
        const places = searchBox.getPlaces();

        if (places.length == 0) {
          return;
        }

        // Clear out the old markers.
        markers.forEach(marker => {
          marker.setMap(null);
        });
        markers = [];

        // For each place, get the icon, name and location.
        const bounds = new google.maps.LatLngBounds();
        places.forEach(place => {
          if (!place.geometry) {
            console.log('Returned place contains no geometry');
            return;
          }
          const icon = {
            url: place.icon,
            size: new google.maps.Size(71, 71),
            origin: new google.maps.Point(0, 0),
            anchor: new google.maps.Point(17, 34),
            scaledSize: new google.maps.Size(25, 25)
          };

          this.location = place.formatted_address;
          this.event.location = { name: place.formatted_address, location: place.geometry.location };
          // Create a marker for each place.
          markers.push(new google.maps.Marker({
            map,
            icon,
            title: place.name,
            position: place.geometry.location
          }));

          if (place.geometry.viewport) {
            // Only geocodes have viewport.
            bounds.union(place.geometry.viewport);
          } else {
            bounds.extend(place.geometry.location);
          }
        });
        map.fitBounds(bounds);
      });
    }
  }

  const config = $stateProvider => {
    $stateProvider.state('app.trainer_calendar_event_edit', {
      url: '/calendar/event/edit/{eventId:[0-9]+}/time/{eventTimeId:[0-9]+}/edit?{date_time}',
      views: {
        menuContent: {
          templateUrl: 'trainer/calendar/views/calendarEventEdit.html',
          controller: 'CalendarEventEditController',
          controllerAs: 'vm'
        }
      },
      resolve: {
        event($stateParams, ApiClientService) {
          return ApiClientService.calendarEvents.get($stateParams.eventId, {
            date_time: $stateParams.date_time,
            event_time: $stateParams.eventTimeId
          }).success(response => response.event);
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('CalendarEventEditController', CalendarEventEditController);
})();
(() => {
  class TrainerCalendarEventRegisterController {
    /**
     * @param ApiClientService
     * @param event
     * @param $timeout
     * @param $stateParams
     * @param AuthenticationStateService
     * @param ToastService
     * @param ConfirmDialogService
     */
    constructor(ApiClientService, event, $timeout, $stateParams, AuthenticationStateService, ToastService, ConfirmDialogService, NavigateService) {
      this.ApiClientService = ApiClientService;
      this.event = event;
      this.$timeout = $timeout;
      this.$stateParams = $stateParams;
      this.AuthenticationStateService = AuthenticationStateService;
      this.ToastService = ToastService;
      this.ConfirmDialogService = ConfirmDialogService;
      this.NavigateService = NavigateService;

      this.currentUser = AuthenticationStateService.getCurrentUser();
      this.activeBookings = () => this.event.bookings.filter(booking => booking.status === 'active' || booking.status === 'attended');
      this.checkInCustomerCount = () => this.event.bookings.filter(booking => booking.status === 'attended').length;
      this.waitingBookings = () => this.event.bookings.filter(booking => booking.status === 'waiting');
      this.processing = false;

      $timeout(() => {
        const updateTimes = () => {
          this.startTime = moment.utc($stateParams.date_time).tz(this.currentUser.timezone).format('h:mm A');
          this.endTime = moment.utc($stateParams.date_time).add('minutes', this.event.duration).tz(this.currentUser.timezone).format('h:mm A');
          if (this.event.global_exceptions && this.event.global_exceptions.new_date_time) {
            this.startTime = moment.utc(this.event.global_exceptions.new_date_time).tz(this.currentUser.timezone).format('h:mm A');
            this.endTime = moment.utc(this.event.global_exceptions.new_date_time).add('minutes', this.event.duration).tz(this.currentUser.timezone).format('h:mm A');
          }
          if (this.event.exceptions) {
            this.startTime = moment.utc(this.event.exceptions.new_date_time).tz(this.currentUser.timezone).format('h:mm A');
            this.endTime = moment.utc(this.event.exceptions.new_date_time).add('minutes', this.event.duration).tz(this.currentUser.timezone).format('h:mm A');
          }
        };
        updateTimes();
      });
    }

    /**
     * Checks the client into the event.
     * @param booking
     */
    checkIn(booking) {
      this.ApiClientService.calendarEventsBookings.checkIn(booking.id, {
        workouts: this.event.workouts_ids
      }).then(response => {
        const bookingData = response.data.booking;
        console.log(bookingData);
        const idx = this.event.bookings.findIndex(item => item.id === booking.id);
        if (idx !== -1) {
          if (bookingData.status === 'attended') {
            this.event.bookings[idx].status = 'attended';
            this.ToastService.show('Checked In');
          } else {
            this.event.bookings[idx].status = 'active';
            this.ToastService.show('Checked Out');
          }
        } else {
          this.ToastService.show('Unable to check this client in');
        }
      });
    }

    /**
     * Forces the client onto the event.
     * @param booking
     */
    forceBooking(booking) {
      this.ApiClientService.calendarEventsBookings.forceBooking(booking.id).then(() => {
        const idx = this.event.bookings.findIndex(item => item.id === booking.id);
        if (idx !== -1) {
          this.event.bookings[idx].status = 'active';
        } else {
          this.ToastService.show('Unable to force book this client');
        }
      });
    }

    /**
     * Completes the event after prompting.
     */
    completeEvent() {
      this.processing = true;
      this.ConfirmDialogService.open({
        title: 'Complete Event',
        message: 'Are you sure you want to end this event?',
        buttonConfirm: 'Complete Event',
        buttonCancel: 'No, go back'
      }).then(() => {
        this.ApiClientService.calendarEventTimes.complete(this.$stateParams.eventTimeId, {
          calendar_event_id: this.event.id,
          calendar_event_time_id: this.$stateParams.eventTimeId,
          date_time: this.$stateParams.date_time
        }).then(() => {
          this.ToastService.show('Event Completed');
          this.NavigateService.disableBack().go('app.trainer_calendar');
          this.NavigateService.clearCache();
          this.processing = false;
        });
      });
    }
  }

  const config = $stateProvider => {
    $stateProvider.state('app.trainer_calendar_event_register', {
      url: '/calendar/event/register/{eventId:[0-9]+}/time/{eventTimeId:[0-9]+}/register?{date_time}',
      views: {
        menuContent: {
          templateUrl: 'trainer/calendar/views/calendarEventRegister.html',
          controller: 'TrainerCalendarEventRegisterController',
          controllerAs: 'vm'
        }
      },
      resolve: {
        event($stateParams, ApiClientService) {
          return ApiClientService.calendarEvents.get($stateParams.eventId, {
            date_time: $stateParams.date_time,
            event_time: $stateParams.eventTimeId
          }).success(response => response.event);
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('TrainerCalendarEventRegisterController', TrainerCalendarEventRegisterController);
})();
(() => {
  class TrainerCalendarEventTrainerAssignmentController {
    /**
     * @param ApiClientService
     * @param event
     * @param trainers
     * @param $stateParams
     * @param AuthenticationStateService
     * @param NavigateService
     * @param ToastService
     */
    constructor(ApiClientService, event, trainers, $stateParams, AuthenticationStateService, NavigateService, ToastService) {
      this.ApiClientService = ApiClientService;
      this.event = event;
      this.trainers = trainers;
      this.$stateParams = $stateParams;
      this.AuthenticationStateService = AuthenticationStateService;
      this.NavigateService = NavigateService;
      this.ToastService = ToastService;

      this.isRecurring = event.dates.find(time => time.id === parseInt(this.$stateParams.eventTimeId, 10)).recurring;
      this.addToAll = false;
      this.date = $stateParams.date_time;
      this.selectedTrainers = this.event.trainers_ids;
      this.processing = false;

      console.log(this.selectedTrainers);
    }

    /**
     * @param value
     * @param skip
     * @returns {*}
     */
    selectStored(value, skip = false) {
      const index = this.selectedTrainers.indexOf(value);
      if (index > -1) {
        return skip ? null : this.selectedTrainers.splice(index, 1);
      }
      return this.selectedTrainers.push(value);
    }

    update(addToAll = false) {
      if (this.selectedTrainers.length === 0) {
        this.ToastService.show('At least one trainer must be selected.');
        return false;
      }
      this.processing = true;
      const eventDate = this.date;
      const data = {
        selectedTrainers: this.selectedTrainers,
        calendar_event_id: this.event.id,
        calendar_event_time_id: this.$stateParams.eventTimeId,
        add_to_all: addToAll,
        date: eventDate
      };

      this.ApiClientService.calendarEventsBookings.bookTrainers(this.event.id, data).then(() => {
        this.processing = false;
        this.ToastService.show('Updated Attending Trainers');
        this.NavigateService.go('app.trainer_calendar_event_booking', {
          eventId: this.$stateParams.eventId,
          eventTimeId: this.$stateParams.eventTimeId,
          date_time: this.$stateParams.date_time
        });
      }).catch(() => {
        this.processing = false;
        this.ToastService.show('Unable to update attending trainers');
      });
    }
  }

  const config = $stateProvider => {
    $stateProvider.state('app.trainer_calendar_event_trainer_assignment', {
      url: '/calendar/event/trainers/{eventId:[0-9]+}/time/{eventTimeId:[0-9]+}/trainers?{date_time}',
      views: {
        menuContent: {
          templateUrl: 'trainer/calendar/views/calendarEventTrainerAssignment.html',
          controller: 'TrainerCalendarEventTrainerAssignmentController',
          controllerAs: 'vm'
        }
      },
      resolve: {
        event($stateParams, ApiClientService) {
          return ApiClientService.calendarEvents.get($stateParams.eventId, {
            date_time: $stateParams.date_time,
            event_time: $stateParams.eventTimeId
          }).success(response => response.event);
        },
        trainers(ApiClientService) {
          return ApiClientService.trainerUsers.list().then(response => response.trainerUsers);
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('TrainerCalendarEventTrainerAssignmentController', TrainerCalendarEventTrainerAssignmentController);
})();
(() => {
  class TrainerCalendarEventClientViewController {
    /**
     *
     * @param {object} ApiClientService service
     * @param {object} event object
     * @param {object} $ionicActionSheet service
     * @param {object} ToastService service
     * @param {object} $state service
     */
    constructor(ApiClientService, event, $ionicActionSheet, ToastService, $state) {
      this.ApiClientService = ApiClientService;
      this.event = event;
      this.$ionicActionSheet = $ionicActionSheet;
      this.$state = $state;
      this.ToastService = ToastService;

      this.activeBookings = () => this.event.bookings.filter(booking => booking.status === 'active' || booking.status === 'attended');
      this.waitingBookings = () => this.event.bookings.filter(booking => booking.status === 'waiting');
    }
    /**
     * @returns {array} array holding the bookings with a status of cancelled.
     */
    cancelledBookings() {
      return this.event.bookings.filter(booking => booking.status === 'cancelled');
    }

    /**
     *
     * @param {object} booking the booking object
     */
    restoreBooking(booking) {
      return this.ApiClientService.calendarEventsBookings.restore(booking.id).then(response => {
        this.ToastService.show('Booking Restored');

        const idx = this.event.bookings.findIndex(i => i.id === booking.id);

        if (idx !== -1) {
          this.event.bookings[idx].status = 'active';
          this.event.bookings[idx].cancelled = false;
        }
      }).catch(() => {
        this.ToastService.show('Unable to restore booking');
      });
    }
    /**
     * @param booking
     */
    cancelBooking(booking) {
      this.ApiClientService.calendarEventsBookings.cancel(booking.id).then(() => {
        this.ToastService.show('Booking Cancelled');
        const idx = this.event.bookings.findIndex(i => i.id === booking.id);
        if (idx !== -1) {
          this.event.bookings[idx].status = 'cancelled';
          this.event.bookings[idx].cancelled = true;
        }
      }).catch(() => {
        this.ToastService.show('Unable to cancel booking');
      });
    }

    /**
     * @param booking
     */
    bookClient(booking) {
      this.ApiClientService.calendarEventsBookings.moveToBookList(booking.id).then(() => {
        this.ToastService.show('Client Booked');
        const idx = this.event.bookings.findIndex(i => i.id === booking.id);
        if (idx !== -1) {
          this.event.bookings[idx].status = 'active';
        }
      }).catch(() => {
        this.ToastService.show('Unable to book client');
      });
    }

    /**
     * @param booking
     */
    forceBooking(booking) {
      this.ApiClientService.calendarEventsBookings.forceBooking(booking.id).then(() => {
        this.ToastService.show('Client Booked');
        const idx = this.event.bookings.findIndex(i => i.id === booking.id);
        if (idx !== -1) {
          this.event.bookings[idx].status = 'active';
        }
      }).catch(() => {
        this.ToastService.show('Unable to book client');
      });
    }

    /**
     * @param booking
     * @returns {boolean}
     */
    showBookingMenu(booking) {
      this.$ionicActionSheet.show({
        buttons: [{
          text: 'Cancel Booking',
          action: () => {
            this.cancelBooking(booking);
          },
          display: !booking.cancelled
        }, {
          text: 'Restore Booking',
          action: () => {
            this.restoreBooking(booking);
          },
          display: booking.cancelled
        }].filter(item => item.display),
        cancelText: 'Cancel',
        buttonClicked(index) {
          if (typeof this.buttons[index] !== 'undefined' && typeof this.buttons[index].action === 'function') {
            this.buttons[index].action();
          }
          return true;
        }
      });
    }

    showWaitingMenu(booking) {
      this.$ionicActionSheet.show({
        buttons: [{
          text: 'Book Client',
          action: () => {
            this.bookClient(booking);
          },
          display: this.slotsAvailable() >= 1
        }, {
          text: 'Force Booking',
          action: () => {
            this.forceBooking(booking);
          },
          display: this.slotsAvailable() < 1
        }, {
          text: 'Cancel Booking',
          action: () => {
            this.cancelBooking(booking);
          },
          display: !booking.cancelled
        }].filter(item => item.display),
        cancelText: 'Cancel',
        buttonClicked(index) {
          if (typeof this.buttons[index] !== 'undefined' && typeof this.buttons[index].action === 'function') {
            this.buttons[index].action();
          }
          return true;
        }
      });
    }

    slotsAvailable() {
      const slots = this.event.spaces_available - this.event.customers_attending_ids.length;
      return slots <= 0 ? 0 : slots;
    }
  }

  const config = $stateProvider => {
    $stateProvider.state('app.trainer_calendar_event_client_view', {
      url: '/calendar/event/clients-view/{eventId:[0-9]+}/time/{eventTimeId:[0-9]+}/clients?{date_time}',
      views: {
        menuContent: {
          templateUrl: 'trainer/calendar/views/calendarEventViewClients.html',
          controller: 'TrainerCalendarEventClientViewController',
          controllerAs: 'vm'
        }
      },
      resolve: {
        event($stateParams, ApiClientService) {
          return ApiClientService.calendarEvents.get($stateParams.eventId, {
            date_time: $stateParams.date_time,
            event_time: $stateParams.eventTimeId
          }).success(response => response.event);
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('TrainerCalendarEventClientViewController', TrainerCalendarEventClientViewController);
})();
(function () {

    angular.module('MyPTHub').factory('CustomerStoreService', service);

    service.$inject = ['ApiClientService', 'NetworkStateService', 'SqlStorageService', 'PermissionsService', '$q', 'AuthenticationService', '$rootScope', 'AuthenticationStateService'];

    function service(ApiClientService, NetworkStateService, SqlStorageService, PermissionsService, $q, AuthenticationService, $rootScope, AuthenticationStateService) {

        var dirty = true;

        var fireEventHandlers = function (type, data) {
            data = data || false;
            $rootScope.$emit('CustomerStoreService_' + type, data);
        };

        var registerEventHandler = function (scope, type, handler) {
            var unsub = $rootScope.$on('CustomerStoreService_' + type, handler);
            scope.$on('$destroy', unsub);
            return unsub;
        };

        var parseRow = function (row) {

            row.data = JSON.parse(row.data);

            return row;
        };

        var parseRows = function (rows) {
            return rows.map(parseRow);
        };

        var getOffline = function (customerId) {

            return $q(function (resolve, reject) {

                SqlStorageService.select('SELECT * FROM customers WHERE id = ?', [customerId]).then(function (result) {

                    resolve(parseRow(result));
                }, function (error) {
                    reject(false);
                });
            });
        };

        var storeCustomer = function (customer) {

            return $q(function (resolve, reject) {

                getOffline(customer.id).then(function () {

                    SqlStorageService.query("UPDATE customers SET data = ?, first_name = ?, last_name = ?, last_data_update = datetime('now'), enabled = ?, trainer_user_id = ? WHERE id = ?", [JSON.stringify(customer), customer.user.first_name, customer.user.last_name, customer.enabled, customer.trainer_user_id, customer.id]).then(function () {

                        getOffline(customer.id).then(resolve);
                    });
                }, function () {

                    SqlStorageService.query("INSERT INTO customers (id, first_name, last_name, user_id, trainer_id, trainer_user_id, enabled, data, last_data_update) VALUES(?, ?, ?, ?, ?, ?, ?, ?, datetime('now'))", [customer.id, customer.user.first_name, customer.user.last_name, customer.user_id, customer.trainer_id, customer.trainer_user_id, customer.enabled, JSON.stringify(customer)]).then(function () {

                        getOffline(customer.id).then(resolve);
                    });
                });
            });
        };

        var getLive = function (customerId) {

            return $q(function (resolve, reject) {

                if (!NetworkStateService.isOnline()) {
                    reject(false);
                    return;
                }

                var getCustomer = ApiClientService.noCache().customers.get(customerId);

                getCustomer.success(function (response) {
                    storeCustomer(response.customer).then(resolve);
                });

                getCustomer.error(function (response) {
                    reject(response);
                });
            });
        };

        var deleteLive = function (customerId) {

            return ApiClientService.customers.delete(customerId);
        };

        var deleteLocal = function (customerId) {

            return SqlStorageService.query("DELETE FROM customers WHERE id = ?", [customerId]);
        };

        var cleanupRemotelyDeleted = function (wheres, trainerUserId) {

            var deferred = $q.defer();

            var flags = [];
            for (var index = 0; index < wheres.length; ++index) {
                flags.push('?');
            }

            wheres.push(PermissionsService.getCurrentUserEntityId());

            wheres.push(trainerUserId);

            SqlStorageService.selectAll('SELECT id FROM customers WHERE id NOT IN (' + flags.join(',') + ') AND trainer_id = ? AND trainer_user_id = ?', wheres).then(function (results) {

                var deletes = [];

                results.forEach(function (item) {
                    deletes.push(deleteLocal(item.id));
                });

                $q.all(deletes).finally(deferred.resolve);
            }, deferred.reject);

            return deferred.promise;
        };

        var service = {

            setup: function () {

                var download = function (resolve) {

                    if (PermissionsService.userIsTrainer()) {

                        dirty = true;

                        service.downloadTrainerCustomers().finally(resolve);
                    }
                };

                /**
                 * Trigger when login successful
                 */
                AuthenticationService.registerLoginSuccessEventHandler(download);

                if (AuthenticationStateService.isLoggedIn()) {
                    download();
                }
            },

            isDirty: function () {
                return dirty;
            },

            setDirty: function () {
                dirty = true;
            },

            // Gets cust from local DB (and updates local DB first if online)
            get: function (customerId) {

                var deferred = $q.defer();

                getOffline(customerId).then(function (customer) {

                    deferred.resolve(customer);
                }, function () {

                    /**
                     * We are offline or can't access API so we can't use
                     * the customer. Try and get it from the local database.
                     */

                    getLive(customerId).then(deferred.resolve, deferred.reject);
                });

                return deferred.promise;
            },

            delete: function (customerId) {

                dirty = true;

                return $q.all([deleteLive(customerId), deleteLocal(customerId)]);
            },

            download: function (customerId) {

                var deferred = $q.defer();

                if (!NetworkStateService.isOnline()) {
                    deferred.reject();
                    return deferred.promise;
                }

                getLive(customerId).then(deferred.resolve, deferred.reject);

                return deferred.promise;
            },

            listAllTrainerCustomers: function (trainerUserId) {

                if (!trainerUserId) {
                    trainerUserId = PermissionsService.getLoggedInUserId();
                }

                var values = [PermissionsService.getCurrentUserEntityId(), trainerUserId];

                return $q(function (resolve, reject) {

                    SqlStorageService.selectAll('SELECT * FROM customers WHERE trainer_id = ? AND trainer_user_id = ? ORDER BY first_name, last_name', values).then(function (results) {
                        resolve(parseRows(results));
                    }, function () {
                        reject();
                    });
                });
            },

            downloadTrainerCustomers: function (trainerUserId) {

                var deferred = $q.defer();

                if (!NetworkStateService.isOnline()) {
                    deferred.reject();
                    return deferred.promise;
                }

                if (!trainerUserId) {
                    trainerUserId = PermissionsService.getLoggedInUserId();
                }

                ApiClientService.noCache().customers.list(trainerUserId).success(function (response) {

                    var stored = [];
                    var currentIds = [];

                    angular.forEach(response.customers, function (customer) {
                        stored.push(storeCustomer(customer));
                        currentIds.push(customer.id);
                    });

                    $q.all(stored).then(function (storedResponse) {

                        cleanupRemotelyDeleted(currentIds, trainerUserId).finally(function () {
                            deferred.resolve(storedResponse);

                            fireEventHandlers('download_complete', storedResponse);

                            dirty = false;
                        });
                    });
                });

                return deferred.promise;
            },

            onDownloadComplete: function (scope, handler) {
                return registerEventHandler(scope, 'download_complete', handler);
            }

        };

        return service;
    }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').controller('clientAssignmentModal', controller);

  controller.$inject = ['parameters', '$scope', 'ApiClientService', '$rootScope'];

  function controller(parameters, $scope, ApiClientService, $rootScope) {

    var vm = this;
    vm.client = parameters.client;

    vm.data = [{
      code: parameters.client.activation_code
    }];

    ApiClientService.trainerUsers.list().success(response => {
      vm.trainers = response.trainerUsers;
    });

    vm.submit = () => {
      // firing an event upwards
      ApiClientService.customerTrainers.assignTrainerToCustomer({ data: vm.data }).success(response => {
        $scope.$emit('closeClientAssignmentModalEvent', true);
        $rootScope.$broadcast('closeClientAssignmentModalEvent', true);
        $scope.cancelModal(null);
      }).error(error => {});
    };
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('TrainerClientsController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.trainer_clients', {
            url: "/clients/list/:trainerUserId?",
            views: {
                'menuContent': {
                    templateUrl: "trainer/clients/clients.html",
                    controller: 'TrainerClientsController as vm'
                }
            }
        });
    }

    controller.$inject = ['CustomerStoreService', 'NetworkStateService', '$scope', '$stateParams', 'NavigateService', '$filter', 'AnalyticsService', '$ionicScrollDelegate', 'ClientGroupListService', 'PromptDialogService', 'ApiClientService', 'ModalService'];

    function controller(CustomerStoreService, NetworkStateService, $scope, $stateParams, NavigateService, $filter, AnalyticsService, $ionicScrollDelegate, ClientGroupListService, PromptDialogService, ApiClientService, ModalService) {

        var vm = this;

        vm.selectedTabIndex = 0;

        vm.goToClient = function (customer) {
            if (!customer.enabled) {
                return false;
            }

            NavigateService.go('app.trainer_client_profile', {
                customerId: customer.id
            });
        };

        vm.goToGroup = function (group) {
            NavigateService.go('app.trainer_client_group', {
                groupId: group.id
            });
        };

        const getCustomers = () => {
            ApiClientService.customerTrainers.getUnassignedClients().success(response => {
                vm.unassignedCustomers = response.customers;
                console.log(vm.unassignedCustomers);
            }).error(error => {});
        };

        var loadCustomers = function (init) {

            /**
             * Grabs from the local DB first, and then if online it will
             * get the latest data from the API, to make for a better
             * experience by having something available immediately.
             */

            init = init || false;

            CustomerStoreService.listAllTrainerCustomers(vm.trainerUserId).then(function (response) {

                // set full name on root object for search filter
                response.map(function (c) {
                    c.full_name = c.data.user.first_name + ' ' + c.data.user.last_name;
                });

                vm.clients = response;

                vm.filteredClients = angular.copy(response);

                vm.noClients = response.length === 0 && !init;

                if (init) {
                    $scope.$broadcast('scroll.refreshComplete');
                }
            });

            if (init && NetworkStateService.isOnline()) {

                CustomerStoreService.downloadTrainerCustomers(vm.trainerUserId).then(function () {
                    loadCustomers();
                }).finally(function () {
                    if (init) {
                        $scope.$broadcast('scroll.refreshComplete');
                    }
                });
            }
        };

        // Customer List Search filter
        $scope.$watch(function () {
            return vm.clientSearchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.clientSearchFilterActive = true;
                vm.filteredClients = $filter('filter')(vm.clients, {
                    full_name: newVal
                });
            } else {
                vm.clientSearchFilterActive = false;
                vm.filteredClients = vm.clients;
            }
        });

        vm.allShow = function () {
            $ionicScrollDelegate.resize();
            loadCustomers();
        };

        var loadGroups = function (refreshing) {
            refreshing = refreshing || false;

            vm.groupsSearchFilter = '';

            if (refreshing) {
                ClientGroupListService.dirty();
            }

            ClientGroupListService.list(vm.trainerUserId, ['assigned_customer_count']).then(function (groups) {
                vm.groups = groups;
                vm.filteredGroups = angular.copy(groups);
            }).finally(function () {
                if (refreshing) {
                    $scope.$broadcast('scroll.refreshComplete');
                }
            });
        };

        vm.createGroup = function () {
            PromptDialogService.open({
                title: "Create Client Group",
                message: "Please enter a name for the group",
                buttonConfirm: "Create",
                buttonCancel: "Cancel",
                required: true
            }).then(function (name) {

                ApiClientService.customerGroups.create({
                    title: name
                }, vm.trainerUserId).success(function (response) {
                    loadGroups(true);
                    vm.goToGroup(response.group);
                });
            });
        };

        // Group List Search filter
        $scope.$watch(function () {
            return vm.groupsSearchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.groupSearchFilterActive = true;
                vm.filteredGroups = $filter('filter')(vm.groups, {
                    title: newVal
                });
            } else {
                vm.groupSearchFilterActive = false;
                vm.filteredGroups = vm.groups;
            }
        });

        vm.groupsShow = function () {
            $ionicScrollDelegate.resize();
            loadGroups();
        };

        vm.clientPool = () => {
            getCustomers();
        };

        // Assign Trainers Box
        vm.assignTrainer = client => {

            ModalService.show({
                templateUrl: 'trainer/clients/clientAssignmentModal.html',
                controller: 'clientAssignmentModal as vm',
                params: {
                    client
                }
            });
        };

        $scope.$on('closeClientAssignmentModalEvent', function (event, data) {
            getCustomers();
        });

        vm.refresh = function () {
            switch (vm.selectedTabIndex) {

                // Clients
                case 0:
                    loadCustomers(true);
                    break;

                // Groups
                case 1:
                    loadGroups(true);
                    break;

                // Groups
                case 2:
                    loadGroups(true);
                    break;

            }
        };

        $scope.$on('$ionicView.beforeEnter', function () {

            vm.trainerUserId = $stateParams.trainerUserId;

            switch (vm.selectedTabIndex) {

                // Clients
                case 0:
                    loadCustomers(true);
                    break;

                // Groups
                case 1:
                    loadGroups();
                    break;

                // Groups
                case 2:
                    loadGroups();
                    break;

            }
        });

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Trainer - View Client List');
        });
    }
})();
(() => {
  class TrainerEventsCreateController {
    /**
     * @param ApiClientService
     * @param ToastService
     * @param NavigateService
     */
    constructor(ApiClientService, ToastService, NavigateService) {
      this.ApiClientService = ApiClientService;
      this.ToastService = ToastService;
      this.NavigateService = NavigateService;
      this.event = {
        title: null,
        description: null,
        color: '#323232',
        private: false,
        time_off: false,
        live: true,
        waiting_list: true,
        spaces_available: 10,
        duration: 60,
        image: null,
        credit_cost: 1,
        free: false
      };
      this.processing = false;
    }

    /**
     * Creates an event.
     */
    create() {
      if (this.event.is_free) {
        this.event.credit_cost = 0;
      }
      this.ApiClientService.events.create(this.event).then(() => {
        this.ToastService.show('Event Created');
        this.NavigateService.go('app.events');
        this.NavigateService.clearCache();
      }).catch(() => {
        this.ToastService.show('Unable to create event');
      });
    }
  }

  /**
   * @param $stateProvider
   */
  const config = $stateProvider => {
    $stateProvider.state('app.events_create', {
      url: '/events/create',
      views: {
        menuContent: {
          templateUrl: 'trainer/events/views/create.html',
          controller: 'TrainerEventsCreateController',
          controllerAs: 'vm'
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('TrainerEventsCreateController', TrainerEventsCreateController);
})();
(() => {
  class TrainerEventEditController {
    /**
     * @param eventData
     * @param ApiClientService
     * @param ToastService
     * @param NavigateService
     */
    constructor(eventData, ApiClientService, ToastService, NavigateService) {
      this.event = eventData;
      this.ApiClientService = ApiClientService;
      this.ToastService = ToastService;
      this.NavigateService = NavigateService;
      this.processing = false;
    }

    /**
     * Send a post request to the API with the updated event
     */
    update() {
      if (this.event.is_free) {
        this.event.credit_cost = 0;
      }
      this.processing = true;
      this.event.image = null;
      this.ApiClientService.events.update(this.event.id, this.event).then(() => {
        this.ToastService.show('Event Updated');
        this.NavigateService.go('app.events');
        this.NavigateService.clearCache();
        this.processing = false;
      }).catch(error => {
        this.ToastService.show('Unable to update event');
        console.error('Error updating event', error);
        this.processing = false;
      });
    }
  }

  const config = $stateProvider => {
    $stateProvider.state('app.events_edit', {
      resolve: {
        eventData(ApiClientService, $stateParams) {
          return ApiClientService.events.get($stateParams.id).then(resp => resp.event);
        }
      },
      url: '/events/:id/update',
      views: {
        menuContent: {
          templateUrl: 'trainer/events/views/edit.html',
          controller: 'TrainerEventEditController',
          controllerAs: 'vm'
        }
      }
    });
  };

  angular.module('MyPTHub').controller('TrainerEventEditController', TrainerEventEditController).config(config);
})();
var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

(() => {
  class TrainerEventsController {
    /**
     * @param eventsData
     * @param NavigateService
     * @param PromptDialogService
     * @param $ionicActionSheet
     * @param ApiClientService
     * @param AuthenticationStateService
     * @param ToastService
     */
    constructor(eventsData, NavigateService, PromptDialogService, $ionicActionSheet, ApiClientService, AuthenticationStateService, ToastService, $scope, ConfirmDialogService) {
      this.events = eventsData;
      this.NavigateService = NavigateService;
      this.PromptDialogService = PromptDialogService;
      this.$ionicActionSheet = $ionicActionSheet;
      this.ApiClientService = ApiClientService;
      this.AuthenticationStateService = AuthenticationStateService;
      this.ToastService = ToastService;
      this.$scope = $scope;
      this.ConfirmDialogService = ConfirmDialogService;
      this.currentUser = this.AuthenticationStateService.getCurrentUser();
    }

    /**
     * @description checks if the use has any provisioned packages
     * @returns {boolean}
     */
    hasPackagesActivated() {
      return this.currentUser.merchant_is_provisioned;
    }

    /**
     * Navigates to the view event page.
     *
     * @param event
     */
    viewEvent(event) {
      this.NavigateService.go('app.events_view', { id: event.id });
    }

    /**
     * Navigates to the edit event page.
     *
     * @param event
     */
    editEvent(event) {
      this.NavigateService.go('app.events_edit', { id: event.id });
    }

    /**
     * Opens a modal to rename the event.
     *
     * @param event
     */
    renameEvent(event) {
      this.PromptDialogService.open({
        title: 'Rename Event',
        message: 'Enter new event name',
        buttonConfirm: 'Save',
        buttonCancel: 'Cancel',
        required: true,
        defaultText: event.title
      }).then(title => {
        const eventData = _extends({}, event);
        // const eventData = Object.assign({}, event);
        eventData.title = title;

        this.ApiClientService.events.update(event.id, eventData).success(resp => {
          event.title = resp.event.title;
          this.ToastService.show('Event Updated');
        });
      });
    }

    /**
     * Opens the menu for each event.
     *
     * @param event
     * @returns {boolean}
     */
    showEventMenu(event) {
      this.$ionicActionSheet.show({
        buttons: [
        // {
        //   text: 'View Event',
        //   action: () => {
        //     this.viewEvent(event);
        //   },
        // },
        {
          text: 'Edit Event',
          action: () => {
            this.editEvent(event);
          }
        }, {
          text: 'Rename Event',
          action: () => {
            this.renameEvent(event);
          }
        }],
        cancelText: 'Cancel',
        destructiveText: 'Delete Event',
        destructiveButtonClicked: () => {
          this.ConfirmDialogService.open({
            title: 'Confirm removal of event',
            message: `Are you sure you want to remove ${event.title}?`,
            buttonConfirm: 'Yes, Remove',
            buttonCancel: 'Cancel'
          }).then(() => {
            this.ApiClientService.events.delete(event.id).then(() => {
              const idx = this.events.findIndex(eventItem => eventItem.id === event.id);
              if (idx !== -1) {
                this.events.splice(idx, 1);
              }
              this.ToastService.show('Deleted Event');
            }).catch(() => {
              this.ToastService.show('Unable to delete event.');
            });
          });

          return true;
        },
        buttonClicked(index) {
          if (typeof this.buttons[index] !== 'undefined' && typeof this.buttons[index].action === 'function') {
            this.buttons[index].action();
          }
          return true;
        }
      });
    }

    refresh() {
      this.ApiClientService.events.getAll().then(resp => {
        this.events = resp.events;
        this.$scope.$broadcast('scroll.refreshComplete');
      });
    }
  }

  /**
   * @param $stateProvider
   * @returns {PromiseLike<T | never> | Promise<T | never>}
   */
  const config = $stateProvider => {
    $stateProvider.state('app.events', {
      resolve: {
        eventsData(ApiClientService) {
          return ApiClientService.events.getAll().then(resp => resp.events);
        }
      },
      url: '/events',
      views: {
        menuContent: {
          templateUrl: 'trainer/events/views/events.html',
          controller: 'TrainerEventsController',
          controllerAs: 'vm'
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('TrainerEventsController', TrainerEventsController);
})();
(() => {
  angular.module('MyPTHub').config(config).controller('TrainerHomeController', controller);

  function config($stateProvider) {
    $stateProvider.state('app.trainer_home', {
      url: '/trainer_home',
      params: {
        isAnimated: false
      },
      views: {
        menuContent: {
          templateUrl: 'trainer/home/home.html',
          controller: 'TrainerHomeController as vm'
        }
      }
    });
  }

  function controller($scope, $q, SessionStoreService, SessionScheduleService, SyncManagerService, AnalyticsService, SideMenuService, $mdSidenav, NavigateService, ScheduleDayPickerService, NotificationsService, SubscriptionStateService, AuthenticationStateService, ModalService) {
    const vm = this;
    vm.hasUnlimitedPlus = SubscriptionStateService.hasUnlimitedPlus();
    const currentUser = AuthenticationStateService.getCurrentUser();

    vm.showBanner = currentUser.merchant_is_provisioned && !currentUser.stripe_standard;
    vm.pastCutoff = () => moment().year() === 2019;

    if (currentUser.hasOwnProperty('permissions_active')) {
      vm.canBookEvents = currentUser.permissions_active.can_book_events;
    } else {
      vm.canBookEvents = true;
    }

    const setDuration = sessions => {
      let totalSeconds = 0;

      sessions.map(session => {
        if (session.data.time_elapsed) {
          totalSeconds += session.data.time_elapsed;
        }
        return true;
      });

      const duration = moment.duration(totalSeconds, 'seconds');

      const hours = duration.hours();
      const mins = duration.minutes();

      const hoursPad = hours < 10 ? `0${hours}` : hours;
      const minsPad = mins < 10 ? `0${mins}` : mins;

      vm.duration = `${hoursPad}:${minsPad}`;
    };

    const getSessionStats = (refetch = false) => {
      const dateStart = moment().startOf('day');
      const dateEnd = moment().endOf('day');

      return $q(resolve => {
        const getStats = () => {
          SessionScheduleService.getTrainerSchedule(dateStart, dateEnd).then(result => {
            resolve(result.length);

            vm.sessionCount = result.length;

            setDuration(result);
          });
        };

        if (refetch) {
          SessionStoreService.downloadTrainerSessions(dateStart, dateEnd).then(getStats);
        } else {
          getStats();
        }
      });
    };

    const loadAll = (done, sessionStats, refreshing = false) => {
      const tasks = [];

      if (refreshing && typeof vm.loadActivityLog === 'function') {
        tasks.push(vm.loadActivityLog());
      }

      if (sessionStats) {
        tasks.push(getSessionStats(true));
      }

      $q.all(tasks).then(done);
    };

    const updateSyncingStatus = function () {
      vm.unsyncedSessions = 0;

      SessionStoreService.countUnSyncedSessions().then(count => {
        vm.unsyncedSessions = count;
      });
    };

    SessionStoreService.registerEventHandler($scope, 'offline_sessions_syncing', updateSyncingStatus);
    SessionStoreService.registerEventHandler($scope, 'local_session_deleted', updateSyncingStatus);

    SessionStoreService.registerEventHandler($scope, 'offline_sessions_synced', () => {
      updateSyncingStatus();

      vm.refreshAll();
    });

    SessionStoreService.registerEventHandler($scope, 'online_session_synced', () => {
      vm.refreshAll();
    });

    vm.openSyncManager = function () {
      SyncManagerService.open();
    };

    const clearSessionDownloadCompleteHandler = SessionStoreService.registerEventHandler($scope, 'download_complete', () => {
      getSessionStats();

      // Don't need this handler anymore
      clearSessionDownloadCompleteHandler();
    });

    // Load the un-cached view for the first time
    loadAll(() => {});

    // Do on every load despite cache
    $scope.$on('$ionicView.beforeEnter', () => {
      const today = moment();

      vm.todayDate = {
        day: today.format('D'),
        month: today.format('MMMM')
      };

      updateSyncingStatus();
    });

    vm.goToSchedule = function () {
      ScheduleDayPickerService.reset();
      NavigateService.go('app.trainer_calendar');
    };

    // Do on every load despite cache
    $scope.$on('$ionicView.enter', (scopes, states) => {
      if (typeof vm.loadActivityLog === 'function' && states.fromCache) {
        vm.loadActivityLog();
      }

      getSessionStats();

      AnalyticsService.trackView('Trainer - Home');

      if (!SideMenuService.hasSeenSideMenu()) {
        $mdSidenav('left').open();
        SideMenuService.sideMenuSeen();
      }

      NotificationsService.load();
    });

    NotificationsService.onDirty($scope, () => {
      NotificationsService.load();
    });

    const unWatchEnteredView = $scope.$watch(() => vm.loadActivityLog, () => {
      if (typeof vm.loadActivityLog === 'function') {
        vm.loadActivityLog();
        unWatchEnteredView();
      }
    });

    // Pull refresh handler
    vm.refreshAll = () => {
      loadAll(() => {
        $scope.$broadcast('scroll.refreshComplete');
      }, true, true);
    };

    vm.showModal = () => {
      ModalService.show({
        templateUrl: 'trainer/home/stripeModal/stripeModal.html',
        controller: 'StripeModalController as vm'
      }).then(() => {});
    };
  }
})();
// Intro Controller
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('TrainerIntroController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.trainer_intro', {
            url: "/intro",
            cache: false,
            params: {
                isAnimated: false
            },
            views: {
                'menuContent': {
                    templateUrl: "trainer/intro/intro.html",
                    controller: 'TrainerIntroController as vm'
                }
            }
        });
    }

    controller.$inject = ['$ionicSlideBoxDelegate', 'IntroService', 'NavigateService', 'PermissionsService'];

    function controller($ionicSlideBoxDelegate, IntroService, NavigateService, PermissionsService) {

        var vm = this;

        vm.slideIndex = 0;

        vm.businessName = PermissionsService.getCurrentUser().branding.business_name;

        vm.start = function () {
            NavigateService.disableBack().go('app.trainer_home');
        };

        vm.next = function () {
            if (vm.slideIndex != 4) {
                $ionicSlideBoxDelegate.next();
            } else if (vm.slideIndex === 4) {
                vm.start();
            }
        };

        vm.previous = function () {

            if (vm.slideIndex === 0) {
                vm.start();
            } else {
                $ionicSlideBoxDelegate.previous();
            }
        };

        // Called each time the slide changes
        vm.slideChanged = function (index) {
            vm.slideIndex = index;
        };

        // Ensure we set the flag in local storage that the intro has been seen
        // so it doesn't run again unless local storage is killed for any reason
        IntroService.introSeen();
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('IntroService', service);

    service.$inject = ['LocalStorageService', 'BrandingService'];

    function service(LocalStorageService, BrandingService) {

        var service = {};

        var state = {
            seen: false
        };

        var storeState = function () {

            LocalStorageService.setKey('introState', JSON.stringify(state));
        };

        var getStoredState = function () {

            var getData = LocalStorageService.getKey('introState');

            if (!getData) {
                return false;
            }

            return JSON.parse(getData);
        };

        service.setup = function () {

            var storedState = getStoredState();

            if (storedState) {

                state.seen = storedState.seen;
            }
        };

        service.hasSeenIntro = function () {

            if (BrandingService.isCustomApp()) {
                return true;
            }

            return state.seen;
        };

        service.introSeen = function () {

            state.seen = true;

            storeState();
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('TrainerNutritionPlanCreateModalController', controller).factory('NutritionPlanCreateModalService', service);

    controller.$inject = ['parameters', '$scope', 'NutritionEditorService', 'ApiClientService', '$q', 'NavigateService', 'AnalyticsService'];

    function controller(parameters, $scope, NutritionEditorService, ApiClientService, $q, NavigateService, AnalyticsService) {

        var vm = this;

        vm.nutritionPlan = NutritionEditorService.newInstance('plan');
        vm.trainerUserId = parameters.trainerUserId;
        if (parameters.trainerUserId) {
            vm.nutritionPlan.trainer_user_id = parameters.trainerUserId;
        }
        vm.nutritionPlanTemplates = [];
        vm.ready = false;

        ApiClientService.trainers.getAllNutritionPlans(vm.trainerUserId).success(function (response) {

            vm.nutritionPlanTemplates = response.nutritionPlans;

            vm.ready = true;
        }).error(function () {
            vm.ready = true;
        });

        vm.doCreate = function () {

            vm.processing = true;

            var getTemplate;

            if (vm.nutritionPlan.useTemplate !== 'new') {

                getTemplate = ApiClientService.nutrition.getPlan(vm.nutritionPlan.useTemplate);
            } else {

                getTemplate = $q.resolve(false);
            }

            getTemplate.then(function (templateContent) {

                if (templateContent) {
                    NutritionEditorService.setTemplate(templateContent.nutritionPlan, 'plan');
                }

                var postData = JSON.stringify(NutritionEditorService.getItem());

                ApiClientService.nutrition.addPlan(postData).success(function (response) {

                    NavigateService.go('app.trainer_nutrition_plan_edit', {
                        nutritionPlanId: response.nutritionPlan.id
                    });

                    $scope.closeModal();
                }).finally(function () {
                    vm.processing = false;
                });
            });
        };

        AnalyticsService.trackView('Trainer - Create Nutrition Plan Start');
    }

    service.$inject = ['ModalService'];

    function service(ModalService) {

        return {

            show: function (trainerUserId) {

                trainerUserId = trainerUserId || false;

                return ModalService.show({
                    templateUrl: "trainer/nutrition/createModal.html",
                    controller: 'TrainerNutritionPlanCreateModalController as vm',
                    params: {
                        trainerUserId: trainerUserId
                    }
                });
            }

        };
    }
})();
(() => {
  angular.module('MyPTHub').config(config).controller('TrainerNutritionPlanEditController', controller);

  config.$inject = ['$stateProvider'];

  function config($stateProvider) {

    $stateProvider.state('app.trainer_nutrition_plan_edit', {
      url: "/trainer_nutrition/plan/:nutritionPlanId",
      views: {
        'menuContent': {
          templateUrl: "trainer/nutrition/edit.html",
          controller: 'TrainerNutritionPlanEditController as vm'
        }
      }
    });

    $stateProvider.state('app.edit_trainer_nutrition_plan_via_customer', {
      url: "/trainer_nutrition/via_customer/{customerId:[0-9]+}/plan/{nutritionPlanId:[0-9]+}",
      views: {
        'menuContent': {
          templateUrl: "trainer/nutrition/edit.html",
          controller: 'TrainerNutritionPlanEditController as vm'
        }
      }
    });
  }

  controller.$inject = ['$stateParams', '$scope', 'ApiClientService', 'NutritionEditorService', 'NutritionPlanEditService', 'NavigateService', 'PromptDialogService', 'ModalService', 'PermissionsService', 'ToastService', 'ConfirmDialogService', 'AnalyticsService', 'CustomerFinderModalService'];

  function controller($stateParams, $scope, ApiClientService, NutritionEditorService, NutritionPlanEditService, NavigateService, PromptDialogService, ModalService, PermissionsService, ToastService, ConfirmDialogService, AnalyticsService, CustomerFinderModalService) {
    const vm = this;
    const mimes = {
      image: ['image/gif', 'image/x-icon', 'image/jpeg', 'image/jpg', 'image/png', 'image/svg+xml', 'image/tiff', 'image/webp'],
      document: ['application/msword', 'application/vnd.oasis.opendocument.text', 'application/rtf', 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'],
      pdf: ['application/pdf'],
      archive: ['application/x-tar', 'application/x-bzip2', 'application/gzip', 'application/x-7z-compressed', 'application/x-apple-diskimage', 'application/x-rar-compressed', 'application/zip'],
      spreadsheet: ['application/vnd.oasis.opendocument.spreadsheet', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet', 'application/vnd.ms-excel']
    };
    vm.saving = false;

    vm.doUpdate = () => {

      vm.saving = true;

      NutritionPlanEditService.saveChanges().then(() => {
        NavigateService.goBack();
        vm.saving = false;
      }, () => {
        vm.saving = false;
      });
    };

    vm.addMeal = () => {

      PromptDialogService.open({
        title: "Add Meal",
        message: "Please enter a name for the meal",
        buttonConfirm: "Create",
        buttonCancel: "Cancel",
        required: true
      }).then(function (name) {

        vm.nutritionPlan.addMeal(name);

        vm.nutritionPlan.updateTotals();
      });
    };

    vm.addMealFromTemplate = () => {
      ModalService.show({
        templateUrl: 'client/nutritionLog/selectMealTemplateModal.html',
        controller: 'SelectMealTemplateModalController as vm'
      }).then(function (assigned) {
        angular.forEach(assigned, template => {
          vm.nutritionPlan.addMealFromTemplate(template);
        });
      });
    };

    vm.editNote = () => {

      ModalService.show({
        templateUrl: "trainer/nutrition/editNoteModal.html",
        controller: 'TrainerEditNutritionPlanNoteModalController as vm'
      });
    };

    vm.editSettings = () => {

      ModalService.show({
        templateUrl: "trainer/nutrition/editSettingsModal.html",
        controller: 'TrainerEditNutritionPlanSettingsModalController as vm'
      });
    };

    vm.delete = () => {

      ConfirmDialogService.open({
        title: "Confirm delete",
        message: "Do you want to remove this nutrition plan?",
        buttonConfirm: "Confirm",
        buttonCancel: "Cancel"
      }).then(function () {

        ApiClientService.nutrition.deletePlan(vm.nutritionPlanId).then(() => {

          NutritionEditorService.reset();

          NavigateService.clearCache().then(NavigateService.goBack);
        }, () => {
          vm.saving = false;
        });
      });
    };

    vm.assignToClient = () => {
      CustomerFinderModalService.open({
        trainerUserId: vm.nutritionPlan.trainer_user_id,
        currentCustomerAssignments: vm.nutritionPlan.assigned_customers,
        currentGroupAssignments: vm.nutritionPlan.customer_groups,
        modalTitle: 'Assign Clients',
        mode: 'multi',
        includeGroups: true
      }).then(assignedItems => {
        const assignedCustomerIds = [];
        const assignedCustomerGroupIds = [];

        assignedItems.customers.map(customer => assignedCustomerIds.push(customer.id));
        assignedItems.customerGroups.map(customerGroup => assignedCustomerGroupIds.push(customerGroup.id));

        ApiClientService.nutrition.updatePlan(vm.nutritionPlan.id, {
          assigned_customer_ids: assignedCustomerIds,
          assigned_customer_group_ids: assignedCustomerGroupIds
        }).then(() => {
          vm.nutritionPlan.assigned_customers = assignedItems.customers;
          vm.nutritionPlan.customer_groups = assignedItems.customerGroups;
        }, () => ToastService.show('Could not assign clients, please try again later'));
      });

      /*
       ModalService.show({
       templateUrl: "trainer/nutrition/customerAssignModal.html",
       controller: 'TrainerEditNutritionPlanCustomerAssignModalController as vm',
       params: {
       assignedCustomers: vm.nutritionPlan.assigned_customers,
       nutritionPlanId: vm.nutritionPlan.id,
       nutritionPlan: vm.nutritionPlan
       }
       }).then(function (assignedClients) {
       vm.nutritionPlan.assigned_customers = assignedClients;
       });
       */

      /*
       ApiClientService.nutrition.updatePlan(parameters.nutritionPlanId, {
       assigned_customer_ids: ids
       }).then(function() {
       $scope.closeModal(list);
       }, function() {
       vm.saving = false;
       });
       */
    };

    vm.assignToGroup = () => {

      ModalService.show({
        templateUrl: "trainer/nutrition/groupAssignModal.html",
        controller: 'TrainerEditNutritionPlanGroupAssignModalController as vm',
        params: {
          assignedGroups: vm.nutritionPlan.nutrition_plan_groups,
          nutritionPlanId: vm.nutritionPlan.id,
          nutritionPlan: vm.nutritionPlan
        }
      }).then(function (assignedGroups) {
        vm.nutritionPlan.nutrition_plan_groups = assignedGroups;
      });
    };

    $scope.$on('$ionicView.beforeEnter', function () {

      vm.nutritionPlanId = parseInt($stateParams.nutritionPlanId);

      vm.ready = false;

      var currentItem = NutritionEditorService.getItem();

      if (!currentItem || currentItem.id !== vm.nutritionPlanId) {

        ApiClientService.nutrition.getPlan(vm.nutritionPlanId).success(function (response) {

          vm.canEdit = PermissionsService.userIsTrainerAdminOrOwner(response.nutritionPlan.trainer_user_id);

          vm.title = vm.canEdit ? 'Edit' : 'Nutrition Plan';

          vm.nutritionPlan = NutritionEditorService.setItem(response.nutritionPlan, 'plan');

          vm.nutritionPlan.canEdit = vm.canEdit;

          vm.nutritionPlan.assigned_customers = response.nutritionPlan.assigned_customers;
          vm.nutritionPlan.customer_groups = response.nutritionPlan.customer_groups;
          vm.nutritionPlan.nutrition_plan_groups = response.nutritionPlan.nutrition_plan_groups;

          let files = response.nutritionPlan.files;
          files.map(file => {
            if (mimes.image.includes(file.mime)) {
              file.icon = 'fa-file-image-o';
            } else if (mimes.spreadsheet.includes(file.mime)) {
              file.icon = 'fa-file-excel-o';
            } else if (mimes.pdf.includes(file.mime)) {
              file.icon = 'fa-file-pdf-o';
            } else if (mimes.archive.includes(file.mime)) {
              file.icon = 'fa-file-archive-o';
            } else if (mimes.document.includes(file.mime)) {
              file.icon = 'fa-file-text-o';
            } else {
              file.icon = 'fa-file-o';
            }
            file.name = `${file.original_name}.${file.extension}`;
          });

          vm.files = files;

          vm.ready = true;
        }).error(function (response) {

          if (response.error && response.error.error_name && response.error.error_name === 'SUBSCRIPTION_UPGRADE_REQUIRED') {
            ToastService.show(response.error.message);
            NavigateService.goBack();
          } else {
            ToastService.show('Could not get nutrition plan, please try again later');
          }
        });
      } else {

        console.log('restored item');
        vm.nutritionPlan = NutritionEditorService.getItem();
        vm.title = vm.canEdit ? 'Edit Nutrition Plan' : 'Nutrition Plan';
        vm.canEdit = vm.nutritionPlan.canEdit;
        vm.ready = true;
      }
    });

    $scope.$on('$ionicView.enter', function () {
      AnalyticsService.trackView('Trainer - Edit Nutrition Plan');
    });

    vm.viewFile = function (file) {
      ModalService.show({
        templateUrl: 'common/files/filesViewModal.html',
        controller: 'FilesViewController as vm',
        params: {
          file,
          trainer: false
        }
      });
    };
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('TrainerEditNutritionPlanNoteModalController', controller);

    controller.$inject = ['$scope', 'NutritionEditorService', 'NutritionPlanEditService'];

    function controller($scope, NutritionEditorService, NutritionPlanEditService) {

        var vm = this;

        var currentItem = NutritionEditorService.getItem();

        vm.note = currentItem.note;
        vm.canEdit = currentItem.canEdit;

        vm.save = function () {

            vm.saving = true;

            currentItem.note = vm.note;

            NutritionPlanEditService.saveChanges().then(function () {
                $scope.closeModal(true);
            }, function () {
                vm.saving = false;
            });
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('NutritionPlanEditService', service);

    service.$inject = ['ApiClientService', 'NutritionEditorService', 'ToastService'];

    function service(ApiClientService, NutritionEditorService, ToastService) {

        var service = {};

        service.saveChanges = function () {

            return ApiClientService.nutrition.updatePlan(NutritionEditorService.getItem().id, NutritionEditorService.getPayload()).error(function () {

                ToastService.show('Could not save - please try again');
            });
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('TrainerEditNutritionPlanSettingsModalController', controller);

    controller.$inject = ['NutritionPlanEditService', '$scope'];

    function controller(NutritionPlanEditService, $scope) {

        var vm = this;

        vm.save = function () {

            vm.saving = true;

            NutritionPlanEditService.saveChanges().then(function () {
                $scope.closeModal(true);
            }, function () {
                vm.saving = false;
            });
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('TrainerEditNutritionPlanGroupAssignModalController', controller);

    controller.$inject = ['parameters', '$scope', '$filter', 'ApiClientService', 'PermissionsService', 'AnalyticsService'];

    function controller(parameters, $scope, $filter, ApiClientService, PermissionsService, AnalyticsService) {

        var vm = this;

        vm.currentAssignments = parameters.assignedGroups;

        vm.isCurrentTrainerUser = PermissionsService.getLoggedInUserId() === parameters.nutritionPlan.trainer_user_id;

        vm.assignedIds = [];
        angular.forEach(vm.currentAssignments, function (group) {
            vm.assignedIds.push(group.id);
        });

        var initAssignedCount = vm.assignedIds.length;

        ApiClientService.nutritionGroups.list(parameters.nutritionPlan.trainer_user_id).success(function (response) {

            response.groups.map(function (group) {
                group.use = vm.assignedIds.indexOf(group.id) !== -1;
            });

            vm.groups = response.groups;

            vm.filteredGroups = angular.copy(response.groups);

            vm.noGroups = response.length === 0;
        });

        // Customer List Search filter
        $scope.$watch(function () {
            return vm.searchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.searchFilterActive = true;
                vm.filteredGroups = $filter('filter')(vm.groups, {
                    title: newVal
                });
            } else {
                vm.searchFilterActive = false;
                vm.filteredGroups = vm.groups;
            }
        });

        vm.assign = function (group) {

            group.use = !group.use;

            var findOriginal = $filter('filter')(vm.groups, {
                id: group.id
            }, true);

            if (findOriginal.length > 0) {

                var originalIndex = vm.groups.indexOf(findOriginal[0]);

                if (originalIndex !== -1) {
                    vm.groups[originalIndex].use = group.use;
                }
            }
        };

        vm.save = function () {

            vm.saving = true;

            var toAssign = $filter('filter')(vm.groups, {
                use: true
            });

            if (toAssign.length === 0 && initAssignedCount === 0) {

                $scope.closeModal([]);

                return true;
            }

            var ids = [];
            var list = [];

            toAssign.map(function (group) {
                ids.push(group.id);
                list.push(group);
            });

            ApiClientService.nutrition.updatePlan(parameters.nutritionPlanId, {
                assigned_group_ids: ids
            }).then(function () {
                $scope.closeModal(list);
            }, function () {
                vm.saving = false;
            });
        };

        AnalyticsService.trackView('Trainer - Assign Nutrition Plan To Groups');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('TrainerNutritionController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.trainer_nutrition', {
            url: "/trainer_nutrition/:trainerUserId?",
            views: {
                'menuContent': {
                    templateUrl: "trainer/nutrition/nutrition.html",
                    controller: 'TrainerNutritionController as vm'
                }
            }
        });
    }

    controller.$inject = ['$stateParams', '$scope', 'ApiClientService', '$filter', 'PermissionsService', 'NavigateService', 'PromptDialogService', 'NutritionPlanCreateModalService', 'AnalyticsService', '$ionicLoading', 'ToastService', 'mpthconf', '$q', '$ionicScrollDelegate'];

    function controller($stateParams, $scope, ApiClientService, $filter, PermissionsService, NavigateService, PromptDialogService, NutritionPlanCreateModalService, AnalyticsService, $ionicLoading, ToastService, mpthconf, $q, $ionicScrollDelegate) {

        // temp horrible hack to work around issue of disappearing tabs
        // in some weird circumstances.
        $scope.$on('$ionicView.afterEnter', function () {
            $(window).trigger('resize');
        });

        var vm = this;

        vm.cdnBase = mpthconf.cdn.base;

        vm.goToNutritionPlan = function (nutritionPlan) {

            if (nutritionPlan.enabled) {
                NavigateService.go('app.trainer_nutrition_plan_edit', {
                    nutritionPlanId: nutritionPlan.id
                });
            }
        };

        vm.goToClient = function (customer) {

            NavigateService.go('app.trainer_client_profile', {
                customerId: customer.id,
                tab: 'nutrition'
            });
        };

        vm.goToGroup = function (group) {

            NavigateService.go('app.trainer_nutrition_plans_group', {
                groupId: group.id
            });
        };

        vm.createPlan = function () {
            NutritionPlanCreateModalService.show(vm.trainerUserId);
        };

        /*****************************************************************
         * All Tab
         *****************************************************************/

        var loadAll = function (refreshing) {

            refreshing = refreshing || false;

            if (!vm.allNutritionPlans || refreshing) {

                vm.allNutritionPlansSearchFilter = '';

                ApiClientService.trainers.getAllNutritionPlans(vm.trainerUserId).success(function (response) {

                    vm.allNutritionPlans = response.nutritionPlans;
                    vm.filteredAllNutritionPlans = angular.copy(response.nutritionPlans);

                    vm.noAllNutritionPlans = vm.allNutritionPlans.length === 0;
                }).finally(function () {
                    if (refreshing) {
                        $scope.$broadcast('scroll.refreshComplete');
                    }
                });
            }
        };

        // All Nutrition Plans Search filter
        $scope.$watch(function () {
            return vm.allNutritionPlansSearchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.filteredAllNutritionPlans = $filter('filter')(vm.allNutritionPlans, {
                    title: newVal
                });
            } else {
                vm.filteredAllNutritionPlans = vm.allNutritionPlans;
            }
        });

        vm.allShow = function () {
            $ionicScrollDelegate.resize();
            loadAll();
        };

        /*****************************************************************
         * Groups Tab
         *****************************************************************/

        var loadGroups = function (refreshing) {

            refreshing = refreshing || false;

            if (!vm.groups || refreshing) {

                vm.groupsSearchFilter = '';

                ApiClientService.trainers.getAllNutritionPlansByGroup(vm.trainerUserId).success(function (response) {

                    response.groups.map(function (g) {
                        g.nutritionPlanCount = g.nutrition_plans.length;
                    });

                    vm.groups = response.groups;

                    vm.filteredGroups = angular.copy(response.groups);

                    vm.noGroups = vm.groups.length === 0;
                }).finally(function () {
                    if (refreshing) {
                        $scope.$broadcast('scroll.refreshComplete');
                    }
                });
            }
        };

        // Groups Search filter
        $scope.$watch(function () {
            return vm.groupsSearchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.filteredGroups = $filter('filter')(vm.groups, {
                    title: newVal
                });
            } else {
                vm.filteredGroups = vm.groups;
            }
        });

        vm.groupsShow = function () {
            $ionicScrollDelegate.resize();
            loadGroups();
        };

        vm.createGroup = function () {

            PromptDialogService.open({
                title: "Create Nutrition Plan Group",
                message: "Please enter a name for the group",
                buttonConfirm: "Create",
                buttonCancel: "Cancel",
                required: true
            }).then(function (name) {

                ApiClientService.nutritionGroups.create({
                    title: name
                }, vm.trainerUserId).success(function () {
                    loadGroups(true);
                });
            });
        };

        /*****************************************************************
         * Clients Tab
         *****************************************************************/

        var loadClients = function (refreshing) {

            refreshing = refreshing || false;

            if (!vm.clients || refreshing) {

                vm.clientsSearchFilter = '';

                ApiClientService.customers.list(vm.trainerUserId, ['direct_nutrition_plan_count']).success(function (response) {

                    // set full name on root object for search filter
                    response.customers.map(function (c) {
                        c.full_name = c.user.first_name + ' ' + c.user.last_name;
                    });

                    vm.clients = response.customers;

                    vm.filteredClients = angular.copy(response.customers);

                    vm.noClients = vm.clients.length === 0;
                }).finally(function () {
                    if (refreshing) {
                        $scope.$broadcast('scroll.refreshComplete');
                    }
                });
            }
        };

        // Clients Search filter
        $scope.$watch(function () {
            return vm.clientsSearchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.filteredClients = $filter('filter')(vm.clients, {
                    full_name: newVal
                });
            } else {
                vm.filteredClients = vm.clients;
            }
        });

        vm.byClientShow = function () {
            $ionicScrollDelegate.resize();
            loadClients();
        };

        /*****************************************************************
         * Find a Plan Tab
         *****************************************************************/

        vm.plansSearchFilter = null;

        vm.viewNutritionPlans = function (group) {

            NavigateService.go('app.trainer_nutrition_library_group', {
                groupId: group.id
            });
        };

        var loadLibraryNutritionPlans = function (refreshing) {

            refreshing = refreshing || false;

            if (!vm.libraryPlans || refreshing) {

                ApiClientService.nutritionLibrary.getLibraryGroups().success(function (response) {

                    vm.libraryPlans = response.groups;

                    vm.noLibraryPlans = vm.libraryPlans.length === 0;
                }).finally(function () {
                    if (refreshing) {
                        $scope.$broadcast('scroll.refreshComplete');
                    }
                });
            }
        };

        vm.findAPlan = function () {
            loadLibraryNutritionPlans();
        };

        vm.useLibraryGroup = function (group) {

            vm.processing = true;

            var currentUser = PermissionsService.getCurrentUser();

            var tag = 'metric';

            if (currentUser.country === 'US') {
                tag = 'imperial';
            }

            ApiClientService.nutritionGroups.get(group.id, tag).success(function (response) {

                vm.processing = false;

                var nutritionPlanIds = [];

                response.group.nutrition_plans.map(function (nutritionPlan) {
                    nutritionPlanIds.push(nutritionPlan.id);
                });

                $ionicLoading.show({
                    template: 'Please wait...'
                }).then(function () {
                    vm.toDuplicateCount = nutritionPlanIds.length;
                    vm.duplicatedCount = 1;
                    vm.creatingGroup = false;

                    vm.percentComplete = 0;

                    var duplicateNutritionPlanActions = [];
                    angular.forEach(nutritionPlanIds, function (id) {

                        var duplicate = ApiClientService.nutrition.duplicatePlan(id);

                        duplicateNutritionPlanActions.push(duplicate);

                        duplicate.success(function () {
                            vm.duplicatedCount++;
                            vm.percentComplete += 90 / vm.toDuplicateCount;
                        });
                    });

                    $q.all(duplicateNutritionPlanActions).then(function (duplicateResponses) {
                        vm.creatingGroup = true;

                        var newNutritionPlanIds = [];
                        duplicateResponses.map(function (response) {
                            newNutritionPlanIds.push(response.data.nutritionPlan.id);
                        });

                        ApiClientService.nutritionGroups.create({
                            title: response.group.title,
                            nutrition_plan_ids: newNutritionPlanIds
                        }).success(function (response) {

                            ToastService.show('Nurition Plans successfully added to your account');
                            $ionicLoading.hide();
                        });
                    }, function (response) {
                        ToastService.show(response.data.error.message);
                        $ionicLoading.hide();
                    });
                });
            });
        };

        // Pull refresh handler
        vm.refresh = function () {

            switch (vm.selectedTabIndex) {

                // All
                case 0:
                    loadAll(true);
                    break;

                // Groups
                case 1:
                    loadGroups(true);
                    break;

                // By Client
                case 2:
                    loadClients(true);
                    break;

                // Find a Plan
                case 3:
                    loadLibraryNutritionPlans(true);
                    break;

            }
        };

        $scope.$on('$ionicView.beforeEnter', function () {

            vm.isCurrentTrainerUser = !$stateParams.trainerUserId || PermissionsService.getLoggedInUserId() === parseInt($stateParams.trainerUserId);

            vm.trainerUserId = $stateParams.trainerUserId;

            vm.canEdit = !vm.trainerUserId || PermissionsService.userIsTrainerAdminOrOwner(vm.trainerUserId);
        });

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Trainer - Nutrition');
        });
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('TrainerClientNutritionLogDayController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.trainer_view_nutrition_log_day', {
            url: "/trainer_nutrition_log_day_viewer/:customerId/:date",
            views: {
                'menuContent': {
                    templateUrl: "trainer/nutritionLogDay/nutritionLogDay.html",
                    controller: 'TrainerClientNutritionLogDayController as vm'
                }
            }
        });
    }

    controller.$inject = ['$scope', '$stateParams', 'ApiClientService', 'DayPickerService', 'NavigateService', 'NutritionEditorService', '$q', 'AnalyticsService'];

    function controller($scope, $stateParams, ApiClientService, DayPickerService, NavigateService, NutritionEditorService, $q, AnalyticsService) {

        var vm = this;

        vm.datePickerToggle = DayPickerService.toggleOpen;

        var getCustomer = function () {

            return ApiClientService.customers.get($stateParams.customerId).success(function (response) {
                vm.customer = response.customer;
            });
        };

        var getLogDay = function () {

            return ApiClientService.nutritionlog.dayLog($stateParams.customerId, $stateParams.date).success(function (response) {

                response.nutritionLogDay.canEdit = false;

                vm.dayLog = NutritionEditorService.setItem(response.nutritionLogDay, 'log');
            }).error(function (response) {

                if (response.error && response.error.error_name && response.error.error_name === 'CUSTOMER_NUTRITION_DAY_LOG_NOT_CREATED_YET') {
                    vm.nothingLogged = true;
                }
            });
        };

        $scope.$on('$ionicView.beforeEnter', function () {

            vm.dayLog = null;
            vm.customer = null;

            vm.date = $stateParams.date;

            vm.ready = false;
            vm.entered = false;
            vm.nothingLogged = false;

            $q.all([getCustomer(), getLogDay()]).finally(function () {
                vm.ready = true;
            });

            var setDate = function () {
                vm.currentDate = DayPickerService.getDate();
                vm.currentDateFormatted = vm.currentDate.format('Do MMMM YYYY');

                var urlDate = vm.currentDate.format('YYYY-MM-DD');
                if (urlDate !== vm.date) {
                    NavigateService.go('app.trainer_view_nutrition_log_day', {
                        date: urlDate,
                        customerId: $stateParams.customerId
                    });
                }
            };

            DayPickerService.registerDayPickedEventHandler($scope, setDate);

            var date = moment(vm.date, 'YYYY-MM-DD').hour(12);

            DayPickerService.setDate(date);
        });

        $scope.$on('$ionicView.enter', function () {

            vm.entered = true;

            AnalyticsService.trackView('Trainer - View Logged Nutrition');
        });
    }
})();
(() => {
  angular.module('MyPTHub').config(config).controller('ProvisionAccountController', controller);

  function config($stateProvider) {
    $stateProvider.state('app.provision_account', {
      url: '/packages/provision',
      views: {
        menuContent: {
          templateUrl: 'trainer/provision/views/index.html',
          controller: 'ProvisionAccountController',
          controllerAs: 'vm'
        }
      },
      resolve: {
        currentUser(AuthenticationStateService) {
          return AuthenticationStateService.getCurrentUser();
        },
        provisioned(AuthenticationStateService, NavigateService, ToastService) {
          const user = AuthenticationStateService.getCurrentUser();
          if (user.merchant_is_provisioned) {
            ToastService.show('This account is already provisioned');
          }

          return user.merchant_is_provisioned;
        }
      }
    });
  }

  function controller(LogoService, NavigateService, PermissionsService, ToastService, currentUser, provisioned, SubscriptionStateService) {
    if (provisioned) {
      NavigateService.replaceCurrentState().disableBack().go('app.trainer_packages');
      return NavigateService.clearCache();
    }
    const vm = this;
    vm.logoUrl = LogoService.getLogo();
    vm.currentUser = currentUser;
    vm.hasUnlimitedPlus = SubscriptionStateService.hasUnlimitedPlus();

    vm.goHome = () => {
      NavigateService.goHome();
    };

    vm.provision = type => {
      NavigateService.go('app.trainer_provision_form', {
        type
      });
    };
  }
})();
(() => {
  class TrainerProvisionPackagesController {
    /**
     * @param ApiClientService
     * @param $stateParams
     * @param currentUser
     * @param AuthenticationService
     * @param AuthenticationStateService
     * @param ToastService
     * @param NavigateService
     */
    constructor(ApiClientService, $stateParams, currentUser, AuthenticationService, AuthenticationStateService, ToastService, PackageSwitcherService, NavigateService) {
      this.ApiClientService = ApiClientService;
      this.$stateParams = $stateParams;
      this.currentUser = currentUser;
      this.AuthenticationService = AuthenticationService;
      this.AuthenticationStateService = AuthenticationStateService;
      this.ToastService = ToastService;
      this.NavigateService = NavigateService;
      this.PackageSwitcherService = PackageSwitcherService;

      this.accountType = $stateParams.type === 'personal' ? 'individual' : 'company';
      this.personalPackages = $stateParams.type === 'personal';
      if (currentUser.dob) {
        const dob = moment(currentUser.dob, 'YYYY-MM-DD');
        this.dob = {
          d: dob.format('DD'),
          m: dob.format('MM'),
          y: dob.format('YYYY')
        };
      }
      this.showFields = {
        dob: !currentUser.dob || currentUser.dob.length === 0
      };
      this.merchant = {
        business_type: this.accountType,
        address1: currentUser.address1,
        address2: currentUser.address2,
        town_city: currentUser.town_city,
        county: currentUser.county,
        postcode: currentUser.postcode,
        dob: currentUser.dob,
        country: currentUser.country,
        currency: currentUser.merchant_currency,
        allow_personal_packages: this.personalPackages
      };
      this.months = [];
      this.days = [];
      this.years = [];

      let s = 1;
      let e = 31;
      while (s <= e) {
        this.days.push((s < 10 ? '0' : '') + s);
        s += 1;
      }

      s = 1;
      e = 12;
      while (s <= e) {
        this.months.push((s < 10 ? '0' : '') + s);
        s += 1;
      }

      const currentYear = new Date().getFullYear();
      s = currentYear - 100;
      e = currentYear - 16;
      while (s <= e) {
        s += 1;
        this.years.push(`${s}`);
      }
    }

    provision() {
      this.processing = true;
      this.merchant.dob = `${this.dob.y}-${this.dob.m}-${this.dob.d}`;

      this.ApiClientService.merchant.provision(this.merchant).success(() => {
        const user = this.AuthenticationStateService.getCurrentUser();
        this.AuthenticationStateService.setCurrentUser(user);
        this.ToastService.show('Account Provisioned');
        this.NavigateService.removeCache(['app.trainer_packages']);
        this.NavigateService.goHome();
        this.processing = false;
        this.merchant = {
          business_type: this.accountType,
          address1: this.currentUser.address1,
          address2: this.currentUser.address2,
          town_city: this.currentUser.town_city,
          county: this.currentUser.county,
          postcode: this.currentUser.postcode,
          dob: this.currentUser.dob,
          country: this.currentUser.country,
          currency: this.currentUser.merchant_currency,
          allow_personal_packages: this.personalPackages
        };
      }).error(response => {
        if (response.error) {
          this.ToastService.show(response.error.message);
        } else {
          this.ToastService.show('There was a problem, please try again');
        }
        this.processing = false;
      });
    }
  }

  const config = $stateProvider => {
    $stateProvider.state('app.trainer_provision_form', {
      url: '/packages/provision/create/{type}',
      views: {
        menuContent: {
          templateUrl: 'trainer/provision/views/form.html',
          controller: 'TrainerProvisionPackagesController',
          controllerAs: 'vm'
        }
      },
      resolve: {
        currentUser(AuthenticationStateService) {
          return AuthenticationStateService.getCurrentUser();
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('TrainerProvisionPackagesController', TrainerProvisionPackagesController);
})();
(() => {
  angular.module('MyPTHub').config(config).controller('PackagesIndexController', controller);

  function config($stateProvider) {
    $stateProvider.state('app.trainer_packages', {
      url: '/packages',
      views: {
        menuContent: {
          templateUrl: 'trainer/packages/views/index.html',
          controller: 'PackagesIndexController',
          controllerAs: 'vm'
        }
      },
      resolve: {
        provisioned(AuthenticationStateService, AuthenticationService) {
          return AuthenticationService.refreshUser().then(user => {
            AuthenticationStateService.setCurrentUser(user);
            return user.merchant_is_provisioned;
          });
        }
      }
    });
  }

  function controller($scope, NavigateService, PermissionsService, ToastService, provisioned, AuthenticationStateService, ConfirmDialogService, ApiClientService, PackageSwitcherService, $ionicHistory, SubscriptionStateService, ModalService) {
    const currentUser = AuthenticationStateService.getCurrentUser();

    if (PermissionsService.userIsTrainerAdmin()) {
      if (!provisioned) {
        NavigateService.replaceCurrentState().disableBack().go('app.provision_account');
      }
    } else if (currentUser.permissions_active.hasOwnProperty('can_create_packages') && currentUser.permissions_active.can_create_packages === false) {
      console.log('literally no');
      ToastService.show('You\'re not allowed to create packages');
      NavigateService.goHome();
    } else if (!provisioned && !PermissionsService.userIsTrainerAdmin()) {
      NavigateService.replaceCurrentState().disableBack().go('app.trainer_provision_form', {
        type: 'personal'
      });
    }

    const vm = this;

    const year = moment().year();

    vm.legacyUser = currentUser.merchant_is_provisioned && !currentUser.stripe_standard;
    vm.pastCutoff = () => {
      if (year === 2018) {
        return false;
      }

      return vm.legacyUser;
    };

    vm.showModal = () => {
      ModalService.show({
        templateUrl: 'trainer/home/stripeModal/stripeModal.html',
        controller: 'StripeModalController as vm'
      }).then(() => {});
    };

    vm.unlimitedPlusEnabled = SubscriptionStateService.hasUnlimitedPlus();

    vm.returnHome = () => NavigateService.goHome();

    vm.refresh = () => {
      switch (vm.innerView) {
        case 'packages':
          $scope.$broadcast('refresh.packages.packages');
          $scope.$broadcast('scroll.refreshComplete');
      }
    };

    vm.canCreateNewAccount = () => currentUser.trainer_accounts < 2 && currentUser.is_owner && SubscriptionStateService.hasUnlimitedPlus();
    vm.createText = currentUser.merchant_account_type === 'company' ? 'Create Personal Account' : 'Create Organisation Account';
    vm.switchText = currentUser.merchant_account_type === 'company' ? ' Personal Account' : ' Organisation Account';
    vm.switchType = currentUser.merchant_account_type === 'company' ? 'personal' : 'company';
    vm.title = currentUser.merchant_account_type === 'company' ? 'Company Packages' : 'Personal Packages';

    vm.createAccount = () => {
      const accountType = currentUser.merchant_account_type === 'company' ? 'a personal' : 'an organisation';
      const accountCreate = currentUser.merchant_account_type === 'company' ? 'personal' : 'company';
      ConfirmDialogService.open({
        title: 'Confirm',
        message: `Are you sure you want to create ${accountType} account?`,
        buttonConfirm: 'Confirm',
        buttonCancel: 'Cancel'
      }).then(() => {
        NavigateService.go('app.trainer_provision_form', {
          type: accountCreate
        });
        // Disabled for Apps.
        // Stripe accounts are now done via oauth
        // ApiClientService.swapper.createSecondaryAccount().then(() => {
        //   ToastService.show('Account Created');
        //   NavigateService.go('app.trainer_provision_form', {
        //     type: accountCreate,
        //   });
        // });
      });
    };

    vm.showSwitcher = () => currentUser.trainer_accounts >= 2 && currentUser.is_owner;

    vm.switchAccount = () => {
      PackageSwitcherService.changeAccount(vm.switchType);
    };
  }
})();
(() => {
  class TrainerPackageCreateAndEditController {
    /**
     * @param ApiClientService
     * @param $scope
     * @param NavigateService
     * @param ToastService
     * @param $stateParams
     * @param $ionicSlideBoxDelegate
     * @param AuthenticationStateService
     */
    constructor(ApiClientService, $scope, NavigateService, ToastService, $stateParams, $state, $ionicSlideBoxDelegate, AuthenticationStateService, SubscriptionStateService) {
      // TODO: Refactor this mess.
      this.ApiClientService = ApiClientService;
      this.$scope = $scope;
      this.NavigateService = NavigateService;
      this.ToastService = ToastService;
      this.$stateParams = $stateParams;
      this.$ionicSlideBoxDelegate = $ionicSlideBoxDelegate;
      this.AuthenticationStateService = AuthenticationStateService;
      this.currentUser = AuthenticationStateService.getCurrentUser();
      this.$state = $state;
      this.SubscriptionStateService = SubscriptionStateService;

      this.hasUnlimitedPlus = SubscriptionStateService.hasUnlimitedPlus();
      this.mode = $stateParams.packageId ? 'edit' : 'create';
      this.preventInput = this.mode === 'edit';
      this.schedules = [];
      this.selectedSchedules = [];
      this.packageTags = [];
      this.selectedPackageTags = [];
      this.nutritionPlans = [];
      this.selectedNutritionPlans = [];
      this.workouts = [];
      this.selectedWorkouts = [];
      this.files = [];
      this.selectedFiles = [];
      this.customers = [];
      this.selectedCustomers = [];
      this.trainers = [];
      this.selectedTrainer = [];
      this.fieldErrors = {};
      this.events = [];
      this.selectedEvents = [];
      this.package = {
        title: null,
        description: null,
        tag: null,
        photo: null,
        price: null,
        videoUrl: null,
        schedules: [],
        nutritionPlans: [],
        workouts: [],
        sessions: 0,
        files: [],
        visibility: true,
        free: false,
        timePeriod: null,
        numberOfSessions: null,
        interval_amount: 1,
        interval_unit: 'month',
        available_from: null,
        available_until: null,
        unlimited_credit: false,
        topup: !!$stateParams.topUp
      };
      // this.package.topUp = !!$stateParams.topUp;
      this.enableCredits = false;
      this.availableNow = true;
      this.availableForever = true;
      this.slideIndex = 0;
      this.initialLoading = true;
      this.loaded = [];
      this.scheduleDetails = [];
      this.paymentPeriods = [{ title: 'Days', value: 'day' }, { title: 'Weeks', value: 'week' }, { title: 'Months', value: 'month' }, { title: 'Years', value: 'year' }];
      this.processing = false;
      if (this.package.topup) {
        this.enableCredits = true;
      }

      console.log(this.package);

      this.availablePeriods = [{ title: 'Recurring', value: 'recurring', free: false }, { title: 'Set Period', value: 'setperiod', free: false }, { title: 'Doesn\'t Expire (One Off Payment)', value: 'single', free: false }, { title: 'Set Period', value: 'setperiod', free: true }, { title: 'Forever', value: 'single', free: true }];

      this.initialLoad();
      this.NavigateService.removeCache(['app.trainer_packages']);
      this.packageFreeChanged();
    }

    packageFreeChanged() {
      this.package.type = false;
      this.periods = this.availablePeriods.filter(i => i.free === this.package.free);
      this.package.type = 'single';
    }

    logLoaded(name) {
      this.loaded.push(name);

      if (this.loaded.length === 8) {
        this.initialLoading = false;
      }
    }

    showEvents() {
      if (this.package.unlimited_credit) {
        this.hasEvents = true;

        return setTimeout(() => {
          this.$ionicSlideBoxDelegate.update();
        }, 1500);
      }
      this.hasEvents = false;
      this.$ionicSlideBoxDelegate.update();
    }

    initialLoad() {
      this.loadFiles();
      this.loadWorkouts();
      this.loadNutritionPlans();
      this.loadSchedules();
      this.loadPackageTags();
      this.loadCustomers();
      this.loadTrainers();
      this.loadEvents();
      if (this.mode === 'edit') {
        this.ApiClientService.packages.get(parseInt(this.$stateParams.packageId, 10)).success(resp => {
          this.package = resp.package;
          if (this.package.available_from) {
            this.package.available_from = moment(this.package.available_from).toDate();
          }
          if (this.package.available_until) {
            this.package.available_until = moment(this.package.available_until).toDate();
          }
          if (resp.package.schedules.length > 0) {
            this.enableSchedules = true;
            resp.package.schedules.map(schedule => {
              this.selectedSchedules[schedule.id] = true;
              this.scheduleDetails[schedule.id] = {
                schedule_id: schedule.id,
                start_date: schedule.pivot.start_date ? moment(schedule.pivot.start_date).toDate() : null,
                unassign_entities_on_end: schedule.pivot.unassign_entities_on_end === 1,
                customer_can_change_start_date: schedule.pivot.customer_can_change_start_date === 1
              };
              return true;
            });
          }
          if (resp.package.nutrition_plans.length > 0) {
            this.enableNutritionPlans = true;
            resp.package.nutrition_plans.map(nutrition => {
              this.selectedNutritionPlans[nutrition.id] = true;

              return true;
            });
          }
          if (resp.package.calendar_event.length > 0) {
            resp.package.calendar_event.map(event => {
              this.selectedEvents[event.id] = true;
              return true;
            });
          }
          if (resp.package.workouts.length > 0) {
            this.enableWorkouts = true;
            resp.package.workouts.map(workout => {
              this.selectedWorkouts[workout.id] = true;
              return true;
            });
          }
          if (resp.package.files.length > 0) {
            this.enableFiles = true;
            resp.package.files.map(file => {
              this.selectedFiles[file.id] = true;
              return true;
            });
          }
          this.selectedTrainer = resp.package.credit_assign_to_trainer_user_id ? resp.package.credit_assign_to_trainer_user_id.toString() : null;
          this.enableCredits = resp.package.credit_amount > 0;
          this.price = resp.package.price / 100;
          this.packageCredits = this.package.credit_amount;

          this.preventInput = false;

          this.$ionicSlideBoxDelegate.update();
        });
      }
    }

    loadTrainers() {
      this.ApiClientService.trainerUsers.list().success(response => {
        this.trainers = response.trainerUsers;
        this.logLoaded('trainers');
      }).error(resp => {
        console.error('unable to load trainers', resp);
      });
    }

    loadEvents() {
      this.ApiClientService.events.getAll().success(resp => {
        this.events = resp.events;
        this.logLoaded('events');
      }).error(resp => {
        console.error('unable to load events', resp);
      });
    }

    loadFiles() {
      this.ApiClientService.files.listAll().success(resp => {
        this.files = resp.files;
        this.logLoaded('files');
      }).error(resp => {
        console.error('Unable to load files', resp);
      });
    }

    loadWorkouts() {
      this.ApiClientService.trainers.getAllWorkouts().success(resp => {
        this.workouts = resp.workouts;
        this.logLoaded('workouts');
      }).error(resp => {
        console.error('Unable to load workouts', resp);
      });
    }

    loadNutritionPlans() {
      this.ApiClientService.trainers.getAllNutritionPlans().success(resp => {
        this.nutritionPlans = resp.nutritionPlans;
        this.logLoaded('nutrition');
      }).error(resp => {
        console.error('Unable to load nutrition plans', resp);
      });
    }

    loadSchedules() {
      this.ApiClientService.scheduler.scheduleTemplateIndexTrainerUser(this.currentUser.id).success(resp => {
        this.schedules = resp.templates;
        this.logLoaded('schedules');
      }).error(resp => {
        console.error('Unable to load schedules', resp);
      });
    }

    loadPackageTags() {
      this.ApiClientService.packages.listAllTags().success(resp => {
        this.packageTags = resp.tags;
        this.logLoaded('tags');
      }).error(resp => {
        console.error('Unable to load package tags', resp);
      });
    }

    loadCustomers() {
      this.ApiClientService.customers.list().success(resp => {
        this.customers = resp.customers;
        this.logLoaded('customers');
      }).error(resp => {
        console.error('Unable to load customers', resp);
      });
    }

    submit() {
      if (this.package.topup && this.mode === 'create') {
        this.package.assigned_event_ids = this.getSelectedKeys(this.selectedEvents);
        this.package.selectedTrainer = this.selectedtrainer;
        this.package.credit_amount = this.packageCredits;
        this.package.price = this.price * 100;

        this.ApiClientService.packages.createTopUpPackage(this.package).success(() => {
          this.ToastService.show('Package Created');
          this.NavigateService.go('app.trainer_packages');
          this.NavigateService.clearCache();
          this.NavigateService.clearHistory();
          this.NavigateService.removeCache(['app.trainer_create_package', 'app.trainer_edit_package']);
        }).error(resp => {
          this.fieldErrors = resp.error.fieldErrors;
        }).finally(() => {
          this.loading = false;
        });
      } else {
        if (this.package.visibility) {
          this.package.visibility = 'public';
        } else {
          this.package.visibility = 'private';
        }
        this.loading = true;
        const selectedSchedules = this.getSelectedKeys(this.selectedSchedules);
        const assignedSchedules = [];
        selectedSchedules.map(scheduleId => {
          const sId = parseInt(scheduleId, 10);
          const sIdx = this.schedules.findIndex(s => sId === s.id);

          let startDate = null;
          let unassignEntities = null;
          let canChangeStartDate = null;

          if (typeof this.scheduleDetails[sId] !== 'undefined') {
            if (this.scheduleDetails[sId].hasOwnProperty('customer_can_change_start_date')) {
              canChangeStartDate = this.scheduleDetails[sId].customer_can_change_start_date;
            }
            if (this.scheduleDetails[sId].hasOwnProperty('unassign_entities_on_end')) {
              unassignEntities = this.scheduleDetails[sId].unassign_entities_on_end;
            }
            if (this.scheduleDetails[sId].hasOwnProperty('start_date')) {
              if (this.scheduleDetails[sId].start_date !== null) {
                startDate = moment.utc(this.scheduleDetails[sId].start_date).format('YYYY-MM-DD HH:mm:ss');
              }
            }
          }

          assignedSchedules.push({
            schedule_id: sId,
            start_date: startDate,
            unassign_entities_on_end: unassignEntities,
            customer_can_change_start_date: canChangeStartDate,
            title: this.schedules[sIdx].name
          });

          return true;
        });
        this.package.assigned_schedules = assignedSchedules;
        this.package.assigned_workout_ids = this.getSelectedKeys(this.selectedWorkouts);
        this.package.files = this.getSelectedKeys(this.selectedFiles);
        this.package.assigned_nutrition_plan_ids = this.getSelectedKeys(this.selectedNutritionPlans);
        this.package.assigned_event_ids = this.getSelectedKeys(this.selectedEvents);
        this.package.selectedTrainer = this.selectedtrainer;
        this.package.credit_amount = this.packageCredits;
        this.package.price = this.price * 100;

        if (this.mode === 'create') {
          this.ApiClientService.packages.create(this.package).success(() => {
            this.ToastService.show('Package Created');
            this.NavigateService.go('app.trainer_packages');
            this.NavigateService.clearCache();
            this.NavigateService.clearHistory();
            this.NavigateService.removeCache(['app.trainer_create_package', 'app.trainer_edit_package']);
          }).error(resp => {
            this.fieldErrors = resp.error.fieldErrors;
          }).finally(() => {
            this.loading = false;
          });
        } else {
          this.ApiClientService.packages.update(this.package.id, this.package).success(() => {
            this.ToastService.show('Package Updated');
            this.NavigateService.go('app.trainer_packages');
            this.NavigateService.clearCache();
            this.NavigateService.clearHistory();
            this.NavigateService.removeCache(['app.trainer_create_package', 'app.trainer_edit_package']);
          });
        }
      }
    }

    getSelectedKeys(checkboxValues) {
      return Object.keys(checkboxValues).filter(i => checkboxValues[i] === true);
    }

    slideChanged(index) {
      this.slideIndex = index;
    }

    nextSlide() {
      if (this.slideIndex === 7) {
        this.NavigateService.go('app.trainer_packages');
        this.NavigateService.clearCache();
        this.NavigateService.clearHistory();
        this.NavigateService.removeCache(['app.trainer_create_package', 'app.trainer_edit_package']);
      } else {
        this.$ionicSlideBoxDelegate.next();
      }
    }

    previousSlide() {
      return this.NavigateService.go('app.trainer_packages');
    }
  }

  const config = $stateProvider => {
    $stateProvider.state('app.trainer_create_package', {
      url: '/packages/create/:topUp?',
      views: {
        menuContent: {
          templateUrl: 'trainer/packages/views/create.html',
          controller: 'TrainerPackageCreateAndEditController',
          controllerAs: 'vm'
        }
      }
    }).state('app.trainer_edit_package', {
      url: '/packages/{packageId:[0-9]+}/edit',
      views: {
        menuContent: {
          templateUrl: 'trainer/packages/views/create.html',
          controller: 'TrainerPackageCreateAndEditController',
          controllerAs: 'vm'
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('TrainerPackageCreateAndEditController', TrainerPackageCreateAndEditController);
})();
(() => {
  class TrainerPackageViewController {
    constructor(ApiClientService, $stateParams, ToastService, NavigateService, mpthconf, $state) {
      ApiClientService.packages.get(parseInt($stateParams.packageId, 10)).success(resp => {
        this.selectedPackage = resp.package;
        this.fixedPrice = (resp.package.price / 100).toFixed(2);
        this.subscribers = this.getSubscribers();
        this.imagePath = `${mpthconf.cdn.base}users/l/`;
        this.state = $state;
        this.NavigateService = NavigateService;
      }).error(resp => {
        console.error('Unable to load package', resp, $stateParams.packageId);
        ToastService.show('Unable to load package');
        NavigateService.go('app.trainer_packages');
      });
    }

    $onInit() {
      this.NavigateService.removeCache(['app.trainer_packages']);
    }

    getSubscribers() {
      return this.selectedPackage.active_purchases.map(purchase => purchase.customer.user);
    }
  }

  const config = $stateProvider => {
    $stateProvider.state('app.trainer_package_view', {
      url: '/packages/{packageId:[0-9]+}',
      views: {
        menuContent: {
          templateUrl: 'trainer/packages/views/view.html',
          controller: 'TrainerPackageViewController',
          controllerAs: 'vm'
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('TrainerPackageViewController', TrainerPackageViewController);
})();
(() => {
  class PackagesVoucherCreateController {
    constructor(ApiClientService, TrainerPackageCouponService, AuthenticationStateService, ToastService, $state) {
      this.ApiClientService = ApiClientService;
      this.AuthenticationStateService = AuthenticationStateService;
      this.TrainerPackageCouponService = TrainerPackageCouponService;
      this.ToastService = ToastService;
      this.$state = $state;
      const currentUser = AuthenticationStateService.getCurrentUser();
      this.currency = currentUser.merchant_currency_symbol;
      this.processing = false;
      this.coupon = {
        amount_type: 'percentage',
        duration: 'once'
      };

      const vms = this;
      this.TrainerPackageCouponService.load(vms);

      this.TrainerPackageCouponService.setup(this.coupon);

      this.title = 'Create Voucher';
      this.ready = true;
    }

    createVoucher() {
      this.processing = true;
      this.coupon.expires_at = this.TrainerPackageCouponService.getExpiryDate();
      this.coupon.amount_type = this.TrainerPackageCouponService.getAmountType();
      this.coupon.amount = this.TrainerPackageCouponService.getAmount();

      const save = this.ApiClientService.packages.createCoupon(this.coupon);

      save.success(() => {
        this.processing = false;
        this.ToastService.show('Voucher created successfully');
        this.$state.go('app.trainer_packages');
      }).error(resp => {
        this.processing = false;
        this.ToastService.show('Unable to create voucher');
        console.error('Unable to create voucher', resp);
      });
    }
  }

  const config = $stateProvider => {
    $stateProvider.state('app.trainer_create_coupon', {
      url: '/packages/create-voucher',
      views: {
        menuContent: {
          templateUrl: 'trainer/packages/views/createVoucher.html',
          controller: 'PackagesVoucherCreateController',
          controllerAs: 'vm'
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('PackagesVoucherCreateController', PackagesVoucherCreateController);
})();
(() => {
  class PackagesVoucherUpdateController {
    constructor(ApiClientService, TrainerPackageCouponService, AuthenticationStateService, ToastService, $state, $stateParams) {
      this.ApiClientService = ApiClientService;
      this.AuthenticationStateService = AuthenticationStateService;
      this.TrainerPackageCouponService = TrainerPackageCouponService;
      this.ToastService = ToastService;
      this.$state = $state;
      const currentUser = AuthenticationStateService.getCurrentUser();
      this.currency = currentUser.merchant_currency_symbol;
      this.processing = false;
      this.coupon = {
        amount_type: 'percentage',
        duration: 'once'
      };

      const vms = this;
      this.TrainerPackageCouponService.load(vms);

      ApiClientService.packages.getCoupon($stateParams.couponId).success(response => {
        this.coupon = response.coupon;
        this.TrainerPackageCouponService.setup(this.coupon);
        this.ready = true;
      });

      this.title = 'Update Voucher';
      this.updating = true;
    }

    createVoucher() {
      this.processing = true;
      this.coupon.expires_at = this.TrainerPackageCouponService.getExpiryDate();
      this.coupon.amount_type = this.TrainerPackageCouponService.getAmountType();
      this.coupon.amount = this.TrainerPackageCouponService.getAmount();

      const save = this.ApiClientService.packages.updateCoupon(this.coupon.id, this.coupon);

      save.success(() => {
        this.processing = false;
        this.ToastService.show('Voucher Updated successfully');
        this.$state.go('app.trainer_packages');
      }).error(resp => {
        this.processing = false;
        this.ToastService.show('Unable to Update voucher');
        console.error('Unable to update voucher', resp);
      });
    }
  }

  const config = $stateProvider => {
    $stateProvider.state('app.trainer_update_coupon', {
      url: '/packages/voucher/:couponId',
      views: {
        menuContent: {
          templateUrl: 'trainer/packages/views/createVoucher.html',
          controller: 'PackagesVoucherUpdateController',
          controllerAs: 'vm'
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('PackagesVoucherUpdateController', PackagesVoucherUpdateController);
})();
(function () {
  'use strict';

  angular.module('MyPTHub').controller('ScheduleTemplateAssignCustomerModalController', controller);

  controller.$inject = ['ApiClientService', 'PermissionsService', 'parameters', '$q', '$filter'];

  function controller(ApiClientService, PermissionsService, parameters, $q, $filter) {
    var vm = this;
    vm.loading = true;
    vm.formData = {};
    vm.customers = [];
    var currentCustomerIds = [];
    var currentCustomerGroupIds = [];

    // Defaults
    vm.formData.start_date = null;
    vm.formData.customer_can_change_start_date = false;

    // Current IDs
    parameters.assignedEntities.map(function (assignedEntity) {
      switch (assignedEntity.entity_type) {
        case 'group':
          currentCustomerGroupIds.push(assignedEntity.data.id);
          break;
        case 'assigned_template':
          currentCustomerIds.push(assignedEntity.data.customer.id);
          break;
      }
    });

    // Load Customers
    var getCustomers = ApiClientService.customers.list(PermissionsService.getLoggedInUserId());

    // Load Customer Groups
    var getCustomerGroups = ApiClientService.customerGroups.list(PermissionsService.getLoggedInUserId());

    $q.all([getCustomers, getCustomerGroups]).then(function (responses) {

      vm.loading = false;

      // Customers

      // Filter out customers already assigned
      var customers = $filter('filter')(responses[0].customers, function (customer) {
        return currentCustomerIds.indexOf(customer.id) === -1;
      }).map(function (customer) {
        return {
          title: customer.user.first_name + ' ' + customer.user.last_name,
          entity_type: 'assigned_template',
          customer: customer
        };
      });

      // Customer Groups

      // Filter out customers already assigned
      var customerGroups = $filter('filter')(responses[1].groups, function (group) {
        return currentCustomerGroupIds.indexOf(group.id) === -1;
      }).map(function (group) {
        group.entity_type = 'group';
        return group;
      });

      vm.assignableEntities = [];
      if (customerGroups.length > 0) {
        vm.assignableEntities.push({
          divider: true,
          title: 'Client Groups'
        });
        vm.assignableEntities.push.apply(vm.assignableEntities, customerGroups);
      }

      if (customerGroups.length > 0 && customers.length > 0) {
        vm.assignableEntities.push({
          divider: true,
          title: 'Clients'
        });
      }
      vm.assignableEntities.push.apply(vm.assignableEntities, customers);

      vm.noCustomers = vm.assignableEntities.length === 0;
    }).finally(function () {
      vm.loading = false;
    });

    vm.close = function () {
      vm.closeModal(false);
    };

    vm.assign = function () {
      var newAssignment = {};
      newAssignment.data = vm.formData.entity;
      newAssignment.title = vm.formData.entity.title;
      newAssignment.entity_type = vm.formData.entity.entity_type;
      newAssignment.start_date = vm.formData.start_date ? moment(vm.formData.start_date).format('YYYY-MM-DD HH:mm:ss') : null;
      newAssignment.customer_can_change_start_date = vm.formData.customer_can_change_start_date;

      if (vm.formData.start_date) {
        var startDate = moment(vm.formData.start_date).format('Do MMM YYYY');
        newAssignment.description = 'Starts on ' + startDate;
      } else {
        newAssignment.description = 'No start date specified';
      }

      vm.closeModal(newAssignment);
    };
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('ScheduleTemplateCreateModalController', controller);

    controller.$inject = ['parameters', 'ApiClientService', 'ToastService', '$scope', 'TrainerScheduleTemplatesService'];

    function controller(parameters, ApiClientService, ToastService, $scope, TrainerScheduleTemplatesService) {
        $scope.newTemplateData = {};
        $scope.newTemplateData.name = '';
        $scope.processing = false;

        if (parameters.trainerUserId) {
            $scope.newTemplateData.trainer_user_id = parameters.trainerUserId;
        }

        $scope.close = function () {
            $scope.closeModal(false);
        };

        $scope.save = function () {
            $scope.processing = true;
            var doSave = ApiClientService.scheduler.scheduleTemplateCreate($scope.newTemplateData);
            doSave.success(function (response) {
                ToastService.show('Nice! Schedule Saved Successfully');
                TrainerScheduleTemplatesService.dirty();
                $scope.processing = false;
                $scope.closeModal(response.template);
            });
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('ScheduleTemplateDuplicateDayModalService', service).controller('ScheduleTemplateDuplicateDayModalController', controller);

    service.$inject = ['ModalService'];

    function service(ModalService) {
        return {
            open: function (template, sourceDay, sourceWeek) {
                return ModalService.show({
                    templateUrl: "trainer/scheduler/views/scheduleTemplateDuplicateDayModal.html",
                    controller: 'ScheduleTemplateDuplicateDayModalController as vm',
                    params: {
                        template: template,
                        sourceDay: sourceDay,
                        sourceWeek: sourceWeek
                    }
                });
            }
        };
    }

    controller.$inject = ['parameters', 'AnalyticsService', '$scope', 'ToastService'];

    function controller(parameters, AnalyticsService, $scope, ToastService) {

        var vm = this;

        vm.template = parameters.template;
        var sourceDay = parameters.sourceDay;
        var sourceWeek = parameters.sourceWeek;

        var weekPosition = vm.template.weeks.indexOf(sourceWeek);
        var dayPosition = sourceWeek.days.indexOf(sourceDay);

        var originalPositionDesc = 'Day ' + (dayPosition + 1) + ' (Week ' + (weekPosition + 1) + ')';

        vm.title = 'Duplicate ' + originalPositionDesc;

        for (var i = 0, length = vm.template.weeks.length; i < length; i++) {
            var week = vm.template.weeks[i];
            week.description = 'Week ' + (i + 1);

            for (var di = 0, dlength = week.days.length; di < dlength; di++) {
                var day = week.days[di];
                day.description = 'Day ' + (di + 1);
            }
        }

        vm.weekChanged = function () {
            vm.chosenDay = null;
        };

        vm.duplicate = function () {
            var chosenWeekPosition = vm.template.weeks.indexOf(vm.chosenWeek);
            var chosenDayPosition = vm.chosenWeek.days.indexOf(vm.chosenDay);

            if (chosenWeekPosition === weekPosition && chosenDayPosition === dayPosition) {
                ToastService.show('Cannot duplicate a day onto itself');
                return false;
            }

            vm.template.weeks[chosenWeekPosition].days[chosenDayPosition] = angular.copy(sourceDay);

            var newPositionDesc = vm.chosenDay.description + ' (' + vm.chosenWeek.description + ')';
            ToastService.show(originalPositionDesc + ' was successfully duplicated to ' + newPositionDesc);

            $scope.closeModal(true);
        };

        AnalyticsService.trackView('Trainer - Duplicate Schedule Day');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('ScheduleTemplateDuplicateWeekService', service);

    service.$inject = ['ApiClientService', 'ToastService'];

    function service(ApiClientService, ToastService) {
        return {
            duplicateToTemplate: function (weekToDuplicate, destinationTemplate) {

                var getDestination = ApiClientService.scheduler.scheduleTemplateGet(destinationTemplate.id);

                getDestination.then(function (response) {

                    var destinationTemplateFull = response.template;

                    var newTemplateData = {
                        id: destinationTemplate.id
                    };

                    var weekOrder = 0;
                    newTemplateData.weeks = [];

                    // Build the existing weeks
                    angular.forEach(destinationTemplateFull.weeks, function (week) {
                        var days = [];
                        angular.forEach(week.days, function (day) {
                            days.push({
                                nutrition: day.nutrition && day.nutrition.id ? day.nutrition.id : null,
                                rest_day: day.rest_day ? true : false,
                                update_photo: day.update_photo ? true : false,
                                update_metrics: day.update_metrics ? true : false,
                                workout_am: day.workout_am && day.workout_am.id ? day.workout_am.id : null,
                                workout_pm: day.workout_pm && day.workout_pm.id ? day.workout_pm.id : null
                            });
                        });

                        newTemplateData.weeks.push({
                            order: weekOrder++,
                            days: days
                        });
                    });

                    // Add the week to the end
                    var days = [];
                    var daysAfterLastDay = 0;
                    angular.forEach(weekToDuplicate.days, function (day) {
                        days.push({
                            nutrition: day.nutrition && day.nutrition.id ? day.nutrition.id : null,
                            rest_day: day.rest_day ? true : false,
                            update_photo: day.update_photo ? true : false,
                            update_metrics: day.update_metrics ? true : false,
                            workout_am: day.workout_am && day.workout_am.id ? day.workout_am.id : null,
                            workout_pm: day.workout_pm && day.workout_pm.id ? day.workout_pm.id : null
                        });
                        if (day.past_last_day) {
                            daysAfterLastDay++;
                        }
                    });

                    newTemplateData.weeks.push({
                        order: weekOrder++,
                        days: days
                    });

                    newTemplateData.day_count = newTemplateData.weeks.length * 7 - daysAfterLastDay;

                    // Save the data
                    ApiClientService.scheduler.scheduleTemplateUpdate(destinationTemplate.id, newTemplateData).then(function (response) {
                        ToastService.show('Nice! Week successfully duplicated to ' + destinationTemplateFull.name);
                    }, function (response) {
                        ToastService.show(response.error.message);
                    });
                });
            }
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('TrainerScheduleTemplateManageController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.trainer_scheduler_update_schedule', {
            url: "/calendar/schedule/:templateId",
            views: {
                'menuContent': {
                    templateUrl: "trainer/scheduler/views/templateManage.html",
                    controller: 'TrainerScheduleTemplateManageController as vm'
                }
            },
            params: {
                goBackOnSave: false,
                goToWeek: false
            }
        });
    }

    controller.$inject = ['$scope', 'ApiClientService', 'PermissionsService', 'ToastService', 'ModalService', 'NavigateService', '$stateParams', '$ionicActionSheet', '$timeout', 'ConfirmDialogService', '$rootScope', 'EntityFinderModalService', '$q', 'ScheduleTemplateDuplicateWeekService', 'ScheduleTemplateWeekReorderModalService', 'TrainerScheduleTemplatesService', 'ScheduleTemplateDuplicateDayModalService', 'ScheduleEventStoreService'];

    function controller($scope, ApiClientService, PermissionsService, ToastService, ModalService, NavigateService, $stateParams, $ionicActionSheet, $timeout, ConfirmDialogService, $rootScope, EntityFinderModalService, $q, ScheduleTemplateDuplicateWeekService, ScheduleTemplateWeekReorderModalService, TrainerScheduleTemplatesService, ScheduleTemplateDuplicateDayModalService, ScheduleEventStoreService) {
        var vm = this;

        var cleantemplate;
        var skipUnsavedChangesCheck = false;
        vm.currentSlide = 0;
        vm.backText = 'Back';

        var defaultAmHour = '07';
        var defaultAmMinute = '00';
        var defaultPmHour = '19';
        var defaultPmMinute = '00';

        vm.slideOptions = {
            loop: false,
            effect: 'slide',
            speed: 500,
            pagination: '.week-pagination',
            paginationClickable: true
        };

        var dayTemplate = {
            nutrition: null,
            rest_day: false,
            workout_am: null,
            workout_pm: null
        };

        var weekTemplate = {
            schedule_id: null,
            order: null,
            days: [angular.copy(dayTemplate), angular.copy(dayTemplate), angular.copy(dayTemplate), angular.copy(dayTemplate), angular.copy(dayTemplate), angular.copy(dayTemplate), angular.copy(dayTemplate)]
        };

        var checkLastDay = function () {
            if (isContentAfterLastDay()) {
                resetDays();
            }

            recountDays();
        };

        var recountDays = function () {
            var lastDayReached = false;
            var dayCount = 0;
            angular.forEach(vm.template.weeks, function (week) {
                angular.forEach(week.days, function (day) {
                    if (!lastDayReached) {
                        dayCount++;

                        day.count = dayCount;

                        if (day.last_day) {
                            lastDayReached = true;
                        }
                    }
                });
            });

            vm.template.day_count = dayCount;
        };

        var resetDays = function (keepLastEndDate) {
            keepLastEndDate = keepLastEndDate || false;
            var lastDay = null;
            var dayCounter = 0;
            angular.forEach(vm.template.weeks, function (week) {
                angular.forEach(week.days, function (day) {
                    dayCounter++;
                    if (keepLastEndDate) {
                        if (day.last_day) {
                            lastDay = dayCounter;
                        }
                        day.past_last_day = lastDay && dayCounter > lastDay;
                    } else {
                        day.past_last_day = false;
                        day.last_day = false;
                    }
                });
            });
            recountDays();
        };

        var setDayCount = function (count) {
            var lastDayReached = false;
            var dayCounter = 0;

            angular.forEach(vm.template.weeks, function (week) {
                angular.forEach(week.days, function (day) {
                    dayCounter++;
                    day.last_day = false;

                    if (dayCounter == count) {
                        day.last_day = true;
                        lastDayReached = true;
                    } else if (lastDayReached) {
                        day.past_last_day = true;
                    }
                });
            });

            recountDays();
        };

        var isContentAfterLastDay = function () {
            var isContent = false;
            var lastDayReached = false;
            angular.forEach(vm.template.weeks, function (week) {
                angular.forEach(week.days, function (day) {
                    if (lastDayReached) {
                        if (day.workout_am || day.workout_pm || day.nutrition || day.update_photo || day.update_metrics || day.rest_day) {
                            isContent = true;
                        }
                    } else if (day.last_day) {
                        lastDayReached = true;
                    }
                });
            });

            return isContent;
        };

        var goToWeek = function (whichWeek) {
            $timeout(function () {
                vm.slider.slideTo(whichWeek);
            }, 100);
        };

        $scope.$on('$ionicView.beforeEnter', function () {
            skipUnsavedChangesCheck = false;
            vm.currentUser = PermissionsService.getCurrentUser();
            vm.template = {};
            vm.saving = false;
            vm.loading = true;
            vm.context = '';
            vm.collapsed = false;
            vm.currentSlide = 0;

            var doGet = ApiClientService.scheduler.scheduleTemplateGet($stateParams.templateId);

            doGet.success(function (response) {
                vm.template = response.template;
                vm.loading = false;

                var amTime = moment().hour(defaultAmHour).minute(defaultAmMinute);

                var pmTime = moment().hour(defaultPmHour).minute(defaultPmMinute);

                if (vm.template.preferred_am_hour) {
                    amTime.hour(vm.template.preferred_am_hour);
                }

                if (vm.template.preferred_am_minute) {
                    amTime.minute(vm.template.preferred_am_minute);
                }

                if (vm.template.preferred_pm_hour) {
                    pmTime.hour(vm.template.preferred_pm_hour);
                }

                if (vm.template.preferred_pm_minute) {
                    pmTime.minute(vm.template.preferred_pm_minute);
                }

                vm.recommendedMorningWorkoutTime = amTime.startOf('minute').toDate();
                vm.srecommendedAfternoonWorkoutTime = pmTime.startOf('minute').toDate();

                // Add week 1 by default
                if (!vm.template.weeks || vm.template.weeks && !vm.template.weeks.length) {
                    vm.addWeek(false, true);
                }

                // Only set last day if the day count is not equal to the end of the week (multiple of 7)
                if (vm.template.day_count % 7 !== 0) {
                    setDayCount(vm.template.day_count);
                } else {
                    recountDays();
                }

                vm.template.data = {
                    assigned_entities: []
                };

                vm.template.assigned_customer_groups.map(function (group) {

                    var description;

                    if (group.pivot.start_date) {
                        var startDate = moment(group.pivot.start_date).format('Do MMM YYYY');
                        description = 'Starts on ' + startDate;
                    } else {
                        description = 'No start date specified';
                    }

                    vm.template.data.assigned_entities.push({
                        data: group,
                        entity_type: 'group',
                        title: group.title,
                        description: description,
                        start_date: group.pivot.start_date ? group.pivot.start_date : null,
                        customer_can_change_start_date: group.pivot.customer_can_change_start_date
                    });

                    return group;
                });

                vm.template.assigned_templates.map(function (assigned) {

                    var description;

                    if (assigned.start_date) {
                        var startDate = moment.tz(assigned.start_date, assigned.customerTimezone).format('Do MMM YYYY');

                        description = 'Starts on ' + startDate;

                        var endDate = false;
                        if (assigned.started) {
                            endDate = moment.tz(assigned.end_date, assigned.customerTimezone).format('Do MMM YYYY');
                            description = startDate + ' - ' + endDate;
                        }
                    } else {
                        description = 'No start date specified';
                    }

                    vm.template.data.assigned_entities.push({
                        data: assigned,
                        entity_type: 'assigned_template',
                        title: assigned.customer.user.first_name + ' ' + assigned.customer.user.last_name,
                        description: description,
                        start_date: assigned.start_date ? assigned.start_date : null,
                        customer_can_change_start_date: assigned.customer_can_change_start_date
                    });

                    return assigned;
                });

                cleantemplate = angular.copy(vm.template);

                vm.slider.updateLoop();

                if (vm.template.template) {
                    vm.showHeaderSaveBtn = false;
                    vm.showHeaderDeleteBtn = true;
                    vm.editingAssignedSchedule = false;
                } else {
                    vm.showHeaderSaveBtn = true;
                    vm.showHeaderDeleteBtn = false;
                    vm.editingAssignedSchedule = true;
                }

                /*if ($stateParams.goToWeek) {
                    goToWeek($stateParams.goToWeek);
                }*/
            });

            doGet.error(function (response) {
                ToastService.show(response.error.message);
                vm.loading = false;
            });
        });

        vm.addWeek = function (week, forcePreserveLastDay) {
            var newWeek;

            forcePreserveLastDay = forcePreserveLastDay || false;

            if (!vm.template.weeks) {
                vm.template.weeks = [];
            }

            var isFirstWeek = vm.template.weeks.length === 0;

            if (week) {
                newWeek = angular.copy(week);
            } else {
                newWeek = angular.copy(weekTemplate);
            }

            if (isFirstWeek) {
                newWeek.days[0].update_photo = true;
                newWeek.days[0].update_metrics = true;
            }

            vm.template.weeks.push(newWeek);

            // Slide to new week
            vm.slider.updateLoop();

            goToWeek(vm.slider.slides.length);

            var preserveLastDay = true;
            if (week || forcePreserveLastDay) {
                preserveLastDay = false;
            }

            resetDays(preserveLastDay);
        };

        vm.triggerSaveTemplate = function (event) {
            event.preventDefault();

            $rootScope.$broadcast('makeSubmit', {
                formName: 'scheduleTemplateEditForm'
            });
        };

        vm.saveTemplate = function (event, form) {
            event.preventDefault();

            if (!form.$valid) {
                ToastService.show('Some fields were invalid, please check they are filled correctly');
                $timeout(function () {
                    vm.slider.slideTo(0);
                }, 100);
                return false;
            }

            vm.saving = true;

            var weekOrder = 0;
            var newTemplateData = {};
            if (vm.template.id) {
                newTemplateData.id = vm.template.id;
            }

            // Meta
            newTemplateData.day_count = vm.template.day_count;

            var amTime = moment(vm.recommendedMorningWorkoutTime);
            var pmTime = moment(vm.srecommendedAfternoonWorkoutTime);

            newTemplateData.preferred_am_hour = amTime.hour();
            newTemplateData.preferred_am_minute = amTime.minute();
            newTemplateData.preferred_pm_hour = pmTime.hour();
            newTemplateData.preferred_pm_minute = pmTime.minute();

            newTemplateData.name = vm.template.name ? vm.template.name : '';

            // Week data
            newTemplateData.weeks = [];
            angular.forEach(vm.template.weeks, function (week) {
                var days = [];
                angular.forEach(week.days, function (day) {
                    days.push({
                        nutrition: day.nutrition && day.nutrition.id ? day.nutrition.id : null,
                        rest_day: day.rest_day ? true : false,
                        update_photo: day.update_photo ? true : false,
                        update_metrics: day.update_metrics ? true : false,
                        workout_am: day.workout_am && day.workout_am.id ? day.workout_am.id : null,
                        workout_pm: day.workout_pm && day.workout_pm.id ? day.workout_pm.id : null
                    });
                });

                newTemplateData.weeks.push({
                    order: weekOrder++,
                    days: days
                });
            });

            // Assigned Entities
            if (vm.template.template) {
                newTemplateData.assigned_customers = [];
                newTemplateData.assigned_customer_groups = [];

                vm.template.data.assigned_entities.map(function (assignedEntity) {
                    switch (assignedEntity.entity_type) {
                        case 'group':
                            newTemplateData.assigned_customer_groups.push({
                                customer_group_id: assignedEntity.data.id,
                                start_date: assignedEntity.start_date,
                                customer_can_change_start_date: assignedEntity.customer_can_change_start_date
                            });
                            break;
                        case 'assigned_template':
                            newTemplateData.assigned_customers.push({
                                customer_id: assignedEntity.data.customer.id,
                                trainer_recommended_start_date: assignedEntity.start_date,
                                customer_can_change_start_date: assignedEntity.customer_can_change_start_date
                            });
                            break;
                    }
                });
            }
            // Perform save
            var doSave = ApiClientService.scheduler.scheduleTemplateUpdate(vm.template.id, newTemplateData);

            doSave.success(function (response) {
                cleantemplate = angular.copy(vm.template);
                TrainerScheduleTemplatesService.dirty();
                ScheduleEventStoreService.setDirty();
                ToastService.show('Nice! Schedule Saved Successfully');

                if ($stateParams.goBackOnSave) {
                    NavigateService.goBack();
                } else {
                    NavigateService.go('app.trainer_scheduler');
                }

                vm.saving = false;
            });

            doSave.error(function (response) {
                ToastService.show(response.error.message);
                vm.saving = false;
            });
        };

        vm.dayAddAction = function (day, week) {

            var weekPosition = vm.template.weeks.indexOf(week),
                totalWeeks = vm.template.weeks.length,
                isLastWeek = totalWeeks === weekPosition + 1,
                weekNo = weekPosition + 1,
                dayPosition = week.days.indexOf(day),
                dayNo = dayPosition + 1;

            var buttons = [];

            if (!day.workout_am) {
                buttons.push({
                    text: 'AM Workout',
                    action: function () {
                        EntityFinderModalService.open(vm.template.trainer_user_id, 'workout', 'Workout', day.workout_am ? day.workout_am.id : false, 'AM Workout').then(function (useWorkout) {
                            day.workout_am = useWorkout;
                            day.rest_day = false;
                            checkLastDay();
                        });
                    }
                });
            }

            if (!day.workout_pm) {
                buttons.push({
                    text: 'PM Workout',
                    action: function () {
                        EntityFinderModalService.open(vm.template.trainer_user_id, 'workout', 'Workout', day.workout_pm ? day.workout_pm.id : false, 'PM Workout').then(function (useWorkout) {
                            day.workout_pm = useWorkout;
                            day.rest_day = false;
                            checkLastDay();
                        });
                    }
                });
            }

            if (!day.nutrition) {
                buttons.push({
                    text: 'Nutrition Plan',
                    action: function () {
                        EntityFinderModalService.open(vm.template.trainer_user_id, 'nutritionPlan', 'Nutrition Plan', day.nutrition ? day.nutrition.id : false).then(function (useNutritionPlan) {
                            day.nutrition = useNutritionPlan;
                            checkLastDay();
                        });
                    }
                });
            }

            if (!day.rest_day) {
                buttons.push({
                    text: 'Rest Day',
                    action: function () {
                        // Rest Day
                        day.rest_day = !day.rest_day;
                        day.workout_am = false;
                        day.workout_pm = false;
                        checkLastDay();
                    }
                });
            }

            if (!day.update_photo) {
                buttons.push({
                    text: 'Update Photo',
                    action: function () {
                        // Update Photo
                        day.rest_day = false;
                        day.update_photo = !day.update_photo;
                        checkLastDay();
                    }
                });
            }

            if (!day.update_metrics) {
                buttons.push({
                    text: 'Update Measurements',
                    action: function () {
                        // Update Measurements
                        day.rest_day = false;
                        day.update_metrics = !day.update_metrics;
                        checkLastDay();
                    }
                });
            }

            if (isLastWeek && !day.last_day) {
                buttons.push({
                    text: 'Make Last Day',
                    action: function () {
                        var lastDayReached = false;
                        angular.forEach(vm.template.weeks, function (thisWeek, thisWeekPosition) {
                            angular.forEach(thisWeek.days, function (thisWeekDay, thisDayPosition) {
                                thisWeekDay.last_day = false;
                                thisWeekDay.past_last_day = false;

                                if (lastDayReached) {
                                    thisWeekDay.past_last_day = true;
                                    thisWeekDay.workout_am = null;
                                    thisWeekDay.workout_pm = null;
                                    thisWeekDay.nutrition = null;
                                    thisWeekDay.update_photo = false;
                                    thisWeekDay.update_metrics = false;
                                    thisWeekDay.rest_day = false;
                                } else if (weekPosition == thisWeekPosition && dayPosition === thisDayPosition) {
                                    thisWeekDay.last_day = true;
                                    thisWeekDay.update_photo = true;
                                    thisWeekDay.update_metrics = true;
                                    lastDayReached = true;
                                }
                            });
                        });

                        recountDays();
                    }
                });
            }

            if (!day.past_last_day) {
                buttons.push({
                    text: 'Duplicate',
                    action: function () {
                        ScheduleTemplateDuplicateDayModalService.open(vm.template, day, week).then(function () {
                            checkLastDay();
                        });
                    }
                });
            }

            $ionicActionSheet.show({
                buttons: buttons,
                titleText: 'Add to Day ' + dayNo + ' (Week ' + weekNo + ')',
                cancelText: 'Cancel',
                cancel: function () {
                    // cancel
                },
                buttonClicked: function (index) {
                    if (typeof buttons[index] !== 'undefined' && typeof buttons[index].action === 'function') {
                        buttons[index].action(index);
                    }
                    return true;
                }
            });
        };

        /**
         * Examines a week, and checks if it has any data for the given type of
         * item. If it does then it it will request user confirmation before
         * resolving the returning promise, otherwise it resolves immediately.
         * @param week
         * @param type
         * @param name
         * @return promise
         */
        var confirmWeekOverwrite = function (week, type, name) {
            var hasData = 0;
            angular.forEach(week.days, function (day) {
                if (day[type]) {
                    hasData++;
                }
            });

            if (hasData === 0) {
                $q.when();
            }

            return ConfirmDialogService.open({
                title: "Confirm Overwrite",
                message: "This week already has one or more days with " + name + "s assigned, are you sure you want to overwite these?",
                buttonConfirm: "Yes, Overwrite",
                buttonCancel: "Cancel"
            });
        };

        vm.weekAssignAction = function (week) {

            var weekPosition = vm.template.weeks.indexOf(week),
                weekNo = weekPosition + 1;

            var buttons = [];
            buttons.push({
                text: 'AM Workout',
                action: function () {
                    EntityFinderModalService.open(vm.template.trainer_user_id, 'workout', 'Workout', false, 'AM Workout').then(function (useWorkout) {
                        confirmWeekOverwrite(week, 'workout_am', 'AM Workout').then(function () {
                            angular.forEach(week.days, function (day) {
                                day.workout_am = useWorkout;
                                day.rest_day = false;
                            });
                            checkLastDay();
                        });
                    });
                }
            });

            buttons.push({
                text: 'PM Workout',
                action: function () {
                    EntityFinderModalService.open(vm.template.trainer_user_id, 'workout', 'Workout', false, 'PM Workout').then(function (useWorkout) {
                        confirmWeekOverwrite(week, 'workout_pm', 'PM Workout').then(function () {
                            angular.forEach(week.days, function (day) {
                                day.workout_pm = useWorkout;
                                day.rest_day = false;
                            });
                            checkLastDay();
                        });
                    });
                }
            });

            buttons.push({
                text: 'Nutrition Plan',
                action: function () {
                    EntityFinderModalService.open(vm.template.trainer_user_id, 'nutritionPlan', 'Nutrition Plan', false).then(function (useNutritionPlan) {
                        confirmWeekOverwrite(week, 'nutrition', 'Nutrition Plan').then(function () {
                            angular.forEach(week.days, function (day) {
                                day.nutrition = useNutritionPlan;
                            });
                            checkLastDay();
                        });
                    });
                }
            });

            buttons.push({
                text: 'Rest Day',
                action: function () {
                    angular.forEach(week.days, function (day) {
                        day.rest_day = true;
                    });
                    checkLastDay();
                }
            });

            buttons.push({
                text: 'Update Photo',
                action: function () {
                    angular.forEach(week.days, function (day) {
                        day.update_photo = true;
                    });
                    checkLastDay();
                }
            });

            buttons.push({
                text: 'Update Measurements',
                action: function () {
                    angular.forEach(week.days, function (day) {
                        day.update_metrics = true;
                    });
                    checkLastDay();
                }
            });

            $ionicActionSheet.show({
                buttons: buttons,
                titleText: 'Assign to every day in Week ' + weekNo,
                cancelText: 'Cancel',
                cancel: function () {
                    // cancel
                },
                buttonClicked: function (index) {
                    if (typeof buttons[index] !== 'undefined' && typeof buttons[index].action === 'function') {
                        buttons[index].action(index);
                    }
                    return true;
                }
            });
        };

        vm.weekDuplicateAction = function (week) {

            var weekPosition = vm.template.weeks.indexOf(week),
                weekNo = weekPosition + 1;

            var buttons = [];

            buttons.push({
                text: 'Duplicate in this template',
                action: function () {
                    vm.addWeek(week);
                }
            });

            buttons.push({
                text: 'Duplicate to another template',
                action: function () {
                    EntityFinderModalService.open(vm.template.trainer_user_id, 'schedule', 'Schedule', vm.template.id, 'Duplicate To Schedule').then(function (destinationSchedule) {
                        ScheduleTemplateDuplicateWeekService.duplicateToTemplate(week, destinationSchedule);
                    });
                }
            });

            $ionicActionSheet.show({
                buttons: buttons,
                titleText: 'Duplicate Week ' + weekNo,
                cancelText: 'Cancel',
                cancel: function () {
                    // cancel
                },
                buttonClicked: function (index) {
                    if (typeof buttons[index] !== 'undefined' && typeof buttons[index].action === 'function') {
                        buttons[index].action(index);
                    }
                    return true;
                }
            });
        };

        vm.editDayAssignment = function (day, field) {
            var current = day[field],
                entityType,
                entityTypeName,
                currentEntityId,
                modalTitle;

            switch (field) {
                case 'workout_am':
                    entityType = 'workout';
                    entityTypeName = 'AM Workout';
                    modalTitle = 'Change AM Workout';
                    break;

                case 'workout_pm':
                    entityType = 'workout';
                    entityTypeName = 'PM Workout';
                    modalTitle = 'Change PM Workout';
                    break;

                case 'nutrition':
                    entityType = 'nutritionPlan';
                    entityTypeName = 'Nutrition Plan';
                    modalTitle = 'Change Nutrition Plan';
                    break;
            }

            if (current) {
                currentEntityId = current.id;
            }

            if (entityType) {
                EntityFinderModalService.open(vm.template.trainer_user_id, entityType, entityTypeName, currentEntityId, modalTitle).then(function (entity) {
                    day[field] = entity;
                });
            }
        };

        vm.weekMenu = function (week) {
            var weekPosition = vm.template.weeks.indexOf(week);
            if (weekPosition === -1) {
                return false;
            }

            var buttons = [];

            buttons.push({
                text: 'Assign Entire Week',
                action: function () {
                    vm.weekAssignAction(week);
                }
            });

            buttons.push({
                text: 'Duplicate',
                action: function () {
                    vm.weekDuplicateAction(week);
                }
            });

            if (vm.template.weeks.length > 1) {
                buttons.push({
                    text: 'Change Week Order',
                    action: function () {
                        ScheduleTemplateWeekReorderModalService.open(vm.template);
                    }
                });
            }

            var hideSheet = $ionicActionSheet.show({
                buttons: buttons,
                titleText: 'Week ' + (weekPosition + 1),
                destructiveText: 'Delete',
                cancelText: 'Cancel',
                destructiveButtonClicked: function () {
                    ConfirmDialogService.open({
                        title: "Confirm Removal Of Week",
                        message: "Are you sure you want to remove this entire week?",
                        buttonConfirm: "Yes, Remove",
                        buttonCancel: "Cancel"
                    }).then(function () {
                        vm.template.weeks.splice(weekPosition, 1);
                        resetDays();
                        hideSheet();
                        vm.slider.updateLoop();
                    });
                },
                cancel: function () {
                    // cancel
                },
                buttonClicked: function (index) {
                    if (typeof buttons[index] !== 'undefined' && typeof buttons[index].action === 'function') {
                        buttons[index].action(index);
                    }
                    return true;
                }
            });
        };

        vm.openCustomerTemplateUnassignment = function (assignedEntity) {
            var buttons = [];

            if (assignedEntity.entity_type === 'group') {
                buttons.push({ text: 'Unassign Client Group' });
            } else {
                buttons.push({ text: 'Unassign Client' });
            }

            $ionicActionSheet.show({
                buttons: buttons,
                titleText: assignedEntity.title,
                cancelText: 'Cancel',
                cancel: function () {
                    // cancel
                },
                buttonClicked: function (index) {
                    switch (index) {
                        case 0:
                            // Unassign
                            vm.template.data.assigned_entities.splice(vm.template.data.assigned_entities.indexOf(assignedEntity), 1);
                            break;
                    }
                    return true;
                }
            });
        };

        vm.openCustomerAssignmentModal = function () {
            ModalService.show({
                templateUrl: "trainer/scheduler/views/customerAssignmentModal.html",
                controller: 'ScheduleTemplateAssignCustomerModalController as vm',
                params: {
                    assignedEntities: vm.template.data.assigned_entities
                }
            }).then(function (entity) {
                if (!entity) {
                    return false;
                }
                vm.template.data.assigned_entities.push(entity);
            });
        };

        vm.delete = function (event) {
            event.preventDefault();

            ConfirmDialogService.open({
                title: "Confirm Delete",
                message: "Are you sure you want to delete this schedule? This cannot be undone!",
                buttonConfirm: "OK, Delete Schedule",
                buttonCancel: "Cancel"
            }).then(function () {
                ScheduleEventStoreService.setDirty();
                ApiClientService.scheduler.scheduleTemplateDelete(vm.template.id).then(function (response) {
                    skipUnsavedChangesCheck = true;
                    ToastService.show('Schedule successfully deleted');
                    TrainerScheduleTemplatesService.dirty();
                    NavigateService.goBack();
                }, function (response) {
                    ToastService.show(response.error.message);
                });
            });
        };

        vm.goBack = function () {
            if (currentlySliding) {
                return false;
            }

            if (vm.currentSlide > 0) {
                $timeout(function () {
                    vm.slider.slideTo(vm.currentSlide - 1);
                }, 100);
            } else {
                NavigateService.goBack();
            }
        };

        var currentlySliding = false;
        $scope.$on("$ionicSlides.sliderInitialized", function (event, data) {
            vm.slider = data.slider;
            vm.currentSlide = data.slider.activeIndex;
        });

        $scope.$on("$ionicSlides.slideChangeStart", function (event, data) {
            currentlySliding = true;
        });

        $scope.$on("$ionicSlides.slideChangeEnd", function (event, data) {
            vm.currentSlide = data.slider.activeIndex;
            currentlySliding = false;
            $timeout(function () {
                if (vm.currentSlide > 1) {
                    vm.backText = 'Week ' + (vm.currentSlide - 1);
                } else if (vm.currentSlide === 1) {
                    vm.backText = 'Details';
                } else {
                    vm.backText = 'Back';
                }

                if (vm.template.template) {
                    vm.showHeaderSaveBtn = vm.currentSlide > 0;
                    vm.showHeaderDeleteBtn = vm.currentSlide === 0;
                } else {
                    vm.showHeaderSaveBtn = true;
                    vm.showHeaderDeleteBtn = false;
                }
            });
        });

        $scope.$on('$stateChangeStart', function (event, next) {
            var originalIsSame = angular.equals(cleantemplate, vm.template);

            if (originalIsSame || skipUnsavedChangesCheck) {
                return true;
            }

            event.preventDefault();

            ConfirmDialogService.open({
                title: "Unsaved Changes",
                message: "You have some unsaved changes. To discard your changes, click OK. To continue editing this schedule, click Cancel.",
                buttonConfirm: "OK, Discard Changes",
                buttonCancel: "Cancel"
            }).then(function () {
                cleantemplate = angular.copy(vm.template);

                if (next.name === 'app.trainer_scheduler') {
                    NavigateService.goBack();
                } else {
                    NavigateService.go(next);
                }
            }, function () {
                console.log('cancelled navigation due to user cancel on confirm dialog');
            });
        });
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('ScheduleTemplateWeekReorderModalService', service).controller('ScheduleTemplateWeekReorderModalController', controller);

    service.$inject = ['ModalService'];

    function service(ModalService) {
        return {
            open: function (template) {
                return ModalService.show({
                    templateUrl: "trainer/scheduler/views/scheduleTemplateWeekReorderModal.html",
                    controller: 'ScheduleTemplateWeekReorderModalController as vm',
                    params: {
                        template: template
                    }
                });
            }
        };
    }

    controller.$inject = ['parameters', 'AnalyticsService'];

    function controller(parameters, AnalyticsService) {

        var vm = this;

        vm.template = parameters.template;

        for (var i = 0, length = vm.template.weeks.length; i < length; i++) {
            vm.template.weeks[i].originalPosition = i + 1;
        }

        vm.moveWeek = function (week, fromIndex, toIndex) {
            vm.template.weeks.splice(fromIndex, 1);
            vm.template.weeks.splice(toIndex, 0, week);

            for (var i = 0, length = vm.template.weeks.length; i < length; i++) {
                vm.template.weeks[i].order = i;
            }
        };

        AnalyticsService.trackView('Trainer - Reorder Schedule Weeks');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).factory('TrainerScheduleTemplatesService', service).controller('TrainerScheduleTemplatesController', controller);

    function service() {

        var dirty = false;

        return {
            isDirty: function () {
                return dirty;
            },
            dirty: function () {
                dirty = true;
            },
            reset: function () {
                dirty = false;
            }
        };
    }

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.trainer_scheduler', {
            url: "/calendar/schedule",
            views: {
                'menuContent': {
                    templateUrl: "trainer/scheduler/views/templateIndex.html",
                    controller: 'TrainerScheduleTemplatesController as vm'
                }
            }
        });
    }

    controller.$inject = ['$scope', 'ApiClientService', 'PermissionsService', 'ToastService', '$filter', 'ModalService', 'NavigateService', 'TrainerScheduleTemplatesService', 'mpthconf', '$ionicLoading'];

    function controller($scope, ApiClientService, PermissionsService, ToastService, $filter, ModalService, NavigateService, TrainerScheduleTemplatesService, mpthconf, $ionicLoading) {
        var vm = this;
        vm.cdnBase = mpthconf.cdn.base;

        var lang = [];
        lang['ERROR_LOAD_TEMPLATES'] = 'There was an error loading your templates';
        lang['ERROR_SAVE_TEMPLATE'] = 'There was an error saving your template';
        lang['TITLE_REMOVE_TEMPLATE'] = 'Remove Template';
        lang['TITLE_REMOVE_TEMPLATE_MULTIPLE'] = 'Remove Selected Templates';
        lang['MSG_CONFIRM_DELETE'] = 'Are you sure you want to delete';
        lang['MSG_CONFIRM_DELETE_MULTIPLE'] = 'Are you sure you want to delete the selected templates?';
        lang['BTN_YES'] = 'Yes, Remove Template';
        lang['BTN_YES_MULTIPLE'] = 'Yes, Remove Templates';
        lang['BTN_NO'] = 'No, Cancel';
        lang['SUCCESS_TEMPLATE_DELETE'] = 'Template successfully deleted';

        vm.currentUser = PermissionsService.getCurrentUser();
        vm.templates = [];
        vm.templatesFiltered = [];
        vm.loading = true;
        vm.loaded = false;

        vm.schedulerEnabled = PermissionsService.schedulerIsEnabled();
        vm.currency = vm.currentUser.subscription.currency;
        vm.currencySymbol = vm.currentUser.subscription.currency_symbol;
        vm.currencyIsDollar = vm.currentUser.subscription.currency_is_dollar;

        $scope.$watch(function () {
            return vm.templateSearchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.templatesFiltered = $filter('filter')(vm.templates, {
                    name: newVal
                });
            } else {
                vm.templatesFiltered = vm.templates;
            }
        });

        var loadAll = function () {
            vm.loading = true;

            var templateIndex = ApiClientService.scheduler.scheduleTemplateIndexTrainerUser(vm.currentUser.id);

            templateIndex.success(function (response) {

                TrainerScheduleTemplatesService.reset();

                vm.loading = false;
                vm.loaded = true;

                if (response.templates.length === 0) {
                    vm.templates = [];
                    vm.templatesFiltered = [];
                } else {
                    vm.templates = response.templates;
                    vm.templatesFiltered = angular.copy(response.templates);
                }

                $scope.$broadcast('scroll.refreshComplete');
            });

            templateIndex.error(function (response) {
                vm.loading = false;
                vm.loaded = true;

                ToastService.show(lang['ERROR_LOAD_TEMPLATES']);
                $scope.$broadcast('scroll.refreshComplete');
            });
        };

        vm.allShow = function () {
            if (vm.schedulerEnabled) {
                loadAll();
            }
        };

        vm.plansSearchFilter = null;

        var loadLibrary = function () {
            vm.loading = true;

            ApiClientService.scheduleLibrary.listAll().then(function (response) {

                vm.libraryPlans = response.schedules;

                vm.loading = false;
            });
        };

        vm.findAPlan = function () {
            loadLibrary();
        };

        vm.useLibrarySchedule = function (schedule) {
            if (vm.processing) {
                return false;
            }

            vm.processing = true;

            $ionicLoading.show({
                template: 'Please wait...'
            }).then(function () {
                ApiClientService.scheduleLibrary.duplicate(schedule.id).then(function (response) {
                    ToastService.show('Schedule successfully added to your account');
                }).finally(function () {
                    vm.processing = false;
                    $ionicLoading.hide();
                });
            });
        };

        $scope.$on('$ionicView.beforeEnter', function () {
            if (vm.schedulerEnabled && TrainerScheduleTemplatesService.isDirty()) {
                loadAll();
            }
        });

        vm.goToEdit = function (template) {
            NavigateService.go('app.trainer_scheduler_update_schedule', {
                templateId: template.id
            });
        };

        vm.openCreateTemplateModal = function () {
            ModalService.show({
                templateUrl: "trainer/scheduler/views/createTemplateModal.html",
                controller: 'ScheduleTemplateCreateModalController as vm',
                params: {
                    trainerUserId: vm.currentUser.id
                }
            }).then(function (template) {
                if (template && template.id) {
                    NavigateService.go('app.trainer_scheduler_update_schedule', {
                        templateId: template.id
                    });
                }
            });
        };

        vm.viewSchedule = function (schedule) {
            NavigateService.go('app.trainer_schedule_preview', {
                scheduleId: schedule.id
            });
        };
    }
})();
(function () {
  angular.module('MyPTHub').config(config).controller('TrainerSettingsController', controller);

  config.$inject = ['$stateProvider'];

  function config($stateProvider) {
    $stateProvider.state('app.trainer_settings', {
      url: '/trainer_settings',
      views: {
        menuContent: {
          templateUrl: 'trainer/settings/settings.html',
          controller: 'TrainerSettingsController as vm'
        }
      }
    });
  }

  controller.$inject = ['AuthenticationHelperService', 'NavigateService', '$scope', 'AuthenticationStateService', 'ApiClientService', 'AvatarUploadService', 'AnalyticsService', 'mpthconf', 'ModalService', 'DeviceInfoService', 'PermissionsService', 'SubscriptionStateService', 'BrandingMenuService', 'ToastService', 'AuthenticationService'];

  function controller(AuthenticationHelperService, NavigateService, $scope, AuthenticationStateService, ApiClientService, AvatarUploadService, AnalyticsService, mpthconf, ModalService, DeviceInfoService, PermissionsService, SubscriptionStateService, BrandingMenuService, ToastService, AuthenticationService) {
    const vm = this;
    vm.isTrainerOwner = PermissionsService.userIsTrainerOwner();
    vm.hasUnlimitedPlus = PermissionsService.userHasUnlimitedPlus();

    // temp horrible hack to work around issue of disappearing tabs
    // in some weird circumstances.
    $scope.$on('$ionicView.afterEnter', () => {
      $(window).trigger('resize');
    });

    vm.setMode = function (mode) {
      vm.mode = mode;
    };

    vm.webcalUrl = '';
    vm.trainerUser = {};

    const setTrainerUser = function (data) {
      vm.trainerUser.first_name = data.first_name;
      vm.trainerUser.last_name = data.last_name;
      vm.trainerUser.address1 = data.address1;
      vm.trainerUser.address2 = data.address2;
      vm.trainerUser.town_city = data.town_city;
      vm.trainerUser.postcode = data.postcode;
      vm.trainerUser.country = data.country;
      vm.trainerUser.timezone = data.timezone;
      vm.trainerUser.min_cancellation = data.min_cancellation;
      vm.trainerUser.dob = data.dob;

      vm.webcalUrl = `${mpthconf[mpthconf.env].webcal.current + data.id}/ical?token=${data.cal_token}`;
    };

    vm.accountSwapper = () => {
      NavigateService.go('app.accounts');
    };

    vm.showAccountSwapper = () => AuthenticationStateService.getCurrentUser().accounts > 1;

    vm.periods = [{ mins: 60, name: '1 Hour' }, { mins: 180, name: '3 Hours' }, { mins: 360, name: '6 Hours' }, { mins: 720, name: '12 Hours' }, { mins: 1440, name: '24 Hours' }, { mins: 2880, name: '2 Days' }, { mins: 5760, name: '4 Days' }, { mins: 10080, name: '1 Week' }];

    vm.openReportIssueModal = function () {
      ModalService.show({
        templateUrl: 'common/bugReport/bugReportModal.html',
        controller: 'BugReportModalController as vm'
      });
    };

    const setDob = function (source) {
      vm.dob = moment(source, 'YYYY-MM-DD').toDate();
    };

    const parseDob = function (source) {
      return moment(source).format('YYYY-MM-DD');
    };

    vm.logout = AuthenticationHelperService.logout;

    vm.saveSettings = function () {
      vm.saving = true;

      if (vm.dob) {
        vm.trainerUser.dob = parseDob(vm.dob);
      }

      ApiClientService.trainerUsers.update(vm.user.id, vm.trainerUser).success(() => {
        NavigateService.goBack();
      }).finally(() => {
        vm.saving = false;
      });
    };

    vm.takePhoto = function () {
      vm.uploading = true;

      AvatarUploadService.takePhoto(vm.user.id).then(() => {
        vm.user = AuthenticationStateService.getCurrentUser();
        setTrainerUser(vm.user);
      }).finally(() => {
        vm.uploading = false;
      });
    };

    vm.browsePhoto = function () {
      vm.uploading = true;

      AvatarUploadService.browsePhoto(vm.user.id).then(() => {
        vm.user = AuthenticationStateService.getCurrentUser();
        setTrainerUser(vm.user);
      }).finally(() => {
        vm.uploading = false;
      });
    };

    vm.openBrandingMenu = function () {
      BrandingMenuService.start();
    };

    vm.upgrade = function () {
      NavigateService.go('app.trainer_upgrade');
    };

    DeviceInfoService.appVersion().then(appVer => {
      vm.appVer = appVer;
    });

    $scope.$on('$ionicView.beforeEnter', () => {
      vm.loaded = false;

      vm.user = AuthenticationStateService.getCurrentUser();

      setDob(vm.user.dob);

      setTrainerUser(vm.user);

      ApiClientService.trainerUsers.get(vm.user.id).success(response => {
        vm.user = response.currentUser;

        setDob(vm.user.dob);

        setTrainerUser(vm.user);

        vm.loaded = true;
      });

      vm.showIapLink = SubscriptionStateService.getPlatform() === 'apple_iap' && DeviceInfoService.isIos() && PermissionsService.userIsTrainerOwner();
      vm.canUpgrade = SubscriptionStateService.userCanUpgrade() && DeviceInfoService.isIos();
    });

    $scope.$on('$ionicView.enter', () => {
      AnalyticsService.trackView('Trainer - Settings');
    });

    // Permissions stuff.
    vm.checkStatus = value => {
      if (!value) {
        vm.allowAllPermissions = false;
      }
    };

    vm.cancellationValues = [{
      mins: 60,
      label: '1 Hour'
    }, {
      mins: 180,
      label: '3 Hours'
    }, {
      mins: 360,
      label: '6 Hours'
    }, {
      mins: 720,
      label: '12 Hours'
    }, {
      mins: 1440,
      label: '24 Hours'
    }, {
      mins: 2880,
      label: '2 Days'
    }, {
      mins: 5760,
      label: '4 Days'
    }, {
      mins: 10080,
      label: '1 Week'
    }];
    vm.requests = [];

    vm.currentUser = AuthenticationStateService.getCurrentUser();
    vm.allowAllPermissions = vm.currentUser.subscription.allow_personal_packages;

    vm.advance_booking_amount = vm.currentUser.events_advance_booking_amount;
    vm.advance_booking_value = vm.currentUser.events_advance_booking_value;
    vm.min_cancellation_period = vm.currentUser.events_min_cancellation_period;

    ApiClientService.trainerUsers.list().success(response => {
      vm.trainers = response.trainerUsers.filter(trainer => vm.currentUser.id !== trainer.id);
      vm.loading = false;
    });

    ApiClientService.permissions.getAll().success(response => {
      vm.permissions = response.permissions;
    });

    vm.setValue = (trainer, permission) => {
      if (vm.allowAllPermissions) {
        return permission.id;
      }
      return trainer.permission_ids.includes(permission.id) ? permission.id : 0;
    };

    $scope.$watch('vm.allowAllPermissions', (newValue, oldValue) => {
      if (newValue) {
        vm.requests.forEach(request => {
          vm.permissions.forEach(permission => {
            request.permissions[permission.id] = permission.id;
          });
        });
      }
    });

    vm.sendRequest = () => {
      if (vm.allowAllPermissions) {
        ApiClientService.permissions.updateGlobal({
          allowAllPermissions: vm.allowAllPermissions,
          min_cancellation_period: vm.min_cancellation_period,
          advance_booking_value: vm.advance_booking_value,
          advance_booking_amount: vm.advance_booking_amount
        }).success(() => {
          updateAccount();
          ToastService.show('Permissions updated');
        });
      }
      vm.requests.forEach(request => {
        request.min_cancellation_period = vm.min_cancellation_period;
        request.advance_booking_value = vm.advance_booking_value;
        request.advance_booking_amount = vm.advance_booking_amount;
        ApiClientService.permissions.create(request).success(response => {
          ToastService.show('Permissions updated');
          updateAccount();
        }).error(error => {
          ToastService('You are not allowed to change this');
        });
      });
    };

    const updateAccount = () => Promise.resolve().then(() => AuthenticationService.refreshUser()).then(() => AuthenticationStateService.getCurrentUser()).then(user => {
      AuthenticationService.updateCurrentUser(user);
      return AuthenticationStateService.getCurrentUser();
    });

    vm.mode = 'permissions';
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('TrainerTrainersController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.trainer_trainers', {
            url: "/trainers",
            views: {
                'menuContent': {
                    templateUrl: "trainer/trainers/trainers.html",
                    controller: 'TrainerTrainersController as vm'
                }
            }
        });
    }

    controller.$inject = ['ApiClientService', 'PermissionsService', '$scope', 'NavigateService', '$filter', 'AnalyticsService'];

    function controller(ApiClientService, PermissionsService, $scope, NavigateService, $filter, AnalyticsService) {

        var vm = this;

        vm.goToTrainer = function (trainerUser) {

            NavigateService.go('app.trainer_profile', {
                trainerUserId: trainerUser.id
            });
        };

        var loadTrainers = function (refreshing) {

            refreshing = refreshing || false;

            const refreshComplete = () => {
                if (refreshing) {
                    $scope.$broadcast('scroll.refreshComplete');
                }
            };

            try {
                ApiClientService.trainerUsers.list(PermissionsService.getLoggedInUserId()).success(function (response) {

                    var trainerUsers = response.trainerUsers;

                    // set full name on root object for search filter
                    trainerUsers.map(function (trainerUser) {
                        trainerUser.full_name = trainerUser.first_name + ' ' + trainerUser.last_name;
                    });

                    vm.trainerUsers = trainerUsers;

                    vm.filteredTrainerUsers = angular.copy(trainerUsers);

                    vm.noTrainerUsers = vm.trainerUsers.length === 0;
                }).finally(refreshComplete);
            } catch (e) {
                refreshComplete();
            }
        };

        vm.refresh = function () {
            loadTrainers(true);
        };

        // List Search filter
        $scope.$watch(function () {
            return vm.searchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.searchFilterActive = true;
                vm.filteredTrainerUsers = $filter('filter')(vm.trainerUsers, {
                    full_name: newVal
                });
            } else {
                vm.searchFilterActive = false;
                vm.filteredTrainerUsers = vm.trainerUsers;
            }
        });

        $scope.$on('$ionicView.beforeEnter', function () {

            loadTrainers();
        });

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Trainer - List Trainers');
        });
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('TrainerUpgradeController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.trainer_upgrade', {
            url: "/trainer_upgrade",
            cache: false,
            views: {
                'menuContent': {
                    templateUrl: "trainer/upgrade/upgrade.html",
                    controller: 'TrainerUpgradeController as vm'
                }
            },
            resolve: {
                canUpgrade: ['SubscriptionStateService', '$q', function (SubscriptionStateService, $q) {
                    if (SubscriptionStateService.userCanUpgrade()) {
                        return $q.when();
                    }

                    return $q.reject();
                }]
            }
        });
    }

    controller.$inject = ['$scope', 'AnalyticsService', 'InAppPurchaseService', 'SubscriptionStateService', 'ToastService', 'ApiClientService', '$ionicLoading', 'NavigateService', 'AuthenticationService', 'DeviceInfoService'];

    function controller($scope, AnalyticsService, InAppPurchaseService, SubscriptionStateService, ToastService, ApiClientService, $ionicLoading, NavigateService, AuthenticationService, DeviceInfoService) {

        var vm = this;

        vm.subscribe = function (product, event) {
            event.preventDefault();

            if (vm.subscribing) {
                return false;
            }

            vm.subscribing = product.productId;

            InAppPurchaseService.subscribe(product.productId).then(function (response) {
                $ionicLoading.show({
                    template: 'Please wait...'
                }).then(function () {
                    ApiClientService.inAppPurchase.verifyReceipt(response.receipt).then(function (response) {
                        vm.subscribing = false;

                        ToastService.show('Upgrade successful!');

                        $ionicLoading.hide();

                        NavigateService.disableBack().go('app.trainer_home');

                        AuthenticationService.updateCurrentUser(response.data.currentUser);

                        SubscriptionStateService.stateChanged();
                    }, function (response) {
                        ToastService.show(response.error ? response.error.message : 'Unable to verify - please contact support');

                        vm.subscribing = false;
                        $ionicLoading.hide();
                    });
                });
            }, function (error) {
                vm.subscribing = false;

                if (typeof error === 'object') {
                    ToastService.show(error.errorMessage);
                } else {
                    ToastService.show(error);
                }
            });
        };
        /*
                vm.products = [
                    {
                        description: "Full access to all features.",
                        price: "$11.49",
                        productId: "monthly",
                        title: "Monthly"
                    },
                    {
                        description: "Full access to all features.",
                        price: "$95.99",
                        productId: "yearly",
                        title: "Yearly"
                    }
                ];
        */
        const transformProducts = products => {
            products.map(product => {
                switch (product.title) {
                    case 'Monthly':
                        product.cycleDescription = 'per month';
                        break;
                    case 'Yearly':
                        product.cycleDescription = 'per year';
                        break;
                    default:
                        product.cycleDescription = '';

                }
            });
            return products;
        };

        vm.loading = true;

        DeviceInfoService.bundleId().then(function (bundleId) {
            InAppPurchaseService.getProducts([bundleId + '.upgrade.yearly', bundleId + '.upgrade.monthly']).then(function (products) {
                vm.products = transformProducts(products);
                vm.loading = false;
            });
        });

        $scope.$on('$ionicView.beforeEnter', function () {

            vm.onTrial = SubscriptionStateService.onFreeTrial();
            vm.trialRemainingString = null;

            if (vm.onTrial) {
                var trialEndDate = moment.utc(SubscriptionStateService.getTrialEnd(), 'YYYY-MM-DD HH:mm:ss');

                var now = moment().utc();

                var remaining = trialEndDate.diff(now);

                if (trialEndDate.diff(now, 'days') < 3) {
                    vm.trialRemainingString = 'soon';
                } else {
                    vm.trialRemainingString = moment.duration(remaining).humanize(true);
                }
            }
        });

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Trainer - Upgrade');
        });
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).factory('TrainerWorkoutsListService', service).controller('TrainerWorkoutsController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.trainer_workouts', {
            url: "/trainer_workouts/:trainerUserId?",
            views: {
                'menuContent': {
                    templateUrl: "trainer/workouts/workouts.html",
                    controller: 'TrainerWorkoutsController as vm'
                }
            }
        });
    }

    controller.$inject = ['$stateParams', '$scope', 'ApiClientService', '$filter', 'PermissionsService', 'NavigateService', 'PromptDialogService', 'CreateWorkoutService', 'TrainerWorkoutsListService', 'mpthconf', '$ionicLoading', '$q', 'ToastService', 'AnalyticsService', '$ionicScrollDelegate'];

    function controller($stateParams, $scope, ApiClientService, $filter, PermissionsService, NavigateService, PromptDialogService, CreateWorkoutService, TrainerWorkoutsListService, mpthconf, $ionicLoading, $q, ToastService, AnalyticsService, $ionicScrollDelegate) {

        var vm = this;
        vm.cdnBase = mpthconf.cdn.base;

        vm.editWorkout = function (workout) {
            if (workout.enabled) {
                NavigateService.go('app.edit_workout', {
                    workoutId: workout.id
                });
            }
        };

        vm.goToClient = function (customer) {

            NavigateService.go('app.trainer_client_profile', {
                customerId: customer.id,
                tab: 'workouts'
            });
        };

        vm.goToGroup = function (group) {

            NavigateService.go('app.trainer_workouts_group', {
                groupId: group.id
            });
        };

        /*****************************************************************
         * All Tab
         *****************************************************************/

        var loadAll = function (refreshing) {

            refreshing = refreshing || false;

            vm.allWorkoutsSearchFilter = '';

            if (refreshing) {
                TrainerWorkoutsListService.dirty();
            }

            TrainerWorkoutsListService.list(vm.trainerUserId).then(function (workouts) {
                vm.allWorkouts = workouts;
                vm.filteredAllWorkouts = angular.copy(workouts);

                vm.noAllWorkouts = vm.allWorkouts.length === 0;
            }).finally(function () {
                if (refreshing) {
                    $scope.$broadcast('scroll.refreshComplete');
                }
            });
        };

        // All Workouts Search filter
        $scope.$watch(function () {
            return vm.allWorkoutsSearchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.filteredAllWorkouts = $filter('filter')(vm.allWorkouts, {
                    title: newVal
                });
            } else {
                vm.filteredAllWorkouts = vm.allWorkouts;
            }
        });

        vm.allShow = function () {
            $ionicScrollDelegate.resize();
            loadAll();
        };

        vm.createWorkout = function () {
            CreateWorkoutService.start({
                trainerUserId: vm.trainerUserId
            });
        };

        /*****************************************************************
         * Groups Tab
         *****************************************************************/

        var loadGroups = function (refreshing) {

            refreshing = refreshing || false;

            if (!vm.groups || refreshing) {

                vm.groupsSearchFilter = '';

                ApiClientService.trainers.getAllWorkoutsByGroup(vm.trainerUserId).success(function (response) {

                    response.groups.map(function (g) {
                        g.workoutCount = g.workouts.length;
                    });

                    vm.groups = response.groups;

                    vm.filteredGroups = angular.copy(response.groups);

                    vm.noGroups = vm.groups.length === 0;
                }).finally(function () {
                    if (refreshing) {
                        $scope.$broadcast('scroll.refreshComplete');
                    }
                });
            }
        };

        // Groups Search filter
        $scope.$watch(function () {
            return vm.groupsSearchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.filteredGroups = $filter('filter')(vm.groups, {
                    title: newVal
                });
            } else {
                vm.filteredGroups = vm.groups;
            }
        });

        vm.groupsShow = function () {
            $ionicScrollDelegate.resize();
            loadGroups();
        };

        vm.createGroup = function () {

            PromptDialogService.open({
                title: "Create Workout Group",
                message: "Please enter a name for the group",
                buttonConfirm: "Create",
                buttonCancel: "Cancel",
                required: true
            }).then(function (name) {

                ApiClientService.workoutGroups.create({
                    title: name
                }, vm.trainerUserId).success(function () {
                    loadGroups(true);
                });
            });
        };

        /*****************************************************************
         * Clients Tab
         *****************************************************************/

        var loadClients = function (refreshing) {

            refreshing = refreshing || false;

            if (!vm.clients || refreshing) {

                vm.clientsSearchFilter = '';

                ApiClientService.customers.list(vm.trainerUserId, ['direct_workout_count']).success(function (response) {

                    // set full name on root object for search filter
                    response.customers.map(function (c) {
                        c.full_name = c.user.first_name + ' ' + c.user.last_name;
                    });

                    vm.clients = response.customers;

                    vm.filteredClients = angular.copy(response.customers);

                    vm.noClients = vm.clients.length === 0;
                }).finally(function () {
                    if (refreshing) {
                        $scope.$broadcast('scroll.refreshComplete');
                    }
                });
            }
        };

        // Clients Search filter
        $scope.$watch(function () {
            return vm.clientsSearchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.filteredClients = $filter('filter')(vm.clients, {
                    full_name: newVal
                });
            } else {
                vm.filteredClients = vm.clients;
            }
        });

        vm.byClientShow = function () {
            $ionicScrollDelegate.resize();
            loadClients();
        };

        /*****************************************************************
         * Find a Plan Tab
         *****************************************************************/

        vm.plansSearchFilter = null;

        vm.viewWorkouts = function (group) {

            NavigateService.go('app.trainer_workouts_library_group', {
                groupId: group.id
            });
        };

        var loadLibraryWorkouts = function (refreshing) {

            refreshing = refreshing || false;

            if (!vm.libraryPlans || refreshing) {

                ApiClientService.workoutLibrary.getLibraryGroups().success(function (response) {

                    vm.libraryPlans = response.groups;

                    vm.noLibraryPlans = vm.libraryPlans.length === 0;
                }).finally(function () {
                    if (refreshing) {
                        $scope.$broadcast('scroll.refreshComplete');
                    }
                });
            }
        };

        vm.findAPlan = function () {
            loadLibraryWorkouts();
        };

        vm.useLibraryGroup = function (group) {

            vm.processing = true;

            var currentUser = PermissionsService.getCurrentUser();

            var tag = 'metric';

            if (currentUser.country === 'US') {
                tag = 'imperial';
            }

            ApiClientService.workoutGroups.get(group.id, tag).success(function (response) {

                vm.processing = false;

                var workoutIds = [];

                response.group.workouts.map(function (workout) {
                    workoutIds.push(workout.id);
                });

                $ionicLoading.show({
                    template: 'Please wait...'
                }).then(function () {
                    vm.toDuplicateCount = workoutIds.length;
                    vm.duplicatedCount = 1;
                    vm.creatingGroup = false;

                    vm.percentComplete = 0;

                    var duplicateWorkoutActions = [];
                    angular.forEach(workoutIds, function (id) {

                        var duplicate = ApiClientService.workouts.duplicate(id);

                        duplicateWorkoutActions.push(duplicate);

                        duplicate.success(function () {
                            vm.duplicatedCount++;
                            vm.percentComplete += 90 / vm.toDuplicateCount;
                        });
                    });

                    $q.all(duplicateWorkoutActions).then(function (duplicateResponses) {
                        vm.creatingGroup = true;

                        var newWorkoutIds = [];
                        duplicateResponses.map(function (response) {
                            newWorkoutIds.push(response.data.workout.id);
                        });

                        ApiClientService.workoutGroups.create({
                            title: response.group.title,
                            workout_ids: newWorkoutIds
                        }).success(function (response) {

                            ToastService.show('Workouts successfully added to your account');
                            $ionicLoading.hide();
                        });
                    }, function (response) {
                        ToastService.show(response.data.error.message);
                        $ionicLoading.hide();
                    });
                });
            });
        };

        // Pull refresh handler
        vm.refresh = function () {

            switch (vm.selectedTabIndex) {

                // All
                case 0:
                    loadAll(true);
                    break;

                // Groups
                case 1:
                    loadGroups(true);
                    break;

                // By Client
                case 2:
                    loadClients(true);
                    break;

                // Find a Plan
                case 3:
                    loadLibraryWorkouts(true);
                    break;

            }
        };

        $scope.$on('$ionicView.beforeEnter', function () {

            vm.isCurrentTrainerUser = !$stateParams.trainerUserId || PermissionsService.getLoggedInUserId() === parseInt($stateParams.trainerUserId);

            vm.trainerUserId = $stateParams.trainerUserId;

            vm.canEdit = !vm.trainerUserId || PermissionsService.userIsTrainerAdminOrOwner(vm.trainerUserId);

            /*
                Ensure that we are refreshing dirtied lists (maybe something was
                deleted or title edited, don't want the old cached data being
                shown.
             */
            switch (vm.selectedTabIndex) {

                // All
                case 0:
                    loadAll();
                    break;

                // Groups
                case 1:
                    //loadGroups();
                    break;

                // By Client
                case 2:
                    //loadClients();
                    break;

            }
        });

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Trainer - Workouts');
        });

        // temp horrible hack to work around issue of disappearing tabs
        // in some weird circumstances.
        $scope.$on('$ionicView.afterEnter', function () {
            $(window).trigger('resize');
        });
    }

    service.$inject = ['$q', 'ApiClientService'];

    function service($q, ApiClientService) {

        var dirty = false;
        var lastTrainerUserId;

        var workouts = [];

        var reset = function () {
            workouts = [];
        };

        return {

            reset: reset,

            dirty: function () {
                dirty = true;
                this.reset();
            },

            list: function (trainerUserId) {

                trainerUserId = trainerUserId || false;

                if (typeof lastTrainerUserId === 'undefined') {
                    lastTrainerUserId = trainerUserId;
                }

                if (!dirty && workouts.length > 0 && lastTrainerUserId === trainerUserId) {
                    lastTrainerUserId = trainerUserId;
                    return $q.when(workouts);
                }

                lastTrainerUserId = trainerUserId;

                return $q(function (resolve, reject) {

                    ApiClientService.trainers.getAllWorkouts(trainerUserId).success(function (response) {

                        workouts = response.workouts;

                        dirty = false;

                        resolve(workouts);
                    }).error(reject);
                });
            }

        };
    }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').config(config).controller('accountSwapper', controller);

  config.$inject = ['$stateProvider'];

  function config($stateProvider) {
    $stateProvider.state('app.accounts', {
      url: '/choose-trainer',
      views: {
        'menuContent': {
          templateUrl: 'common/accountSwapper/accountSwapper.html',
          controller: 'accountSwapper',
          controllerAs: 'vm'
        }
      }
    });
  }

  controller.$inject = ['ApiClientService', 'ToastService', '$scope', 'BrandingService', 'AuthenticationStateService', 'AccountSwapperService', '$filter', 'NavigateService'];

  function controller(ApiClientService, ToastService, $scope, BrandingService, AuthenticationStateService, AccountSwapperService, $filter, NavigateService) {

    var vm = this;

    vm.loadingAccounts = true;

    const getAccounts = () => {
      ApiClientService.customerTrainers.get().success(response => {
        vm.trainers = response.trainers;
        vm.trainerAccount = response.trainerAccount;
        vm.loadingAccounts = false;
        vm.showPanel = () => {
          return vm.trainers.length > 0 || vm.trainerAccount;
        };
        vm.currentUser = AuthenticationStateService.getCurrentUser();
      }).error(error => {
        console.log(error);
      });
    };

    vm.refresh = () => {
      getAccounts();
      return true;
    };

    getAccounts();

    vm.getLogo = logo => {

      return "../../logos/" + logo;
    };
    vm.currentBranding = BrandingService.applyStyles();

    vm.showActiveCustomer = account => {

      return vm.currentUser.branding.subdomain === account.trainer.subdomain && account.user.type === 'customer' && account.trainer_user_id === vm.currentUser.trainer_user_id;
    };

    vm.isAccountDisabled = account => {
      return !account.trainer.is_subscribed && !account.trainer.on_free_trial && !account.trainer.is_paid_subscription;
    };

    vm.showActiveTrainer = account => {

      return vm.currentUser.type === 'trainer' && account.type === 'trainer';
    };

    vm.changeAccount = (account, isTrainer) => {
      if (!isTrainer) {
        if (vm.isAccountDisabled(account)) {
          return ToastService.show('Sorry this account has been disabled.');
        }
      }
      AccountSwapperService.changeAccount(account, isTrainer);
    };

    $scope.$on('activeUsed:updated', () => {
      getAccounts();
    });
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('accountSwapperModalController', controller);

    controller.$inject = ['$scope', 'ApiClientService', '$rootScope'];

    function controller($scope, ApiClientService, $rootScope) {

        var vm = this;

        $scope.$on('accountSwapperCloseModal:close', () => {
            $scope.cancelModal(null);
        });
    }
})();
/* eslint-disable default-case */
(() => {
  angular.module('MyPTHub').directive('activityLog', directive);

  /**
   * Defines the directive
   *
   * @returns {Object} Directive
   */
  function directive() {
    return {
      restrict: 'E',
      replace: false,
      scope: {
        customerId: '=?',
        init: '=',
        active: '=?',
        noActivityDescription: '@?',
        showUsernames: '=?'
      },
      templateUrl: 'common/activityLog/activityLog.html',
      controller,
      controllerAs: 'vm'
    };
  }

  /**
   * @param {Object} ApiClientService Api Methods
   * @param {Object} $scope $scope
   * @param {Object} PermissionsService Permissions Service
   * @param {Object} $ionicScrollDelegate Ionic scrolling
   * @param {Object} NavigateService Navigation Service
   * @param {Object} PackagesCustomerPurchasesDialogService Package purchase modal
   * @param {Object} $filter Angular's $filter
   * @param {Object} NutritionLogService Nutrition Log Service
   * @param {Object} timeAgoSettings Settings for timeAgo
   * @param {Object} SubscriptionStateService Subscription State Service
   */
  function controller(ApiClientService, $scope, PermissionsService, $ionicScrollDelegate, NavigateService, PackagesCustomerPurchasesDialogService, $filter, NutritionLogService, timeAgoSettings, SubscriptionStateService) {
    const vm = this;

    timeAgoSettings.fullDateAfterSeconds = 60 * 60 * 24;
    timeAgoSettings.refreshMillis = 30000;

    vm.animationsEnabled = false;

    vm.noActivityDesc = $scope.noActivityDescription;
    vm.showUsernames = $scope.showUsernames;

    vm.isTrainer = PermissionsService.userIsTrainer();
    vm.isCustomer = PermissionsService.userIsCustomer();
    vm.trainerSubscribed = SubscriptionStateService.trainerIsSubscribed();

    vm.trainerSubscribed = SubscriptionStateService.trainerIsSubscribed();

    vm.isYou = true;
    if ($scope.customerId && vm.isTrainer) {
      vm.isYou = false;
    }

    const resultsPerPage = 6;
    vm.loadingMore = false;

    vm.pagination = {
      page: 1,
      nextPage: null,
      loading: false
    };

    /**
     * @param {Array} items An array of items for the activity log
     * @returns {Array} Transformed activity log
     */
    const transformItems = items => {
      // filter out item types that are not yet setup on this frontend
      const activityLogItems = $filter('filter')(items, item => {
        const supportedType = ['photo', 'customermetric', 'session', 'nutritionlogday', 'packagepurchase', 'form', 'schedule', 'file', 'customer', 'calendarcompletedevent'].indexOf(item.type) !== -1;

        if (!supportedType) {
          return false;
        }

        if (item.type === 'photo' || item.type === 'file') {
          return true;
        } else if (item.subject) {
          return true;
        }

        return false;
      });

      activityLogItems.map(item => {
        item.animate = true;

        switch (item.type) {
          case 'photo':
            item.icon_name = 'camera_alt';
            break;

          case 'customermetric':
            item.icon_name = 'straighten';
            break;

          case 'session':
            item.icon_name = 'fitness_center';
            break;

          case 'nutritionlogday':
            item.icon_name = 'restaurant_menu';
            /**
             * Format logged day date for nutrition log.
             */
            let formattedDay;
            if (item.subject.hasOwnProperty('day')) {
              const parsedDay = moment(item.subject.day, 'YYYY-MM-DD');
              formattedDay = parsedDay.format('Do MMMM');;
            } else {
              formattedDay = null;
            }

            item.subject.day_formatted = formattedDay;
            break;

          case 'packagepurchase':
            item.icon_name = 'credit_card';
            break;

          case 'form':
            item.icon_name = 'assignment';
            break;

          case 'schedule':
            item.icon_name = 'date_range';
            break;

          case 'file':
            item.icon_name = 'insert_drive_file';
            break;

          case 'customer':
            if (!vm.trainerSubscribed) {
              item.hide = true;
              break;
            }
            item.color = '#00c861';
            switch (item.action) {
              case 'activity_met.steps':
                item.goal_type = 'steps';
                item.icon_name = 'directions_walk';
                break;
              case 'activity_met.calories':
                item.goal_type = 'calories';
                item.icon_name = 'whatshot';
                break;
              case 'activity_met.floors':
                item.goal_type = 'floors';
                item.icon_src = 'img/stairs.svg';
                break;
              case 'activity_met.distance':
                item.goal_type = 'distance';
                item.icon_src = 'img/transfer.svg';
                break;
            }
            break;
          default:
            item.icon_name = 'info';
        }
        return item;
      });
      return activityLogItems;
    };

    vm.items = [];

    /**
     * Gets the activity log for the user
     * @param {Boolean} paginating Determines if we are paginating or not
     * @returns {*} Activity Log
     */
    const getActivityData = (paginating = false) => {
      if (!paginating) {
        vm.pagination.page = 1;
        vm.pagination.nextPage = null;
      }

      vm.pagination.loading = true;

      let getLog;

      if ($scope.customerId) {
        getLog = ApiClientService.activityLog.customer($scope.customerId, resultsPerPage, vm.pagination.page);
      } else {
        getLog = ApiClientService.activityLog.currentUser(resultsPerPage, vm.pagination.page);
      }

      getLog.success(response => {
        if (!paginating && vm.items.length > 0) {
          vm.animationsEnabled = false;
          vm.items = [];
        } else {
          vm.animationsEnabled = true;
        }

        vm.pagination.nextPage = response.activityLog.pagination.nextPage || null;

        vm.items.map(item => {
          item.animate = false;
          return true;
        });

        vm.items.push(...transformItems(response.activityLog.data));

        $scope.$broadcast('scroll.infiniteScrollComplete');

        $ionicScrollDelegate.resize();
      });

      getLog.finally(() => {
        vm.pagination.loading = false;
      });

      return getLog;
    };

    vm.canLoadMore = () => {
      let active = true;

      if ($scope.hasOwnProperty('active') && !$scope.active) {
        active = false;
      }

      return active && vm.pagination.loading === false && vm.pagination.nextPage !== null;
    };

    vm.loadMore = () => {
      vm.pagination.page += 1;
      getActivityData(true);
    };

    vm.goToSubject = (item, event) => {
      event.preventDefault();

      if (PermissionsService.userIsTrainer()) {
        switch (item.type) {
          case 'session':
            NavigateService.go('app.session_preview', {
              sessionId: item.subject.id
            });
            break;

          case 'customermetric':
            NavigateService.go('app.trainer_update_customer_metrics', {
              customerId: item.subject.customer_id
            });
            break;

          case 'nutritionlogday':
            NavigateService.go('app.trainer_view_nutrition_log_day', {
              customerId: item.subject.customer_id,
              date: item.subject.day
            });
            break;

          case 'photo':
            NavigateService.go('app.trainer_client_profile', {
              customerId: item.customer.id,
              tab: 'photos'
            });
            break;

          case 'packagepurchase':
            NavigateService.go('app.trainer_client_profile', {
              customerId: item.customer.id,
              tab: 'sessions'
            }).then(() => {
              PackagesCustomerPurchasesDialogService.show(item.customer.id);
            });
            break;

          case 'form':
            NavigateService.go('app.view_customer_form', {
              customerId: item.customer.id,
              formId: item.subject.id
            });
            break;

          case 'schedule':
            NavigateService.go('app.view_customer_schedule', {
              customerId: item.customer.id,
              scheduleId: item.subject.id
            });
            break;

          case 'file':
            NavigateService.go('app.files');
            break;

          case 'customer':
            NavigateService.go('app.trainer_client_profile', {
              customerId: item.customer.id,
              tab: 'progress'
            });
            break;
          case 'calendarcompletedevent':
            NavigateService.go('app.trainer_calendar_event_booking', {
              eventId: item.subject.calendar_event.id,
              eventTimeId: item.subject.calendar_event_time_id,
              date_time: item.subject.date_time
            });
            break;
        }
      } else {
        switch (item.type) {
          case 'session':
            NavigateService.go('app.session_preview', {
              sessionId: item.subject.id
            });
            break;

          case 'customermetric':
            NavigateService.go('app.client_update_metrics');
            break;

          case 'nutritionlogday':
            NutritionLogService.startLogger(item.subject.day);
            break;

          case 'photo':
            NavigateService.go('app.client_progress_photos');
            break;

          case 'packagepurchase':
            NavigateService.go('app.client_packages_session_quota').then(() => {
              PackagesCustomerPurchasesDialogService.show(item.customer.id);
            });
            break;

          case 'form':
            NavigateService.go('app.view_customer_form', {
              customerId: item.customer.id,
              formId: item.subject.id
            });
            break;

          case 'schedule':
            NavigateService.go('app.view_customer_schedule', {
              customerId: item.customer.id,
              scheduleId: item.subject.id
            });
            break;
          case 'file':
            NavigateService.go('app.files');
            break;
          case 'calendarcompletedevent':
            NavigateService.go('app.trainer_calendar_event_booking', {
              eventId: item.subject.calendar_event.id,
              eventTimeId: item.subject.calendar_event_time_id,
              date_time: item.subject.date_time
            });
            break;
        }
      }
    };

    $scope.init = () => getActivityData();
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('BugReportModalController', controller);

    controller.$inject = ['ApiClientService', 'ToastService', '$scope', 'DeviceInfoService', 'PermissionsService'];

    function controller(ApiClientService, ToastService, $scope, DeviceInfoService, PermissionsService) {

        var vm = this;
        var loggedInUserDetails = PermissionsService.getCurrentUser();

        vm.data = {};
        vm.processing = false;

        vm.data.from = loggedInUserDetails.first_name + ' ' + loggedInUserDetails.last_name;
        vm.data.fromEmail = loggedInUserDetails.email;
        vm.data.platform = DeviceInfoService.platform();
        vm.data.appVer = 'Unknown';
        vm.data.userType = loggedInUserDetails.type;
        vm.data.messageContent = '';

        DeviceInfoService.appVersion().then(function (appVer) {
            vm.data.appVer = appVer;
        });

        vm.submitIssue = function () {
            vm.processing = true;

            ApiClientService.bugReport.send(vm.data).success(function (response) {
                vm.processing = false;

                ToastService.show('Issue Reported Successfully');

                $scope.closeModal();
            });
        };
    }
})();
(() => {
  class ClientCalendarController {
    /**
     * @param ApiClientService
     * @param $scope
     * @param PermissionsService
     * @param $stateParams
     * @param DayScheduleService
     * @param ScheduleDayPickerService
     * @param SessionStoreService
     * @param NetworkStateService
     * @param $q
     * @param SessionScheduleService
     * @param ScheduleEventService
     * @param AnalyticsService
     * @param NavigateService
     * @param CustomerSwitcherService
     * @param ScheduleEventStoreService
     */
    constructor(ApiClientService, $scope, PermissionsService, $stateParams, DayScheduleService, ScheduleDayPickerService, SessionStoreService, NetworkStateService, $q, SessionScheduleService, ScheduleEventService, AnalyticsService, NavigateService, CustomerSwitcherService, ScheduleEventStoreService) {
      this.ApiClientService = ApiClientService;
      this.$scope = $scope;
      this.PermissionsService = PermissionsService;
      this.$stateParams = $stateParams;
      this.customerId = false;
      this.DayScheduleService = DayScheduleService;
      this.ScheduleDayPickerService = ScheduleDayPickerService;
      this.SessionStoreService = SessionStoreService;
      this.NetworkStateService = NetworkStateService;
      this.$q = $q;
      this.SessionScheduleService = SessionScheduleService;
      this.ScheduleEventService = ScheduleEventService;
      this.AnalyticsService = AnalyticsService;
      this.NavigateService = NavigateService;
      this.CustomerSwitcherService = CustomerSwitcherService;
      this.ScheduleEventStoreService = ScheduleEventStoreService;

      // Vars
      this.showQuotaButton = false;
      this.datePickerToggle = ScheduleDayPickerService.toggleOpen;
      this.currentDate = false;
      this.currentDateFormatted = false;
      this.schedulerEnabled = false;
      this.active = false;
      this.currentUser = PermissionsService.getCurrentUser();
      this.userIsTrainer = PermissionsService.userIsTrainer();
      this.customerId = $stateParams.customerId ? parseInt($stateParams.customerId, 10) : PermissionsService.getCurrentUserEntityId();

      // Other functions.
      this.$scope.$on('$ionicView.beforeEnter', () => {
        this.showQuotaButton = !PermissionsService.userIsTrainer() && PermissionsService.merchantProvisioned();

        this.customerId = $stateParams.customerId ? parseInt($stateParams.customerId, 10) : PermissionsService.getCurrentUserEntityId();

        this.DayScheduleService.reset();

        this.setDate();
      });
      // eslint-disable-next-line consistent-return
      this.ScheduleDayPickerService.registerDayPickedEventHandler($scope, () => {
        if (!this.active) {
          return false;
        }

        this.setDate();
      });
      this.$scope.$on('$ionicView.afterEnter', () => {
        this.active = true;
        this.schedulerEnabled = this.PermissionsService.schedulerIsEnabled();
        let message;
        if (this.PermissionsService.userIsTrainer()) {
          message = 'Trainer - View Client Calendar';
        } else {
          message = 'Client - Calendar';
        }
        this.AnalyticsService.trackView(message);
      });
      this.$scope.$on('$ionicView.beforeLeave', () => {
        this.active = false;
      });
      // eslint-disable-next-line consistent-return
      this.CustomerSwitcherService.onSwitched($scope, (ev, customer) => {
        if (!this.active) {
          return false;
        }

        if (customer.customer_id) {
          NavigateService.replaceCurrentState().go('app.client_calendar', {
            customerId: customer.customer_id
          });
        }
        if (customer.hasOwnProperty('type')) {
          console.log('Go to company calendar');
          this.NavigateService.replaceCurrentState().go('app.company_calendar');
        } else {
          NavigateService.replaceCurrentState().go('app.trainer_calendar', {
            trainerUserId: customer.trainer_user_id ? customer.trainer_user_id : this.currentUser.trainer_user_id
          });
        }
      });
    }

    /**
     * @param reset
     */
    setDate(reset = false) {
      if (reset) {
        this.DayScheduleService.reset();
      }

      this.currentDate = this.ScheduleDayPickerService.getDate();
      this.currentDateFormatted = this.currentDate.format('Do MMMM YYYY');
      this.load(true);
      this.DayScheduleService.setDate(this.currentDate);
    }

    /**
     * @param init
     */
    load(init = false) {
      const chosenDate = this.ScheduleDayPickerService.getDate();
      const dateStart = moment(chosenDate).startOf('day'); //.subtract(6, 'week');
      const dateEnd = moment(chosenDate).endOf('day'); //.add(6, 'week');


      if (this.SessionStoreService.dirty()) {
        // eslint-disable-next-line consistent-return
        this.loadLive(dateStart, dateEnd).then(() => {
          if (!this.active) {
            return false;
          }

          this.loadCached(dateStart, dateEnd);
        });
      } else {
        this.loadCached(dateStart, dateEnd).then(() => {
          if (init && this.NetworkStateService.isOnline()) {
            // eslint-disable-next-line consistent-return
            this.loadLive(dateStart, dateEnd).then(() => {
              if (!this.active) {
                return false;
              }

              this.loadCached(dateStart, dateEnd);
            });
          }
        });
      }
    }

    /**
     * @param start
     * @param end
     * @returns {*}
     */
    loadLive(start, end) {
      const reqs = [this.SessionStoreService.downloadCustomerSessions(this.customerId, start, end, false)];

      if (this.schedulerEnabled) {
        reqs.push(this.ScheduleEventStoreService.downloadCustomerEvents(this.customerId));
      }

      return this.$q.all(reqs);
    }

    /**
     * @param start
     * @param end
     * @returns {PromiseLike<T | never> | Promise<T | never>}
     */
    loadCached(start, end) {
      const reqs = [this.SessionScheduleService.getCustomerSchedule(start, end, this.customerId)];
      if (this.schedulerEnabled) {
        reqs.push(this.ScheduleEventService.getCustomerEvents(start, end, this.customerId));
      }
      if (this.NetworkStateService.isOnline()) {
        const data = {
          startDate: start.format('YYYY-MM-DD'),
          endDate: end.format('YYYY-MM-DD'),
          selectedUser: this.customerId,
          type: 'customer'
        };
        reqs.push(this.ApiClientService.calendarEventTimes.getAllFrom(data));
      }

      // eslint-disable-next-line consistent-return
      return this.$q.all(reqs).then(responses => {
        if (!this.active) {
          return false;
        }

        const sessions = responses[0];
        let schedulerEvents = this.schedulerEnabled ? responses[1] : [];
        const events = this.schedulerEnabled ? responses[2] : responses[1];

        schedulerEvents = schedulerEvents.hasOwnProperty('events') ? schedulerEvents.events : schedulerEvents;

        this.DayScheduleService.setCalendarEvents({
          sessions,
          schedulerEvents,
          events
        });
      });
    }

    /**
     * Sends the user to start an adhoc sessions, if viewing a clients calendar it starts
     * an adhoc session with them pre-selected (apparently?)
     */
    startSession() {
      const config = {};
      if (this.$stateParams.customerId) {
        config.customerPreselectId = this.$stateParams.customerId;
      }

      this.NavigateService('app.start_adhoc_session', config);
    }
  }

  const config = $stateProvider => {
    $stateProvider.state('app.client_calendar', {
      url: '/client_calendar/:customerId?',
      views: {
        menuContent: {
          templateUrl: 'common/clientCalendar/calendar.html',
          controller: 'ClientCalendarController',
          controllerAs: 'vm'
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('ClientCalendarController', ClientCalendarController);
})();
(() => {
  class CompanyCalendarController {
    /**
     * @param ApiClientService
     */
    constructor(ApiClientService, CustomerSwitcherService, $scope, NavigateService, PermissionsService, ScheduleDayPickerService, AnalyticsService, DayScheduleService) {
      this.ApiClientService = ApiClientService;
      this.CustomerSwitcherService = CustomerSwitcherService;
      this.$scope = $scope;
      this.NavigateService = NavigateService;
      this.PermissionsService = PermissionsService;
      this.ScheduleDayPickerService = ScheduleDayPickerService;
      this.AnalyticsService = AnalyticsService;
      this.DayScheduleService = DayScheduleService;
      this.datePickerToggle = ScheduleDayPickerService.toggleOpen;

      this.currentUser = PermissionsService.getCurrentUser();
      this.active = true;

      // eslint-disable-next-line consistent-return
      this.CustomerSwitcherService.onSwitched($scope, (ev, customer) => {
        if (!this.active) {
          return false;
        }

        if (customer.customer_id) {
          NavigateService.replaceCurrentState().go('app.client_calendar', {
            customerId: customer.customer_id
          });
        }
        if (customer.hasOwnProperty('type')) {
          this.NavigateService.replaceCurrentState().go('app.company_calendar');
        } else {
          NavigateService.replaceCurrentState().go('app.trainer_calendar', {
            trainerUserId: customer.trainer_user_id ? customer.trainer_user_id : this.currentUser.trainer_user_id
          });
        }
      });

      // eslint-disable-next-line consistent-return
      ScheduleDayPickerService.registerDayPickedEventHandler($scope, () => {
        if (!this.active) {
          return false;
        }

        this.setDate();
      });

      $scope.$on('$ionicView.afterEnter', () => {
        this.AnalyticsService.trackView('Trainer - Schedule');
        this.active = true;
      });

      $scope.$on('$ionicView.beforeEnter', () => {
        this.setDate();
      });

      $scope.$on('$ionicView.beforeLeave', () => {
        this.active = false;
      });
    }

    /**
     * @param reset
     */
    setDate(reset = false) {
      if (reset) {
        this.DayScheduleService.reset();
      }

      this.currentDate = this.ScheduleDayPickerService.getDate();
      this.currentDateFormatted = this.currentDate.format('Do MMMM YYYY');
      this.load(true);
      this.DayScheduleService.setDate(this.currentDate);
    }

    load(init = false) {
      const chosenDate = this.ScheduleDayPickerService.getDate();
      const dateStart = moment(chosenDate).startOf('day'); //.subtract(6, 'week');
      const dateEnd = moment(chosenDate).endOf('day').add(1, 'day'); //.add(6, 'week');

      const data = {
        startDate: dateStart.format('YYYY-MM-DD'),
        endDate: dateEnd.format('YYYY-MM-DD'),
        type: 'company'
      };

      this.ApiClientService.calendarEventTimes.getAllFrom(data).then(response => {
        this.DayScheduleService.setEvents(response.events);
      });
    }
  }

  const config = $stateProvider => {
    $stateProvider.state('app.company_calendar', {
      url: '/company_calendar',
      views: {
        menuContent: {
          templateUrl: 'common/companyCalendar/views/companyCalendar.html',
          controller: 'CompanyCalendarController',
          controllerAs: 'vm'
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('CompanyCalendarController', CompanyCalendarController);
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('connectionStatusAlert', directive);

    function directive() {
        return {
            restrict: 'E',
            replace: true,
            template: '<div ng-if="vm.offline" class="internetConnectionError list animated">' + '<div class="item item-icon-left">' + '<i class="icon material-icons">error_outline</i>' + 'No internet connection available' + '</div>' + '</div>',
            controller: controller,
            controllerAs: 'vm',
            scope: {}
        };
    }

    controller.$inject = ['$scope', 'NetworkStateService'];

    function controller($scope, NetworkStateService) {

        var vm = this;

        var updateState = function () {
            vm.offline = !NetworkStateService.isOnline();
        };

        updateState();

        NetworkStateService.registerEventHandler($scope, updateState);
    }
})();

(function () {
    'use strict';

    angular.module('MyPTHub').factory('CustomerFinderModalService', service).controller('CustomerFinderModalController', controller);

    service.$inject = ['ModalService'];

    function service(ModalService) {
        return {
            open: function (config) {

                var mode = false;
                if (config.hasOwnProperty('mode')) {
                    mode = config.mode;
                }

                var currentCustomerAssignments = false;
                if (config.hasOwnProperty('currentCustomerAssignments')) {
                    currentCustomerAssignments = config.currentCustomerAssignments;
                }

                var currentGroupAssignments = false;
                if (config.hasOwnProperty('currentGroupAssignments')) {
                    currentGroupAssignments = config.currentGroupAssignments;
                }

                var modalTitle = false;
                if (config.hasOwnProperty('modalTitle')) {
                    modalTitle = config.modalTitle;
                }

                var includeGroups = false;
                if (config.hasOwnProperty('includeGroups')) {
                    includeGroups = config.includeGroups;
                }

                var trainerUserId = config.trainerUserId;

                return ModalService.show({
                    templateUrl: "common/customerFinder/customerFinderModal.html",
                    controller: 'CustomerFinderModalController as vm',
                    params: {
                        trainerUserId: trainerUserId,
                        mode: mode,
                        currentCustomerAssignments: currentCustomerAssignments,
                        currentGroupAssignments: currentGroupAssignments,
                        modalTitle: modalTitle,
                        includeGroups: includeGroups
                    }
                });
            }
        };
    }

    controller.$inject = ['parameters', '$scope', 'CustomerStoreService', '$filter', 'PermissionsService', 'AnalyticsService', 'ApiClientService', '$q'];

    function controller(parameters, $scope, CustomerStoreService, $filter, PermissionsService, AnalyticsService, ApiClientService, $q) {

        var vm = this;

        vm.currentCustomerAssignments = parameters.currentCustomerAssignments || [];
        vm.currentGroupAssignments = parameters.currentGroupAssignments || [];
        vm.mode = parameters.mode || 'single';
        vm.isCurrentTrainerUser = PermissionsService.getLoggedInUserId() === parameters.trainerUserId;
        vm.includeGroups = parameters.includeGroups || false;

        vm.assignedCustomerIds = [];
        angular.forEach(vm.currentCustomerAssignments, function (customer) {
            vm.assignedCustomerIds.push(customer.id);
        });

        vm.assignedGroupIds = [];
        angular.forEach(vm.currentGroupAssignments, function (group) {
            vm.assignedGroupIds.push(group.id);
        });

        var reqs = [];

        reqs.push(CustomerStoreService.downloadTrainerCustomers().then(function () {
            return CustomerStoreService.listAllTrainerCustomers(parameters.trainerUserId).then(function (response) {
                // set full name on root object for search filter
                return response.map(function (c) {
                    c = c.data;

                    c.searchableString = c.user.first_name + ' ' + c.user.last_name;

                    c.use = vm.assignedCustomerIds.indexOf(c.id) !== -1;

                    c.type = 'customer';

                    return c;
                });
            });
        }));

        if (vm.includeGroups) {
            reqs.push(ApiClientService.customerGroups.list(parameters.trainerUserId).then(function (response) {
                var groups = response.groups;

                groups.map(function (group) {
                    group.type = 'group';
                    group.searchableString = group.title;
                    group.use = vm.assignedGroupIds.indexOf(group.id) !== -1;
                    return group;
                });

                return groups;
            }));
        }

        $q.all(reqs).then(function (response) {
            vm.items = response[0];

            if (typeof response[1] !== 'undefined') {
                vm.items = response[1].concat(response[0]);
            }

            vm.noItems = vm.items.length === 0;
            vm.filteredItems = angular.copy(vm.items);
        });

        // Customer List Search filter
        $scope.$watch(function () {
            return vm.searchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.searchFilterActive = true;
                vm.filteredItems = $filter('filter')(vm.items, {
                    searchableString: newVal
                });
            } else {
                vm.searchFilterActive = false;
                vm.filteredItems = vm.items;
            }
        });

        var clearAllSelected = function () {
            vm.items.map(function (item) {
                item.use = false;
            });
            vm.filteredItems.map(function (item) {
                item.use = false;
            });
        };

        vm.assign = function (item) {
            if (vm.mode === 'single') {
                clearAllSelected();
            }

            item.use = !item.use;

            var findOriginal = $filter('filter')(vm.items, {
                id: item.id, type: item.type
            }, true);

            if (findOriginal.length > 0) {

                var originalIndex = vm.items.indexOf(findOriginal[0]);

                if (originalIndex !== -1) {
                    vm.items[originalIndex].use = item.use;
                }
            }
        };

        vm.save = function () {
            var toAssign = $filter('filter')(vm.items, {
                use: true
            });

            if (toAssign.length === 0) {
                $scope.closeModal([]);
                return true;
            }

            var list = {
                customers: [],
                customerGroups: []
            };

            toAssign.map(function (item) {
                if (item.type === 'group') {
                    list.customerGroups.push(item);
                }
                if (item.type === 'customer') {
                    list.customers.push(item);
                }
            });

            $scope.closeModal(list);
        };

        AnalyticsService.trackView('Trainer - Customer Finder (Assignment)');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('CustomerNoteFormModalController', controller);

    controller.$inject = ['parameters', 'ApiClientService', 'ToastService', '$scope'];

    function controller(parameters, ApiClientService, ToastService, $scope) {

        var vm = this;

        vm.mode = 'creating';
        vm.note = {};
        vm.processing = false;

        if (parameters.note) {
            vm.note = parameters.note;
            vm.mode = 'editing';
        }

        vm.createNote = function () {
            vm.processing = true;

            ApiClientService.customers.createNote(parameters.customer_id, vm.note).success(function (response) {
                vm.processing = false;

                ToastService.show('Note successfully created');

                $scope.closeModal(response.note);
            });
        };

        vm.updateNote = function () {
            vm.processing = true;

            ApiClientService.customers.updateNote(parameters.customer_id, vm.note.id, vm.note).success(function (response) {
                vm.processing = false;

                ToastService.show('Note successfully updated');

                $scope.closeModal(response.note);
            });
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('EntityFinderModalService', service).controller('EntityFinderModalController', controller);

    service.$inject = ['ModalService'];

    function service(ModalService) {
        return {
            open: function (trainerUserId, entityType, entityTypeName, filterEntityIds, modalTitle) {
                filterEntityIds = filterEntityIds || false;
                modalTitle = modalTitle || false;

                return ModalService.show({
                    templateUrl: "common/entityFinder/entityFinderModal.html",
                    controller: 'EntityFinderModalController as vm',
                    params: {
                        trainerUserId: trainerUserId,
                        entityType: entityType,
                        entityTypeName: entityTypeName,
                        filterEntityIds: filterEntityIds,
                        modalTitle: modalTitle
                    }
                });
            }
        };
    }

    controller.$inject = ['parameters', '$scope', '$filter', 'ApiClientService', 'PermissionsService', 'AnalyticsService'];

    function controller(parameters, $scope, $filter, ApiClientService, PermissionsService, AnalyticsService) {

        var vm = this;

        vm.trainerUserId = parameters.trainerUserId;
        vm.entityTypeName = parameters.entityTypeName;

        vm.modalTitle = 'Assign a ' + vm.entityTypeName;
        if (parameters.modalTitle) {
            vm.modalTitle = parameters.modalTitle;
        }

        vm.idField = 'id';
        vm.titleField = 'title';

        var baseEntityListFilter;
        if (parameters.filterEntityIds) {
            var filterConf = {};
            filterConf[vm.idField] = '!' + parameters.filterEntityIds;

            baseEntityListFilter = function (entities) {
                return $filter('filter')(entities, function (entity) {
                    var filterIds = parameters.filterEntityIds;
                    if (typeof parameters.filterEntityIds !== 'array') {
                        filterIds = [parameters.filterEntityIds];
                    }

                    return filterIds.indexOf(entity.id) !== -1;
                });
            };
        }

        vm.isCurrentTrainerUser = PermissionsService.getLoggedInUserId() === vm.trainerUserId;

        switch (parameters.entityType) {
            case 'nutritionPlan':
                vm.entityIcon = 'restaurant_menu';
                ApiClientService.trainers.getAllNutritionPlans(vm.trainerUserId).success(function (response) {
                    if (typeof baseEntityListFilter === 'function') {
                        vm.entities = baseEntityListFilter(response.nutritionPlans);
                    } else {
                        vm.entities = response.nutritionPlans;
                    }
                    vm.filteredEntities = angular.copy(vm.entities);
                    vm.noEntities = vm.entities.length === 0;
                });
                break;

            case 'nutritionPlanGroup':
                vm.entityIcon = 'folder_open';
                ApiClientService.trainers.getAllNutritionPlansByGroup(vm.trainerUserId).success(function (response) {
                    if (typeof baseEntityListFilter === 'function') {
                        vm.entities = baseEntityListFilter(response.groups);
                    } else {
                        vm.entities = response.groups;
                    }
                    vm.filteredEntities = angular.copy(vm.entities);
                    vm.noEntities = vm.entities.length === 0;
                });
                break;

            case 'workout':
                vm.entityIcon = 'fitness_center';
                ApiClientService.trainers.getAllWorkouts(vm.trainerUserId).success(function (response) {
                    if (typeof baseEntityListFilter === 'function') {
                        vm.entities = baseEntityListFilter(response.workouts);
                    } else {
                        vm.entities = response.workouts;
                    }
                    vm.filteredEntities = angular.copy(vm.entities);
                    vm.noEntities = vm.entities.length === 0;
                });
                break;

            case 'workoutGroup':
                vm.entityIcon = 'folder_open';
                ApiClientService.trainers.getAllWorkoutsByGroup(vm.trainerUserId).success(function (response) {
                    if (typeof baseEntityListFilter === 'function') {
                        vm.entities = baseEntityListFilter(response.groups);
                    } else {
                        vm.entities = response.groups;
                    }
                    vm.filteredEntities = angular.copy(vm.entities);
                    vm.noEntities = vm.entities.length === 0;
                });
                break;

            case 'schedule':
                vm.entityIcon = 'schedule';
                vm.titleField = 'name';
                ApiClientService.trainers.scheduleTemplateIndexTrainerUser(vm.trainerUserId).success(function (response) {
                    if (typeof baseEntityListFilter === 'function') {
                        vm.entities = baseEntityListFilter(response.templates);
                    } else {
                        vm.entities = response.templates;
                    }
                    vm.filteredEntities = angular.copy(vm.entities);
                    vm.noEntities = vm.entities.length === 0;
                });
                break;
        }

        // List Search filter
        $scope.$watch(function () {
            return vm.searchFilter;
        }, function (newVal, oldVal) {

            var filterConf = {};
            filterConf[vm.titleField] = newVal;

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.searchFilterActive = true;
                vm.filteredEntities = $filter('filter')(vm.entities, filterConf);
            } else {
                vm.searchFilterActive = false;
                vm.filteredEntities = vm.entities;
            }
        });

        vm.choose = function (entity) {
            $scope.closeModal(entity);
        };

        AnalyticsService.trackView('Trainer - Search For An Entity (' + vm.entityTypeName + ')');
    }
})();
(() => {
  class TrainerEventBookingController {
    /**
     * @param ApiClientService
     * @param ToastService
     * @param $state
     * @param AuthenticationStateService
     * @param eventsData
     * @param trainersData
     * @param workoutData
     * @param $scope
     */
    constructor(ApiClientService, ToastService, $state, AuthenticationStateService, eventsData, trainersData, workoutData, $scope, $timeout) {
      this.ApiClientService = ApiClientService;
      this.ToastService = ToastService;
      this.$state = $state;
      this.currentUser = AuthenticationStateService.getCurrentUser();
      this.booking = {};
      this.time = {};
      this.$scope = $scope;
      const address = this.currentUser.address1;
      const city = this.currentUser.town_city;
      const postcode = this.currentUser.postcode;
      this.fullAddress = `${address} ${city} ${postcode}`;
      this.latitude = null;
      this.longitude = null;
      this.location = this.fullAddress;
      this.createData();

      this.events = eventsData;
      this.trainers = trainersData;
      this.workouts = workoutData;

      this.weekDays = moment.weekdays();
      this.months = moment.months();
      this.days = this.constructor.createDays();
      this.hours = this.constructor.createHours();
      this.minutes = this.constructor.createMinutes();
      this.processing = false;

      $scope.$watch(() => this.booking.calendar_event_id, value => {
        if (value !== null) {
          if (typeof this.booking.selectedTrainers === 'undefined') {
            this.booking.selectedTrainers = [];
          }
          const idx = this.events.findIndex(i => i.id === parseInt(value, 10));
          if (idx !== -1) {
            if (!this.booking.selectedTrainers.includes(this.events[idx].event_owner.id.toString())) {
              this.booking.selectedTrainers = [this.events[idx].event_owner.id.toString()];
              // this.booking.selectedTrainers.push(this.events[idx].event_owner.id.toString());
            }
          }
        }
      });

      $timeout(() => {
        this.initMap();
      });
    }

    /**
     * @returns {number[]}
     */
    static createDays() {
      return [...Array(31).keys()].map(i => i + 1);
    }

    /**
     * Creates an array of the hours of the day
     *
     * @returns {string[]}
     */
    static createHours() {
      return [...Array(24).keys()].map(i => i < 10 ? `0${i}` : `${i}`);
    }

    /**
     * @returns {string[]}
     */
    static createMinutes() {
      return [...Array(60).keys()].map(i => i < 10 ? `0${i}` : `${i}`);
    }

    createData() {
      this.booking = {
        recurring: false,
        time: {
          hour: '01',
          minute: '00',
          section: 'PM'
        },
        customCron: {
          day_of_week: [],
          day_of_month: [],
          month: []
        },
        calendar_event_id: null,
        selectedTrainers: [],
        cron: 'weekly',
        selectedWorkouts: null,
        date_time: null,
        location: null
      };
      this.time = {
        minute: '00',
        hour: '00'
      };
      this.getDefaultLocation();
    }

    getDefaultLocation() {
      const geocoder = new google.maps.Geocoder();
      return geocoder.geocode({
        address: this.fullAddress
      }, (results, status) => {
        if (status === google.maps.GeocoderStatus.OK) {
          this.latitude = results[0].geometry.location.lat();
          this.longitude = results[0].geometry.location.lng();
        }

        this.booking.location = {
          name: this.fullAddress,
          location: {
            lat: this.latitude,
            lng: this.longitude
          }
        };
      });
    }

    /**
     * Creates the booking based on this.booking
     */
    book() {
      this.processing = true;
      this.booking.date_time = this.convertDate(this.booking.date, this.time);
      /* end_date is the actual date entity, endDate is the boolean for if the booking has an end date. */
      if (this.end_date && this.endDate) {
        this.booking.end_date = moment(this.end_date).format('YYYY-MM-DD');
      }
      if (!this.addWorkout) {
        this.booking.selectedWorkouts = null;
      }

      this.ApiClientService.calendarEventTimes.create(this.booking).then(() => {
        this.ToastService.show('Event booked');
        this.createData();
        this.$state.go('app.trainer_calendar');
        this.processing = false;
      }).catch(err => {
        this.ToastService.show('Unable to create booking.');
        console.error('Unable to create booking', err);
        this.processing = false;
      });
    }

    /**
     * @param date {string}
     * @param time {object}
     * @returns {*} moment
     */
    convertDate(date, time) {
      date = moment(date).format('YYYY-MM-DD');
      const hours = parseInt(time.hour, 10);
      const minutes = parseInt(time.minute, 10);
      return moment.tz(date, this.currentUser.timezone).hours(hours).minutes(minutes).format('YYYY-MM-DD HH:mm');
    }

    initMap() {
      let map = null;
      const geocoder = new google.maps.Geocoder();
      geocoder.geocode({
        address: this.fullAddress
      }, (results, status) => {
        if (status === google.maps.GeocoderStatus.OK) {
          this.latitude = results[0].geometry.location.lat();
          this.longitude = results[0].geometry.location.lng();

          map = new google.maps.Map(document.getElementById('GoogleMap'), {
            center: {
              lat: this.latitude || null,
              lng: this.longitude || null
            },
            zoom: 15,
            disableDefaultUI: true
          });

          google.maps.event.addListenerOnce(map, 'idle', () => {
            $('.pac-container').attr('data-tap-disabled', 'true');
          });

          const input = document.getElementById('pac-input');

          const searchBox = new google.maps.places.SearchBox(input);

          map.controls[google.maps.ControlPosition.TOP_LEFT].push(input);

          // Bias the SearchBox results towards current map's viewport.
          map.addListener('bounds_changed', () => {
            searchBox.setBounds(map.getBounds());
          });

          const location = new google.maps.LatLng(this.latitude, this.longitude);
          new google.maps.Marker({
            position: location,
            map
          });

          let markers = [];
          // Listen for the event fired when the user selects a prediction and retrieve
          // more details for that place.
          searchBox.addListener('places_changed', () => {
            const places = searchBox.getPlaces();

            if (places.length == 0) {
              return;
            }

            // Clear out the old markers.
            markers.forEach(marker => {
              marker.setMap(null);
            });
            markers = [];

            // For each place, get the icon, name and location.
            const bounds = new google.maps.LatLngBounds();
            places.forEach(place => {
              if (!place.geometry) {
                console.log('Returned place contains no geometry');
                return;
              }

              this.location = place.formatted_address;

              this.booking.location = {
                name: place.formatted_address,
                location: place.geometry.location
              };

              console.log(this.booking.location);
              console.log(this.booking);

              // Create a marker for each place.
              markers.push(new google.maps.Marker({
                map,
                title: place.name,
                position: place.geometry.location
              }));

              if (place.geometry.viewport) {
                // Only geocodes have viewport.
                bounds.union(place.geometry.viewport);
              } else {
                bounds.extend(place.geometry.location);
              }
            });
            map.fitBounds(bounds);
          });
        }
      });
    }
  }

  /**
   * @param $stateProvider
   */
  const config = $stateProvider => {
    $stateProvider.state('app.trainer_book_event', {
      resolve: {
        eventsData(ApiClientService) {
          return ApiClientService.calendarEvents.getAllFrom().then(resp => resp.events);
        },
        trainersData(ApiClientService) {
          return ApiClientService.trainerUsers.list().then(resp => resp.trainerUsers);
        },
        workoutData(ApiClientService) {
          return ApiClientService.trainers.getAllWorkouts().then(resp => resp.workouts);
        }
      },
      url: '/book_event',
      views: {
        menuContent: {
          templateUrl: 'common/event/views/book.html',
          controller: 'TrainerEventBookingController',
          controllerAs: 'vm'
        }
      }
    });
  };

  angular.module('MyPTHub').config(config).controller('TrainerEventBookingController', TrainerEventBookingController);
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('ExerciseViewerService', service).controller('ExerciseViewerController', controller);

    controller.$inject = ['ApiClientService', '$sce', '$filter', 'mpthconf', 'NetworkStateService', 'ImageCacheService', 'parameters', 'AnalyticsService'];

    function controller(ApiClientService, $sce, $filter, mpthconf, NetworkStateService, ImageCacheService, parameters, AnalyticsService) {

        var nameId = parameters.nameId;

        var gender = 'male';
        if (parameters.gender) {
            gender = parameters.gender;
        }

        var vm = this;

        vm.ready = false;
        vm.video = {
            type: null,
            url: null,
            present: false
        };

        vm.playerVisible = false;
        vm.showPlayer = function () {
            vm.playerVisible = true;
        };

        var setup = function () {

            if (vm.exercise.hasOwnProperty('aliases') && vm.exercise.aliases.length > 0) {

                if (!nameId) {
                    nameId = vm.exercise.default_name_id;
                }

                var name = $filter('filter')(vm.exercise.aliases, {
                    id: nameId
                }, true);

                if (name.length > 0) {
                    vm.exercise.name = name[0].name;
                }
            }

            vm.videoUrl = false;
            if (vm.exercise.video_id || vm.exercise.video_id_f) {

                var useId = vm.exercise.video_id || vm.exercise.video_id_f;
                if (useId) {
                    var videoType = vm.exercise.video_type;
                }

                if (gender === 'female' && vm.exercise.video_id_f) {
                    useId = vm.exercise.video_id_f;
                    var videoType = vm.exercise.video_type_f;
                }

                if (NetworkStateService.isOnline() && !vm.video.present) {

                    if (videoType == 'vimeo') {
                        vm.video.url = $sce.trustAsResourceUrl('https://player.vimeo.com/video/' + useId + '?title=0&byline=0&portrait=0&badge=0');
                    } else {
                        vm.video.url = $sce.trustAsResourceUrl('https://www.youtube-nocookie.com/embed/' + useId + '?rel=0&controls=0&showinfo=0&autoplay=true');
                    }

                    vm.video.type = 'live';
                    vm.video.present = true;
                }
            }

            vm.images = [];
            if (vm.exercise.images && vm.exercise.images.length > 0) {
                angular.forEach(vm.exercise.images, function (image) {

                    var file = mpthconf.cdn.base + 'exercise/l/' + image.filename;

                    ImageCacheService.get(file).then(function (result) {
                        vm.images.push({
                            imgUrl: $sce.trustAsResourceUrl(result)
                        });
                    });
                });
            }

            vm.description = $sce.trustAsHtml(vm.exercise.description);
        };

        if (typeof parameters.exercise === 'object') {

            vm.exercise = parameters.exercise;

            setup();
        } else {

            var getExercise = ApiClientService.exercises.get(parameters.exercise);

            getExercise.success(function (response) {

                vm.exercise = response.exercise;

                setup();
            });
        }

        AnalyticsService.trackView('View Exercise');
    }

    service.$inject = ['ModalService'];

    function service(ModalService) {

        var service = {};

        service.open = function (exercise, nameId, gender) {

            nameId = nameId || false;
            gender = gender || false;

            ModalService.show({
                templateUrl: "common/exerciseViewer/exercise.html",
                controller: 'ExerciseViewerController as vm',
                params: {
                    exercise: exercise,
                    nameId: nameId,
                    gender: gender
                }
            });
        };

        return service;
    }
})();
(() => {
  angular.module('MyPTHub').config(config).controller('FilesController', controller).controller('ClientFilesController', clientController);

  config.$inject = ['$stateProvider'];

  function config($stateProvider) {
    $stateProvider.state('app.files', {
      url: '/files',
      views: {
        'menuContent': {
          templateUrl: 'common/files/files.html',
          controller: 'FilesController',
          controllerAs: 'vm'
        }
      }
    });

    $stateProvider.state('app.client_files', {
      url: '/client_files',
      views: {
        'menuContent': {
          templateUrl: 'common/files/files.html',
          controller: 'ClientFilesController',
          controllerAs: 'vm'
        }
      }
    });
  }

  controller.$inject = ['ApiClientService', '$scope', '$filter', 'ModalService', 'AuthenticationStateService'];

  function controller(ApiClientService, $scope, $filter, ModalService, AuthenticationStateService) {
    const vm = this;
    const mimes = {
      image: ['image/gif', 'image/x-icon', 'image/jpeg', 'image/jpg', 'image/png', 'image/svg+xml', 'image/tiff', 'image/webp'],
      document: ['application/msword', 'application/vnd.oasis.opendocument.text', 'application/rtf', 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'],
      pdf: ['application/pdf'],
      archive: ['application/x-tar', 'application/x-bzip2', 'application/gzip', 'application/x-7z-compressed', 'application/x-apple-diskimage', 'application/x-rar-compressed', 'application/zip'],
      spreadsheet: ['application/vnd.oasis.opendocument.spreadsheet', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet', 'application/vnd.ms-excel']
    };
    vm.loading = true;
    vm.trainer = true;
    vm.currentUser = AuthenticationStateService.getCurrentUser();

    const loadFiles = refreshingParam => {
      ApiClientService.trainers.get(vm.currentUser.trainer_id).success(resp => {
        vm.trainer = resp.trainer;
        vm.storageUsed = parseInt(resp.trainer.storage_used);
      });
      const refreshing = refreshingParam || false;
      vm.loading = true;

      ApiClientService.files.listAll().success(resp => {
        let files = resp.files;
        files.map(file => {
          if (mimes.image.includes(file.mime)) {
            file.icon = 'fa-file-image-o';
          } else if (mimes.spreadsheet.includes(file.mime)) {
            file.icon = 'fa-file-excel-o';
          } else if (mimes.pdf.includes(file.mime)) {
            file.icon = 'fa-file-pdf-o';
          } else if (mimes.archive.includes(file.mime)) {
            file.icon = 'fa-file-archive-o';
          } else if (mimes.document.includes(file.mime)) {
            file.icon = 'fa-file-text-o';
          } else {
            file.icon = 'fa-file-o';
          }
          file.name = `${file.original_name}.${file.extension}`;
          file.own = true;
        });

        vm.files = files;
        vm.filteredFiles = angular.copy(vm.files);
        vm.noFiles = files.length === 0;
        vm.loading = false;
      }).finally(() => {
        return refreshing ? $scope.$broadcast('scroll.refreshComplete') : true;
      });
    };

    vm.viewFile = function (file) {
      ModalService.show({
        templateUrl: 'common/files/filesViewModal.html',
        controller: 'FilesViewController as vm',
        params: {
          file,
          trainer: vm.trainer
        }
      }).then(res => {
        if (res) {
          vm.refresh();
        }
      });
    };

    vm.refresh = function () {
      loadFiles(true);
    };

    $scope.$watch(() => vm.searchFilter, val => {
      if (typeof val === 'undefined') {
        return false;
      }

      if (val.length > 0) {
        vm.searchFilterActive = true;
        let searchQuery = val.toLowerCase();

        vm.filteredFiles = $filter('filter')(vm.files, file => file.name.toLowerCase().includes(searchQuery));
      } else {
        vm.searchFilterActive = false;
        vm.filteredFiles = vm.files;
      }
    });

    $scope.$on('$ionicView.beforeEnter', () => loadFiles());
  }

  clientController.$inject = ['AuthenticationStateService', 'ApiClientService', '$scope', 'ModalService'];

  function clientController(AuthenticationStateService, ApiClientService, $scope, ModalService) {
    const vm = this;
    const mimes = {
      image: ['image/gif', 'image/x-icon', 'image/jpeg', 'image/jpg', 'image/png', 'image/svg+xml', 'image/tiff', 'image/webp'],
      document: ['application/msword', 'application/vnd.oasis.opendocument.text', 'application/rtf', 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'],
      pdf: ['application/pdf'],
      archive: ['application/x-tar', 'application/x-bzip2', 'application/gzip', 'application/x-7z-compressed', 'application/x-apple-diskimage', 'application/x-rar-compressed', 'application/zip'],
      spreadsheet: ['application/vnd.oasis.opendocument.spreadsheet', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet', 'application/vnd.ms-excel']
    };
    vm.trainer = false;
    vm.selectedTabIndex = 0;
    vm.currentUser = AuthenticationStateService.getCurrentUser();

    const loadFiles = (refreshingParam, assignedParam) => {
      const refreshing = refreshingParam || false;
      const assignedFiles = assignedParam || false;
      vm.loading = true;

      ApiClientService.customers.get(vm.currentUser.customer_id).success(resp => {
        console.log(resp);
        vm.customer = resp.customer;

        let files;

        if (assignedFiles) {
          files = resp.customer.files;
        } else {
          files = resp.customer.user.uploaded_files;
        }

        files.map(file => {
          if (mimes.image.includes(file.mime)) {
            file.icon = 'fa-file-image-o';
          } else if (mimes.spreadsheet.includes(file.mime)) {
            file.icon = 'fa-file-excel-o';
          } else if (mimes.pdf.includes(file.mime)) {
            file.icon = 'fa-file-pdf-o';
          } else if (mimes.archive.includes(file.mime)) {
            file.icon = 'fa-file-archive-o';
          } else if (mimes.document.includes(file.mime)) {
            file.icon = 'fa-file-text-o';
          } else {
            file.icon = 'fa-file-o';
          }
          file.name = `${file.original_name}.${file.extension}`;
          file.own = !assignedFiles;
        });

        vm.files = files;
        vm.filteredFiles = angular.copy(vm.files);
        vm.noFiles = files.length === 0;
        vm.loading = false;
      }).finally(() => {
        return refreshing ? $scope.$broadcast('scroll.refreshComplete') : true;
      });
    };

    vm.refresh = function () {
      if (vm.selectedTabIndex === 1) {
        loadFiles(true, true);
      } else {
        loadFiles(true);
      }
    };

    vm.viewFile = function (file) {
      ModalService.show({
        templateUrl: 'common/files/filesViewModal.html',
        controller: 'FilesViewController as vm',
        params: {
          file,
          trainer: vm.trainer
        }
      }).finally(() => {
        vm.refresh();
      });
    };

    $scope.$on('$ionicView.beforeEnter', () => loadFiles());

    $scope.$watch(() => {
      return vm.selectedTabIndex;
    }, val => {
      vm.refresh(true, val === 1);
    });
  }
})();
(() => {
  angular.module('MyPTHub').controller('FilesViewController', controller).config($sceDelegateProvider => {
    $sceDelegateProvider.resourceUrlWhitelist(['self', "https://d3qcjjq30235ix.cloudfront.net/**"]);
  });

  controller.$inject = ['parameters', '$cordovaFileTransfer', '$cordovaFileOpener2', 'mpthconf', '$ionicPlatform', 'PromptDialogService', 'ApiClientService', 'ConfirmDialogService', '$scope'];

  function controller(parameters, $cordovaFileTransfer, $cordovaFileOpener2, mpthconf, $ionicPlatform, PromptDialogService, ApiClientService, ConfirmDialogService, $scope) {
    const vm = this;
    vm.play = false;

    const mimes = {
      image: ['image/gif', 'image/x-icon', 'image/jpeg', 'image/jpg', 'image/png', 'image/svg+xml', 'image/tiff', 'image/webp'],
      document: ['application/msword', 'application/vnd.oasis.opendocument.text', 'application/rtf', 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'],
      pdf: ['application/pdf'],
      archive: ['application/x-tar', 'application/x-bzip2', 'application/gzip', 'application/x-7z-compressed', 'application/x-apple-diskimage', 'application/x-rar-compressed', 'application/zip'],
      spreadsheet: ['application/vnd.oasis.opendocument.spreadsheet', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet', 'application/vnd.ms-excel']
    };
    vm.file = parameters.file;

    if (!vm.file.hasOwnProperty('name')) {
      // Give the file a name and mime etc....

      if (mimes.image.includes(vm.file.mime)) {
        vm.file.icon = 'fa-file-image-o';
      } else if (mimes.spreadsheet.includes(vm.file.mime)) {
        vm.file.icon = 'fa-file-excel-o';
      } else if (mimes.pdf.includes(vm.file.mime)) {
        vm.file.icon = 'fa-file-pdf-o';
      } else if (mimes.archive.includes(vm.file.mime)) {
        vm.file.icon = 'fa-file-archive-o';
      } else if (mimes.document.includes(vm.file.mime)) {
        vm.file.icon = 'fa-file-text-o';
      } else {
        vm.file.icon = 'fa-file-o';
      }
      vm.file.name = `${vm.file.original_name}.${vm.file.extension}`;
    }
    vm.trainer = parameters.trainer;

    vm.downloadFile = file => {
      document.addEventListener('deviceready', onDeviceReady, false);

      function onDeviceReady() {}

      const url = `${mpthconf.cdn.base}file_uploads/${file.trainer_id}/${file.filename}`;
      //const targetPath = `${cordova.file.documentsDirectory}/${file.name}`;
      const targetPath = `${cordova.file.dataDirectory}${file.name}`;

      $cordovaFileTransfer.download(url, targetPath, {}, true).then(res => {
        $ionicPlatform.ready(() => {
          $cordovaFileOpener2.open(decodeURIComponent(res.nativeURL), file.mime).then(function (a) {
            console.log('a', a, JSON.stringify(a));
            // file opened successfully
          }, function (err) {
            console.log('err', err, JSON.stringify(err));
            // An error occurred. Show a message to the user
          });
        });
      }, err => {
        console.log(['err', err]);
      }, prog => {}, false);
    };

    vm.renameFile = file => {
      PromptDialogService.open({
        title: 'Rename File',
        message: 'Enter New File Name',
        buttonConfirm: 'Save',
        buttonCancel: 'Cancel',
        required: true,
        defaultText: vm.file.original_name
      }).then(name => {
        const data = {
          original_name: name
        };
        ApiClientService.files.update(file.id, data).success(() => {
          vm.file.name = `${name}.${vm.file.extension}`;
          file.name = `${name}.${vm.file.extension}`;
          vm.file.original_name = name;
          file.original_name = name;
        });
      });
    };

    vm.videoPath = `${mpthconf.cdn.base}file_uploads/${vm.file.trainer_id}/${vm.file.filename}`;

    vm.deleteFile = file => {
      ConfirmDialogService.open({
        title: 'Confirm delete',
        message: 'Do you want to remove this file?',
        buttonConfirm: 'Confirm',
        buttonCancel: 'Cancel'
      }).then(() => {
        ApiClientService.files.delete(file.id).success(() => {
          $scope.closeModal(true);
        });
      });
    };
  }
})();
(() => {
  angular.module('MyPTHub').component('fitbitClientDashboardGoals', {
    templateUrl: 'common/fitbitClientDashboard/fitbitClientDashboard.html',
    controller,
    controllerAs: 'vm',
    bindings: {
      customerId: '='
    },
    resolves: []
  });

  function controller(SubscriptionStateService, ApiClientService, UnitService) {
    const vm = this;
    const customer = vm.customerId;

    vm.loadingGoals = true;
    vm.progressDay = moment();
    vm.today = moment();
    vm.dayOffset = 0;
    vm.activities;
    vm.caloriesGoal;
    vm.caloriesNow;
    vm.caloriesComplete = false;
    vm.stepsGoal;
    vm.stepsNow;
    vm.stepsComplete = false;
    vm.distanceGoal;
    vm.distanceNow;
    vm.distanceComplete = false;
    vm.distanceUnit = UnitService.unitLabel('distance');
    vm.floorsGoal;
    vm.floorsNow;
    vm.floorComplete = false;

    const getDayGoals = () => {
      ApiClientService.analytics.activities(customer, vm.progressDay.format('YYYY-MM-DD')).success(function (response) {
        vm.loadingGoals = false;
        vm.activities = response.data;
        if (vm.activities) {
          vm.caloriesGoal = response.data.goals.caloriesOut;
          vm.caloriesNow = response.data.summary.caloriesOut;
          vm.caloriesComplete = vm.caloriesGoal <= vm.caloriesNow ? true : false;
          vm.stepsGoal = response.data.goals.steps;
          vm.stepsNow = response.data.summary.steps;
          vm.stepsComplete = vm.stepsGoal <= vm.stepsNow ? true : false;
          vm.floorsGoal = response.data.goals.floors;
          vm.floorsNow = response.data.summary.floors;
          vm.floorsComplete = vm.floorsGoal <= vm.floorsNow ? true : false;
          vm.distanceGoal = UnitService.unitDisplay('distance', response.data.goals.distance);
          vm.distanceNow = UnitService.unitDisplay('distance', response.data.summary.distances[0].distance);
          vm.distanceComplete = vm.distanceGoal <= vm.distanceNow ? true : false;
        }
      });
    };

    getDayGoals();

    vm.nextDay = () => {
      if (vm.dayOffset > 0) {
        vm.loadingGoals = true;
        vm.dayOffset--;

        vm.progressDay = vm.progressDay.add(1, 'd');

        getDayGoals();
      }
    };

    vm.previousDay = () => {
      if (vm.dayOffset < 7) {
        vm.loadingGoals = true;

        vm.dayOffset++;

        vm.progressDay = vm.progressDay.subtract(1, 'd');

        getDayGoals();
      }
    };
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('CreateNutritionController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.create_nutrition', {
            url: "/nutrition/create-nutrition/:barcode",
            views: {
                'menuContent': {
                    templateUrl: "common/foodManage/foodForm.html",
                    controller: 'CreateNutritionController as vm'
                }
            }
        });
    }

    controller.$inject = ['ApiClientService', 'NavigateService', 'FoodViewerService', '$stateParams', '$scope', 'AnalyticsService'];

    function controller(ApiClientService, NavigateService, FoodViewerService, $stateParams, $scope, AnalyticsService) {

        var vm = this;

        vm.barcode = $stateParams.barcode || false;

        vm.mode = 'creating';
        vm.nutritionItemSuccess = false;
        vm.nutritionItemError = false;
        vm.nutritionItemSaving = false;
        vm.nutritionItem = {};
        vm.measurementSizes = [{ name: "cup", display: "Cup" }, { name: "centilitre", display: "Centilitre" }, { name: "fl", display: "Fl" }, { name: "gram", display: "Gram" }, { name: "gallon", display: "Gallon" }, { name: "kg", display: "Kg" }, { name: "lb", display: "Lb" }, { name: "litre", display: "Litre" }, { name: "ml", display: "Ml" }, { name: "mg", display: "Mg" }, { name: "oz", display: "Oz" }, { name: "pound", display: "Pound" }, { name: "pint", display: "Pint" }, { name: "quart", display: "Quart" }, { name: "tablespoon", display: "Tablespoon" }, { name: "teaspoon", display: "Teaspoon" }];

        vm.submitNutrition = function () {

            vm.nutritionItemSaving = true;

            // Inject the barcode
            if (vm.barcode) {
                vm.nutritionItem.barcode = vm.barcode;
            }

            var newNutritionItem = ApiClientService.nutrition.addNutritionItem(vm.nutritionItem);

            newNutritionItem.success(function (response) {

                var getFood = ApiClientService.nutrition.getFood(response.nutritionItem.id);

                getFood.success(function (response) {

                    vm.nutritionItemSuccess = true;
                    vm.nutritionItemError = false;
                    vm.nutritionItemSaving = false;

                    var food = response.food;

                    FoodViewerService.show(food, null, true).then(function () {

                        NavigateService.goBack(-2);
                    });
                });
            });
        };

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Food Create');
        });
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('EditNutritionController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.edit_nutrition', {

            resolve: {
                getNutritionItem: ['ApiClientService', '$stateParams', function (ApiClientService, $stateParams) {
                    return ApiClientService.nutrition.getCustomNutritionItem($stateParams.foodId);
                }]
            },

            url: "/nutrition/edit-nutrition/{foodId:int}",
            views: {
                'menuContent': {
                    templateUrl: "common/foodManage/foodForm.html",
                    controller: 'EditNutritionController as vm'
                }
            }
        });
    }

    controller.$inject = ['ApiClientService', 'NavigateService', '$mdToast', 'getNutritionItem', '$scope', 'AnalyticsService'];

    function controller(ApiClientService, NavigateService, $mdToast, getNutritionItem, $scope, AnalyticsService) {

        var vm = this;

        vm.mode = 'editing';
        vm.nutritionItemSuccess = false;
        vm.nutritionItemError = false;
        vm.nutritionItemSaving = false;
        vm.nutritionItem = getNutritionItem.foodItem;
        vm.measurementSizes = [{ name: "cup", display: "Cup" }, { name: "centilitre", display: "Centilitre" }, { name: "fl", display: "Fl" }, { name: "gram", display: "Gram" }, { name: "gallon", display: "Gallon" }, { name: "kg", display: "Kg" }, { name: "lb", display: "Lb" }, { name: "litre", display: "Litre" }, { name: "ml", display: "Ml" }, { name: "mg", display: "Mg" }, { name: "oz", display: "Oz" }, { name: "pound", display: "Pound" }, { name: "pint", display: "Pint" }, { name: "quart", display: "Quart" }, { name: "tablespoon", display: "Tablespoon" }, { name: "teaspoon", display: "Teaspoon" }];

        vm.submitNutrition = function () {

            vm.nutritionItemSaving = true;

            var editNutritionItem = ApiClientService.nutrition.updateNutritionItem(vm.nutritionItem.id, vm.nutritionItem);

            editNutritionItem.success(function (response) {

                vm.nutritionItemSuccess = true;
                vm.nutritionItemError = false;
                vm.nutritionItemSaving = false;

                $mdToast.show($mdToast.simple().textContent('Nutrition Item updated successfully').position('top right'));

                NavigateService.goBack();
            });
        };

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Food Edit');
        });
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('FoodSearchController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.food_search', {
            url: "/nutrition/foodsearch",
            views: {
                'menuContent': {
                    templateUrl: "common/foodSearch/search.html",
                    controller: 'FoodSearchController as vm'
                }
            }
        });
    }

    controller.$inject = ['ApiClientService', '$timeout', 'FoodViewerService', 'NavigateService', '$cordovaBarcodeScanner', '$mdDialog', '$scope', 'DeviceInfoService', 'AnalyticsService', 'ToastService'];

    function controller(ApiClientService, $timeout, FoodViewerService, NavigateService, $cordovaBarcodeScanner, $mdDialog, $scope, DeviceInfoService, AnalyticsService, ToastService) {
        var vm = this;

        // temp horrible hack to work around issue of disappearing tabs
        // in some weird circumstances.
        $scope.$on('$ionicView.afterEnter', function () {
            $(window).trigger('resize');
        });

        var fetchDataDelay = 500; // milliseconds
        var fetchDataTimer;

        // Set default mode
        vm.mode = 'recent';
        vm.recentFood = [];
        vm.frequentFood = [];
        vm.personalFood = [];

        vm.recentFoodNoResults = false;
        vm.frequentFoodNoResults = false;
        vm.personalFoodNoResults = false;
        vm.foodPage = 1;
        vm.foodSearch = null;

        // Personal Initial Load
        vm.searching = true;

        // Cache types of food
        vm.getRecentFrequentPersonal = function (page, search) {
            var foodCache = ApiClientService.nutrition.getRecentFrequentPersonal({
                paginate: true,
                page,
                search
            });
            foodCache.success(function (response) {
                vm.recentFood = response.recent;
                vm.frequentFood = response.frequent;
                vm.personalFood = response.personal.data;

                if (!vm.recentFood.length) {
                    vm.recentFoodNoResults = true;
                }

                if (!vm.frequentFood.length) {
                    vm.frequentFoodNoResults = true;
                }

                if (!vm.personalFood.length) {
                    vm.personalFoodNoResults = true;
                }

                vm.searching = false;
            });
        };

        $scope.$on("$ionicView.enter", function () {

            // Simulate Reload
            vm.searching = true;
            vm.personalFood = [];
            vm.getRecentFrequentPersonal(vm.foodPage, vm.foodSearch);

            AnalyticsService.trackView('Food Search');
        });

        vm.searchFood = function (query) {

            if (!query || query === 0) {
                return 0;
            }

            vm.searchResults = [];
            vm.customSearchResults = [];
            vm.mode = 'normal';
            vm.searching = true;
            vm.noResults = false;

            $timeout.cancel(fetchDataTimer);

            fetchDataTimer = $timeout(function () {

                var searchFood = ApiClientService.nutrition.searchFood(query);

                searchFood.success(function (response) {

                    if (response.food && response.food.length > 0) {

                        vm.searchResults = response.food;
                        vm.customSearchResults = response.customFood;

                        vm.noResults = false;
                    } else {

                        vm.noResults = true;
                    }

                    vm.searching = false;
                });
            }, fetchDataDelay);
        };

        vm.useFood = function (result, internal) {
            internal = typeof internal !== 'undefined' ? internal : false;

            if (internal || result.source == 'internal') {
                var getFood = ApiClientService.nutrition.getFood(result.id);
            } else {
                var getFood = ApiClientService.nutrition.getFoodByExternalId(result.external_id);
            }

            getFood.success(function (response) {

                var food = response.food;

                FoodViewerService.show(food, null, true).then(function () {
                    NavigateService.goBack();
                });
            });
        };

        vm.barcodeLookup = function (barcode, event) {
            var barcodeLookupResult = ApiClientService.nutrition.findNutritionByBarcode(barcode);

            // We found a barcode, so lets use that food item
            barcodeLookupResult.success(function (response) {
                if (response.food.source == 'fatsecret') {
                    vm.useFood(response.food);
                } else {
                    vm.useFood(response.food, true);
                }
            });

            // We didn't find a barcode, so ask if the user wants to create a food based off it
            barcodeLookupResult.error(function (response) {
                var confirm = $mdDialog.confirm().title('We couldn\'t find that barcode, would you like to create a new food item for it?').ok('Yes').targetEvent(event).cancel('No, Cancel');

                $mdDialog.show(confirm).then(function () {
                    // Show the create page
                    NavigateService.go('app.create_nutrition', {
                        barcode: barcode
                    });
                });
            });
        };

        vm.scanBarcode = function () {

            var scan = function () {
                $cordovaBarcodeScanner.scan().then(function (imageData) {
                    if (!imageData.cancelled) {
                        vm.barcodeLookup(imageData.text);
                    }
                }, function (error) {
                    ToastService.show('There was an error with the barcode scanner: ' + error);
                });
            };

            if (DeviceInfoService.isAndroid()) {

                var permissions = cordova.plugins.permissions;
                permissions.hasPermission(permissions.CAMERA, function (status) {
                    if (status.hasPermission) {
                        scan();
                        return true;
                    }

                    var errorCallback = function () {
                        console.warn('Camera permission is not turned on');
                        ToastService.show('We could not get permission to use the camera');
                    };

                    permissions.requestPermission(permissions.CAMERA, function (status) {
                        if (!status.hasPermission) {
                            errorCallback();
                            return false;
                        }

                        scan();
                    }, errorCallback);
                }, null);
            } else {

                scan();
            }
        };

        vm.deleteNutritionItem = function (food, event) {
            var confirm = $mdDialog.confirm().title('Are you sure you want to remove ' + food.name + '?').ok('Yes, Remove').targetEvent(event).cancel('No, Cancel');

            $mdDialog.show(confirm).then(function () {

                var doDelete = ApiClientService.nutrition.deleteCustomNutritionItem(food.id);

                doDelete.success(function () {
                    vm.personalFood.splice(vm.personalFood.indexOf(food), 1);
                    ToastService.show('Nutrition Item successfully deleted');
                });

                doDelete.error(function (response) {

                    var error;
                    if (response.error) {
                        error = response.error.message;
                    } else {
                        error = "Unable to delete, please try again";
                    }

                    ToastService.show(error);
                });
            });
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('FoodViewerModalController', controller);

    controller.$inject = ['parameters', '$scope', '$filter', 'NutritionEditorService', 'AnalyticsService'];

    function controller(parameters, $scope, $filter, NutritionEditorService, AnalyticsService) {

        var vm = this;

        var meal = NutritionEditorService.getCurrentMeal();
        var item = NutritionEditorService.getItem();

        vm.addMode = false;
        if (parameters.addMode) {
            vm.addMode = true;
        }

        AnalyticsService.trackView('View Food' + (vm.addMode ? ' (Add mode)' : ''));

        vm.food = parameters.food;

        vm.servingQty = 1;

        if (parameters.defaultServingId) {

            vm.serving = $filter('filter')(vm.food.servings, {
                id: parameters.defaultServingId
            }, true)[0];
        } else {

            vm.serving = vm.food.servings[0];
        }

        var calculateNutrient = function (value, qty) {

            if (value === null) {
                return null;
            }

            return Math.round(value * qty);
        };

        vm.updateServing = function () {

            var qty = parseFloat(vm.servingQty);

            vm.totals = {
                calories: calculateNutrient(vm.serving.calories, qty),
                carbohydrate: calculateNutrient(vm.serving.carbohydrate, qty),
                cholesterol: calculateNutrient(vm.serving.cholesterol, qty),
                fat: calculateNutrient(vm.serving.fat, qty),
                fibre: calculateNutrient(vm.serving.fibre, qty),
                monounsaturated_fat: calculateNutrient(vm.serving.monounsaturated_fat, qty),
                polyunsaturated_fat: calculateNutrient(vm.serving.polyunsaturated_fat, qty),
                potassium: calculateNutrient(vm.serving.potassium, qty),
                protein: calculateNutrient(vm.serving.protein, qty),
                saturated_fat: calculateNutrient(vm.serving.saturated_fat, qty),
                sodium: calculateNutrient(vm.serving.sodium, qty),
                sugar: calculateNutrient(vm.serving.sugar, qty),

                calcium: calculateNutrient(vm.serving.calcium, qty),
                iron: calculateNutrient(vm.serving.iron, qty),
                vitamin_a: calculateNutrient(vm.serving.vitamin_a, qty),
                vitamin_c: calculateNutrient(vm.serving.vitamin_c, qty)
            };
        };

        vm.updateServing();

        vm.addFood = function () {

            vm.food.food_id = vm.food.id;

            var newFood = meal.addFood(vm.food, vm.serving.id, vm.servingQty);

            item.updateTotals();

            $scope.closeModal(true);
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('FoodViewerService', service);

    service.$inject = ['ModalService'];

    function service(ModalService) {

        return {

            show: function (food, servingId, addMode) {

                servingId = servingId || false;
                addMode = addMode || false;

                return ModalService.show({
                    templateUrl: "common/foodViewer/foodViewer.html",
                    controller: 'FoodViewerModalController as vm',
                    params: {
                        food: food,
                        defaultServingId: servingId,
                        addMode: addMode
                    }
                });
            }

        };
    }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').config(config).controller('ViewCustomerFormController', controller);

  config.$inject = ['$stateProvider'];

  function config($stateProvider) {

    $stateProvider.state('app.view_customer_form', {
      url: "/clients/:customerId/form/:formId",
      views: {
        'menuContent': {
          templateUrl: "common/forms/customerForm.html",
          controller: 'ViewCustomerFormController as vm'
        }
      },
      resolve: {
        form($stateParams, ApiClientService) {
          return ApiClientService.forms.get($stateParams.formId).then(response => response.form);
        },
        formQuestions(ApiClientService, $stateParams) {
          return ApiClientService.form_questions.getAll($stateParams.formId, $stateParams.customerId).then(response => response.questions);
        }
      },
      params: {
        customerId: null,
        formId: null,
        readonly: false
      }
    });
  }

  controller.$inject = ['$scope', 'AnalyticsService', 'ApiClientService', '$stateParams', '$q', 'ToastService', 'NavigateService', 'MarkdownService', '$sce', 'ModalService', 'form', 'formQuestions'];

  function controller($scope, AnalyticsService, ApiClientService, $stateParams, $q, ToastService, NavigateService, MarkdownService, $sce, ModalService, form, formQuestions) {
    var vm = this;

    $scope.forms = {};

    vm.triggerSubmit = function () {
      $scope.forms.questionForm.commit();
    };

    vm.save = function () {
      if (vm.ro) {
        return false;
      }

      if (!$scope.forms.questionForm.$valid) {
        ToastService.show('Please enter answers for all questions marked *');
        return false;
      }

      vm.saving = true;

      var answers = {
        create: [],
        update: []
      };

      for (var i = 0; i < vm.questions.length; i++) {
        var question = vm.questions[i];

        if (question.type === 8) {
          question.answer.value = question.answer.getSignatureValue();
        }

        if (question.answer.value === 0) {
          question.answer.value = '0';
        }
        answers[question.answer.id ? 'update' : 'create'].push({
          id: question.answer.id || false,
          customer_id: $stateParams.customerId,
          form_id: vm.form.id,
          question_id: question.id,
          value: question.answer.value || '',
          additional_value: question.answer.additional_value || ''
        });
      }

      var saveRequests = [];

      if (answers.create.length > 0) {
        saveRequests.push(ApiClientService.form_question_answers.createBatch(answers.create));
      }

      if (answers.update.length > 0) {
        saveRequests.push(ApiClientService.form_question_answers.updateBatch(answers.update));
      }

      $q.all(saveRequests).then(function (response) {
        ToastService.show('Form answers saved successfully');
        NavigateService.go('app.client_forms', {
          reload: true
        });
      }, function () {
        ToastService.show('Could not save - please try again');
      }).finally(function () {
        vm.saving = false;
      });
    };

    $scope.$on('$ionicView.beforeEnter', function () {
      vm.ready = false;
      vm.canSave = false;
      vm.readonly = form.editable === 1 ? false : $stateParams.readonly;

      vm.questions = [];
      vm.answered = false;
      vm.ro = false;

      const processForm = form => {
        vm.form = form;
        const mimes = {
          image: ['image/gif', 'image/x-icon', 'image/jpeg', 'image/jpg', 'image/png', 'image/svg+xml', 'image/tiff', 'image/webp'],
          document: ['application/msword', 'application/vnd.oasis.opendocument.text', 'application/rtf', 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'],
          pdf: ['application/pdf'],
          archive: ['application/x-tar', 'application/x-bzip2', 'application/gzip', 'application/x-7z-compressed', 'application/x-apple-diskimage', 'application/x-rar-compressed', 'application/zip'],
          spreadsheet: ['application/vnd.oasis.opendocument.spreadsheet', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet', 'application/vnd.ms-excel']
        };

        vm.files = form.files.map(file => {
          if (mimes.image.includes(file.mime)) {
            file.icon = 'fa-file-image-o';
          } else if (mimes.spreadsheet.includes(file.mime)) {
            file.icon = 'fa-file-excel-o';
          } else if (mimes.pdf.includes(file.mime)) {
            file.icon = 'fa-file-pdf-o';
          } else if (mimes.archive.includes(file.mime)) {
            file.icon = 'fa-file-archive-o';
          } else if (mimes.document.includes(file.mime)) {
            file.icon = 'fa-file-text-o';
          } else {
            file.icon = 'fa-file-o';
          }
          file.name = `${file.original_name}.${file.extension}`;
          return file;
        });

        vm.form.description = $sce.trustAsHtml(MarkdownService.markdownToHtml(vm.form.description) || '');
      };

      const processFormQuestions = formQuestions => {
        vm.questions = formQuestions;
        vm.noQuestions = !formQuestions.length;
      };

      processForm(form);
      processFormQuestions(formQuestions);

      if (vm.questions && vm.questions.length) {
        for (var i = 0; i < vm.questions.length; i++) {
          if (vm.questions[i].answer) {
            var value = vm.questions[i].answer.value;
            if (!isNaN(value)) {
              vm.questions[i].answer.value = parseInt(value);
            }
            vm.answered = true;
          } else {
            vm.questions[i].answer = {
              value: ''
            };
          }
        }

        if (vm.answered) {
          if (!vm.form.editable) {
            vm.ro = 'aasdasd';
            vm.canSave = false;
          } else {
            vm.canSave = true;
          }
        } else {
          vm.canSave = true;
        }
      }

      vm.ready = true;
    });

    $scope.$on('$ionicView.enter', function () {
      AnalyticsService.trackView('View Customer Form');
    });

    vm.viewFile = function (file) {
      ModalService.show({
        templateUrl: 'common/files/filesViewModal.html',
        controller: 'FilesViewController as vm',
        params: {
          file,
          trainer: false
        }
      });
    };
  }
})();
// Workouts Controller
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('LoginForgotPasswordController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.login_forgot_password', {
            url: "/login/forgot_password",
            params: {
                isAnimated: false,
                isLogin: true
            },
            views: {
                'menuContent': {
                    templateUrl: "common/login/forgotPassword.html",
                    controller: 'LoginForgotPasswordController as vm'
                }
            }
        });
    }

    controller.$inject = ['$scope', 'ApiClientService', 'ToastService'];

    function controller($scope, ApiClientService, ToastService) {

        var vm = this;

        vm.doRequest = function () {

            vm.processing = true;

            var request = ApiClientService.users.requestPasswordReset(vm.email);

            request.success(function () {

                vm.processing = false;
                vm.done = true;
            });

            request.error(function (response) {

                var error;

                if (response.error) {
                    error = response.error.message;
                } else {
                    error = 'Unable to reset password, please try again.';
                }

                ToastService.show(error);

                vm.processing = false;
            });
        };
    }
})();
// Workouts Controller
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('LoginController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.login', {
            url: "/login",
            cache: false,
            params: {
                isAnimated: false,
                isLogin: true
            },
            views: {
                'menuContent': {
                    templateUrl: "common/login/login.html",
                    controller: 'LoginController as vm'
                }
            }
        });
    }

    controller.$inject = ['AuthenticationService', 'PermissionsService', 'ToastService', '$scope', 'NavigateService', 'IntroService', 'BrandingService', 'AnalyticsService', 'baseBrand', 'LogoService', 'AuthenticationStateService'];

    function controller(AuthenticationService, PermissionsService, ToastService, $scope, NavigateService, IntroService, BrandingService, AnalyticsService, baseBrand, LogoService, AuthenticationStateService) {
        var vm = this;

        vm.customBrandEnabled = baseBrand ? true : false;
        vm.businessName = baseBrand ? baseBrand.business_name : 'Welcome';

        if (vm.customBrandEnabled) {

            // Sets logo if it is loaded after view rendered
            LogoService.registerLogoChangedHandler($scope, function (event, logo) {
                vm.logo = logo;
            });
        }

        $scope.$on('$ionicView.enter', function () {

            vm.loading = false;

            AnalyticsService.trackView('Login');

            // Sets logo if loaded before view rendered
            if (vm.customBrandEnabled && LogoService.customLogoIsSet()) {
                vm.logo = LogoService.getLogo();
            }
        });

        var successGo = function () {

            var loginRedirectState;

            if (PermissionsService.userIsTrainer()) {
                loginRedirectState = 'app.trainer_home';

                if (!IntroService.hasSeenIntro()) {
                    loginRedirectState = 'app.trainer_intro';
                }
            } else {
                loginRedirectState = 'app.client_home';

                if (!IntroService.hasSeenIntro()) {
                    loginRedirectState = 'app.client_intro';
                }
            }

            if (AuthenticationStateService.getCurrentUser().accounts > 1) {
                loginRedirectState = 'app.accounts';
            }

            NavigateService.disableBack().go(loginRedirectState);
        };

        vm.doLogin = function () {

            vm.loading = true;

            var resolver = false;
            if (!BrandingService.isCustomApp()) {
                resolver = function () {

                    if (!BrandingService.brandingEnabled()) {
                        return false;
                    }

                    return BrandingService.applyStyles();
                };
            }

            AuthenticationService.passwordLogin(vm.email, vm.password, resolver).then(successGo, function (response) {

                vm.loading = false;

                if (response && response.hasOwnProperty('status') && response.status === -1) {

                    ToastService.show('Connection failure, please ensure there is a data connection');
                } else if (response.data.error && response.data.error.message && response.data.error.error_name && response.data.error.error_name === "SUBSCRIPTION_UPGRADE_REQUIRED") {

                    ToastService.show(response.data.error.message);
                } else {

                    ToastService.show('Incorrect email or password');
                }
            });
        };

        vm.doReset = function () {
            NavigateService.go('app.login_forgot_password');
        };
    }
})();
// Workouts Controller
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('LogoutController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.logout', {
            url: "/logout",
            views: {
                'menuContent': {
                    template: "",
                    controller: 'LogoutController as vm'
                }
            }
        });
    }

    controller.$inject = ['AuthenticationHelperService'];

    function controller(AuthenticationHelperService) {
        AuthenticationHelperService.logout();
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('MessagesController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.messages', {
            url: "/messages",
            views: {
                'menuContent': {
                    templateUrl: "common/messages/messages.html",
                    controller: 'MessagesController as vm'
                }
            }
        });
    }

    controller.$inject = ['ApiClientService', '$scope', 'NavigateService', '$filter', 'AnalyticsService', 'MessagesService', 'AuthenticationStateService'];

    function controller(ApiClientService, $scope, NavigateService, $filter, AnalyticsService, MessagesService, AuthenticationStateService) {

        const vm = this;
        const currentUser = AuthenticationStateService.getCurrentUser();
        vm.inbox = [];
        let page = 1;
        let lastPage = 0;

        /**
         * Checks if we need to show load more
         */
        vm.showLoadMore = () => page <= lastPage && !vm.loading;

        /**
         * Checks if the message is seen
         */
        vm.isSeen = thread => thread.seen === 0 && thread.to_user_id === currentUser.id;

        vm.goToMessage = function (thread) {
            NavigateService.go('app.message_conversation', {
                conversationId: thread.conversation_id
            });
        };

        vm.getData = refreshing => {
            vm.loading = true;
            ApiClientService.messages.getInbox({
                page
            }).success(function (response) {

                page = response.pagination.current_page + 1;
                lastPage = response.pagination.last_page;
                vm.inbox = vm.inbox.concat(response.inbox);

                vm.noConversations = !!vm.inbox.length;

                vm.loading = false;
            }).finally(function () {

                if (refreshing) {
                    $scope.$broadcast('scroll.refreshComplete');
                }
            });
        };

        const loadMessages = function (refreshing) {

            refreshing = refreshing || false;

            vm.loading = true;

            vm.getData(refreshing);
        };

        vm.refresh = function () {
            loadMessages(true);
        };

        $scope.$on('$ionicView.beforeEnter', function () {
            loadMessages();
        });

        $scope.$on('$ionicView.afterEnter', function () {
            MessagesService.markSeen();
            AnalyticsService.trackView('Messages - Inbox');
        });
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('MessagesService', service);

    service.$inject = ['$rootScope'];

    function service($rootScope) {

        var service = {};

        var isSeen = true;

        var fireStateChangeHandlers = function (isSeen) {
            $rootScope.$emit('MessagesServiceSeenStateChange', isSeen);
        };

        service.isSeen = function () {
            return isSeen;
        };

        service.markSeen = function () {
            if (isSeen) {
                return false;
            }
            isSeen = true;
            fireStateChangeHandlers(isSeen);
        };

        service.markUnseen = function () {
            if (!isSeen) {
                return false;
            }
            isSeen = false;
            fireStateChangeHandlers(isSeen);
        };

        service.onSeenStateChange = function (scope, handler) {
            var unsub = $rootScope.$on('MessagesServiceSeenStateChange', handler);

            scope.$on('$destroy', unsub);

            return unsub;
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('MetricsService', service);

    function service() {

        var service = {};

        service.calculateBodyFat = function (gender, age, skinfold_chest, skinfold_abdominal, skinfold_thigh, skinfold_tricep, skinfold_subscapular, skinfold_suprailiac, skinfold_midaxillary) {

            age = parseInt(age);

            skinfold_chest = parseFloat(skinfold_chest);
            skinfold_abdominal = parseFloat(skinfold_abdominal);
            skinfold_thigh = parseFloat(skinfold_thigh);
            skinfold_tricep = parseFloat(skinfold_tricep);
            skinfold_subscapular = parseFloat(skinfold_subscapular);
            skinfold_suprailiac = parseFloat(skinfold_suprailiac);
            skinfold_midaxillary = parseFloat(skinfold_midaxillary);

            if (skinfold_chest <= 0 || skinfold_abdominal <= 0 || skinfold_thigh <= 0 || skinfold_tricep <= 0 || skinfold_subscapular <= 0 || skinfold_suprailiac <= 0 || skinfold_midaxillary <= 0) {
                return false;
            }

            var sumSites = skinfold_chest + skinfold_abdominal + skinfold_thigh + skinfold_tricep + skinfold_subscapular + skinfold_suprailiac + skinfold_midaxillary;

            var bodyfat;

            if (gender === 'male') {

                bodyfat = 495 / (1.112 - 0.00043499 * sumSites + 0.00000055 * sumSites * sumSites - 0.00028826 * age) - 450;
            } else {

                bodyfat = 495 / (1.097 - 0.00046971 * sumSites + 0.00000056 * sumSites * sumSites - 0.00012828 * age) - 450;
            }

            var calculation = Math.round(bodyfat * 100) / 100;

            if (calculation < 0) {
                return 0;
            }

            return calculation;
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('MetricsBodyFatCategoriesModalController', controller);

    controller.$inject = ['AnalyticsService'];

    function controller(AnalyticsService) {

        AnalyticsService.trackView('Body Fat Categories');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('MetricsController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.trainer_update_customer_metrics', {
            url: "/clients/:customerId/metrics",
            views: {
                'menuContent': {
                    templateUrl: "common/metrics/metrics.html",
                    controller: 'MetricsController as vm'
                }
            },
            params: {
                goBack: false
            }
        });

        $stateProvider.state('app.client_update_metrics', {
            url: "/clients_metrics",
            views: {
                'menuContent': {
                    templateUrl: "common/metrics/metrics.html",
                    controller: 'MetricsController as vm'
                }
            },
            params: {
                goBack: false
            }
        });
    }

    controller.$inject = ['$scope', 'PermissionsService', 'ApiClientService', '$stateParams', 'UnitService', 'MetricsService', 'ModalService', 'ToastService', 'AlertDialogService', 'NavigateService', '$ionicActionSheet', '$filter', 'ConfirmDialogService', 'AnalyticsService', 'NotificationsService'];

    function controller($scope, PermissionsService, ApiClientService, $stateParams, UnitService, MetricsService, ModalService, ToastService, AlertDialogService, NavigateService, $ionicActionSheet, $filter, ConfirmDialogService, AnalyticsService, NotificationsService) {

        // temp horrible hack to work around issue of disappearing tabs
        // in some weird circumstances.
        $scope.$on('$ionicView.afterEnter', function () {
            $(window).trigger('resize');
        });

        var vm = this;

        vm.updateMetrics = function () {

            vm.saving = true;

            var metrics;
            if (vm.selectedTabIndex === 0) {
                metrics = vm.metricsMeasurements;
                if (metrics.blood_pressure_systolic || metrics.blood_pressure_diastolic) {
                    if (!metrics.blood_pressure_systolic && metrics.blood_pressure_diastolic) {
                        ToastService.show('A systolic measurement is required.');
                        vm.saving = false;
                        return false;
                    }
                    if (metrics.blood_pressure_systolic && !metrics.blood_pressure_diastolic) {
                        ToastService.show('A diastolic measurement is required.');
                        vm.saving = false;
                        return false;
                    }
                }
            } else {
                metrics = vm.metricsFat;
            }

            metrics.date = vm.date.format('YYYY-MM-DD');

            var saveMetrics = ApiClientService.metrics.update(vm.customerId, metrics);

            saveMetrics.success(function (response) {

                vm.saving = false;

                ToastService.show('Measurements saved successfully');

                NotificationsService.dirty();

                if ($stateParams.goBack) {
                    NavigateService.goBack();
                } else {
                    loadHistory();
                }
            });

            saveMetrics.error(function (response) {

                vm.saving = false;

                ToastService.show(response.error.message);

                vm.fieldErrors = response.error.fieldErrors;
            });
        };

        var loadCustomer = function () {

            return ApiClientService.customers.get(vm.customerId).success(function (response) {

                if (response.customer.user.dob) {
                    vm.age = moment().diff(moment(response.customer.user.dob, 'YYYY-MM-DD'), 'years');
                }

                vm.customer = response.customer;
            }).error(function (response) {

                if (response.error && response.error.error_name && response.error.error_name === 'SUBSCRIPTION_UPGRADE_REQUIRED') {
                    ToastService.show(response.error.message);
                    NavigateService.goBack();
                } else {
                    ToastService.show('Could not get measurements, please try again later');
                }
            });
        };

        var loadLatestMetrics = function () {

            return ApiClientService.metrics.latest(vm.customerId).success(function (response) {

                var metrics = response.metrics;

                vm.metricsFat = {
                    skinfold_chest_l: metrics.skinfold_chest_l,
                    skinfold_triceps_l: metrics.skinfold_triceps_l,
                    skinfold_biceps_l: metrics.skinfold_biceps_l,
                    skinfold_subscapular_l: metrics.skinfold_subscapular_l,
                    skinfold_midaxilary_l: metrics.skinfold_midaxilary_l,
                    skinfold_abdominal_l: metrics.skinfold_abdominal_l,
                    skinfold_supraliac_l: metrics.skinfold_supraliac_l,
                    skinfold_thigh_l: metrics.skinfold_thigh_l,
                    skinfold_chest_r: metrics.skinfold_chest_r,
                    skinfold_triceps_r: metrics.skinfold_triceps_r,
                    skinfold_biceps_r: metrics.skinfold_biceps_r,
                    skinfold_subscapular_r: metrics.skinfold_subscapular_r,
                    skinfold_midaxilary_r: metrics.skinfold_midaxilary_r,
                    skinfold_abdominal_r: metrics.skinfold_abdominal_r,
                    skinfold_supraliac_r: metrics.skinfold_supraliac_r,
                    skinfold_thigh_r: metrics.skinfold_thigh_r,
                    bodyfat_percent: metrics.bodyfat_percent,
                    bodyfat_percent_l: metrics.bodyfat_percent_l,
                    bodyfat_percent_r: metrics.bodyfat_percent_r,
                    date: vm.date
                };

                vm.metricsMeasurements = {
                    circ_arm_l: metrics.circ_arm_l,
                    circ_calf_l: metrics.circ_calf_l,
                    circ_chest_l: metrics.circ_chest_l,
                    circ_hips_l: metrics.circ_hips_l,
                    circ_neck_l: metrics.circ_neck_l,
                    circ_thigh_l: metrics.circ_thigh_l,
                    circ_waist_l: metrics.circ_waist_l,
                    circ_arm_r: metrics.circ_arm_r,
                    circ_calf_r: metrics.circ_calf_r,
                    circ_chest_r: metrics.circ_chest_r,
                    circ_hips_r: metrics.circ_hips_r,
                    circ_neck_r: metrics.circ_neck_r,
                    circ_thigh_r: metrics.circ_thigh_r,
                    circ_waist_r: metrics.circ_waist_r,
                    weight: metrics.weight,
                    resting_heart_rate: metrics.resting_heart_rate,
                    blood_pressure_systolic: metrics.blood_pressure_systolic,
                    blood_pressure_diastolic: metrics.blood_pressure_diastolic,
                    date: vm.date
                };
            });
        };

        var loadHistory = function () {

            return ApiClientService.metrics.all(vm.customerId).success(function (response) {

                vm.metricsHistory = response.metrics;

                var visibleWeight = 0;
                var visibleWeightLeft = 0;
                var visibleWeightRight = 0;

                var visibleFat = 0;
                var visibleFatLeft = 0;
                var visibleFatRight = 0;

                var legacyDataCountMeasurements = 0;
                var legacyDataCountBodyFat = 0;
                angular.forEach(vm.metricsHistory, function (row) {

                    if (row.skinfold_legacy) {
                        legacyDataCountBodyFat++;
                    }

                    if (row.circ_legacy) {
                        legacyDataCountMeasurements++;
                    }

                    // weight showWeightCirc
                    if (row.weight > 0 || row.circ_neck_l > 0 || row.circ_arm_l > 0 || row.circ_chest_l > 0 || row.circ_waist_l > 0 || row.circ_thigh_l > 0 || row.circ_hips_l > 0 || row.circ_calf_l > 0) {
                        row.showWeightCircLeft = true;
                        visibleWeight++;
                        visibleWeightLeft++;
                    }

                    if (row.weight > 0 || row.circ_neck_r > 0 || row.circ_arm_r > 0 || row.circ_chest_r > 0 || row.circ_waist_r > 0 || row.circ_thigh_r > 0 || row.circ_hips_r > 0 || row.circ_calf_r > 0) {
                        row.showWeightCircRight = true;
                        visibleWeight++;
                        visibleWeightRight++;
                    }

                    //fat showFat
                    if (row.skinfold_chest_l > 0 || row.skinfold_triceps_l > 0 || row.skinfold_biceps_l > 0 || row.skinfold_subscapular_l > 0 || row.skinfold_midaxilary_l > 0 || row.skinfold_abdominal_l > 0 || row.skinfold_supraliac_l > 0 || row.skinfold_thigh_l > 0 || row.bodyfat_percent_l > 0 || row.bodyfat_percent > 0) {
                        row.showFatLeft = true;
                        visibleFat++;
                        visibleFatLeft++;
                    }

                    if (row.skinfold_chest_r > 0 || row.skinfold_triceps_r > 0 || row.skinfold_biceps_r > 0 || row.skinfold_subscapular_r > 0 || row.skinfold_midaxilary_r > 0 || row.skinfold_abdominal_r > 0 || row.skinfold_supraliac_r > 0 || row.skinfold_thigh_r > 0 || row.bodyfat_percent_r > 0 || row.bodyfat_percent > 0) {
                        row.showFatRight = true;
                        visibleFat++;
                        visibleFatRight++;
                    }

                    if (row.circ_neck_l) {
                        row.circ_neck_l = UnitService.unitDisplay('length', row.circ_neck_l, 'cm', true);
                    }

                    if (row.circ_arm_l) {
                        row.circ_arm_l = UnitService.unitDisplay('length', row.circ_arm_l, 'cm', true);
                    }

                    if (row.circ_chest_l) {
                        row.circ_chest_l = UnitService.unitDisplay('length', row.circ_chest_l, 'cm', true);
                    }

                    if (row.circ_waist_l) {
                        row.circ_waist_l = UnitService.unitDisplay('length', row.circ_waist_l, 'cm', true);
                    }

                    if (row.circ_thigh_l) {
                        row.circ_thigh_l = UnitService.unitDisplay('length', row.circ_thigh_l, 'cm', true);
                    }

                    if (row.circ_hips_l) {
                        row.circ_hips_l = UnitService.unitDisplay('length', row.circ_hips_l, 'cm', true);
                    }

                    if (row.circ_calf_l) {
                        row.circ_calf_l = UnitService.unitDisplay('length', row.circ_calf_l, 'cm', true);
                    }

                    if (row.circ_neck_r) {
                        row.circ_neck_r = UnitService.unitDisplay('length', row.circ_neck_r, 'cm', true);
                    }

                    if (row.circ_arm_r) {
                        row.circ_arm_r = UnitService.unitDisplay('length', row.circ_arm_r, 'cm', true);
                    }

                    if (row.circ_chest_r) {
                        row.circ_chest_r = UnitService.unitDisplay('length', row.circ_chest_r, 'cm', true);
                    }

                    if (row.circ_waist_r) {
                        row.circ_waist_r = UnitService.unitDisplay('length', row.circ_waist_r, 'cm', true);
                    }

                    if (row.circ_thigh_r) {
                        row.circ_thigh_r = UnitService.unitDisplay('length', row.circ_thigh_r, 'cm', true);
                    }

                    if (row.circ_hips_r) {
                        row.circ_hips_r = UnitService.unitDisplay('length', row.circ_hips_r, 'cm', true);
                    }

                    if (row.circ_calf_r) {
                        row.circ_calf_r = UnitService.unitDisplay('length', row.circ_calf_r, 'cm', true);
                    }

                    if (row.skinfold_chest_l) {
                        row.skinfold_chest_l = UnitService.unitDisplay('length', row.skinfold_chest_l, 'mm', true);
                    }

                    if (row.skinfold_triceps_l) {
                        row.skinfold_triceps_l = UnitService.unitDisplay('length', row.skinfold_triceps_l, 'mm', true);
                    }

                    if (row.skinfold_biceps_l) {
                        row.skinfold_biceps_l = UnitService.unitDisplay('length', row.skinfold_biceps_l, 'mm', true);
                    }

                    if (row.skinfold_subscapular_l) {
                        row.skinfold_subscapular_l = UnitService.unitDisplay('length', row.skinfold_subscapular_l, 'mm', true);
                    }

                    if (row.skinfold_midaxilary_l) {
                        row.skinfold_midaxilary_l = UnitService.unitDisplay('length', row.skinfold_midaxilary_l, 'mm', true);
                    }

                    if (row.skinfold_abdominal_l) {
                        row.skinfold_abdominal_l = UnitService.unitDisplay('length', row.skinfold_abdominal_l, 'mm', true);
                    }

                    if (row.skinfold_supraliac_l) {
                        row.skinfold_supraliac_l = UnitService.unitDisplay('length', row.skinfold_supraliac_l, 'mm', true);
                    }

                    if (row.skinfold_thigh_l) {
                        row.skinfold_thigh_l = UnitService.unitDisplay('length', row.skinfold_thigh_l, 'mm', true);
                    }

                    if (row.skinfold_chest_r) {
                        row.skinfold_chest_r = UnitService.unitDisplay('length', row.skinfold_chest_r, 'mm', true);
                    }

                    if (row.skinfold_triceps_r) {
                        row.skinfold_triceps_r = UnitService.unitDisplay('length', row.skinfold_triceps_r, 'mm', true);
                    }

                    if (row.skinfold_biceps_r) {
                        row.skinfold_biceps_r = UnitService.unitDisplay('length', row.skinfold_biceps_r, 'mm', true);
                    }

                    if (row.skinfold_subscapular_r) {
                        row.skinfold_subscapular_r = UnitService.unitDisplay('length', row.skinfold_subscapular_r, 'mm', true);
                    }

                    if (row.skinfold_midaxilary_r) {
                        row.skinfold_midaxilary_r = UnitService.unitDisplay('length', row.skinfold_midaxilary_r, 'mm', true);
                    }

                    if (row.skinfold_abdominal_r) {
                        row.skinfold_abdominal_r = UnitService.unitDisplay('length', row.skinfold_abdominal_r, 'mm', true);
                    }

                    if (row.skinfold_supraliac_r) {
                        row.skinfold_supraliac_r = UnitService.unitDisplay('length', row.skinfold_supraliac_r, 'mm', true);
                    }

                    if (row.skinfold_thigh_r) {
                        row.skinfold_thigh_r = UnitService.unitDisplay('length', row.skinfold_thigh_r, 'mm', true);
                    }

                    if (row.weight) {
                        row.weight = UnitService.unitDisplay('bodyweight', row.weight, false, true);
                    }

                    if (row.resting_heart_rate) {
                        row.resting_heart_rate = `${row.resting_heart_rate}BPM`;
                    }

                    if (row.blood_pressure_systolic && row.blood_pressure_diastolic) {
                        row.blood_pressure = `${row.blood_pressure_systolic}/${row.blood_pressure_diastolic}mmHg`;
                    }

                    if (row.muscle_mass) {
                        row.muscle_mass = UnitService.unitDisplay('weight', row.muscle_mass, false, true);
                    }
                });

                if (visibleFat > 0) {

                    vm.showFatHistory = true;

                    if (visibleFatLeft > 0) {
                        vm.showLeftFatHistory = true;
                    }
                    if (visibleFatRight > 0) {
                        vm.showRightFatHistory = true;
                    }
                } else {

                    vm.showFatHistory = false;
                    vm.showLeftFatHistory = false;
                    vm.showRightFatHistory = false;
                }

                if (visibleWeight > 0) {

                    vm.showWeightHistory = true;

                    if (visibleWeightLeft > 0) {
                        vm.showLeftMeasurementsHistory = true;
                    }
                    if (visibleWeightRight > 0) {
                        vm.showRightMeasurementsHistory = true;
                    }
                } else {

                    vm.showWeightHistory = false;
                    vm.showRightMeasurementsHistory = false;
                    vm.showLeftMeasurementsHistory = false;
                }

                vm.historyReady = true;
            });
        };

        var setupUnits = function () {

            // lengths
            vm.lengthSystem = UnitService.getCurrentUnitType('length');
            vm.lengthLabelCircumference = vm.lengthSystem === 'metric' ? 'cm' : 'in.';
            vm.lengthLabelBodyFat = vm.lengthSystem === 'metric' ? 'mm' : 'in.';
            vm.lengthsDoubleField = vm.lengthSystem === 'imperial';

            // bodyweight
            vm.bodyWeightSystem = UnitService.getCurrentUnitType('bodyweight');
            if (vm.bodyWeightSystem === 'imperial_gb') {

                vm.bodyWeightLabel = 'st.';
                vm.bodyWeightDoubleField = true;
            } else if (vm.bodyWeightSystem === 'imperial_us') {

                vm.bodyWeightLabel = 'lbs';
                vm.bodyWeightDoubleField = true;
            } else {

                vm.bodyWeightLabel = 'kg';
                vm.bodyWeightDoubleField = false;
            }

            // weight
            vm.weightSystem = UnitService.getCurrentUnitType('weight');
            vm.weightLabel = vm.weightSystem === 'metric' ? 'kg' : 'lbs';
            vm.weightDoubleField = vm.weightSystem === 'imperial';
        };

        var averageValues = function (value_l, value_r, useLeft, useRight) {

            var sum = 0;
            var i = 0;

            value_l = parseFloat(value_l);
            value_r = parseFloat(value_r);

            if (value_l > 0 && useLeft) {
                sum += value_l;
                i++;
            }

            if (value_r > 0 && useRight) {
                sum += value_r;
                i++;
            }

            if (sum === 0) {
                return 0;
            }

            return sum / i;
        };

        var setBodyFatValue = function (value, side) {

            switch (side) {
                case 'l':
                    vm.metricsFat.bodyfat_percent_l = value;
                    break;

                case 'r':
                    vm.metricsFat.bodyfat_percent_r = value;
                    break;

                default:
                    vm.metricsFat.bodyfat_percent = value;
            }
        };

        var calculateBodyFat = function (chest, abdominal, thigh, triceps, subscapular, supraliac, midaxilary, side) {

            side = side || false;

            var bodyFatCalculated = MetricsService.calculateBodyFat(vm.customer.user.gender, vm.age, chest, abdominal, thigh, triceps, subscapular, supraliac, midaxilary);

            if (!bodyFatCalculated) {
                bodyFatCalculated = 0;
            }

            setBodyFatValue(bodyFatCalculated, side);

            return true;
        };

        vm.doBodyFatCalc = function (side, leftPresent, rightPresent) {

            side = side || false;
            leftPresent = leftPresent || false;
            rightPresent = rightPresent || false;

            var sideName;

            var values = {
                chest: {
                    l: vm.metricsFat.skinfold_chest_l,
                    r: vm.metricsFat.skinfold_chest_r
                },
                abdominal: {
                    l: vm.metricsFat.skinfold_abdominal_l,
                    r: vm.metricsFat.skinfold_abdominal_r
                },
                thigh: {
                    l: vm.metricsFat.skinfold_thigh_l,
                    r: vm.metricsFat.skinfold_thigh_r
                },
                triceps: {
                    l: vm.metricsFat.skinfold_triceps_l,
                    r: vm.metricsFat.skinfold_triceps_r
                },
                subscapular: {
                    l: vm.metricsFat.skinfold_subscapular_l,
                    r: vm.metricsFat.skinfold_subscapular_r
                },
                supraliac: {
                    l: vm.metricsFat.skinfold_supraliac_l,
                    r: vm.metricsFat.skinfold_supraliac_r
                },
                midaxilary: {
                    l: vm.metricsFat.skinfold_midaxilary_l,
                    r: vm.metricsFat.skinfold_midaxilary_r
                }
            };

            var chest = 0,
                abdominal = 0,
                thigh = 0,
                triceps = 0,
                subscapular = 0,
                supraliac = 0,
                midaxilary = 0;

            switch (side) {

                case 'l':
                    sideName = 'left';
                    chest = values.chest.l;
                    abdominal = values.abdominal.l;
                    thigh = values.thigh.l;
                    triceps = values.triceps.l;
                    subscapular = values.subscapular.l;
                    supraliac = values.supraliac.l;
                    midaxilary = values.midaxilary.l;
                    break;

                case 'r':
                    sideName = 'right';
                    chest = values.chest.r;
                    abdominal = values.abdominal.r;
                    thigh = values.thigh.r;
                    triceps = values.triceps.r;
                    subscapular = values.subscapular.r;
                    supraliac = values.supraliac.r;
                    midaxilary = values.midaxilary.r;
                    break;

                default:
                    sideName = 'total';
                    chest = averageValues(values.chest.l, values.chest.r, leftPresent, rightPresent), abdominal = averageValues(values.abdominal.l, values.abdominal.r, leftPresent, rightPresent), thigh = averageValues(values.thigh.l, values.thigh.r, leftPresent, rightPresent), triceps = averageValues(values.triceps.l, values.triceps.r, leftPresent, rightPresent), subscapular = averageValues(values.subscapular.l, values.subscapular.r, leftPresent, rightPresent), supraliac = averageValues(values.supraliac.l, values.supraliac.r, leftPresent, rightPresent), midaxilary = averageValues(values.midaxilary.l, values.midaxilary.r, leftPresent, rightPresent);

            }

            vm.bodyFatCalcError[sideName] = false;

            if (!chest || !abdominal || !thigh || !triceps || !subscapular || !supraliac || !midaxilary) {

                var msg = 'Some of the 7 required measurements are missing for body fat calculation. Please fill all measurements in and try again.';

                vm.bodyFatCalcError[sideName] = msg;

                if (sideName === 'total') {
                    AlertDialogService.show({
                        message: msg,
                        title: 'Could not calculate body fat'
                    });
                }

                setBodyFatValue(0, side);

                return false;
            }

            return calculateBodyFat(chest, abdominal, thigh, triceps, subscapular, supraliac, midaxilary, side);
        };

        vm.doBodyFatCalcs = function () {

            if (vm.customer.user.gender && vm.age) {

                var l = vm.doBodyFatCalc('l');
                var r = vm.doBodyFatCalc('r');

                vm.doBodyFatCalc(false, l, r);
            } else {

                ModalService.show({
                    templateUrl: "common/metrics/setCustomerKeyValuesModal.html",
                    controller: 'MetricsSetCustomerKeyValuesModalController as vm',
                    params: {
                        customer: vm.customer
                    }
                }).then(function () {

                    loadCustomer(function () {
                        vm.doBodyFatCalcs();
                    });
                });
            }
        };

        vm.showBodyFatCats = function () {

            ModalService.show({
                templateUrl: "common/metrics/categoriesModal.html",
                controller: 'MetricsBodyFatCategoriesModalController as vm'
            });
        };

        vm.setSide = function (side) {
            vm.currentSide = side;
        };

        var clearedMeasurements = function (date, side) {

            var data = {
                date: date,
                weight: '',
                no_activity_log: true
            };

            data['circ_arm_' + side] = '';
            data['circ_calf_' + side] = '';
            data['circ_chest_' + side] = '';
            data['circ_hips_' + side] = '';
            data['circ_neck_' + side] = '';
            data['circ_thigh_' + side] = '';
            data['circ_waist_' + side] = '';

            return data;
        };

        var clearedFat = function (date, side) {

            var data = {
                date: date,
                bodyfat_percent: '',
                no_activity_log: true
            };

            data['skinfold_chest_' + side] = '';
            data['skinfold_triceps_' + side] = '';
            data['skinfold_biceps_' + side] = '';
            data['skinfold_subscapular_' + side] = '';
            data['skinfold_midaxilary_' + side] = '';
            data['skinfold_abdominal_' + side] = '';
            data['skinfold_supraliac_' + side] = '';
            data['skinfold_thigh_' + side] = '';
            data['bodyfat_percent_' + side] = '';

            return data;
        };

        vm.clearDateMenu = function (metrics) {

            var formattedDate = $filter('momentdate')(metrics.date, 'Do MMM YYYY', false);

            $ionicActionSheet.show({
                titleText: formattedDate,
                destructiveText: 'Delete',
                cancelText: 'Cancel',
                destructiveButtonClicked: function () {

                    ConfirmDialogService.open({
                        title: "Confirm Delete",
                        message: "Are you sure you want to remove " + vm.currentSide + " side " + (vm.selectedTabIndex === 1 ? 'body fat ' : '') + "measurements for " + formattedDate + "?",
                        buttonConfirm: "Delete",
                        buttonCancel: "Cancel"
                    }).then(function () {

                        metrics.deleting = true;

                        var side = vm.currentSide === 'right' ? 'r' : 'l';

                        var newData;

                        if (vm.selectedTabIndex === 0) {
                            newData = clearedMeasurements(metrics.date, side);
                        } else {
                            newData = clearedFat(metrics.date, side);
                        }

                        var save = ApiClientService.metrics.update(vm.customer.id, newData);

                        save.finally(loadHistory);
                    });

                    return true;
                }
            });
        };

        $scope.$on('$ionicView.beforeEnter', function () {

            vm.currentSide = 'left';

            vm.date = moment();

            vm.metricsFat = {
                skinfold_chest_l: null,
                skinfold_triceps_l: null,
                skinfold_biceps_l: null,
                skinfold_subscapular_l: null,
                skinfold_midaxilary_l: null,
                skinfold_abdominal_l: null,
                skinfold_supraliac_l: null,
                skinfold_thigh_l: null,
                skinfold_chest_r: null,
                skinfold_triceps_r: null,
                skinfold_biceps_r: null,
                skinfold_subscapular_r: null,
                skinfold_midaxilary_r: null,
                skinfold_abdominal_r: null,
                skinfold_supraliac_r: null,
                skinfold_thigh_r: null,
                bodyfat_percent: null,
                bodyfat_percent_l: null,
                bodyfat_percent_r: null,
                date: null
            };
            vm.metricsMeasurements = {
                circ_arm_l: null,
                circ_calf_l: null,
                circ_chest_l: null,
                circ_hips_l: null,
                circ_neck_l: null,
                circ_thigh_l: null,
                circ_waist_l: null,
                circ_arm_r: null,
                circ_calf_r: null,
                circ_chest_r: null,
                circ_hips_r: null,
                circ_neck_r: null,
                circ_thigh_r: null,
                circ_waist_r: null,
                weight: null,
                resting_heart_rate: null,
                blood_pressure: null,
                date: null
            };

            vm.canEdit = false;

            if (PermissionsService.userIsCustomer()) {
                vm.customerId = PermissionsService.getCurrentUserEntityId();
            } else {
                vm.customerId = $stateParams.customerId;
            }

            vm.ready = false;

            vm.bodyFatCalcError = {
                left: false,
                right: false,
                total: false
            };

            vm.age = false;

            loadCustomer().then(function () {

                vm.canEdit = PermissionsService.userIsCustomer() || PermissionsService.userIsTrainerAdminOrOwner(vm.customer.trainer_user_id);

                loadLatestMetrics().finally(function () {
                    vm.ready = true;
                    loadHistory();
                });
            });

            setupUnits();
        });

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Measurements');
        });

        $scope.$on('displayUnitChanged', setupUnits);
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('MetricsSetCustomerKeyValuesModalController', controller);

    controller.$inject = ['parameters', '$scope', 'ApiClientService', 'PermissionsService', 'ToastService'];

    function controller(parameters, $scope, ApiClientService, PermissionsService, ToastService) {

        var vm = this;

        vm.isTrainer = PermissionsService.userIsTrainer();

        vm.customer = parameters.customer;

        vm.gender = null;
        vm.dob = null;

        if (vm.customer.user.dob) {
            var dob = moment(vm.customer.user.dob, 'YYYY-MM-DD');
            vm.dob = dob.toDate();
        }

        if (vm.customer.user.gender) {
            vm.gender = vm.customer.user.gender;
        }

        vm.updateCustomer = function () {

            vm.saving = true;
            vm.error = false;

            var data = {};

            if (vm.dob) {
                data.dob = moment(vm.dob).format('YYYY-MM-DD');
            }

            if (vm.gender) {
                data.gender = vm.gender;
            }

            var updateCustomer = ApiClientService.customers.update(vm.customer.id, data);

            updateCustomer.success(function (response) {

                vm.error = false;

                $scope.closeModal(true);
            });

            updateCustomer.error(function (response) {

                vm.saving = false;

                if (response.error) {

                    ToastService.show(response.error.message);
                } else {

                    ToastService.show("Unable to save, please try again.");
                }
            });
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('NoteViewerModalController', controller).factory('NoteViewerModalService', service);

    controller.$inject = ['parameters'];

    function controller(parameters) {

        var vm = this;

        vm.note = parameters.note;
    }

    service.$inject = ['ModalService'];

    function service(ModalService) {

        var service = {};

        service.show = function (note) {

            ModalService.show({
                templateUrl: "common/noteViewer/noteViewerModal.html",
                controller: 'NoteViewerModalController as vm',
                params: {
                    note: note
                }
            });
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').component('notificationSettings', {
        templateUrl: 'common/notificationSettings/notificationSettings.html',
        controller: controller,
        controllerAs: 'vm',
        bindings: {
            // isModal: '='
        }
    });

    controller.$inject = ['ApiClientService', '$mdToast', 'NavigateService', 'PermissionsService', '$filter'];

    function controller(ApiClientService, $mdToast, NavigateService, PermissionsService, $filter) {

        var vm = this;

        var isTrainer = PermissionsService.userIsTrainer();
        var isCustomer = PermissionsService.userIsCustomer();

        vm.notificationNames = {

            'session_booked': 'Sessions Booked',
            'session_booked_mobile': 'Sessions Booked',

            'session_cancelled': 'Sessions Cancelled',
            'session_cancelled_mobile': 'Sessions Cancelled',

            'session_cancel_request': 'Session Cancellation Requests',
            'session_cancel_request_mobile': 'Session Cancellation Requests',

            'session_edited': 'Sessions Edited',
            'session_edited_mobile': 'Sessions Edited',

            'session_completed_mobile': 'Sessions Completed by Client',

            'customer_metrics_updated_mobile': 'Measurements Updated by Client',

            'nutrition_logged_mobile': 'Nutrition Logged by Client',

            'progress_photo_uploaded_mobile': 'Progress Photo Uploaded by Client',

            'new_message': 'New Messages',
            'new_message_mobile': 'New Messages',

            'merchant_payment_received': 'Payment Received',
            'merchant_package_start': isTrainer ? 'Client Purchased Package' : 'Purchased Package',
            'merchant_package_end': isTrainer ? 'Client Package Expired' : 'Package Expired',
            'merchant_package_low_sessions': isTrainer ? 'Client Package Low Sessions Remaining' : 'Package Low Sessions Remaining',

            'custom_form': 'Custom Forms',
            'custom_form_mobile': 'Custom Forms',

            'client_started_schedule_mobile': 'Client has started Schedule',
            'schedule_am_workout_mobile': 'Scheduled AM workout reminder',
            'schedule_pm_workout_mobile': 'Scheduled PM workout reminder',
            'schedule_nutrition_mobile': 'Reminder to log nutrition',
            'schedule_assigned_mobile': 'New Schedule',
            'schedule_time_update_metrics_mobile': 'Reminder to update measurements',
            'schedule_time_update_photo_mobile': 'Reminder to update progress photo',
            'schedule_trainer_day_before_mobile': 'Client\'s schedule ends tomorrow',
            'schedule_trainer_week_before_mobile': 'Client\'s schedule ends in 1 week'

        };

        vm.notifications = [];
        vm.mobileNotifications = [];

        /****************************
         * Email notifications
         ****************************/

        vm.notifications.push({
            "notification_name": "new_message",
            "enabled": true,
            "schedule": 'immediate'
        });
        vm.notifications.push({
            "notification_name": "session_booked",
            "enabled": true,
            "schedule": 'immediate'
        });
        vm.notifications.push({
            "notification_name": "session_edited",
            "enabled": true,
            "schedule": 'immediate'
        });
        if (isTrainer) {
            vm.notifications.push({
                "notification_name": "session_cancel_request",
                "enabled": true,
                "schedule": 'immediate'
            });
        }
        vm.notifications.push({
            "notification_name": "session_cancelled",
            "enabled": true,
            "schedule": 'immediate'
        });
        if (isTrainer) {
            vm.notifications.push({
                "notification_name": "merchant_payment_received",
                "enabled": true,
                "schedule": 'immediate'
            });
        }
        vm.notifications.push({
            "notification_name": "merchant_package_start",
            "enabled": true,
            "schedule": 'immediate'
        });
        vm.notifications.push({
            "notification_name": "merchant_package_end",
            "enabled": true,
            "schedule": 'immediate'
        });
        vm.notifications.push({
            "notification_name": "merchant_package_low_sessions",
            "enabled": true,
            "schedule": 'immediate'
        });
        vm.notifications.push({
            "notification_name": "custom_form",
            "enabled": true,
            "schedule": 'immediate'
        });

        // Push


        vm.mobileNotifications.push({
            "notification_name": "new_message_mobile",
            "enabled": true
        });
        if (isCustomer) {
            vm.mobileNotifications.push({
                "notification_name": "session_booked_mobile",
                "enabled": true
            });
            vm.mobileNotifications.push({
                "notification_name": "session_edited_mobile",
                "enabled": true
            });
        }
        vm.mobileNotifications.push({
            "notification_name": "session_cancel_request_mobile",
            "enabled": true
        });
        vm.mobileNotifications.push({
            "notification_name": "session_cancelled_mobile",
            "enabled": true
        });
        if (isTrainer) {
            vm.mobileNotifications.push({
                "notification_name": "session_completed_mobile",
                "enabled": true
            });
            vm.mobileNotifications.push({
                "notification_name": "customer_metrics_updated_mobile",
                "enabled": true
            });
            vm.mobileNotifications.push({
                "notification_name": "nutrition_logged_mobile",
                "enabled": true
            });
            vm.mobileNotifications.push({
                "notification_name": "progress_photo_uploaded_mobile",
                "enabled": true
            });
            vm.mobileNotifications.push({
                "notification_name": "client_started_schedule_mobile",
                "enabled": true
            });
            vm.mobileNotifications.push({
                "notification_name": "schedule_trainer_day_before_mobile",
                "enabled": true
            });
            vm.mobileNotifications.push({
                "notification_name": "schedule_trainer_week_before_mobile",
                "enabled": true
            });
        }
        if (isCustomer) {
            vm.mobileNotifications.push({
                "notification_name": "schedule_am_workout_mobile",
                "enabled": true
            });
            vm.mobileNotifications.push({
                "notification_name": "schedule_pm_workout_mobile",
                "enabled": true
            });
            vm.mobileNotifications.push({
                "notification_name": "schedule_nutrition_mobile",
                "enabled": true
            });
            vm.mobileNotifications.push({
                "notification_name": "schedule_time_update_metrics_mobile",
                "enabled": true
            });
            vm.mobileNotifications.push({
                "notification_name": "schedule_time_update_photo_mobile",
                "enabled": true
            });
            vm.mobileNotifications.push({
                "notification_name": "schedule_assigned_mobile",
                "enabled": true
            });
        }

        vm.loading = true;

        var getSettings = ApiClientService.notificationSettings.get();

        getSettings.success(function (response) {

            vm.loading = false;
            vm.settings = response.notifications;

            angular.forEach(vm.settings, function (settings, notification_name) {

                var notifications = vm.notifications;

                if (notification_name.substring(notification_name.length - 6) === 'mobile') {
                    notifications = vm.mobileNotifications;
                }

                var notification = $filter('filter')(notifications, {
                    notification_name: notification_name
                }, true)[0];

                if (!notification) {
                    return false;
                }

                if (settings.enabled) {
                    notification.enabled = true;
                } else {
                    notification.enabled = false;
                }

                if (settings.schedule) {
                    notification.schedule = settings.schedule;
                }
            });
        });

        vm.updateSchedule = function (notification) {
            if (notification.schedule === 'off') {
                notification.enabled = false;
            } else {
                notification.enabled = true;
            }
        };

        vm.saveNotificationSettings = function (noBack) {

            vm.saving = true;

            noBack = noBack || false;

            var notifications = angular.copy(vm.notifications);

            var save = ApiClientService.notificationSettings.set(notifications.concat(vm.mobileNotifications));

            save.success(function () {

                vm.saving = false;

                if (noBack === false) {
                    NavigateService.goBack();
                }
            });

            save.error(function (response) {
                vm.saving = false;
                $mdToast.show($mdToast.simple().textContent('Could not save notification settings. Please try again').position('top right'));
            });
        };
    }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').config(config).controller('NotificationsController', controller);

  config.$inject = ['$stateProvider'];

  function config($stateProvider) {

    $stateProvider.state('app.notifications', {
      url: '/notifications',
      params: {
        isAnimated: false
      },
      views: {
        'menuContent': {
          templateUrl: 'common/notifications/notifications.html',
          controller: 'NotificationsController as vm'
        }
      }
    });
  }

  controller.$inject = ['$scope', 'ApiClientService', 'NavigateService', 'PermissionsService', '$ionicActionSheet', 'NotificationsService', '$ionicScrollDelegate', 'NutritionLogService', '$filter'];

  function controller($scope, ApiClientService, NavigateService, PermissionsService, $ionicActionSheet, NotificationsService, $ionicScrollDelegate, NutritionLogService, $filter) {
    var vm = this;
    vm.notifications = [];
    vm.noNotifications = false;
    vm.loading = true;
    vm.currentUser = PermissionsService.getCurrentUser();
    vm.unreadNotificationCount = 0;

    var transformNotification = function (notification) {
      notification.localDate = moment.tz(notification.created_at, notification.customer.timezone).format('YYYY-MM-DD');
      // const { name } = notification.schedule;

      switch (notification.type) {
        case 'assigned_schedule':
          notification.icon = 'fiber_new';
          notification.description = `You have been assigned a new schedule, ${notification.schedule.name}! Take a look`;
          break;

        case 'workout_am':
        case 'workout_pm':
          notification.icon = 'fitness_center';
          notification.description = `Today's ${notification.type === 'workout_am' ? 'AM' : 'PM'} Workout: ${notification.workout.title}`;
          break;

        case 'nutrition':
          notification.icon = 'restaurant_menu';
          notification.description = `Today's Nutrition: ${notification.nutrition_plan.title}`;
          break;

        case 'update_photo':
          notification.icon = 'photo_camera';
          notification.description = 'Time to upload progress photo';
          break;

        case 'update_metrics':
          notification.icon = 'straighten';
          notification.description = 'Time to update measurements';
          break;

        case 'started_schedule':
          notification.icon = 'new_releases';
          notification.description = `${notification.customer.user.first_name} ${notification.customer.user.last_name} started their assigned schedule`;
          break;

        case 'trainer_week_before':
          notification.icon = 'last_page';
          notification.description = `This is ${notification.customer.user.first_name} ${notification.customer.user.last_name}s last week, check in on them and see how they're doing!`;
          break;

        case 'trainer_day_before':
          notification.icon = 'last_page';
          notification.description = `Tomorrow is ${notification.customer.user.first_name} ${notification.customer.user.last_name}s last day, check in on them and see how they're doing!`;
          break;

        case 'customer_updated_metrics_from_schedule':
        case 'customer_updated_metrics':
          notification.icon = 'straighten';
          notification.description = `${notification.customer.user.first_name} ${notification.customer.user.last_name} has updated their measurements!`;
          break;

        case 'customer_uploaded_photo':
          notification.icon = 'photo_camera';
          notification.description = `${notification.customer.user.first_name} ${notification.customer.user.last_name} has uploaded a photo`;
          break;

        case 'customer_completed_workout':
          notification.icon = 'fitness_center';
          notification.description = `${notification.customer.user.first_name} ${notification.customer.user.last_name} has logged a workout`;
          break;

        case 'customer_logged_nutrition':
          notification.icon = 'restaurant_menu';
          notification.description = `${notification.customer.user.first_name} ${notification.customer.user.last_name} has logged nutrition for the day`;
          break;

        case 'customer_uploaded_file':
          notification.icon = 'cloud_upload';
          notification.description = `${notification.customer.user.first_name} ${notification.customer.user.last_name} has uploaded a file`;
          break;

        case 'file_assigned':
          notification.icon = 'cloud_upload';
          notification.description = 'You have been assigned a new file';
      }
    };

    var resultsPerPage = 12;
    vm.loadingMore = false;

    vm.pagination = {
      page: 1,
      nextPage: null,
      loading: false
    };

    var filterSupportedNotifications = function (data) {

      // filter out item types that are not yet setup on this frontend
      return $filter('filter')(data, function (item) {
        var isSupported = ['assigned_schedule', 'workout_am', 'workout_pm', 'nutrition', 'update_photo', 'update_metrics', 'started_schedule', 'trainer_week_before', 'trainer_day_before', 'customer_updated_metrics_from_schedule', 'customer_updated_metrics', 'customer_uploaded_photo', 'customer_completed_workout', 'customer_logged_nutrition', 'customer_uploaded_file', 'file_assigned'].indexOf(item.type) !== -1;

        if (!isSupported) {
          return false;
        }

        // Filter out if the relation doesn't exist for any reason
        switch (item.type) {
          case 'workout_am':
          case 'workout_pm':
            if (!item.workout) {
              return false;
            }
            break;
          case 'nutrition':
            if (!item.nutrition_plan) {
              return false;
            }
            break;
        }

        return true;
      });
    };

    var loadNotifications = function (paginating) {
      paginating = paginating || false;

      if (!paginating) {
        vm.pagination.page = 1;
        vm.pagination.nextPage = null;
      }

      vm.pagination.loading = true;

      var notificationIndex = ApiClientService.notifications.index(vm.currentUser.id, vm.pagination.page, resultsPerPage);

      notificationIndex.then(function (response) {
        if (response.notifications.data.length > 0) {
          response.notifications.data = filterSupportedNotifications(response.notifications.data);
        }

        if (!paginating) {
          vm.notifications = [];
          vm.noNotifications = response.notifications.data.length === 0;
        }

        if (vm.pagination.page >= response.notifications.last_page) {
          vm.pagination.nextPage = null;
        } else {
          vm.pagination.nextPage = vm.pagination.page + 1;
        }

        response.notifications.data.map(transformNotification);

        vm.notifications.push.apply(vm.notifications, response.notifications.data);
      });

      notificationIndex.finally(function (response) {
        vm.pagination.loading = false;
        $scope.$broadcast('scroll.refreshComplete');
        $ionicScrollDelegate.resize();
      });

      return notificationIndex;
    };

    vm.canLoadMore = function () {
      if (vm.pagination.loading === false && vm.pagination.nextPage !== null) {
        return true;
      }

      return false;
    };

    vm.loadMore = function () {
      vm.pagination.page++;
      loadNotifications(true);
    };

    vm.reload = function () {
      loadNotifications();
    };

    $scope.$on('$ionicView.beforeEnter', function (scopes, states) {
      loadNotifications();
    });

    $scope.$on('$ionicView.afterEnter', function (scopes, states) {
      NotificationsService.markSeen();
    });

    var hideMenuFunc;

    var markAsRead = function (notification) {
      // Mark as read
      notification.read = 1;

      // Decrement the counter
      vm.unreadNotificationCount--;

      // Send off the api call to mark as read
      ApiClientService.notifications.markAsRead(notification.id);
    };

    var remove = function (notification) {

      vm.notifications.splice(vm.notifications.indexOf(notification), 1);

      // Decrement the counter
      vm.unreadNotificationCount--;

      // Send off the api call to delete
      ApiClientService.notifications.delete(notification.id);
    };

    vm.notificationMenu = function (notification, event) {
      event.stopPropagation();

      var buttons = [];

      if (!notification.read) {
        buttons.push({
          text: 'Mark As Read',
          click: function () {
            markAsRead(notification);
          }
        });
      }

      buttons.push({
        text: 'Remove',
        click: function () {
          remove(notification);
        }
      });

      var config = {
        cancelText: 'Cancel',
        titleText: 'Notification',
        buttons: buttons,
        buttonClicked: function (index) {
          buttons[index].click();
          if (typeof hideMenuFunc === 'function') {
            hideMenuFunc();
          }
        }
      };

      hideMenuFunc = $ionicActionSheet.show(config);
    };

    var markAllAsRead = function () {
      // Decrement the counter
      vm.unreadNotificationCount = 0;

      // Send off the api call to mark as read
      ApiClientService.notifications.markAllAsRead(vm.currentUser.id);

      angular.forEach(vm.notifications, function (notification) {
        notification.read = 1;
      });
    };

    var removeAll = function () {
      // Decrement the counter
      vm.unreadNotificationCount = 0;

      // Send off the api call to delete all
      ApiClientService.notifications.deleteAll(vm.currentUser.id);

      vm.notifications.length = 0;
      vm.noNotifications = true;
    };

    vm.allMenu = function () {
      var buttons = [];

      buttons.push({
        text: 'Mark All As Read',
        click: function () {
          markAllAsRead();
        }
      });

      buttons.push({
        text: 'Remove All',
        click: function () {
          removeAll();
        }
      });

      var config = {
        cancelText: 'Cancel',
        titleText: 'Notification',
        buttons: buttons,
        buttonClicked: function (index) {
          buttons[index].click();
          if (typeof hideMenuFunc === 'function') {
            hideMenuFunc();
          }
        }
      };

      hideMenuFunc = $ionicActionSheet.show(config);
    };

    vm.notificationTap = function (notification) {

      markAsRead(notification);

      var stateName,
          stateConfig = {};

      switch (notification.type) {
        case 'assigned_schedule':
          stateName = 'app.view_customer_schedule';
          stateConfig = {
            customerId: vm.currentUser.customer_id,
            scheduleId: notification.schedule_id
          };
          break;

        case 'workout_am':
        case 'workout_pm':
          stateName = 'app.view_workout_schedule';
          stateConfig = {
            workoutId: notification.workout.id,
            date: notification.localDate, scheduleId: notification.schedule_id,
            ampm: notification.type === 'workout_am' ? 'am' : '',
            customerId: vm.currentUser.customer_id
          };
          break;

        case 'nutrition':
          NutritionLogService.startLogger(notification.localDate);
          return false;
          break;

        case 'update_photo':
          stateName = 'app.view_customer_schedule';
          stateConfig = {
            customerId: vm.currentUser.customer_id,
            scheduleId: notification.schedule_id,
            trigger: 'update_photo',
            triggerDate: notification.localDate
          };
          break;

        case 'update_metrics':
          stateName = 'app.client_update_metrics';
          stateConfig = {};
          break;

        case 'started_schedule':
          stateName = 'app.view_customer_schedule';
          stateConfig = {
            customerId: notification.customer_id,
            scheduleId: notification.schedule_id
          };
          break;

        case 'trainer_week_before':
          stateName = 'app.view_customer_schedule';
          stateConfig = {
            customerId: notification.customer_id,
            scheduleId: notification.schedule_id
          };
          break;

        case 'trainer_day_before':
          stateName = 'app.view_customer_schedule';
          stateConfig = {
            customerId: notification.customer_id,
            scheduleId: notification.schedule_id
          };
          break;

        case 'customer_updated_metrics_from_schedule':
          stateName = 'app.trainer_update_customer_metrics';
          stateConfig = {
            customerId: notification.customer_id
          };
          break;

        case 'customer_updated_metrics':
          stateName = 'app.trainer_update_customer_metrics';
          stateConfig = {
            customerId: notification.customer_id
          };
          break;

        case 'customer_uploaded_photo':
          stateName = 'app.trainer_client_profile';
          stateConfig = {
            customerId: notification.customer_id,
            tab: 'photos'
          };
          break;

        case 'customer_completed_workout':
          stateName = 'app.session_preview';
          stateConfig = {
            sessionId: notification.session_id
          };
          break;

        case 'customer_logged_nutrition':
          stateName = 'app.trainer_view_nutrition_log_day';
          stateConfig = {
            date: notification.nutrition_day_log_date,
            customerId: notification.customer_id
          };
          break;

        case 'customer_uploaded_file':
          stateName = 'app.files';
          stateConfig = {};
          break;

        case 'file_assigned':
          stateName = 'app.client_files';
          stateConfig = {};
      }

      if (!stateName) {
        return false;
      }

      NavigateService.go(stateName, stateConfig);
    };
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('NotificationsService', service);

    service.$inject = ['ApiClientService', 'PermissionsService', '$rootScope', 'AuthenticationService'];

    function service(ApiClientService, PermissionsService, $rootScope, AuthenticationService) {
        var service = {};

        var isSeen = true;
        var dirty = true;

        var fireStateChangeHandlers = function (isSeen) {
            $rootScope.$emit('NotificationsServiceSeenStateChange', isSeen);
        };

        service.isSeen = function () {
            return isSeen;
        };

        service.markSeen = function () {
            isSeen = true;
            fireStateChangeHandlers(isSeen);
        };

        service.markUnseen = function () {
            isSeen = false;
            fireStateChangeHandlers(isSeen);
        };

        service.onSeenStateChange = function (scope, handler) {
            var unsub = $rootScope.$on('NotificationsServiceSeenStateChange', handler);

            scope.$on('$destroy', unsub);

            return unsub;
        };

        service.dirty = function () {
            dirty = true;
            $rootScope.$emit('NotificationsServiceDirty');
        };

        service.onDirty = function (scope, handler) {
            var unsub = $rootScope.$on('NotificationsServiceDirty', handler);

            scope.$on('$destroy', unsub);

            return unsub;
        };

        service.isDirty = function () {
            return dirty;
        };

        var loading = false;
        service.load = function () {
            if (!dirty || loading) {
                return false;
            }

            loading = true;

            ApiClientService.notifications.countUnread(PermissionsService.getLoggedInUserId()).then(function (response) {
                dirty = false;
                if (response.unread > 0) {
                    service.markUnseen();
                } else {
                    service.markSeen();
                }

                AuthenticationService.updateCurrentUser(response.currentUser);
            }).finally(function () {
                loading = false;
            });
        };

        return service;
    }
})();
(() => {
  'use strict';

  angular.module('MyPTHub').config(config).controller('NutritionPlanPreviewController', controller);

  config.$inject = ['$stateProvider'];

  function config($stateProvider) {
    $stateProvider.state('app.view_nutrition_plan', {
      url: "/nutrition/view/:nutritionPlanId",
      views: {
        'menuContent': {
          templateUrl: "common/nutritionPlan/view.html",
          controller: 'NutritionPlanPreviewController as vm'
        }
      },
      params: {
        date: null
      }
    });
  }

  controller.$inject = ['$scope', 'ApiClientService', '$stateParams', 'ToastService', 'NavigateService', 'NutritionEditorService', 'AnalyticsService', 'NoteViewerModalService', 'ModalService'];

  function controller($scope, ApiClientService, $stateParams, ToastService, NavigateService, NutritionEditorService, AnalyticsService, NoteViewerModalService, ModalService) {

    var vm = this;
    const mimes = {
      image: ['image/gif', 'image/x-icon', 'image/jpeg', 'image/jpg', 'image/png', 'image/svg+xml', 'image/tiff', 'image/webp'],
      document: ['application/msword', 'application/vnd.oasis.opendocument.text', 'application/rtf', 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'],
      pdf: ['application/pdf'],
      archive: ['application/x-tar', 'application/x-bzip2', 'application/gzip', 'application/x-7z-compressed', 'application/x-apple-diskimage', 'application/x-rar-compressed', 'application/zip'],
      spreadsheet: ['application/vnd.oasis.opendocument.spreadsheet', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet', 'application/vnd.ms-excel']
    };

    vm.viewNote = () => {
      if (!vm.plan.note) {
        return false;
      }

      NoteViewerModalService.show(vm.plan.note);

      return true;
    };

    $scope.$on('$ionicView.beforeEnter', function () {

      vm.nutritionPlanId = parseInt($stateParams.nutritionPlanId);
      vm.date = $stateParams.date;

      vm.plan = {};

      vm.ready = false;

      //var currentItem = NutritionEditorService.getItem();

      //if (!currentItem || currentItem.id !== vm.nutritionPlanId) {

      ApiClientService.nutrition.getPlan($stateParams.nutritionPlanId).success(function (response) {

        vm.plan = NutritionEditorService.setItem(response.nutritionPlan, 'plan');

        let files = response.nutritionPlan.files;
        files.map(file => {
          if (mimes.image.includes(file.mime)) {
            file.icon = 'fa-file-image-o';
          } else if (mimes.spreadsheet.includes(file.mime)) {
            file.icon = 'fa-file-excel-o';
          } else if (mimes.pdf.includes(file.mime)) {
            file.icon = 'fa-file-pdf-o';
          } else if (mimes.archive.includes(file.mime)) {
            file.icon = 'fa-file-archive-o';
          } else if (mimes.document.includes(file.mime)) {
            file.icon = 'fa-file-text-o';
          } else {
            file.icon = 'fa-file-o';
          }
          file.name = `${file.original_name}.${file.extension}`;
        });

        vm.files = files;

        vm.plan.canEdit = false;
        vm.ready = true;
      }).error(function (response) {

        if (response.error && response.error.error_name && response.error.error_name === 'SUBSCRIPTION_UPGRADE_REQUIRED') {
          ToastService.show(response.error.message);
          NavigateService.goBack();
        } else {
          ToastService.show('Could not get nutrition plan, please try again later');
        }
      });
      /*
       } else {
        console.log('restored item');
       vm.plan = NutritionEditorService.getItem();
       vm.ready = true;
        }
       */
    });

    $scope.$on('$ionicView.enter', function () {
      AnalyticsService.trackView('View Nutrition Plan');
    });

    vm.viewFile = function (file) {
      ModalService.show({
        templateUrl: 'common/files/filesViewModal.html',
        controller: 'FilesViewController as vm',
        params: {
          file,
          trainer: false
        }
      });
    };
  }
})();
(() => {
  angular.module('MyPTHub').factory('PackagesService', service);

  function service($filter, CurrencyService) {
    const service = {};

    const currencyAmount = (amount, currency = false) => {
      const currencyData = CurrencyService.getCurrency(currency);
      return `${currencyData.symbol + $filter('currency')(amount, '')} ${currencyData.dollar ? currencyData.currency : ''}`;
    };

    const applyDiscount = function (price, coupon) {
      let discount = coupon.amount;

      if (coupon.amount_type === 'percentage') {
        discount = coupon.amount * price / 100;
      }

      return price - discount;
    };

    service.getPrice = function (packageObj, coupon = false) {
      const price = packageObj.price;

      if (coupon) {
        return applyDiscount(price, coupon) / 100;
      }

      return price / 100;
    };

    service.renderPrice = function (packageObj, coupon = false) {
      const normalPeriod = service.buildPriceDescription(packageObj);

      if (packageObj.free) {
        return `Free ${normalPeriod}`;
      }

      if (coupon && packageObj.type === 'recurring' && coupon.duration === 'once') {
        // Need to render differently as only first billing cycle has
        // the discounted price.

        const fullPrice = currencyAmount(service.getPrice(packageObj), packageObj.currency);
        const discountedPrice = currencyAmount(service.getPrice(packageObj, coupon), packageObj.currency);

        const firstPeriod = service.buildPeriodDescription(packageObj);

        return `${discountedPrice} first ${firstPeriod} (Usually ${fullPrice} ${normalPeriod})`;
      }

      return `${currencyAmount(service.getPrice(packageObj, coupon), packageObj.currency)} ${normalPeriod}`;
    };

    service.buildPeriodDescription = function (packageObj) {
      let desc = '';

      if (packageObj.interval_amount > 1 || packageObj.type === 'fixedterm') {
        desc += `${packageObj.interval_amount} `;
      }

      desc += packageObj.interval_unit + (packageObj.interval_amount > 1 ? 's' : '');

      return desc;
    };

    service.buildPriceDescription = function (packageObj) {
      if (packageObj.type === 'recurring') {
        return `per ${service.buildPeriodDescription(packageObj)}`;
      }

      if (packageObj.type === 'fixedterm') {
        return `for ${service.buildPeriodDescription(packageObj)}`;
      }

      return '';
    };

    return service;
  }
})();
(function () {
  angular.module('MyPTHub').factory('PaymentCardService', service);

  service.$inject = [];

  function service() {
    const service = {};

    service.getClass = brand => {
      switch (brand) {
        case 'Visa':
          return 'visa';
        case 'American Express':
          return 'amex';
        case 'MasterCard':
          return 'mastercard';
        case 'Discover':
          return 'generic';
        case 'JCB':
          return 'generic';
        case 'Diners Club':
          return 'generic';
        default:
          return 'generic';
      }
    };

    return service;
  }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').factory('PhotoViewerModalService', service).controller('PhotoViewerModalController', controller);

  service.$inject = ['ModalService'];

  function service(ModalService) {
    return {
      open: function (photoUrl, photoDate, options) {
        return ModalService.show({
          templateUrl: "common/photoViewer/photoViewerModal.html",
          controller: 'PhotoViewerModalController as vm',
          params: {
            url: photoUrl,
            date: photoDate,
            share: options
          }
        });
      }
    };
  }

  controller.$inject = ['parameters'];

  function controller(parameters) {
    var vm = this;
    vm.photo = parameters.url;
    vm.date = parameters.date;
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('SchedulerService', service);

    service.$inject = ['ToastService'];

    function service(ToastService) {

        var service = {};

        service.dayIsNotInFuture = function (date) {
            if (moment(date).isBefore(moment()) || moment(date).isSame(moment(), 'day')) {
                return true;
            }

            return false;
        };

        service.canLogDate = function (date) {
            if (this.dayIsNotInFuture(date)) {
                return true;
            }

            ToastService.show('Oops! This day is in the future');

            return false;
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('ViewClientScheduleController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.view_customer_schedule', {
            url: "/customer/:customerId/schedule/:scheduleId",
            views: {
                'menuContent': {
                    templateUrl: "common/scheduler/viewSchedule.html",
                    controller: 'ViewClientScheduleController as vm'
                }
            },
            params: {
                trigger: false,
                triggerDate: false
            }
        });
    }

    controller.$inject = ['$stateParams', 'UnitService', 'ApiClientService', 'ClientViewScheduleService', 'ToastService', '$scope', 'NavigateService', 'ConfirmDialogService', 'PermissionsService', 'ScheduleActionProgressPhotoService', 'ViewClientScheduleWeightGraphModalService', 'ViewClientScheduleChangeStartDateModalService', 'NotificationsService', 'ScrollService', 'ModalService'];

    function controller($stateParams, UnitService, ApiClientService, ClientViewScheduleService, ToastService, $scope, NavigateService, ConfirmDialogService, PermissionsService, ScheduleActionProgressPhotoService, ViewClientScheduleWeightGraphModalService, ViewClientScheduleChangeStartDateModalService, NotificationsService, ScrollService, ModalService) {
        var vm = this;

        var load = function (noCache, update = false) {

            noCache = noCache || false;

            return ClientViewScheduleService.get($stateParams.scheduleId, noCache).then(function (schedule) {
                if (vm.scheduleData && angular.equals(schedule, vm.scheduleData)) {
                    if (!update) {
                        return true;
                    }
                }

                vm.scheduleData = schedule;
                vm.scheduleStarted = vm.scheduleData.started;

                // Percentages
                vm.workoutsLoggedPercent = vm.scheduleData.workoutsLogged / vm.scheduleData.totalWorkoutsToLog * 100;
                if (vm.workoutsLoggedPercent > 100) {
                    vm.workoutsLoggedPercent = 100;
                }

                vm.workoutsMissedPercent = vm.scheduleData.missedWorkouts.length / vm.scheduleData.totalWorkoutsToLog * 100;
                if (vm.workoutsMissedPercent > 100) {
                    vm.workoutsMissedPercent = 100;
                }

                vm.nutritionPlansLoggedPercent = vm.scheduleData.nutritionPlansLogged / vm.scheduleData.totalNutritionPlansToLog * 100;
                if (vm.nutritionPlansLoggedPercent > 100) {
                    vm.nutritionPlansLoggedPercent = 100;
                }

                vm.nutritionPlansMissedPercent = vm.scheduleData.missedNutritionPlans.length / vm.scheduleData.totalNutritionPlansToLog * 100;
                if (vm.nutritionPlansMissedPercent > 100) {
                    vm.nutritionPlansMissedPercent = 100;
                }

                // Weight units
                vm.latestWeight = UnitService.unitDisplay('bodyweight', vm.scheduleData.latestWeight, false, true);
                vm.startedWeight = UnitService.unitDisplay('bodyweight', vm.scheduleData.startedWeight, false, true);
                vm.weightChange = UnitService.unitDisplay('bodyweight', vm.scheduleData.weightChange, false, true);
                vm.weightUnit = UnitService.unitLabel('bodyweight');

                return true;
            }, function () {
                ToastService.show('Oops! Unable to load that schedule');
            });
        };

        vm.refresh = function () {
            load(true, true).finally(function () {
                $scope.$broadcast('scroll.refreshComplete');
            });
        };

        $scope.$on('ScheduleLabelUpdated', () => {
            load(true, true);
        });

        $scope.$on('$ionicView.beforeEnter', function () {

            vm.isCustomer = PermissionsService.userIsCustomer();

            vm.processing = true;

            load().then(function () {
                vm.processing = false;

                if (vm.isCustomer) {
                    vm.canEdit = true;
                    vm.canChangeStartDate = vm.scheduleData.customer_can_change_start_date;
                    vm.canEditTemplate = false;
                } else {
                    vm.canEdit = PermissionsService.userIsTrainerAdminOrOwner(vm.scheduleData.trainer_user_id);
                    vm.canChangeStartDate = vm.canEdit;
                    vm.canEditTemplate = vm.canEdit && !vm.scheduleData.ended;
                }

                // If this data is marked dirty, get a fresh uncached version
                if (ClientViewScheduleService.isDirty()) {
                    load(true, true);
                }

                if (vm.canEdit) {
                    var triggerDay;
                    if ($stateParams.triggerDate) {
                        angular.forEach(vm.scheduleData.weeks, function (week) {
                            angular.forEach(week.days, function (day) {
                                if (day.date === $stateParams.triggerDate) {
                                    triggerDay = day;
                                }
                            });
                        });
                    }

                    if ($stateParams.trigger === 'update_photo') {
                        ScrollService.bottom();
                        $stateParams.trigger = false;
                        ScheduleActionProgressPhotoService.startUpload(typeof triggerDay !== 'undefined' ? triggerDay : false, vm.scheduleData);
                    }
                }
            });
        });

        vm.viewScheduleDays = function () {
            NavigateService.go('app.view_customer_schedule_days', {
                customerId: vm.scheduleData.customer_id,
                scheduleId: vm.scheduleData.id
            });
        };

        vm.startNow = function (autoStarting, beforeToday) {
            autoStarting = autoStarting || false;
            beforeToday = beforeToday || false;

            var config = {
                title: 'Start now',
                message: 'Are you sure you want to start this schedule now?',
                buttonConfirm: "Start Now",
                buttonCancel: "Cancel"
            };

            if (autoStarting) {
                config.title = 'Start now?';
                if (beforeToday) {
                    config.message = 'This schedule is now set to start on a date in the past, do you want to start it now?';
                } else {
                    config.message = 'This schedule is now set to start today, do you want to start it now?';
                }
            }

            ConfirmDialogService.open(config).then(function () {
                vm.starting = true;
                ApiClientService.customers.startSchedule(vm.scheduleData.id).then(function () {
                    load(true, true).finally(function () {
                        vm.starting = false;
                        NotificationsService.dirty();
                    });
                }, function () {
                    ToastService.show('Oops! Something went wrong, please try again');
                    vm.starting = false;
                });
            });
        };

        vm.changeStartDate = function () {
            ViewClientScheduleChangeStartDateModalService.open(vm.scheduleData).then(function (newStartDate) {
                vm.scheduleData.start_date = newStartDate;

                if (vm.scheduleData.start_date) {
                    // If the date was changed to today, then we'll start the schedule
                    var nowForCustomer = moment().utc().tz(vm.scheduleData.customerTimezone);
                    var startDateForCustomer = moment.tz(vm.scheduleData.start_date, vm.scheduleData.customerTimezone);

                    if (startDateForCustomer.isSame(nowForCustomer, 'day')) {
                        vm.startNow(true);
                    } else if (startDateForCustomer.isBefore(nowForCustomer, 'day')) {
                        vm.startNow(true, true);
                    }
                }
            });
        };

        vm.goToMetrics = function () {
            if (vm.isCustomer) {
                NavigateService.go('app.client_update_metrics', {
                    scheduleId: vm.scheduleData.id,
                    goBack: true
                });
            } else {
                NavigateService.go('app.trainer_update_customer_metrics', {
                    customerId: vm.scheduleData.customer_id,
                    scheduleId: vm.scheduleData.id,
                    goBack: true
                });
            }
        };

        vm.addPhoto = function () {
            ScheduleActionProgressPhotoService.startUpload(false, vm.scheduleData);
        };

        vm.viewPhoto = function () {
            var date = moment.utc(vm.scheduleData.latestPhotoUpdatedTime, 'YYYY-MM-DD HH:mm:ss').local().format('Do MMMM YYYY');

            ModalService.show({
                templateUrl: "common/photoViewer/photoViewerModal.html",
                controller: 'PhotoViewerModalController as vm',
                params: {
                    url: vm.scheduleData.latestPhotoLarge,
                    date: date
                }
            });
        };

        vm.openWeightGraph = function () {
            ViewClientScheduleWeightGraphModalService.open(vm.scheduleData, vm.canEdit).then(function () {
                // new weight
                load(true, true);
            });
        };

        vm.goToEdit = function () {
            ClientViewScheduleService.dirty();
            NavigateService.go('app.trainer_scheduler_update_schedule', {
                templateId: vm.scheduleData.id,
                goBackOnSave: true
            });
        };

        ScheduleActionProgressPhotoService.registerUploadEventHandler($scope, function (event, isUploading) {
            // status changed to not uploading from uploading, let's reload
            if (vm.photoUploading && !isUploading) {
                load(true, true).then(function () {
                    vm.photoUploading = false;
                });
            } else {
                vm.photoUploading = isUploading;
            }
        });
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('ViewClientScheduleChangeStartDateModalService', service).controller('ViewClientScheduleChangeStartDateModalController', controller);

    service.$inject = ['ModalService'];

    function service(ModalService) {
        return {
            open: function (scheduleData) {
                return ModalService.show({
                    templateUrl: "common/scheduler/viewClientScheduleChangeStartDateModal.html",
                    controller: 'ViewClientScheduleChangeStartDateModalController as vm',
                    params: {
                        scheduleData: scheduleData
                    }
                });
            }
        };
    }

    controller.$inject = ['parameters', '$scope', 'ApiClientService', 'ToastService'];

    function controller(parameters, $scope, ApiClientService, ToastService) {

        var vm = this;

        vm.startDate = moment(parameters.scheduleData.start_date).toDate();

        vm.save = function () {

            vm.saving = true;

            var newStartDate = moment(vm.startDate).format('YYYY-MM-DD HH:mm:ss');

            var doSave = ApiClientService.customers.scheduleUpdate(parameters.scheduleData.id, {
                start_date: newStartDate
            });

            doSave.then(function (response) {
                ToastService.show('Nice! Schedule Start Date Update Successfully');
                $scope.closeModal(newStartDate);
            }, function (response) {
                ToastService.show('Oops, something went wrong');
            }).finally(function () {
                vm.saving = false;
            });
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('ViewClientScheduleDaysController', controller).config(config);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.view_customer_schedule_days', {
            url: "/customer/:customerId/schedule/:scheduleId/days",
            views: {
                'menuContent': {
                    templateUrl: "common/scheduler/viewScheduleDays.html",
                    controller: 'ViewClientScheduleDaysController as vm'
                }
            },
            params: {
                goToDate: false
            }
        });
    }

    controller.$inject = ['NavigateService', '$stateParams', 'PermissionsService', '$scope', 'ClientViewScheduleService', 'ScheduleActionProgressPhotoService', 'ProgressPhotoUploadService', '$timeout', 'SchedulerService', 'NutritionLogService', 'NotificationsService', '$location', '$ionicScrollDelegate', 'ModalService'];

    function controller(NavigateService, $stateParams, PermissionsService, $scope, ClientViewScheduleService, ScheduleActionProgressPhotoService, ProgressPhotoUploadService, $timeout, SchedulerService, NutritionLogService, NotificationsService, $location, $ionicScrollDelegate, ModalService) {

        var vm = this;

        var load = function (noCache) {
            noCache = noCache || false;

            return ClientViewScheduleService.get($stateParams.scheduleId, noCache).then(function (schedule) {
                if (vm.scheduleData && angular.equals(schedule, vm.scheduleData)) {
                    return true;
                }
                vm.scheduleData = schedule;
                vm.scheduleStarted = vm.scheduleData.started;

                return true;
            });
        };

        vm.refresh = function () {
            load(true).finally(function () {
                $scope.$broadcast('scroll.refreshComplete');
            });
        };

        /*
         If we're entering this view from an uncached state, move
         to current week if possible. If a date has been supplied,
         find that date in a week and go to that week.
         */
        var goToWeek = function (states) {
            var whichWeek;
            var passedCurrent = false;
            var currentDayNo;
            for (var i = 0; i < vm.scheduleData.weeks.length; i++) {

                var week = vm.scheduleData.weeks[i];

                for (var di = 0; di < week.days.length; di++) {
                    var day = week.days[di];

                    if (day.current) {
                        passedCurrent = true;
                        currentDayNo = di + 1;
                    }

                    if ($stateParams.goToDate && $stateParams.goToDate === day.date) {
                        whichWeek = i;
                    }
                }

                // If we're entering this view from an uncached state, move
                // to current week.
                if (!states.fromCache && passedCurrent && !$stateParams.goToDate) {
                    if (i > 0) {
                        whichWeek = i;
                    }
                    break;
                }
            }

            if (whichWeek) {
                $timeout(function () {
                    vm.slider.slideTo(whichWeek);
                }, 100);
            }

            // Scroll to the current day if this week contains the current day.
            // Find the scroll delegate by ID.
            if (currentDayNo && currentDayNo > 1) {
                $timeout(function () {
                    var instances = $ionicScrollDelegate["_instances"];
                    var scrollWeek = whichWeek || 0;

                    var instance = null;
                    for (var index in instances) {
                        if (instances[index].$element[0].id === 'week' + (scrollWeek + 1)) {
                            instance = instances[index];
                        }
                    }

                    if (instance) {
                        $location.hash('week' + (scrollWeek + 1) + 'day' + currentDayNo);
                        instance.anchorScroll(true);
                    }
                });
            }
        };

        $scope.$on('$ionicView.beforeEnter', function (scopes, states) {
            vm.isCustomer = PermissionsService.userIsCustomer();

            vm.loaded = false;
            vm.scheduleStarted = false;

            load().then(function () {
                vm.loaded = true;

                if (vm.isCustomer) {
                    vm.canEdit = true;
                    vm.canEditTemplate = false;
                } else {
                    vm.canEdit = PermissionsService.userIsTrainerAdminOrOwner(vm.scheduleData.trainer_user_id);
                    vm.canEditTemplate = vm.canEdit && !vm.scheduleData.ended;
                }

                goToWeek(states);

                // If this data is marked dirty, get a fresh uncached version
                if (ClientViewScheduleService.isDirty()) {
                    load(true);
                }
            });
        });

        // Reload the uncached version when a photo upload is done
        ProgressPhotoUploadService.registerUploadSuccessEventHandler($scope, function () {
            load(true);
            NotificationsService.dirty();
        });

        vm.goToWorkout = function (day, ampm, date) {

            ClientViewScheduleService.dirty();

            if (ampm == 'am') {
                var workout = day.workout_am;
                var workout_completed = day.workout_am_completed;
                var workout_completed_session_id = day.workout_am_completed_session_id;
            }

            if (ampm == 'pm') {
                var workout = day.workout_pm;
                var workout_completed = day.workout_pm_completed;
                var workout_completed_session_id = day.workout_pm_completed_session_id;
            }

            // Check if the workout has been completed, if so we want to view it
            if (workout_completed) {
                NavigateService.go('app.session_preview', {
                    sessionId: workout_completed_session_id
                });
            } else {
                NavigateService.go('app.view_workout_schedule', {
                    workoutId: workout.id,
                    date: date,
                    scheduleId: vm.scheduleData.id,
                    ampm: ampm,
                    customerId: vm.scheduleData.customer_id
                });
            }
        };

        vm.goToNutrition = function (nutritionPlanId, date) {
            if (SchedulerService.dayIsNotInFuture(date)) {

                ClientViewScheduleService.dirty();

                if (vm.isCustomer) {
                    NutritionLogService.startLogger(date);
                    return false;
                } else {
                    NavigateService.go('app.trainer_view_nutrition_log_day', {
                        date: date,
                        customerId: vm.scheduleData.customer_id
                    });
                }
            } else {
                NavigateService.go('app.view_nutrition_plan', {
                    nutritionPlanId: nutritionPlanId,
                    date: date
                });
            }
        };

        vm.goToMetrics = function (date) {
            if (!SchedulerService.canLogDate(date)) {
                return false;
            }

            ClientViewScheduleService.dirty();

            if (vm.isCustomer) {
                NavigateService.go('app.client_update_metrics', {
                    scheduleId: vm.scheduleData.id,
                    goBack: true
                });
            } else {
                NavigateService.go('app.trainer_update_customer_metrics', {
                    customerId: vm.scheduleData.customer_id,
                    scheduleId: vm.scheduleData.id,
                    goBack: true
                });
            }
        };

        vm.progressPhotoAction = function (day) {
            if (!SchedulerService.canLogDate(day.date)) {
                return false;
            }

            ScheduleActionProgressPhotoService.startUpload(day, vm.scheduleData);
        };

        vm.goToEdit = function () {
            ClientViewScheduleService.dirty();
            NavigateService.go('app.trainer_scheduler_update_schedule', {
                templateId: vm.scheduleData.id,
                goBackOnSave: true,
                goToWeek: vm.currentSlide + 1
            });
        };

        vm.slideOptions = {
            loop: false,
            effect: 'slide',
            speed: 500,
            pagination: '.week-pagination',
            paginationClickable: true
        };

        $scope.$on("$ionicSlides.sliderInitialized", function (event, data) {
            // data.slider is the instance of Swiper
            vm.slider = data.slider;
            $timeout(function () {
                vm.currentSlide = data.slider.activeIndex;
                vm.currentlySliding = false;
            });
        });

        $scope.$on("$ionicSlides.slideChangeStart", function (event, data) {
            $timeout(function () {
                vm.currentlySliding = true;
            });
        });

        $scope.$on("$ionicSlides.slideChangeEnd", function (event, data) {
            $timeout(function () {
                vm.currentSlide = data.slider.activeIndex;
                vm.currentlySliding = false;
            });
        });

        vm.viewFile = function (file) {
            console.log(file);
            ModalService.show({
                templateUrl: 'common/files/filesViewModal.html',
                controller: 'FilesViewController as vm',
                params: {
                    file,
                    trainer: false
                }
            });
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('ViewClientScheduleMissedItemsController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.view_customer_schedule_missed_items', {
            url: "/customer/:customerId/schedule/:scheduleId/missed/:mode",
            views: {
                'menuContent': {
                    templateUrl: "common/scheduler/viewScheduleMissedItems.html",
                    controller: 'ViewClientScheduleMissedItemsController as vm'
                }
            }
        });
    }

    controller.$inject = ['$stateParams', 'ApiClientService', 'ClientViewScheduleService', 'ToastService', '$scope', 'NavigateService', 'ConfirmDialogService', 'PermissionsService'];

    function controller($stateParams, ApiClientService, ClientViewScheduleService, ToastService, $scope, NavigateService, ConfirmDialogService, PermissionsService) {
        var vm = this;

        var load = function (noCache) {
            noCache = noCache || false;

            return ClientViewScheduleService.get($stateParams.scheduleId, noCache).then(function (schedule) {
                if (vm.scheduleData && angular.equals(schedule, vm.scheduleData)) {
                    return true;
                }

                vm.scheduleData = angular.copy(schedule);

                vm.missed = {};
                var missedCount = 0;

                if (vm.mode === 'workouts') {
                    vm.itemClass = 'workout';
                    vm.entityName = 'Workout';

                    missedCount = vm.scheduleData.missedWorkouts.length;

                    for (var i = 0; i < missedCount; i++) {
                        var missedWorkout = vm.scheduleData.missedWorkouts[i];

                        if (!vm.missed.hasOwnProperty(missedWorkout.date)) {
                            vm.missed[missedWorkout.date] = [];
                        }

                        vm.missed[missedWorkout.date].push(missedWorkout);
                    }
                }

                if (vm.mode === 'nutritionLogs') {
                    vm.itemClass = 'nutrition';
                    vm.entityName = 'Nutrition Plan';

                    missedCount = vm.scheduleData.missedNutritionPlans.length;

                    for (var i = 0; i < missedCount; i++) {
                        var missedNutritionLog = vm.scheduleData.missedNutritionPlans[i];

                        if (!vm.missed.hasOwnProperty(missedNutritionLog.date)) {
                            vm.missed[missedNutritionLog.date] = [];
                        }

                        vm.missed[missedNutritionLog.date].push(missedNutritionLog);
                    }
                }

                if (missedCount === 0) {
                    vm.noneMissed = true;
                }

                vm.weeks = [];
                var passedCurrent = false;

                for (var i = 0; i < vm.scheduleData.weeks.length; i++) {
                    if (passedCurrent) {
                        break;
                    }

                    var week = vm.scheduleData.weeks[i];
                    var newWeek = {
                        number: i + 1,
                        noneMissed: false,
                        days: []
                    };

                    for (var di = 0; di < week.days.length; di++) {
                        if (passedCurrent) {
                            break;
                        }

                        var day = week.days[di];
                        day.number = di + 1;

                        if (vm.missed.hasOwnProperty(day.date)) {
                            day.missed = true;
                        }

                        if (day.current) {
                            passedCurrent = true;
                        }

                        if (day.missed) {
                            newWeek.days.push(day);
                        }
                    }

                    if (newWeek.days.length === 0) {
                        newWeek.noneMissed = true;
                    }

                    vm.weeks.push(newWeek);
                }

                return true;
            }, function () {
                ToastService.show('Oops! Unable to load that schedule');
            });
        };

        vm.goToItem = function (missed) {
            switch (vm.mode) {
                case 'workouts':
                    NavigateService.go('app.view_workout_schedule', {
                        workoutId: missed.workout_id,
                        date: missed.date,
                        scheduleId: vm.scheduleData.id,
                        ampm: missed.am_pm,
                        customerId: vm.scheduleData.customer_id
                    });
                    break;
                case 'nutritionLogs':
                    NavigateService.go('app.view_nutrition_plan', {
                        nutritionPlanId: missed.nutrition_plan_id
                    });
                    break;
            }
        };

        $scope.$on('$ionicView.beforeEnter', function () {

            vm.mode = $stateParams.mode;

            switch ($stateParams.mode) {
                case 'workouts':
                    vm.entityName = 'Workout';
                    vm.itemClass = 'workout';
                    break;

                case 'nutritionLogs':
                    vm.entityName = 'Nutrition Plan';
                    vm.itemClass = 'nutrition';
                    break;
            }

            vm.isCustomer = PermissionsService.userIsCustomer();

            vm.processing = true;

            load().then(function () {
                vm.processing = false;

                if (vm.isCustomer) {
                    vm.canEdit = true;
                } else {
                    vm.canEdit = PermissionsService.userIsTrainerAdminOrOwner(vm.scheduleData.trainer_user_id);
                }

                // If this data is marked dirty, get a fresh uncached version
                if (ClientViewScheduleService.isDirty()) {
                    load(true);
                }
            });
        });

        vm.slideOptions = {
            loop: false,
            effect: 'slide',
            speed: 500,
            pagination: '.week-pagination',
            paginationClickable: true
        };

        $scope.$on("$ionicSlides.sliderInitialized", function (event, data) {
            // data.slider is the instance of Swiper
            $scope.slider = data.slider;
        });
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('ClientViewScheduleService', service);

    /**
     * Wrapper service that loads a schedule into memory so it can be used
     * across views without a reload.
     */
    service.$inject = ['$q', 'ApiClientService'];

    function service($q, ApiClientService) {

        var schedule;
        var dirty = false;

        var service = {};

        /**
         * Fetches the cached schedule when it can, otherwise loads from the
         * remote API.
         * @param scheduleId
         * @param [noCache] optionally you can ignore the cached version
         * @returns promise
         */
        service.get = function (scheduleId, noCache) {

            noCache = noCache || false;
            scheduleId = parseInt(scheduleId);

            var deferred = $q.defer();

            // Resolve successful cache get instantly
            if (!noCache && schedule && schedule.id === scheduleId) {
                deferred.resolve(schedule);
                return deferred.promise;
            }

            // Fetch remote version from API
            ApiClientService.customers.getSpecificSchedule(scheduleId).then(function (response) {

                schedule = response.schedule;

                dirty = false;

                deferred.resolve(schedule);
            }, deferred.reject);

            return deferred.promise;
        };

        /**
         * Marks the existing data as dirty, so users of the service can
         * reload the data if they choose to.
         */
        service.dirty = function () {
            dirty = true;
        };

        service.isDirty = function () {
            return dirty;
        };

        return service;
    }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').factory('ViewClientScheduleWeightGraphModalService', service).controller('ViewClientScheduleWeightGraphModalController', controller);

  service.$inject = ['ModalService'];

  function service(ModalService) {
    return {
      open: function (scheduleData, canEdit) {
        canEdit = canEdit || false;
        return ModalService.show({
          templateUrl: 'common/scheduler/viewClientScheduleWeightGraphModal.html',
          controller: 'ViewClientScheduleWeightGraphModalController as vm',
          params: {
            scheduleData: scheduleData,
            canEdit: canEdit
          }
        });
      }
    };
  }

  controller.$inject = ['parameters', 'ToastService', 'UnitService', 'ApiClientService', '$scope', '$rootScope'];

  function controller(parameters, ToastService, UnitService, ApiClientService, $scope, $rootScope) {

    var vm = this;

    vm.scheduleData = parameters.scheduleData;
    vm.canEdit = parameters.canEdit;

    // Weight units
    vm.latestWeight = UnitService.unitDisplay('bodyweight', vm.scheduleData.latestWeight, false, true);
    vm.startedWeight = UnitService.unitDisplay('bodyweight', vm.scheduleData.startedWeight, false, true);
    vm.weightChange = UnitService.unitDisplay('bodyweight', vm.scheduleData.weightChange, false, true);
    vm.weightUnit = UnitService.unitLabel('bodyweight');

    vm.newWeight = angular.copy(vm.scheduleData.latestWeight);

    var startDate = moment(vm.scheduleData.start_date).startOf('day');

    var endDate = moment(vm.scheduleData.end_date, 'YYYY-MM-DD').endOf('day');

    vm.graphTitle = startDate.format('Do MMM') + ' - ' + endDate.format('Do MMM');

    vm.weightChangeChart = {};
    vm.weightChangeChart.data = [];
    vm.weightChangeChart.labels = [];
    vm.weightChangeChart.series = [];
    vm.weightChangeChart.options = [];
    vm.chartColors = ['#7a7bb7'];

    const setup = () => {
      var getWeight = ApiClientService.analytics.bodyweight(vm.scheduleData.customer_id, startDate.format('YYYY-MM-DD'), endDate.format('YYYY-MM-DD'));

      getWeight.success(function (response) {
        vm.weightChangeChart = {};
        vm.weightChangeChart.ready = false;
        vm.weightChangeChart.labels = [];
        vm.weightChangeChart.series = ['Total'];
        vm.weightChangeChart.data = [[]];
        vm.weightChangeChart.data_raw = [];
        vm.weightChangeChart.options = {
          maintainAspectRatio: false,
          responsive: true,
          tooltips: {
            callbacks: {
              title: function (tooltipItem, data) {
                return vm.weightChangeChart.data_raw[tooltipItem[0].index].tooltipTitle;
              },
              label: function (tooltipItem, data) {
                return vm.weightChangeChart.labels[tooltipItem.index];
              }
            }
          },
          scales: {
            xAxes: [{
              ticks: {
                autoSkip: true,
                maxTicksLimit: 10
              }
            }],
            yAxes: [{
              ticks: {
                beginAtZero: false,
                autoSkip: true,
                maxTicksLimit: 3,
                callback: function (value, index, values) {
                  return value + UnitService.unitLabel('bodyweight'); //vm.weightUnit;
                }
              }
            }]
          }

        };

        var chartData = response.data;

        vm.weightChangeChart.ready = true;

        chartData.map(function (rangeTick) {
          rangeTick.tooltipTitle = UnitService.unitDisplay('bodyweight', rangeTick.weight) + UnitService.unitLabel('bodyweight');
          rangeTick.total = UnitService.unitDisplay('bodyweight', rangeTick.weight, false, false, true);

          var label = moment.utc(rangeTick.date_raw).local().format('Do MMM');
          vm.weightChangeChart.labels.push(label);
          vm.weightChangeChart.data[0].push(rangeTick.total);
          vm.weightChangeChart.data_raw.push(rangeTick);
        });
        console.log(vm.weightChangeChart);
      });
    };

    setup();

    var setupUnits = function () {
      vm.bodyWeightSystem = UnitService.getCurrentUnitType('bodyweight');
      vm.latestWeight = UnitService.unitDisplay('bodyweight', vm.scheduleData.latestWeight, false, true);
      vm.startedWeight = UnitService.unitDisplay('bodyweight', vm.scheduleData.startedWeight, false, true);
      vm.weightChange = UnitService.unitDisplay('bodyweight', vm.scheduleData.weightChange, false, true);
      setup();
      $rootScope.$broadcast('ScheduleLabelUpdated');
    };
    setupUnits();

    $scope.$on('displayUnitChanged', setupUnits);

    vm.updateWeight = function () {
      if (!vm.newWeight) {
        return false;
      }

      vm.saving = true;

      var saveMetrics = ApiClientService.metrics.update(vm.scheduleData.customer_id, {
        date: moment().format('YYYY-MM-DD'),
        weight: vm.newWeight
      });

      saveMetrics.success(function (response) {
        vm.saving = false;
        $rootScope.$broadcast('ScheduleLabelUpdated');
        ToastService.show('Weight saved successfully');
        $scope.closeModal(true);
      });

      saveMetrics.error(function (response) {
        vm.saving = false;
        ToastService.show(response.error.message);
      });
    };

    //AnalyticsService.trackView('Schedule - View Schedule Weight Graph');
  }
})();
(() => {
  angular.module('MyPTHub').config(config).controller('SessionBookController', controller);

  config.$inject = ['$stateProvider'];

  function config($stateProvider) {

    $stateProvider.state('app.trainer_book_session', {
      url: "/book_session/:trainerUserId?",
      views: {
        'menuContent': {
          templateUrl: "common/session/book.html",
          controller: 'SessionBookController as vm'
        }
      }
    });
  }

  controller.$inject = ['ApiClientService', 'WorkoutStoreService', 'NetworkStateService', 'CustomerStoreService', '$scope', '$stateParams', 'NavigateService', 'ToastService', 'AnalyticsService', 'PermissionsService'];

  function controller(ApiClientService, WorkoutStoreService, NetworkStateService, CustomerStoreService, $scope, $stateParams, NavigateService, ToastService, AnalyticsService, PermissionsService) {

    var vm = this;
    var currentUser = PermissionsService.getCurrentUser();

    vm.session = {
      date_raw: moment().startOf('minute').toDate(),
      time_raw: moment().startOf('minute').toDate(),
      useWorkout: true
    };

    // Set defaults for RRULEs
    vm.selectedRecurranceType = 'daily';

    // rs = recurring selected
    vm.rsWeekDays = {
      monday: {
        rruleFormat: 'MO',
        dayNum: 1,
        selected: false
      },
      tuesday: {
        rruleFormat: 'TU',
        dayNum: 2,
        selected: false
      },
      wednesday: {
        rruleFormat: 'WE',
        dayNum: 3,
        selected: false
      },
      thursday: {
        rruleFormat: 'TH',
        dayNum: 4,
        selected: false
      },
      friday: {
        rruleFormat: 'FR',
        dayNum: 5,
        selected: false
      },
      saturday: {
        rruleFormat: 'SA',
        dayNum: 6,
        selected: false
      },
      sunday: {
        rruleFormat: 'SU',
        dayNum: 7,
        selected: false
      }
    };

    var weekdaysHaveBeenSelected = function () {
      var selectedCount = 0;
      _.each(vm.rsWeekDays, function (day, key) {
        if (day.selected) {
          selectedCount++;
        }
      });

      return selectedCount > 0;
    };

    vm.rsMonthDays = {
      1: false,
      2: false,
      3: false,
      4: false,
      5: false,
      6: false,
      7: false,
      8: false,
      9: false,
      10: false,
      11: false,
      12: false,
      13: false,
      14: false,
      15: false,
      16: false,
      17: false,
      18: false,
      19: false,
      20: false,
      21: false,
      22: false,
      23: false,
      24: false,
      25: false,
      26: false,
      27: false,
      28: false,
      29: false,
      30: false,
      31: false
    };

    var monthdaysHaveBeenSelected = function () {
      var selectedCount = 0;
      _.each(vm.rsMonthDays, function (day, key) {
        if (day) {
          selectedCount++;
        }
      });

      return selectedCount > 0;
    };

    vm.rsYearMonths = {
      1: false,
      2: false,
      3: false,
      4: false,
      5: false,
      6: false,
      7: false,
      8: false,
      9: false,
      10: false,
      11: false,
      12: false
    };

    var monthsHaveBeenSelected = function () {
      var selectedCount = 0;
      _.each(vm.rsYearMonths, function (month, key) {
        if (month) {
          selectedCount++;
        }
      });

      return selectedCount > 0;
    };

    vm.rsDayInt = 1;
    vm.rsWeekInt = 1;
    vm.rsMonthInt = 1;
    vm.rsYearInt = 1;

    vm.timeHours = ['00', '01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', '23'];

    vm.timeMinutes = ['00', '05', '10', '15', '20', '25', '30', '35', '40', '45', '50', '55'];

    vm.recurringBooking = false;

    /*
     * Generate an RRULE string from the given data
     * Sample: 'FREQ=MONTHLY;BYDAY=FR;BYMONTHDAY=13'
     *
     * Possible Options:
     * FREQ = [DAILY, WEEKLY, MONTHLY, YEARLY]
     * INTERVAL = Int, e.g. 4 = every 4 days, every 4 years
     * BYDAY = 2 char upper case comma seperated, MO,TU,WE
     * BYWEEKNO = Week number, 1-52
     * BYMONTH = Month number, 1-12, comma seperated 2,3,7
     * BYMONTHDAY = Day of month 1-31, comma seperated 2,3,7
     * BYYEARDAY = Year day number, 1-365, e.g. 1,100,200 = 1st, 100th and 200th day
     * BYHOUR = Hour Number 24 hour format, 1-24
     * BYMINUTE = Minute Number, 0-59
     */

    var convertWeekDayNameToFull = function (weekday) {
      switch (weekday) {
        case 'MO':
          return "Monday";
        case 'TU':
          return "Tuesday";
        case 'WE':
          return "Wednesday";
        case 'TH':
          return "Thursday";
        case 'FR':
          return "Friday";
        case 'SA':
          return "Saturday";
        case 'SU':
          return "Sunday";
      }
    };

    var generateRrule = function () {
      var rrule = '';

      if (vm.recurringBooking) {
        switch (vm.selectedRecurranceType) {
          case 'daily':
            // FREQ
            rrule = rrule + 'FREQ=DAILY;';

            // INTERVAL
            rrule = rrule + 'INTERVAL=' + vm.rsDayInt + ';';

            break;
          case 'weekly':
            // FREQ
            rrule = rrule + 'FREQ=WEEKLY;';

            // INTERVAL
            rrule = rrule + 'INTERVAL=' + vm.rsWeekInt + ';';

            // BYDAY
            var tmpToAdd = [];
            angular.forEach(vm.rsWeekDays, function (weekday) {
              if (weekday.selected) {
                tmpToAdd.push(weekday.rruleFormat);
              }
            });

            // Convert days to UTC
            var utcWeekDays = [];
            angular.forEach(tmpToAdd, function (day) {
              utcWeekDays.push(moment().isoWeekday(convertWeekDayNameToFull(day)).hour(moment(vm.session.time_raw).hour()).minute(moment(vm.session.time_raw).minute())
              // .utc()
              .format('dd').toUpperCase());
            });

            // Set a default
            if (utcWeekDays.length) {
              rrule = rrule + 'BYDAY=' + utcWeekDays.join(',') + ';';
            } else {
              rrule = rrule + 'BYDAY=MO;';
            }

            break;
          case 'monthly':
            // FREQ
            rrule = rrule + 'FREQ=MONTHLY;';

            // INTERVAL
            rrule = rrule + 'INTERVAL=' + vm.rsMonthInt + ';';

            // BYMONTHDAY
            var tmpToAdd = [];
            angular.forEach(vm.rsMonthDays, function (monthDaySelected, monthDay) {
              if (monthDaySelected) {
                tmpToAdd.push(monthDay);
              }
            });

            // Convert days to UTC
            var utcMonthDays = [];
            angular.forEach(tmpToAdd, function (day) {
              utcMonthDays.push(moment().date(day).hour(moment(vm.session.time_raw).hour()).minute(moment(vm.session.time_raw).minute()).utc().format('D'));
            });

            // Set a default
            if (utcMonthDays.length) {
              rrule = rrule + 'BYMONTHDAY=' + utcMonthDays.join(',') + ';';
            } else {
              rrule = rrule + 'BYMONTHDAY=1;';
            }

            break;
          case 'yearly':
            // FREQ
            rrule = rrule + 'FREQ=YEARLY;';

            // INTERVAL
            rrule = rrule + 'INTERVAL=' + vm.rsYearInt + ';';

            // BYMONTH
            var tmpToAdd = [];
            angular.forEach(vm.rsYearMonths, function (yearMonthelected, yearMonth) {
              if (yearMonthelected) {
                tmpToAdd.push(yearMonth);
              }
            });

            // Set a default
            if (tmpToAdd.length) {
              rrule = rrule + 'BYMONTH=' + tmpToAdd.join(',') + ';';
            } else {
              rrule = rrule + 'BYMONTH=1;';
            }

            break;
        }

        // Add the hour
        rrule = rrule + 'BYHOUR=' + moment(vm.session.time_raw).hour() + ';';

        // Add the minute
        rrule = rrule + 'BYMINUTE=' + moment(vm.session.time_raw).minute() + ';';
      }

      return rrule;
    };

    vm.book = function () {

      if (vm.selectedRecurranceType === 'weekly' && !weekdaysHaveBeenSelected()) {
        ToastService.show('Please select the day(s) of the week you want the session to repeat on');
        return false;
      }

      if (vm.selectedRecurranceType === 'monthly' && !monthdaysHaveBeenSelected()) {
        ToastService.show('Please select the day(s) of the month you want the session to repeat on');
        return false;
      }

      if (vm.selectedRecurranceType === 'yearly' && !monthsHaveBeenSelected()) {
        ToastService.show('Please select the month(s) of the year you want the session to repeat on');
        return false;
      }

      vm.booking = true;

      var timeObj = moment(vm.session.time_raw);

      var date = moment.tz(moment.utc(vm.session.date_raw), currentUser.timezone);

      if (timeObj) {
        date.hour(timeObj.hour()).minute(timeObj.minute());
      }

      var workoutId = null;

      if (vm.session.useWorkout) {
        workoutId = vm.session.workout_id;
      }

      var generatedRrule = generateRrule();

      var endDate = false;
      if (vm.setEndDate && vm.endDate) {
        var endDateObj = moment(vm.endDate);
        endDate = endDateObj.endOf('day').format('YYYY-MM-DD HH:mm:ss');
      }

      var bookSession = ApiClientService.sessions.book(workoutId, vm.session.customer_id, date, false, vm.trainerUserId, generatedRrule, endDate, '', '', vm.location);

      bookSession.success(function (response) {

        ToastService.show('Session booked successfully');

        vm.booking = false;

        NavigateService.goBack();
      });

      bookSession.error(function (error) {

        ToastService.show('Unable to book the session, please try again later');

        vm.booking = false;
      });
    };

    var sortWorkouts = function (a, b) {
      var textA = a.title.toUpperCase();
      var textB = b.title.toUpperCase();
      return textA < textB ? -1 : textA > textB ? 1 : 0;
    };

    var allWorkouts = [];
    vm.customerChosen = function () {

      vm.workouts = allWorkouts;

      if (!NetworkStateService.isOnline()) {
        return false;
      }

      if (!vm.session.customer_id) {
        return false;
      }

      /*
       Since we are online it is possible to load the client
       specifically assigned workouts also.
       */

      ApiClientService.customers.listWorkouts(vm.session.customer_id, false, false).success(function (response) {

        if (response.workouts.length === 0) {
          return false;
        }

        var workoutList = [{
          divider: true,
          title: "• Assigned Workouts •",
          id: null
        }];

        var clientAssigned = response.workouts;

        clientAssigned.sort(sortWorkouts);

        workoutList.push.apply(workoutList, clientAssigned);

        workoutList.push({
          divider: true,
          title: "• All Workouts •",
          id: null
        });

        workoutList.push.apply(workoutList, allWorkouts);

        vm.workouts = workoutList;
      });
    };

    var loadWorkouts = function (init) {

      /**
       * Grabs from the local DB first, and then if online it will
       * get the latest data from the API, to make for a better
       * experience by having something available immediately.
       */

      init = init || false;

      WorkoutStoreService.listAllTrainerWorkouts(vm.trainerUserId).then(function (response) {

        response.sort(sortWorkouts);

        vm.workouts = response;

        vm.workouts = response;
        allWorkouts = response;

        if (allWorkouts.length > 0) {
          vm.session.workout_id = allWorkouts[0].id;
          vm.selectedWorkout = allWorkouts[0];
        }
      });

      if (init && NetworkStateService.isOnline()) {
        WorkoutStoreService.downloadTrainerWorkouts(vm.trainerUserId).then(function () {
          loadWorkouts();
        });
      }
    };

    vm.updateWorkout = function () {

      var workout = vm.selectedWorkout;

      if (!workout) {
        return false;
      }

      vm.session.workout_id = workout.id;
      vm.selectedWorkoutDays = [];

      if (workout.data) {

        if (workout.data.day_1) {
          vm.selectedWorkoutDays.push('Monday');
        }
        if (workout.data.day_2) {
          vm.selectedWorkoutDays.push('Tuesday');
        }
        if (workout.data.day_3) {
          vm.selectedWorkoutDays.push('Wednesday');
        }
        if (workout.data.day_4) {
          vm.selectedWorkoutDays.push('Thursday');
        }
        if (workout.data.day_5) {
          vm.selectedWorkoutDays.push('Friday');
        }
        if (workout.data.day_6) {
          vm.selectedWorkoutDays.push('Saturday');
        }
        if (workout.data.day_7) {
          vm.selectedWorkoutDays.push('Sunday');
        }
      }
    };

    var loadCustomers = function (init) {

      /**
       * Grabs from the local DB first, and then if online it will
       * get the latest data from the API, to make for a better
       * experience by having something available immediately.
       */

      init = init || false;

      CustomerStoreService.listAllTrainerCustomers(vm.trainerUserId).then(function (response) {
        vm.customers = response;
      });

      if (init && NetworkStateService.isOnline()) {

        CustomerStoreService.downloadTrainerCustomers(vm.trainerUserId).then(function () {
          loadCustomers();
        });
      }
    };

    $scope.$on('$ionicView.beforeEnter', function () {

      vm.trainerUserId = $stateParams.trainerUserId;

      vm.selectedWworkout = null;
      vm.selectedWorkoutDays = [];

      vm.session = {
        date_raw: moment().startOf('minute').toDate(),
        time_raw: moment().startOf('minute').toDate(),
        useWorkout: true
      };

      loadWorkouts(true);
      loadCustomers(true);
    });

    $scope.$on('$ionicView.enter', function () {
      AnalyticsService.trackView('Session - Book');
    });
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('SessionLoggerConsultationController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.session_logger_consultation', {
            url: "/session_logger/:sessionId/consultation",
            views: {
                'menuContent': {
                    templateUrl: "common/session/consultation.html",
                    controller: 'SessionLoggerConsultationController as vm'
                }
            }
        });
    }

    controller.$inject = ['SessionLoggerService', 'ToastService', '$scope', 'PermissionsService', 'NavigateService', 'AnalyticsService', 'SessionLoggerDebitControlService'];

    function controller(SessionLoggerService, ToastService, $scope, PermissionsService, NavigateService, AnalyticsService, SessionLoggerDebitControlService) {

        var vm = this;

        $scope.$on('$ionicView.beforeEnter', function () {
            vm.session = SessionLoggerService.getSession();
            vm.feedback = '';
        });

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Session - Consultation');
        });

        vm.complete = function () {

            vm.processing = true;

            var completeSession = SessionLoggerService.completeSession(vm.feedback, null, SessionLoggerDebitControlService.getIsBillable(), SessionLoggerDebitControlService.getBillableToQuotaId());

            completeSession.then(function (response) {

                var isBillable = SessionLoggerDebitControlService.getIsBillable();

                SessionLoggerDebitControlService.reset();

                var msg = 'Session saved';

                if (!response) {
                    msg = 'Offline - Your session will be saved when there is an internet connection';
                }

                ToastService.show(msg);

                if (PermissionsService.userIsTrainer()) {

                    var params = {
                        customerId: vm.session.customer_id
                    };

                    if (isBillable) {
                        params.tab = 'sessions';
                    }

                    NavigateService.isRoot().disableBack().go('app.trainer_client_profile', params);
                } else {

                    NavigateService.goHome();
                }
            }, function () {

                vm.processing = false;

                ToastService.show('Could not complete the session. Please try again');
            });
        };
    }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').config(config).controller('SessionEditController', controller);

  config.$inject = ['$stateProvider'];

  function config($stateProvider) {

    $stateProvider.state('app.trainer_edit_session', {
      url: "/edit_session/:sessionId",
      views: {
        'menuContent': {
          templateUrl: "common/session/edit.html",
          controller: 'SessionEditController as vm'
        }
      }
    });
  }

  controller.$inject = ['ApiClientService', '$scope', '$stateParams', '$q', 'PermissionsService', 'NavigateService', 'ToastService', 'AnalyticsService', 'ConfirmDialogService'];

  function controller(ApiClientService, $scope, $stateParams, $q, PermissionsService, NavigateService, ToastService, AnalyticsService, ConfirmDialogService) {

    var vm = this;
    var currentUser = PermissionsService.getCurrentUser();

    vm.update = function () {

      vm.saving = true;

      var timeObj = moment(vm.session.time_raw);

      var date = moment.utc(vm.session.created_at).isAfter('2018-07-31') ? moment.tz(moment.utc(vm.session.date_raw), currentUser.timezone) : moment.utc(vm.session.date_raw);

      var timeChanged = false;
      var workoutChanged = false;

      if (timeObj) {
        date.hour(timeObj.hour()).minute(timeObj.minute());
      }

      var workoutId = null;

      if (vm.session.useWorkout) {
        workoutId = vm.session.workout_id;
      }

      // Ask if the user wants to update all (rrule) or just this one
      if (timeObj.hour() + timeObj.minute() != vm.originalDate.hour() + vm.originalDate.minute()) {
        // Time Changed
        timeChanged = true;
      }

      if (vm.session.workout_id != vm.originalWorkoutId) {
        // Workout Changed
        workoutChanged = true;
      }

      // Prompt for confirmation to update single or all sessions
      if (vm.session.recurring && (timeChanged || workoutChanged)) {

        var title = '';
        var wording = 'Would you like to update the ';

        if (timeChanged && workoutChanged) {
          title = 'Workout and Time';
          wording = wording + 'workout and time';
        } else if (timeChanged) {
          title = 'Time';
          wording = wording + 'time';
        } else {
          title = 'Workout';
          wording = wording + 'workout';
        }

        wording = wording + ' for all sessions or just this one?';

        ConfirmDialogService.open({
          title: title,
          message: wording,
          buttonConfirm: "Update All Sessions",
          buttonCancel: "Only Update This Session"
        }).then(function () {
          // Confirm
          performUpdate(true);
        }, function () {
          // Cancel
          performUpdate(false);
        });
      } else {
        // Perform standard update
        performUpdate(false);
      }

      function performUpdate(updateOccurrences) {
        var sessionIdParts = $stateParams.sessionId.split('_');
        if (sessionIdParts.length === 2) {
          var updateSession = ApiClientService.sessions.update(sessionIdParts[0], workoutId, date, sessionIdParts[1], updateOccurrences, vm.session.location);
        } else {
          var updateSession = ApiClientService.sessions.update(sessionIdParts[0], workoutId, date, null, updateOccurrences, vm.session.location);
        }

        updateSession.success(function (response) {
          ToastService.show('Session updated successfully');

          var resSession = response.session;

          if (resSession.phantom) {
            var date = resSession.date;
            var sessionId = resSession.id + '_' + date.split(' ').join('+');
          } else {
            var sessionId = resSession.id;
          }

          NavigateService.goBack(-2);
        });

        updateSession.error(function (error) {

          ToastService.show('Unable to save the session, please try again later');

          vm.saving = false;
        });
      }
    };

    var sortWorkouts = function (a, b) {
      var textA = a.title.toUpperCase();
      var textB = b.title.toUpperCase();
      return textA < textB ? -1 : textA > textB ? 1 : 0;
    };

    var allWorkouts = [];
    var loadWorkouts = function () {
      return ApiClientService.trainers.getAllWorkouts(vm.trainerUserId).then(function (response) {
        vm.workouts = response.workouts;
        allWorkouts = response.workouts;
      });
    };

    var loadCustomerWorkouts = function () {
      var load = ApiClientService.customers.listWorkouts(vm.session.customer_id, false, false);

      load.then(function (response) {

        if (response.workouts.length === 0) {
          return false;
        }

        var workoutList = [{
          divider: true,
          title: "• Assigned Workouts •",
          id: null
        }];

        var clientAssigned = response.workouts;

        clientAssigned.sort(sortWorkouts);

        workoutList.push.apply(workoutList, clientAssigned);

        workoutList.push({
          divider: true,
          title: "• All Workouts •",
          id: null
        });

        workoutList.push.apply(workoutList, allWorkouts);

        vm.workouts = workoutList;
      });

      return load;
    };

    var loadSession = function () {
      var sessionIdParts = $stateParams.sessionId.split('_');
      if (sessionIdParts.length === 2) {
        var getSession = ApiClientService.sessions.get(sessionIdParts[0], sessionIdParts[1]);
      } else {
        var getSession = ApiClientService.sessions.get(sessionIdParts[0]);
      }

      getSession.success(function (response) {

        vm.session = response.session;
        vm.originalWorkoutId = vm.session.workout_id;

        if (vm.session.workout) {
          vm.session.useWorkout = true;
        }

        if (moment.utc(vm.session.created_at).isAfter('2018-07-31')) {
          vm.session.date_raw = moment.utc(vm.session.date, 'YYYY-MM-DD HH:mm:ss').startOf('minute').toDate();
          vm.session.time_raw = moment.utc(vm.session.date, 'YYYY-MM-DD HH:mm:ss').startOf('minute').toDate();
        } else {
          vm.session.date_raw = moment(vm.session.date, 'YYYY-MM-DD[T]HH:mm:ss').startOf('minute').toDate();
          vm.session.time_raw = moment(vm.session.date, 'YYYY-MM-DD[T]HH:mm:ss').startOf('minute').toDate();
        }
        vm.originalDate = moment(vm.session.time_raw);
      });

      return getSession;
    };

    $scope.$on('$ionicView.beforeEnter', function () {

      vm.ready = false;

      vm.trainerUserId = $stateParams.trainerUserId;
      $q.all([loadSession(), loadWorkouts()]).then(function () {
        vm.ready = true;
        loadCustomerWorkouts();
      });
    });

    $scope.$on('$ionicView.enter', function () {
      AnalyticsService.trackView('Session - Edit');
    });
  }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').config(config).controller('SessionPreviewController', controller);

  config.$inject = ['$stateProvider'];

  function config($stateProvider) {

    $stateProvider.state('app.session_preview', {
      url: "/sessions/preview/:sessionId",
      views: {
        'menuContent': {
          templateUrl: "common/session/preview.html",
          controller: 'SessionPreviewController as vm'
        }
      }
    });
  }

  controller.$inject = ['SessionStartService', '$stateParams', '$scope', 'SessionStoreService', 'NoteViewerModalService', 'PermissionsService', 'SessionCancelService', 'NavigateService', 'AnalyticsService', 'SessionReviewCancelRequestService'];

  function controller(SessionStartService, $stateParams, $scope, SessionStoreService, NoteViewerModalService, PermissionsService, SessionCancelService, NavigateService, AnalyticsService, SessionReviewCancelRequestService) {

    var vm = this;

    vm.session = {};
    vm.sessionData = {};

    vm.isTrainer = PermissionsService.userIsTrainer();
    vm.canEdit = vm.isTrainer;
    var currentUser = PermissionsService.getCurrentUser();
    vm.gender = false;

    $scope.$on('$ionicView.beforeEnter', function () {

      vm.loaded = false;
      vm.session = {};
      vm.sessionData = {};
      vm.isConsultation = false;
      vm.isCompleted = false;

      vm.sessionId = $stateParams.sessionId;

      SessionStoreService.get($stateParams.sessionId).then(function (session) {

        vm.session = session;

        vm.session.date = moment.utc(vm.session.data.created_at).isAfter('2018-07-31') ? moment.utc(vm.session.date).tz(currentUser.timezone).format('Do MMM YYYY - HH:mm') : moment.utc(vm.session.data.date).format('Do MMM YYYY - HH:mm');

        if (!session.data.workout) {
          vm.isConsultation = true;
        }

        vm.sessionData = session.data;

        vm.cancellationRequested = vm.sessionData.cancellation_request;

        vm.isCompleted = session.completed;
        vm.loaded = true;

        // If this user is a trainer, set the gender to the client's gender
        if (PermissionsService.userIsCustomer()) {
          vm.gender = currentUser.gender;
        } else if (vm.sessionData.customer && typeof vm.sessionData.customer.user !== 'undefined') {
          vm.gender = vm.sessionData.customer.user.gender;
        }

        AnalyticsService.trackView('Session - ' + (vm.isCompleted ? 'Review' : 'Preview'));
      });
    });

    vm.goToCalendar = function () {
      if (PermissionsService.userIsTrainer()) {
        NavigateService.disableBack().go('app.trainer_calendar');
      } else {
        NavigateService.disableBack().go('app.client_calendar');
      }
    };

    vm.start = function () {
      SessionStartService.start(vm.sessionId);
    };

    vm.showNote = function () {
      NoteViewerModalService.show(vm.sessionData.workout.note);
    };

    vm.cancel = function () {
      SessionCancelService.startCancel(vm.sessionData);
    };

    vm.reviewCancelRequest = function () {
      SessionReviewCancelRequestService.review(vm.sessionData);
    };

    vm.edit = function () {
      NavigateService.go('app.trainer_edit_session', {
        sessionId: vm.sessionId
      });
    };
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('SessionStartAdHocController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.start_adhoc_session', {
            url: "/start_adhoc_session",
            views: {
                'menuContent': {
                    templateUrl: "common/session/startAdHoc.html",
                    controller: 'SessionStartAdHocController as vm'
                }
            },
            params: {
                customerPreselectId: false
            }
        });
    }

    controller.$inject = ['SessionStartService', 'WorkoutStoreService', 'PermissionsService', 'NetworkStateService', 'CustomerStoreService', '$scope', 'ApiClientService', 'AnalyticsService', '$q', 'ScheduleEventService', 'ScheduleEventStoreService', 'ToastService', '$stateParams'];

    function controller(SessionStartService, WorkoutStoreService, PermissionsService, NetworkStateService, CustomerStoreService, $scope, ApiClientService, AnalyticsService, $q, ScheduleEventService, ScheduleEventStoreService, ToastService, $stateParams) {

        var vm = this;

        var customerWasChosen = false;
        var downloadedScheduledWorkouts = false;

        vm.isTrainer = PermissionsService.userIsTrainer();

        vm.start = function () {
            vm.starting = true;

            var timeObj = moment(vm.session.time_raw);

            var date = moment(vm.session.date_raw);

            if (timeObj) {
                date.hour(timeObj.hour()).minute(timeObj.minute());
            }

            SessionStartService.startAdHoc(vm.session.useWorkout ? vm.session.workout_id : false, vm.session.customer_id, date, vm.session.useWorkout ? vm.session.schedule_id : false, vm.session.useWorkout ? vm.session.schedule_type : false).finally(function () {
                vm.starting = false;
            });
        };

        var getScheduledWorkouts = function () {
            vm.dayAmWorkouts = [];
            vm.dayPmWorkouts = [];
            vm.hasScheduledWorkouts = false;

            if (!vm.session.customer_id) {
                return false;
            }

            var getCached = function () {
                vm.dayAmWorkouts = [];
                vm.dayPmWorkouts = [];
                return ScheduleEventService.getCustomerEventsOnDate(moment(vm.session.date_raw).format('YYYY-MM-DD'), vm.session.customer_id, ['workout_am', 'workout_pm']).then(function (events) {
                    console.log('Date\'s workouts:', events);
                    console.log(vm.session);
                    var currentlySelectedStillAvailable = false;
                    angular.forEach(events, function (event) {
                        if (event.type === 'workout_am' && !event.data.completed) {
                            vm.dayAmWorkouts.push(event);
                            if (vm.session.schedule_id && vm.session.schedule_type === 'am' && vm.session.schedule_id === event.schedule_id && event.data.workout_id === vm.session.workout_id) {
                                currentlySelectedStillAvailable = true;
                            }
                        }
                        if (event.type === 'workout_pm' && !event.data.completed) {
                            vm.dayPmWorkouts.push(event);
                            if (vm.session.schedule_id && vm.session.schedule_type === 'pm' && vm.session.schedule_id === event.schedule_id && event.data.workout_id === vm.session.workout_id) {
                                currentlySelectedStillAvailable = true;
                            }
                        }
                    });
                    if (!currentlySelectedStillAvailable) {
                        vm.session.schedule_id = null;
                        vm.session.schedule_type = null;
                    }
                    vm.hasScheduledAmWorkouts = vm.dayAmWorkouts.length > 0;
                    vm.hasScheduledPmWorkouts = vm.dayPmWorkouts.length > 0;
                    vm.hasScheduledWorkouts = vm.hasScheduledAmWorkouts || vm.hasScheduledPmWorkouts;
                });
            };

            getCached();

            if ((!downloadedScheduledWorkouts || ScheduleEventStoreService.isDirty(vm.session.customer_id)) && NetworkStateService.isOnline()) {
                ScheduleEventStoreService.downloadCustomerEvents(vm.session.customer_id).then(function () {
                    getCached();
                    downloadedScheduledWorkouts = true;
                });
            }
        };

        var sortWorkouts = function (a, b) {
            var textA = a.title.toUpperCase();
            var textB = b.title.toUpperCase();
            return textA < textB ? -1 : textA > textB ? 1 : 0;
        };

        var allWorkouts = [];
        vm.customerChosen = function () {

            customerWasChosen = true;
            downloadedScheduledWorkouts = false;

            vm.workouts = allWorkouts;

            getScheduledWorkouts();

            if (!NetworkStateService.isOnline()) {
                return false;
            }

            if (!vm.session.customer_id) {
                return false;
            }

            /*
             Since we are online it is possible to load the client
             specifically assigned workouts also.
             */

            var workoutRequests = [];

            workoutRequests.push(ApiClientService.customers.listWorkouts(vm.session.customer_id, false, false));

            workoutRequests.push(ApiClientService.customers.listCreatedWorkouts(vm.session.customer_id));

            $q.all(workoutRequests).then(function (response) {
                var assignedWorkouts = response[0].workouts;
                var createdWorkouts = response[1].workouts;

                if (assignedWorkouts.length === 0 && createdWorkouts.length === 0) {
                    return false;
                }

                var workoutList = [];

                if (assignedWorkouts.length > 0) {

                    workoutList.push({
                        divider: true,
                        title: "• Assigned Workouts •",
                        id: null
                    });

                    assignedWorkouts.sort(sortWorkouts);

                    workoutList.push.apply(workoutList, assignedWorkouts);
                }

                if (createdWorkouts.length > 0) {

                    workoutList.push({
                        divider: true,
                        title: "• Client Created Workouts •",
                        id: null
                    });

                    createdWorkouts.sort(sortWorkouts);

                    workoutList.push.apply(workoutList, createdWorkouts);
                }

                workoutList.push({
                    divider: true,
                    title: "• All Workouts •",
                    id: null
                });

                workoutList.push.apply(workoutList, allWorkouts);

                vm.workouts = workoutList;

                //vm.reloadSchedule();
            });
        };

        vm.dateChanged = function () {
            var chosenDate = moment(vm.session.date_raw);
            var now = moment();
            if (chosenDate.isAfter(now) && !chosenDate.isSame(now, 'day')) {
                vm.session.date_raw = moment().startOf('minute').toDate();
                ToastService.show('Cannot use a date in the future');
            }

            getScheduledWorkouts();
        };

        vm.changedWorkout = function () {
            vm.session.schedule_id = null;
            vm.session.schedule_type = null;

            if (!vm.hasScheduledWorkouts) {
                console.log('no scheduled workouts');
                return true;
            }

            // find a matching schedule if possible

            angular.forEach(vm.dayAmWorkouts, function (scheduledWorkout) {
                if (scheduledWorkout.data.workout_id === vm.session.workout_id) {
                    vm.session.schedule_id = scheduledWorkout.schedule_id;
                    vm.session.schedule_type = 'am';
                    return true;
                }
            });

            if (vm.session.schedule_id) {
                return true;
            }

            angular.forEach(vm.dayPmWorkouts, function (scheduledWorkout) {
                if (scheduledWorkout.data.workout_id === vm.session.workout_id) {
                    vm.session.schedule_id = scheduledWorkout.schedule_id;
                    vm.session.schedule_type = 'pm';
                    return true;
                }
            });

            return true;
        };

        vm.chooseScheduledWorkout = function (workoutData, ampm) {
            vm.session.workout_id = workoutData.workout_id;
            vm.session.schedule_id = workoutData.schedule_id;
            vm.session.schedule_type = ampm;
        };

        var loadWorkouts = function (init) {

            /**
             * Grabs from the local DB first, and then if online it will
             * get the latest data from the API, to make for a better
             * experience by having something available immediately.
             */

            init = init || false;

            if (vm.isTrainer) {

                WorkoutStoreService.listAllTrainerWorkouts().then(function (response) {

                    if (customerWasChosen) {
                        return false;
                    }

                    response.sort(sortWorkouts);

                    vm.workouts = response;
                    allWorkouts = response;
                });
            } else {

                WorkoutStoreService.listAllUserWorkouts().then(function (response) {

                    response.sort(sortWorkouts);

                    vm.workouts = response;
                    allWorkouts = response;
                    console.log('current user workouts', allWorkouts);
                });

                WorkoutStoreService.listAllCustomerCreatedWorkouts(PermissionsService.getCurrentUserEntityId()).then(function (response) {

                    if (response.length === 0) {
                        return false;
                    }

                    response.sort(sortWorkouts);

                    var clientCreated = response;

                    var workoutList = [];

                    if (allWorkouts.length > 0) {

                        workoutList.push({
                            divider: true,
                            title: "• Assigned Workouts •",
                            id: null
                        });

                        workoutList.push.apply(workoutList, allWorkouts);
                    }

                    workoutList.push({
                        divider: true,
                        title: "• My Workouts •",
                        id: null
                    });

                    workoutList.push.apply(workoutList, clientCreated);

                    vm.workouts = workoutList;
                });
            }

            if (init && NetworkStateService.isOnline()) {

                if (vm.isTrainer) {

                    WorkoutStoreService.downloadTrainerWorkouts().then(function () {
                        loadWorkouts();
                    });
                } else {

                    var customerId = PermissionsService.getCurrentUserEntityId();

                    WorkoutStoreService.downloadCustomerWorkouts(customerId).then(WorkoutStoreService.downloadCustomerCreatedWorkouts(customerId)).then(function () {
                        loadWorkouts();
                    });
                }
            }
        };

        var loadCustomers = function (init) {

            if (!vm.isTrainer) {
                return false;
            }

            /**
             * Grabs from the local DB first, and then if online it will
             * get the latest data from the API, to make for a better
             * experience by having something available immediately.
             */

            init = init || false;

            CustomerStoreService.listAllTrainerCustomers().then(function (response) {
                vm.customers = response;
            });

            if (init && NetworkStateService.isOnline()) {

                CustomerStoreService.downloadTrainerCustomers().then(function (resp) {
                    // Instead of re-running this function, we are just setting the customers to what is returned
                    // from the API
                    vm.customers = resp;
                    // loadCustomers();
                });
            }
        };

        $scope.$on('$ionicView.beforeEnter', function () {

            vm.session = {
                date_raw: moment().startOf('minute').toDate(),
                time_raw: moment().startOf('minute').toDate(),
                useWorkout: true
            };

            if ($stateParams.customerPreselectId) {
                vm.session.customer_id = parseInt($stateParams.customerPreselectId);
            }

            if (!vm.isTrainer) {
                vm.session.customer_id = PermissionsService.getCurrentUserEntityId();
                getScheduledWorkouts();
            }

            loadWorkouts(true);

            loadCustomers(true);
        });

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Session - Start Ad-Hoc');
        });

        vm.refresh = () => {
            loadCustomers(true);
            loadWorkouts(true);

            $scope.$broadcast('scroll.refreshComplete');
        };
    }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').config(config).controller('WorkoutViewerController', controller);

  config.$inject = ['$stateProvider'];

  function config($stateProvider) {

    $stateProvider.state('app.view_workout', {
      url: '/workout_viewer/:workoutId',
      views: {
        'menuContent': {
          templateUrl: 'common/workoutViewer/workoutViewer.html',
          controller: 'WorkoutViewerController as vm'
        }
      }
    });

    $stateProvider.state('app.view_workout_schedule', {
      url: '/workouts/view/{workoutId:[0-9]+}/{customerId:[0-9]+}/{date}/{scheduleId:[0-9]+}/{ampm}',
      views: {
        'menuContent': {
          templateUrl: 'common/workoutViewer/workoutViewer.html',
          controller: 'WorkoutViewerController as vm'
        }
      }
    });
  }

  controller.$inject = ['$scope', '$stateParams', 'ApiClientService', 'NoteViewerModalService', 'PermissionsService', 'AnalyticsService', 'ConfirmDialogService', 'SessionStartService', 'ModalService'];

  function controller($scope, $stateParams, ApiClientService, NoteViewerModalService, PermissionsService, AnalyticsService, ConfirmDialogService, SessionStartService, ModalService) {

    var vm = this;
    const mimes = {
      image: ['image/gif', 'image/x-icon', 'image/jpeg', 'image/jpg', 'image/png', 'image/svg+xml', 'image/tiff', 'image/webp'],
      document: ['application/msword', 'application/vnd.oasis.opendocument.text', 'application/rtf', 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'],
      pdf: ['application/pdf'],
      archive: ['application/x-tar', 'application/x-bzip2', 'application/gzip', 'application/x-7z-compressed', 'application/x-apple-diskimage', 'application/x-rar-compressed', 'application/zip'],
      spreadsheet: ['application/vnd.oasis.opendocument.spreadsheet', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet', 'application/vnd.ms-excel']
    };

    var currentUser = PermissionsService.getCurrentUser();

    vm.showNote = function () {
      NoteViewerModalService.show(vm.workout.note);
    };

    $scope.$on('$ionicView.beforeEnter', function (event, viewData) {

      viewData.enableBack = true;

      vm.ready = false;
      vm.dateEnabled = false;
      vm.userIsTrainer = PermissionsService.userIsTrainer();
      vm.userIsCustomer = PermissionsService.userIsCustomer();
      vm.gender = currentUser.gender;

      var get = ApiClientService.workouts.get($stateParams.workoutId);

      get.success(function (response) {
        vm.workout = response.workout;

        let files = response.workout.files;
        files.map(file => {
          if (mimes.image.includes(file.mime)) {
            file.icon = 'fa-file-image-o';
          } else if (mimes.spreadsheet.includes(file.mime)) {
            file.icon = 'fa-file-excel-o';
          } else if (mimes.pdf.includes(file.mime)) {
            file.icon = 'fa-file-pdf-o';
          } else if (mimes.archive.includes(file.mime)) {
            file.icon = 'fa-file-archive-o';
          } else if (mimes.document.includes(file.mime)) {
            file.icon = 'fa-file-text-o';
          } else {
            file.icon = 'fa-file-o';
          }
          file.name = `${file.original_name}.${file.extension}`;
        });

        vm.files = files;

        vm.ready = true;

        if ($stateParams.date) {
          vm.dateEnabled = true;
          vm.startDate = moment($stateParams.date, 'YYYY-MM-DD');
          vm.startBtnEnabled = vm.startDate.isBefore(moment().endOf('day'));
          if (vm.userIsTrainer) {
            vm.startBtnEnabled = vm.startBtnEnabled && PermissionsService.userIsTrainerAdminOrOwner(vm.workout.trainer_user_id);
          }
        }
      });
    });

    $scope.$on('$ionicView.enter', function () {
      AnalyticsService.trackView('View Workout');
    });

    vm.startNow = function () {

      ConfirmDialogService.open({
        title: 'Start Workout',
        message: 'Are you sure you want to start this workout now?',
        buttonConfirm: 'Yes, Start',
        buttonCancel: 'No, Cancel'
      }).then(function () {
        vm.starting = true;

        var currentTime = moment();
        var sessionDate = moment($stateParams.date, 'YYYY-MM-DD').hour(currentTime.hour()).minute(currentTime.minute()).second(currentTime.second());

        var scheduleId, ampm;
        if ($stateParams.scheduleId) {
          scheduleId = $stateParams.scheduleId;
          ampm = $stateParams.ampm;
        }

        SessionStartService.startAdHoc($stateParams.workoutId, $stateParams.customerId, sessionDate, scheduleId, ampm).finally(function () {
          vm.starting = false;
        });
      });
    };

    vm.viewFile = function (file) {
      ModalService.show({
        templateUrl: 'common/files/filesViewModal.html',
        controller: 'FilesViewController as vm',
        params: {
          file,
          trainer: false
        }
      });
    };
  }
})();
(function () {

    angular.module('MyPTHub').factory('WorkoutStoreService', service);

    service.$inject = ['ApiClientService', 'NetworkStateService', 'SqlStorageService', 'PermissionsService', '$q', 'AuthenticationService', 'AuthenticationStateService'];

    function service(ApiClientService, NetworkStateService, SqlStorageService, PermissionsService, $q, AuthenticationService, AuthenticationStateService) {

        var eventHandlers = {
            download_complete: []
        };

        var cancelPromises = {
            downloadCustomerWorkouts: null
        };

        var fireEventHandlers = function (type) {

            for (var i = 0; i < eventHandlers[type].length; i++) {

                var handler = eventHandlers[type][i];

                handler(function () {
                    eventHandlers[type].splice(i, 1);
                });
            }
        };

        var registerEventHandler = function (type, handler) {
            eventHandlers[type].push(handler);
        };

        var parseRow = function (row) {

            row.data = JSON.parse(row.data);

            return row;
        };

        var parseRows = function (rows) {
            return rows.map(parseRow);
        };

        var getOffline = function (workoutId) {

            return $q(function (resolve, reject) {

                SqlStorageService.select('SELECT * FROM workouts WHERE id = ? AND user_id = ?', [workoutId, PermissionsService.getLoggedInUserId()]).then(function (result) {

                    resolve(parseRow(result));
                }, function (error) {
                    reject(false);
                });
            });
        };

        var storeWorkout = function (workout, customerId) {

            customerId = customerId || null;

            return $q(function (resolve, reject) {

                getOffline(workout.id).then(function () {

                    SqlStorageService.query("UPDATE workouts SET data = ?, title = ?, enabled = ?, last_data_update = datetime('now') WHERE id = ? AND user_id = ?", [JSON.stringify(workout), workout.title, workout.enabled, workout.id, PermissionsService.getLoggedInUserId()]).then(function () {

                        getOffline(workout.id).then(resolve);
                    }, function (response) {
                        console.log('storeWorkout update ' + workout.id + ' error:', response);
                    });
                }, function () {

                    SqlStorageService.query("INSERT INTO workouts (id, title, user_id, trainer_id, trainer_user_id, customer_id, enabled, data, created_by, last_data_update) VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, datetime('now'))", [workout.id, workout.title, PermissionsService.getLoggedInUserId(), workout.trainer_id, workout.trainer_user_id, customerId, workout.enabled, JSON.stringify(workout), workout.created_by]).then(function () {

                        getOffline(workout.id).then(resolve);
                    }, function (response) {
                        console.log('storeWorkout insert ' + workout.id + ' error:', response);
                    });
                });
            });
        };

        var getLive = function (workoutId) {

            return $q(function (resolve, reject) {

                if (!NetworkStateService.isOnline()) {
                    reject(false);
                    return;
                }

                var getWorkout = ApiClientService.noCache().workouts.get(workoutId);

                getWorkout.success(function (response) {
                    storeWorkout(response.workout).then(resolve);
                });

                getWorkout.error(function (response) {
                    reject(response);
                });
            });
        };

        var cleanupRemotelyDeleted = function (wheres, trainerUserId, createdBy) {

            createdBy = createdBy || 'trainer';

            var deferred = $q.defer();

            var flags = [];
            for (var index = 0; index < wheres.length; ++index) {
                flags.push('?');
            }

            var whereStr = '';

            if (trainerUserId) {
                wheres.push(PermissionsService.getCurrentUserEntityId());
                wheres.push(trainerUserId);
                whereStr = 'AND trainer_id = ? AND trainer_user_id = ?';
            } else {
                wheres.push(PermissionsService.getLoggedInUserId());
                whereStr = 'AND user_id = ?';
            }

            if (createdBy) {
                wheres.push(createdBy);
                whereStr += 'AND created_by = ?';
            }

            SqlStorageService.selectAll('SELECT id FROM workouts ' + 'WHERE id NOT IN (' + flags.join(',') + ') ' + whereStr, wheres).then(function (results) {

                var deletes = [];

                results.forEach(function (item) {
                    deletes.push(deleteLocal(item.id));
                });

                $q.all(deletes).finally(deferred.resolve);
            }, deferred.reject);

            return deferred.promise;
        };

        var deleteLive = function (workoutId) {

            return ApiClientService.workouts.delete(workoutId);
        };

        var deleteLocal = function (workoutId) {

            return SqlStorageService.query("DELETE FROM workouts WHERE id = ?", [workoutId]);
        };

        var service = {

            setup: function () {

                var download = function (resolve) {

                    // Download latest schedule data
                    if (PermissionsService.userIsTrainer()) {

                        service.downloadTrainerWorkouts().finally(resolve);
                    } else {

                        service.downloadCustomerWorkouts(PermissionsService.getCurrentUserEntityId()).then(service.downloadCustomerCreatedWorkouts(PermissionsService.getCurrentUserEntityId())).then(function () {
                            console.log('workout store finished setup');
                        }).finally(resolve);
                    }
                };

                /**
                 * Trigger when login successful
                 */
                AuthenticationService.registerLoginSuccessEventHandler(download);

                if (AuthenticationStateService.isLoggedIn()) {
                    download();
                }
            },

            // Gets workout from local DB (and updates local DB first if online)
            get: function (workoutId) {

                var deferred = $q.defer();

                getLive(workoutId).then(function (workout) {

                    deferred.resolve(workout);
                }, function () {

                    /**
                     * We are offline or can't access API so we can't use
                     * the workout. Try and get it from the local database.
                     */

                    getOffline(workoutId).then(deferred.resolve, deferred.reject);
                });

                return deferred.promise;
            },

            listAllTrainerWorkouts: function (trainerUserId) {

                if (!trainerUserId) {
                    trainerUserId = PermissionsService.getLoggedInUserId();
                }

                var trainerId = PermissionsService.getCurrentUserEntityId();

                var values = [trainerId, trainerUserId];

                return $q(function (resolve, reject) {

                    SqlStorageService.selectAll('SELECT * FROM workouts WHERE trainer_id = ? AND trainer_user_id = ? AND created_by = \'trainer\' ORDER BY title', values).then(function (results) {
                        resolve(parseRows(results));
                    }, function () {
                        reject();
                    });
                });
            },

            listAllCustomerCreatedWorkouts: function (customerId) {

                var values = [PermissionsService.getLoggedInUserId(), customerId];

                return $q(function (resolve, reject) {

                    SqlStorageService.selectAll('SELECT * FROM workouts WHERE user_id = ? AND customer_id = ? AND created_by = \'customer\' ORDER BY title', values).then(function (results) {
                        resolve(parseRows(results));
                    }, function () {
                        reject();
                    });
                });
            },

            listAllUserWorkouts: function () {

                var values = [PermissionsService.getLoggedInUserId()];

                return $q(function (resolve, reject) {

                    SqlStorageService.selectAll('SELECT * FROM workouts WHERE user_id = ? AND created_by = \'trainer\' ORDER BY title', values).then(function (results) {
                        resolve(parseRows(results));
                    }, function () {
                        reject();
                    });
                });
            },

            downloadTrainerWorkouts: function (trainerUserId) {

                var deferred = $q.defer();

                if (!NetworkStateService.isOnline()) {
                    deferred.reject();
                    return deferred.promise;
                }

                trainerUserId = trainerUserId || false;

                console.log('download trainer workouts started');

                ApiClientService.noCache().trainers.getAllWorkouts(trainerUserId, true).success(function (response) {

                    console.log('download trainer workouts success', response);

                    var stored = [];
                    var currentIds = [];

                    angular.forEach(response.workouts, function (workout) {
                        stored.push(storeWorkout(workout));
                        currentIds.push(workout.id);
                    });

                    $q.all(stored).then(function (storedResponse) {

                        console.log('all storeWorkout resolved');

                        cleanupRemotelyDeleted(currentIds, trainerUserId, 'trainer').finally(function () {

                            deferred.resolve(storedResponse);

                            fireEventHandlers('download_complete');
                        });
                    });
                }, function (response) {
                    console.log('download trainer workouts fail', response);
                });

                return deferred.promise;
            },

            downloadCustomerWorkouts: function (customerId) {

                var deferred = $q.defer();

                if (!NetworkStateService.isOnline()) {
                    deferred.reject();
                    return deferred.promise;
                }

                ApiClientService.noCache().customers.listWorkouts(customerId, false, true).success(function (response) {

                    var stored = [];
                    var currentIds = [];

                    angular.forEach(response.workouts, function (workout) {
                        stored.push(storeWorkout(workout));
                        currentIds.push(workout.id);
                    });

                    $q.all(stored).then(function (storedResponse) {

                        cleanupRemotelyDeleted(currentIds, false, 'trainer').finally(function () {

                            deferred.resolve(storedResponse);

                            fireEventHandlers('download_complete');
                        });
                    });
                });

                return deferred.promise;
            },

            downloadCustomerCreatedWorkouts: function (customerId) {

                var deferred = $q.defer();

                if (!NetworkStateService.isOnline()) {
                    deferred.reject();
                    return deferred.promise;
                }

                ApiClientService.noCache().customers.listCreatedWorkouts(customerId, true).success(function (response) {

                    var stored = [];
                    var currentIds = [];

                    angular.forEach(response.workouts, function (workout) {
                        stored.push(storeWorkout(workout, customerId));
                        currentIds.push(workout.id);
                    });

                    $q.all(stored).then(function (storedResponse) {

                        cleanupRemotelyDeleted(currentIds, false, 'customer').finally(function () {

                            deferred.resolve(storedResponse);

                            fireEventHandlers('download_complete');
                        });
                    });
                });

                return deferred.promise;
            },

            delete: function (workoutId) {

                return $q.all([deleteLive(workoutId), deleteLocal(workoutId)]);
            },

            registerEventHandler: registerEventHandler

        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('TrainerCreateClientController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.trainer_create_client', {
            url: "/clients/create/:trainerUserId?",
            views: {
                'menuContent': {
                    templateUrl: "trainer/clients/create/create.html",
                    controller: 'TrainerCreateClientController as vm'
                }
            }
        });
    }

    controller.$inject = ['ApiClientService', 'ToastService', 'UnitService', '$scope', 'NavigateService', 'AnalyticsService', 'CustomerStoreService', 'AuthenticationStateService'];

    function controller(ApiClientService, ToastService, UnitService, $scope, NavigateService, AnalyticsService, CustomerStoreService, AuthenticationStateService) {

        var vm = this;

        var setupUnits = function () {
            vm.bodyWeightSystem = UnitService.getCurrentUnitType('bodyweight');
            vm.heightSystem = UnitService.getCurrentUnitType('height');
        };

        ApiClientService.trainerUsers.list().success(response => {
            vm.trainers = response;
        });

        vm.loggedInUser = AuthenticationStateService.getCurrentUser();

        vm.doCreate = function (form) {

            //ToastService.show(msg);
            vm.creating = true;

            vm.fieldErrors = {};

            var createCustomer = ApiClientService.customers.create(vm.customer);

            createCustomer.success(function (response) {

                vm.creating = false;

                form.$setUntouched();

                ToastService.show('Your client was successfully added');

                CustomerStoreService.setDirty();

                NavigateService.goHome();
            });

            createCustomer.error(function (response) {

                vm.creating = false;

                ToastService.show(response.error.message);

                vm.fieldErrors = response.error.fieldErrors;

                //document.body.scrollTop = document.documentElement.scrollTop = 0;
            });
        };

        $scope.$on('displayUnitChanged', setupUnits);

        $scope.$on('$ionicView.beforeEnter', function () {

            setupUnits();

            vm.customer = {
                weight: 0,
                height: 0
            };
        });

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Trainer - Add Client');
        });
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('ClientGroupListService', service);

    service.$inject = ['$q', 'ApiClientService'];

    function service($q, ApiClientService) {

        var dirty = false;
        var lastTrainerUserId;

        var groups = [];

        var reset = function () {
            groups = [];
        };

        return {

            reset: reset,

            dirty: function () {
                dirty = true;
                this.reset();
            },

            list: function (trainerUserId, includeCounts) {

                trainerUserId = trainerUserId || false;
                includeCounts = includeCounts || false;

                if (typeof lastTrainerUserId === 'undefined') {
                    lastTrainerUserId = trainerUserId;
                }

                if (!dirty && groups.length > 0 && lastTrainerUserId === trainerUserId) {
                    lastTrainerUserId = trainerUserId;
                    return $q.when(groups);
                }

                lastTrainerUserId = trainerUserId;

                return $q(function (resolve, reject) {

                    ApiClientService.customerGroups.list(trainerUserId, includeCounts).then(function (response) {

                        groups = response.groups;

                        dirty = false;

                        resolve(groups);
                    }, reject);
                });
            }

        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('TrainerEntityAssignToClientModalController', controller).factory('TrainerEntityAssignToClientModalService', service);

    service.$inject = ['ModalService'];

    function service(ModalService) {
        return {
            open: function (type, customer, assigned, assignedGroups) {
                return ModalService.show({
                    templateUrl: "trainer/clients/profile/assignModal.html",
                    controller: 'TrainerEntityAssignToClientModalController as vm',
                    params: {
                        type: type,
                        customer: customer,
                        assignedGroups: assignedGroups,
                        assigned: assigned
                    }
                });
            }
        };
    }

    controller.$inject = ['parameters', '$scope', '$filter', 'ApiClientService', '$q'];

    function controller(parameters, $scope, $filter, ApiClientService, $q) {

        var vm = this;

        vm.ready = false;

        vm.entityName = parameters.type === 'workouts' ? 'Workout' : 'Nutrition Plan';

        //vm.isCurrentTrainerUser = PermissionsService.getLoggedInUserId() === parameters.customer.trainer_user_id;

        var currentAssignments = parameters.assigned;
        var currentGroupAssignments = parameters.assignedGroups;

        var assignedIds = [];
        angular.forEach(currentAssignments, function (item) {
            assignedIds.push(item.id);
        });

        var assignedGroupIds = [];
        angular.forEach(currentGroupAssignments, function (group) {
            assignedGroupIds.push(group.id);
        });

        //var initAssignedCount = vm.assignedIds.length;

        var filterByIds = function (ids, items) {
            return $filter('filter')(items, function (item) {
                return ids.indexOf(item.id) === -1;
            });
        };

        var load = function () {

            var load = [];

            // List entities
            load.push(ApiClientService.trainers[parameters.type === 'workouts' ? 'getAllWorkouts' : 'getAllNutritionPlans'](parameters.customer.trainer_user_id).success(function (response) {
                vm.items = filterByIds(assignedIds, response[parameters.type === 'workouts' ? 'workouts' : 'nutritionPlans']);
                vm.filteredItems = angular.copy(vm.items);
                vm.noItems = vm.items.length === 0;
            }));

            // List groups
            load.push(ApiClientService[parameters.type === 'workouts' ? 'workoutGroups' : 'nutritionGroups'].list(parameters.customer.trainer_user_id).success(function (response) {
                vm.groups = filterByIds(assignedGroupIds, response.groups);
                vm.filteredGroups = angular.copy(vm.groups);
                vm.noGroups = vm.groups.length === 0;
            }));

            $q.all(load).finally(function () {
                vm.ready = true;
            });
        };

        if (!currentAssignments) {

            var loadExisting = [];

            if (parameters.type === 'workouts') {

                loadExisting.push(ApiClientService.customers.listWorkouts(parameters.customer.id, true).success(function (response) {
                    currentAssignments = response.workouts;
                }));

                loadExisting.push(ApiClientService.workoutGroups.listForCustomer(parameters.customer.id).success(function (response) {
                    currentGroupAssignments = response.groups;
                }));
            } else {

                loadExisting.push(ApiClientService.customers.listNutritionPlans(parameters.customer.id, true).success(function (response) {
                    currentAssignments = response.nutritionPlans;
                }));

                loadExisting.push(ApiClientService.nutritionGroups.listForCustomer(parameters.customer.id).success(function (response) {
                    currentGroupAssignments = response.groups;
                }));
            }

            $q.all(loadExisting).finally(load);
        } else {
            load();
        }

        // Entities List Search filter
        $scope.$watch(function () {
            return vm.searchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.searchFilterActive = true;
                vm.filteredItems = $filter('filter')(vm.items, {
                    title: newVal
                });
            } else {
                vm.searchFilterActive = false;
                vm.filteredItems = vm.items;
            }
        });

        // Groups List Search filter
        $scope.$watch(function () {
            return vm.groupSearchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.groupSearchFilterActive = true;
                vm.filteredGroups = $filter('filter')(vm.groups, {
                    title: newVal
                });
            } else {
                vm.groupSearchFilterActive = false;
                vm.filteredGroups = vm.groups;
            }
        });

        vm.assignItem = function (item) {

            item.use = !item.use;

            var findOriginal = $filter('filter')(vm.items, {
                id: item.id
            }, true);

            if (findOriginal.length > 0) {

                var originalIndex = vm.items.indexOf(findOriginal[0]);

                if (originalIndex !== -1) {
                    vm.items[originalIndex].use = item.use;
                }
            }
        };

        vm.assignGroup = function (group) {

            group.use = !group.use;

            var findOriginal = $filter('filter')(vm.groups, {
                id: group.id
            }, true);

            if (findOriginal.length > 0) {

                var originalIndex = vm.groups.indexOf(findOriginal[0]);

                if (originalIndex !== -1) {
                    vm.groups[originalIndex].use = group.use;
                }
            }
        };

        vm.save = function () {

            vm.saving = true;

            var toAssign = $filter('filter')(vm.items, {
                use: true
            }, true);

            var toAssignGroups = $filter('filter')(vm.groups, {
                use: true
            });

            if (toAssign.length === 0 && toAssignGroups.length === 0) {

                $scope.closeModal();

                return true;
            }

            var assignRequests = [];

            if (parameters.type === 'workouts') {

                toAssign.map(function (item) {
                    assignRequests.push(ApiClientService.workouts.assignCustomer(item.id, parameters.customer.id));
                });

                toAssignGroups.map(function (group) {
                    assignRequests.push(ApiClientService.workoutGroups.assignCustomer(group.id, parameters.customer.id));
                });
            } else {

                toAssign.map(function (item) {
                    assignRequests.push(ApiClientService.nutrition.assignCustomer(item.id, parameters.customer.id));
                });

                toAssignGroups.map(function (group) {
                    assignRequests.push(ApiClientService.nutritionGroups.assignCustomer(group.id, parameters.customer.id));
                });
            }

            return $q.all(assignRequests).finally(function () {
                vm.saving = false;
                $scope.closeModal();
            });
        };
    }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').factory('CustomerEntityAssignmentPathModalService', service).controller('CustomerEntityAssignmentPathModalController', controller);

  service.$inject = ['ModalService'];

  function service(ModalService) {
    return {
      open: function (assignedEntity, assignedEntityType, customer) {
        return ModalService.show({
          templateUrl: "trainer/clients/profile/assignmentPathViewerModal.html",
          controller: 'CustomerEntityAssignmentPathModalController as vm',
          params: {
            assignedEntity: assignedEntity,
            assignedEntityType: assignedEntityType,
            customer: customer
          }
        });
      }
    };
  }

  controller.$inject = ['parameters', 'AnalyticsService'];

  function controller(parameters, AnalyticsService) {
    var vm = this;

    var entityName = 'item';
    switch (parameters.assignedEntityType) {
      case 'workout':
        entityName = 'workout';
        vm.entityIcon = 'fitness_center';
        vm.typeLabel = 'Workout';
        break;
      case 'workout_group':
        entityName = 'workout group';
        vm.entityIcon = 'folder_open';
        vm.typeLabel = 'Workout Group';
        break;
      case 'nutrition_plan':
        entityName = 'nutrition plan';
        vm.entityIcon = 'restaurant_menu';
        vm.typeLabel = 'Nutrition Plan';
        break;
      case 'nutrition_plan_group':
        entityName = 'nutrition plan group';
        vm.entityIcon = 'folder_open';
        vm.typeLabel = 'Nutrition Plan Group';
        break;
    }

    vm.assignedEntity = parameters.assignedEntity;
    vm.customer = parameters.customer;

    vm.assignmentPath = (parameters.assignedEntity.assignment_path || []).filter(function (entity) {
      return entity.entity_type != 'package_purchase';
    }).map(function (entity) {
      switch (entity.entity_type) {
        case 'nutrition_plan_group':
          entity.icon = 'restaurant_menu';
          entity.groupIcon = true;
          entity.typeLabel = 'Nutrition Plan Group';
          break;
        case 'workout_group':
          entity.icon = 'fitness_center';
          entity.groupIcon = true;
          entity.typeLabel = 'Workout Group';
          break;
        case 'package':
          entity.icon = 'layers';
          entity.groupIcon = false;
          entity.typeLabel = 'Package';
          break;
        case 'customer_group':
          entity.icon = 'folder_shared';
          entity.groupIcon = false;
          entity.typeLabel = 'Client Group';
          break;
        default:
          entity.icon = 'help_outline';
          entity.groupIcon = false;
          entity.typeLabel = false;
      }

      return entity;
    });

    AnalyticsService.trackView('View Assignment Path');
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('TrainerCustomerEmailViewerModalController', controller);

    controller.$inject = ['parameters', 'AnalyticsService'];

    function controller(parameters, AnalyticsService) {

        var vm = this;

        var customer = parameters.customer.data;

        if (customer.demo) {
            vm.email = 'tom@example.com';
        } else {
            vm.email = customer.user.email;
        }

        AnalyticsService.trackView('Trainer - View Client Email Address');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('TrainerCustomerGoalEditorModalController', controller);

    controller.$inject = ['parameters', '$scope', 'ApiClientService', 'AnalyticsService'];

    function controller(parameters, $scope, ApiClientService, AnalyticsService) {

        var vm = this;

        vm.customer = parameters.customer.data;
        vm.description = vm.customer.description;

        vm.saveGoal = function () {

            vm.saving = true;

            ApiClientService.customers.update(vm.customer.id, {
                description: vm.description
            }).success(function () {
                $scope.closeModal(true);
            }).finally(function () {
                vm.saving = false;
            });
        };

        AnalyticsService.trackView('Trainer - Edit Client Goal');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('TrainerParqViewerModalController', controller);

    controller.$inject = ['parameters', 'ApiClientService', 'AnalyticsService'];

    function controller(parameters, ApiClientService, AnalyticsService) {

        var vm = this;

        vm.customer = parameters.customer.data;

        vm.hasAnswer = function (question) {
            return question.hasOwnProperty('answer') && question.answer;
        };

        ApiClientService.parq.getQuestions(vm.customer.id).success(function (response) {

            vm.questions = response.questions;

            var answerCount = 0;

            angular.forEach(vm.questions, function (q) {
                if (q.answer) {
                    answerCount++;
                }
            });

            if (answerCount === 0) {
                vm.noAnswers = true;
            }
        });

        AnalyticsService.trackView('Trainer - View Client PARQ');
    }
})();
(() => {
  angular.module('MyPTHub').config(config).controller('TrainerClientProfileController', controller);

  config.$inject = ['$stateProvider'];

  function config($stateProvider) {
    $stateProvider.state('app.trainer_client_profile', {
      url: '/clients/:customerId/profile/:tab?',
      views: {
        menuContent: {
          templateUrl: 'trainer/clients/profile/profile.html',
          controller: 'TrainerClientProfileController as vm'
        }
      }
    });
  }

  controller.$inject = ['CustomerStoreService', 'NetworkStateService', '$scope', '$ionicActionSheet', '$stateParams', 'ApiClientService', 'ImageCacheService', '$q', 'ModalService', 'ConfirmDialogService', 'ToastService', 'NavigateService', 'PermissionsService', 'ProgressPhotoUploadService', 'AnalyticsService', '$filter', 'TrainerEntityAssignToClientModalService', 'SessionStoreService', 'ScheduleEventStoreService', 'SessionQuotaDisplayService', 'CustomerEntityAssignmentPathModalService', 'SubscriptionStateService', 'AuthenticationStateService', 'PromptDialogService', '$ionicModal'];

  function controller(CustomerStoreService, NetworkStateService, $scope, $ionicActionSheet, $stateParams, ApiClientService, ImageCacheService, $q, ModalService, ConfirmDialogService, ToastService, NavigateService, PermissionsService, ProgressPhotoUploadService, AnalyticsService, $filter, TrainerEntityAssignToClientModalService, SessionStoreService, ScheduleEventStoreService, SessionQuotaDisplayService, CustomerEntityAssignmentPathModalService, SubscriptionStateService, AuthenticationStateService, PromptDialogService, $ionicModal) {
    const vm = this;
    const currentUser = AuthenticationStateService.getCurrentUser();
    const mimes = {
      image: ['image/gif', 'image/x-icon', 'image/jpeg', 'image/jpg', 'image/png', 'image/svg+xml', 'image/tiff', 'image/webp'],
      document: ['application/msword', 'application/vnd.oasis.opendocument.text', 'application/rtf', 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'],
      pdf: ['application/pdf'],
      archive: ['application/x-tar', 'application/x-bzip2', 'application/gzip', 'application/x-7z-compressed', 'application/x-apple-diskimage', 'application/x-rar-compressed', 'application/zip'],
      spreadsheet: ['application/vnd.oasis.opendocument.spreadsheet', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet', 'application/vnd.ms-excel']
    };

    $scope.$on('$ionicView.afterEnter', () => {
      vm.entered = true;
      // temp horrible hack to work around issue of disappearing tabs
      // in some weird circumstances.
      $(window).trigger('resize');
    });

    $scope.$on('$ionicView.leave', () => {
      vm.entered = false;
    });

    vm.customerId = $stateParams.customerId;
    vm.enablePackages = PermissionsService.merchantProvisioned();
    vm.schedulerEnabled = PermissionsService.schedulerIsEnabled();
    vm.apiConnected = null;
    vm.trainerSubscribed = SubscriptionStateService.trainerIsSubscribed();
    vm.hasUnlimitedPlus = PermissionsService.userHasUnlimitedPlus();
    vm.userCredits = [];

    const hasCredits = () => {
      return CustomerStoreService.get($stateParams.customerId).then(customer => {
        return ApiClientService.packagesCredits.getAll({
          trainer_user_id: customer.trainer_user_id,
          user_id: customer.user_id
        }).then(({ credits }) => vm.hasCredits = credits.length > 0);
      });
    };

    hasCredits();

    const tabs = [];

    tabs.push('progress');
    if (vm.schedulerEnabled) {
      tabs.push('schedules');
    }
    tabs.push('workouts');
    tabs.push('nutrition');
    if (vm.enablePackages) {
      tabs.push('sessions');
    }
    tabs.push('photos');
    tabs.push('notes');
    tabs.push('forms');
    tabs.push('files');
    tabs.push('credits');

    vm.tabMap = {};

    for (let i = 0, length = tabs.length; i < length; i++) {
      vm.tabMap[tabs[i]] = i;
    }

    const loadCustomer = function () {
      // Immediately load from the local DB
      CustomerStoreService.get($stateParams.customerId).then(customer => {
        vm.customer = customer;
        vm.canEdit = PermissionsService.userIsTrainerAdminOrOwner(customer.trainer_user_id);

        if (vm.trainerSubscribed) {
          checkFitbitAvailability();
        }
      });

      // Gets fresh data straight away if online
      if (NetworkStateService.isOnline()) {
        CustomerStoreService.download($stateParams.customerId).then(customer => {
          vm.customer = customer;
        });
      }
    };

    const checkFitbitAvailability = () => {
      ApiClientService.hardware.checkAuthentication(vm.customer.user_id).success(response => {
        if (response == 'true') {
          vm.apiConnected = true;
        } else {
          vm.apiConnected = false;
        }
      });
    };

    $scope.$on('$ionicView.beforeEnter', () => {
      loadCustomer();

      if ($stateParams.tab && vm.tabMap.hasOwnProperty($stateParams.tab)) {
        vm.selectedTabIndex = vm.tabMap[$stateParams.tab];
      }
    });

    $scope.$on('$ionicView.enter', () => {
      AnalyticsService.trackView('Trainer - View Client Profile');
    });

    const resetShown = function () {
      vm.activityLogActive = false;
      vm.exitPhotosDeleteMode();
    };

    const refreshComplete = function () {
      $scope.$broadcast('scroll.refreshComplete');
    };

    vm.activityLogShow = function () {
      resetShown();

      vm.activityLogActive = true;

      vm.refreshTab = function () {
        vm.loadActivityLog().finally(refreshComplete);
      };

      vm.loadActivityLog();
    };

    SessionStoreService.registerEventHandler($scope, 'online_session_synced', () => {
      vm.loadActivityLog();
    });
    SessionStoreService.registerEventHandler($scope, 'offline_sessions_synced', () => {
      vm.loadActivityLog();
    });

    // Scheduler

    let loadingSchedules = false;
    const loadSchedules = function (refresh) {
      refresh = refresh || false;

      if (!refresh && (vm.currentSchedules || vm.previousSchedules)) {
        return false;
      }

      if (loadingSchedules) {
        return false;
      }

      loadingSchedules = true;

      const scheduleRequests = [];

      const getCurrent = ApiClientService.customers.getCurrentSchedules($stateParams.customerId);

      const getPrevious = ApiClientService.customers.getPreviousSchedules($stateParams.customerId);

      scheduleRequests.push(getCurrent);
      scheduleRequests.push(getPrevious);

      getCurrent.then(response => {
        angular.forEach(response.schedules, schedule => {
          if (schedule.start_date) {
            const startDate = moment.tz(schedule.start_date, schedule.customerTimezone).format('Do MMM YYYY');

            schedule.description = `Starts on ${startDate}`;

            let endDate = false;
            if (schedule.started) {
              endDate = moment.tz(schedule.end_date, schedule.customerTimezone).format('Do MMM YYYY');
              schedule.description = `${startDate} - ${endDate}`;
            }
          } else {
            schedule.description = 'No start date specified';
          }

          schedule.active = schedule.started && !schedule.ended;
        });
        vm.currentSchedules = response.schedules;
        vm.noCurrentSchedules = vm.currentSchedules.length === 0;
        vm.filteredCurrentSchedules = angular.copy(vm.currentSchedules);
      });

      getPrevious.then(response => {
        angular.forEach(response.schedules, schedule => {
          const startDate = moment.tz(schedule.start_date, schedule.customerTimezone).format('Do MMM YYYY');

          let endDate = false;
          if (schedule.end_date) {
            endDate = moment.tz(schedule.end_date, schedule.customerTimezone).format('Do MMM YYYY');
          }

          if (endDate) {
            schedule.description = `Finished (${startDate} - ${endDate})`;
          } else {
            schedule.description = startDate;
          }
        });
        vm.previousSchedules = response.schedules;
        vm.noPreviousSchedules = vm.previousSchedules.length === 0;
        vm.filteredPreviousSchedules = angular.copy(vm.previousSchedules);
      });

      const requests = $q.all(scheduleRequests);

      requests.finally(() => {
        loadingSchedules = false;
      });

      return requests;
    };

    // Schedule Search filter
    vm.scheduleTabLabels = {
      current: 'Current',
      previous: 'Previous'
    };
    $scope.$watch(() => vm.schedulesSearchFilter, (newVal, oldVal) => {
      if (typeof newVal === 'undefined') {
        return false;
      }

      if (newVal.length > 0) {
        vm.filteredCurrentSchedules = $filter('filter')(vm.currentSchedules, {
          name: newVal
        });
        vm.scheduleTabLabels.current = `Current (${vm.filteredCurrentSchedules.length})`;

        vm.filteredPreviousSchedules = $filter('filter')(vm.previousSchedules, {
          name: newVal
        });
        vm.scheduleTabLabels.previous = `Previous (${vm.filteredPreviousSchedules.length})`;
      } else {
        vm.filteredCurrentSchedules = vm.currentSchedules;
        vm.filteredPreviousSchedules = vm.previousSchedules;
        vm.scheduleTabLabels = {
          current: 'Current',
          previous: 'Previous'
        };
      }
    });

    vm.schedulerShow = function () {
      resetShown();

      loadSchedules();

      vm.refreshTab = function () {
        const refresh = loadSchedules(true);

        if (refresh) {
          refresh.finally(refreshComplete);
          return true;
        }

        refreshComplete();
      };
    };

    vm.goToSchedule = function (schedule) {
      NavigateService.go('app.view_customer_schedule', {
        customerId: $stateParams.customerId,
        scheduleId: schedule.id
      });
    };

    vm.scheduleItemMenu = function (schedule, mode) {
      const config = {
        titleText: schedule.name,
        destructiveText: 'Remove',
        cancelText: 'Cancel'
      };

      config.destructiveButtonClicked = function () {
        // Delete button
        ConfirmDialogService.open({
          title: 'Confirm Remove',
          message: `Are you sure you want to remove this schedule (${schedule.name}) from ${vm.customer.first_name}s profile?`,
          buttonConfirm: 'Remove',
          buttonCancel: 'Cancel'
        }).then(() => {
          ApiClientService.customers.scheduleDelete(schedule.id);

          if (mode === 'current') {
            vm.currentSchedules.splice(vm.currentSchedules.indexOf(schedule), 1);
            vm.filteredCurrentSchedules.splice(vm.filteredCurrentSchedules.indexOf(schedule), 1);
          } else {
            vm.previousSchedules.splice(vm.previousSchedules.indexOf(schedule), 1);
            vm.filteredPreviousSchedules.splice(vm.filteredPreviousSchedules.indexOf(schedule), 1);
          }
        });

        return true;
      };

      $ionicActionSheet.show(config);
    };

    // Workouts

    let loadingWorkouts = false;
    const loadWorkouts = function (refresh) {
      refresh = refresh || false;

      if (!refresh && vm.workouts) {
        return false;
      }

      if (loadingWorkouts) {
        return false;
      }

      loadingWorkouts = true;

      const workoutRequests = [];

      const getWorkouts = ApiClientService.customers.listWorkouts($stateParams.customerId, false);

      const getGroups = ApiClientService.workoutGroups.listForCustomer($stateParams.customerId, true);

      const getCustomerCreatedWorkouts = ApiClientService.customers.listCreatedWorkouts($stateParams.customerId);

      workoutRequests.push(getWorkouts);
      workoutRequests.push(getGroups);
      workoutRequests.push(getCustomerCreatedWorkouts);

      getWorkouts.success(response => {
        vm.workouts = response.workouts;

        vm.workouts.map(item => {
          item.type = 'workout';
          item.icon = 'fitness_center';
          return item;
        });
      });

      getGroups.success(response => {
        vm.workoutGroups = response.groups;

        vm.workoutGroups.map(item => {
          item.type = 'group';
          item.icon = 'folder_open';
          return item;
        });
      });

      getCustomerCreatedWorkouts.success(response => {
        vm.createdWorkouts = response.workouts;

        vm.noCreatedWorkouts = vm.createdWorkouts.length === 0;

        vm.createdWorkouts.map(item => {
          item.type = 'workout';
          item.icon = 'fitness_center';
          return item;
        });

        vm.filteredCreatedWorkouts = angular.copy(vm.createdWorkouts);
      });

      const requests = $q.all(workoutRequests);

      requests.finally(() => {
        vm.noWorkouts = vm.workouts.length === 0 && vm.workoutGroups.length === 0;

        vm.workoutItems = vm.workouts.concat(vm.workoutGroups).sort((a, b) => {
          const textA = a.title.toUpperCase();
          const textB = b.title.toUpperCase();
          return textA < textB ? -1 : textA > textB ? 1 : 0;
        });

        vm.filteredWorkoutItems = angular.copy(vm.workoutItems);

        loadingWorkouts = false;
      });

      return requests;
    };

    // Workouts Search filter
    vm.workoutTabLabels = {
      assigned: 'Assigned',
      created: 'Client Created'
    };
    $scope.$watch(() => vm.workoutsSearchFilter, (newVal, oldVal) => {
      if (typeof newVal === 'undefined') {
        return false;
      }

      if (newVal.length > 0) {
        vm.filteredWorkoutItems = $filter('filter')(vm.workoutItems, {
          title: newVal
        });
        vm.workoutTabLabels.assigned = `Assigned (${vm.filteredWorkoutItems.length})`;

        vm.filteredCreatedWorkouts = $filter('filter')(vm.createdWorkouts, {
          title: newVal
        });
        vm.workoutTabLabels.created = `Client Created (${vm.filteredCreatedWorkouts.length})`;
      } else {
        vm.filteredWorkoutItems = vm.workoutItems;
        vm.filteredCreatedWorkouts = vm.createdWorkouts;
        vm.workoutTabLabels = {
          assigned: 'Assigned',
          created: 'Client Created'
        };
      }
    });

    vm.workoutsShow = function () {
      resetShown();

      loadWorkouts();

      vm.refreshTab = function () {
        const refresh = loadWorkouts(true);

        if (refresh) {
          refresh.finally(refreshComplete);
          return true;
        }

        refreshComplete();
      };
    };

    vm.editWorkout = function (workout) {
      NavigateService.go('app.edit_trainer_workout_via_customer', {
        workoutId: workout.id,
        customerId: $stateParams.customerId
      });
    };

    vm.editWorkoutGroup = function (group) {
      NavigateService.go('app.trainer_workouts_group', {
        groupId: group.id
      });
    };

    vm.workoutItemTap = function (workoutItem) {
      switch (workoutItem.type) {
        case 'workout':
          vm.editWorkout(workoutItem);
          break;
        case 'group':
          vm.editWorkoutGroup(workoutItem);
          break;
      }
    };

    vm.workoutItemMenu = function (workoutItem) {
      var config = {
        buttons: [],
        cancelText: 'Cancel',
        buttonClicked(index) {
          if (typeof config.buttons[index] !== 'undefined' && typeof config.buttons[index].action === 'function') {
            config.buttons[index].action(index);
          }
          return true;
        }
      };

      const viewAssignmentButtonConfig = {
        text: 'View Assignment',
        action() {
          const type = workoutItem.type === 'group' ? 'workout_group' : 'workout';
          CustomerEntityAssignmentPathModalService.open(workoutItem, type, vm.customer);
        }
      };

      switch (workoutItem.type) {
        case 'workout':
          config.titleText = 'Workout';

          if (workoutItem.assigned_via === 'direct') {
            config.destructiveText = 'Unassign';
            config.destructiveButtonClicked = function () {
              // Delete button
              ConfirmDialogService.open({
                title: 'Confirm Unassign',
                message: `Are you sure you want to remove this workout (${workoutItem.title}) from ${vm.customer.first_name}'s profile? `,
                buttonConfirm: 'Unassign',
                buttonCancel: 'Cancel'
              }).then(() => {
                ApiClientService.workouts.unassignCustomer(workoutItem.id, vm.customer.id);
                vm.workoutItems.splice(vm.workoutItems.indexOf(workoutItem), 1);
                vm.workouts.splice(vm.workouts.indexOf(workoutItem), 1);
              });

              return true;
            };
          } else {
            config.buttons.push(viewAssignmentButtonConfig);
          }

          break;
        case 'group':
          config.titleText = 'Workout Group';

          if (workoutItem.assigned_via === 'direct') {
            config.destructiveText = 'Unassign';

            config.destructiveButtonClicked = function () {
              // Delete button
              ConfirmDialogService.open({
                title: 'Confirm Unassign',
                message: `Are you sure you want to remove this workout group (${workoutItem.title}) from ${vm.customer.first_name}'s profile?`,
                buttonConfirm: 'Unassign',
                buttonCancel: 'Cancel'
              }).then(() => {
                ApiClientService.workoutGroups.unassignCustomer(workoutItem.id, vm.customer.id);
                vm.workoutItems.splice(vm.workoutItems.indexOf(workoutItem), 1);
                vm.workoutGroups.splice(vm.workoutGroups.indexOf(workoutItem), 1);
              });

              return true;
            };
          } else {
            config.buttons.push(viewAssignmentButtonConfig);
          }

          break;
      }

      $ionicActionSheet.show(config);
    };

    vm.createdWorkoutItemMenu = function (workoutItem) {
      const config = {
        destructiveText: 'Delete',
        cancelText: 'Cancel'
      };

      config.destructiveButtonClicked = function () {
        // Delete button
        ConfirmDialogService.open({
          title: 'Confirm Delete',
          message: `Are you sure you want to remove the workout that ${vm.customer.first_name} created? (${workoutItem.title})`,
          buttonConfirm: 'Delete',
          buttonCancel: 'Cancel'
        }).then(() => {
          ApiClientService.workouts.delete(workoutItem.id);
          vm.createdWorkouts.splice(vm.createdWorkouts.indexOf(workoutItem), 1);

          const inFiltered = vm.filteredCreatedWorkouts.indexOf(workoutItem);
          if (inFiltered !== -1) {
            vm.filteredCreatedWorkouts.splice(inFiltered, 1);
          }
        });

        return true;
      };

      $ionicActionSheet.show(config);
    };

    vm.assignWorkouts = function () {
      TrainerEntityAssignToClientModalService.open('workouts', vm.customer, vm.workouts, vm.workoutGroups).then(() => {
        loadWorkouts(true);
        vm.selectedTabIndex = 1;
      });
    };

    let loadingNutritionPlans = false;
    const loadNutritionPlans = function (refresh) {
      refresh = refresh || false;

      if (!refresh && vm.nutritionPlans) {
        return false;
      }

      if (loadingNutritionPlans) {
        return false;
      }

      loadingNutritionPlans = true;

      const nutritionPlanRequests = [];

      const getNutritionPlans = ApiClientService.customers.listNutritionPlans($stateParams.customerId, false);

      const getNutritionPlanGroups = ApiClientService.nutritionGroups.listForCustomer($stateParams.customerId, true);

      nutritionPlanRequests.push(getNutritionPlans);
      nutritionPlanRequests.push(getNutritionPlanGroups);

      getNutritionPlans.success(response => {
        vm.nutritionPlans = response.nutritionPlans;

        vm.nutritionPlans.map(item => {
          item.type = 'nutritionPlan';
          item.icon = 'restaurant_menu';
          return item;
        });
      });

      getNutritionPlanGroups.success(response => {
        vm.nutritionPlanGroups = response.groups;

        vm.nutritionPlanGroups.map(item => {
          item.type = 'group';
          item.icon = 'folder_open';
          return item;
        });
      });

      const requests = $q.all(nutritionPlanRequests);

      requests.finally(() => {
        vm.noNutritionPlans = vm.nutritionPlans.length === 0 && vm.nutritionPlanGroups.length === 0;

        vm.nutritionPlanItems = vm.nutritionPlans.concat(vm.nutritionPlanGroups).sort((a, b) => {
          const textA = a.title.toUpperCase();
          const textB = b.title.toUpperCase();
          return textA < textB ? -1 : textA > textB ? 1 : 0;
        });

        loadingNutritionPlans = false;
      });

      return requests;
    };

    vm.nutritionShow = function () {
      resetShown();

      loadNutritionPlans();

      vm.refreshTab = function () {
        const refresh = loadNutritionPlans(true);

        if (refresh) {
          refresh.finally(refreshComplete);
          return true;
        }

        refreshComplete();
      };
    };

    vm.editNutritionPlan = function (nutritionPlan) {
      NavigateService.go('app.edit_trainer_nutrition_plan_via_customer', {
        nutritionPlanId: nutritionPlan.id,
        customerId: $stateParams.customerId
      });
    };

    vm.editNutritionGroup = function (nutritionGroup) {
      NavigateService.go('app.trainer_nutrition_plans_group', {
        groupId: nutritionGroup.id
      });
    };

    vm.nutritionPlanItemTap = function (nutritionPlanItem) {
      switch (nutritionPlanItem.type) {
        case 'nutritionPlan':
          vm.editNutritionPlan(nutritionPlanItem);
          break;
        case 'group':
          vm.editNutritionGroup(nutritionPlanItem);
          break;
      }
    };

    vm.nutritionPlanItemMenu = function (nutritionPlanItem) {
      var config = {
        buttons: [],
        cancelText: 'Cancel',
        buttonClicked(index) {
          if (typeof config.buttons[index] !== 'undefined' && typeof config.buttons[index].action === 'function') {
            config.buttons[index].action(index);
          }
          return true;
        }
      };

      const viewAssignmentButtonConfig = {
        text: 'View Assignment',
        action() {
          const type = nutritionPlanItem.type === 'group' ? 'nutrition_plan_group' : 'nutrition_plan';
          CustomerEntityAssignmentPathModalService.open(nutritionPlanItem, type, vm.customer);
        }
      };

      switch (nutritionPlanItem.type) {
        case 'nutritionPlan':
          config.titleText = 'Nutrition Plan';
          if (nutritionPlanItem.assigned_via === 'direct') {
            config.destructiveText = 'Unassign';
            config.destructiveButtonClicked = function () {
              // Delete button
              ConfirmDialogService.open({
                title: 'Confirm Unassign',
                message: `Are you sure you want to remove this nutrition plan (${nutritionPlanItem.title}) from ${vm.customer.first_name}'s profile?`,
                buttonConfirm: 'Unassign',
                buttonCancel: 'Cancel'
              }).then(() => {
                ApiClientService.nutrition.unassignCustomer(nutritionPlanItem.id, vm.customer.id);
                vm.nutritionPlanItems.splice(vm.nutritionPlanItems.indexOf(nutritionPlanItem), 1);
                vm.nutritionPlans.splice(vm.nutritionPlans.indexOf(nutritionPlanItem), 1);
              });

              return true;
            };
          } else {
            config.buttons.push(viewAssignmentButtonConfig);
          }

          break;
        case 'group':

          config.titleText = 'Nutrition Plan Group';

          if (nutritionPlanItem.assigned_via === 'direct') {
            config.destructiveText = 'Unassign';
            config.destructiveButtonClicked = function () {
              // Delete button
              ConfirmDialogService.open({
                title: 'Confirm Unassign',
                message: `Are you sure you want to remove this nutrition plan group (${nutritionPlanItem.title}) from ${vm.customer.first_name}'s profile?`,
                buttonConfirm: 'Unassign',
                buttonCancel: 'Cancel'
              }).then(() => {
                ApiClientService.nutritionGroups.unassignCustomer(nutritionPlanItem.id, vm.customer.id);
                vm.nutritionPlanItems.splice(vm.nutritionPlanItems.indexOf(nutritionPlanItem), 1);
                vm.nutritionPlanGroups.splice(vm.nutritionPlanGroups.indexOf(nutritionPlanItem), 1);
              });

              return true;
            };
          } else {
            config.buttons.push(viewAssignmentButtonConfig);
          }
          break;
      }

      $ionicActionSheet.show(config);
    };

    vm.assignNutritionPlans = function () {
      TrainerEntityAssignToClientModalService.open('nutrition', vm.customer, vm.nutritionPlans, vm.nutritionPlanGroups).then(() => {
        loadNutritionPlans(true);
        vm.selectedTabIndex = 2;
      });
    };

    const loadPhotos = function (refresh) {
      refresh = refresh || false;

      if (!refresh && vm.photos) {
        return false;
      }

      return ApiClientService.progressPhotos.list($stateParams.customerId).success(response => {
        if (response.photos.length === 0) {
          vm.noPhotos = true;
          return false;
        }
        vm.noPhotos = false;

        const cacheResolvers = [];

        const brokenImageSmall = 'img/broken-image.png';

        response.photos.map(photo => {
          cacheResolvers.push(ImageCacheService.get(photo.small_url).then(newUrl => {
            photo.small_url = newUrl;
          }, () => {
            photo.small_url = brokenImageSmall;
          }));
        });

        $q.all(cacheResolvers).finally(() => {
          vm.photosReady = true;

          vm.photos = response.photos;

          vm.photosRange = function (cols) {
            const range = [];

            for (let i = 0; i < vm.photos.length; i += cols) {
              range.push(i);
            }

            return range;
          };
        });
      });
    };

    vm.photosShow = function () {
      resetShown();

      loadPhotos();

      vm.refreshTab = function () {
        const refresh = loadPhotos(true);

        if (refresh) {
          refresh.finally(refreshComplete);
          return true;
        }

        refreshComplete();
      };
    };

    vm.showLargePhoto = function (photo) {
      const date = moment.utc(photo.created_at, 'YYYY-MM-DD HH:mm:ss').local().format('Do MMMM YYYY');

      ModalService.show({
        templateUrl: "common/photoViewer/photoViewerModal.html",
        controller: 'PhotoViewerModalController as vm',
        params: {
          url: photo.large_url,
          date: date
        }
      });
    };

    vm.toggleDeleteFlag = function (photo) {
      photo.toDelete = !photo.toDelete;
    };

    const getPhotosWithDeleteChecked = function () {
      if (!vm.photos) {
        return [];
      }

      return $filter('filter')(vm.photos, {
        toDelete: true
      });
    };

    const clearCheckedPhotosForDelete = function () {
      getPhotosWithDeleteChecked().map(item => {
        item.toDelete = false;
      });
    };

    vm.exitPhotosDeleteMode = function () {
      vm.photoDeleteMode = false;
      vm.hasCheckedPhotosForDelete = false;
      clearCheckedPhotosForDelete();
    };

    vm.toggledDelete = function () {
      vm.hasCheckedPhotosForDelete = getPhotosWithDeleteChecked().length > 0;
    };

    vm.deleteCheckedPhotos = function () {
      ConfirmDialogService.open({
        title: 'Confirm Delete',
        message: 'Are you sure you want to delete these photos?',
        buttonConfirm: 'Delete Photos',
        buttonCancel: 'Cancel'
      }).then(() => {
        getPhotosWithDeleteChecked().map(item => {
          vm.photos.splice(vm.photos.indexOf(item), 1);

          ApiClientService.progressPhotos.delete(vm.customerId, item.id);
        });

        vm.exitPhotosDeleteMode();
      });
    };

    vm.photoAction = function (photo) {
      if (vm.photoDeleteMode) {
        vm.toggleDeleteFlag(photo);
        vm.toggledDelete();
        return true;
      }

      vm.showLargePhoto(photo);
    };

    vm.uploadProgressPhoto = function () {
      vm.selectedTabIndex = 3;

      vm.uploading = true;

      ProgressPhotoUploadService.browsePhoto(vm.customer.id).finally(() => {
        loadPhotos(true).finally(() => {
          vm.uploading = false;
        });
        ScheduleEventStoreService.setDirty();
      });
    };

    vm.takeProgressPhoto = function () {
      vm.selectedTabIndex = 3;

      vm.uploading = true;

      ProgressPhotoUploadService.takePhoto(vm.customer.id).finally(() => {
        loadPhotos(true).finally(() => {
          vm.uploading = false;
        });
        ScheduleEventStoreService.setDirty();
      });
    };

    vm.photoDeleteMode = false;
    vm.showPhotoOptions = function () {
      vm.photoDeleteMode = true;
    };

    const loadNotes = function (refresh) {
      refresh = refresh || false;

      if (!refresh && vm.notes) {
        return false;
      }

      return ApiClientService.customers.getNotes($stateParams.customerId).success(response => {
        vm.notes = response.notes;
        vm.noNotes = response.notes.length === 0;
      });
    };

    vm.notesShow = function () {
      resetShown();

      loadNotes();

      vm.refreshTab = function () {
        const refresh = loadNotes(true);

        if (refresh) {
          refresh.finally(refreshComplete);
          return true;
        }

        refreshComplete();
      };

      vm.showSheet = function (note) {
        // Show the action sheet
        const hideSheet = $ionicActionSheet.show({
          buttons: [{ text: 'Edit' }],
          destructiveText: 'Delete',
          titleText: 'Manage Note',
          cancelText: 'Cancel',
          buttonClicked(index) {
            // Edit button
            ModalService.show({
              templateUrl: 'common/customerNoteForm/customerNoteFormModal.html',
              controller: 'CustomerNoteFormModalController as vm',
              params: {
                customer_id: $stateParams.customerId,
                note
              }
            }).then(note => {
              loadNotes(true);
            });

            return true;
          },
          destructiveButtonClicked() {
            // Delete button
            ConfirmDialogService.open({
              title: 'Confirm Delete',
              message: 'Are you sure you want to delete this note?',
              buttonConfirm: 'Delete',
              buttonCancel: 'Cancel'
            }).then(() => {
              ApiClientService.customers.deleteNote($stateParams.customerId, note.id).success(() => {
                ToastService.show('Note successfully deleted');
              });

              vm.notes.splice(vm.notes.indexOf(note), 1);
            });

            return true;
          }
        });
      };
    };

    vm.showCreateNoteForm = function () {
      vm.selectedTabIndex = 4;

      ModalService.show({
        templateUrl: 'common/customerNoteForm/customerNoteFormModal.html',
        controller: 'CustomerNoteFormModalController as vm',
        params: {
          customer_id: $stateParams.customerId
        }
      }).then(note => {
        loadNotes(true);
      });
    };

    const loadForms = function (refresh) {
      refresh = refresh || false;

      if (!refresh && vm.forms) {
        return false;
      }

      return ApiClientService.forms.getAllForCustomer($stateParams.customerId).success(response => {
        vm.forms = response.forms;
        vm.noForms = vm.forms.length === 0;
      });
    };

    vm.formsShow = function () {
      resetShown();

      loadForms();

      vm.refreshTab = function () {
        const refresh = loadForms(true);

        if (refresh) {
          refresh.finally(refreshComplete);
          return true;
        }

        refreshComplete();
      };
    };

    vm.formItemTap = function (form) {
      if (form.editable) {
        NavigateService.go('app.view_customer_form', {
          formId: form.id,
          customerId: $stateParams.customerId
        });
      } else {
        NavigateService.go('app.view_customer_form', {
          formId: form.id,
          customerId: $stateParams.customerId,
          readonly: true
        });
      }
    };

    vm.sessionsShow = function () {
      resetShown();
      vm.refreshTab = function () {
        SessionQuotaDisplayService.reload();
        refreshComplete();
      };
    };

    vm.formItemMenu = function (form) {
      const buttons = [];

      if (form.completed && form.editable || !form.completed) {
        buttons.push({
          text: 'Notify Client',
          click() {
            ConfirmDialogService.open({
              title: 'Confirm Notify Client',
              message: 'Are you sure you want to send this client an email prompting them to fill this form in?',
              buttonConfirm: 'Send Notification',
              buttonCancel: 'Cancel'
            }).then(() => {
              ApiClientService.forms.notify($stateParams.customerId, form.id).then(() => {
                ToastService.show('Notification sent');
              });
            });
          }
        });
      }

      if (form.editable) {
        buttons.push({
          text: 'Edit Answers',
          click() {
            NavigateService.go('app.view_customer_form', {
              formId: form.id,
              customerId: $stateParams.customerId
            });
          }
        });
      }

      buttons.push({
        text: 'Preview Answers',
        click() {
          NavigateService.go('app.view_customer_form', {
            formId: form.id,
            customerId: $stateParams.customerId,
            readonly: true
          });
        }
      });

      const config = {
        cancelText: 'Cancel',
        titleText: form.title,
        buttons,
        buttonClicked(index) {
          buttons[index].click();
        }
      };

      $ionicActionSheet.show(config);
    };

    vm.showSettingsMenu = function () {
      const config = {
        titleText: 'Additional Actions',
        buttons: [{
          text: 'View PARQ'
        }, {
          text: 'View Email Address'
        }, {
          text: 'Edit Goal'
        }],
        cancelText: 'Cancel',
        cancel() {},
        buttonClicked(index) {
          if (index === 0) {
            ModalService.show({
              templateUrl: 'trainer/clients/profile/parqViewerModal.html',
              controller: 'TrainerParqViewerModalController as vm',
              params: {
                customer: vm.customer
              }
            });
          }

          if (index === 1) {
            ModalService.show({
              templateUrl: 'trainer/clients/profile/emailViewerModal.html',
              controller: 'TrainerCustomerEmailViewerModalController as vm',
              params: {
                customer: vm.customer
              }
            });
          }

          if (index === 2) {
            ModalService.show({
              templateUrl: 'trainer/clients/profile/goalEditorModal.html',
              controller: 'TrainerCustomerGoalEditorModalController as vm',
              params: {
                customer: vm.customer
              }
            }).then(() => {
              loadCustomer();
            });
          }

          return true;
        }
      };

      if (vm.canEdit) {
        config.destructiveText = 'Delete Client';

        config.destructiveButtonClicked = function () {
          ConfirmDialogService.open({
            title: 'Confirm Delete',
            message: 'Are you sure you want to delete this client?',
            buttonConfirm: 'Delete Client',
            buttonCancel: 'Cancel'
          }).then(() => {
            // perform the delete
            CustomerStoreService.delete(vm.customer.id).then(() => {
              ToastService.show('Client successfully deleted');

              NavigateService.goHome();
            });
          });

          return true;
        };
      }

      $ionicActionSheet.show(config);
    };

    const loadFiles = (refresh = false) => {
      if (!refresh && vm.files) {
        return false;
      }

      return ApiClientService.customers.getFiles($stateParams.customerId).success(resp => {
        const filesObject = resp.files;
        filesObject.map(file => {
          if (mimes.image.includes(file.mime)) {
            file.icon = 'fa-file-image-o';
          } else if (mimes.spreadsheet.includes(file.mime)) {
            file.icon = 'fa-file-excel-o';
          } else if (mimes.pdf.includes(file.mime)) {
            file.icon = 'fa-file-pdf-o';
          } else if (mimes.archive.includes(file.mime)) {
            file.icon = 'fa-file-archive-o';
          } else if (mimes.document.includes(file.mime)) {
            file.icon = 'fa-file-text-o';
          } else {
            file.icon = 'fa-file-o';
          }
          file.name = `${file.original_name}.${file.extension}`;
          return true;
        });

        vm.files = filesObject;
        vm.noFiles = vm.files.length === 0;
      });
    };

    vm.filesShow = () => {
      resetShown();

      loadFiles();
      vm.refreshTab = () => {
        const refresh = loadFiles(true);

        if (refresh) {
          refresh.finally(refreshComplete);
          return true;
        }

        refreshComplete();
        return true;
      };
    };

    vm.viewFile = file => {
      ModalService.show({
        templateUrl: 'common/files/filesViewModal.html',
        controller: 'FilesViewController as vm',
        params: {
          file,
          trainer: true
        }
      }).finally(() => {
        loadFiles(true);
      });
    };

    vm.creditsShow = () => {
      loadCreditTrainers();
    };

    vm.credits = [];
    vm.credittrainers = [];
    const loadCreditTrainers = () => {
      vm.creditsReady = false;
      return ApiClientService.noCache().packagesCredits.getTrainers({
        user_id: vm.customer.user_id
      }).then(response => {
        vm.creditTrainers = response.data.trainers.filter(trainer => trainer.trainer_id === currentUser.id);
        vm.creditTrainers.forEach(trainer => {
          const trainerId = trainer.id;
          vm.credits[trainerId] = {
            allowance: 0,
            gifted: 0,
            purchased: 0,
            redeemed: 0
          };

          return ApiClientService.packagesCredits.getAll({
            trainer_user_id: trainer.user_id,
            user_id: vm.customer.user_id
          }).then(response => {
            vm.userCredits = response.credits;
            response.credits.forEach(credit => {
              if (!credit.unlimited) {
                vm.credits[trainer.id].allowance += credit.allowance;
                vm.credits[trainer.id].gifted += credit.gifted;
                vm.credits[trainer.id].purchased += credit.purchased;
                vm.credits[trainer.id].redeemed += credit.redeemed;
              }
            });
            vm.mainCredit = vm.userCredits.find(credit => !credit.is_unlimited);
            vm.canGiftCredits = currentUser.type === 'trainer' && typeof vm.mainCredit !== 'undefined';
          });
        });
        vm.creditsReady = true;
      });
    };

    vm.giftCredits = () => {
      $ionicModal.fromTemplateUrl('trainer/clients/profile/gift-credits.html', {
        scope: $scope,
        animation: 'slide-in-up'
      }).then(modal => {
        $scope.modal = modal;
        $scope.modal.show();
      });
    };

    vm.confirmGift = credits => {
      const credit = vm.mainCredit;

      if (vm.userCredits.length >= 1) {
        ApiClientService.packagesCredits.addAllowance(credit.id, {
          amount: parseInt(credits, 10)
        }).success(resp => {
          vm.mainCredit = resp.credit;
          vm.loadCredits();
          $scope.modal.hide();
        });
      } else {
        ApiClientService.packagesCredits.create({
          amount: parseInt(credits, 10),
          user_id: vm.customer.user_id,
          trainer_user_id: currentUser.id
        }).success(() => {
          vm.loadCredits();
          $scope.modal.hide();
          $scope.modal.remove();
        });
      }
    };

    vm.deductCredits = () => {
      $ionicModal.fromTemplateUrl('trainer/clients/profile/deduct-credits.html', {
        scope: $scope,
        animation: 'slide-in-up'
      }).then(modal => {
        $scope.modal = modal;
        $scope.modal.show();
      });
    };

    vm.confirmDeduct = amount => {
      const credit = vm.mainCredit;

      return ApiClientService.packagesCredits.deductAllowance(credit.id, {
        amount
      }).success(resp => {
        vm.mainCredit = resp.credit;
        vm.loadCredits();
        $scope.modal.hide();
        $scope.modal.remove();
      });
    };

    vm.loadCredits = () => {
      resetShown();
      loadCreditTrainers();
    };
  }
})();
(() => {
  angular.module('MyPTHub').controller('StripeModalController', controller);

  /**
   * @param {*} $scope Scope
   */
  function controller($scope) {
    $scope.close = () => $scope.closeModal();
    $scope.pastCutoff = () => moment().year() === 2019;
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('TrainerNutritionGroupCustomerAssignModalController', controller);

    controller.$inject = ['parameters', '$scope', 'CustomerStoreService', '$filter', 'ApiClientService', 'PermissionsService', 'AnalyticsService'];

    function controller(parameters, $scope, CustomerStoreService, $filter, ApiClientService, PermissionsService, AnalyticsService) {

        var vm = this;

        vm.group = parameters.group;
        vm.currentAssignments = vm.group.assigned_customers;

        vm.isCurrentTrainerUser = PermissionsService.getLoggedInUserId() === vm.group.trainer_user_id;

        vm.assignedIds = [];
        angular.forEach(vm.currentAssignments, function (customer) {
            vm.assignedIds.push(customer.id);
        });

        var initAssignedCount = vm.assignedIds.length;

        CustomerStoreService.downloadTrainerCustomers().then(function () {

            CustomerStoreService.listAllTrainerCustomers(vm.group.trainer_user_id).then(function (response) {

                // set full name on root object for search filter
                response.map(function (c) {

                    c.full_name = c.data.user.first_name + ' ' + c.data.user.last_name;

                    c.use = vm.assignedIds.indexOf(c.id) !== -1;
                });

                vm.clients = response;

                vm.filteredClients = angular.copy(response);

                vm.noClients = response.length === 0;
            });
        });

        // Customer List Search filter
        $scope.$watch(function () {
            return vm.searchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.searchFilterActive = true;
                vm.filteredClients = $filter('filter')(vm.clients, {
                    full_name: newVal
                });
            } else {
                vm.searchFilterActive = false;
                vm.filteredClients = vm.clients;
            }
        });

        vm.assign = function (client) {

            client.use = !client.use;

            var findOriginal = $filter('filter')(vm.clients, {
                id: client.id
            }, true);

            if (findOriginal.length > 0) {

                var originalIndex = vm.clients.indexOf(findOriginal[0]);

                if (originalIndex !== -1) {
                    vm.clients[originalIndex].use = client.use;
                }
            }
        };

        vm.save = function () {

            vm.saving = true;

            var toAssign = $filter('filter')(vm.clients, {
                use: true
            });

            if (toAssign.length === 0 && initAssignedCount === 0) {

                $scope.closeModal([]);

                return true;
            }

            var ids = [];
            var list = [];

            toAssign.map(function (customer) {
                ids.push(customer.id);
                list.push(customer.data);
            });

            ApiClientService.nutritionGroups.update(vm.group.id, {
                customer_ids: ids
            }).then(function () {
                $scope.closeModal(list);
            }, function () {
                vm.saving = false;
            });
        };

        AnalyticsService.trackView('Trainer - Assign Clients To Nutrition Plan Group');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('TrainerNutritionPlansGroupController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.trainer_nutrition_plans_group', {
            url: "/trainer_nutrition_plans/groups/:groupId?",
            views: {
                'menuContent': {
                    templateUrl: "trainer/nutrition/group/group.html",
                    controller: 'TrainerNutritionPlansGroupController as vm'
                }
            }
        });
    }

    controller.$inject = ['$stateParams', '$scope', 'ApiClientService', '$filter', 'PermissionsService', 'NavigateService', 'ConfirmDialogService', '$ionicActionSheet', 'AnalyticsService', 'ModalService'];

    function controller($stateParams, $scope, ApiClientService, $filter, PermissionsService, NavigateService, ConfirmDialogService, $ionicActionSheet, AnalyticsService, ModalService) {

        var vm = this;

        vm.group = {
            nutrition_plans: []
        };

        var load = function (refreshing) {

            refreshing = refreshing || false;

            vm.loaded = false;
            vm.nutritionPlansSearchFilter = '';

            ApiClientService.nutritionGroups.get(vm.groupId).success(function (response) {

                vm.group = response.group;
                vm.filteredNutritionPlans = angular.copy(vm.group.nutrition_plans);
                vm.loaded = true;
                vm.isCurrentTrainerUser = PermissionsService.getLoggedInUserId() === vm.group.trainer_user_id;
                vm.canEdit = PermissionsService.userIsTrainerAdminOrOwner(vm.group.trainer_user_id);

                // Plan Search filter
                vm.filteredNutritionPlans = vm.group.nutrition_plans;
                $scope.$watch(function () {
                    return vm.nutritionPlansSearchFilter;
                }, function (newVal, oldVal) {

                    if (typeof newVal === 'undefined') {
                        return false;
                    }

                    if (newVal.length > 0) {
                        vm.filteredNutritionPlans = $filter('filter')(vm.group.nutrition_plans, {
                            title: newVal
                        });
                    } else {
                        vm.filteredNutritionPlans = vm.group.nutrition_plans;
                    }
                });

                // Client Search filter
                vm.filteredCustomers = vm.group.assigned_customers;
                vm.group.assigned_customers.map(function (customer) {

                    customer.name = customer.user.first_name + '' + customer.user.last_name;

                    return customer;
                });
                $scope.$watch(function () {
                    return vm.customerSearchFilter;
                }, function (newVal, oldVal) {

                    if (typeof newVal === 'undefined') {
                        return false;
                    }

                    if (newVal.length > 0) {
                        vm.filteredCustomers = $filter('filter')(vm.group.assigned_customers, {
                            name: newVal
                        });
                    } else {
                        vm.filteredCustomers = vm.group.assigned_customers;
                    }
                });
            }).finally(function () {

                if (refreshing) {
                    $scope.$broadcast('scroll.refreshComplete');
                }
            });
        };

        var resetNutritionPlanSearch = function () {
            vm.nutritionPlansSearchFilter = '';
            vm.filteredNutritionPlans = vm.group.nutrition_plans;
        };

        var resetCustomerSearch = function () {
            vm.customerSearchFilter = '';
            vm.filteredCustomers = vm.group.assigned_customers;
        };

        vm.editNutritionPlan = function (nutritionPlan) {
            NavigateService.go('app.trainer_nutrition_plan_edit', {
                nutritionPlanId: nutritionPlan.id
            });
        };

        vm.refresh = function () {
            load(true);
        };

        vm.delete = function () {

            ConfirmDialogService.open({
                title: "Confirm delete",
                message: "Do you want to remove this group?",
                buttonConfirm: "Confirm",
                buttonCancel: "Cancel"
            }).then(function () {

                ApiClientService.nutritionGroups.delete(vm.groupId).success(function () {
                    NavigateService.clearCache().then(NavigateService.goBack);
                });
            });
        };

        vm.nutritionPlanMenu = function (nutritionPlan) {

            $ionicActionSheet.show({
                titleText: nutritionPlan.title,
                buttons: [{
                    text: 'Edit nutrition plan'
                }],
                destructiveText: 'Remove from group',
                cancelText: 'Cancel',
                buttonClicked: function (index) {

                    if (index === 0) {
                        NavigateService.go('app.trainer_nutrition_plan_edit', {
                            nutritionPlanId: nutritionPlan.id
                        });
                    }

                    return true;
                },
                destructiveButtonClicked: function () {

                    ConfirmDialogService.open({
                        title: "Confirm",
                        message: "Do you want to remove " + nutritionPlan.title + " from the group?",
                        buttonConfirm: "Confirm",
                        buttonCancel: "Cancel"
                    }).then(function () {

                        var inGroupIds = [];

                        nutritionPlan.nutrition_plan_groups.map(function (currentGroup) {

                            if (currentGroup.id === vm.group.id) {
                                return false;
                            }

                            inGroupIds.push(currentGroup.id);
                        });

                        var saveGroupAssignments = ApiClientService.nutrition.updatePlan(nutritionPlan.id, {
                            assigned_group_ids: inGroupIds
                        });

                        saveGroupAssignments.success(function () {

                            vm.group.nutrition_plans.splice(vm.group.nutrition_plans.indexOf(nutritionPlan), 1);

                            var filteredPosition = vm.filteredNutritionPlans.indexOf(nutritionPlan);

                            if (filteredPosition !== -1) {
                                vm.filteredNutritionPlans.splice(filteredPosition, 1);
                            }
                        });
                    });

                    return true;
                }
            });
        };

        vm.assignedCustomerMenu = function (customer) {

            $ionicActionSheet.show({
                titleText: customer.user.first_name + ' ' + customer.user.last_name,
                destructiveText: 'Remove from group',
                cancelText: 'Cancel',
                destructiveButtonClicked: function () {

                    ConfirmDialogService.open({
                        title: "Confirm",
                        message: "Do you want to remove " + customer.user.first_name + ' ' + customer.user.last_name + " from the group?",
                        buttonConfirm: "Confirm",
                        buttonCancel: "Cancel"
                    }).then(function () {
                        ApiClientService.nutritionGroups.unassignCustomer(vm.group.id, customer.id).then(function () {
                            vm.group.assigned_customers.splice(vm.group.assigned_customers.indexOf(customer), 1);
                        });
                    });

                    return true;
                }
            });
        };

        vm.assignNutritionPlans = function () {

            vm.selectedTabIndex = 0;

            ModalService.show({
                templateUrl: "trainer/nutrition/group/nutritionPlanAssignModal.html",
                controller: 'TrainerNutritionGroupNutritionPlanAssignModalController as vm',
                params: {
                    group: vm.group
                }
            }).then(function (assigned) {
                vm.group.nutrition_plans = assigned;
                resetNutritionPlanSearch();
            });
        };

        vm.assignClients = function () {

            vm.selectedTabIndex = 1;

            ModalService.show({
                templateUrl: "trainer/nutrition/group/customerAssignModal.html",
                controller: 'TrainerNutritionGroupCustomerAssignModalController as vm',
                params: {
                    group: vm.group
                }
            }).then(function (assignedClients) {
                vm.group.assigned_customers = assignedClients;
                resetCustomerSearch();
            });
        };

        $scope.$on('$ionicView.beforeEnter', function () {

            vm.groupId = $stateParams.groupId;

            load();
        });

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Trainer - View Nutrition Group');
        });
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('TrainerNutritionGroupNutritionPlanAssignModalController', controller);

    controller.$inject = ['parameters', '$scope', '$filter', 'ApiClientService', 'PermissionsService', 'AnalyticsService'];

    function controller(parameters, $scope, $filter, ApiClientService, PermissionsService, AnalyticsService) {

        var vm = this;

        vm.group = parameters.group;
        vm.currentAssignments = vm.group.nutrition_plans;

        vm.isCurrentTrainerUser = PermissionsService.getLoggedInUserId() === vm.group.trainer_user_id;

        vm.assignedIds = [];
        angular.forEach(vm.currentAssignments, function (nutritionPlan) {
            vm.assignedIds.push(nutritionPlan.id);
        });

        var initAssignedCount = vm.assignedIds.length;
        ApiClientService.trainers.getAllNutritionPlans(vm.group.trainer_user_id).success(function (response) {

            // set full name on root object for search filter
            response.nutritionPlans.map(function (nutritionPlan) {
                nutritionPlan.use = vm.assignedIds.indexOf(nutritionPlan.id) !== -1;
            });

            vm.nutritionPlans = response.nutritionPlans;

            vm.filteredNutritionPlans = angular.copy(response.nutritionPlans);

            vm.noNutritionPlans = response.nutritionPlans.length === 0;
        });

        // List Search filter
        $scope.$watch(function () {
            return vm.searchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.searchFilterActive = true;
                vm.filteredNutritionPlans = $filter('filter')(vm.nutritionPlans, {
                    title: newVal
                });
            } else {
                vm.searchFilterActive = false;
                vm.filteredNutritionPlans = vm.nutritionPlans;
            }
        });

        vm.assign = function (nutritionPlan) {

            nutritionPlan.use = !nutritionPlan.use;

            var findOriginal = $filter('filter')(vm.nutritionPlans, {
                id: nutritionPlan.id
            }, true);

            if (findOriginal.length > 0) {

                var originalIndex = vm.nutritionPlans.indexOf(findOriginal[0]);

                if (originalIndex !== -1) {
                    vm.nutritionPlans[originalIndex].use = nutritionPlan.use;
                }
            }
        };

        vm.save = function () {

            vm.saving = true;

            var toAssign = $filter('filter')(vm.nutritionPlans, {
                use: true
            });

            if (toAssign.length === 0 && initAssignedCount === 0) {

                $scope.closeModal([]);

                return true;
            }

            var ids = [];
            var list = [];

            toAssign.map(function (nutritionPlan) {
                ids.push(nutritionPlan.id);
                list.push(nutritionPlan);
            });

            ApiClientService.nutritionGroups.update(vm.group.id, {
                nutrition_plan_ids: ids
            }).then(function () {
                $scope.closeModal(list);
            }, function () {
                vm.saving = false;
            });
        };

        AnalyticsService.trackView('Trainer - Assign Nutrition Plans To Nutrition Plan Group');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('TrainerNutritionPlanLibraryGroupController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.trainer_nutrition_library_group', {
            url: '/library/nutrition/group/:groupId',
            views: {
                'menuContent': {
                    templateUrl: "trainer/nutrition/library/librarygroup.html",
                    controller: 'TrainerNutritionPlanLibraryGroupController as vm'
                }
            },
            resolve: {
                getGroup: ['ApiClientService', '$stateParams', 'PermissionsService', function (ApiClientService, $stateParams, PermissionsService) {

                    var currentUser = PermissionsService.getCurrentUser();

                    var tag = 'metric';

                    if (currentUser.country === 'US') {
                        tag = 'imperial';
                    }

                    return ApiClientService.nutritionGroups.get($stateParams.groupId, tag);
                }]
            }
        });
    }

    controller.$inject = ['$scope', 'getGroup', '$filter', 'mpthconf', '$ionicLoading', 'ApiClientService', '$q', 'ToastService'];

    function controller($scope, getGroup, $filter, mpthconf, $ionicLoading, ApiClientService, $q, ToastService) {

        var vm = this;

        vm.group = getGroup.group;
        vm.cdnBase = mpthconf.cdn.base;

        vm.bannerImageUrl = false;
        if (vm.group.banner_image) {
            vm.bannerImageUrl = vm.cdnBase + 'nutritionplanlibrary/' + vm.group.banner_image;
        }

        vm.canUseChecked = false;

        var getIds = function (nutritionPlans) {

            var ids = [];

            nutritionPlans.map(function (nutritionPlan) {
                ids.push(nutritionPlan.id);
            });

            return ids;
        };

        var getChecked = function () {
            return $filter('filter')(vm.group.nutrition_plans, { use: true });
        };

        var createGroup = function (nutritionPlanIds) {

            $ionicLoading.show({
                template: 'Please wait...'
            }).then(function () {
                vm.toDuplicateCount = nutritionPlanIds.length;
                vm.duplicatedCount = 1;
                vm.creatingGroup = false;

                vm.percentComplete = 0;

                var duplicateActions = [];
                angular.forEach(nutritionPlanIds, function (id) {

                    var duplicate = ApiClientService.nutrition.duplicatePlan(id);

                    duplicateActions.push(duplicate);

                    duplicate.success(function () {
                        vm.duplicatedCount++;
                        vm.percentComplete += 90 / vm.toDuplicateCount;
                    });
                });

                $q.all(duplicateActions).then(function (duplicateResponses) {
                    vm.creatingGroup = true;

                    var newNutritionPlanIds = [];
                    duplicateResponses.map(function (response) {
                        newNutritionPlanIds.push(response.data.nutritionPlan.id);
                    });

                    ApiClientService.nutritionGroups.create({
                        title: vm.group.title,
                        nutrition_plan_ids: newNutritionPlanIds
                    }).success(function (response) {

                        ToastService.show('Nutrition Plans successfully added to your account');
                        $ionicLoading.hide();
                    });
                }, function (response) {
                    ToastService.show(response.data.error.message);
                    $ionicLoading.hide();
                });
            });
        };

        vm.useChecked = function () {

            var useNutritionPlans = getChecked();

            if (useNutritionPlans.length === 0) {
                return false;
            }

            var ids = getIds(useNutritionPlans);

            createGroup(ids);
        };

        vm.useAll = function () {

            var ids = getIds(vm.group.nutrition_plans);

            createGroup(ids);
        };

        vm.onCheck = function () {

            if (getChecked().length > 0) {
                vm.canUseChecked = true;
            } else {
                vm.canUseChecked = false;
            }
        };

        $scope.$on('$ionicView.beforeEnter', function (event, viewData) {
            viewData.enableBack = true;
        });
    }
})();
(() => {
  class MerchantHeadlineStatsComponentController {
    constructor(ApiClientService, $scope, CurrencyService, DateRangeService, $state, AuthenticationStateService) {
      this.ApiClientService = ApiClientService;
      this.$scope = $scope;
      this.DateRangeService = DateRangeService;
      this.$state = $state;
      this.AuthenticationStateService = AuthenticationStateService;
      this.currentUser = AuthenticationStateService.getCurrentUser();
      this.currency = CurrencyService.getCurrency(this.currentUser.merchant_currency);
      this.ready = false;
      this.data = {
        charges: 0,
        total: 0,
        customers_provisioned: 0,
        sessions: 0,
        customers_free: 0
      };

      DateRangeService.registerRangeChangedHandler($scope, () => {
        this.load();
      });

      this.load();
    }

    load() {
      this.ready = false;
      const params = {
        trainer_user_id: this.currentUser.id,
        date_from: this.DateRangeService.getStartDateApiFormatted(),
        date_to: this.DateRangeService.getEndDateApiFormatted()
      };

      const getData = this.ApiClientService.merchant.getAnalyticsOverview(params);

      getData.success(resp => {
        this.data = resp.data;
        this.data.total /= 100;
        this.ready = true;
      });
    }
  }

  angular.module('MyPTHub').component('merchantHeadlineStatsComponent', {
    templateUrl: 'trainer/packages/components/views/merchantHeadlineStatsComponent.html',
    controller: MerchantHeadlineStatsComponentController,
    controllerAs: 'vm',
    bindings: {
      currentUser: '=?',
      selectableDateRange: '=?'
    }
  });
})();
(() => {
  class TrainerPackageAnalyticsController {
    constructor(ApiClientService, AuthenticationStateService, CurrencyService, $filter, DateRangeService, $scope) {
      this.ApiClientService = ApiClientService;
      this.$filter = $filter;
      this.DateRangeService = DateRangeService;
      this.currentUser = AuthenticationStateService.getCurrentUser();
      this.currency = CurrencyService.getCurrency(this.currentUser.merchant_currency);
      this.chartColors = ['#00e2ee'];
      const vm = this;

      this.grossVolChart = {
        ready: false,
        labels: [],
        series: ['Total'],
        data: [[]],
        data_raw: [],
        options: {
          maintainAspectRatio: false,
          responsive: true,
          tooltips: {
            callbacks: {
              title(tooltipItem, data) {
                return vm.grossVolChart.data_raw[tooltipItem[0].index].tooltipTitle;
              },
              label(tooltipItem, data) {
                const value = data.datasets[0].data[tooltipItem.index];
                return vm.currency.symbol + $filter('currency')(value, '');
              }
            }
          },
          scales: {
            xAxes: [{
              ticks: {
                autoSkip: true,
                maxTicksLimit: 10,
                callback(value, index, values) {
                  return vm.grossVolChart.data_raw[index].tooltipTitle;
                }
              }
            }],
            yAxes: [{
              ticks: {
                beginAtZero: true,
                autoSkip: true,
                maxTicksLimit: 3,
                callback(label, index, labels) {
                  return vm.currency.symbol + $filter('currency')(label, '');
                }
              }
            }]
          }
        }
      };
      this.successfulChargesChart = {
        ready: false,
        labels: [],
        series: ['Total'],
        data: [[]],
        options: {
          maintainAspectRatio: false,
          responsive: true,
          tooltips: {
            callbacks: {
              title(tooltipItem, data) {
                return vm.grossVolChart.data_raw[tooltipItem[0].index].tooltipTitle;
              },
              label(tooltipItem, data) {
                return data.datasets[0].data[tooltipItem.index] || '0';
              }
            }
          },
          scales: {
            xAxes: [{
              ticks: {
                autoSkip: true,
                maxTicksLimit: 4,
                callback(value, index, values) {
                  return vm.grossVolChart.data_raw[index].tooltipTitle;
                }
              }
            }],
            yAxes: [{
              ticks: {
                beginAtZero: true,
                autoSkip: true,
                maxTicksLimit: 3,
                callback(value) {
                  if (value % 1 === 0) {
                    return value;
                  }
                }
              }
            }]
          }
        }
      };
      this.customersChart = {
        ready: false,
        labels: [],
        series: ['Total'],
        data: [[]],
        data_raw: [],
        options: {
          maintainAspectRatio: false,
          responsive: true,
          tooltips: {
            callbacks: {
              title(tooltipItem, data) {
                return vm.customersChart.data_raw[tooltipItem[0].index].tooltipTitle;
              },
              label(tooltipItem, data) {
                return data.datasets[0].data[tooltipItem.index] || '0';
              }
            }
          },
          scales: {
            xAxes: [{
              ticks: {
                autoSkip: true,
                maxTicksLimit: 4,
                callback(value, index, values) {
                  return vm.customersChart.data_raw[index].tooltipTitle;
                }
              }
            }],
            yAxes: [{
              ticks: {
                beginAtZero: true,
                autoSkip: true,
                maxTicksLimit: 3,
                callback(value) {
                  if (value % 1 === 0) {
                    return value;
                  }
                }
              }
            }]
          }
        }
      };

      DateRangeService.registerRangeChangedHandler($scope, () => {
        this.load();
      });

      this.load();
    }

    createGraphTooltipDateTitle(rangeData) {
      const startTime = moment.utc(rangeData.start_time, 'X').local();
      const endTime = moment.utc(rangeData.end_time, 'X').local();

      if (endTime.diff(startTime, 'hours') < 23) {
        // Hours
        return startTime.local().format('HH:mm');
      }

      if (endTime.diff(startTime, 'months') === 0 && endTime.diff(startTime, 'days') === 0) {
        // Days
        return startTime.local().format('Do MMM');
      }

      if (endTime.diff(startTime, 'weeks') === 0) {
        // Week
        return `${startTime.local().format('DD/MM')} - ${endTime.local().format('DD/MM')}`;
      }

      if (endTime.diff(startTime, 'years') === 0) {
        // Months
        return startTime.local().format('MMM YYYY');
      }

      // Years
      return startTime.local().format('YYYY');
    }

    load() {
      this.grossVolChart.ready = false;
      this.successfulChargesChart.ready = false;
      this.customersChart.ready = false;
      this.grossVolChart.data[0] = [];
      this.grossVolChart.data_raw = [];
      this.grossVolChart.labels = [];

      this.successfulChargesChart.data[0] = [];
      this.successfulChargesChart.labels = [];

      this.customersChart.data[0] = [];
      this.customersChart.data_raw = [];
      this.customersChart.labels = [];

      const getChargeData = this.ApiClientService.merchant.getAnalyticsChargeChartData({
        trainer_user_id: this.currentUser.id, // TODO: Change to get user
        date_from: this.DateRangeService.getStartDateApiFormatted(),
        date_to: this.DateRangeService.getEndDateApiFormatted()
      });

      const getCustomerData = this.ApiClientService.merchant.getAnalyticsCustomerChartData({
        trainer_user_id: this.currentUser.id, // TODO: Change to get user
        date_from: this.DateRangeService.getStartDateApiFormatted(),
        date_to: this.DateRangeService.getEndDateApiFormatted()
      });

      getChargeData.success(resp => {
        const chartData = resp.data;

        this.grossVolChart.total = this.$filter('currency')(chartData.total.total / 100, '');
        this.grossVolChart.ready = true;

        this.successfulChargesChart.total = chartData.total.charges;
        this.successfulChargesChart.ready = true;

        chartData.range.map(rangeTick => {
          rangeTick.tooltipTitle = this.createGraphTooltipDateTitle(rangeTick);
          rangeTick.total /= 100;
          const label = moment.unix(rangeTick.start_time).local();
          this.grossVolChart.labels.push(label);
          this.successfulChargesChart.labels.push(label);
          this.grossVolChart.data[0].push(rangeTick.total);
          this.successfulChargesChart.data[0].push(rangeTick.charges);

          this.grossVolChart.data_raw.push(rangeTick);
        });
      });

      getCustomerData.success(resp => {
        const chartData = resp.data;
        this.customersChart.total = chartData.total.created;
        this.customersChart.ready = true;

        chartData.range.map(rangeTick => {
          rangeTick.tooltipTitle = this.createGraphTooltipDateTitle(rangeTick);
          const label = moment.utc(rangeTick.start_time, 'X').local();
          this.customersChart.labels.push(label);
          this.customersChart.data[0].push(rangeTick.created);
          this.customersChart.data_raw.push(rangeTick);
        });
      });
    }
  }

  angular.module('MyPTHub').component('packageAnalytics', {
    templateUrl: 'trainer/packages/components/views/packageAnalytics.html',
    controller: TrainerPackageAnalyticsController,
    controllerAs: 'vm'
  });
})();
(() => {
  class TrainerPackageBalanceController {}

  angular.module('MyPTHub').component('packageBalance', {
    templateUrl: 'trainer/packages/components/views/packageBalance.html',
    controller: TrainerPackageBalanceController,
    controllerAs: 'vm'
  });
})();
(() => {
  class PackageBalanceLogController {
    constructor(DateRangeService, ApiClientService, AuthenticationStateService, CurrencyService, $scope, $filter) {
      this.DateRangeService = DateRangeService;
      this.ApiClientService = ApiClientService;
      this.AuthenticationStateService = AuthenticationStateService;
      this.CurrencyService = CurrencyService;
      this.$scope = $scope;
      this.$filter = $filter;

      this.currentUser = AuthenticationStateService.getCurrentUser();
      this.currency = CurrencyService.getCurrency(this.currentUser.merchant_currency);
      this.ready = false;
      this.balanceReady = false;
      this.transactions = [];
      this.transactionsLastId = false;
      this.balance = {
        available: 0,
        pending: 0
      };

      DateRangeService.registerRangeChangedHandler($scope, () => {
        this.load();
      });

      this.load();
    }

    load() {
      this.ready = false;
      this.balanceReady = false;

      const getBalance = this.ApiClientService.merchant.getBalance();
      getBalance.success(resp => {
        console.log(resp);
        this.balance = {
          available: resp.balance.available[0].amount / 100,
          pending: resp.balance.pending[0].amount / 100
        };
      }).error(resp => {
        console.error('Error getting balance', resp);
      }).then(() => {
        this.balanceReady = true;
      });
      this.loadTransactions();
    }

    loadTransactions(lastPageId = false) {
      const params = {};
      let moreResults = false;

      if (lastPageId) {
        params.starting_after = lastPageId;
      }

      const getTransactions = this.ApiClientService.merchant.getBalanceTransactions(params);

      getTransactions.success(resp => {
        console.log('loaded transactions', resp);
        moreResults = resp.transactions.has_more;
        console.log('Are there more results?', moreResults);

        resp.transactions.data.map(transaction => {
          transaction.type = PackageBalanceLogController.getTransactionType(transaction.type);
          transaction.net_debit = transaction.net < 0;
          transaction.net = this.currency.symbol + this.$filter('currency')(Math.abs(transaction.net) / 100, '');
          transaction.net_display = transaction.net_debit ? `(${transaction.net})` : transaction.net;
          transaction.amount_debit = transaction.amount < 0;
          transaction.amount = this.currency.symbol + this.$filter('currency')(Math.abs(transaction.amount) / 100, '');
          transaction.amount_display = transaction.amount_debit ? `(${transaction.amount})` : transaction.amount;
          transaction.fee_debit = transaction.fee < 0;
          transaction.hasFee = transaction.fee !== 0;
          transaction.fee = this.currency.symbol + this.$filter('currency')(Math.abs(transaction.fee) / 100, '');
          transaction.fee_display = transaction.fee_debit ? transaction.fee : `(${transaction.fee})`;
          transaction.available_on = moment.utc(transaction.available_on, 'X').local().format('DD/MM/YYYY');
          this.transactions.push(transaction);
          this.transactionsLastId = transaction.id;
          return true;
        });

        return moreResults;
      }).then(loadMore => {
        if (loadMore) {
          this.loadTransactions(this.transactionsLastId);
        } else {
          this.ready = true;
          console.log('ALL TXNS', this.transactions);
        }
      });
    }

    static getTransactionType(type) {
      switch (type) {
        case 'adjustment':
          return 'Adjustment';
        case 'application_fee':
          return 'Fee';
        case 'application_fee_refund':
          return 'Fee Refund';
        case 'charge':
          return 'Payment';
        case 'payment':
          return 'Payment';
        case 'payment_refund':
          return 'Refund';
        case 'refund':
          return 'Refund';
        case 'transfer':
        case 'payout':
          return 'Transfer';
        case 'transfer_cancel':
          return 'Cancelled Transfer';
        case 'transfer_failure':
          return 'Failed Transfer';
        case 'transfer_refund':
          return 'Transfer Refund';
        default:
          return 'Unknown';
      }
    }
  }

  angular.module('MyPTHub').component('packageBalanceLog', {
    templateUrl: 'trainer/packages/components/views/packageBalanceLog.html',
    controller: PackageBalanceLogController,
    controllerAs: 'vm'
  });
})();
(() => {
  angular.module('MyPTHub').component('packageSwitcher', {
    templateUrl: 'trainer/packages/components/views/packageSwitcher.html',
    controller,
    controllerAs: 'vm'
  }).factory('PackageSwitcherService', service);

  function controller(AuthenticationStateService, $timeout, PackageSwitcherService) {
    const vm = this;
    vm.currentUser = AuthenticationStateService.getCurrentUser();

    vm.changeAccount = () => {
      PackageSwitcherService.changeAccount(vm.packagesAccount);
    };

    $timeout(() => {
      vm.packagesAccount = vm.currentUser.merchant_account_type;
    });
  }

  function service(AuthenticationStateService, AuthenticationService, ToastService, ApiClientService, NavigateService) {
    const service = {};
    const currentUser = AuthenticationStateService.getCurrentUser();
    const showMessage = message => {
      ToastService.show(message);
    };

    const updateAccount = () => {
      AuthenticationService.refreshUser().then(user => {
        AuthenticationStateService.setCurrentUser(user);
      }).then(() => {
        ToastService.show('Switched Accounts');

        NavigateService.goHome();
        NavigateService.removeCache(['app.trainer_packages']);

        NavigateService.clearCache();
        NavigateService.clearHistory();
      });
    };

    const refreshAccount = () => {
      AuthenticationService.refreshUser().then(user => {
        AuthenticationStateService.setCurrentUser(user);
      }).then(() => {
        ToastService.show('Switched Accounts');
        NavigateService.go(state);
      });
    };

    service.changeAccount = account => {
      ApiClientService.swapper.changeTrainerAccount({
        accountType: account
      }).success(() => {
        updateAccount();
      }).error(error => {
        console.log(error);
        showMessage(error.error.message);
      });
    };

    service.forceAccountChange = () => {
      const accountType = currentUser.merchant_account_type === 'company' ? 'personal' : 'company';
      ApiClientService.swapper.changeTrainerAccount({
        accountType
      }).success(() => {
        updateAccount();
      }).error(error => {
        showMessage(error.error.message);
      });
    };

    return service;
  }
})();
(() => {
  class PackageTransfersLogController {
    constructor(ApiClientService, AuthenticationStateService, CurrencyService, $scope, $filter) {
      this.ApiClientService = ApiClientService;
      this.AuthenticationStateService = AuthenticationStateService;
      this.CurrencyService = CurrencyService;
      this.$scope = $scope;
      this.$filter = $filter;

      this.currentUser = AuthenticationStateService.getCurrentUser();
      this.currency = CurrencyService.getCurrency(this.currentUser.merchant_currency);
      this.ready = false;
      this.transfers = [];

      this.load();
      console.log('lmao');
    }

    load() {
      this.ready = false;
      this.loadTransfers();
    }

    loadTransfers(lastPageId = false) {
      const params = {};

      if (lastPageId) {
        params.starting_after = lastPageId;
      }

      const getTransfers = this.ApiClientService.merchant.getTransfers(params);

      getTransfers.success(resp => {
        resp.transfers.data.map(transfer => {
          transfer.amount /= 100;
          transfer.amount_display = this.currency.symbol + this.$filter('currency')(transfer.amount, '');
          transfer.amount_reversed /= 100;
          transfer.amount_reversed_display = this.currency.symbol + this.$filter('currency')(transfer.amount_reversed, '');
          transfer.reversed_description = '';
          if (transfer.reversed) {
            transfer.reversed_description = transfer.amount_reversed === transfer.amount ? 'Reversed' : `${transfer.amount_reversed_display} reversed`;
            if (transfer.failure_message) {
              transfer.reversed_description += ` - ${transfer.failure_message}`;
            }
          }

          transfer.date_created = moment.utc(transfer.created, 'X').local().format('DD/MM/YYYY');
          // transfer.date_expected = moment.utc(transfer.arrival_date, 'X').local().format('DD/MM/YYYY');
          transfer.status = PackageTransfersLogController.getStatusString(transfer.status);

          this.transfers.push(transfer);

          this.lastTransferId = transfer.id;
          return true;
        });

        return resp.transfers.has_more;
      }).then(loadMore => {
        console.log('gottem');
        if (loadMore) {
          this.loadTransfers(this.lastTransferId);
        } else {
          this.ready = true;
        }
      });
    }

    static getStatusString(status) {
      switch (status) {
        case 'paid':
          return 'Paid';
        case 'pending':
          return 'Pending';
        case 'in_transit':
          return 'In Transit';
        case 'canceled':
          return 'Canceled';
        case 'failed':
          return 'Failed';
        default:
          return 'Unknown';
      }
    }
  }

  angular.module('MyPTHub').component('packageTransfersLog', {
    templateUrl: 'trainer/packages/components/views/packageTransfersLog.html',
    controller: PackageTransfersLogController,
    controllerAs: 'vm'
  });
})();
(() => {
  class TrainerPackageBillingController {}

  angular.module('MyPTHub').component('packageBilling', {
    templateUrl: 'trainer/packages/components/views/packageBilling.html',
    controller: TrainerPackageBillingController,
    controllerAs: 'vm'
  });
})();
(() => {
  class ViewPackageShareLinkController {
    constructor(parameters, $scope, BrandingService, AuthenticationStateService, mpthconf) {
      this.parameters = parameters;
      this.$scope = $scope;
      const domain = mpthconf[mpthconf.env].webapp.current.replace('{subdomain}', AuthenticationStateService.getCurrentUser().branding.subdomain);
      this.url = `${domain}p/${parameters.pkg.id}`;
    }
  }

  class TrainerPackageListController {
    constructor(ApiClientService, $ionicActionSheet, PromptDialogService, ConfirmDialogService, ModalService, NavigateService, ToastService, $scope, AuthenticationStateService) {
      this.ApiClientService = ApiClientService;
      this.$ionicActionSheet = $ionicActionSheet;
      this.PromptDialogService = PromptDialogService;
      this.ConfirmDialogService = ConfirmDialogService;
      this.ModalService = ModalService;
      this.NavigateService = NavigateService;
      this.ToastService = ToastService;
      this.$scope = $scope;
      this.AuthenticationStateService = AuthenticationStateService;

      this.currentUser = AuthenticationStateService.getCurrentUser();

      this.packages = [];
      this.archivedPackages = [];
      this.coupons = [];
      this.loaded = [];

      this.loading = true;
      this.initialLoad();
      this.$scope.$on('refresh.packages.packages', () => this.refresh());
      this.legacyUser = this.currentUser.merchant_is_provisioned && !this.currentUser.stripe_standard;
    }

    /**
     * @returns {*|boolean} yes or no.
     */
    pastCutoff() {
      return this.legacyUser && moment().year() === 2019;
    }

    $onInit() {
      this.NavigateService.removeCache(['app.trainer_package_view']);
    }

    initialLoad() {
      this.loading = true;
      this.loadAllPackages(true);
      this.loadArchivedPackages(true);
      this.loadCoupons(true);
    }

    addLoaded(section) {
      if (this.loaded.length === 3) {
        this.loading = false;
      } else {
        this.loaded.push(section);
        switch (section) {
          case 'packages':
            this.loadingPackages = false;
            break;
          case 'archived':
            this.loadingArchived = false;
            break;
          case 'coupons':
            this.loadingCoupons = false;
            break;
          default:
        }

        if (this.loaded.length === 3) {
          this.loadingPackages = false;
          this.loadingArchived = false;
          this.loadingCoupons = false;
        }
      }
    }

    refresh() {
      switch (this.selectedTabIndex) {
        case 0:
          this.loadAllPackages(true, true);
          break;
        case 1:
          this.loadArchivedPackages(true, true);
          break;
        case 2:
          this.loadCoupons(true, true);
          break;
        default:
          this.$scope.$broadcast('scroll.refreshComplete');
      }
    }

    loadAllPackages(init = false, refreshing = false) {
      if (!init) {
        return;
      }
      if (!refreshing) {
        this.loadingPackages = true;
      }
      return this.ApiClientService.packages.list().success(resp => {
        this.packages = resp.packages;
        if (init) {
          this.addLoaded('packages');
        } else {
          this.loadingPackages = false;
        }
        if (refreshing) {
          this.$scope.$broadcast('scroll.refreshComplete');
        }
      }).error(resp => {
        console.error('Unable to load packages', resp);
        this.ToastService.show('Unable to load packages');
      });
    }

    loadArchivedPackages(init = false, refreshing = false) {
      if (!init) {
        return;
      }
      if (!refreshing) {
        this.loadingArchived = true;
      }
      this.ApiClientService.packages.listDeleted().success(resp => {
        this.archivedPackages = resp.packages;
        if (init) {
          this.addLoaded('archived');
        } else {
          this.loadingArchived = false;
        }
        if (refreshing) {
          this.$scope.$broadcast('scroll.refreshComplete');
        }
      }).error(resp => {
        console.error('Unable to load Archive packages', resp);
        this.ToastService.show('Unable to load packages');
      });
    }

    duplicatePackage(pkg) {
      this.ApiClientService.packages.duplicate(pkg.id).success(resp => {
        this.packages.push(resp.package);
        this.ToastService.show('Package Duplicated');
      }).error(resp => {
        console.error('Unable to duplicate package', pkg, resp);
        this.ToastService.show('Unable to duplicate package');
      });
    }

    deletePackage(pkg, softDelete = false) {
      this.ConfirmDialogService.open({
        title: 'Confirm Delete',
        message: 'Are you sure you want to delete this package?',
        buttonConfirm: 'Delete',
        buttonCancel: 'Cancel'
      }).then(() => {
        let idx;
        let method;
        if (softDelete) {
          idx = this.packages.findIndex(p => p.id === pkg.id);
          method = this.ApiClientService.packages.softDelete(pkg.id);
        } else {
          idx = this.archivedPackages.findIndex(p => p.id === pkg.id);
          method = this.ApiClientService.packages.hardDelete(pkg.id);
        }
        if (idx !== 1) {
          method.success(() => {
            if (softDelete) {
              this.archivedPackages.push(pkg);
              this.packages.splice(idx, 1);
            } else {
              this.archivedPackages.splice(idx, 1);
            }
            this.ToastService.show('Package deleted');
          }).error(resp => {
            console.error('Unable to soft delete package', pkg, resp);
            this.ToastService.show('Unable to delete package');
          });
        } else {
          this.ToastService.show('Unable to delete package');
        }
      });
    }

    restorePackage(pkg) {
      this.ApiClientService.packages.restoreSoftDeleted(pkg.id).success(() => {
        const idx = this.archivedPackages.findIndex(p => p.id === pkg.id);
        if (idx !== -1) {
          this.archivedPackages.splice(idx, 1);
          this.packages.push(pkg);
          this.ToastService.show('Package restored');
        } else {
          this.ToastService.show('Unable to restore package');
        }
      }).error(resp => {
        console.error('Unable to restore package', pkg, resp);
        this.ToastService.show('Unable to restore package');
      });
    }

    renamePackage(pkg) {
      this.PromptDialogService.open({
        title: 'Rename package',
        message: 'Enter a new package name',
        buttonConfirm: 'Save',
        buttonCancel: 'Cancel',
        required: true,
        defaultText: pkg.title
      }).then(title => {
        const data = {
          title
        };
        this.ApiClientService.packages.update(pkg.id, data).success(() => {
          const idx = this.packages.findIndex(p => p.id === pkg.id);
          if (idx !== -1) {
            this.packages[idx].title = title;
          }
          this.ToastService.show('Package renamed');
        }).error(resp => {
          console.error('Package rename failed', title, pkg, resp);
          this.ToastService.show('Unable to rename package');
        });
      });
    }

    viewPackage(pkg) {
      console.log(pkg);
      this.NavigateService.go('app.trainer_package_view', {
        packageId: pkg.id
      });
    }

    editPackage(pkg) {
      if (this.pastCutoff()) {
        return;
      }
      this.NavigateService.go('app.trainer_edit_package', {
        packageId: pkg.id
      });
    }

    showShareLink(pkg) {
      console.log(pkg);
      this.ModalService.show({
        templateUrl: 'trainer/packages/components/views/packageShareLink.html',
        controller: 'ViewPackageShareLinkController as vm',
        params: { pkg }
      });
    }

    loadCoupons(init = false, refreshing = false) {
      if (!init) {
        return;
      }
      if (!refreshing) {
        this.loadingCoupons = true;
      }
      return this.ApiClientService.packages.listAllCoupons().success(resp => {
        this.coupons = resp.coupons;
        if (init) {
          this.addLoaded('coupons');
        } else {
          this.loadingCoupons = false;
        }
        if (refreshing) {
          this.$scope.$broadcast('scroll.refreshComplete');
        }
      }).error(resp => {
        console.error('unable to load coupons', resp);
        this.ToastService.show('Unable to load coupons');
      });
    }

    editCoupon(coupon) {
      this.NavigateService.go('app.trainer_update_coupon', {
        couponId: coupon.id
      });
    }

    deleteCoupon(coupon) {
      this.ConfirmDialogService.open({
        title: 'Confirm Delete',
        message: 'Are you sure you want to permanently delete this voucher?',
        buttonConfirm: 'Delete Permanently',
        buttonCancel: 'Cancel'
      }).then(() => {
        const idx = this.coupons.findIndex(c => c.id === coupon.id);
        if (idx !== -1) {
          this.ApiClientService.packages.deleteCoupon(coupon.id).success(() => {
            this.coupons.splice(idx, 1);
            this.ToastService.show('Voucher Deleted');
          }).error(resp => {
            console.error('Unable to delete coupon', coupon, resp);
            this.ToastService.show('Unable to delete voucher');
          });
        } else {
          console.error('Voucher not found', coupon);
          this.ToastService.show('Unable to delete voucher');
        }
      });
    }

    showCouponItemMenu(coupon) {
      this.$ionicActionSheet.show({
        buttons: [{
          text: 'Edit Voucher',
          action: () => {
            this.editCoupon(coupon);
          }
        }],
        cancelText: 'Cancel',
        destructiveText: 'Delete',
        destructiveButtonClicked: () => {
          this.deleteCoupon(coupon);
          return true;
        },
        buttonClicked(index) {
          if (typeof this.buttons[index] !== 'undefined' && typeof this.buttons[index].action === 'function') {
            this.buttons[index].action(index);
          }
          return true;
        }
      });
    }

    showPackageItemMenu(pkg) {
      this.$ionicActionSheet.show({
        buttons: [{
          text: 'View Package',
          action: () => {
            this.viewPackage(pkg);
          },
          when: false
        }, {
          text: 'Edit Package',
          action: () => {
            this.editPackage(pkg);
          },
          when: this.selectedTabIndex === 0 && !this.pastCutoff()
        }, {
          text: 'Rename',
          action: () => {
            this.renamePackage(pkg);
          },
          when: this.selectedTabIndex === 0 && !this.pastCutoff()
        }, {
          text: 'Duplicate Package',
          action: () => {
            this.duplicatePackage(pkg);
          },
          when: this.selectedTabIndex === 0 && !this.pastCutoff()
        }, {
          text: 'Get Share Link',
          action: () => {
            this.showShareLink(pkg);
          },
          when: this.selectedTabIndex === 0 && !this.pastCutoff()
        }, {
          text: 'Restore package',
          action: () => {
            this.restorePackage(pkg);
          },
          when: this.selectedTabIndex === 1 && !this.pastCutoff()
        }].filter(item => item.when), // Only show buttons with the 'when' property
        cancelText: 'Cancel',
        destructiveText: this.selectedTabIndex === 1 ? 'Delete permanently' : 'Delete',
        destructiveButtonClicked: () => {
          if (this.selectedTabIndex === 1) {
            this.deletePackage(pkg);
          } else {
            this.deletePackage(pkg, true);
          }
          return true;
        },
        buttonClicked(index) {
          if (typeof this.buttons[index] !== 'undefined' && typeof this.buttons[index].action === 'function') {
            this.buttons[index].action(index);
          }
          return true;
        }
      });
    }
  }

  angular.module('MyPTHub').component('packagesList', {
    templateUrl: 'trainer/packages/components/views/packageList.html',
    controller: TrainerPackageListController,
    controllerAs: 'vm'
  }).controller('ViewPackageShareLinkController', ViewPackageShareLinkController);
})();
(() => {
  class PaymentsActivityLogController {
    constructor(DateRangeService, ApiClientService, AuthenticationStateService, CurrencyService, $scope) {
      this.DateRangeService = DateRangeService;
      this.ApiClientService = ApiClientService;
      this.AuthenticationStateService = AuthenticationStateService;
      this.CurrencyService = CurrencyService;
      this.$scope = $scope;

      this.currentUser = AuthenticationStateService.getCurrentUser();
      this.charges = [];
      this.free = [];
      this.ready = false;
      this.payments = [];

      DateRangeService.registerRangeChangedHandler($scope, () => {
        this.load();
      });

      this.load();
    }

    load() {
      this.ready = false;
      const getCharges = this.ApiClientService.merchant.getCharges({
        trainer_user_id: this.currentUser.id,
        date_from: this.DateRangeService.getStartDateApiFormatted(),
        date_to: this.DateRangeService.getEndDateApiFormatted()
      });

      getCharges.success(resp => {
        this.free = resp.free;
        resp.charges.map(charge => {
          charge.currency = this.CurrencyService.getCurrency(charge.currency);
          if (charge.amount_refunded > 0) {
            charge.amount_refunded /= 100;
          }
          charge.amount /= 100;
          const unix = charge.created;
          charge.created = moment.utc(unix, 'X').local().format('DD-MM-YYYY HH:mm:ss');
          charge.created_at = moment.utc(unix, 'X').local().format('YYYY-MM-DD HH:mm:ss');
          charge.free = false;
          charge.canCancelSub = charge.purchase && charge.purchase.active && charge.purchase.type === 'recurring' && charge.purchase.ends_at === null;

          return charge;
        });

        this.charges = resp.charges;
      }).then(() => {
        const charges = this.charges.concat(this.free);
        charges.sort((a, b) => {
          const date1 = new Date(a.created_at);
          const date2 = new Date(b.created_at);
          return date1 > date2 ? -1 : date1 < date2 ? 1 : 0;
        });
        this.payments = charges;
        this.ready = true;
      });
    }
  }

  angular.module('MyPTHub').component('paymentsActivityLog', {
    templateUrl: 'trainer/packages/components/views/paymentsActivityLog.html',
    controller: PaymentsActivityLogController,
    controllerAs: 'vm'
  });
})();
(() => {
  class SubscribersLogController {
    constructor(DateRangeService, ApiClientService, AuthenticationStateService, $scope) {
      this.DateRangeService = DateRangeService;
      this.ApiClientService = ApiClientService;
      this.AuthenticationStateService = AuthenticationStateService;
      this.$scope = $scope;

      this.currentUser = AuthenticationStateService.getCurrentUser();
      this.subscribers = [];
      this.ready = false;
      this.payments = [];

      DateRangeService.registerRangeChangedHandler($scope, () => {
        this.load();
      });

      this.load();
    }

    load() {
      this.ready = false;

      const getCustomers = this.ApiClientService.merchant.getCustomers({
        trainer_user_id: this.currentUser.id,
        date_from: this.DateRangeService.getStartDateApiFormatted(),
        date_to: this.DateRangeService.getEndDateApiFormatted()
      });

      getCustomers.success(resp => {
        resp.customers.map(customer => {
          customer.stripe_provisioned_at = moment.utc(customer.stripe_provisioned_at, 'YYYY-MM-DD HH:mm:ss').local().format('DD/MM/YYYY HH:mm:ss');
          return customer;
        });

        this.subscribers = resp.customers;
        this.ready = true;
      });
    }
  }

  angular.module('MyPTHub').component('subscribersLog', {
    templateUrl: 'trainer/packages/components/views/subscribersLog.html',
    controller: SubscribersLogController,
    controllerAs: 'vm'
  });
})();
(() => {
  const service = () => {
    const service = {};
    let formVm;

    service.getExpiryDate = () => {
      if (!formVm.couponExpiry) {
        return null;
      }

      const date = moment(formVm.couponExpiry).endOf('day').utc();

      return date.format('YYYY-MM-DD HH:mm:ss');
    };

    service.setAvailableUntilDate = couponObj => {
      if (!couponObj.expires_at) {
        return null;
      }

      formVm.couponExpiry = moment.utc(couponObj.expires_at, 'YYYY-MM-DD HH:mm:ss').toDate();
      return true;
    };

    service.setup = couponObj => {
      service.setAvailableUntilDate(couponObj);
      service.setAmountType(couponObj);
      service.setAmount(couponObj);
    };

    service.getAmountType = () => formVm.couponAmountType;

    service.setAmountType = couponObj => {
      formVm.couponAmountType = couponObj.amount_type;
    };

    service.setAmount = couponObj => {
      if (!couponObj.amount) {
        return null;
      }

      if (couponObj.amount_type === 'amount') {
        formVm.couponAmount = couponObj.amount / 100;
      } else {
        formVm.couponAmount = couponObj.amount;
      }
      return true;
    };

    service.getAmount = () => {
      if (service.getAmountType() === 'amount') {
        return formVm.couponAmount * 100;
      }

      return formVm.couponAmount;
    };

    service.load = vm => {
      formVm = vm;

      return true;
    };

    return service;
  };

  angular.module('MyPTHub').factory('TrainerPackageCouponService', service);
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('PackagesGiftSessionsDialogService', service).controller('PackagesGiftSessionsDialogController', controller);

    service.$inject = ['ModalService'];

    function service(ModalService) {

        return {

            show: function (customerId) {

                return ModalService.show({
                    templateUrl: "trainer/packages/giftSessions/giftSessions.html",
                    controller: 'PackagesGiftSessionsDialogController as vm',
                    params: {
                        customerId: customerId
                    }
                });
            }

        };
    }

    controller.$inject = ['$scope', 'ApiClientService', 'parameters', '$mdToast'];

    function controller($scope, ApiClientService, parameters, $mdToast) {

        var vm = this;

        vm.ready = false;
        vm.quantity = 1;

        var getActivePurchases = ApiClientService.packages.listActiveCustomerPurchases(parameters.customerId);

        getActivePurchases.success(function (response) {

            vm.ready = true;

            response.purchases.map(function (purchase) {

                purchase.remaining_sessions = purchase.currentPeriodSessionQuota.sessions_available - purchase.currentPeriodSessionQuota.sessions_used;

                var word = purchase.remaining_sessions !== 1 ? 'sessions' : 'session';

                purchase.description = purchase.package.title + ' (' + (purchase.remaining_sessions > 0 ? purchase.remaining_sessions : 'no') + ' ' + word + ' remaining)';

                return purchase;
            });

            vm.purchases = response.purchases;

            if (vm.purchases.length === 0) {
                vm.hasPurchases = false;
                vm.enablePurchase = false;
            } else {
                vm.hasPurchases = true;
                vm.usePackagePurchase = vm.purchases[0].id;
                vm.enablePurchase = true;
            }
        });

        getActivePurchases.error(function (response) {

            if (response.error) {
                $mdToast.show($mdToast.simple().textContent(response.error.message).position('top right'));
            } else {
                $mdToast.show($mdToast.simple().textContent('Could not load active packages').position('top right'));
            }

            $scope.cancelModal();
        });

        vm.doGift = function () {

            vm.processing = true;

            var gift;

            if (vm.enablePurchase) {

                gift = ApiClientService.merchant.giftPackageSessions(vm.usePackagePurchase, {
                    quantity: vm.quantity
                });
            } else {

                gift = ApiClientService.merchant.giftSessions(parameters.customerId, {
                    quantity: vm.quantity
                });
            }

            gift.success(function (response) {
                $scope.closeModal(true);
            });

            gift.error(function (response) {

                if (response.error) {
                    $mdToast.show($mdToast.simple().textContent(response.error.message).position('top right'));
                } else {
                    $mdToast.show($mdToast.simple().textContent('Unable to gift sessions - please try again').position('top right'));
                }

                vm.processing = false;
            });
        };

        vm.close = function () {
            $scope.cancelModal();
        };
    }
})();
(() => {
  angular.module('MyPTHub').factory('TrainerPackageFormService', service);

  function service(ApiClientService, AuthenticationStateService) {
    const service = {};

    return service;
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('TrainerSchedulePreviewController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.trainer_schedule_preview', {
            url: "/scheduler/preview/:scheduleId",
            views: {
                'menuContent': {
                    templateUrl: "trainer/scheduler/preview/preview.html",
                    controller: 'TrainerSchedulePreviewController as vm'
                }
            }
        });
    }

    controller.$inject = ['$scope', 'ApiClientService', 'PermissionsService', 'ToastService', '$ionicLoading', 'NavigateService', '$stateParams', '$timeout', 'mpthconf'];

    function controller($scope, ApiClientService, PermissionsService, ToastService, $ionicLoading, NavigateService, $stateParams, $timeout, mpthconf) {
        var vm = this;

        vm.cdnBase = mpthconf.cdn.base;

        vm.currentSlide = 0;
        vm.backText = 'Back';

        vm.slideOptions = {
            loop: false,
            effect: 'slide',
            speed: 500,
            pagination: '.week-pagination',
            paginationClickable: true
        };

        var dayTemplate = {
            nutrition: null,
            rest_day: false,
            workout_am: null,
            workout_pm: null
        };

        var weekTemplate = {
            schedule_id: null,
            order: null,
            days: [angular.copy(dayTemplate), angular.copy(dayTemplate), angular.copy(dayTemplate), angular.copy(dayTemplate), angular.copy(dayTemplate), angular.copy(dayTemplate), angular.copy(dayTemplate)]
        };

        var checkLastDay = function () {
            if (isContentAfterLastDay()) {
                resetDays();
            }

            recountDays();
        };

        var recountDays = function () {
            var lastDayReached = false;
            var dayCount = 0;
            angular.forEach(vm.template.weeks, function (week) {
                angular.forEach(week.days, function (day) {
                    if (!lastDayReached) {
                        dayCount++;

                        day.count = dayCount;

                        if (day.last_day) {
                            lastDayReached = true;
                        }
                    }
                });
            });

            vm.template.day_count = dayCount;
        };

        var resetDays = function (keepLastEndDate) {
            keepLastEndDate = keepLastEndDate || false;
            var lastDay = null;
            var dayCounter = 0;
            angular.forEach(vm.template.weeks, function (week) {
                angular.forEach(week.days, function (day) {
                    dayCounter++;
                    if (keepLastEndDate) {
                        if (day.last_day) {
                            lastDay = dayCounter;
                        }
                        day.past_last_day = lastDay && dayCounter > lastDay;
                    } else {
                        day.past_last_day = false;
                        day.last_day = false;
                    }
                });
            });
            recountDays();
        };

        var setDayCount = function (count) {
            var lastDayReached = false;
            var dayCounter = 0;

            angular.forEach(vm.template.weeks, function (week) {
                angular.forEach(week.days, function (day) {
                    dayCounter++;
                    day.last_day = false;

                    if (dayCounter == count) {
                        day.last_day = true;
                        lastDayReached = true;
                    } else if (lastDayReached) {
                        day.past_last_day = true;
                    }
                });
            });

            recountDays();
        };

        var isContentAfterLastDay = function () {
            var isContent = false;
            var lastDayReached = false;
            angular.forEach(vm.template.weeks, function (week) {
                angular.forEach(week.days, function (day) {
                    if (lastDayReached) {
                        if (day.workout_am || day.workout_pm || day.nutrition || day.update_photo || day.update_metrics || day.rest_day) {
                            isContent = true;
                        }
                    } else if (day.last_day) {
                        lastDayReached = true;
                    }
                });
            });

            return isContent;
        };

        $scope.$on('$ionicView.beforeEnter', function () {
            vm.currentUser = PermissionsService.getCurrentUser();
            vm.template = {};
            vm.saving = false;
            vm.loading = true;
            vm.context = '';
            vm.collapsed = false;
            vm.currentSlide = 0;

            var doGet = ApiClientService.scheduler.scheduleTemplateGet($stateParams.scheduleId);

            doGet.success(function (response) {
                vm.template = response.template;
                vm.loading = false;

                // Add week 1 by default
                if (!vm.template.weeks || vm.template.weeks && !vm.template.weeks.length) {
                    vm.addWeek(false, true);
                }

                // Only set last day if the day count is not equal to the end of the week (multiple of 7)
                if (vm.template.day_count % 7 !== 0) {
                    setDayCount(vm.template.day_count);
                } else {
                    recountDays();
                }

                vm.slider.updateLoop();
            });

            doGet.error(function (response) {
                ToastService.show(response.error.message);
                vm.loading = false;
            });
        });

        vm.addWeek = function (week, forcePreserveLastDay) {
            var newWeek;

            forcePreserveLastDay = forcePreserveLastDay || false;

            if (!vm.template.weeks) {
                vm.template.weeks = [];
            }

            var isFirstWeek = vm.template.weeks.length === 0;

            if (week) {
                newWeek = angular.copy(week);
            } else {
                newWeek = angular.copy(weekTemplate);
            }

            if (isFirstWeek) {
                newWeek.days[0].update_photo = true;
                newWeek.days[0].update_metrics = true;
            }

            vm.template.weeks.push(newWeek);

            // Slide to new week
            vm.slider.updateLoop();

            goToWeek(vm.slider.slides.length);

            var preserveLastDay = true;
            if (week || forcePreserveLastDay) {
                preserveLastDay = false;
            }

            resetDays(preserveLastDay);
        };

        vm.goBack = function () {
            if (currentlySliding) {
                return false;
            }

            if (vm.currentSlide > 0) {
                $timeout(function () {
                    vm.slider.slideTo(vm.currentSlide - 1);
                }, 100);
            } else {
                NavigateService.goBack();
            }
        };

        vm.useSchedule = function () {
            if (vm.processing) {
                return false;
            }

            vm.processing = true;

            $ionicLoading.show({
                template: 'Please wait...'
            }).then(function () {
                ApiClientService.scheduleLibrary.duplicate(vm.template.id).then(function (response) {
                    ToastService.show('Schedule successfully added to your account');
                }).finally(function () {
                    vm.processing = false;
                    $ionicLoading.hide();
                });
            });
        };

        var currentlySliding = false;
        $scope.$on("$ionicSlides.sliderInitialized", function (event, data) {
            vm.slider = data.slider;
            vm.currentSlide = data.slider.activeIndex;
        });

        $scope.$on("$ionicSlides.slideChangeStart", function (event, data) {
            currentlySliding = true;
        });

        $scope.$on("$ionicSlides.slideChangeEnd", function (event, data) {
            vm.currentSlide = data.slider.activeIndex;
            currentlySliding = false;
            $timeout(function () {
                if (vm.currentSlide > 1) {
                    vm.backText = 'Week ' + (vm.currentSlide - 1);
                } else {
                    vm.backText = 'Back';
                }
            });
        });
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('TrainerProfileEmailModalController', controller);

    controller.$inject = ['parameters', 'PermissionsService', 'ApiClientService', 'ToastService', '$scope'];

    function controller(parameters, PermissionsService, ApiClientService, ToastService, $scope) {

        var vm = this;

        vm.trainerUser = parameters.trainerUser;

        vm.email = vm.trainerUser.email;

        vm.canEdit = PermissionsService.userIsTrainerAdminOrOwner(vm.trainerUser.id);

        vm.save = function () {

            vm.saving = true;

            ApiClientService.trainerUsers.update(vm.trainerUser.id, {
                email: vm.email
            }).success(function () {

                $scope.closeModal();
            }).error(function () {

                vm.saving = false;

                ToastService.show('Unable to update the email address, please try again later');
            });
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('TrainerProfileController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.trainer_profile', {
            url: "/trainers/profile/:trainerUserId",
            views: {
                'menuContent': {
                    templateUrl: "trainer/trainers/profile/profile.html",
                    controller: 'TrainerProfileController as vm'
                }
            }
        });
    }

    controller.$inject = ['$stateParams', 'ApiClientService', '$scope', 'CustomerStoreService', 'NetworkStateService', 'NavigateService', '$filter', 'PermissionsService', '$ionicActionSheet', 'ConfirmDialogService', 'ModalService', 'ToastService', 'AnalyticsService'];

    function controller($stateParams, ApiClientService, $scope, CustomerStoreService, NetworkStateService, NavigateService, $filter, PermissionsService, $ionicActionSheet, ConfirmDialogService, ModalService, ToastService, AnalyticsService) {

        var vm = this;

        var loadTrainer = function () {

            ApiClientService.trainerUsers.get($stateParams.trainerUserId).success(function (response) {
                vm.trainerUser = response.trainerUser;
                vm.ready = true;
            });
        };

        var loadCustomers = function (init) {

            /**
             * Grabs from the local DB first, and then if online it will
             * get the latest data from the API, to make for a better
             * experience by having something available immediately.
             */

            init = init || false;

            CustomerStoreService.listAllTrainerCustomers($stateParams.trainerUserId).then(function (response) {

                // set full name on root object for search filter
                response.map(function (c) {
                    c.full_name = c.data.user.first_name + ' ' + c.data.user.last_name;
                });

                vm.clients = response;

                vm.filteredClients = angular.copy(response);

                vm.noClients = response.length === 0 && !init;
            });

            if (init && NetworkStateService.isOnline()) {

                CustomerStoreService.downloadTrainerCustomers($stateParams.trainerUserId).then(function () {
                    loadCustomers();
                });
            }
        };

        // Customer List Search filter
        $scope.$watch(function () {
            return vm.searchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.searchFilterActive = true;
                vm.filteredClients = $filter('filter')(vm.clients, {
                    full_name: newVal
                });
            } else {
                vm.searchFilterActive = false;
                vm.filteredClients = vm.clients;
            }
        });

        $scope.$on('$ionicView.beforeEnter', function () {

            vm.trainerUserId = $stateParams.trainerUserId;

            vm.canEdit = PermissionsService.userIsTrainerAdminOrOwner(vm.trainerUserId);

            loadTrainer();

            loadCustomers(true);
        });

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Trainer - View Trainer Profile');
        });

        vm.goToClient = function (customer) {
            NavigateService.go('app.trainer_client_profile', {
                customerId: customer.id
            });
        };

        vm.goToWorkouts = function () {
            NavigateService.go('app.trainer_workouts', {
                trainerUserId: vm.trainerUserId
            });
        };

        vm.goToNutrition = function () {
            NavigateService.go('app.trainer_nutrition', {
                trainerUserId: vm.trainerUserId
            });
        };

        vm.goToSchedule = function () {
            NavigateService.go('app.trainer_calendar', {
                trainerUserId: vm.trainerUserId
            });
        };

        vm.showSettingsMenu = function () {

            $ionicActionSheet.show({
                titleText: 'Additional Actions',
                buttons: [{
                    text: 'Email Address'
                }],
                destructiveText: 'Delete Trainer',
                cancelText: 'Cancel',
                buttonClicked: function (index) {

                    if (index === 0) {

                        ModalService.show({
                            templateUrl: "trainer/trainers/profile/emailModal.html",
                            controller: 'TrainerProfileEmailModalController as vm',
                            params: {
                                trainerUser: vm.trainerUser
                            }
                        }).then(loadTrainer);
                    }

                    return true;
                },
                destructiveButtonClicked: function () {

                    ConfirmDialogService.open({
                        title: "Confirm Delete",
                        message: "Are you sure you want to remove " + vm.trainerUser.first_name + "? If you do remove " + vm.trainerUser.first_name + ", they will no longer be able to access their account. In addition, all of their clients, workouts, nutrition plans and booked sessions will be removed.",
                        buttonConfirm: "Delete Trainer",
                        buttonCancel: "Cancel"
                    }).then(function () {

                        // perform the delete
                        ApiClientService.trainerUsers.delete(vm.trainerUserId).then(function () {

                            ToastService.show('Trainer successfully deleted');

                            NavigateService.goHome();
                        });
                    });

                    return true;
                }
            });
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('CreateWorkoutStartModalController', controller);

    controller.$inject = ['ApiClientService', 'parameters', 'WorkoutEditorService', '$state', 'ToastService', '$scope', 'AnalyticsService'];

    function controller(ApiClientService, parameters, WorkoutEditorService, $state, ToastService, $scope, AnalyticsService) {

        var vm = this;

        vm.ready = false;

        var options = parameters.options;

        var customer = false;
        if (parameters.customer) {
            customer = parameters.customer;
        }

        vm.newWorkout = {};

        vm.createMode = 'new';

        vm.loaded = false;
        vm.workouts = [];
        vm.chosenTemplateWorkoutId = null;

        var trainerUserId = false;
        if (options.trainerUserId) {
            trainerUserId = options.trainerUserId;
        }

        ApiClientService.subscription_checker.canCreateWorkout().success(function () {

            loadWorkouts();
        }).error(function (response) {

            if (response.error && response.error.error_name === 'SUBSCRIPTION_UPGRADE_REQUIRED') {
                ToastService.show('Upgrade required to add more clients.');
            }

            $scope.cancelModal();
        });

        var loadWorkouts = function () {
            ApiClientService.trainers.getAllWorkouts(trainerUserId).success(function (result) {

                vm.loaded = true;
                vm.workouts = result.workouts;
            }).error(function () {
                vm.loaded = true;
            });
        };

        var workoutCreate = function (workout) {

            workout.title = vm.newWorkout.title;

            if (customer) {
                workout.assigned_customer_ids = [getCustomer.id];
            }

            if (options.viaGroup) {
                workout.assigned_group_ids = [options.viaGroup];
            }

            if (trainerUserId) {
                workout.trainer_user_id = trainerUserId;
            } else if (customer) {
                workout.trainer_user_id = customer.trainer_user_id;
            }

            workout = WorkoutEditorService.transformWorkout(workout);

            ApiClientService.workouts.create(workout).success(function (response) {

                var stateName = 'app.edit_workout';

                var params = {
                    workoutId: response.workout.id
                };

                if (options.viaGroup) {
                    params.viaGroup = options.viaGroup;
                }

                if (customer) {
                    params.customerId = customer.id;
                    stateName = 'app.edit_trainer_workout_via_customer';
                }

                $state.go(stateName, params).then(function () {
                    $scope.cancelModal();
                });
            }).error(function (response) {

                if (response.error) {
                    ToastService.show(response.error.message);
                }

                vm.creating = false;
            });
        };

        vm.doCreate = function () {

            vm.creating = true;

            var workout = {};

            if (vm.createMode === 'existing' && vm.chosenTemplateWorkoutId) {

                ApiClientService.workouts.get(vm.chosenTemplateWorkoutId, true).success(function (response) {

                    delete response.workout.customer_id;
                    delete response.workout.title;
                    delete response.workout.assigned_customers;

                    workout = response.workout;

                    WorkoutEditorService.transformWorkout(workout);

                    workoutCreate(workout);
                });
            } else {

                workoutCreate(workout);
            }
        };

        AnalyticsService.trackView('Trainer - Create Workout Start');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('CreateWorkoutService', service);

    service.$inject = ['ModalService'];

    function service(ModalService) {

        return {

            start: function (options, customer) {

                customer = customer || false;

                ModalService.show({
                    templateUrl: "trainer/workouts/Create/CreateModal.html",
                    controller: 'CreateWorkoutStartModalController as vm',
                    params: {
                        options: options,
                        getCustomer: customer
                    }
                });
            }

        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('TrainerWorkoutGroupCustomerAssignModalController', controller);

    controller.$inject = ['parameters', '$scope', 'CustomerStoreService', '$filter', 'ApiClientService', 'PermissionsService', 'AnalyticsService'];

    function controller(parameters, $scope, CustomerStoreService, $filter, ApiClientService, PermissionsService, AnalyticsService) {

        var vm = this;

        vm.group = parameters.group;
        vm.currentAssignments = vm.group.assigned_customers;

        vm.isCurrentTrainerUser = PermissionsService.getLoggedInUserId() === vm.group.trainer_user_id;

        vm.assignedIds = [];
        angular.forEach(vm.currentAssignments, function (customer) {
            vm.assignedIds.push(customer.id);
        });

        var initAssignedCount = vm.assignedIds.length;

        CustomerStoreService.downloadTrainerCustomers().then(function () {

            CustomerStoreService.listAllTrainerCustomers(vm.group.trainer_user_id).then(function (response) {

                // set full name on root object for search filter
                response.map(function (c) {

                    c.full_name = c.data.user.first_name + ' ' + c.data.user.last_name;

                    c.use = vm.assignedIds.indexOf(c.id) !== -1;
                });

                vm.clients = response;

                vm.filteredClients = angular.copy(response);

                vm.noClients = response.length === 0;
            });
        });

        // Customer List Search filter
        $scope.$watch(function () {
            return vm.searchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.searchFilterActive = true;
                vm.filteredClients = $filter('filter')(vm.clients, {
                    full_name: newVal
                });
            } else {
                vm.searchFilterActive = false;
                vm.filteredClients = vm.clients;
            }
        });

        vm.assign = function (client) {
            client.use = !client.use;

            var findOriginal = $filter('filter')(vm.clients, {
                id: client.id
            }, true);

            if (findOriginal.length > 0) {

                var originalIndex = vm.clients.indexOf(findOriginal[0]);

                if (originalIndex !== -1) {
                    vm.clients[originalIndex].use = client.use;
                }
            }
        };

        vm.save = function () {

            vm.saving = true;

            var toAssign = $filter('filter')(vm.clients, {
                use: true
            });

            if (toAssign.length === 0 && initAssignedCount === 0) {

                $scope.closeModal([]);

                return true;
            }

            var ids = [];
            var list = [];

            toAssign.map(function (customer) {
                ids.push(customer.id);
                list.push(customer.data);
            });

            ApiClientService.workoutGroups.update(vm.group.id, {
                customer_ids: ids
            }).then(function () {
                $scope.closeModal(list);
            }, function () {
                vm.saving = false;
            });
        };

        AnalyticsService.trackView('Trainer - Assign Clients To Workout Group');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('TrainerWorkoutsGroupController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.trainer_workouts_group', {
            url: "/trainer_workouts/groups/:groupId?",
            views: {
                'menuContent': {
                    templateUrl: "trainer/workouts/group/group.html",
                    controller: 'TrainerWorkoutsGroupController as vm'
                }
            }
        });
    }

    controller.$inject = ['$stateParams', '$scope', 'ApiClientService', '$filter', 'PermissionsService', 'NavigateService', 'ConfirmDialogService', '$ionicActionSheet', 'AnalyticsService', 'ModalService'];

    function controller($stateParams, $scope, ApiClientService, $filter, PermissionsService, NavigateService, ConfirmDialogService, $ionicActionSheet, AnalyticsService, ModalService) {

        var vm = this;

        vm.group = {
            workouts: []
        };

        var load = function (refreshing) {

            refreshing = refreshing || false;

            vm.loaded = false;

            vm.workoutsSearchFilter = '';

            ApiClientService.workoutGroups.get(vm.groupId).success(function (response) {

                vm.group = response.group;
                vm.filteredWorkouts = angular.copy(vm.group.workouts);
                vm.loaded = true;
                vm.isCurrentTrainerUser = PermissionsService.getLoggedInUserId() === vm.group.trainer_user_id;
                vm.canEdit = PermissionsService.userIsTrainerAdminOrOwner(vm.group.trainer_user_id);

                // Workouts Search filter
                vm.filteredWorkouts = vm.group.workouts;
                $scope.$watch(function () {
                    return vm.workoutsSearchFilter;
                }, function (newVal, oldVal) {

                    if (typeof newVal === 'undefined') {
                        return false;
                    }

                    if (newVal.length > 0) {
                        vm.filteredWorkouts = $filter('filter')(vm.group.workouts, {
                            title: newVal
                        });
                    } else {
                        vm.filteredWorkouts = vm.group.workouts;
                    }
                });

                // Client Search filter
                vm.filteredCustomers = vm.group.assigned_customers;
                vm.group.assigned_customers.map(function (customer) {

                    customer.name = customer.user.first_name + '' + customer.user.last_name;

                    return customer;
                });
                $scope.$watch(function () {
                    return vm.customerSearchFilter;
                }, function (newVal, oldVal) {

                    if (typeof newVal === 'undefined') {
                        return false;
                    }

                    if (newVal.length > 0) {
                        vm.filteredCustomers = $filter('filter')(vm.group.assigned_customers, {
                            name: newVal
                        });
                    } else {
                        vm.filteredCustomers = vm.group.assigned_customers;
                    }
                });
            }).finally(function () {

                if (refreshing) {
                    $scope.$broadcast('scroll.refreshComplete');
                }
            });
        };

        var resetWorkoutSearch = function () {
            vm.workoutsSearchFilter = '';
            vm.filteredWorkouts = vm.group.workouts;
        };

        var resetCustomerSearch = function () {
            vm.customerSearchFilter = '';
            vm.filteredCustomers = vm.group.assigned_customers;
        };

        vm.refresh = function () {
            load(true);
        };

        vm.editWorkout = function (workout) {
            NavigateService.go('app.edit_workout', {
                workoutId: workout.id
            });
        };

        vm.delete = function () {

            ConfirmDialogService.open({
                title: "Confirm delete",
                message: "Do you want to remove this group?",
                buttonConfirm: "Confirm",
                buttonCancel: "Cancel"
            }).then(function () {

                ApiClientService.workoutGroups.delete(vm.groupId).success(function () {
                    NavigateService.clearCache().then(NavigateService.goBack);
                });
            });
        };

        vm.workoutMenu = function (workout) {

            $ionicActionSheet.show({
                titleText: workout.title,
                buttons: [{
                    text: 'Edit workout'
                }],
                destructiveText: 'Remove from group',
                cancelText: 'Cancel',
                buttonClicked: function (index) {

                    if (index === 0) {
                        NavigateService.go('app.edit_workout', {
                            workoutId: workout.id
                        });
                    }

                    return true;
                },
                destructiveButtonClicked: function () {

                    ConfirmDialogService.open({
                        title: "Confirm",
                        message: "Do you want to remove " + workout.title + " from the group?",
                        buttonConfirm: "Confirm",
                        buttonCancel: "Cancel"
                    }).then(function () {

                        var inGroupIds = [];

                        workout.workout_groups.map(function (currentGroup) {

                            if (currentGroup.id === vm.group.id) {
                                return false;
                            }

                            inGroupIds.push(currentGroup.id);
                        });

                        var saveGroupAssignments = ApiClientService.workouts.update(workout.id, {
                            assigned_group_ids: inGroupIds
                        });

                        saveGroupAssignments.success(function () {

                            vm.group.workouts.splice(vm.group.workouts.indexOf(workout), 1);

                            var filteredPosition = vm.filteredWorkouts.indexOf(workout);

                            if (filteredPosition !== -1) {
                                vm.filteredWorkouts.splice(filteredPosition, 1);
                            }
                        });
                    });

                    return true;
                }
            });
        };

        vm.assignedCustomerMenu = function (customer) {

            $ionicActionSheet.show({
                titleText: customer.user.first_name + ' ' + customer.user.last_name,
                destructiveText: 'Remove from group',
                cancelText: 'Cancel',
                destructiveButtonClicked: function () {

                    ConfirmDialogService.open({
                        title: "Confirm",
                        message: "Do you want to remove " + customer.user.first_name + ' ' + customer.user.last_name + " from the group?",
                        buttonConfirm: "Confirm",
                        buttonCancel: "Cancel"
                    }).then(function () {
                        ApiClientService.workoutGroups.unassignCustomer(vm.group.id, customer.id).then(function () {
                            vm.group.assigned_customers.splice(vm.group.assigned_customers.indexOf(customer), 1);
                        });
                    });

                    return true;
                }
            });
        };

        vm.assignWorkouts = function () {

            vm.selectedTabIndex = 0;

            ModalService.show({
                templateUrl: "trainer/workouts/group/workoutAssignModal.html",
                controller: 'TrainerWorkoutGroupWorkoutAssignModalController as vm',
                params: {
                    group: vm.group
                }
            }).then(function (assignedWorkouts) {
                vm.group.workouts = assignedWorkouts;
                resetWorkoutSearch();
            });
        };

        vm.assignClients = function () {

            vm.selectedTabIndex = 1;

            ModalService.show({
                templateUrl: "trainer/workouts/group/customerAssignModal.html",
                controller: 'TrainerWorkoutGroupCustomerAssignModalController as vm',
                params: {
                    group: vm.group
                }
            }).then(function (assignedClients) {
                vm.group.assigned_customers = assignedClients;
                resetCustomerSearch();
            });
        };

        $scope.$on('$ionicView.beforeEnter', function () {

            vm.groupId = $stateParams.groupId;

            load();
        });

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Trainer - View Workout Group');
        });
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('TrainerWorkoutGroupWorkoutAssignModalController', controller);

    controller.$inject = ['parameters', '$scope', '$filter', 'ApiClientService', 'PermissionsService', 'AnalyticsService'];

    function controller(parameters, $scope, $filter, ApiClientService, PermissionsService, AnalyticsService) {

        var vm = this;

        vm.group = parameters.group;
        vm.currentAssignments = vm.group.workouts;

        vm.isCurrentTrainerUser = PermissionsService.getLoggedInUserId() === vm.group.trainer_user_id;

        vm.assignedIds = [];
        angular.forEach(vm.currentAssignments, function (workout) {
            vm.assignedIds.push(workout.id);
        });

        var initAssignedCount = vm.assignedIds.length;
        ApiClientService.trainers.getAllWorkouts(vm.group.trainer_user_id).success(function (response) {

            // set full name on root object for search filter
            response.workouts.map(function (workout) {
                workout.use = vm.assignedIds.indexOf(workout.id) !== -1;
            });

            vm.workouts = response.workouts;

            vm.filteredWorkouts = angular.copy(response.workouts);

            vm.noWorkouts = response.workouts.length === 0;
        });

        // List Search filter
        $scope.$watch(function () {
            return vm.searchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.searchFilterActive = true;
                vm.filteredWorkouts = $filter('filter')(vm.workouts, {
                    title: newVal
                });
            } else {
                vm.searchFilterActive = false;
                vm.filteredWorkouts = vm.workouts;
            }
        });

        vm.assign = function (workout) {

            workout.use = !workout.use;

            var findOriginal = $filter('filter')(vm.workouts, {
                id: workout.id
            }, true);

            if (findOriginal.length > 0) {

                var originalIndex = vm.workouts.indexOf(findOriginal[0]);

                if (originalIndex !== -1) {
                    vm.workouts[originalIndex].use = workout.use;
                }
            }
        };

        vm.save = function () {

            vm.saving = true;

            var toAssign = $filter('filter')(vm.workouts, {
                use: true
            });

            if (toAssign.length === 0 && initAssignedCount === 0) {

                $scope.closeModal([]);

                return true;
            }

            var ids = [];
            var list = [];

            toAssign.map(function (workout) {
                ids.push(workout.id);
                list.push(workout);
            });

            ApiClientService.workoutGroups.update(vm.group.id, {
                workout_ids: ids
            }).then(function () {
                $scope.closeModal(list);
            }, function () {
                vm.saving = false;
            });
        };

        AnalyticsService.trackView('Trainer - Assign Workouts To Workout Group');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('TrainerWorkoutsLibraryGroupController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        // Current Trainer's Workouts Route
        $stateProvider.state('app.trainer_workouts_library_group', {
            url: '/library/workouts/group/:groupId',
            views: {
                'menuContent': {
                    templateUrl: "trainer/workouts/library/librarygroup.html",
                    controller: 'TrainerWorkoutsLibraryGroupController as vm'
                }
            },
            resolve: {
                getGroup: ['ApiClientService', '$stateParams', 'PermissionsService', function (ApiClientService, $stateParams, PermissionsService) {

                    var currentUser = PermissionsService.getCurrentUser();

                    var tag = 'metric';

                    if (currentUser.country === 'US') {
                        tag = 'imperial';
                    }

                    return ApiClientService.workoutGroups.get($stateParams.groupId, tag);
                }]
            }
        });
    }

    controller.$inject = ['$scope', 'getGroup', '$filter', 'mpthconf', '$ionicLoading', 'ApiClientService', '$q', 'ToastService'];

    function controller($scope, getGroup, $filter, mpthconf, $ionicLoading, ApiClientService, $q, ToastService) {

        var vm = this;

        vm.group = getGroup.group;
        vm.cdnBase = mpthconf.cdn.base;

        vm.bannerImageUrl = false;
        if (vm.group.banner_image) {
            vm.bannerImageUrl = vm.cdnBase + 'workoutlibrary/' + vm.group.banner_image;
        }

        vm.canUseChecked = false;

        var getIds = function (workouts) {

            var ids = [];

            workouts.map(function (workout) {
                ids.push(workout.id);
            });

            return ids;
        };

        var getChecked = function () {
            return $filter('filter')(vm.group.workouts, { use: true });
        };

        var createGroup = function (workoutIds) {

            $ionicLoading.show({
                template: 'Please wait...'
            }).then(function () {
                vm.toDuplicateCount = workoutIds.length;
                vm.duplicatedCount = 1;
                vm.creatingGroup = false;

                vm.percentComplete = 0;

                var duplicateWorkoutActions = [];
                angular.forEach(workoutIds, function (id) {

                    var duplicate = ApiClientService.workouts.duplicate(id);

                    duplicateWorkoutActions.push(duplicate);

                    duplicate.success(function () {
                        vm.duplicatedCount++;
                        vm.percentComplete += 90 / vm.toDuplicateCount;
                    });
                });

                $q.all(duplicateWorkoutActions).then(function (duplicateResponses) {
                    vm.creatingGroup = true;

                    var newWorkoutIds = [];
                    duplicateResponses.map(function (response) {
                        newWorkoutIds.push(response.data.workout.id);
                    });

                    ApiClientService.workoutGroups.create({
                        title: vm.group.title,
                        workout_ids: newWorkoutIds
                    }).success(function (response) {

                        ToastService.show('Workouts successfully added to your account');
                        $ionicLoading.hide();
                    });
                }, function (response) {
                    ToastService.show(response.data.error.message);
                    $ionicLoading.hide();
                });
            });
        };

        vm.useChecked = function () {

            var useWorkouts = getChecked();

            if (useWorkouts.length === 0) {
                return false;
            }

            var ids = getIds(useWorkouts);

            createGroup(ids);
        };

        vm.useAll = function () {

            var ids = getIds(vm.group.workouts);

            createGroup(ids);
        };

        vm.onCheck = function () {

            if (getChecked().length > 0) {
                vm.canUseChecked = true;
            } else {
                vm.canUseChecked = false;
            }
        };

        $scope.$on('$ionicView.beforeEnter', function (event, viewData) {
            viewData.enableBack = true;
        });
    }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').component('questionEsignatureInput', {
    templateUrl: 'common/forms/inputComponents/esignatureInput.html',
    controller: controller,
    controllerAs: 'vm',
    bindings: {
      'question': '=',
      'readonly': '=?'
    }
  });

  controller.$inject = ['$element', 'ConfirmDialogService'];

  function controller($element, ConfirmDialogService) {
    var vm = this;

    var parent = $('.list', $element)[0];

    vm.readonly = this.readonly ? this.readonly : false;

    vm.canvas = $('canvas', $element)[0];
    vm.signaturePad = new SignaturePad(vm.canvas);
    vm.canvas.width = $(parent).width();
    vm.canvas.height = 200;

    vm.on = false;
    vm.signaturePad.off();

    vm.toggleEdit = function () {
      vm.on = !vm.on;
      if (vm.on) {
        vm.signaturePad.on();
      } else {
        vm.signaturePad.off();
      }
    };

    // Prevent scroll on signature draw
    $element.find('canvas').on('mousewheel touchmove', function (e) {
      if (vm.on) {
        e.stopPropagation();
        return false;
      }
    });

    if (vm.question.answer) {
      vm.signaturePad.fromDataURL(vm.question.answer.value, {
        width: vm.canvas.width,
        height: vm.canvas.height
      });
    }

    vm.clearCanvas = function () {
      ConfirmDialogService.open({
        title: "Confirm clear",
        message: "Do you want to clear this signature?",
        buttonConfirm: "Clear",
        buttonCancel: "Cancel"
      }).then(function () {
        vm.signaturePad.clear();
      });
    };

    vm.question.answer.getSignatureValue = function () {
      return vm.signaturePad.toDataURL();
    };
  }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').directive('formQuestionInput', directive);

  directive.$inject = ['$compile'];

  function directive($compile) {

    function typeToComponent(typeNo) {
      switch (typeNo) {
        case 1:
          return '<question-scale-input question="vm.question" readonly="vm.readonly"></question-scale-input>';
          break;
        case 2:
          return '<question-polar-input question="vm.question" readonly="vm.readonly"></question-polar-input>';
          break;
        case 3:
          return '<question-number-text-input question="vm.question" readonly="vm.readonly"></question-number-text-input>';
          break;
        case 4:
          return '<question-opinion-based-input question="vm.question" readonly="vm.readonly"></question-opinion-based-input>';
          break;
        case 5:
          return '<question-star-input question="vm.question" readonly="vm.readonly"></question-star-input>';
          break;
        case 6:
          return '<question-polar-description-input question="vm.question" readonly="vm.readonly"></question-polar-description-input>';
          break;
        case 7:
          return '<question-multiple-choice-input question="vm.question" readonly="vm.readonly"></question-multiple-choice-input>';
          break;
        case 8:
          return '<question-esignature-input question="vm.question" readonly="vm.readonly"></question-esignature-input>';
          break;
      }
    }

    return {
      template: '<div class="formQuestionInputContainer"></div>',
      link: function (scope, element, attrs, controller, transcludeFn) {

        var template = typeToComponent(scope.question.type);

        if (template) {
          element.append($compile(template)(scope));
        }
      },
      controller: controller,
      controllerAs: 'vm',
      scope: {
        'question': '=',
        'readonly': '=?'
      }
    };
  }

  controller.$inject = ['$scope'];

  function controller($scope) {
    var vm = this;

    vm.question = $scope.question;
    vm.readonly = $scope.readonly ? $scope.readonly : false;
  }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').component('questionMultipleChoiceInput', {
    templateUrl: 'common/forms/inputComponents/multipleChoiceInput.html',
    controller: controller,
    controllerAs: 'vm',
    bindings: {
      'question': '=',
      'readonly': '=?'
    }
  });

  controller.$inject = [];

  function controller() {

    var vm = this;

    vm.select = function (index) {
      var value = vm.question.answer.value;
      var optionIndex = getOptionIndex(index);

      if ((value & optionIndex) > 0) {
        value ^= optionIndex;
      } else {
        value |= optionIndex;
      }

      vm.question.answer.value = value;
    };

    vm.question.options = [];

    function getOptionIndex(index) {
      var value = 1;

      for (var i = 0; i < index; i++) {
        value *= 2;
      }
      return value;
    }

    if (vm.question.answer) {
      vm.question.option = vm.question.answer.value;
    }

    if (vm.question.values.length == 0) {
      for (let i = 0; i < 3; i++) {
        vm.question.values[i] = {
          value: 'Option ' + (parseInt(i) + 1),
          display_order: i
        };
      }
    }

    if (!vm.question.options || vm.question.options.length != vm.question.values.length) {
      vm.question.options = [];

      for (let i = 0; i < vm.question.values.length; i++) {
        vm.question.options[i] = (vm.question.answer.value & getOptionIndex(i)) > 0;
      }
    }
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').component('questionNumberTextInput', {
        templateUrl: 'common/forms/inputComponents/numberTextInput.html',
        controller: controller,
        controllerAs: 'vm',
        bindings: {
            'question': '=',
            'readonly': '=?'
        }
    });

    controller.$inject = [];

    function controller() {

        var vm = this;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').component('questionOpinionBasedInput', {
        templateUrl: 'common/forms/inputComponents/opinionBasedInput.html',
        controller: controller,
        controllerAs: 'vm',
        bindings: {
            'question': '=',
            'readonly': '=?'
        }
    });

    controller.$inject = [];

    function controller() {

        var vm = this;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').component('questionPolarDescriptionInput', {
        templateUrl: 'common/forms/inputComponents/polarDescriptionInput.html',
        controller: controller,
        controllerAs: 'vm',
        bindings: {
            'question': '=',
            'readonly': '=?'
        }
    });

    controller.$inject = [];

    function controller() {

        var vm = this;

        vm.setAnswer = function (answer) {
            if (vm.readonly) {
                return false;
            }

            vm.question.answer.value = answer;
        };

        vm.descriptionPlaceholder = 'Type an answer' + (vm.question.is_mandatory ? '*' : '');
        if (vm.readonly) {
            vm.descriptionPlaceholder = 'No answer entered';
        }
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').component('questionPolarInput', {
        templateUrl: 'common/forms/inputComponents/polarInput.html',
        controller: controller,
        controllerAs: 'vm',
        bindings: {
            'question': '=',
            'readonly': '=?'
        }
    });

    controller.$inject = [];

    function controller() {

        var vm = this;

        vm.setAnswer = function (answer) {
            if (vm.readonly) {
                return false;
            }

            vm.question.answer.value = answer;
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').component('questionScaleInput', {
        templateUrl: 'common/forms/inputComponents/scaleInput.html',
        controller: controller,
        controllerAs: 'vm',
        bindings: {
            'question': '=',
            'readonly': '=?'
        }
    });

    controller.$inject = [];

    function controller() {

        var vm = this;

        if (!vm.question.answer.value) {
            vm.question.answer.value = 5;
        }

        vm.setRangeValue = function (question, value) {
            event.preventDefault();
            question.answer.value = value;
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').component('questionStarInput', {
        templateUrl: 'common/forms/inputComponents/starInput.html',
        controller: controller,
        controllerAs: 'vm',
        bindings: {
            'question': '=',
            'readonly': '=?'
        }
    });

    controller.$inject = [];

    function controller() {

        var vm = this;

        vm.setRating = function (star) {
            if (vm.readonly) {
                return false;
            }

            vm.question.answer.value = star;
        };
    }
})();
(function () {
  angular.module('MyPTHub').config(config).controller('MessageConversationController', controller);

  config.$inject = ['$stateProvider'];

  function config($stateProvider) {
    $stateProvider.state('app.message_conversation', {
      url: '/messages/conversation/:conversationId',
      views: {
        menuContent: {
          templateUrl: 'common/messages/conversation/conversation.html',
          controller: 'MessageConversationController as vm'
        }
      }
    });
  }

  controller.$inject = ['$scope', '$stateParams', 'ApiClientService', '$ionicScrollDelegate', '$timeout', '$interval', 'AuthenticationStateService', '$rootScope', 'AnalyticsService', 'MessagesService'];

  function controller($scope, $stateParams, ApiClientService, $ionicScrollDelegate, $timeout, $interval, AuthenticationStateService, $rootScope, AnalyticsService, MessagesService) {
    /**
     * Very hacky, mostly ripped from https://codepen.io/rossmartin/pen/XJmpQr
     * to get this done quickly! Requires refactor into directive etc.
     */

    const vm = this;

    let messageCheckTimer;

    const viewScroll = $ionicScrollDelegate.$getByHandle('userMessageScroll');
    let footerBar;
    let scroller;
    let txtInput;
    let page = 1;
    let lastPage = 0;
    let initialId;
    let latestId = 0;
    vm.messages = [];
    vm.loadingMore = false;

    /**
     * Checks if we need to show load more
     */
    vm.showLoadMore = () => page <= lastPage && vm.ready;
    vm.getData = () => {
      vm.loadingMore = true;
      getConversation(true, false);
    };

    const newMessageObject = {
      sending: true,
      created_at: null,
      message: '',
      from_user: AuthenticationStateService.getCurrentUser()
    };

    let keyboardHeight = 0;

    window.addEventListener('native.keyboardshow', keyboardShowHandler);
    window.addEventListener('native.keyboardhide', keyboardHideHandler);

    function keyboardShowHandler(e) {
      keyboardHeight = e.keyboardHeight;
      $timeout(() => {
        viewScroll.scrollBottom();
        var objDiv = document.getElementById("scroll");
        objDiv.scrollTop = objDiv.scrollHeight;
        let input = document.getElementsByClassName('inputactions')[0];
        input.style.paddingBottom = '44px';
      }, 0);
    }

    function keyboardHideHandler(e) {
      keyboardHeight = 0;
      $timeout(() => {
        viewScroll.scrollBottom();
        var objDiv = document.getElementById("scroll");
        objDiv.scrollTop = objDiv.scrollHeight;
        let input = document.getElementsByClassName('inputactions')[0];
        input.style.paddingBottom = '0px';
      }, 0);
    }

    const setFooterHeight = function (newInputHeight) {
      if (!footerBar) return;

      let newFooterHeight = newInputHeight + 40;
      newFooterHeight = newFooterHeight > 44 ? newFooterHeight : 44;

      footerBar.style.height = `${newFooterHeight}px`;

      // for iOS you will need to add the keyboardHeight to the scroller.style.bottom
      if (typeof device !== 'undefined' && device.platform.toLowerCase() === 'ios') {
        scroller.style.bottom = `${newFooterHeight + keyboardHeight}px`;
      } else {
        scroller.style.bottom = `${newFooterHeight}px`;
      }
    };

    const getLatestMessages = () => ApiClientService.messages.getLatest($stateParams.conversationId, latestId).success(resp => {
      const messages = resp.messages.reverse();
      if (messages.length) {
        latestId = messages[0].id;
        messages.reverse().map(message => {
          return vm.messages.push(message);
        });

        $rootScope.$emit('conversationLoaded');
      }
    });

    var getConversation = function (reloading = false, scroll = true) {
      console.log('initialID', initialId);
      return ApiClientService.messages.getThread($stateParams.conversationId, {
        page,
        from_id: initialId
      }).success(response => {
        vm.ready = true;
        vm.loadingMore = false;
        if (page === 1) {
          initialId = response.messages[0].id;
          latestId = response.messages[0].id;
        }
        page = response.pagination.current_page + 1;
        lastPage = response.pagination.last_page;

        vm.messages = response.messages.reverse().concat(vm.messages);

        const originalCount = vm.messages ? vm.messages.length : 0;
        const newCount = response.messages.length;

        vm.otherPerson = vm.messages[0].other_user || null;

        if (reloading && originalCount !== newCount) {
          $timeout(() => {
            viewScroll.scrollBottom();
            var objDiv = document.getElementById("scroll");
            objDiv.scrollTop = objDiv.scrollHeight;
          }, 0);
        }

        if (!reloading && scroll) {
          $rootScope.$emit('conversationLoaded');
        }
      });
    };

    $scope.$on('$ionicView.beforeEnter', () => {
      vm.ready = false;

      getConversation();
    });

    $scope.$on('$ionicView.enter', () => {
      $timeout(() => {
        footerBar = document.body.querySelector('#userMessagesConversationView .inputactions');
        scroller = document.body.querySelector('#userMessagesConversationView .scroll-content');
        txtInput = angular.element(footerBar.querySelector('textarea'));
      }, 0);

      messageCheckTimer = $interval(() => {
        // here you could check for new messages if your app doesn't use push notifications or user disabled them
        // getConversation(true);
        getLatestMessages();
      }, 20000);

      if (window.cordova && window.cordova.plugins.Keyboard) {
        cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
      }
    });

    $scope.$on('$ionicView.afterEnter', () => {
      MessagesService.markSeen();
      AnalyticsService.trackView('Messages - Conversation');
    });

    $scope.$on('$ionicView.leave', () => {
      if (angular.isDefined(messageCheckTimer)) {
        $interval.cancel(messageCheckTimer);

        messageCheckTimer = undefined;
      }

      if (window.cordova && window.cordova.plugins.Keyboard) {
        cordova.plugins.Keyboard.hideKeyboardAccessoryBar(false);
      }
    });

    const conversationLoadedEventUnsub = $rootScope.$on('conversationLoaded', () => {
      $timeout(() => {
        viewScroll.scrollBottom();
        var objDiv = document.getElementById("scroll");
        objDiv.scrollTop = objDiv.scrollHeight;
      }, 0);
    });

    $scope.$on('$destroy', conversationLoadedEventUnsub);

    vm.sendMessage = function (sendMessageForm) {
      const send = ApiClientService.messages.send(vm.otherPerson.id, vm.message);

      vm.sending = true;

      const tempMessage = angular.copy(newMessageObject);
      tempMessage.message = vm.message;

      sendMessageForm.$setPristine();

      vm.messages.push(tempMessage);

      keepKeyboardOpen();

      vm.message = '';

      $timeout(() => {
        viewScroll.scrollBottom(true);
        var objDiv = document.getElementById("scroll");
        objDiv.scrollTop = objDiv.scrollHeight;
      }, 0);

      send.success(response => {
        vm.sending = false;

        vm.messages[vm.messages.indexOf(tempMessage)] = response.message;
      });

      send.error(response => {
        vm.sending = false;

        vm.messages[vm.messages.indexOf(tempMessage)].error = true;
        vm.messages[vm.messages.indexOf(tempMessage)].sending = false;
      });
    };

    vm.resend = function (messageObject) {
      messageObject.error = false;
      messageObject.sending = true;

      const send = ApiClientService.messages.send(vm.otherPerson.id, messageObject.message);

      send.success(response => {
        vm.sending = false;

        messageObject = response.message;
      });

      send.error(response => {
        vm.sending = false;

        $timeout(() => {
          messageObject.error = true;
          messageObject.sending = false;
        }, 500);
      });
    };

    // this keeps the keyboard open on a device only after sending a message, it is non obtrusive
    function keepKeyboardOpen() {
      txtInput.one('blur', () => {
        txtInput[0].focus();
      });
    }

    // I emit this event from the monospaced.elastic directive, read line 480 in the Codepen JS
    $scope.$on('elastic:resize', (e, ta) => {
      if (!ta) return;

      const taHeight = ta[0].offsetHeight;

      setFooterHeight(taHeight);
    });
  }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').config(config).controller('NewMessageController', controller);

  config.$inject = ['$stateProvider'];

  function config($stateProvider) {

    $stateProvider.state('app.new_message', {
      url: '/messages/new/:userId?',
      views: {
        'menuContent': {
          templateUrl: 'common/messages/new/new.html',
          controller: 'NewMessageController as vm'
        }
      }
    });
  }

  controller.$inject = ['$scope', 'ApiClientService', '$stateParams', 'NavigateService', 'AnalyticsService', '$timeout', '$ionicScrollDelegate'];

  function controller($scope, ApiClientService, $stateParams, NavigateService, AnalyticsService, $timeout, $ionicScrollDelegate) {

    var vm = this;

    var viewScroll = $ionicScrollDelegate.$getByHandle('userMessageScroll');
    var footerBar;
    var scroller;
    var txtInput;

    var keyboardHeight = 0;

    window.addEventListener('native.keyboardshow', keyboardShowHandler);
    window.addEventListener('native.keyboardhide', keyboardHideHandler);

    function keyboardShowHandler(e) {
      keyboardHeight = e.keyboardHeight;
      $timeout(function () {
        viewScroll.scrollBottom();
      }, 0);
    }

    function keyboardHideHandler(e) {
      keyboardHeight = 0;
      $timeout(function () {
        viewScroll.scrollBottom();
      }, 0);
    }

    var setFooterHeight = function (newInputHeight) {

      if (!footerBar) return;

      var newFooterHeight = newInputHeight + 40;
      newFooterHeight = newFooterHeight > 44 ? newFooterHeight : 44;

      footerBar.style.height = newFooterHeight + 'px';

      // for iOS you will need to add the keyboardHeight to the scroller.style.bottom
      if (typeof device !== 'undefined' && device.platform.toLowerCase() === 'ios') {
        scroller.style.bottom = newFooterHeight + keyboardHeight + 'px';
      } else {
        scroller.style.bottom = newFooterHeight + 'px';
      }
    };

    vm.send = function () {
      vm.sending = true;
      let sendMessage;
      let messageType;

      if (typeof vm.toUser === 'string' && vm.toUser.startsWith('g-')) {
        messageType = 'group';
        const data = {
          message: vm.message
        };
        sendMessage = ApiClientService.customerGroups.message(vm.toUser.slice(2), data);
      } else {
        messageType = 'user';
        sendMessage = ApiClientService.messages.send(vm.toUser, vm.message);
      }

      sendMessage.success(function (response) {
        if (messageType === 'user') {
          // Go to the conversation.
          NavigateService.replaceCurrentState().go('app.message_conversation', {
            conversationId: response.message.conversation_id
          });
        } else {
          // Go to the inbox.
          NavigateService.replaceCurrentState().go('app.messages');
        }
      });

      sendMessage.error(function (response) {
        // toast
        vm.sending = false;
      });
    };

    $scope.$on('elastic:resize', function (e, ta) {

      if (!ta) return;

      var taHeight = ta[0].offsetHeight;

      setFooterHeight(taHeight);
    });

    $scope.$on('$ionicView.beforeEnter', function () {

      vm.ready = false;
      vm.message = '';
      vm.sending = false;

      var getContacts = ApiClientService.messages.getContacts();

      getContacts.success(function (response) {
        vm.contacts = response.users;
        vm.ready = true;
      });

      if ($stateParams.userId) {
        vm.toUser = $stateParams.userId;
      }
    });

    $scope.$on('$ionicView.enter', function () {

      $timeout(function () {
        footerBar = document.body.querySelector('#userMessagesConversationView .bar-footer');
        scroller = document.body.querySelector('#userMessagesConversationView .scroll-content');
        txtInput = angular.element(footerBar.querySelector('textarea'));
      }, 0);

      AnalyticsService.trackView('Messages - New');

      if (window.cordova && window.cordova.plugins.Keyboard) {
        cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
      }
    });

    $scope.$on('$ionicView.leave', function () {
      if (window.cordova && window.cordova.plugins.Keyboard) {
        cordova.plugins.Keyboard.hideKeyboardAccessoryBar(false);
      }
    });
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('ScheduleActionProgressPhotoService', service);

    service.$inject = ['$ionicActionSheet', 'ProgressPhotoUploadService', 'ToastService', '$rootScope', 'ScheduleEventStoreService', 'ModalService'];

    function service($ionicActionSheet, ProgressPhotoUploadService, ToastService, $rootScope, ScheduleEventStoreService, ModalService) {

        var service = {};

        service.registerUploadEventHandler = function (scope, callback) {
            var unsub = $rootScope.$on('ScheduleUploadingProgressPhotoStateChange', callback);
            scope.$on('$destroy', unsub);
            return unsub;
        };

        service.startUpload = function (day, scheduleData) {
            day = day || false;

            var existingPhoto = false;

            if (day && day.update_photo_completed && day.photo) {
                existingPhoto = day.photo;
            }

            var nowForCustomer = moment().utc().tz(scheduleData.customerTimezone);

            var dateLocalToCustomer = nowForCustomer;
            if (day) {
                dateLocalToCustomer = moment.tz(day.date, scheduleData.customerTimezone);

                // If uploading for today then use the current time.
                if (dateLocalToCustomer.isSame(nowForCustomer, 'day')) {
                    dateLocalToCustomer.hour(nowForCustomer.hour()).minute(nowForCustomer.minute()).second(nowForCustomer.second());
                }
            }

            var date = false;
            var titleText = 'Upload photo for ' + dateLocalToCustomer.format('MMM Do YYYY');
            date = dateLocalToCustomer.utc().format('YYYY-MM-DD HH:mm:ss');

            var buttons = [{
                text: 'Take new photo',
                click: function () {
                    if (day) {
                        day.update_photo_uploading = true;
                    }

                    $rootScope.$emit('ScheduleUploadingProgressPhotoStateChange', true);

                    ProgressPhotoUploadService.takePhoto(scheduleData.customer_id, date).then(function () {
                        ToastService.show('Photo uploaded successfully');
                        ScheduleEventStoreService.setDirty();
                    }).finally(function () {
                        if (day) {
                            day.update_photo_uploading = false;
                        }
                        $rootScope.$emit('ScheduleUploadingProgressPhotoStateChange', false);
                    });
                }
            }, {
                text: 'Browse for a photo',
                click: function () {
                    if (day) {
                        day.update_photo_uploading = true;
                    }

                    $rootScope.$emit('ScheduleUploadingProgressPhotoStateChange', true);

                    ProgressPhotoUploadService.browsePhoto(scheduleData.customer_id, date).then(function () {
                        ToastService.show('Photo uploaded successfully');
                        ScheduleEventStoreService.setDirty();
                    }).finally(function () {
                        if (day) {
                            day.update_photo_uploading = false;
                        }
                        $rootScope.$emit('ScheduleUploadingProgressPhotoStateChange', false);
                    });
                }
            }];

            if (existingPhoto) {
                buttons.unshift({
                    text: 'View existing photo',
                    click: function () {
                        var date = moment.utc(existingPhoto.created_at, 'YYYY-MM-DD HH:mm:ss').local().format('Do MMMM YYYY');

                        ModalService.show({
                            templateUrl: "common/photoViewer/photoViewerModal.html",
                            controller: 'PhotoViewerModalController as vm',
                            params: {
                                url: existingPhoto.large_url,
                                date: date
                            }
                        });
                    }
                });
            }

            $ionicActionSheet.show({
                titleText: titleText,
                buttons: buttons,
                cancelText: 'Cancel',
                buttonClicked: function (index) {

                    if (typeof buttons[index] !== 'undefined' && typeof buttons[index].click === 'function') {
                        buttons[index].click(index);
                    }

                    return true;
                }
            });
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('SessionCancelService', service);

    service.$inject = ['ModalService', 'PermissionsService'];

    function service(ModalService, PermissionsService) {

        var service = {};

        service.startCancel = function (session) {

            if (PermissionsService.userIsTrainer()) {

                ModalService.show({
                    templateUrl: "common/session/cancel/cancelModal.html",
                    controller: 'SessionCancelModalController as vm',
                    params: {
                        session: session
                    }
                });
            } else {

                ModalService.show({
                    templateUrl: "common/session/cancel/requestCancelModal.html",
                    controller: 'SessionRequestCancelModalController as vm',
                    params: {
                        session: session
                    }
                });
            }
        };

        return service;
    }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').controller('SessionCancelModalController', controller);

  controller.$inject = ['parameters', '$scope', 'PermissionsService', 'ApiClientService', 'NavigateService', 'ToastService', 'AnalyticsService'];

  function controller(parameters, $scope, PermissionsService, ApiClientService, NavigateService, ToastService, AnalyticsService) {

    var vm = this;

    vm.cancel = {};
    vm.session = parameters.session;

    vm.showCancelPeriodNotice = false;

    if (PermissionsService.userIsCustomer()) {

      var currentUser = PermissionsService.getCurrentUser();

      if (currentUser.trainer_details.min_cancellation > 0) {

        var currentDate = moment().utc();
        var sessionDate = moment.utc(parameters.session.date, 'YYYY-MM-DD HH:mm:ss');
        var minutesUntilSessionStarts = sessionDate.diff(currentDate, 'minutes');

        if (minutesUntilSessionStarts < currentUser.trainer_details.min_cancellation) {

          vm.showCancelPeriodNotice = true;
          vm.cancellationPeriodReadable = moment.duration(currentUser.trainer_details.min_cancellation, 'minutes').humanize();
          vm.trainerName = currentUser.trainer_details.first_name;
        }
      }
    }

    vm.cancelSession = function (cancelAllOccurrences) {
      vm.processing = true;

      var datetime = null;
      if (vm.session.phantom) {
        datetime = vm.session.date;
      }
      var cancelReq = ApiClientService.sessions.cancel(parameters.session.id, vm.cancel.reason, datetime, cancelAllOccurrences);

      cancelReq.success(function (response) {

        ToastService.show('Session cancelled successfully');
        $scope.cancelModal();
        NavigateService.goHome();
      });

      cancelReq.error(function (response) {

        ToastService.show(response.error.message);

        vm.processing = false;
      });
    };

    AnalyticsService.trackView('Cancel Session');
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('SessionRequestCancelModalController', controller);

    controller.$inject = ['parameters', '$scope', 'PermissionsService', 'ApiClientService', 'NavigateService', 'ToastService', 'AnalyticsService'];

    function controller(parameters, $scope, PermissionsService, ApiClientService, NavigateService, ToastService, AnalyticsService) {

        var vm = this;

        vm.reason = null;
        vm.session = parameters.session;

        vm.showCancelPeriodNotice = false;

        var currentUser = PermissionsService.getCurrentUser();

        if (currentUser.trainer_details.min_cancellation > 0) {

            var currentDate = moment().utc();
            var sessionDate = moment.utc(vm.session.date, 'YYYY-MM-DD HH:mm:ss');
            var minutesUntilSessionStarts = sessionDate.diff(currentDate, 'minutes');

            if (minutesUntilSessionStarts < currentUser.trainer_details.min_cancellation) {

                vm.showCancelPeriodNotice = true;
                vm.cancellationPeriodReadable = moment.duration(currentUser.trainer_details.min_cancellation, 'minutes').humanize();
                vm.trainerName = currentUser.trainer_details.first_name;
            }
        }

        vm.doRequest = function () {

            vm.processing = true;

            var dateTime = null;
            if (vm.session.phantom) {
                dateTime = vm.session.date;
            }

            var cancelReq = ApiClientService.sessions.requestCancellation(vm.session.id, vm.reason, dateTime);

            cancelReq.success(function (response) {
                ToastService.show('Cancellation request sent to trainer');
                $scope.cancelModal();
                NavigateService.goHome();
            });

            cancelReq.error(function (response) {
                vm.processing = false;
                ToastService.show(response.error ? response.error.message : 'Unable to send cancellation request');
            });
        };

        AnalyticsService.trackView('Cancel Session');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('SessionReviewCancelRequestModalController', controller).factory('SessionReviewCancelRequestService', service);

    service.$inject = ['ModalService'];

    function service(ModalService) {

        var service = {};

        service.review = function (session) {

            ModalService.show({
                templateUrl: "common/session/cancel/reviewCancelRequestModal.html",
                controller: 'SessionReviewCancelRequestModalController as vm',
                params: {
                    session: session
                }
            });
        };

        return service;
    }

    controller.$inject = ['parameters', '$scope', 'PermissionsService', 'ApiClientService', 'NavigateService', 'ToastService', 'AnalyticsService', 'SessionLoggerDebitControlService'];

    function controller(parameters, $scope, PermissionsService, ApiClientService, NavigateService, ToastService, AnalyticsService, SessionLoggerDebitControlService) {

        var vm = this;

        vm.reason = null;
        vm.session = parameters.session;
        vm.request = vm.session.cancellation_request;
        vm.customer = vm.session.customer;
        vm.customerName = vm.customer.user.first_name;

        var dateTime = null;
        if (vm.session.phantom) {
            dateTime = vm.session.date;
        }

        if (vm.request.reason) {
            vm.request.reason = '"' + vm.request.reason + '"';
        }

        vm.showCancelPeriodNotice = false;

        var currentUser = PermissionsService.getCurrentUser();

        if (currentUser.min_cancellation > 0) {

            var requestedDate = moment.utc(vm.request.created_at, 'YYYY-MM-DD HH:mm:ss');

            var sessionDate = moment.utc(dateTime, 'YYYY-MM-DD HH:mm:ss');

            var minutesBetweenRequestAndSession = sessionDate.diff(requestedDate, 'minutes');

            if (minutesBetweenRequestAndSession < currentUser.min_cancellation) {

                vm.showCancelPeriodNotice = true;

                vm.cancellationPeriodReadable = moment.duration(currentUser.min_cancellation, 'minutes').humanize();
            }
        }

        vm.reject = function () {

            vm.processing = true;

            var rejectReq = ApiClientService.sessions.rejectCancellationRequest(vm.session.id, vm.reason, dateTime);

            rejectReq.success(function (response) {
                ToastService.show('Session cancellation request rejected');
                $scope.cancelModal();
            });

            rejectReq.error(function (response) {
                vm.processing = false;
                ToastService.show(response.error ? response.error.message : 'Unable to reject cancellation request');
            });
        };

        vm.cancelSession = function () {

            vm.processing = true;

            var cancelReq = ApiClientService.sessions.cancel(vm.session.id, vm.reason, dateTime, false, SessionLoggerDebitControlService.getIsBillable(), SessionLoggerDebitControlService.getBillableToQuotaId());

            cancelReq.success(function (response) {

                SessionLoggerDebitControlService.reset();

                ToastService.show('Session cancelled');
                $scope.cancelModal();
                NavigateService.goHome();
            });

            cancelReq.error(function (response) {

                vm.processing = false;

                ToastService.show(response.error ? response.error.message : 'Unable to cancel session');
            });
        };

        AnalyticsService.trackView('Cancel Session');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('setValueInput', directive);

    directive.$inject = [];

    function directive() {

        return {
            restrict: 'E',
            replace: true,
            templateUrl: "common/session/Inputs/setValueInput.html",
            scope: {
                field: '=',
                set: '='
            },
            controllerAs: 'vm',
            controller: ['$scope', function ($scope) {

                var vm = this;

                vm.changed = function () {
                    if ($scope.set.isDone && $scope.set.done[$scope.field.name]) {
                        $scope.set.current_target[$scope.field.name] = $scope.set.done[$scope.field.name];
                    }
                };
            }]
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('SessionHistoryService', service);

    service.$inject = ['ApiClientService', '$q', '$filter'];

    function service(ApiClientService, $q, $filter) {

        var service = {};

        var historyCache = {
            this: {},
            all: {}
        };
        var fieldHistoryCache = {
            this: {},
            all: {}
        };

        var loadFromCache = function (pivotId, customerId, allWorkouts) {

            var which = allWorkouts ? 'all' : 'this';

            if (historyCache[which].hasOwnProperty(customerId) && historyCache[which][customerId].hasOwnProperty(pivotId)) {

                return historyCache[which][customerId][pivotId];
            }

            return false;
        };

        var loadFromFieldCache = function (pivotId, customerId, allWorkouts) {

            var which = allWorkouts ? 'all' : 'this';

            if (fieldHistoryCache[which].hasOwnProperty(customerId) && fieldHistoryCache[which][customerId].hasOwnProperty(pivotId)) {

                return fieldHistoryCache[which][customerId][pivotId];
            }

            return false;
        };

        var saveToCache = function (pivotId, customerId, data, allWorkouts) {

            var which = allWorkouts ? 'all' : 'this';

            if (!historyCache[which].hasOwnProperty(customerId)) {
                historyCache[which][customerId] = {};
            }

            historyCache[which][customerId][pivotId] = angular.copy(data);

            return data;
        };

        var saveToFieldCache = function (pivotId, customerId, data, allWorkouts) {

            var which = allWorkouts ? 'all' : 'this';

            if (!fieldHistoryCache[which].hasOwnProperty(customerId)) {
                fieldHistoryCache[which][customerId] = {};
            }

            if (!fieldHistoryCache[which][customerId].hasOwnProperty(pivotId)) {
                fieldHistoryCache[which][customerId][pivotId] = {};
            }

            fieldHistoryCache[which][customerId][pivotId] = angular.copy(data);

            return data;
        };

        var buildOverallExerciseOutput = function (data) {

            var output = {};

            angular.forEach(data, function (date) {

                angular.forEach(date.data, function (field) {

                    angular.forEach(field.sets, function (setItem) {

                        angular.forEach(setItem.instances, function (instanceItem) {

                            if (!output.hasOwnProperty(instanceItem.instance)) {

                                output[instanceItem.instance] = {
                                    date: date.date,
                                    sets: {}
                                };
                            }

                            if (!output[instanceItem.instance].sets.hasOwnProperty(setItem.set_number)) {
                                output[instanceItem.instance].sets[setItem.set_number] = {};
                            }

                            if (!output[instanceItem.instance].sets[setItem.set_number].hasOwnProperty(field.type)) {
                                output[instanceItem.instance].sets[setItem.set_number][field.type] = {
                                    value: instanceItem.value,
                                    name: field.label,
                                    type: field.type
                                };
                            }
                        });
                    });
                });
            });

            return $filter('orderBy')(_.values(output), 'date', true);
        };

        var addFieldLists = function (day) {

            day.fields = [];

            if (day.sets.hasOwnProperty(1)) {
                angular.forEach(day.sets[1], function (field) {
                    day.fields.push({
                        name: field.name,
                        type: field.type
                    });
                });
            }

            return day;
        };

        var buildOutput = function (data) {

            var overAllData = buildOverallExerciseOutput(data);

            return overAllData.map(addFieldLists);
        };

        var buildFieldOutput = function (data) {
            return _.values(data);
        };

        var extractField = function (data, field, setNo, exercise) {

            var label = exercise.meta[field.name + '_label'];

            var key = field.type;

            if (field.type === 'speed') {
                key = 'speed_' + label;
            }

            var output = {};

            angular.forEach(data, function (date) {

                if (date.data.hasOwnProperty(key) && date.data[key].sets.hasOwnProperty(setNo)) {

                    output[date.date] = {
                        date: date.date,
                        instances: date.data[key].sets[setNo].instances
                    };
                }
            });

            return _.values(output);
        };

        service.loadHistory = function (id, customerId, allWorkouts, idType, fields, workoutId) {

            allWorkouts = allWorkouts || false;

            return $q(function (resolve) {

                var cached = loadFromCache(id, customerId, allWorkouts);

                if (cached) {

                    resolve(cached);

                    return true;
                }

                ApiClientService.workouts.getExerciseHistory(id, customerId, allWorkouts, idType, fields, workoutId).then(function (response) {

                    var output = buildOutput(response.history);

                    resolve(saveToCache(id, customerId, output, allWorkouts));
                });

                return true;
            });
        };

        service.loadFieldHistory = function (id, customerId, field, setNo, exercise, allWorkouts, idType, fields) {

            allWorkouts = allWorkouts || false;

            return $q(function (resolve) {

                var cached = loadFromFieldCache(id, customerId, allWorkouts);

                if (cached) {

                    cached = extractField(cached, field, setNo, exercise);

                    resolve(cached);

                    return true;
                }

                ApiClientService.workouts.getExerciseHistory(id, customerId, allWorkouts, idType, fields).then(function (response) {

                    var output = buildFieldOutput(response.history);

                    output = saveToFieldCache(id, customerId, output, allWorkouts);

                    resolve(extractField(output, field, setNo, exercise));
                });

                return true;
            });
        };

        service.clearCache = function () {
            historyCache = {
                this: {},
                all: {}
            };
            fieldHistoryCache = {
                this: {},
                all: {}
            };
        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('SessionExerciseFieldHistoryModalController', controller);

    controller.$inject = ['parameters', 'SessionHistoryService', '$filter', 'AnalyticsService'];

    function controller(parameters, SessionHistoryService, $filter, AnalyticsService) {

        var vm = this;

        var pivotId = parameters.pivotId;
        var exercise = parameters.exercise;
        var field = parameters.field;
        var customerId = parameters.customerId;
        var setNo = parameters.setNo;
        var defaultAllWorkouts = parameters.defaultAllWorkouts;

        vm.loaded = false;
        vm.fieldType = field.type;
        vm.setNo = setNo;
        vm.exercise = exercise;
        vm.allWorkouts = defaultAllWorkouts;
        vm.tabIndex = defaultAllWorkouts ? 1 : 0;

        vm.showForAllWorkouts = function (allWorkouts) {
            vm.allWorkouts = allWorkouts;
            vm.loadHistory();
        };

        vm.loadHistory = function () {

            var idType = 'pivot';
            var fields = {};
            var id = pivotId;

            if (exercise.hasOwnProperty('originalExercise')) {

                idType = 'exercise';
                id = exercise.exercise.id;

                // get fields from meta
                for (var i = 1; i <= 5; i++) {

                    var key = 'field_' + i;
                    var labelKey = 'field_' + i + '_label';

                    fields[key] = exercise.meta[key];
                    fields[labelKey] = exercise.meta[labelKey];
                }
            }

            vm.loading = true;

            SessionHistoryService.loadFieldHistory(id, customerId, field, setNo, exercise, vm.allWorkouts, idType, fields).then(function (history) {

                if (vm.allWorkouts) {
                    vm.historyAll = history;
                } else {
                    vm.history = history;
                }

                vm.loaded = true;
                vm.loading = false;
            });
        };

        vm.displayValue = function (value) {

            var label = exercise.meta[field.name + '_label'];

            return $filter('setValueDisplay')(value, label, field.type);
        };

        vm.loadHistory();

        AnalyticsService.trackView('Exercise History - Field');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('SessionExerciseHistoryModalController', controller);

    controller.$inject = ['parameters', 'SessionHistoryService', '$filter', 'AnalyticsService'];

    function controller(parameters, SessionHistoryService, $filter, AnalyticsService) {

        var vm = this;

        var exercise = parameters.exercise;
        var customerId = parameters.customerId;
        var defaultAllWorkouts = parameters.defaultAllWorkouts;
        var workoutId = parameters.workoutId;

        vm.loaded = false;
        vm.exercise = exercise;
        vm.allWorkouts = defaultAllWorkouts;
        vm.tabIndex = defaultAllWorkouts ? 1 : 0;

        vm.showForAllWorkouts = function (allWorkouts) {
            vm.allWorkouts = allWorkouts;
            vm.loadHistory();
        };

        vm.loadHistory = function () {

            vm.loading = true;

            SessionHistoryService.loadHistory(exercise.exercise.id, customerId, vm.allWorkouts, 'exercise', {}, workoutId).then(function (history) {

                if (vm.allWorkouts) {
                    vm.historyAll = history;
                } else {
                    vm.history = history;
                }

                vm.loaded = true;
                vm.loading = false;
            });
        };

        vm.displayValue = function (setNo, set, col) {

            var label = exercise.meta[col.name + '_label'];

            if (col.type === 'sets') {
                return setNo;
            }

            if (col.type === 'speed' && set['speed_' + label]) {
                set['speed'] = set['speed_' + label];
            }

            if (!set[col.type]) {
                return '—';
            }

            return $filter('setValueDisplay')(set[col.type].value, label, col.type);
        };

        vm.loadHistory();

        AnalyticsService.trackView('Exercise History');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('SessionSwapExerciseFieldsModalController', controller);

    controller.$inject = ['parameters', '$filter', 'AnalyticsService', 'AuthenticationStateService'];

    function controller(parameters, $filter, AnalyticsService, AuthenticationStateService) {

        var currentUser = AuthenticationStateService.getCurrentUser();

        var defaultLabels = {
            speed: 'kph',
            weight: 'kg',
            distance: 'km'
        };

        if (currentUser.country === 'US') {
            defaultLabels = {
                speed: 'mph',
                weight: 'lbs',
                distance: 'miles'
            };
        }

        var vm = this;

        vm.origin = parameters.originalExercise;
        vm.chosen = parameters.chosenExercise;
        vm.chosenMeta = angular.copy(vm.origin.meta);

        vm.setFields = function () {
            this.closeModal({
                exercise: vm.chosen,
                meta: vm.chosenMeta
            });
        };

        vm.close = this.cancelModal;

        // Set available field types
        vm.filterAvailableFieldTypes = function () {

            var usedTypes = [];
            vm.chosenMeta.availableFieldTypesFiltered = {};

            angular.forEach(vm.chosenMeta.set_fields, function (setField) {
                usedTypes.push(vm.chosenMeta[setField.name]);
            });

            angular.forEach(vm.chosenMeta.set_fields, function (setField) {
                vm.chosenMeta.availableFieldTypesFiltered[setField.name] = $filter('filter')(vm.chosenMeta.availableFieldTypes, function (availableType) {
                    if (usedTypes.indexOf(availableType.type) !== -1 && availableType.type !== 'n/a') {
                        if (vm.chosenMeta[setField.name] !== usedTypes[usedTypes.indexOf(availableType.type)]) {
                            return false;
                        }
                    }
                    return true;
                });
            });
        };

        vm.filterAvailableFieldTypes();

        vm.changedFieldType = function (field) {
            if (vm.chosenMeta[field.name] === 'speed') {
                vm.chosenMeta[field.name + '_label'] = defaultLabels.speed;
            } else if (vm.chosenMeta[field.name] === 'weight') {
                vm.chosenMeta[field.name + '_label'] = defaultLabels.weight;
            } else if (vm.chosenMeta[field.name] === 'distance') {
                vm.chosenMeta[field.name + '_label'] = defaultLabels.distance;
            } else {
                vm.chosenMeta[field.name + '_label'] = null;
            }

            field.type = vm.chosenMeta[field.name];

            vm.filterAvailableFieldTypes();
        };

        vm.fieldLabels = {
            speed: ['kph', 'mph', 'rpm'],
            distance: ['km', 'miles'],
            weight: ['kg', 'lbs']
        };

        AnalyticsService.trackView('Exercise Swap - Set Fields');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('SwapExerciseSearchController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.session_logger_swap_exercise', {
            url: "/log_session/swap/search",
            views: {
                'menuContent': {
                    templateUrl: "common/session/exerciseSwap/swapExerciseSearch.html",
                    controller: 'SwapExerciseSearchController as vm'
                }
            }
        });
    }

    controller.$inject = ['$stateParams', '$scope', 'ApiClientService', 'SessionLoggerService', '$ionicScrollDelegate', 'lodash', 'NavigateService', 'SessionHistoryService', 'ModalService', 'AnalyticsService', 'PermissionsService'];

    function controller($stateParams, $scope, ApiClientService, SessionLoggerService, $ionicScrollDelegate, lodash, NavigateService, SessionHistoryService, ModalService, AnalyticsService, PermissionsService) {

        var vm = this;

        vm.isTrainer = PermissionsService.userIsTrainer();

        vm.suggestedAlternatives = [];

        var resultsPerPage = 30;
        vm.loadingMore = false;

        var setupBodyPartList = function () {

            ApiClientService.exercises.getAttributeValuesByNameId(2).success(function (response) {

                vm.bodyParts = [{
                    name: 'Select Body Part',
                    value: null
                }];

                angular.forEach(response.values, function (attribute) {

                    vm.bodyParts.push({
                        name: attribute.value,
                        value: attribute.id
                    });
                });

                vm.selectedPart = {
                    type: vm.bodyParts[0].value
                };
            });
        };

        var loadSuggestedAlternatives = function () {

            vm.suggestedAlternatives.length = 0;

            return ApiClientService.exercises.getAlternatives(vm.originalExercise.exercise.id).success(function (response) {

                vm.suggestedAlternatives = response.exercises;

                if (vm.suggestedAlternatives.length > 0) {
                    vm.setMode('suggest');
                } else {
                    vm.setMode('search');
                }
            });
        };

        var resetSearch = function () {
            vm.search = {
                query: null,
                results: [],
                noResults: false,
                page: 1,
                nextPage: null,
                loading: false
            };
        };
        resetSearch();

        var resetBodypartSearch = function () {
            vm.bodyPartSearch = {
                type: null,
                results: [],
                noResults: false,
                page: 1,
                nextPage: null,
                loading: false
            };
        };
        resetBodypartSearch();

        var resetRecent = function () {
            vm.recentSearch = {
                results: [],
                noResults: false,
                page: 1,
                nextPage: null,
                loading: false
            };
        };
        resetRecent();

        var resetFrequent = function () {
            vm.frequentSearch = {
                results: [],
                noResults: false,
                page: 1,
                nextPage: null,
                loading: false
            };
        };
        resetFrequent();

        var resetCustom = function () {
            vm.customSearch = {
                results: [],
                noResults: false,
                page: 1,
                nextPage: null,
                loading: false
            };
        };
        resetCustom();

        var doSearch = function (paginating) {

            paginating = paginating || false;

            if (!vm.search.query) {

                resetSearch();

                return false;
            }

            if (!paginating) {
                vm.search.results.length = 0;
                vm.search.page = 1;
                vm.search.nextPage = null;
            }

            vm.search.loading = true;
            vm.loadingMore = true;

            ApiClientService.exercises.search(vm.search.query, resultsPerPage, vm.search.page).success(function (response) {

                vm.search.nextPage = response.pagination.nextPage || null;

                if (paginating) {
                    vm.search.results.push.apply(vm.search.results, angular.copy(response.results));
                } else {
                    vm.search.results = response.results;
                }

                vm.search.loading = false;
                vm.search.noResults = vm.search.results.length === 0 && vm.search.query;
                vm.loadingMore = false;

                $ionicScrollDelegate.resize();
            });
        };

        // One time cache-able setup
        setupBodyPartList();

        vm.doBodyPartSearch = function (paginating) {

            paginating = paginating || false;

            if (!paginating) {
                vm.bodyPartSearch.results.length = 0;
                vm.bodyPartSearch.page = 1;
                vm.bodyPartSearch.nextPage = null;
            }

            vm.bodyPartSearch.loading = true;
            vm.loadingMore = true;

            ApiClientService.exercises.getExercisesByValueId(vm.bodyPartSearch.type, resultsPerPage, vm.bodyPartSearch.page).success(function (response) {

                vm.bodyPartSearch.nextPage = response.pagination.nextPage || null;

                if (paginating) {
                    vm.bodyPartSearch.results.push.apply(vm.bodyPartSearch.results, angular.copy(response.results));
                } else {
                    vm.bodyPartSearch.results = response.results;
                }

                vm.bodyPartSearch.loading = false;
                vm.bodyPartSearch.noResults = vm.bodyPartSearch.results.length === 0;
                vm.loadingMore = false;

                $ionicScrollDelegate.resize();
            });
        };

        vm.doRecentSearch = function (paginating) {

            paginating = paginating || false;

            if (!paginating && vm.recentSearch.results.length > 0) {
                return false;
            }

            vm.recentSearch.loading = true;
            vm.loadingMore = true;

            ApiClientService.exercises.getRecentlyUsed(resultsPerPage, vm.recentSearch.page).success(function (response) {

                vm.recentSearch.nextPage = response.pagination.nextPage || null;

                if (paginating) {
                    vm.recentSearch.results.push.apply(vm.recentSearch.results, angular.copy(response.exercises));
                } else {
                    vm.recentSearch.results = response.exercises;
                }

                vm.recentSearch.loading = false;
                vm.recentSearch.noResults = vm.recentSearch.results.length === 0;
                vm.loadingMore = false;

                $ionicScrollDelegate.resize();
            });
        };

        vm.doFrequentSearch = function (paginating) {

            paginating = paginating || false;

            if (!paginating && vm.frequentSearch.results.length > 0) {
                return false;
            }

            vm.frequentSearch.loading = true;
            vm.loadingMore = true;

            ApiClientService.exercises.getFrequentlyUsed(resultsPerPage, vm.frequentSearch.page).success(function (response) {

                vm.frequentSearch.nextPage = response.pagination.nextPage || null;

                if (paginating) {
                    vm.frequentSearch.results.push.apply(vm.frequentSearch.results, angular.copy(response.exercises));
                } else {
                    vm.frequentSearch.results = response.exercises;
                }

                vm.frequentSearch.loading = false;
                vm.frequentSearch.noResults = vm.frequentSearch.results.length === 0;
                vm.loadingMore = false;

                $ionicScrollDelegate.resize();
            });
        };

        vm.doCustomSearch = function (paginating) {

            paginating = paginating || false;

            if (!paginating && vm.customSearch.results.length > 0) {
                return false;
            }

            vm.customSearch.loading = true;
            vm.loadingMore = true;

            ApiClientService.exercises.getAllForTrainer(resultsPerPage, vm.customSearch.page).success(function (response) {

                vm.customSearch.nextPage = response.pagination.nextPage || null;

                if (paginating) {
                    vm.customSearch.results.push.apply(vm.customSearch.results, angular.copy(response.exercises));
                } else {
                    vm.customSearch.results = response.exercises;
                }

                vm.customSearch.loading = false;
                vm.customSearch.noResults = vm.customSearch.results.length === 0;
                vm.loadingMore = false;

                $ionicScrollDelegate.resize();
            });
        };

        // Non cache-able setup
        $scope.$on('$ionicView.enter', function () {

            vm.ready = false;

            vm.session = SessionLoggerService.getSession();
            vm.originalExercise = SessionLoggerService.getSwappingExercise();

            loadSuggestedAlternatives().then(function () {
                vm.ready = true;
            });

            resetSearch();
            resetBodypartSearch();
            resetRecent();
            resetFrequent();
            resetCustom();
        });

        vm.setMode = function (mode) {
            vm.currentMode = mode;

            switch (vm.currentMode) {

                case 'recent':
                    vm.doRecentSearch();
                    break;

                case 'frequent':
                    vm.doFrequentSearch();
                    break;

                case 'custom':
                    vm.doCustomSearch();
                    break;

            }
        };

        vm.isMode = function (checkMode) {
            return vm.currentMode === checkMode;
        };

        vm.canLoadMore = function () {

            switch (vm.currentMode) {

                case 'search':
                    if (vm.search.results.length > 0 && vm.search.loading === false && vm.search.nextPage !== null) {
                        return true;
                    }
                    break;

                case 'bodypart':
                    if (vm.bodyPartSearch.results.length > 0 && vm.bodyPartSearch.loading === false && vm.bodyPartSearch.nextPage !== null) {
                        return true;
                    }
                    break;

                case 'recent':
                    if (vm.recentSearch.results.length > 0 && vm.recentSearch.loading === false && vm.recentSearch.nextPage !== null) {
                        return true;
                    }
                    break;

                case 'frequent':
                    if (vm.frequentSearch.results.length > 0 && vm.frequentSearch.loading === false && vm.frequentSearch.nextPage !== null) {
                        return true;
                    }
                    break;

                case 'custom':
                    if (vm.customSearch.results.length > 0 && vm.customSearch.loading === false && vm.customSearch.nextPage !== null) {
                        return true;
                    }
                    break;

            }

            return false;
        };

        vm.loadMore = function () {

            switch (vm.currentMode) {

                case 'search':
                    vm.search.page++;
                    doSearch(true);
                    break;

                case 'bodypart':
                    vm.bodyPartSearch.page++;
                    vm.doBodyPartSearch(true);
                    break;

                case 'recent':
                    vm.recentSearch.page++;
                    vm.doRecentSearch(true);
                    break;

                case 'frequent':
                    vm.frequentSearch.page++;
                    vm.doFrequentSearch(true);
                    break;

                case 'custom':
                    vm.customSearch.page++;
                    vm.doCustomSearch(true);
                    break;

            }
        };

        vm.chooseExercise = function (exercise) {

            if (exercise.hasOwnProperty('exercise')) {

                exercise = lodash.merge(angular.copy(exercise), angular.copy(exercise.exercise));

                delete exercise.exercise;
            }

            ModalService.show({
                templateUrl: "common/session/exerciseSwap/swapExerciseFields.html",
                controller: 'SessionSwapExerciseFieldsModalController as vm',
                params: {
                    originalExercise: vm.originalExercise,
                    chosenExercise: exercise
                }
            }).then(function (chosenExercise) {

                SessionLoggerService.swapExercise(chosenExercise);

                SessionHistoryService.clearCache();

                NavigateService.goBack();
            });
        };

        $scope.$watch(function () {
            return vm.search.query;
        }, function () {
            doSearch();
        });

        AnalyticsService.trackView('Exercise Swap - Search');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('SessionLoggerAddExerciseModalController', controller);

    controller.$inject = ['$scope', 'ApiClientService', '$ionicScrollDelegate', 'lodash', '$ionicActionSheet', 'AnalyticsService', 'SessionLoggerService', 'PermissionsService'];

    function controller($scope, ApiClientService, $ionicScrollDelegate, lodash, $ionicActionSheet, AnalyticsService, SessionLoggerService, PermissionsService) {

        var vm = this;

        vm.isTrainer = PermissionsService.userIsTrainer();

        var resultsPerPage = 30;
        vm.loadingMore = false;

        var setupBodyPartList = function () {

            ApiClientService.exercises.getAttributeValuesByNameId(2).success(function (response) {

                vm.bodyParts = [{
                    name: 'Select Body Part',
                    value: null
                }];

                angular.forEach(response.values, function (attribute) {

                    vm.bodyParts.push({
                        name: attribute.value,
                        value: attribute.id
                    });
                });

                vm.selectedPart = {
                    type: vm.bodyParts[0].value
                };
            });
        };

        var resetSearch = function () {
            vm.search = {
                query: null,
                results: [],
                noResults: false,
                page: 1,
                nextPage: null,
                loading: false
            };
        };
        resetSearch();

        var resetBodypartSearch = function () {
            vm.bodyPartSearch = {
                type: null,
                results: [],
                noResults: false,
                page: 1,
                nextPage: null,
                loading: false
            };
        };
        resetBodypartSearch();

        var resetRecent = function () {
            vm.recentSearch = {
                results: [],
                noResults: false,
                page: 1,
                nextPage: null,
                loading: false
            };
        };
        resetRecent();

        var resetFrequent = function () {
            vm.frequentSearch = {
                results: [],
                noResults: false,
                page: 1,
                nextPage: null,
                loading: false
            };
        };
        resetFrequent();

        var resetCustom = function () {
            vm.customSearch = {
                results: [],
                noResults: false,
                page: 1,
                nextPage: null,
                loading: false
            };
        };
        resetCustom();

        var doSearch = function (paginating) {

            paginating = paginating || false;

            if (!vm.search.query) {

                resetSearch();

                return false;
            }

            if (!paginating) {
                vm.search.results.length = 0;
                vm.search.page = 1;
                vm.search.nextPage = null;
            }

            vm.search.loading = true;
            vm.loadingMore = true;

            ApiClientService.exercises.search(vm.search.query, resultsPerPage, vm.search.page).success(function (response) {

                vm.search.nextPage = response.pagination.nextPage || null;

                if (paginating) {
                    vm.search.results.push.apply(vm.search.results, angular.copy(response.results));
                } else {
                    vm.search.results = response.results;
                }

                vm.search.loading = false;
                vm.search.noResults = vm.search.results.length === 0 && vm.search.query;
                vm.loadingMore = false;

                $ionicScrollDelegate.resize();
            });
        };

        // One time cache-able setup
        setupBodyPartList();

        vm.doBodyPartSearch = function (paginating) {

            paginating = paginating || false;

            if (!paginating) {
                vm.bodyPartSearch.results.length = 0;
                vm.bodyPartSearch.page = 1;
                vm.bodyPartSearch.nextPage = null;
            }

            vm.bodyPartSearch.loading = true;
            vm.loadingMore = true;

            ApiClientService.exercises.getExercisesByValueId(vm.bodyPartSearch.type, resultsPerPage, vm.bodyPartSearch.page).success(function (response) {

                vm.bodyPartSearch.nextPage = response.pagination.nextPage || null;

                if (paginating) {
                    vm.bodyPartSearch.results.push.apply(vm.bodyPartSearch.results, angular.copy(response.results));
                } else {
                    vm.bodyPartSearch.results = response.results;
                }

                vm.bodyPartSearch.loading = false;
                vm.bodyPartSearch.noResults = vm.bodyPartSearch.results.length === 0;
                vm.loadingMore = false;

                $ionicScrollDelegate.resize();
            });
        };

        vm.doRecentSearch = function (paginating) {

            paginating = paginating || false;

            if (!paginating && vm.recentSearch.results.length > 0) {
                return false;
            }

            vm.recentSearch.loading = true;
            vm.loadingMore = true;

            ApiClientService.exercises.getRecentlyUsed(resultsPerPage, vm.recentSearch.page).success(function (response) {

                vm.recentSearch.nextPage = response.pagination.nextPage || null;

                if (paginating) {
                    vm.recentSearch.results.push.apply(vm.recentSearch.results, angular.copy(response.exercises));
                } else {
                    vm.recentSearch.results = response.exercises;
                }

                vm.recentSearch.loading = false;
                vm.recentSearch.noResults = vm.recentSearch.results.length === 0;
                vm.loadingMore = false;

                $ionicScrollDelegate.resize();
            });
        };

        vm.doFrequentSearch = function (paginating) {

            paginating = paginating || false;

            if (!paginating && vm.frequentSearch.results.length > 0) {
                return false;
            }

            vm.frequentSearch.loading = true;
            vm.loadingMore = true;

            ApiClientService.exercises.getFrequentlyUsed(resultsPerPage, vm.frequentSearch.page).success(function (response) {

                vm.frequentSearch.nextPage = response.pagination.nextPage || null;

                if (paginating) {
                    vm.frequentSearch.results.push.apply(vm.frequentSearch.results, angular.copy(response.exercises));
                } else {
                    vm.frequentSearch.results = response.exercises;
                }

                vm.frequentSearch.loading = false;
                vm.frequentSearch.noResults = vm.frequentSearch.results.length === 0;
                vm.loadingMore = false;

                $ionicScrollDelegate.resize();
            });
        };

        vm.doCustomSearch = function (paginating) {

            paginating = paginating || false;

            if (!paginating && vm.customSearch.results.length > 0) {
                return false;
            }

            vm.customSearch.loading = true;
            vm.loadingMore = true;

            ApiClientService.exercises.getAllForTrainer(resultsPerPage, vm.customSearch.page).success(function (response) {

                vm.customSearch.nextPage = response.pagination.nextPage || null;

                if (paginating) {
                    vm.customSearch.results.push.apply(vm.customSearch.results, angular.copy(response.exercises));
                } else {
                    vm.customSearch.results = response.exercises;
                }

                vm.customSearch.loading = false;
                vm.customSearch.noResults = vm.customSearch.results.length === 0;
                vm.loadingMore = false;

                $ionicScrollDelegate.resize();
            });
        };

        resetSearch();
        resetBodypartSearch();
        resetRecent();
        resetFrequent();
        resetCustom();

        vm.setMode = function (mode) {
            vm.currentMode = mode;

            switch (vm.currentMode) {

                case 'recent':
                    vm.doRecentSearch();
                    break;

                case 'frequent':
                    vm.doFrequentSearch();
                    break;

                case 'custom':
                    vm.doCustomSearch();
                    break;

            }
        };

        vm.isMode = function (checkMode) {
            return vm.currentMode === checkMode;
        };

        vm.canLoadMore = function () {

            switch (vm.currentMode) {

                case 'search':
                    if (vm.search.results.length > 0 && vm.search.loading === false && vm.search.nextPage !== null) {
                        return true;
                    }
                    break;

                case 'bodypart':
                    if (vm.bodyPartSearch.results.length > 0 && vm.bodyPartSearch.loading === false && vm.bodyPartSearch.nextPage !== null) {
                        return true;
                    }
                    break;

                case 'recent':
                    if (vm.recentSearch.results.length > 0 && vm.recentSearch.loading === false && vm.recentSearch.nextPage !== null) {
                        return true;
                    }
                    break;

                case 'frequent':
                    if (vm.frequentSearch.results.length > 0 && vm.frequentSearch.loading === false && vm.frequentSearch.nextPage !== null) {
                        return true;
                    }
                    break;

                case 'custom':
                    if (vm.customSearch.results.length > 0 && vm.customSearch.loading === false && vm.customSearch.nextPage !== null) {
                        return true;
                    }
                    break;

            }

            return false;
        };

        vm.loadMore = function () {

            switch (vm.currentMode) {

                case 'search':
                    vm.search.page++;
                    doSearch(true);
                    break;

                case 'bodypart':
                    vm.bodyPartSearch.page++;
                    vm.doBodyPartSearch(true);
                    break;

                case 'recent':
                    vm.recentSearch.page++;
                    vm.doRecentSearch(true);
                    break;

                case 'frequent':
                    vm.frequentSearch.page++;
                    vm.doFrequentSearch(true);
                    break;

                case 'custom':
                    vm.customSearch.page++;
                    vm.doCustomSearch(true);
                    break;

            }
        };

        vm.chooseExercise = function (exercise) {

            if (exercise.hasOwnProperty('exercise')) {

                exercise = lodash.merge(angular.copy(exercise), angular.copy(exercise.exercise));

                delete exercise.exercise;
            }

            if (exercise.exercise_type === 'cardio') {

                $ionicActionSheet.show({
                    titleText: 'Choose cardio type',
                    buttons: [{
                        text: 'Long Slow Duration (LSD)'
                    }, {
                        text: 'Interval Training'
                    }, {
                        text: 'Fartlek Training'
                    }],
                    cancelText: 'Cancel',
                    buttonClicked: function (index) {

                        var type;

                        switch (index) {

                            // LSD
                            case 0:
                                type = 'lsd';
                                break;

                            // Interval
                            case 1:
                                type = 'interval';
                                break;

                            // Fartlek
                            case 2:
                                type = 'fartlek';
                                break;
                        }

                        SessionLoggerService.addCardioExercise(exercise, type);
                        $scope.closeModal(true);

                        return true;
                    }
                });
            } else {

                SessionLoggerService.addStandardExercise(exercise);
                $scope.closeModal(true);
            }
        };

        $scope.$watch(function () {
            return vm.search.query;
        }, function () {
            doSearch();
        });

        AnalyticsService.trackView('Session Logger - Add Exercise Search');

        //$scope.closeModal();
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('SessionLoggerExerciseGrouperModalController', controller);

    controller.$inject = ['parameters', 'AnalyticsService', '$filter', 'SessionLoggerService', '$scope', 'ToastService'];

    function controller(parameters, AnalyticsService, $filter, SessionLoggerService, $scope, ToastService) {

        var vm = this;

        var baseExerciseInstance = parameters.baseExerciseInstance;
        var baseGroup = parameters.baseGroup;

        vm.baseExercise = baseExerciseInstance.exercise;

        var currentWorkout = SessionLoggerService.getWorkoutData();

        var isCardioType = function (type) {

            var groupingDisabledForExerciseTypes = ['fartlek', 'lsd', 'interval'];

            if (groupingDisabledForExerciseTypes.indexOf(type) !== -1) {
                return true;
            }

            return false;
        };

        var getGroupable = function (baseGroup) {

            baseGroup = baseGroup || false;

            var groupables = [];

            angular.forEach(currentWorkout.groups, function (group, groupKey) {

                var firstExercise = group.exercises[0];
                var firstExerciseType = firstExercise.meta.type;

                firstExercise.origGroupKey = groupKey;

                if (group.group_type === 'standalone' && !isCardioType(firstExerciseType) && (!baseGroup || group.$$hashKey !== baseGroup.$$hashKey)) {
                    groupables.push(firstExercise);
                }
            });

            return groupables;
        };

        var getGrouperChosen = function () {
            var exercisesToAddToGroup = getGroupable();
            return $filter('filter')(exercisesToAddToGroup, { use: true });
        };

        var resetGrouperChosen = function () {
            angular.forEach(getGrouperChosen(), function (exerciseInstance) {
                exerciseInstance.use = false;
            });
        };
        resetGrouperChosen();

        var removeEmptyGroups = function () {
            currentWorkout.groups = $filter('filter')(currentWorkout.groups, function (group) {
                return group.exercises.length > 0;
            });
        };

        vm.doGrouping = function () {

            var exercisesToGroup = getGrouperChosen();

            if (exercisesToGroup.length === 0) {
                ToastService.show('Please select one or more exercises');
                return false;
            }

            var firstExercise = baseGroup.exercises[0];

            var newExercises = [firstExercise];
            var exerciseSort = 0;

            angular.forEach(exercisesToGroup, function (exercise) {

                exercise.sort = exerciseSort;
                exercise.use = false;
                newExercises.push(exercise);

                // Remove original exercise
                currentWorkout.groups[exercise.origGroupKey].exercises.splice(0, 1);

                exerciseSort++;
            });

            var exCount = exercisesToGroup.length + 1;

            var type = 'superset';
            if (exCount > 2) {
                type = 'giantset';
            }

            baseGroup.exercises = newExercises;
            baseGroup.group_type = type;

            removeEmptyGroups();

            SessionLoggerService.updateGroupSorts();

            $scope.cancelModal();
        };

        vm.availableExercises = getGroupable(baseGroup);

        vm.toggleExercise = function (exerciseInstance) {
            if (!exerciseInstance.use) {
                exerciseInstance.use = true;
            } else {
                exerciseInstance.use = false;
            }
        };

        AnalyticsService.trackView('Session Logger - Super/Giant Set Grouper');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('SessionLoggerExerciseReorderModalController', controller);

    controller.$inject = ['AnalyticsService', 'SessionLoggerService'];

    function controller(AnalyticsService, SessionLoggerService) {

        var vm = this;

        vm.workout = SessionLoggerService.getWorkoutData();

        vm.moveGroup = function (group, fromIndex, toIndex) {
            vm.workout.groups.splice(fromIndex, 1);
            vm.workout.groups.splice(toIndex, 0, group);

            for (var i = 0, length = vm.workout.groups.length; i < length; i++) {
                vm.workout.groups[i].sort = i;
            }
        };

        AnalyticsService.trackView('Session Logger - Changing Exercise Sort Order');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('SessionLoggerFinishController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.log_session_finish', {
            url: "/log_session/finish",
            views: {
                'menuContent': {
                    templateUrl: "common/session/logger/finish.html",
                    controller: 'SessionLoggerFinishController as vm'
                }
            }
        });
    }

    controller.$inject = ['SessionLoggerService', 'SessionHistoryService', 'ToastService', 'PermissionsService', 'NavigateService', '$scope', 'AnalyticsService', 'SessionLoggerDebitControlService', 'ScheduleEventStoreService', 'NotificationsService'];

    function controller(SessionLoggerService, SessionHistoryService, ToastService, PermissionsService, NavigateService, $scope, AnalyticsService, SessionLoggerDebitControlService, ScheduleEventStoreService, NotificationsService) {

        var vm = this;

        $scope.$on('$ionicView.beforeEnter', function () {
            vm.session = SessionLoggerService.getSession();
            vm.processing = false;
            vm.saveAsTemplate = false;
            vm.feedback = '';
        });

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Session - Finish');
        });

        vm.complete = function () {

            if (vm.processing) {
                return false;
            }

            vm.processing = true;

            SessionHistoryService.clearCache();

            var templateName = false;
            if (vm.saveAsTemplate && vm.templateName) {
                /**
                 * If we are saving this as a workout template, we only want to
                 * set the title of the archived version in the event of not
                 * following a pre-set workout. If we are using a pre-set
                 * workout, then we need to set the template name specifically.
                 */
                if (vm.session.workoutUsedAsTemplate) {
                    templateName = vm.templateName;
                } else {
                    SessionLoggerService.setWorkoutTitle(vm.templateName);
                }
            }

            var completeSession = SessionLoggerService.completeSession(vm.feedback, null, SessionLoggerDebitControlService.getIsBillable(), SessionLoggerDebitControlService.getBillableToQuotaId(), vm.saveAsTemplate, templateName);

            completeSession.then(function (response) {

                var isBillable = SessionLoggerDebitControlService.getIsBillable();

                SessionLoggerDebitControlService.reset();
                ScheduleEventStoreService.setDirty();
                NotificationsService.dirty();

                var msg = 'Session saved';

                if (!response) {
                    msg = 'Offline - Your session will be saved when there is an internet connection';
                }

                ToastService.show(msg);

                SessionLoggerService.inactive();

                if (PermissionsService.userIsTrainer()) {

                    var params = {
                        customerId: vm.session.customer_id
                    };

                    if (isBillable) {
                        params.tab = 'sessions';
                    }

                    NavigateService.isRoot().disableBack().go('app.trainer_client_profile', params);
                } else {

                    NavigateService.isRoot().disableBack().goHome();
                }
            }, function () {

                vm.processing = false;

                ToastService.show('Could not complete the session. Please try again');
            });
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').config(config).controller('SessionLoggerController', controller);

    config.$inject = ['$stateProvider'];

    function config($stateProvider) {

        $stateProvider.state('app.log_session', {
            url: "/log_session/:sessionId",
            views: {
                'menuContent': {
                    templateUrl: "common/session/logger/logger.html",
                    controller: 'SessionLoggerController as vm'
                }
            }
        });
    }

    controller.$inject = ['SessionLoggerService', 'NavigateService', 'ScrollService', '$scope', 'ExerciseViewerService', 'ModalService', 'PermissionsService', 'AnalyticsService', '$state', 'StopwatchService', '$ionicActionSheet', 'NoteViewerModalService'];

    function controller(SessionLoggerService, NavigateService, ScrollService, $scope, ExerciseViewerService, ModalService, PermissionsService, AnalyticsService, $state, StopwatchService, $ionicActionSheet, NoteViewerModalService) {

        var vm = this;
        vm.workoutFinished = false;

        var currentUser = PermissionsService.getCurrentUser();
        var gender = false;
        var updateTargetValues = false;

        vm.intervalStates = {
            1: 'E',
            2: 'M',
            3: 'H'
        };

        $scope.$on('$ionicView.beforeEnter', function () {

            SessionLoggerService.active();

            vm.session = SessionLoggerService.getSession();
            console.log('set session', vm.session);
            vm.workout = SessionLoggerService.getWorkoutData();

            var usingClientCreatedWorkout = vm.session.workoutUsedAsTemplate && typeof vm.workout.created_by !== 'undefined' && vm.workout.created_by === 'customer';

            updateTargetValues = usingClientCreatedWorkout || !vm.session.workoutUsedAsTemplate;

            // If this user is a trainer, set the gender to the client's gender
            if (PermissionsService.userIsCustomer()) {
                gender = currentUser.gender;
            } else if (typeof vm.session.customer !== 'undefined' && typeof vm.session.customer.user !== 'undefined') {
                gender = vm.session.customer.user.gender;
            }
        });

        $scope.$on('$ionicView.enter', function () {
            AnalyticsService.trackView('Session Logger');
        });

        // Exercise History Modal
        vm.openExerciseHistory = function (exerciseInstance, event, session) {

            event.preventDefault();

            ModalService.show({
                templateUrl: "common/session/exerciseHistory/historyModal.html",
                controller: 'SessionExerciseHistoryModalController as vm',
                params: {
                    exercise: exerciseInstance,
                    customerId: session.customer_id,
                    workoutId: session.workout_id
                }
            });
        };

        vm.swapExercise = function (exerciseInstance, event) {

            event.preventDefault();

            SessionLoggerService.resetExerciseSwap();

            SessionLoggerService.setSwappingExercise(exerciseInstance);

            NavigateService.go('app.session_logger_swap_exercise');
        };

        vm.showNote = function (instance, event) {

            event.preventDefault();

            ModalService.show({
                templateUrl: 'common/session/note/noteViewerModal.html',
                controller: 'SessionNoteViewerModalController as vm',
                params: {
                    exerciseInstance: instance
                }
            });
        };

        vm.viewWorkoutNote = function () {
            NoteViewerModalService.show(vm.workout.note);
        };

        vm.viewExercise = function (exerciseInstance) {

            event.preventDefault();

            ExerciseViewerService.open(exerciseInstance.exercise, exerciseInstance.exercise.name_id, gender);
        };

        // Exercise History Modal
        vm.doSuperGiantGrouping = function (exerciseInstance, group, event) {
            event.preventDefault();

            if (group.group_type === 'standalone') {
                ModalService.show({
                    templateUrl: "common/session/logger/exerciseGrouperModal.html",
                    controller: 'SessionLoggerExerciseGrouperModalController as vm',
                    params: {
                        baseGroup: group,
                        baseExerciseInstance: exerciseInstance
                    }
                });
            } else {
                $ionicActionSheet.show({
                    destructiveText: 'Break apart ' + group.group_type,
                    cancelText: 'Cancel',
                    destructiveButtonClicked: function () {
                        SessionLoggerService.breakGroupApart(group);
                        return true;
                    }
                });
            }
        };

        vm.toggleSet = SessionLoggerService.toggleSet;

        vm.toggleSets = SessionLoggerService.toggleSets;

        vm.addSet = SessionLoggerService.addSet;

        vm.addIntervalRound = SessionLoggerService.addIntervalRound;

        vm.startAddExercise = function () {
            ModalService.show({
                templateUrl: "common/session/logger/exerciseAddModal.html",
                controller: 'SessionLoggerAddExerciseModalController as vm'
            }).then(function () {
                ScrollService.bottom();
            });
        };

        vm.startExerciseReorder = function () {
            ModalService.show({
                templateUrl: "common/session/logger/exerciseReorderModal.html",
                controller: 'SessionLoggerExerciseReorderModalController as vm'
            }).then(function () {
                ScrollService.top();
            });
        };

        vm.finishWorkout = function () {

            vm.workoutFinished = true;

            const config = {
                titleText: '<strong>Are you finished with this workout?</strong><br>All invalid/empty sets will be discarded. All sets with valid data will be automatically marked as complete.',
                buttons: [{
                    text: 'Finish Workout'
                }],
                cancelText: 'No, Take Me Back',
                cancel: () => {},
                buttonClicked: index => {
                    if (index === 0) {
                        $state.go('app.log_session_finish');
                    }
                }
            };

            $ionicActionSheet.show(config);
        };

        vm.quit = function () {
            SessionLoggerService.quit();
        };

        var checkIfSetIsDone = function (set, exerciseInstance) {

            var neverPopulates = ['state', 'bodyweight', 'n/a'];

            var fieldsToBePopulated = 0;
            var fieldsPopulated = 0;

            if (!set.done || !set.current_target) {
                return false;
            }

            for (var fieldNo = 1; fieldNo <= 5; fieldNo++) {

                var fieldName = 'field_' + fieldNo;

                /**
                 * If this field is a type that appears in neverPopulates,
                 * then we can skip over it for checking whether the set
                 * is complete.
                 */
                if (exerciseInstance.meta.hasOwnProperty(fieldName) && neverPopulates.indexOf(exerciseInstance.meta[fieldName]) !== -1) {
                    continue;
                }

                /**
                 * If a target value for this field is is defined, then we can
                 * consider this is a field that needs to be populated to be
                 * considered as "done".
                 */
                if (exerciseInstance.meta.hasOwnProperty(fieldName) && set.current_target.hasOwnProperty(fieldName) && set.current_target[fieldName] !== null) {
                    fieldsToBePopulated++;
                    if (set.done.hasOwnProperty(fieldName)) {
                        fieldsPopulated++;
                    }
                }
            }

            if (fieldsToBePopulated === fieldsPopulated) {
                SessionLoggerService.toggleSet(set, exerciseInstance);
                return true;
            }

            return false;
        };

        vm.changedSetValueInput = function (set, field, exerciseInstance) {

            if (!set.isDone) {
                checkIfSetIsDone(set, exerciseInstance);
            }

            if (set.isDone && set.done[field.name]) {
                set.current_target[field.name] = set.done[field.name];
            }

            if (set.done[field.name] && updateTargetValues) {
                set[field.name] = set.done[field.name];
            }
        };

        vm.openStopwatch = function () {
            StopwatchService.open();
        };

        vm.toggleEditExerciseMode = function (event) {
            event.preventDefault();
            event.stopPropagation();
            vm.editExerciseMode = !vm.editExerciseMode;
        };

        vm.removeSet = SessionLoggerService.removeSet;

        vm.removeExercise = function (exerciseInstance, group) {
            SessionLoggerService.removeExercise(exerciseInstance, group).then(function () {
                if (vm.workout.groups.length === 0) {
                    vm.editExerciseMode = false;
                }
            });
        };

        vm.triggerFieldSwitcher = SessionLoggerService.triggerFieldSwitcher;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('SessionNoteViewerModalController', controller);

    controller.$inject = ['parameters'];

    function controller(parameters) {

        var vm = this;

        vm.exerciseInstance = parameters.exerciseInstance;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('SessionLoggerDebitControlService', service).component('sessionLoggerDebitControl', {
        templateUrl: 'common/session/quotaDebitControl/quotaDebitControl.html',
        controller: controller,
        controllerAs: 'vm',
        bindings: {
            customerId: '='
        }
    });

    service.$inject = ['$rootScope'];

    function service($rootScope) {

        var service = {};

        var billable, billableToQuotaId;

        var fireEventHandlers = function (type) {
            $rootScope.$emit('SessionLoggerDebitControlService_' + type);
        };

        var registerEventHandler = function (scope, type, handler) {

            var unsub = $rootScope.$on('SessionLoggerDebitControlService_' + type, handler);

            scope.$on('$destroy', unsub);

            return unsub;
        };

        service.reset = function () {
            var triggerBillableChanged = billable;
            var triggerBillableToQuotaChanged = billableToQuotaId;

            billable = false;
            billableToQuotaId = false;

            if (triggerBillableChanged) {
                fireEventHandlers('changedBillableStatus');
            }

            if (triggerBillableToQuotaChanged) {
                fireEventHandlers('changedBillableToQuotaId');
            }

            fireEventHandlers('reset');
        };

        service.setIsBillable = function (isBillable) {
            billable = isBillable;
            fireEventHandlers('changedBillableStatus');
        };

        service.getIsBillable = function () {
            return billable;
        };

        service.setBillableToQuotaId = function (newBillableToQuotaId) {
            billableToQuotaId = newBillableToQuotaId;
            fireEventHandlers('changedBillableToQuotaId');
        };

        service.getBillableToQuotaId = function () {
            return billableToQuotaId;
        };

        service.registerEventHandler = registerEventHandler;

        return service;
    }

    controller.$inject = ['PermissionsService', 'ApiClientService', 'SessionLoggerDebitControlService', '$filter', '$scope'];

    function controller(PermissionsService, ApiClientService, SessionLoggerDebitControlService, $filter, $scope) {
        var vm = this;

        vm.loaded = false;

        var loggedInUserDetails = PermissionsService.getCurrentUser();

        if (!PermissionsService.userIsTrainer() || !loggedInUserDetails.merchant_is_provisioned) {
            vm.showSessionDebitControls = false;
            return false;
        }

        vm.showSessionDebitControls = true;

        SessionLoggerDebitControlService.reset();

        var updateLocal = function () {
            vm.deductSession = SessionLoggerDebitControlService.getIsBillable();
            vm.useQuota = SessionLoggerDebitControlService.getBillableToQuotaId();
        };
        updateLocal();

        SessionLoggerDebitControlService.registerEventHandler($scope, 'changedBillableStatus', updateLocal);
        SessionLoggerDebitControlService.registerEventHandler($scope, 'changedBillableToQuotaId', updateLocal);
        SessionLoggerDebitControlService.registerEventHandler($scope, 'reset', updateLocal);

        var sync = function () {
            SessionLoggerDebitControlService.setIsBillable(vm.deductSession);
            SessionLoggerDebitControlService.setBillableToQuotaId(vm.useQuota);
        };

        vm.quotaChosen = function () {
            vm.deductSession = true;
            sync();
        };

        vm.toggled = function () {
            sync();
        };

        var getActiveQuotas = ApiClientService.packages.getActiveSessionQuotas(vm.customerId);

        getActiveQuotas.success(function (response) {

            vm.quotas = response.quotas;

            vm.quotas.map(function (quota) {

                quota.remaining_sessions = quota.sessions_available - quota.sessions_used;

                var word = quota.remaining_sessions !== 1 ? 'sessions' : 'session';

                quota.description += ' (' + (quota.remaining_sessions > 0 ? quota.remaining_sessions : 'no') + ' ' + word + ' remaining)';

                return quota;
            });

            vm.quotas = $filter('filter')(vm.quotas, function (quota) {
                return quota.remaining_sessions > 0;
            });

            if (vm.quotas.length > 0) {
                vm.useQuota = vm.quotas[0].id;
            }

            vm.loaded = true;
            vm.canDeductSessionFromQuota = true;

            if (vm.quotas.length === 0) {
                vm.noAvailableQuotas = true;
                return false;
            }

            sync();
        });
    }
})();
(function () {

    angular.module('MyPTHub').factory('SessionLoggerService', service);

    service.$inject = ['SessionStoreService', '$q', 'AuthenticationStateService', '$filter', 'ConfirmDialogService', 'NavigateService', '$ionicActionSheet', 'ScreenStateService'];

    function service(SessionStoreService, $q, AuthenticationStateService, $filter, ConfirmDialogService, NavigateService, $ionicActionSheet, ScreenStateService) {

        var session,
            startTime,
            workoutData,
            swappingExercise,
            loggerActive = false,
            isAdhoc = false;

        var currentUser = AuthenticationStateService.getCurrentUser();

        var fieldLabels = {
            speed: ['kph', 'mph', 'rpm'],
            distance: ['km', 'miles'],
            weight: ['kg', 'lbs']
        };

        var defaultLabels = {
            speed: 'kph',
            weight: 'kg',
            distance: 'km'
        };

        if (currentUser.country === 'US') {
            defaultLabels = {
                speed: 'mph',
                weight: 'lbs',
                distance: 'miles'
            };
        }

        var emptySet = {
            field_1: 0,
            field_2: 0,
            field_3: 0,
            field_4: 0,
            field_5: 0,
            sort: 1,
            current_target: {
                field_1: 0,
                field_2: 0,
                field_3: 0,
                field_4: 0,
                field_5: 0
            },
            done: {}
        };

        var filterAvailableFieldTypes = function (meta) {

            var usedTypes = [];
            meta.availableFieldTypesFiltered = {};

            angular.forEach(meta.set_fields, function (setField) {
                usedTypes.push(meta[setField.name]);
            });

            angular.forEach(meta.set_fields, function (setField) {

                meta.availableFieldTypesFiltered[setField.name] = $filter('filter')(meta.availableFieldTypes, function (availableType) {

                    if (usedTypes.indexOf(availableType.type) !== -1 && availableType.type !== 'n/a') {

                        if (meta[setField.name] !== usedTypes[usedTypes.indexOf(availableType.type)]) {
                            return false;
                        }
                    }

                    return true;
                });
            });

            return meta;
        };

        var service = {

            reset: function () {

                session = null;
                startTime = Date.now();
                workoutData = null;
                swappingExercise = null;
            },

            setSession: function (sessionToSet, adhoc) {
                adhoc = adhoc || false;
                isAdhoc = adhoc;
                session = sessionToSet;
                if (sessionToSet.workout) {
                    workoutData = sessionToSet.workout;
                    workoutData.original_workout_id = workoutData.id;
                    session.workoutUsedAsTemplate = true;

                    angular.forEach(workoutData.groups, function (group, groupIndex) {
                        angular.forEach(group.exercises, function (exercise, exerciseIndex) {
                            exercise.meta.allSetsDone = false;
                            exercise.meta.set_fields = $filter('filter')(exercise.meta.set_fields, function (setField) {
                                return setField.type !== 'n/a';
                            });
                            angular.forEach(exercise.meta.sets, function (set, setIndex) {
                                set.isDone = false;
                                set.current_target = {
                                    field_1: set.field_1 === null ? 0 : set.field_1,
                                    field_2: set.field_2 === null ? 0 : set.field_2,
                                    field_3: set.field_3 === null ? 0 : set.field_3,
                                    field_4: set.field_4 === null ? 0 : set.field_4,
                                    field_5: set.field_5 === null ? 0 : set.field_5
                                };
                                set.done = {};
                            });
                        });
                    });
                } else if (adhoc) {
                    // Ad-hoc session with no pre-set workout being followed
                    // We need to create the base workout object manually.
                    workoutData = {
                        groups: [],
                        title: 'Workout'
                    };
                    session.workoutUsedAsTemplate = false;
                }
            },

            getSession: function () {
                return session;
            },

            getWorkoutData: function () {
                return workoutData;
            },

            setSwappingExercise: function (exerciseInstance) {
                exerciseInstance.swapping = true;
                swappingExercise = exerciseInstance;
            },

            getSwappingExercise: function () {
                return swappingExercise;
            },

            swapExercise: function (newExerciseInstance) {
                angular.forEach(workoutData.groups, function (group, groupIndex) {
                    angular.forEach(group.exercises, function (exercise, exerciseIndex) {
                        if (exercise.swapping) {
                            workoutData.groups[groupIndex].exercises[exerciseIndex].originalExercise = angular.copy(exercise);
                            workoutData.groups[groupIndex].exercises[exerciseIndex].meta = newExerciseInstance.meta;
                            workoutData.groups[groupIndex].exercises[exerciseIndex].exercise = newExerciseInstance.exercise;
                            delete workoutData.groups[groupIndex].exercises[exerciseIndex].swapping;
                        }
                    });
                });
            },

            resetExerciseSwap: function () {
                angular.forEach(workoutData.groups, function (group, groupIndex) {
                    angular.forEach(group.exercises, function (exercise, exerciseIndex) {
                        if (exercise.swapping) {
                            delete workoutData.groups[groupIndex].exercises[exerciseIndex].swapping;
                        }
                    });
                });
            },

            toggleSet: function (setObj, exerciseInstance) {

                if (!setObj.hasOwnProperty('isDone')) {
                    setObj.isDone = true;
                } else {
                    setObj.isDone = !setObj.isDone;
                }
                var doneSets = 0;
                if (setObj.isDone) {
                    if (!setObj.done.field_1 && setObj.current_target.hasOwnProperty('field_1')) {
                        setObj.done.field_1 = parseInt(setObj.current_target.field_1, 10);
                        setObj.field_1 = setObj.current_target.field_1;
                    }
                    if (!setObj.done.field_2 && setObj.current_target.hasOwnProperty('field_2')) {
                        setObj.done.field_2 = parseInt(setObj.current_target.field_2, 10);
                        setObj.field_2 = setObj.current_target.field_2;
                    }
                    if (!setObj.done.field_3 && setObj.current_target.hasOwnProperty('field_3')) {
                        setObj.done.field_3 = parseInt(setObj.current_target.field_3, 10);
                        setObj.field_3 = setObj.current_target.field_3;
                    }
                    if (!setObj.done.field_4 && setObj.current_target.hasOwnProperty('field_4')) {
                        setObj.done.field_4 = parseInt(setObj.current_target.field_4, 10);
                        setObj.field_4 = setObj.current_target.field_4;
                    }
                    if (!setObj.done.field_5 && setObj.current_target.hasOwnProperty('field_5')) {
                        setObj.done.field_5 = parseInt(setObj.current_target.field_5, 10);
                        setObj.field_5 = setObj.current_target.field_5;
                    }
                } else {
                    if (setObj.done.field_1) {
                        setObj.current_target.field_1 = setObj.field_1;
                    }
                    delete setObj.done.field_1;

                    if (setObj.done.field_2) {
                        setObj.current_target.field_2 = setObj.field_2;
                    }
                    delete setObj.done.field_2;

                    if (setObj.done.field_3) {
                        setObj.current_target.field_3 = setObj.field_3;
                    }
                    delete setObj.done.field_3;

                    if (setObj.done.field_4) {
                        setObj.current_target.field_4 = setObj.field_4;
                    }
                    delete setObj.done.field_4;

                    if (setObj.done.field_5) {
                        setObj.current_target.field_5 = setObj.field_5;
                    }
                    delete setObj.done.field_5;
                }
                angular.forEach(exerciseInstance.meta.sets, function (setObj) {
                    if (setObj.hasOwnProperty('isDone') && setObj.isDone) {
                        doneSets++;
                    }
                });

                exerciseInstance.meta.allSetsDone = exerciseInstance.meta.sets.length === doneSets;
            },

            toggleSets: function (exerciseInstance) {
                if (!exerciseInstance.meta.hasOwnProperty('allSetsDone')) {
                    exerciseInstance.meta.allSetsDone = true;
                } else {
                    exerciseInstance.meta.allSetsDone = !exerciseInstance.meta.allSetsDone;
                }
                angular.forEach(exerciseInstance.meta.sets, function (setObj) {
                    setObj.isDone = exerciseInstance.meta.allSetsDone;
                    if (setObj.isDone) {
                        if (!setObj.done.field_1 && setObj.current_target.hasOwnProperty('field_1')) {
                            setObj.done.field_1 = parseInt(setObj.current_target.field_1, 10);
                            setObj.field_1 = setObj.current_target.field_1;
                        }
                        if (!setObj.done.field_2 && setObj.current_target.hasOwnProperty('field_2')) {
                            setObj.done.field_2 = parseInt(setObj.current_target.field_2, 10);
                            setObj.field_2 = setObj.current_target.field_2;
                        }
                        if (!setObj.done.field_3 && setObj.current_target.hasOwnProperty('field_3')) {
                            setObj.done.field_3 = parseInt(setObj.current_target.field_3, 10);
                            setObj.field_3 = setObj.current_target.field_3;
                        }
                        if (!setObj.done.field_4 && setObj.current_target.hasOwnProperty('field_4')) {
                            setObj.done.field_4 = parseInt(setObj.current_target.field_4, 10);
                            setObj.field_4 = setObj.current_target.field_4;
                        }
                        if (!setObj.done.field_5 && setObj.current_target.hasOwnProperty('field_5')) {
                            setObj.done.field_5 = parseInt(setObj.current_target.field_5, 10);
                            setObj.field_5 = setObj.current_target.field_5;
                        }
                    } else {
                        if (setObj.done.field_1) {
                            setObj.current_target.field_1 = setObj.field_1;
                        }
                        delete setObj.done.field_1;

                        if (setObj.done.field_2) {
                            setObj.current_target.field_2 = setObj.field_2;
                        }
                        delete setObj.done.field_2;

                        if (setObj.done.field_3) {
                            setObj.current_target.field_3 = setObj.field_3;
                        }
                        delete setObj.done.field_3;

                        if (setObj.done.field_4) {
                            setObj.current_target.field_4 = setObj.field_4;
                        }
                        delete setObj.done.field_4;

                        if (setObj.done.field_5) {
                            setObj.current_target.field_5 = setObj.field_5;
                        }
                        delete setObj.done.field_5;
                    }
                });
            },

            addSet: function (exerciseInstance) {
                if (exerciseInstance.meta.sets.length > 0) {

                    var setObj = angular.copy(exerciseInstance.meta.sets[exerciseInstance.meta.sets.length - 1]);

                    if (setObj.hasOwnProperty('done')) {
                        setObj.done = {};
                    }

                    setObj.isDone = false;
                    exerciseInstance.meta.allSetsDone = false;

                    exerciseInstance.meta.sets.push(setObj);
                    return true;
                }

                exerciseInstance.meta.sets.push(angular.copy(emptySet));

                return true;
            },

            addIntervalRound: function (exerciseInstance) {

                var existingStates = {};
                angular.forEach(exerciseInstance.meta.sets, function (setObj) {
                    existingStates[setObj.field_1] = setObj;
                });

                var newSet;

                if (existingStates.hasOwnProperty('1')) {
                    newSet = angular.copy(existingStates['1']);
                    newSet.done = {};
                } else {
                    newSet = angular.copy(emptySet);
                    newSet.field_1 = 1;
                }
                exerciseInstance.meta.sets.push(newSet);

                if (existingStates.hasOwnProperty('2')) {
                    newSet = angular.copy(existingStates['2']);
                    newSet.done = {};
                } else {
                    newSet = angular.copy(emptySet);
                    newSet.field_1 = 2;
                }
                exerciseInstance.meta.sets.push(newSet);

                if (existingStates.hasOwnProperty('3')) {
                    newSet = angular.copy(existingStates['3']);
                    newSet.done = {};
                } else {
                    newSet = angular.copy(emptySet);
                    newSet.field_1 = 3;
                }
                exerciseInstance.meta.sets.push(newSet);

                return exerciseInstance;
            },

            removeSet: function (set, setNum, exerciseInstance) {
                return ConfirmDialogService.open({
                    title: "Remove Set",
                    message: "Do you want to remove set #" + setNum + " from " + exerciseInstance.exercise.name + "?",
                    buttonConfirm: "Confirm",
                    buttonCancel: "Cancel"
                }).then(function () {
                    var setIndex = exerciseInstance.meta.sets.indexOf(set);
                    exerciseInstance.meta.sets.splice(setIndex, 1);
                });
            },

            removeExercise: function (exerciseInstance, group) {
                return ConfirmDialogService.open({
                    title: "Remove Exercise",
                    message: "Do you want to remove " + exerciseInstance.exercise.name + "?",
                    buttonConfirm: "Confirm",
                    buttonCancel: "Cancel"
                }).then(function () {

                    var exerciseIndex = group.exercises.indexOf(exerciseIndex);
                    group.exercises.splice(exerciseIndex, 1);

                    // if we deleted from a superset, remaining exercise
                    // becomes standalone, if we deleted from a giantset then
                    // it becomes a superset.
                    var newType = 'standalone';
                    if (group.exercises.length > 2) {
                        newType = 'giantset';
                    } else if (group.exercises.length > 1) {
                        newType = 'superset';
                    }
                    group.group_type = newType;

                    // if we deleted a standalone exercise, the group left behind needs to be removed
                    if (group.exercises.length === 0) {
                        var groupIndex = workoutData.groups.indexOf(group);
                        workoutData.groups.splice(groupIndex, 1);
                    }
                });
            },

            addStandardExercise: function (exercise) {

                var instance = {
                    exercise: exercise,
                    meta: {}
                };

                var set = angular.copy(emptySet);

                var fields = [];

                angular.forEach(exercise.default_cols, function (col, key) {

                    //set[col.name] = col.initial_value;

                    fields.push(col);

                    instance.meta[col.name] = col.type;

                    if (col.type === 'speed') {
                        instance.meta[col.name + '_label'] = defaultLabels.speed;
                    }

                    if (col.type === 'weight') {
                        instance.meta[col.name + '_label'] = defaultLabels.weight;
                    }

                    if (col.type === 'distance') {
                        instance.meta[col.name + '_label'] = defaultLabels.distance;
                    }
                });

                instance.meta.availableFieldTypes = exercise.availableFieldTypes;
                instance.meta.sets = [set];
                instance.meta.set_fields = fields;

                instance.meta = filterAvailableFieldTypes(instance.meta);

                workoutData.groups.push({
                    group_type: 'standalone',
                    exercises: [instance]
                });
            },

            addCardioExercise: function (exercise, cardioType) {

                var instance = {
                    exercise: exercise,
                    meta: {}
                };

                instance.meta.type = cardioType;

                var fields = [];

                var set = angular.copy(emptySet);

                angular.forEach(exercise.default_cols[instance.meta.type], function (col, key) {

                    set[col.name] = col.initial_value;

                    fields.push(col);

                    instance.meta[col.name] = col.type;

                    if (col.type === 'speed') {
                        instance.meta[col.name + '_label'] = defaultLabels.speed;
                    }

                    if (col.type === 'weight') {
                        instance.meta[col.name + '_label'] = defaultLabels.weight;
                    }

                    if (col.type === 'distance') {
                        instance.meta[col.name + '_label'] = defaultLabels.distance;
                    }
                });

                instance.meta.availableFieldTypes = exercise.availableFieldTypes[instance.meta.type];

                if (instance.meta.type === 'interval') {

                    instance.meta.sets = [];

                    set = angular.copy(set);
                    set.field_1 = 1;
                    instance.meta.sets.push(set);

                    set = angular.copy(set);
                    set.field_1 = 2;
                    instance.meta.sets.push(set);

                    set = angular.copy(set);
                    set.field_1 = 3;
                    instance.meta.sets.push(set);
                } else {

                    instance.meta.sets = [set];
                }

                instance.meta.set_fields = fields;

                instance.meta = filterAvailableFieldTypes(instance.meta);

                workoutData.groups.push({
                    group_type: 'standalone',
                    exercises: [instance]
                });
            },

            setWorkoutTitle: function (title) {
                if (!workoutData) {
                    return false;
                }

                workoutData.title = title;
                return true;
            },

            triggerFieldSwitcher: function (exerciseInstance, fieldToChange) {

                if (!fieldToChange.editable) {
                    return false;
                }

                exerciseInstance.meta = filterAvailableFieldTypes(exerciseInstance.meta);

                var availableTypes = [];
                var length = exerciseInstance.meta.availableFieldTypesFiltered[fieldToChange.name].length;
                for (var i = 0; i < length; i++) {

                    var availableFieldType = exerciseInstance.meta.availableFieldTypesFiltered[fieldToChange.name][i];

                    if (fieldLabels.hasOwnProperty(availableFieldType.type)) {

                        var zlength = fieldLabels[availableFieldType.type].length;

                        for (var z = 0; z < zlength; z++) {
                            var label = fieldLabels[availableFieldType.type][z];
                            availableTypes.push({
                                text: availableFieldType.name + ' - ' + label,
                                label: label,
                                field: availableFieldType
                            });
                        }
                    } else {

                        availableTypes.push({
                            text: availableFieldType.name,
                            label: null,
                            field: availableFieldType
                        });
                    }
                }

                $ionicActionSheet.show({
                    titleText: 'Change field type',
                    buttons: availableTypes,
                    cancelText: 'Cancel',
                    buttonClicked: function (index) {

                        var newField = availableTypes[index];

                        fieldToChange.type = newField.field.type;
                        fieldToChange.edited = true;
                        exerciseInstance.meta[fieldToChange.name] = newField.field.type;
                        exerciseInstance.meta[fieldToChange.name + '_label'] = newField.label !== null ? newField.label : null;

                        return true;
                    }
                });
            },

            updateGroupSorts: function () {
                var length = workoutData.groups.length;
                for (var i = 0; i < length; i++) {
                    workoutData.groups[i].sort = i;
                }
            },

            breakGroupApart: function (baseGroup) {
                var newgroups = [];

                // Create new group for each exercise in this group
                var length = baseGroup.exercises.length;
                for (var i = 0; i < length; i++) {
                    newgroups.push({
                        group_type: 'standalone',
                        exercises: [baseGroup.exercises[i]]
                    });
                }

                var groupKey = workoutData.groups.indexOf(baseGroup);

                if (groupKey === -1) {
                    return false;
                }

                // Splice out the original group
                workoutData.groups.splice(groupKey, 1);

                // Splice the groups into where the existing multi-set group is
                Array.prototype.splice.apply(workoutData.groups, [groupKey, 0].concat(newgroups));

                this.updateGroupSorts();
            },

            completeSession: function (feedback, useDate, billable, billableToQuota, saveAsWorkout, templateName) {
                billable = billable || false;
                billableToQuota = billableToQuota || false;
                saveAsWorkout = saveAsWorkout || false;
                templateName = templateName || false;

                var self = this;

                if (workoutData && workoutData.groups.length > 0) {
                    var workout = {
                        title: workoutData.title,
                        note: workoutData.note,
                        groups: []
                    };

                    if (session.workoutUsedAsTemplate) {
                        workout.original_workout_id = session.workout.id;
                    }

                    workoutData.groups.map(function (group) {

                        var exercises = [];

                        group.exercises.map(function (exerciseItem) {

                            var exercise;

                            if (exerciseItem.hasOwnProperty('originalExercise')) {

                                exercise = {
                                    name_id: exerciseItem.originalExercise.exercise.name_id,
                                    exercise_id: exerciseItem.originalExercise.exercise.id,
                                    field_1: exerciseItem.originalExercise.meta.field_1,
                                    field_1_label: exerciseItem.originalExercise.meta.field_1_label,
                                    field_2: exerciseItem.originalExercise.meta.field_2,
                                    field_2_label: exerciseItem.originalExercise.meta.field_2_label,
                                    field_3: exerciseItem.originalExercise.meta.field_3,
                                    field_3_label: exerciseItem.originalExercise.meta.field_3_label,
                                    field_4: exerciseItem.originalExercise.meta.field_4,
                                    field_4_label: exerciseItem.originalExercise.meta.field_4_label,
                                    field_5: exerciseItem.originalExercise.meta.field_5,
                                    field_5_label: exerciseItem.originalExercise.meta.field_5_label,
                                    sets: [],
                                    substitute: {
                                        name_id: exerciseItem.exercise.name_id,
                                        exercise_id: exerciseItem.exercise.id,
                                        field_1: exerciseItem.meta.field_1,
                                        field_1_label: exerciseItem.meta.field_1_label,
                                        field_2: exerciseItem.meta.field_2,
                                        field_2_label: exerciseItem.meta.field_2_label,
                                        field_3: exerciseItem.meta.field_3,
                                        field_3_label: exerciseItem.meta.field_3_label,
                                        field_4: exerciseItem.meta.field_4,
                                        field_4_label: exerciseItem.meta.field_4_label,
                                        field_5: exerciseItem.meta.field_5,
                                        field_5_label: exerciseItem.meta.field_5_label,
                                        sets: []
                                    }
                                };

                                exerciseItem.originalExercise.meta.sets.map(function (set, i) {
                                    var newSet = {
                                        field_1: set.field_1,
                                        field_2: set.field_2,
                                        field_3: set.field_3,
                                        field_4: set.field_4,
                                        field_5: set.field_5
                                    };

                                    exercise.sets.push(newSet);
                                });

                                exerciseItem.meta.sets.map(function (set, i) {
                                    var newSet = {
                                        field_1: set.field_1,
                                        field_2: set.field_2,
                                        field_3: set.field_3,
                                        field_4: set.field_4,
                                        field_5: set.field_5
                                    };

                                    if (set.isDone) {
                                        if (set.hasOwnProperty('done')) {
                                            newSet.done = {
                                                field_1: set.done.field_1,
                                                field_2: set.done.field_2,
                                                field_3: set.done.field_3,
                                                field_4: set.done.field_4,
                                                field_5: set.done.field_5
                                            };
                                        } else {
                                            newSet.done = {
                                                field_1: set.field_1,
                                                field_2: set.field_2,
                                                field_3: set.field_3,
                                                field_4: set.field_4,
                                                field_5: set.field_5
                                            };
                                        }
                                    }

                                    exercise.substitute.sets.push(newSet);
                                });
                            } else {

                                exercise = {
                                    name_id: exerciseItem.exercise.name_id,
                                    exercise_id: exerciseItem.exercise.id,
                                    field_1: exerciseItem.meta.field_1,
                                    field_1_label: exerciseItem.meta.field_1_label,
                                    field_2: exerciseItem.meta.field_2,
                                    field_2_label: exerciseItem.meta.field_2_label,
                                    field_3: exerciseItem.meta.field_3,
                                    field_3_label: exerciseItem.meta.field_3_label,
                                    field_4: exerciseItem.meta.field_4,
                                    field_4_label: exerciseItem.meta.field_4_label,
                                    field_5: exerciseItem.meta.field_5,
                                    field_5_label: exerciseItem.meta.field_5_label,
                                    sets: []
                                };

                                exerciseItem.meta.sets.map(function (set, i) {

                                    var newSet = {
                                        field_1: set.field_1,
                                        field_2: set.field_2,
                                        field_3: set.field_3,
                                        field_4: set.field_4,
                                        field_5: set.field_5
                                    };

                                    if (set.isDone) {
                                        newSet.done = {
                                            field_1: set.done.field_1,
                                            field_2: set.done.field_2,
                                            field_3: set.done.field_3,
                                            field_4: set.done.field_4,
                                            field_5: set.done.field_5
                                        };
                                    }
                                    exercise.sets.push(newSet);
                                });
                            }

                            exercises.push(exercise);
                        });

                        workout.groups.push({
                            'note': group.note,
                            'group_type': group.group_type,
                            'sort': group.sort,
                            'exercises': exercises
                        });
                    });
                }

                var date = session.date.split('+').join(' ');

                if (useDate) {
                    date = moment(useDate).utc().format('YYYY-MM-DD HH:mm:ss');
                }

                var elapsed = false;
                if (startTime > 0) {
                    elapsed = Math.floor((Date.now() - startTime) / 1000);
                }

                elapsed = elapsed || false;

                return $q(function (resolve, reject) {

                    var data = {
                        'date_completed': date,
                        'feedback': feedback
                    };

                    if (workout) {
                        data.workout = workout;
                    }

                    if (elapsed > 0) {
                        data.time_elapsed = elapsed;
                    }

                    if (elapsed > 0) {
                        data.time_elapsed = elapsed;
                    }

                    if (session.phantom) {
                        data.occurrence_date = session.date.split('+').join(' ');
                    }

                    if (billable) {
                        data.billable = billable;
                    }

                    if (billableToQuota) {
                        data.billable_to_quota = billableToQuota;
                    }

                    if (saveAsWorkout) {
                        data.save_as_workout = saveAsWorkout;
                        if (templateName) {
                            data.save_as_workout_title = templateName;
                        }
                    }

                    SessionStoreService.save(session, data).then(function (response) {

                        self.reset();

                        resolve(response);
                    }, function (error) {

                        console.log('ERROR: unable to store session log: ' + error);

                        reject(error);
                    });
                });
            },

            quit: function () {
                return ConfirmDialogService.open({
                    title: "Are you sure?",
                    message: "Please confirm that you want to quit this session - Any data logged during the session will be cleared.",
                    buttonConfirm: "Quit Session",
                    buttonCancel: "Cancel"
                }).then(function () {
                    service.inactive();
                    if (isAdhoc) {
                        SessionStoreService.deleteLocalSession(session.id);
                    }
                    service.reset();
                    NavigateService.replaceCurrentState().goHome();
                });
            },

            active: function () {
                loggerActive = true;
                ScreenStateService.stayOn();
            },

            inactive: function () {
                loggerActive = false;
                ScreenStateService.dontStayOn();
            },

            isActive: function () {
                return loggerActive;
            }

        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('ScheduleEventService', service);

    service.$inject = ['SqlStorageService', 'SessionStoreService', '$q'];

    function service(SqlStorageService, ScheduleEventStoreService, $q) {
        var service = {

            getCustomerEvents: function (dateStart, dateEnd, customerId) {

                var values = [customerId, dateStart.format('YYYY-MM-DD'), dateEnd.format('YYYY-MM-DD')];

                return $q(function (resolve, reject) {

                    SqlStorageService.selectAll('SELECT * FROM schedule_events WHERE customer_id = ? AND date >= ? AND date <= ?', values).then(function (results) {
                        resolve(ScheduleEventStoreService.parseRows(results));
                    }, function () {
                        reject();
                    });
                });
            },

            getCustomerEventsOnDate: function (date, customerId, type) {

                type = type || false;

                var query = 'SELECT * FROM schedule_events WHERE customer_id = ? AND date = ?';

                var values = [customerId, date];

                if (type) {
                    if (typeof type === 'string') {
                        values.push(type);
                        query += ' AND type = ?';
                    } else {
                        query += ' AND type IN(';
                        for (var i = 0; i < type.length; i++) {
                            values.push(type[i]);
                            query += (i > 0 ? ',' : '') + '?';
                        }
                        query += ')';
                    }
                }

                return $q(function (resolve, reject) {

                    SqlStorageService.selectAll(query, values).then(function (results) {
                        resolve(ScheduleEventStoreService.parseRows(results));
                    }, function () {
                        reject();
                    });
                });
            }

        };

        return service;
    }
})();
(function () {

    angular.module('MyPTHub').factory('ScheduleEventStoreService', service);

    service.$inject = ['ApiClientService', 'NetworkStateService', 'SqlStorageService', 'PermissionsService', '$q', 'AuthenticationService', '$rootScope', 'AuthenticationStateService'];

    function service(ApiClientService, NetworkStateService, SqlStorageService, PermissionsService, $q, AuthenticationService, $rootScope, AuthenticationStateService) {

        var downloading = false,
            dirty = true,
            lastCustomerId;

        var parseRow = function (row) {
            row.data = JSON.parse(row.data);
            return row;
        };

        var parseRows = function (rows) {
            return rows.map(parseRow);
        };

        var fireEventHandlers = function (type) {
            $rootScope.$emit('scheduleEventStore_' + type);
        };

        var registerEventHandler = function (scope, type, handler) {

            var unsub = $rootScope.$on('scheduleEventStore_' + type, handler);

            scope.$on('$destroy', unsub);

            return unsub;
        };

        var getOffline = function (customerId, scheduleId, date, type) {
            return $q(function (resolve, reject) {
                SqlStorageService.select('SELECT * FROM schedule_events WHERE customer_id = ? AND schedule_id = ? AND date = ? AND type = ?', [customerId, scheduleId, date, type]).then(function (result) {

                    resolve(parseRow(result));
                }, function (error) {
                    reject(false);
                });
            });
        };

        var storeEvent = function (event, customer) {
            return $q(function (resolve, reject) {
                SqlStorageService.query("INSERT INTO schedule_events (customer_id, trainer_id, trainer_user_id, schedule_id, data, date, type) " + "VALUES(?, ?, ?, ?, ?, ?, ?)", [customer.id, customer.trainer_id, customer.trainer_user_id, event.schedule_id, JSON.stringify(event), event.date, event.type]).then(function () {
                    getOffline(customer.id, event.schedule_id, event.date, event.type).then(resolve, reject);
                }, function (response) {
                    console.log('store insert fail', response);
                    reject(response);
                });
            });
        };

        var clearStoredForCustomer = function (customerId) {
            return SqlStorageService.query("DELETE FROM schedule_events WHERE customer_id = ?", [customerId]);
        };

        var service = {

            setDirty: function () {
                dirty = true;
            },

            isDirty: function (customerId) {
                return customerId === lastCustomerId && dirty;
            },

            setup: function () {

                var download = function (resolve) {
                    // Download events
                    if (PermissionsService.userIsCustomer()) {
                        service.downloadCustomerEvents(PermissionsService.getCurrentUserEntityId()).finally(resolve);
                    }
                };

                /**
                 * Trigger when login successful
                 */
                AuthenticationService.registerLoginSuccessEventHandler(download);

                if (AuthenticationStateService.isLoggedIn()) {
                    download();
                }
            },

            downloadCustomerEvents: function (customerId) {

                console.log('downloadCustomerEvents');

                downloading = true;

                var deferred = $q.defer();

                if (!NetworkStateService.isOnline()) {
                    deferred.reject();
                    return deferred.promise;
                }

                ApiClientService.noCache().customers.getSchedulerEvents(customerId).success(function (response) {

                    clearStoredForCustomer(customerId).then(function () {

                        var stored = [];

                        angular.forEach(response.events, function (event) {
                            stored.push(storeEvent(event, response.customer));
                        });

                        $q.all(stored).then(function (storedResponse) {
                            deferred.resolve(storedResponse);
                            dirty = false;
                            lastCustomerId = customerId;
                            fireEventHandlers('download_complete');
                        }, function (responses) {
                            console.log('store events failed', responses);
                        });
                    });
                }).error(deferred.reject);

                deferred.promise.finally(function () {
                    downloading = false;
                });

                return deferred.promise;
            },

            downloadInProgress: function () {
                return downloading;
            },

            registerEventHandler: registerEventHandler,

            parseRow: parseRow,

            parseRows: parseRows

        };

        return service;
    }
})();
(() => {
  angular.module('MyPTHub').factory('SessionScheduleService', service);

  service.$inject = ['SqlStorageService', 'PermissionsService', 'SessionStoreService', '$q'];

  function service(SqlStorageService, PermissionsService, SessionStoreService, $q) {
    const service = {

      getTrainerSchedule(dateStart, dateEnd, trainerUserId = false) {
        if (!trainerUserId) {
          trainerUserId = PermissionsService.getLoggedInUserId();
        }

        const values = [PermissionsService.getCurrentUserEntityId(), trainerUserId, dateStart.utc().format('YYYY-MM-DD HH:mm:ss'), dateEnd.utc().format('YYYY-MM-DD HH:mm:ss')];

        return $q((resolve, reject) => {
          SqlStorageService.selectAll('SELECT * FROM sessions WHERE trainer_id = ? AND trainer_user_id = ? AND date >= ? AND date <= ?', values).then(results => {
            resolve(SessionStoreService.parseRows(results));
          }, error => {
            console.error(error);
            reject();
          });
        });
      },

      getCustomerSchedule(dateStart, dateEnd, customerId) {
        const values = [customerId, dateStart.utc().format('YYYY-MM-DD HH:mm:ss'), dateEnd.utc().format('YYYY-MM-DD HH:mm:ss')];

        return $q((resolve, reject) => {
          SqlStorageService.selectAll('SELECT * FROM sessions WHERE customer_id = ? AND date >= ? AND date <= ?', values).then(results => {
            resolve(SessionStoreService.parseRows(results));
          }, () => {
            reject();
          });
        });
      }

    };

    return service;
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('SessionStartService', service);

    service.$inject = ['SessionStoreService', 'NavigateService', 'SessionLoggerService', 'ToastService'];

    function service(SessionStoreService, NavigateService, SessionLoggerService, ToastService) {

        var start = function (sessionId, adhoc) {

            adhoc = adhoc || false;

            console.log('Session to start:', sessionId);

            var check = function (session) {

                var error = false;

                // Detect already completed
                if (session.completed === 1) {
                    error = 'This session has already been completed';
                }

                if (error) {

                    ToastService.show(error);

                    return false;
                }

                // Redirect consultation
                if (session.data.workout_id === null && !adhoc) {

                    NavigateService.disableAnimation().replaceCurrentState().go('app.session_logger_consultation', {
                        'sessionId': sessionId
                    });

                    return false;
                }

                return true;
            };

            SessionStoreService.get(sessionId).then(function (session) {

                SessionLoggerService.reset();

                SessionLoggerService.setSession(session.data, adhoc);

                if (!check(session)) {
                    return;
                }

                NavigateService.replaceCurrentState().go('app.log_session', {
                    'sessionId': sessionId
                });
            }, function (err) {

                console.log('API ERROR: unable to get session: ' + err);

                ToastService.show('Unable to connect - please try again later');
            });
        };

        var startAdHoc = function (workoutId, customerId, useDate, scheduleId, scheduleType) {
            workoutId = workoutId || null;
            scheduleId = scheduleId || null;
            scheduleType = scheduleType || null;

            return SessionStoreService.createAdHoc(workoutId, customerId, useDate, scheduleId, scheduleType).then(function (session) {

                start(session.id, true);
            });
        };

        return {
            start: start,
            startAdHoc: startAdHoc
        };
    }
})();
(function () {

    angular.module('MyPTHub').factory('SessionStoreService', service);

    service.$inject = ['ApiClientService', 'NetworkStateService', 'SqlStorageService', 'PermissionsService', '$q', 'AuthenticationStateService', '$interval', 'AuthenticationService', 'WorkoutStoreService', '$rootScope', 'CustomerStoreService', 'AccountSwapperService'];

    function service(ApiClientService, NetworkStateService, SqlStorageService, PermissionsService, $q, AuthenticationStateService, $interval, AuthenticationService, WorkoutStoreService, $rootScope, CustomerStoreService, AccountSwapperService) {

        var syncing = false;
        var dirty = true;

        var fireEventHandlers = function (type) {
            $rootScope.$emit('sessionStore_' + type);
        };

        var registerEventHandler = function (scope, type, handler) {

            var unsub = $rootScope.$on('sessionStore_' + type, handler);

            scope.$on('$destroy', unsub);

            return unsub;
        };

        var parseRow = function (row) {

            row.data = JSON.parse(row.data);

            return row;
        };

        var parseRows = function (rows) {
            return rows.map(parseRow);
        };

        var generateId = function (session) {
            if (session.phantom) {

                var date = session.date;
                var id;

                if (session.parent_session_id) {
                    id = session.parent_session_id;
                } else {
                    id = session.id;
                }

                return id + '_' + date.split(' ').join('+');
            } else {
                return session.id;
            }
        };

        var getOffline = function (sessionId) {

            sessionId = typeof sessionId !== 'string' ? sessionId.toString() : sessionId;

            return $q(function (resolve, reject) {

                SqlStorageService.select('SELECT * FROM sessions WHERE id = ?', [sessionId]).then(function (result) {

                    resolve(parseRow(result));
                }, function (error) {
                    reject(false);
                });
            });
        };

        var storeSession = function (session, adhoc) {

            adhoc = adhoc || false;

            return $q(function (resolve, reject) {

                var sessionId = generateId(session);
                sessionId = typeof sessionId !== 'string' ? sessionId.toString() : sessionId;

                getOffline(sessionId).then(function (current) {

                    SqlStorageService.query("UPDATE sessions SET data = ?, completed = ?, date = ?, last_data_update = datetime('now') WHERE id = ?", [JSON.stringify(session), session.status === 'finished' && !session.phantom || current.payload ? 1 : 0, session.date, sessionId]);

                    getOffline(sessionId).then(resolve);
                }, function () {
                    SqlStorageService.query("INSERT INTO sessions (id, customer_id, trainer_id, trainer_user_id, data, date, completed, synced, adhoc, last_data_update) VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, datetime('now'))", [sessionId.toString(), session.customer_id, session.trainer_id, session.trainer_user_id, JSON.stringify(session), session.date, session.status === 'finished' && !session.phantom ? 1 : 0, 1, adhoc ? 1 : 0]);

                    getOffline(sessionId).then(resolve);
                });
            });
        };

        var getLive = function (sessionId) {

            return $q(function (resolve, reject) {

                if (!NetworkStateService.isOnline()) {
                    reject(false);
                    return;
                }

                // Normal  = id
                // Phantom = id_date
                var sessionIdParts = String(sessionId).split('_');
                if (sessionIdParts.length === 1) {
                    var getSession = ApiClientService.noCache().sessions.get(sessionIdParts[0]);
                } else if (sessionIdParts.length === 2) {
                    var getSession = ApiClientService.noCache().sessions.get(sessionIdParts[0], sessionIdParts[1]);
                } else {
                    // Invalid ID
                    reject(false);
                    return;
                }

                getSession.success(function (response) {
                    storeSession(response.session).then(resolve);
                });

                getSession.error(function (response) {
                    reject(response);
                });
            });
        };

        var storePayload = function (sessionId, payload) {
            return SqlStorageService.query("UPDATE sessions SET payload = ?, completed = 1, synced = 0 WHERE id = ?", [JSON.stringify(payload), sessionId.toString()]);
        };

        var setLastSyncAttempt = function (sessionId) {
            SqlStorageService.query("UPDATE sessions SET last_sync_attempt = datetime('now'), sync_fail_count = sync_fail_count + 1 WHERE id = ?", [sessionId.toString()]).then(function () {}, function (err) {
                console.log('set last sync attempt query error: ', err);
            });
        };

        var changeId = function (currentId, newId) {

            currentId = typeof currentId !== 'string' ? currentId.toString() : currentId;
            newId = typeof newId !== 'string' ? newId.toString() : newId;

            return $q(function (resolve, reject) {

                SqlStorageService.query("UPDATE sessions SET id = ?, adhoc = 0 WHERE id = ?", [newId, currentId]).then(resolve, reject);
            });
        };

        var saveLive = function (session) {

            var deferred = $q.defer();

            var localId = generateId(session);
            localId = typeof localId !== 'string' ? localId.toString() : localId;

            var doSave = function (id, payload) {

                id = typeof id !== 'string' ? id.toString() : id;

                return $q(function (resolve, reject) {

                    ApiClientService.sessions.completeWithAdhocWorkout(id, payload).then(function (response) {

                        SqlStorageService.query("UPDATE sessions SET synced = 1, adhoc = 0 WHERE id = ?", [id]);

                        getLive(id);

                        resolve(response);
                    }, function (response) {
                        console.log('completeWithAdhocWorkout fail:', response);
                        reject();
                    });
                });
            };

            SqlStorageService.select('SELECT payload, data, adhoc, customer_id, date, trainer_user_id FROM sessions WHERE id = ? AND synced = 0', [localId]).then(function (currentLocal) {

                var payload = JSON.parse(currentLocal.payload);
                var sessionData = JSON.parse(currentLocal.data);

                if (!payload) {
                    deferred.reject(false);
                    return;
                }

                if (currentLocal.adhoc === 1) {

                    // This was not a pre-booked session
                    var scheduleId = sessionData.schedule_id,
                        scheduleType = sessionData.schedule_type;

                    ApiClientService.sessions.book(sessionData.workout_id, currentLocal.customer_id, currentLocal.date, true, currentLocal.trainer_user_id, false, false, scheduleId, scheduleType).then(function (response) {

                        changeId(localId, response.data.session.id).then(function (idChangeResponse) {

                            doSave(response.data.session.id, payload).then(deferred.resolve, deferred.reject);
                        }, deferred.reject);
                    }, deferred.reject);
                } else {

                    // Completing a pre-booked session

                    doSave(localId, payload).then(deferred.resolve, deferred.reject);
                }
            }, function () {

                deferred.reject(false);
            });

            return deferred.promise;
        };

        var getUnSyncedSessions = function (earliestSyncDateFirst, limitRetry) {

            earliestSyncDateFirst = earliestSyncDateFirst || false;
            limitRetry = limitRetry || false;

            var user = PermissionsService.getCurrentUser();

            var whereField;
            var whereValue;

            if (PermissionsService.userIsTrainer()) {

                whereField = 'trainer_user_id';
                whereValue = user.id;
            } else {

                whereField = 'customer_id';
                whereValue = user.customer_id;
            }

            var limitRetryWhere = '';
            if (limitRetry) {
                limitRetryWhere = 'sync_fail_count < 8 AND ';
            }

            var sortQuery = '';
            if (earliestSyncDateFirst) {
                sortQuery = ' ORDER BY CASE WHEN last_sync_attempt IS NULL THEN 0 ELSE 1 END, last_sync_attempt ASC';
            }

            // Get all unsysnced sessions, last sync date ascending.
            return SqlStorageService.selectAll('SELECT * FROM sessions WHERE ' + limitRetryWhere + whereField + ' = ? AND synced = 0 AND completed = 1' + sortQuery, [whereValue]);
        };

        var deleteLive = function (sessionId, reason) {

            return ApiClientService.sessions.cancel(sessionId, reason);
        };

        var deleteLocal = function (sessionId) {

            return SqlStorageService.query("DELETE FROM sessions WHERE id = ?", [sessionId]);
        };

        var cleanupRemotelyDeleted = function (ids, trainerUserId, customerId, dateFrom, dateTo) {

            var deferred = $q.defer();

            angular.forEach(ids, function (id, idKey) {
                ids[idKey] = '"' + id + '"';
            });

            var whereFields = 'id NOT IN (' + ids.join(', ') + ') ';

            var wheres = [];

            if (trainerUserId) {

                wheres.push(PermissionsService.getCurrentUserEntityId());

                wheres.push(trainerUserId);

                whereFields += ' AND trainer_id = ? AND trainer_user_id = ?';
            }

            if (customerId) {

                wheres.push(customerId);

                whereFields += ' AND customer_id = ?';
            }

            whereFields += ' AND (date BETWEEN DATE(?) AND DATE(?))';

            wheres.push(dateFrom);
            wheres.push(dateTo);

            var query = 'SELECT id FROM sessions ' + 'WHERE synced = 1 ' + 'AND ' + whereFields;

            SqlStorageService.selectAll(query, wheres).then(function (results) {

                var deletes = [];

                results.forEach(function (item) {
                    deletes.push(deleteLocal(item.id));
                });

                $q.all(deletes).finally(deferred.resolve);
            }, deferred.reject);

            return deferred.promise;
        };

        var service = {

            setDirty: function () {
                dirty = true;
            },

            dirty: function () {
                return dirty;
            },

            setup: function () {

                /**
                 * Schedule offline session push every x minutes when online.
                 */

                var intervalMins = 1;

                $interval(function () {
                    service.pushOfflineSessions(true);
                }, intervalMins * (60 * 1000));

                var sync = function (resolve) {

                    // Post any sessions that were completed offline to API
                    service.pushOfflineSessions();

                    // Download latest schedule data
                    if (PermissionsService.userIsTrainer()) {

                        service.downloadTrainerSessions().finally(resolve);
                    } else {

                        service.downloadCustomerSessions(PermissionsService.getCurrentUserEntityId()).finally(resolve);
                    }
                };

                /**
                 * Trigger when login successful
                 */
                AuthenticationService.registerLoginSuccessEventHandler(sync);

                if (AuthenticationStateService.isLoggedIn()) {
                    sync();
                }
            },

            // Gets session from local DB and updates it if we are online and
            // the session is not ad-hoc.
            get: function (sessionId) {

                var deferred = $q.defer();

                getOffline(sessionId).then(function (offlineVersion) {

                    if (offlineVersion.adhoc === 1) {

                        deferred.resolve(offlineVersion);
                    } else {

                        getLive(sessionId).then(deferred.resolve, deferred.reject);
                    }
                }, function () {

                    getLive(sessionId).then(deferred.resolve, deferred.reject);
                });

                return deferred.promise;
            },

            createAdHoc: function (workoutId, customerId, useDate, scheduleId, scheduleType) {
                scheduleId = scheduleId || null;
                scheduleType = scheduleType || null;

                var store = function (trainerId, trainerUserId, workout) {

                    workout = workout || null;

                    var data = {
                        id: 'local_' + moment().unix(),
                        customer_id: customerId,
                        trainer_id: trainerId,
                        trainer_user_id: trainerUserId,
                        date: date.utc().format('YYYY-MM-DD HH:mm:ss'),
                        status: 'live',
                        workout_id: workoutId,
                        workout: workout,
                        schedule_id: scheduleId,
                        schedule_type: scheduleType
                    };

                    if (PermissionsService.userIsTrainer()) {
                        return CustomerStoreService.get(customerId).then(function (customer) {

                            data.customer = customer;

                            return storeSession(data, true);
                        });
                    } else {

                        return storeSession(data, true);
                    }
                };

                workoutId = workoutId || null;

                var date;

                if (useDate) {
                    date = useDate;
                } else {
                    date = moment();
                }

                return $q(function (resolve, reject) {

                    if (workoutId) {

                        WorkoutStoreService.get(workoutId).then(function (workout) {
                            store(workout.trainer_id, workout.trainer_user_id, workout.data).then(resolve, reject);
                        });
                    } else {

                        var trainerId;
                        var trainerUserId;

                        if (PermissionsService.userIsTrainer()) {

                            trainerId = PermissionsService.getCurrentUserEntityId();
                            trainerUserId = PermissionsService.getLoggedInUserId();
                        } else {

                            var customer = PermissionsService.getCurrentUser();

                            trainerId = customer.trainer_id;
                            trainerUserId = customer.trainer_user_id;
                        }

                        store(trainerId, trainerUserId).then(resolve, reject);
                    }
                });
            },

            // Saves a completed session to local DB (and tries to publish online if it can)
            save: function (session, payload) {

                var deferred = $q.defer();

                var localStorageId = generateId(session);

                storePayload(localStorageId, payload).then(function () {

                    //if we're online, sync it to the api now
                    if (NetworkStateService.isOnline()) {

                        saveLive(session).then(function (response) {

                            deferred.resolve(response.data);
                            dirty = true;

                            fireEventHandlers('online_session_synced');
                        }, function (error) {

                            deferred.resolve(false);
                        });
                    } else {

                        deferred.resolve(false);

                        console.log('Failed due to offline.');
                    }
                }, function () {

                    console.log('Store payload failed.');

                    deferred.reject(true);
                });

                return deferred.promise;
            },

            // Sends previously un-successfully saved sessions to the API.
            pushOfflineSessions: function (limitRetries) {

                limitRetries = limitRetries || false;

                if (!NetworkStateService.isOnline() || !AuthenticationStateService.isLoggedIn() || syncing) {
                    console.log('Session sync not started');
                    return false;
                }

                console.log('Session sync triggered.');

                ApiClientService.customerTrainers.accountSwapperChecker(AuthenticationStateService.getCurrentUser()).success(response => {
                    if (response.performChange === true) {
                        AccountSwapperService.UpdateUser();
                    }
                }).error(function (error) {});

                syncing = true;

                fireEventHandlers('offline_sessions_syncing');

                getUnSyncedSessions(true, limitRetries).then(function (results) {

                    var processItem = function (session) {

                        return $q(function (resolve, reject) {

                            saveLive(session).then(function (response) {

                                console.log('Successfully synced offline session: ', response, session);

                                resolve(true);

                                dirty = true;
                            }, function (error) {

                                console.warn('Unable to sync offline session: ', error, session);
                                setLastSyncAttempt(session.id);

                                resolve(false);
                            });
                        });
                    };

                    /**
                     * Sequentially process the items, so we don't flood the
                     * internet connection as soon as the device goes back
                     * online.
                     */
                    async.eachSeries(results, function (item, callback) {
                        processItem(item).then(callback);
                    }, function () {

                        syncing = false;

                        if (results.length > 0) {
                            fireEventHandlers('offline_sessions_synced');
                        }
                    });

                    if (results.length === 0) {
                        syncing = false;
                    }
                }, function (result) {
                    console.error(result);
                    syncing = false;
                });
            },

            downloadTrainerSessions: function (dateStart, dateEnd, trainerUserId, fullWorkout = true) {

                var deferred = $q.defer();

                if (!NetworkStateService.isOnline()) {
                    deferred.reject();
                    return deferred.promise;
                }

                if (!trainerUserId) {
                    trainerUserId = PermissionsService.getLoggedInUserId();
                }

                if (!dateStart) {
                    dateStart = moment().subtract(1, 'months').startOf('day');
                }

                if (!dateEnd) {
                    dateEnd = moment().add(1, 'months').endOf('day');
                }

                var dateFromFormatted = dateStart.utc().format('YYYY-MM-DD HH:mm:ss');
                var dateToFormatted = dateEnd.utc().format('YYYY-MM-DD HH:mm:ss');

                ApiClientService.noCache().trainers.getSchedule(dateFromFormatted, dateToFormatted, trainerUserId, fullWorkout).success(function (response) {

                    var stored = [];
                    var currentIds = [];

                    angular.forEach(response.sessions, function (session) {
                        stored.push(storeSession(session));
                        currentIds.push(generateId(session));
                    });

                    $q.all(stored).then(function (storedResponse) {

                        cleanupRemotelyDeleted(currentIds, trainerUserId, false, dateFromFormatted, dateToFormatted).finally(function () {

                            deferred.resolve(storedResponse);
                            dirty = false;

                            fireEventHandlers('download_complete');
                        });
                    });
                });

                return deferred.promise;
            },

            downloadCustomerSessions: function (customerId, dateStart, dateEnd, fullWorkout = true) {

                var deferred = $q.defer();

                if (!NetworkStateService.isOnline()) {
                    deferred.reject();
                    return deferred.promise;
                }

                if (!dateStart) {
                    dateStart = moment().subtract(1, 'months').startOf('day');
                }

                if (!dateEnd) {
                    dateEnd = moment().add(1, 'months').endOf('day');
                }

                var dateFromFormatted = dateStart.utc().format('YYYY-MM-DD HH:mm:ss');
                var dateToFormatted = dateEnd.utc().format('YYYY-MM-DD HH:mm:ss');

                ApiClientService.noCache().customers.getSchedule(customerId, dateFromFormatted, dateToFormatted, fullWorkout).success(function (response) {

                    var stored = [];
                    var currentIds = [];

                    angular.forEach(response.sessions, function (session) {
                        stored.push(storeSession(session));
                        currentIds.push(generateId(session));
                    });

                    $q.all(stored).then(function (storedResponse) {

                        cleanupRemotelyDeleted(currentIds, false, customerId, dateFromFormatted, dateToFormatted).finally(function () {

                            deferred.resolve(storedResponse);

                            fireEventHandlers('download_complete');

                            dirty = false;
                        });
                    });
                });

                return deferred.promise;
            },

            countUnSyncedSessions: function () {

                var deferred = $q.defer();

                var user = PermissionsService.getCurrentUser();

                var whereField;
                var whereValue;

                if (PermissionsService.userIsTrainer()) {

                    whereField = 'trainer_user_id';
                    whereValue = user.id;
                } else {

                    whereField = 'customer_id';
                    whereValue = user.customer_id;
                }

                // Get all unsysnced sessions, last sync date descending.
                SqlStorageService.select('SELECT COUNT(*) AS unsyncedCount FROM sessions WHERE ' + whereField + ' = ? AND synced = 0 AND completed = 1', [whereValue]).then(function (result) {
                    deferred.resolve(result.unsyncedCount);
                }, function () {
                    deferred.resolve(0);
                });

                return deferred.promise;
            },

            getUnSyncedSessions: function () {

                var deferred = $q.defer();

                getUnSyncedSessions().then(function (response) {

                    deferred.resolve(parseRows(response));
                }, deferred.reject);

                return deferred.promise;
            },

            deleteLocalSession: function (sessionId) {
                return SqlStorageService.query("DELETE FROM sessions WHERE id = ?", [sessionId]).then(function () {
                    fireEventHandlers('local_session_deleted');
                });
            },

            isSyncing: function () {
                return syncing;
            },

            registerEventHandler: registerEventHandler,

            parseRow: parseRow,

            parseRows: parseRows

        };

        return service;
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('StopwatchModalController', controller).factory('StopwatchService', service).directive('stopwatchTimeDisplay', directive);

    controller.$inject = ['StopwatchService'];

    function controller(StopwatchService) {

        var vm = this;

        var settings = StopwatchService.getSettings();

        vm.mins = settings.mins.toString();
        vm.secs = settings.secs.toString();
        vm.state = StopwatchService.getState();

        vm.setTime = function () {
            StopwatchService.setTime(vm.mins, vm.secs);
        };
    }

    directive.$inject = ['StopwatchService'];

    function directive(StopwatchService) {
        return {
            restrict: 'E',
            replace: true,
            templateUrl: "common/session/stopwatch/timeDisplay.html",
            scope: {
                state: '=?'
            },
            link: function (scope, element, attrs, timeDisplayVm) {

                var timeMainElement = element.find('#timeMain');
                var timeMillisecondsElement = element.find('#timeMs');

                StopwatchService.tick(function (data) {

                    var mins = data.mins;
                    var secs = data.secs;
                    var ms = data.ms;

                    timeMainElement.text(mins + ':' + secs);
                    timeMillisecondsElement.text(':' + ms);

                    timeDisplayVm.state = data.state;
                    scope.state = data.state;
                });
            },
            controller: function () {

                var timeDisplayVm = this;

                timeDisplayVm.toggle = StopwatchService.toggle;
                timeDisplayVm.reset = StopwatchService.reset;
            },
            controllerAs: 'timeDisplayVm'
        };
    }

    service.$inject = ['$interval', 'ModalService', 'VibrateService', 'BeepService'];

    function service($interval, ModalService, VibrateService, BeepService) {

        var currentTickFunction = function () {};

        var state = 'stopped';
        var interval = null;
        var visible = false;

        var settings = {
            mins: 0,
            secs: 30
        };

        var current = {
            mins: '00',
            secs: '30',
            ms: '00',
            state: state
        };

        var duration;
        var currentEnd;

        var open = function () {

            var closed = function () {
                currentTickFunction = function () {};
                visible = false;
            };

            ModalService.show({
                templateUrl: 'common/session/stopwatch/stopwatch.html',
                controller: 'StopwatchModalController as vm'
            }).then(closed, closed);

            visible = true;
        };

        var padNumber = function (number) {

            if (number < 0) {
                return '00';
            }

            if (number < 10) {
                number = '0' + number;
            }

            return number;
        };

        var update = function () {

            if (state === 'started') {
                duration = moment.duration(currentEnd.diff(moment()));
            }

            current = {
                mins: padNumber(duration.minutes()),
                secs: padNumber(duration.seconds()),
                ms: padNumber(Math.round(duration.milliseconds() / 100)),
                state: state
            };

            currentTickFunction(current);
        };

        var tick = function (callback) {
            currentTickFunction = callback;
            currentTickFunction(current);
        };

        var clearTick = function () {
            currentTickFunction = function () {};
        };

        var doTick = function () {

            if (duration.asMilliseconds() <= 0) {

                duration = moment.duration(0);
                state = 'finished';
                $interval.cancel(interval);

                if (!visible) {
                    open();
                }

                VibrateService.vibrate(100);
                BeepService.beep(1);
            }

            update();
        };

        var start = function () {

            if (state === 'paused') {

                currentEnd = moment().add(duration);
            } else {

                currentEnd = moment().add({
                    minutes: settings.mins,
                    seconds: settings.secs
                });
            }

            state = 'started';

            if (!duration || duration.asMilliseconds() <= 0) {
                update();
            }

            interval = $interval(doTick, 100);
        };

        var pause = function () {

            $interval.cancel(interval);

            state = 'paused';

            update();
        };

        var reset = function () {

            state = 'stopped';

            duration = moment.duration({
                minutes: settings.mins,
                seconds: settings.secs
            });

            update();
        };

        var toggle = function () {

            switch (state) {

                case 'finished':
                    return false;
                    break;

                case 'stopped':
                case 'paused':
                    start();
                    break;

                case 'started':
                    pause();
                    break;

            }
        };

        var setTime = function (mins, secs) {

            settings = {
                mins: parseInt(mins),
                secs: parseInt(secs)
            };

            reset();
        };

        var getSettings = function () {
            return settings;
        };

        var getState = function () {
            return current.state;
        };

        return {

            open: open,
            tick: tick,
            clearTick: clearTick,
            start: start,
            pause: pause,
            reset: reset,
            toggle: toggle,
            update: update,
            setTime: setTime,
            getSettings: getSettings,
            getState: getState

        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('SyncManagerService', service).controller('SessionSyncManagerModalController', controller);

    service.$inject = ['ModalService'];

    function service(ModalService) {

        return {

            open: function () {

                return ModalService.show({
                    templateUrl: "common/session/syncManager/syncManager.html",
                    controller: 'SessionSyncManagerModalController as vm'
                });
            }

        };
    }

    controller.$inject = ['SessionStoreService', '$scope', 'ToastService', 'AnalyticsService'];

    function controller(SessionStoreService, $scope, ToastService, AnalyticsService) {

        var vm = this;

        vm.syncing = false;

        var updateSyncingStatus = function () {
            vm.syncing = SessionStoreService.isSyncing();
        };

        SessionStoreService.registerEventHandler($scope, 'offline_sessions_syncing', updateSyncingStatus);
        SessionStoreService.registerEventHandler($scope, 'offline_sessions_synced', function () {

            updateSyncingStatus();

            load(true);
        });

        var load = function (viaSyncTrigger) {

            viaSyncTrigger = viaSyncTrigger || false;

            SessionStoreService.getUnSyncedSessions().then(function (sessions) {

                sessions.map(function (session) {

                    session.title = 'Session';

                    if (session.data.workout_id) {
                        session.title = session.data.workout.title;
                    }

                    return session;
                });

                vm.sessions = sessions;

                if (sessions.length === 0) {

                    $scope.cancelModal(null);

                    if (viaSyncTrigger) {
                        ToastService.show('All sessions have been sync\'d successfully');
                    }
                }
            });
        };

        load();

        vm.abortSync = function (session) {

            SessionStoreService.deleteLocalSession(session.id).then(load);
        };

        vm.triggerSync = function () {

            SessionStoreService.pushOfflineSessions();

            vm.syncing = true;
        };

        AnalyticsService.trackView('Offline Session Sync Manager');
    }
})();
(() => {
  class creditQuotaController {
    /**
     * @param {object} $scope service
     * @param {object} $mdToast service
     * @param {object} ApiClientService service
     * @param {object} PermissionsService service
     * @param {object} AnalyticsService service
     */
    constructor($scope, $mdToast, ApiClientService, PermissionsService, AnalyticsService) {
      this.$scope = $scope;
      this.$mdToast = $mdToast;
      this.ApiClientService = ApiClientService;
      this.AnalyticsService = AnalyticsService;
      this.currentUser = PermissionsService.getCurrentUser();
      this.creditTrainers = [];
      this.selector = {
        chosenId: this.currentUser.type === 'trainer' ? this.currentUser.id : this.currentUser.trainer_user_id
      };
    }

    /**
     * @description Initialises the component and gets the trainers.
     * It then calls the analytics service
     */
    $onInit() {
      this.getCreditTrainers();
      this.$scope.$on('$ionicView.enter', () => this.AnalyticsService.trackView('Sessions & Credits - Credits'));
    }

    /**
     * @param {object} response the error response of a promise.
     * @description It gets the response and displays the error message
     */
    showNotification(response) {
      this.$mdToast.show(this.$mdToast.simple().textContent(response.error.message).position('top right'));
    }

    /**
     * @description method that gets the trainers that are associated with
     * packages and credits. It then calls the changeCredit in order to
     * transform the results on the view.
     */
    getCreditTrainers() {
      return this.ApiClientService.packagesCredits.getTrainers({
        user_id: this.customerUserId || this.currentUser.id
      }).then(response => {
        this.creditTrainers = response.trainers;
        this.changeCredit(this.selector.chosenId);
      }).catch(error => this.showNotification(error));
    }

    /**
     * @param {int} trainerId id.
     * @description method that gets all the packages credits that are not
     * unlimited and then gets the history.
     */
    changeCredit(trainerId) {
      return this.ApiClientService.packagesCredits.getAll({
        trainer_user_id: trainerId,
        user_id: this.customerUserId || this.currentUser.id
      }).then(response => {
        this.credits = response.credits;
        this.mainCredit = response.credits.find(credit => !credit.is_unlimited);

        if (response.credits.length >= 1) {
          this.getHistory();
        }
      }).catch(error => this.showNotification(error));
    }

    /**
     * @param {int} id of the user
     * @description gets the credits for the specific id.
     */
    changeMainCredit(id) {
      this.mainCredit = this.credits.find(credit => credit.id == id);
      if (this.credits.length >= 1) {
        this.getHistory();
      }
    }

    /**
     * @description returns the history of credits using the mainCredit id.
     */
    getHistory() {
      return this.ApiClientService.packagesCredits.getHistory(this.mainCredit.id).then(response => {
        const addedHistory = response.history.filter(item => item.action !== 'redeemed');
        const redeemedHistory = response.history.filter(item => item.action === 'redeemed');

        this.$scope.$emit('updateCreditHistory', {
          addedHistory,
          redeemedHistory
        });
      }).catch(error => this.showNotification(error));
    }
  }

  angular.module('MyPTHub').component('customerCredits', {
    templateUrl: 'common/session-credits/creditQuotaDisplay/customerCredits.html',
    controller: creditQuotaController,
    controllerAs: 'vm'
  }).controller('creditQuotaController', creditQuotaController);
})();
(function () {
  'use strict';

  angular.module('MyPTHub').factory('PackagesCustomerPurchasesDialogService', service).controller('PackagesCustomerPurchasesDialogController', controller);

  service.$inject = ['ModalService'];

  function service(ModalService) {

    return {

      show: function (customerId) {

        return ModalService.show({
          templateUrl: "common/session-credits/sessionQuotaDisplay/customerPurchases.html",
          controller: 'PackagesCustomerPurchasesDialogController as vm',
          params: {
            customerId: customerId
          }
        });
      }

    };
  }

  controller.$inject = ['$scope', 'ApiClientService', 'PermissionsService', 'parameters', '$filter'];

  function controller($scope, ApiClientService, PermissionsService, parameters, $filter) {

    var vm = this;

    vm.ready = false;

    vm.isTrainer = PermissionsService.userIsTrainer();
    vm.customerId = parameters.customerId;

    var load = function () {

      var getPurchases = ApiClientService.packages.listCustomerPurchases(vm.customerId);

      getPurchases.success(function (response) {

        vm.activePurchases = [];
        vm.inactivePurchases = [];

        vm.ready = true;

        response.purchases.map(function (purchase) {

          purchase.created_at = $filter('momentdate')(purchase.created_at, 'DD/MM/YYYY');
          purchase.ends_at = purchase.ends_at ? $filter('momentdate')(purchase.ends_at, 'DD/MM/YYYY') : '—';
          purchase.cancelled_at = purchase.cancelled_at ? $filter('momentdate')(purchase.cancelled_at, 'DD/MM/YYYY') : '—';

          purchase.canCancel = purchase.active;

          if (purchase.active) {
            vm.activePurchases.push(purchase);
          } else {
            vm.inactivePurchases.push(purchase);
          }
        });

        vm.noActivePackages = vm.activePurchases.length === 0;
        vm.noInactivePackages = vm.inactivePurchases.length === 0;
      });
    };

    load();

    vm.close = function () {
      $scope.cancelModal();
    };
  }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').component('sessionQuotaDisplay', {
    templateUrl: 'common/session-credits/sessionQuotaDisplay/sessionQuotaDisplay.html',
    controller: controller,
    controllerAs: 'vm',
    bindings: {
      customerId: '='
    }
  }).factory('SessionQuotaDisplayService', service);

  controller.$inject = ['PermissionsService', 'ApiClientService', 'PackagesSessionQuotaHistoryDialogService', 'PackagesGiftSessionsDialogService', '$scope', 'SessionQuotaDisplayService', '$interval', 'PackagesCustomerPurchasesDialogService'];

  function controller(PermissionsService, ApiClientService, PackagesSessionQuotaHistoryDialogService, PackagesGiftSessionsDialogService, $scope, SessionQuotaDisplayService, $interval, PackagesCustomerPurchasesDialogService) {

    var vm = this;

    vm.ready = false;

    vm.percentRemaining = 0;

    var load = function () {

      var getQuota = ApiClientService.packages.getSessionQuota(vm.customerId);

      getQuota.success(function (response) {

        vm.ready = true;

        vm.quota = response.quota;

        var percentRemaining = 0;
        var totalSessions = vm.quota.active.purchased + vm.quota.active.gifted;
        if (totalSessions > 0) {
          percentRemaining = Math.round(vm.quota.active.remaining / totalSessions * 100);
        }

        var buildPercent = $interval(function () {

          vm.percentRemaining += 1;
          if (vm.percentRemaining >= percentRemaining) {
            $interval.cancel(buildPercent);
          }
        }, 10);

        //vm.percentRemaining = percentRemaining;

        vm.canGift = PermissionsService.userIsTrainer() && vm.quota.active.activePurchases > 0;
      });
    };

    load();

    SessionQuotaDisplayService.registerReloadHandler($scope, load);

    vm.openHistory = function (event) {
      PackagesSessionQuotaHistoryDialogService.show(vm.customerId, event);
    };

    vm.openGiftSessions = function (event) {
      PackagesGiftSessionsDialogService.show(vm.customerId, event).then(function () {
        load();
      });
    };

    vm.openCustomerPackages = function () {
      PackagesCustomerPurchasesDialogService.show(vm.customerId);
    };
  }

  service.$inject = ['$rootScope'];

  function service($rootScope) {

    var service = {};

    service.registerReloadHandler = function (scope, handler) {

      var unsub = $rootScope.$on('reloadSessionQuotaDisplayComponent', handler);

      scope.$on('$destroy', unsub);

      return unsub;
    };

    service.reload = function () {
      $rootScope.$emit('reloadSessionQuotaDisplayComponent');
    };

    return service;
  }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').factory('PackagesSessionQuotaHistoryDialogService', service).controller('PackagesSessionQuotaHistoryDialogController', controller);

  service.$inject = ['ModalService'];

  function service(ModalService) {

    return {

      show: function (customerId, event) {

        event = event || null;

        return ModalService.show({
          templateUrl: "common/session-credits/sessionQuotaDisplay/sessionQuotaHistory.html",
          controller: 'PackagesSessionQuotaHistoryDialogController as vm',
          params: {
            customerId: customerId
          }
        });
      }

    };
  }

  controller.$inject = ['$scope', 'ApiClientService', '$mdToast', 'parameters'];

  function controller($scope, ApiClientService, $mdToast, parameters) {

    var vm = this;

    vm.ready = false;

    var getHistory = ApiClientService.packages.getSessionQuotaHistory(parameters.customerId);

    getHistory.success(function (response) {

      vm.ready = true;

      response.history.map(function (item) {

        if (item.type === 'period_started') {

          item.desc = 'New billing cycle started on ' + item.object.package_title;

          item.icon = 'date_range';
        } else {

          item.desc = item.object.qty + ' session' + (item.object.qty > 1 ? 's' : '') + ' ';

          item.desc += item.type;
          item.icon = item.type === 'gifted' ? 'add' : 'remove';

          if (item.object.session) {
            item.desc += ' on ' + item.object.session.title;
          }

          if (item.type === 'deducted' && item.object.purchase_id) {
            item.desc += ' from ' + item.object.package_title;
          }
        }

        return item;
      });

      vm.history = response.history;
      vm.noHistory = response.history.length === 0;
    });

    getHistory.error(function (response) {

      if (response.error) {
        $mdToast.show($mdToast.simple().textContent(response.error.message).position('top right'));
      } else {
        $mdToast.show($mdToast.simple().textContent('Unable to get history').position('top right'));
      }

      $scope.cancelModal();
    });

    vm.close = function () {
      $scope.cancelModal();
    };
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').factory('WorkoutEditorService', service);

    service.$inject = ['$filter', 'ApiClientService', 'ToastService', 'WorkoutStoreService', '$rootScope', 'AuthenticationStateService'];

    function service($filter, ApiClientService, ToastService, WorkoutStoreService, $rootScope, AuthenticationStateService) {

        var currentUser = AuthenticationStateService.getCurrentUser();

        var defaultLabels = {
            speed: 'kph',
            weight: 'kg',
            distance: 'km'
        };

        if (currentUser.country === 'US') {
            defaultLabels = {
                speed: 'mph',
                weight: 'lbs',
                distance: 'miles'
            };
        }

        var currentWorkout;

        var emitUnsavedHandler = function () {
            $rootScope.$emit('unsaved');
        };

        var transformWorkout = function (workout) {

            var workoutCopy = angular.copy(workout);

            var groups = [];

            angular.forEach(workoutCopy.groups, function (group, groupKey) {

                if (!group.chooseCardioType) {

                    angular.forEach(group.exercises, function (exerciseInstance) {

                        // Set default labels if they aren't set
                        angular.forEach(exerciseInstance.meta.set_fields, function (setField) {

                            if (setField.type === 'speed') {
                                if (!exerciseInstance.meta.hasOwnProperty(setField.name + '_label')) {
                                    exerciseInstance.meta[setField.name + '_label'] = defaultLabels.speed;
                                }
                            }

                            if (setField.type === 'weight') {
                                if (!exerciseInstance.meta.hasOwnProperty(setField.name + '_label')) {
                                    exerciseInstance.meta[setField.name + '_label'] = defaultLabels.weight;
                                }
                            }

                            if (setField.type === 'distance') {
                                if (!exerciseInstance.meta.hasOwnProperty(setField.name + '_label')) {
                                    exerciseInstance.meta[setField.name + '_label'] = defaultLabels.distance;
                                }
                            }
                        });

                        delete exerciseInstance.exercise.exercise_attributes;
                        delete exerciseInstance.exercise.pivot;
                        delete exerciseInstance.exercise._description;

                        // Remove internal fields from set
                        angular.forEach(exerciseInstance.meta.sets, function (set) {
                            delete set.exercise_workout_exercise_group_id;
                        });

                        if (exerciseInstance.hasOwnProperty('meta')) {
                            for (var metaName in exerciseInstance.meta) {
                                exerciseInstance[metaName] = exerciseInstance.meta[metaName];
                            }
                            delete exerciseInstance.meta;
                        }

                        if (exerciseInstance.hasOwnProperty('exercise')) {
                            for (var exerciseElementKey in exerciseInstance.exercise) {
                                exerciseInstance[exerciseElementKey] = exerciseInstance.exercise[exerciseElementKey];
                            }
                            delete exerciseInstance.exercise;
                        }

                        // Change exercise ID field name
                        if (exerciseInstance.hasOwnProperty('id')) {
                            exerciseInstance.exercise_id = exerciseInstance.id;
                            delete exerciseInstance.id;
                        }
                    });

                    groups.push(group);
                }
            });

            workoutCopy.groups = groups;

            return workoutCopy;
        };

        var transformIntervals = function (meta) {

            if (meta.type === 'interval') {

                var lastState = false;

                meta.stateSetCounts = { 1: 0, 2: 0, 3: 0 };

                angular.forEach(meta.sets, function (set) {

                    if (!lastState || lastState !== set.field_1) {
                        set.stateSort = 1;
                    } else {
                        set.stateSort++;
                    }
                    meta.stateSetCounts[set.field_1]++;

                    lastState = set.field_1;
                });
            }

            return meta;
        };

        var isCardioType = function (type) {

            var groupingDisabledForExerciseTypes = ['fartlek', 'lsd', 'interval'];

            if (groupingDisabledForExerciseTypes.indexOf(type) !== -1) {
                return true;
            }

            return false;
        };

        var groupingPossibleCount = function () {

            // Count of standalone exercises that are not cardio
            var standaloneCount = 0;

            angular.forEach(currentWorkout.groups, function (group, groupKey) {

                var firstExerciseType = group.exercises[0].meta.type;

                if (group.group_type === 'standalone' && !isCardioType(firstExerciseType)) {
                    standaloneCount++;
                }
            });

            return standaloneCount;
        };

        var getGroupable = function (baseGroup) {

            baseGroup = baseGroup || false;

            var groupables = [];

            angular.forEach(currentWorkout.groups, function (group, groupKey) {

                var firstExercise = group.exercises[0];
                var firstExerciseType = firstExercise.meta.type;

                firstExercise.origGroupKey = groupKey;

                if (group.group_type === 'standalone' && !isCardioType(firstExerciseType) && (!baseGroup || group.$$hashKey !== baseGroup.$$hashKey)) {
                    groupables.push(firstExercise);
                }
            });

            return groupables;
        };

        var getGrouperChosen = function () {

            var exercisesToAddToGroup = getGroupable();

            return $filter('filter')(exercisesToAddToGroup, { use: true });
        };

        var resetGrouperChosen = function () {

            angular.forEach(getGrouperChosen(), function (exerciseInstance) {
                exerciseInstance.use = false;
            });
        };

        var changeSetCount = function (changeGroup, forceSetCount) {

            forceSetCount = forceSetCount || false;
            var setCount;

            angular.forEach(changeGroup.exercises, function (exerciseInstance, exerciseIndex) {

                var sets = exerciseInstance.meta.sets;

                if (forceSetCount !== false) {
                    exerciseInstance.meta.setCount = forceSetCount;
                }

                if (exerciseIndex === 0) {
                    setCount = exerciseInstance.meta.setCount;
                }

                if (setCount > sets.length) {

                    while (sets.length < setCount) {
                        var lastSet = angular.copy(sets[sets.length - 1]);
                        lastSet.sort = (lastSet.sort ? lastSet.sort : 1) + 1;
                        sets.push(lastSet);
                    }
                } else if (setCount < sets.length) {

                    var toRemove = sets.length - setCount;

                    changeGroup.exercises[exerciseIndex].meta.sets = sets.slice(0, '-' + toRemove);
                }
            });
        };

        var addIntervalSet = function (meta, state) {

            var newSets = new Array();

            // Split sets into groups of states
            var easy = new Array();
            var medium = new Array();
            var hard = new Array();

            var emptySet = {
                field_1: 0,
                field_2: 0,
                field_3: 0,
                field_4: 0,
                field_5: 0,
                sort: 1
            };

            angular.forEach(meta.sets, function (set) {
                switch (set.field_1) {
                    case 1:
                        easy.push(set);
                        break;
                    case 2:
                        medium.push(set);
                        break;
                    case 3:
                        hard.push(set);
                        break;
                }
            });

            var newSet = emptySet;

            switch (state) {

                // Easy
                case 1:
                    if (easy.length > 0) {
                        newSet = easy[easy.length - 1];
                    }

                    newSet = angular.copy(newSet);

                    newSet.field_1 = 1;
                    newSet.sort = easy.length + 1;

                    easy.push(newSet);

                    break;

                // Med
                case 2:
                    if (medium.length > 0) {
                        newSet = medium[medium.length - 1];
                    } else if (easy.length > 0) {
                        newSet = easy[easy.length - 1];
                    }

                    newSet = angular.copy(newSet);

                    newSet.field_1 = 2;
                    newSet.sort = medium.length + 1;

                    medium.push(newSet);

                    break;

                // Hard
                case 3:
                    if (hard.length > 0) {
                        newSet = hard[hard.length - 1];
                    } else if (medium.length > 0) {
                        newSet = medium[medium.length - 1];
                    }

                    newSet = angular.copy(newSet);

                    newSet.field_1 = 3;
                    newSet.sort = hard.length + 1;

                    hard.push(newSet);

                    break;

            }

            Array.prototype.push.apply(newSets, easy);
            Array.prototype.push.apply(newSets, medium);
            Array.prototype.push.apply(newSets, hard);

            meta.sets = newSets;
            meta.stateSetCounts = {
                1: easy.length,
                2: medium.length,
                3: hard.length
            };

            return meta;
        };

        var addStandardSet = function (meta) {

            var emptySet = {
                field_1: 0,
                field_2: 0,
                field_3: 0,
                field_4: 0,
                field_5: 0,
                sort: 1
            };

            var newSet = emptySet;

            if (meta.sets.length > 0) {
                newSet = angular.copy(meta.sets[meta.sets.length - 1]);
                newSet.sort = meta.sets.length + 1;
            }

            meta.sets.push(newSet);

            return meta;
        };

        var removeEmptyGroups = function () {

            currentWorkout.groups = $filter('filter')(currentWorkout.groups, function (group) {
                return group.exercises.length > 0;
            });
        };

        var updateGroupSorts = function () {
            angular.forEach(currentWorkout.groups, function (group, groupKey) {
                group.sort = groupKey;
            });
        };

        var doGrouping = function (targetGroup) {

            var exercisesToGroup = getGrouperChosen();

            var firstExercise = targetGroup.exercises[0];

            var newExercises = [firstExercise];
            var exerciseSort = 0;
            var setCounts = [];

            angular.forEach(exercisesToGroup, function (exercise) {

                exercise.sort = exerciseSort;
                exercise.use = false;
                setCounts.push(exercise.meta.sets.length);
                newExercises.push(exercise);

                // Remove original exercise
                currentWorkout.groups[exercise.origGroupKey].exercises.splice(0, 1);

                exerciseSort++;
            });

            var exCount = exercisesToGroup.length + 1;

            var type = 'superset';
            if (exCount > 2) {
                type = 'giantset';
            }

            targetGroup.exercises = newExercises;
            targetGroup.group_type = type;

            // Make the set counts even, by setting all set counts to the same as the first exercise
            changeSetCount(targetGroup, firstExercise.meta.sets.length);

            removeEmptyGroups();

            updateGroupSorts();
        };

        var insertArrayAt = function (array, index, arrayToInsert) {
            return Array.prototype.splice.apply(array, [index, 0].concat(arrayToInsert));
        };

        var doUngrouping = function (groupKey) {

            var newgroups = [];
            angular.forEach(currentWorkout.groups[groupKey].exercises, function (exercise) {
                newgroups.push({
                    group_type: 'standalone',
                    exercises: [exercise]
                });
            });

            // Splice out the original group
            currentWorkout.groups.splice(groupKey, 1);

            // Splice the groups into where the existing multi-set is
            insertArrayAt(currentWorkout.groups, groupKey, newgroups);

            updateGroupSorts();
        };

        var removeExercise = function (exerciseInstance, exerciseInstanceKey, group, groupKey) {

            group.exercises.splice(exerciseInstanceKey, 1);

            if (group.group_type !== 'standalone') {

                var newType = 'standalone';

                if (group.exercises.length > 2) {
                    newType = 'giantset';
                } else if (group.exercises.length > 1) {
                    newType = 'superset';
                }

                group.group_type = newType;
            }

            removeEmptyGroups();

            updateGroupSorts();
        };

        var unsaved = false;

        var setUnsaved = function () {
            unsaved = true;
            emitUnsavedHandler();
        };

        var setSaved = function () {
            unsaved = false;
            emitUnsavedHandler();
        };

        var isUnsaved = function () {
            return unsaved;
        };

        var saveChanges = function () {

            return ApiClientService.workouts.update(currentWorkout.id, transformWorkout(currentWorkout)).success(function () {

                // Ensure local DB is updated immediately
                WorkoutStoreService.get(currentWorkout.id);

                setSaved();
            }).error(function () {

                ToastService.show('Could not save - please try again');

                setUnsaved();
            });
        };

        var prepareSetsForDisplay = function (meta) {

            var interval = false;
            if (meta.field_1 === 'state') {
                interval = true;
            }

            var states = {
                1: 'Easy',
                2: 'Medium',
                3: 'Hard'
            };

            meta.sets.forEach(function (set) {
                if (interval) {
                    set.stateDescription = states[set.field_1];
                }
            });

            return meta;
        };

        var filterAvailableFieldTypes = function (meta) {

            var usedTypes = [];
            meta.availableFieldTypesFiltered = {};

            angular.forEach(meta.set_fields, function (setField) {
                usedTypes.push(meta[setField.name]);
            });

            angular.forEach(meta.set_fields, function (setField) {

                meta.availableFieldTypesFiltered[setField.name] = $filter('filter')(meta.availableFieldTypes, function (availableType) {

                    if (usedTypes.indexOf(availableType.type) !== -1 && availableType.type !== 'n/a') {

                        if (meta[setField.name] !== usedTypes[usedTypes.indexOf(availableType.type)]) {
                            return false;
                        }
                    }

                    return true;
                });
            });

            return meta;
        };

        var addStandardExercise = function (exercise) {

            var instance = {
                exercise: exercise,
                meta: {}
            };

            var set = {
                sort: 1
            };

            var fields = [];

            angular.forEach(exercise.default_cols, function (col, key) {

                set[col.name] = col.initial_value;

                fields.push(col);

                instance.meta[col.name] = col.type;

                if (col.type === 'speed') {
                    instance.meta[col.name + '_label'] = defaultLabels.speed;
                }

                if (col.type === 'weight') {
                    instance.meta[col.name + '_label'] = defaultLabels.weight;
                }

                if (col.type === 'distance') {
                    instance.meta[col.name + '_label'] = defaultLabels.distance;
                }
            });

            instance.meta.availableFieldTypes = exercise.availableFieldTypes;
            instance.meta.sets = [set];
            instance.meta.setCount = 1;
            instance.meta.set_fields = fields;

            instance.meta = filterAvailableFieldTypes(instance.meta);

            currentWorkout.groups.push({
                group_type: 'standalone',
                exercises: [instance]
            });
        };

        var addCardioExercise = function (exercise, cardioType) {

            var instance = {
                exercise: exercise,
                meta: {}
            };

            instance.meta.type = cardioType;

            var fields = [];

            var set = {
                sort: 1
            };

            angular.forEach(exercise.default_cols[instance.meta.type], function (col, key) {

                set[col.name] = col.initial_value;

                fields.push(col);

                instance.meta[col.name] = col.type;

                if (col.type === 'speed') {
                    instance.meta[col.name + '_label'] = defaultLabels.speed;
                }

                if (col.type === 'weight') {
                    instance.meta[col.name + '_label'] = defaultLabels.weight;
                }

                if (col.type === 'distance') {
                    instance.meta[col.name + '_label'] = defaultLabels.distance;
                }
            });

            instance.meta.availableFieldTypes = exercise.availableFieldTypes[instance.meta.type];

            if (instance.meta.type === 'interval') {

                instance.meta.sets = [];

                instance.meta.sets.push(set);

                set = angular.copy(set);
                set.field_1 = 3;
                instance.meta.sets.push(set);
            } else {

                instance.meta.sets = [set];
            }

            instance.meta.setCount = 1;
            instance.meta.set_fields = fields;

            instance.meta = filterAvailableFieldTypes(instance.meta);

            currentWorkout.groups.push({
                group_type: 'standalone',
                exercises: [instance]
            });
        };

        var setNote = function (note) {
            currentWorkout.note = note;
        };

        return {

            registerEventHandler: function (scope, callback) {

                var unsub = $rootScope.$on('unsaved', callback);

                scope.$on('$destroy', unsub);

                return unsub;
            },

            setWorkout: function (workout) {
                currentWorkout = workout;
            },

            setExerciseMeta: function (meta, exerciseIndex, groupIndex) {
                currentWorkout.groups[groupIndex].exercises[exerciseIndex].meta = meta;
            },

            setExerciseField: function (value, field, exerciseIndex, groupIndex) {
                currentWorkout.groups[groupIndex].exercises[exerciseIndex].exercise[field] = value;
            },

            getWorkout: function () {

                if (!currentWorkout) {
                    return false;
                }

                return currentWorkout;
            },

            isCardioExerciseType: isCardioType,

            groupingSupersetPossible: function () {
                return groupingPossibleCount() > 1;
            },

            groupingGiantPossible: function () {
                return groupingPossibleCount() > 2;
            },

            getGroupable: getGroupable,

            resetGrouperChosen: resetGrouperChosen,

            doGrouping: doGrouping,

            doUngrouping: doUngrouping,

            removeExercise: removeExercise,

            saveChanges: saveChanges,

            transformWorkout: transformWorkout,

            transformIntervals: transformIntervals,

            prepareSetsForDisplay: prepareSetsForDisplay,

            addExercise: function () {
                prepareSetsForDisplay();
            },

            filterAvailableFieldTypes: filterAvailableFieldTypes,

            setNote: setNote,

            addIntervalSet: addIntervalSet,

            addStandardSet: addStandardSet,

            addStandardExercise: addStandardExercise,

            addCardioExercise: addCardioExercise,

            isUnsaved: isUnsaved

        };
    }
})();
(() => {
  angular.module('MyPTHub').config(config).controller('EditWorkoutController', controller);

  config.$inject = ['$stateProvider'];

  function config($stateProvider) {
    $stateProvider.state('app.edit_workout', {
      url: '/workouts/edit/{workoutId:[0-9]+}',
      views: {
        menuContent: {
          templateUrl: 'common/workouts/editor/edit.html',
          controller: 'EditWorkoutController as vm'
        }
      }
    });

    $stateProvider.state('app.edit_trainer_workout_via_customer', {
      url: "/workouts/via_customer/{customerId:[0-9]+}/workout/{workoutId:[0-9]+}",
      views: {
        menuContent: {
          templateUrl: "common/workouts/editor/edit.html",
          controller: 'EditWorkoutController as vm'
        }
      }
    });
  }

  controller.$inject = ['$scope', '$stateParams', '$q', 'WorkoutEditorService', 'ApiClientService', '$ionicActionSheet', 'ConfirmDialogService', 'ModalService', 'NavigateService', 'WorkoutStoreService', 'TrainerWorkoutsListService', 'PermissionsService', 'ToastService', 'ExerciseViewerService', 'AnalyticsService', 'PromptDialogService', 'CustomerFinderModalService'];

  function controller($scope, $stateParams, $q, WorkoutEditorService, ApiClientService, $ionicActionSheet, ConfirmDialogService, ModalService, NavigateService, WorkoutStoreService, TrainerWorkoutsListService, PermissionsService, ToastService, ExerciseViewerService, AnalyticsService, PromptDialogService, CustomerFinderModalService) {

    // temp horrible hack to work around issue of disappearing tabs
    // in some weird circumstances.
    $scope.$on('$ionicView.afterEnter', () => {
      $(window).trigger('resize');
    });

    var vm = this;

    vm.renameWorkout = () => {
      PromptDialogService.open({
        title: "Rename Workout",
        message: "Enter New Workout Name",
        buttonConfirm: "Save",
        buttonCancel: "Cancel",
        required: true,
        defaultText: vm.workout.title
      }).then(name => {
        var workoutTmp = {
          title: name
        };

        ApiClientService.workouts.update(vm.workout.id, workoutTmp).success(response => {
          vm.workout.title = name;
          ToastService.show(response.message);
        });
      });
    };

    vm.delete = () => {
      ConfirmDialogService.open({
        title: "Confirm delete",
        message: "Do you want to remove this workout?",
        buttonConfirm: "Confirm",
        buttonCancel: "Cancel"
      }).then(() => {
        WorkoutStoreService.delete(vm.workout.id).then(() => {
          if (PermissionsService.userIsTrainer()) {
            TrainerWorkoutsListService.dirty();
          }

          NavigateService.clearCache().then(NavigateService.goBack);
        });
      });
    };

    vm.moveGroup = (group, fromIndex, toIndex) => {
      vm.workout.groups.splice(fromIndex, 1);
      vm.workout.groups.splice(toIndex, 0, group);

      for (let i = 0, length = vm.workout.groups.length; i < length; i++) {
        vm.workout.groups[i].sort = i;
      }

      vm.saving = true;

      WorkoutEditorService.saveChanges().finally(() => {
        vm.saving = false;
      });
    };

    vm.exerciseMenu = (exerciseInstance, exerciseInstanceKey, group, groupKey) => {
      let titleText = 'Exercise';
      let groupTypeName;
      let buttons = [];

      buttons.push({
        text: 'View Exercise',
        handler: () => {
          ExerciseViewerService.open(exerciseInstance.exercise, exerciseInstance.exercise.name_id);
        }
      });

      buttons.push({
        text: 'Edit Exercise',
        handler: () => {
          vm.editExercise(exerciseInstance, exerciseInstanceKey, groupKey);
        }
      });

      buttons.push({
        text: exerciseInstance.exercise.note && exerciseInstance.exercise.note.length > 0 ? 'Edit Note' : 'Add Note',
        handler: () => {
          vm.editExerciseNote(exerciseInstance, exerciseInstanceKey, groupKey);
        }
      });

      if (group.group_type !== 'standalone') {

        groupTypeName = group.group_type === 'giantset' ? 'Giantset' : 'Superset';

        titleText = groupTypeName;

        buttons.push({
          text: 'Break apart ' + groupTypeName,
          handler: () => {

            WorkoutEditorService.doUngrouping(groupKey);

            WorkoutEditorService.saveChanges();
          }
        });
      } else if (!WorkoutEditorService.isCardioExerciseType(exerciseInstance.meta.type) && WorkoutEditorService.groupingSupersetPossible()) {
        buttons.push({
          text: 'Superset / Giantset',
          handler: () => {
            ModalService.show({
              templateUrl: "common/workouts/editor/grouperModal.html",
              controller: 'EditWorkoutGrouperModalController as vm',
              params: {
                baseExercise: exerciseInstance,
                baseGroup: group,
                baseGroupKey: groupKey
              }
            }).finally(() => {
              WorkoutEditorService.resetGrouperChosen();
            });
          }
        });
      }

      if (vm.workout.groups && vm.workout.groups.length > 1) {
        buttons.push({
          text: 'Reorder',
          handler: () => {
            vm.reordering = true;
          }
        });
      }

      $ionicActionSheet.show({
        titleText: titleText,
        buttons: buttons,
        destructiveText: 'Delete Exercise',
        cancelText: 'Cancel',
        buttonClicked: index => {

          if (typeof buttons[index].handler === 'function') {
            buttons[index].handler();
          }

          return true;
        },
        destructiveButtonClicked: () => {

          // Delete exercise from the workout or the group if standalone
          ConfirmDialogService.open({
            title: "Confirm delete",
            message: "Do you want to remove this exercise from the workout?",
            buttonConfirm: "Confirm",
            buttonCancel: "Cancel"
          }).then(() => {

            WorkoutEditorService.removeExercise(exerciseInstance, exerciseInstanceKey, group, groupKey);

            WorkoutEditorService.saveChanges();
          });

          return true;
        }
      });
    };

    vm.editExercise = (exerciseInstance, exerciseIndex, groupIndex) => {

      ModalService.show({
        templateUrl: "common/workouts/editor/exerciseEditModal.html",
        controller: 'EditWorkoutExerciseEditModalController as vm',
        params: {
          exerciseInstance: exerciseInstance,
          exerciseIndex: exerciseIndex,
          groupIndex: groupIndex
        }
      }).finally(() => {
        vm.workout = WorkoutEditorService.getWorkout();
      });
    };

    vm.editExerciseNote = (exerciseInstance, exerciseIndex, groupIndex) => {

      ModalService.show({
        templateUrl: "common/workouts/editor/exerciseNoteEditModal.html",
        controller: 'EditWorkoutExerciseNoteEditModalController as vm',
        params: {
          exerciseInstance: exerciseInstance,
          exerciseIndex: exerciseIndex,
          groupIndex: groupIndex
        }
      }).finally(() => {
        vm.workout = WorkoutEditorService.getWorkout();
      });
    };

    vm.saveNote = () => {

      vm.saving = true;

      WorkoutEditorService.setNote(vm.workout.note);

      WorkoutEditorService.saveChanges().finally(() => {
        vm.saving = false;
      });
    };

    var getWorkout = () => {

      return ApiClientService.workouts.get($stateParams.workoutId, true).success(response => {

        vm.canEdit = response.workout.created_by === 'customer' && PermissionsService.userIsCustomer() || PermissionsService.userIsTrainerAdminOrOwner(response.workout.trainer_user_id);

        WorkoutEditorService.setWorkout(response.workout);

        vm.workout = WorkoutEditorService.getWorkout();

        let files = vm.workout.files;
        const mimes = {
          image: ['image/gif', 'image/x-icon', 'image/jpeg', 'image/jpg', 'image/png', 'image/svg+xml', 'image/tiff', 'image/webp'],
          document: ['application/msword', 'application/vnd.oasis.opendocument.text', 'application/rtf', 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'],
          pdf: ['application/pdf'],
          archive: ['application/x-tar', 'application/x-bzip2', 'application/gzip', 'application/x-7z-compressed', 'application/x-apple-diskimage', 'application/x-rar-compressed', 'application/zip'],
          spreadsheet: ['application/vnd.oasis.opendocument.spreadsheet', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet', 'application/vnd.ms-excel']
        };
        files.map(file => {
          if (mimes.image.includes(file.mime)) {
            file.icon = 'fa-file-image-o';
          } else if (mimes.spreadsheet.includes(file.mime)) {
            file.icon = 'fa-file-excel-o';
          } else if (mimes.pdf.includes(file.mime)) {
            file.icon = 'fa-file-pdf-o';
          } else if (mimes.archive.includes(file.mime)) {
            file.icon = 'fa-file-archive-o';
          } else if (mimes.document.includes(file.mime)) {
            file.icon = 'fa-file-text-o';
          } else {
            file.icon = 'fa-file-o';
          }
          file.name = `${file.original_name}.${file.extension}`;
        });

        vm.workout.files = files;

        vm.assignedCustomers = vm.workout.assigned_customers;
        vm.assignedCustomerGroups = vm.workout.customer_groups;
        vm.noAssignedCustomers = vm.assignedCustomers.length === 0 && vm.assignedCustomerGroups.length === 0;
        vm.assignedGroups = vm.workout.workout_groups;

        console.log(vm.workout);
      }).error(response => {

        if (response && response.error && response.error.error_name && response.error.error_name === 'SUBSCRIPTION_UPGRADE_REQUIRED') {
          ToastService.show(response.error.message);
          NavigateService.goBack();
        } else {
          ToastService.show('Could not get workout, please try again later');
        }
      });
    };

    var getCustomer = () => {

      if ($stateParams.customerId) {

        return ApiClientService.customers.get($stateParams.customerId).success(response => {
          vm.customer = response.customer;
        });
      } else {

        return $q.resolve(false);
      }
    };

    vm.searchExercise = () => {

      vm.selectedTabIndex = 0;

      ModalService.show({
        templateUrl: "common/workouts/editor/exerciseAddModal.html",
        controller: 'EditWorkoutAddExerciseModalController as vm'
      }).then(() => {

        vm.saving = true;

        WorkoutEditorService.saveChanges().finally(() => {
          vm.saving = false;
        });
      });
    };

    vm.createExercise = () => {

      vm.selectedTabIndex = 0;

      ModalService.show({
        templateUrl: "common/workouts/editor/exerciseCreateModal.html",
        controller: 'EditWorkoutCreateExerciseModalController as vm'
      }).then(added => {
        if (added) {
          vm.saving = true;

          WorkoutEditorService.saveChanges().finally(() => {
            vm.saving = false;
          });
        }
      });
    };

    vm.assignToClient = () => {
      vm.selectedTabIndex = 2;

      CustomerFinderModalService.open({
        trainerUserId: vm.workout.trainer_user_id,
        currentCustomerAssignments: vm.assignedCustomers,
        currentGroupAssignments: vm.assignedCustomerGroups,
        modalTitle: 'Assign Clients',
        mode: 'multi',
        includeGroups: true
      }).then(assignedItems => {
        const assignedCustomerIds = [];
        const assignedCustomerGroupIds = [];

        assignedItems.customers.map(customer => assignedCustomerIds.push(customer.id));
        assignedItems.customerGroups.map(customerGroup => assignedCustomerGroupIds.push(customerGroup.id));

        ApiClientService.workouts.update(vm.workout.id, {
          assigned_customer_ids: assignedCustomerIds,
          assigned_customer_group_ids: assignedCustomerGroupIds
        }).then(() => {
          vm.assignedCustomers = assignedItems.customers;
          vm.assignedCustomerGroups = assignedItems.customerGroups;
          vm.noAssignedCustomers = vm.assignedCustomers.length === 0 && vm.assignedCustomerGroups.length === 0;
        }, () => ToastService.show('Could not assign clients, please try again later'));
      });
    };

    vm.assignToGroup = () => {
      ModalService.show({
        templateUrl: 'common/workouts/editor/groupAssignModal.html',
        controller: 'EditWorkoutGroupAssignModalController as vm',
        params: {
          assignedGroups: vm.assignedGroups,
          workoutId: vm.workout.id,
          workout: vm.workout
        }
      }).then(assignedGroups => {
        vm.assignedGroups = assignedGroups;
      });
    };

    vm.assignedCustomerMenu = (customer, index) => {
      $ionicActionSheet.show({
        titleText: customer.user.first_name + ' ' + customer.user.last_name,
        destructiveText: 'Unassign Client',
        cancelText: 'Cancel',
        destructiveButtonClicked: () => {
          ApiClientService.workouts.unassignCustomer(vm.workout.id, customer.id).then(() => {
            vm.assignedCustomers.splice(index, 1);
            vm.noAssignedCustomers = vm.assignedCustomers.length === 0 && vm.assignedCustomerGroups.length === 0;
          });
          return true;
        }
      });
    };

    vm.assignedCustomerGroupMenu = (group, index) => {
      $ionicActionSheet.show({
        titleText: group.title,
        destructiveText: 'Unassign Client Group',
        cancelText: 'Cancel',
        destructiveButtonClicked: () => {
          ApiClientService.workouts.unassignCustomerGroup(vm.workout.id, group.id).then(() => {
            vm.assignedCustomerGroups.splice(index, 1);
            vm.noAssignedCustomers = vm.assignedCustomers.length === 0 && vm.assignedCustomerGroups.length === 0;
          });
          return true;
        }
      });
    };

    vm.stopReordering = () => {
      vm.reordering = false;
    };
    vm.notesShow = vm.stopReordering;
    vm.clientsShow = vm.stopReordering;

    $scope.$on('$ionicView.beforeEnter', () => {
      vm.loaded = false;
      vm.customer = false;
      vm.workout = false;

      vm.isTrainer = PermissionsService.userIsTrainer();

      $q.all([getWorkout(), getCustomer()]).then(() => {
        vm.loaded = true;
      });
    });

    $scope.$on('$ionicView.enter', () => {
      AnalyticsService.trackView(PermissionsService.userIsTrainer() ? 'Trainer - Edit Workout' : 'Client - Edit Own Created Workout');
    });

    vm.viewFile = function (file) {
      ModalService.show({
        templateUrl: 'common/files/filesViewModal.html',
        controller: 'FilesViewController as vm',
        params: {
          file,
          trainer: false
        }
      }).finally(() => {
        getWorkout();
      });
    };
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('EditWorkoutAddExerciseModalController', controller);

    controller.$inject = ['$scope', 'ApiClientService', '$ionicScrollDelegate', 'lodash', 'WorkoutEditorService', '$ionicActionSheet', 'AnalyticsService', 'PermissionsService'];

    function controller($scope, ApiClientService, $ionicScrollDelegate, lodash, WorkoutEditorService, $ionicActionSheet, AnalyticsService, PermissionsService) {

        var vm = this;

        vm.isTrainer = PermissionsService.userIsTrainer();

        var resultsPerPage = 30;
        vm.loadingMore = false;

        var setupBodyPartList = function () {

            ApiClientService.exercises.getAttributeValuesByNameId(2).success(function (response) {

                vm.bodyParts = [{
                    name: 'Select Body Part',
                    value: null
                }];

                angular.forEach(response.values, function (attribute) {

                    vm.bodyParts.push({
                        name: attribute.value,
                        value: attribute.id
                    });
                });

                vm.selectedPart = {
                    type: vm.bodyParts[0].value
                };
            });
        };

        var resetSearch = function () {
            vm.search = {
                query: null,
                results: [],
                noResults: false,
                page: 1,
                nextPage: null,
                loading: false
            };
        };
        resetSearch();

        var resetBodypartSearch = function () {
            vm.bodyPartSearch = {
                type: null,
                results: [],
                noResults: false,
                page: 1,
                nextPage: null,
                loading: false
            };
        };
        resetBodypartSearch();

        var resetRecent = function () {
            vm.recentSearch = {
                results: [],
                noResults: false,
                page: 1,
                nextPage: null,
                loading: false
            };
        };
        resetRecent();

        var resetFrequent = function () {
            vm.frequentSearch = {
                results: [],
                noResults: false,
                page: 1,
                nextPage: null,
                loading: false
            };
        };
        resetFrequent();

        var resetCustom = function () {
            vm.customSearch = {
                results: [],
                noResults: false,
                page: 1,
                nextPage: null,
                loading: false
            };
        };
        resetCustom();

        var doSearch = function (paginating) {

            paginating = paginating || false;

            if (!vm.search.query) {

                resetSearch();

                return false;
            }

            if (!paginating) {
                vm.search.results.length = 0;
                vm.search.page = 1;
                vm.search.nextPage = null;
            }

            vm.search.loading = true;
            vm.loadingMore = true;

            ApiClientService.exercises.search(vm.search.query, resultsPerPage, vm.search.page).success(function (response) {

                vm.search.nextPage = response.pagination.nextPage || null;

                if (paginating) {
                    vm.search.results.push.apply(vm.search.results, angular.copy(response.results));
                } else {
                    vm.search.results = response.results;
                }

                vm.search.loading = false;
                vm.search.noResults = vm.search.results.length === 0 && vm.search.query;
                vm.loadingMore = false;

                $ionicScrollDelegate.resize();
            });
        };

        // One time cache-able setup
        setupBodyPartList();

        vm.doBodyPartSearch = function (paginating) {

            paginating = paginating || false;

            if (!paginating) {
                vm.bodyPartSearch.results.length = 0;
                vm.bodyPartSearch.page = 1;
                vm.bodyPartSearch.nextPage = null;
            }

            vm.bodyPartSearch.loading = true;
            vm.loadingMore = true;

            ApiClientService.exercises.getExercisesByValueId(vm.bodyPartSearch.type, resultsPerPage, vm.bodyPartSearch.page).success(function (response) {

                vm.bodyPartSearch.nextPage = response.pagination.nextPage || null;

                if (paginating) {
                    vm.bodyPartSearch.results.push.apply(vm.bodyPartSearch.results, angular.copy(response.results));
                } else {
                    vm.bodyPartSearch.results = response.results;
                }

                vm.bodyPartSearch.loading = false;
                vm.bodyPartSearch.noResults = vm.bodyPartSearch.results.length === 0;
                vm.loadingMore = false;

                $ionicScrollDelegate.resize();
            });
        };

        vm.doRecentSearch = function (paginating) {

            paginating = paginating || false;

            if (!paginating && vm.recentSearch.results.length > 0) {
                return false;
            }

            vm.recentSearch.loading = true;
            vm.loadingMore = true;

            ApiClientService.exercises.getRecentlyUsed(resultsPerPage, vm.recentSearch.page).success(function (response) {

                vm.recentSearch.nextPage = response.pagination.nextPage || null;

                if (paginating) {
                    vm.recentSearch.results.push.apply(vm.recentSearch.results, angular.copy(response.exercises));
                } else {
                    vm.recentSearch.results = response.exercises;
                }

                vm.recentSearch.loading = false;
                vm.recentSearch.noResults = vm.recentSearch.results.length === 0;
                vm.loadingMore = false;

                $ionicScrollDelegate.resize();
            });
        };

        vm.doFrequentSearch = function (paginating) {

            paginating = paginating || false;

            if (!paginating && vm.frequentSearch.results.length > 0) {
                return false;
            }

            vm.frequentSearch.loading = true;
            vm.loadingMore = true;

            ApiClientService.exercises.getFrequentlyUsed(resultsPerPage, vm.frequentSearch.page).success(function (response) {

                vm.frequentSearch.nextPage = response.pagination.nextPage || null;

                if (paginating) {
                    vm.frequentSearch.results.push.apply(vm.frequentSearch.results, angular.copy(response.exercises));
                } else {
                    vm.frequentSearch.results = response.exercises;
                }

                vm.frequentSearch.loading = false;
                vm.frequentSearch.noResults = vm.frequentSearch.results.length === 0;
                vm.loadingMore = false;

                $ionicScrollDelegate.resize();
            });
        };

        vm.doCustomSearch = function (paginating) {

            paginating = paginating || false;

            if (!paginating && vm.customSearch.results.length > 0) {
                return false;
            }

            vm.customSearch.loading = true;
            vm.loadingMore = true;

            ApiClientService.exercises.getAllForTrainer(resultsPerPage, vm.customSearch.page).success(function (response) {

                vm.customSearch.nextPage = response.pagination.nextPage || null;

                if (paginating) {
                    vm.customSearch.results.push.apply(vm.customSearch.results, angular.copy(response.exercises));
                } else {
                    vm.customSearch.results = response.exercises;
                }

                vm.customSearch.loading = false;
                vm.customSearch.noResults = vm.customSearch.results.length === 0;
                vm.loadingMore = false;

                $ionicScrollDelegate.resize();
            });
        };

        resetSearch();
        resetBodypartSearch();
        resetRecent();
        resetFrequent();
        resetCustom();

        vm.setMode = function (mode) {
            vm.currentMode = mode;

            switch (vm.currentMode) {

                case 'recent':
                    vm.doRecentSearch();
                    break;

                case 'frequent':
                    vm.doFrequentSearch();
                    break;

                case 'custom':
                    vm.doCustomSearch();
                    break;

            }
        };

        vm.isMode = function (checkMode) {
            return vm.currentMode === checkMode;
        };

        vm.canLoadMore = function () {

            switch (vm.currentMode) {

                case 'search':
                    if (vm.search.results.length > 0 && vm.search.loading === false && vm.search.nextPage !== null) {
                        return true;
                    }
                    break;

                case 'bodypart':
                    if (vm.bodyPartSearch.results.length > 0 && vm.bodyPartSearch.loading === false && vm.bodyPartSearch.nextPage !== null) {
                        return true;
                    }
                    break;

                case 'recent':
                    if (vm.recentSearch.results.length > 0 && vm.recentSearch.loading === false && vm.recentSearch.nextPage !== null) {
                        return true;
                    }
                    break;

                case 'frequent':
                    if (vm.frequentSearch.results.length > 0 && vm.frequentSearch.loading === false && vm.frequentSearch.nextPage !== null) {
                        return true;
                    }
                    break;

                case 'custom':
                    if (vm.customSearch.results.length > 0 && vm.customSearch.loading === false && vm.customSearch.nextPage !== null) {
                        return true;
                    }
                    break;

            }

            return false;
        };

        vm.loadMore = function () {

            switch (vm.currentMode) {

                case 'search':
                    vm.search.page++;
                    doSearch(true);
                    break;

                case 'bodypart':
                    vm.bodyPartSearch.page++;
                    vm.doBodyPartSearch(true);
                    break;

                case 'recent':
                    vm.recentSearch.page++;
                    vm.doRecentSearch(true);
                    break;

                case 'frequent':
                    vm.frequentSearch.page++;
                    vm.doFrequentSearch(true);
                    break;

                case 'custom':
                    vm.customSearch.page++;
                    vm.doCustomSearch(true);
                    break;

            }
        };

        vm.chooseExercise = function (exercise) {

            if (exercise.hasOwnProperty('exercise')) {

                exercise = lodash.merge(angular.copy(exercise), angular.copy(exercise.exercise));

                delete exercise.exercise;
            }

            if (exercise.exercise_type === 'cardio') {

                $ionicActionSheet.show({
                    titleText: 'Choose cardio type',
                    buttons: [{
                        text: 'Long Slow Duration (LSD)'
                    }, {
                        text: 'Interval Training'
                    }, {
                        text: 'Fartlek Training'
                    }],
                    cancelText: 'Cancel',
                    buttonClicked: function (index) {

                        var type;

                        switch (index) {

                            // LSD
                            case 0:
                                type = 'lsd';
                                break;

                            // Interval
                            case 1:
                                type = 'interval';
                                break;

                            // Fartlek
                            case 2:
                                type = 'fartlek';
                                break;
                        }

                        WorkoutEditorService.addCardioExercise(exercise, type);
                        $scope.closeModal(true);

                        return true;
                    }
                });
            } else {

                WorkoutEditorService.addStandardExercise(exercise);
                $scope.closeModal(true);
            }
        };

        $scope.$watch(function () {
            return vm.search.query;
        }, function () {
            doSearch();
        });

        AnalyticsService.trackView(PermissionsService.userIsTrainer() ? 'Trainer - Workout Editor Exercise Search' : 'Client - Workout Editor Exercise Search');

        //$scope.closeModal();
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('EditWorkoutCreateExerciseModalController', controller);

    controller.$inject = ['$scope', 'ApiClientService', '$ionicActionSheet', 'WorkoutEditorService', 'AnalyticsService'];

    function controller($scope, ApiClientService, $ionicActionSheet, WorkoutEditorService, AnalyticsService) {

        var vm = this;

        vm.currentAttributes = {};
        vm.selectedOtherMuscles = [];
        vm.availableOtherMuscles = [];
        vm.otherMusclesId = false;

        // Get Available Attributes
        vm.attributesLoaded = false;
        ApiClientService.exercises.getAvailableAttributes().success(function (response) {

            vm.availableAttributeNames = response.attributes;

            angular.forEach(vm.availableAttributeNames, function (attr) {
                if (attr.name === 'Other Muscles') {
                    vm.otherMusclesId = attr.id;
                    angular.forEach(attr.values, function (val) {
                        vm.availableOtherMuscles.push({
                            id: val.id,
                            label: val.value,
                            name_id: attr.id
                        });
                    });
                }
            });

            vm.attributesLoaded = true;
        });

        vm.exercise = {};

        vm.doCreate = function () {

            vm.saving = true;

            var joinedAttrs = {};

            angular.forEach(vm.selectedOtherMuscles, function (attr) {
                if (joinedAttrs.hasOwnProperty(vm.otherMusclesId)) {
                    if (typeof joinedAttrs[vm.otherMusclesId] === 'object') {
                        joinedAttrs[vm.otherMusclesId].push(attr.id);
                    } else {
                        joinedAttrs[vm.otherMusclesId] = [joinedAttrs[vm.otherMusclesId], attr.id];
                    }
                } else {
                    joinedAttrs[vm.otherMusclesId] = attr.id;
                }
            });

            angular.forEach(vm.currentAttributes, function (valueId, nameId) {
                joinedAttrs[nameId] = valueId;
            });

            vm.exercise.replace_attributes = joinedAttrs;

            ApiClientService.exercises.createDraft().success(function (response) {
                ApiClientService.exercises.update(response.exercise.id, vm.exercise).success(function (updateResponse) {
                    ApiClientService.exercises.get(response.exercise.id).success(function (getResponse) {

                        var exercise = getResponse.exercise;

                        exercise.name = exercise.default_name.name;

                        if (exercise.exercise_type === 'cardio') {

                            $ionicActionSheet.show({
                                titleText: 'Choose cardio type',
                                buttons: [{
                                    text: 'Long Slow Duration (LSD)'
                                }, {
                                    text: 'Interval Training'
                                }, {
                                    text: 'Fartlek Training'
                                }],
                                cancelText: 'Cancel',
                                cancel: function () {
                                    $scope.closeModal(false);
                                },
                                buttonClicked: function (index) {

                                    var type;

                                    switch (index) {

                                        // LSD
                                        case 0:
                                            type = 'lsd';
                                            break;

                                        // Interval
                                        case 1:
                                            type = 'interval';
                                            break;

                                        // Fartlek
                                        case 2:
                                            type = 'fartlek';
                                            break;
                                    }

                                    WorkoutEditorService.addCardioExercise(exercise, type);
                                    $scope.closeModal(true);

                                    return true;
                                }
                            });
                        } else {

                            WorkoutEditorService.addStandardExercise(exercise);
                            $scope.closeModal(true);
                        }

                        // if cardio selected
                        // WorkoutEditorService.addCardioExercise(exercise, type);
                        // else
                        // WorkoutEditorService.addStandardExercise(exercise);

                        //$scope.closeModal();
                    });
                });
            });
        };

        AnalyticsService.trackView('Trainer - Workout Editor Create Exercise');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('EditWorkoutExerciseEditModalController', controller);

    controller.$inject = ['parameters', 'WorkoutEditorService', '$scope', 'ConfirmDialogService', 'PermissionsService', 'AnalyticsService', 'AuthenticationStateService'];

    const getFieldByValue = (exercise, value) => {
        if (exercise.field_1 === value) {
            return 'field_1';
        }
        if (exercise.field_2 === value) {
            return 'field_2';
        }
        if (exercise.field_3 === value) {
            return 'field_3';
        }
        if (exercise.field_4 === value) {
            return 'field_4';
        }
        if (exercise.field_5 === value) {
            return 'field_5';
        }
        return null;
    };

    function controller(parameters, WorkoutEditorService, $scope, ConfirmDialogService, PermissionsService, AnalyticsService, AuthenticationStateService) {

        var currentUser = AuthenticationStateService.getCurrentUser();

        var defaultLabels = {
            speed: 'kph',
            weight: 'kg',
            distance: 'km'
        };

        if (currentUser.country === 'US') {
            defaultLabels = {
                speed: 'mph',
                weight: 'lbs',
                distance: 'miles'
            };
        }

        var vm = this;

        vm.workout = WorkoutEditorService.getWorkout();
        vm.canEdit = vm.workout.created_by === 'customer' && PermissionsService.userIsCustomer() || PermissionsService.userIsTrainerAdminOrOwner(vm.workout.trainer_user_id);

        vm.showFieldEdit = false;
        vm.toggleFieldEdit = function () {
            vm.showFieldEdit = !vm.showFieldEdit;
        };

        var exerciseInstance = angular.copy(parameters.exerciseInstance);

        vm.exercise = exerciseInstance.exercise;
        vm.meta = WorkoutEditorService.prepareSetsForDisplay(exerciseInstance.meta);
        vm.meta = WorkoutEditorService.filterAvailableFieldTypes(exerciseInstance.meta);
        vm.meta = WorkoutEditorService.transformIntervals(exerciseInstance.meta);

        vm.addSetCommands = [];

        if (vm.meta.type !== 'interval' && vm.meta.type !== 'lsd') {
            vm.addSetCommands.push({
                label: 'Add set',
                click: function () {
                    vm.meta = WorkoutEditorService.addStandardSet(vm.meta);
                }
            });
        }

        if (vm.meta.type === 'interval') {

            var addIntervalSetCommand = function (state) {
                vm.meta = WorkoutEditorService.addIntervalSet(vm.meta, state);
                vm.meta = WorkoutEditorService.prepareSetsForDisplay(vm.meta);
                vm.meta = WorkoutEditorService.transformIntervals(vm.meta);
            };

            vm.addSetCommands.push({
                label: 'Add hard set',
                click: function () {
                    addIntervalSetCommand(3);
                }
            });

            vm.addSetCommands.push({
                label: 'Add medium set',
                click: function () {
                    addIntervalSetCommand(2);
                }
            });

            vm.addSetCommands.push({
                label: 'Add easy set',
                click: function () {
                    addIntervalSetCommand(1);
                }
            });
        }

        vm.visibleFieldsFilter = function (field) {

            if (field.type === 'state' || field.type === 'sets') {
                return false;
            }

            return true;
        };

        vm.fieldTypeChosen = function (field) {

            if (vm.meta[field.name] === 'speed') {
                vm.meta[field.name + '_label'] = defaultLabels.speed;
            } else if (vm.meta[field.name] === 'weight') {
                vm.meta[field.name + '_label'] = defaultLabels.weight;
            } else if (vm.meta[field.name] === 'distance') {
                vm.meta[field.name + '_label'] = defaultLabels.distance;
            } else {
                vm.meta[field.name + '_label'] = null;
            }

            if (vm.meta[field.name] === 'amrap') {
                const repsField = getFieldByValue(vm.meta, 'reps');
                const weightField = getFieldByValue(vm.meta, 'weight');
                const repRangeField = getFieldByValue(vm.meta, 'rep_range');
                if (repRangeField) {
                    vm.meta[repRangeField] = 'n/a';
                }
                if (repsField) {
                    if (weightField) {
                        vm.meta[repsField] = 'n/a';
                    } else {
                        vm.meta[repsField] = 'weight';
                    }
                }
            }

            if (vm.meta[field.name] === 'reps') {
                const amrapField = getFieldByValue(vm.meta, 'amrap');
                const weightField = getFieldByValue(vm.meta, 'weight');
                const repRangeField = getFieldByValue(vm.meta, 'rep_range');
                if (repRangeField) {
                    vm.meta[repRangeField] = 'n/a';
                }
                if (amrapField) {
                    if (weightField) {
                        vm.meta[amrapField] = 'n/a';
                    } else {
                        vm.meta[amrapField] = 'weight';
                    }
                }
            }

            if (vm.meta[field.name] === 'rep_range') {
                const repsField = getFieldByValue(vm.meta, 'reps');
                const amrapField = getFieldByValue(vm.meta, 'amrap');

                if (repsField) {
                    vm.meta[repsField] = 'n/a';
                }
                if (amrapField) {
                    vm.meta[amrapField] = 'n/a';
                }
            }
            vm.meta = WorkoutEditorService.filterAvailableFieldTypes(vm.meta);
        };

        vm.deleteSet = function (index) {
            ConfirmDialogService.open({
                title: "Confirm delete",
                message: "Do you want to remove this set?",
                buttonConfirm: "Confirm",
                buttonCancel: "Cancel"
            }).then(function () {
                vm.meta.sets.splice(index, 1);
            });
        };

        vm.save = function () {

            vm.saving = true;

            WorkoutEditorService.setExerciseMeta(vm.meta, parameters.exerciseIndex, parameters.groupIndex);

            WorkoutEditorService.saveChanges().then(function () {

                $scope.closeModal();
            }, function () {
                vm.saving = false;
            });
        };

        vm.fieldLabels = {
            speed: ['kph', 'mph', 'rpm'],
            distance: ['km', 'miles'],
            weight: ['kg', 'lbs']
        };

        AnalyticsService.trackView((PermissionsService.userIsCustomer() ? 'Customer' : 'Trainer') + ' - Edit Workout Exercise');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('EditWorkoutExerciseNoteEditModalController', controller);

    controller.$inject = ['parameters', 'WorkoutEditorService', '$scope', 'AnalyticsService', 'PermissionsService'];

    function controller(parameters, WorkoutEditorService, $scope, AnalyticsService, PermissionsService) {

        var vm = this;

        vm.workout = WorkoutEditorService.getWorkout();

        var exerciseInstance = angular.copy(parameters.exerciseInstance);

        vm.exercise = exerciseInstance.exercise;
        vm.meta = exerciseInstance.meta;

        vm.note = vm.exercise.note;

        vm.save = function () {

            vm.saving = true;

            WorkoutEditorService.setExerciseField(vm.note, 'note', parameters.exerciseIndex, parameters.groupIndex);

            WorkoutEditorService.saveChanges().then(function () {

                $scope.closeModal();
            }, function () {
                vm.saving = false;
            });
        };

        AnalyticsService.trackView((PermissionsService.userIsCustomer() ? 'Customer' : 'Trainer') + ' - Edit Workout Exercise Note');
    }
})();

(function () {
    'use strict';

    angular.module('MyPTHub').filter('fieldLabelDisplay', filter);

    function filter() {

        var handlers = {

            speed: function (label) {

                return label;
            },

            heart_rate: function (label) {

                return 'BPM';
            },

            rpe: function (label) {

                return 'RPE';
            },

            intensity: function (label) {

                return '%';
            },

            weight: function (label) {

                if (label === 'lbs') {
                    return 'lbs';
                }

                return 'kg';
            },

            distance: function (label) {
                return label;
            }

        };

        return function (type, label) {

            if (handlers.hasOwnProperty(type)) {

                return handlers[type](label);
            } else {

                return '';
            }
        };
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('EditWorkoutGroupAssignModalController', controller);

    controller.$inject = ['parameters', '$scope', '$filter', 'ApiClientService', 'PermissionsService', 'AnalyticsService'];

    function controller(parameters, $scope, $filter, ApiClientService, PermissionsService, AnalyticsService) {

        var vm = this;

        vm.currentAssignments = parameters.assignedGroups;

        vm.isCurrentTrainerUser = PermissionsService.getLoggedInUserId() === parameters.workout.trainer_user_id;

        vm.assignedIds = [];
        angular.forEach(vm.currentAssignments, function (group) {
            vm.assignedIds.push(group.id);
        });

        var initAssignedCount = vm.assignedIds.length;

        ApiClientService.workoutGroups.list(parameters.workout.trainer_user_id).success(function (response) {

            response.groups.map(function (group) {
                group.use = vm.assignedIds.indexOf(group.id) !== -1;
            });

            vm.groups = response.groups;

            vm.filteredGroups = angular.copy(response.groups);

            vm.noGroups = response.length === 0;
        });

        // Customer List Search filter
        $scope.$watch(function () {
            return vm.searchFilter;
        }, function (newVal, oldVal) {

            if (typeof newVal === 'undefined') {
                return false;
            }

            if (newVal.length > 0) {
                vm.searchFilterActive = true;
                vm.filteredGroups = $filter('filter')(vm.groups, {
                    title: newVal
                });
            } else {
                vm.searchFilterActive = false;
                vm.filteredGroups = vm.groups;
            }
        });

        vm.assign = function (group) {

            group.use = !group.use;

            var findOriginal = $filter('filter')(vm.groups, {
                id: group.id
            }, true);

            if (findOriginal.length > 0) {

                var originalIndex = vm.groups.indexOf(findOriginal[0]);

                if (originalIndex !== -1) {
                    vm.groups[originalIndex].use = group.use;
                }
            }
        };

        vm.save = function () {

            vm.saving = true;

            var toAssign = $filter('filter')(vm.groups, {
                use: true
            });

            if (toAssign.length === 0 && initAssignedCount === 0) {

                $scope.closeModal([]);

                return true;
            }

            var ids = [];
            var list = [];

            toAssign.map(function (group) {
                ids.push(group.id);
                list.push(group);
            });

            ApiClientService.workouts.update(parameters.workoutId, {
                assigned_group_ids: ids
            }).then(function () {
                $scope.closeModal(list);
            }, function () {
                vm.saving = false;
            });
        };

        AnalyticsService.trackView('Trainer - Assign Workout To Groups');
    }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').controller('EditWorkoutGrouperModalController', controller);

    controller.$inject = ['parameters', 'WorkoutEditorService', '$scope', 'AnalyticsService', 'PermissionsService'];

    function controller(parameters, WorkoutEditorService, $scope, AnalyticsService, PermissionsService) {

        var vm = this;

        WorkoutEditorService.resetGrouperChosen();

        var baseExerciseInstance = parameters.baseExercise;
        var baseGroup = parameters.baseGroup;

        vm.baseExercise = baseExerciseInstance.exercise;

        vm.availableExercises = WorkoutEditorService.getGroupable(baseGroup);

        vm.toggleExercise = function (exerciseInstance) {

            if (!exerciseInstance.use) {
                exerciseInstance.use = true;
            } else {
                exerciseInstance.use = false;
            }
        };

        vm.save = function () {

            WorkoutEditorService.doGrouping(baseGroup);

            vm.saving = true;

            WorkoutEditorService.saveChanges().then(function () {
                $scope.cancelModal();
            }, function () {
                vm.saving = false;
            });
        };

        AnalyticsService.trackView((PermissionsService.userIsCustomer() ? 'Customer' : 'Trainer') + ' - Workout Editor Group Exercises');
    }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').config(config).controller('TrainerClientGroupsGroupController', controller);

  config.$inject = ['$stateProvider'];

  function config($stateProvider) {

    $stateProvider.state('app.trainer_client_group', {
      url: "/trainer_clients/groups/:groupId?",
      views: {
        'menuContent': {
          templateUrl: "trainer/clients/groups/group/group.html",
          controller: 'TrainerClientGroupsGroupController as vm'
        }
      }
    });
  }

  controller.$inject = ['$stateParams', '$scope', 'ApiClientService', '$filter', 'PermissionsService', 'NavigateService', 'ConfirmDialogService', '$ionicActionSheet', 'AnalyticsService', 'CustomerFinderModalService', 'EntityFinderModalService', 'PromptDialogService', 'CustomerGroupScheduleAssignModalService', 'ClientGroupListService'];

  function controller($stateParams, $scope, ApiClientService, $filter, PermissionsService, NavigateService, ConfirmDialogService, $ionicActionSheet, AnalyticsService, CustomerFinderModalService, EntityFinderModalService, PromptDialogService, CustomerGroupScheduleAssignModalService, ClientGroupListService) {
    var vm = this;

    vm.unfilteredResults = {};
    vm.filteredResults = {};
    vm.filterQueries = {};

    var resetWorkoutSearch = function () {
      vm.workoutsSearchFilter = '';
      vm.filteredResults.workouts = vm.group.workouts;
    };

    var resetWorkoutGroupSearch = function () {
      vm.workoutGroupsSearchFilter = '';
      vm.filteredResults.workoutGroups = vm.group.workout_groups;
    };

    var resetNutritionPlanSearch = function () {
      vm.nutritionPlansSearchFilter = '';
      vm.filteredResults.nutritionPlans = vm.group.nutrition_plans;
    };

    var resetNutritionPlanGroupSearch = function () {
      vm.nutritionPlanGroupsSearchFilter = '';
      vm.filteredResults.nutritionPlanGroups = vm.group.nutrition_plan_groups;
    };

    var resetScheduleSearch = function () {
      vm.scheduleSearchFilter = '';
      vm.filteredResults.schedules = vm.group.schedules;
    };

    var resetCustomerSearch = function () {
      vm.customerSearchFilter = '';
      vm.filteredResults.customers = vm.group.assigned_customers;
    };

    var createSearchFilter = function (type) {
      $scope.$watch(function () {
        return vm.filterQueries[type];
      }, function (newVal, oldVal) {

        if (typeof newVal === 'undefined') {
          return false;
        }

        if (newVal.length > 0) {

          var config = {
            title: newVal
          };
          if (type === 'customers') {
            config = {
              full_name: newVal
            };
          }
          if (type === 'schedules') {
            config = {
              name: newVal
            };
          }

          vm.filteredResults[type] = $filter('filter')(vm.unfilteredResults[type], config);
        } else {
          vm.filteredResults[type] = vm.unfilteredResults[type];
        }
      });
    };

    createSearchFilter('customers');
    createSearchFilter('workouts');
    createSearchFilter('workoutGroups');
    createSearchFilter('nutritionPlans');
    createSearchFilter('nutritionPlanGroups');
    createSearchFilter('schedules');

    var load = function (refreshing) {

      refreshing = refreshing || false;

      vm.loaded = false;

      ApiClientService.customerGroups.get(vm.groupId).then(function (response) {
        vm.group = response.group;

        vm.group.assigned_customers.map(function (customer) {
          customer.full_name = customer.user.first_name + ' ' + customer.user.last_name;
          return customer;
        });

        vm.group.schedules.map(function (schedule) {
          if (schedule.pivot.start_date) {
            var startDate = moment(schedule.pivot.start_date).format('Do MMM YYYY');

            schedule.description = 'Starts on ' + startDate;
          } else {
            schedule.description = 'No start date specified';
          }

          schedule.active = schedule.started && !schedule.ended;
        });

        vm.filteredResults.customers = angular.copy(vm.group.assigned_customers);
        vm.unfilteredResults.customers = angular.copy(vm.group.assigned_customers);

        vm.filteredResults.workouts = angular.copy(vm.group.workouts);
        vm.unfilteredResults.workouts = angular.copy(vm.group.workouts);

        vm.filteredResults.workoutGroups = angular.copy(vm.group.workout_groups);
        vm.unfilteredResults.workoutGroups = angular.copy(vm.group.workout_groups);

        vm.filteredResults.nutritionPlans = angular.copy(vm.group.nutrition_plans);
        vm.unfilteredResults.nutritionPlans = angular.copy(vm.group.nutrition_plans);

        vm.filteredResults.nutritionPlanGroups = angular.copy(vm.group.nutrition_plan_groups);
        vm.unfilteredResults.nutritionPlanGroups = angular.copy(vm.group.nutrition_plan_groups);

        vm.filteredResults.schedules = angular.copy(vm.group.schedules);
        vm.unfilteredResults.schedules = angular.copy(vm.group.schedules);

        vm.loaded = true;
        vm.isCurrentTrainerUser = PermissionsService.getLoggedInUserId() === vm.group.trainer_user_id;
        vm.canEdit = PermissionsService.userIsTrainerAdminOrOwner(vm.group.trainer_user_id);

        resetWorkoutSearch();
        resetWorkoutGroupSearch();
        resetNutritionPlanSearch();
        resetNutritionPlanGroupSearch();
        resetCustomerSearch();
        resetScheduleSearch();
      }).finally(function () {
        if (refreshing) {
          $scope.$broadcast('scroll.refreshComplete');
        }
      });
    };

    vm.refresh = function () {
      load(true);
    };

    vm.editWorkout = function (workout) {
      NavigateService.go('app.edit_workout', {
        workoutId: workout.id
      });
    };

    vm.editWorkoutGroup = function (workoutGroup) {
      NavigateService.go('app.trainer_workouts_group', {
        groupId: workoutGroup.id
      });
    };

    vm.editNutritionPlan = function (nutritionPlan) {
      NavigateService.go('app.trainer_nutrition_plan_edit', {
        nutritionPlanId: nutritionPlan.id
      });
    };

    vm.editNutritionPlanGroup = function (nutritionPlanGroup) {
      NavigateService.go('app.trainer_nutrition_plans_group', {
        groupId: nutritionPlanGroup.id
      });
    };

    vm.editSchedule = function (schedule) {
      NavigateService.go('app.trainer_scheduler_update_schedule', {
        templateId: schedule.id
      });
    };

    vm.delete = function () {
      ConfirmDialogService.open({
        title: "Confirm delete",
        message: "Do you want to remove this client group?",
        buttonConfirm: "Confirm",
        buttonCancel: "Cancel"
      }).then(function () {
        ApiClientService.customerGroups.delete(vm.groupId).then(function () {
          ClientGroupListService.dirty();
          NavigateService.clearCache().then(NavigateService.goBack);
        });
      });
    };

    vm.rename = function () {
      PromptDialogService.open({
        title: "Rename Client Group",
        message: "Please enter a new name for the group",
        buttonConfirm: "Save",
        buttonCancel: "Cancel",
        required: true,
        defaultText: vm.group.title
      }).then(function (newTitle) {
        ClientGroupListService.dirty();
        ApiClientService.customerGroups.update(vm.group.id, {
          title: newTitle
        }).then(load);
      });
    };

    vm.assignedCustomerMenu = function (customer) {
      $ionicActionSheet.show({
        titleText: customer.user.first_name + ' ' + customer.user.last_name,
        destructiveText: 'Remove from group',
        cancelText: 'Cancel',
        destructiveButtonClicked: function () {
          ConfirmDialogService.open({
            title: "Confirm",
            message: "Do you want to remove " + customer.user.first_name + ' ' + customer.user.last_name + " from the group?",
            buttonConfirm: "Confirm",
            buttonCancel: "Cancel"
          }).then(function () {
            ApiClientService.customerGroups.unassignCustomer(vm.group.id, customer.id).then(load);

            return true;
          });
          return true;
        }
      });
    };

    var entityMenu = function (entity, type) {

      var buttons = [];

      buttons.push({
        text: 'Edit',
        action: function () {
          switch (type) {
            case 'workout':
              vm.editWorkout(entity);
              break;
            case 'workout group':
              vm.editWorkoutGroup(entity);
              break;
            case 'nutrition plan':
              vm.editNutritionPlan(entity);
              break;
            case 'nutrition plan group':
              vm.editNutritionPlanGroup(entity);
              break;
            case 'schedule':
              vm.editSchedule(entity);
              break;
          }
        }
      });

      var unassign = function () {
        ConfirmDialogService.open({
          title: "Confirm",
          message: "Do you want to remove " + entity.title + " from the group?",
          buttonConfirm: "Confirm",
          buttonCancel: "Cancel"
        }).then(function () {
          switch (type) {
            case 'workout':
              ApiClientService.workouts.unassignCustomerGroup(entity.id, vm.group.id).then(load);
              break;
            case 'workout group':
              ApiClientService.workoutGroups.unassignCustomerGroup(entity.id, vm.group.id).then(load);
              break;
            case 'nutrition plan':
              ApiClientService.nutrition.unassignCustomerGroup(entity.id, vm.group.id).then(load);
              break;
            case 'nutrition plan group':
              ApiClientService.nutritionGroups.unassignCustomerGroup(entity.id, vm.group.id).then(load);
              break;
            case 'schedule':
              ApiClientService.scheduler.unassignCustomerGroup(entity.id, vm.group.id).then(load);
              break;
          }
        });

        return true;
      };

      $ionicActionSheet.show({
        buttons: buttons,
        titleText: entity.title,
        cancelText: 'Cancel',
        cancel: function () {},
        buttonClicked: function (index) {
          if (typeof buttons[index] !== 'undefined' && typeof buttons[index].action === 'function') {
            buttons[index].action(index);
          }
          return true;
        },
        destructiveText: 'Remove from group',
        destructiveButtonClicked: unassign
      });
    };

    vm.workoutMenu = function (workout) {
      entityMenu(workout, 'workout');
    };

    vm.workoutGroupMenu = function (workoutGroup) {
      entityMenu(workoutGroup, 'workout group');
    };

    vm.nutritionPlanMenu = function (nutritionPlan) {
      entityMenu(nutritionPlan, 'nutrition plan');
    };

    vm.nutritionPlanGroupMenu = function (nutritionPlanGroup) {
      entityMenu(nutritionPlanGroup, 'nutrition plan group');
    };

    vm.scheduleMenu = function (schedule) {
      entityMenu(schedule, 'schedule');
    };

    vm.assignWorkouts = function () {
      vm.selectedTopLevelTabIndex = 1;
      vm.selectedWorkoutTabIndex = 0;

      EntityFinderModalService.open(vm.group.trainer_user_id, 'workout', 'workout', false, 'Assign Workout').then(function (assignWorkout) {
        ApiClientService.workouts.assignCustomerGroup(assignWorkout.id, vm.group.id).then(load);
      });
    };

    vm.assignWorkoutGroups = function () {
      vm.selectedTopLevelTabIndex = 1;
      vm.selectedWorkoutTabIndex = 1;

      EntityFinderModalService.open(vm.group.trainer_user_id, 'workoutGroup', 'workout group', false, 'Assign Workout Group').then(function (assignWorkoutGroup) {
        ApiClientService.workoutGroups.assignCustomerGroup(assignWorkoutGroup.id, vm.group.id).then(load);
      });
    };

    vm.assignNutritionPlans = function () {
      vm.selectedTopLevelTabIndex = 2;
      vm.selectedNutritionPlanTabIndex = 0;

      EntityFinderModalService.open(vm.group.trainer_user_id, 'nutritionPlan', 'nutrition plan', false, 'Assign Nutrition Plan').then(function (assignNutritionPlan) {
        ApiClientService.nutrition.assignCustomerGroup(assignNutritionPlan.id, vm.group.id).then(load);
      });
    };

    vm.assignNutritionPlanGroups = function () {
      vm.selectedTopLevelTabIndex = 2;
      vm.selectedNutritionPlanTabIndex = 1;

      EntityFinderModalService.open(vm.group.trainer_user_id, 'nutritionPlanGroup', 'nutrition plan group', false, 'Assign Nutrition Plan Group').then(function (assignNutritionPlanGroup) {
        ApiClientService.nutritionGroups.assignCustomerGroup(assignNutritionPlanGroup.id, vm.group.id).then(load);
      });
    };

    vm.assignSchedule = function () {
      CustomerGroupScheduleAssignModalService.open(vm.group).then(load);
    };

    vm.assignClients = function () {
      vm.selectedTopLevelTabIndex = 0;

      CustomerFinderModalService.open({
        trainerUserId: vm.group.trainer_user_id,
        currentCustomerAssignments: vm.group.assigned_customers,
        modalTitle: 'Assign Clients',
        mode: 'multi',
        includeGroups: false
      }).then(function (assignedItems) {
        var assignCustomerIds = [];

        assignedItems.customers.map(function (customer) {
          assignCustomerIds.push(customer.id);
        });

        ApiClientService.customerGroups.update(vm.group.id, {
          customer_ids: assignCustomerIds
        }).then(load);
      });
    };

    $scope.$on('$ionicView.beforeEnter', function () {
      vm.groupId = $stateParams.groupId;
      load();
    });

    $scope.$on('$ionicView.enter', function () {
      AnalyticsService.trackView('Trainer - View Client Group');
    });
  }
})();
(function () {
    'use strict';

    angular.module('MyPTHub').directive('workoutSetValueInput', directive);

    directive.$inject = [];

    function directive() {

        return {
            restrict: 'E',
            replace: true,
            templateUrl: "common/workouts/editor/inputs/setValue.html",
            controller: $scope => {
                $scope.float = parseFloat($scope.set[$scope.field.name]);
                $scope.int = parseInt($scope.set[$scope.field.name]);
                $scope.changeValue = value => $scope.set[$scope.field.name] = value;
            }
        };
    }
})();
(function () {
  'use strict';

  angular.module('MyPTHub').factory('CustomerGroupScheduleAssignModalService', service).controller('CustomerGroupScheduleAssignController', controller);

  service.$inject = ['ModalService'];

  function service(ModalService) {
    return {
      open: function (customerGroup) {
        return ModalService.show({
          templateUrl: "trainer/clients/groups/group/scheduleAssign/scheduleAssignModal.html",
          controller: 'CustomerGroupScheduleAssignController as vm',
          params: {
            customerGroup: customerGroup
          }
        });
      }
    };
  }

  controller.$inject = ['parameters', '$scope', '$filter', 'ApiClientService', 'PermissionsService', 'AnalyticsService'];

  function controller(parameters, $scope, $filter, ApiClientService, PermissionsService, AnalyticsService) {
    var vm = this;

    vm.loading = true;

    vm.formData = {
      schedule: null,
      start_date: null,
      customer_can_change_start_date: null
    };

    vm.customerGroup = parameters.customerGroup;

    var inScheduleIds = [];
    vm.customerGroup.schedules.map(function (schedule) {
      inScheduleIds.push(schedule.id);
    });

    ApiClientService.scheduler.scheduleTemplateIndexTrainerUser(vm.customerGroup.trainer_user_id).success(function (response) {
      vm.schedules = $filter('filter')(response.templates, function (schedule) {
        return inScheduleIds.indexOf(schedule.id) === -1;
      });
      vm.noEntities = vm.schedules.length === 0;
      vm.loading = false;
    });

    vm.assign = function () {
      vm.processing = true;

      ApiClientService.scheduler.assignCustomerGroup(vm.formData.schedule.id, {
        customer_group_id: vm.customerGroup.id,
        start_date: vm.formData.start_date ? moment(vm.formData.start_date).format('YYYY-MM-DD HH:mm:ss') : null,
        customer_can_change_start_date: vm.formData.customer_can_change_start_date
      }).then(function (response) {
        console.log(response);
        $scope.closeModal(vm.formData);
        vm.processing = false;
      }, function (response) {
        console.log(response.data);
        vm.processing = false;
      });
    };

    AnalyticsService.trackView('Customer Group - Assign Schedule');
  }
})();